# 17 "lib/X86_64Lexer.mll"
 
module Make(O:LexUtils.Config) = struct
open Lexing
open LexMisc
open X86_64Parser
module LU = LexUtils.Make(O)

let check_name name =
match name with
| "add"|"ADD"   -> I_ADD
| "addb"|"ADDB"   -> I_ADDB
| "addw"|"ADDW"   -> I_ADDW
| "addl"|"ADDL"   -> I_ADDL
| "addq"|"ADDQ"   -> I_ADDQ
| "xor"|"XOR"    -> I_XOR
| "xorb"|"XORB"    ->I_XORB
| "xorw"|"XORW"    ->I_XORW
| "xorl"|"XORL"    ->I_XORL
| "xorq"|"XORQ"    ->I_XORQ
| "or"|"OR"    ->I_OR
| "and"|"AND"  ->I_AND
| "orb"|"ORB"    ->I_ORB
| "orw"|"ORW"    ->I_ORW
| "orl"|"ORL"    ->I_ORL
| "orq"|"ORQ"    ->I_ORQ
| "shl"|"SHL" ->I_SHL
| "mov"|"MOV"   ->I_MOV
| "movb"|"MOVB"   ->I_MOVB
| "movw"|"MOVW"   ->I_MOVW
| "movl"|"MOVL"   ->I_MOVL
| "movq"|"MOVQ"   ->I_MOVQ
| "movnti"|"MOVNTI"   ->I_MOVNTI
| "movntil"|"MOVNTIL"   ->I_MOVNTIL
| "movntiq"|"MOVNTIQ"   ->I_MOVNTIQ
| "movd"|"MOVD" ->I_MOVD
| "movntdqa"|"MOVNTDQA" ->I_MOVNTDQA
| "dec"|"DEC"    ->I_DEC
| "decb"|"DECB"    ->I_DECB
| "decw"|"DECW"    ->I_DECW
| "decl"|"DECL"    ->I_DECL
| "decq"|"DECQ"    ->I_DECQ
| "inc"|"INC"    ->I_INC
| "incb"|"INCB"    ->I_INCB
| "incw"|"INCW"    ->I_INCW
| "incl"|"INCL"    ->I_INCL
| "incq"|"INCQ"    ->I_INCQ
| "cmp"|"CMP"    ->I_CMP
| "cmpb"|"CMPB"    ->I_CMPB
| "cmpw"|"CMPW"    ->I_CMPW
| "cmpl"|"CMPL"    ->I_CMPL
| "cmpq"|"CMPQ"    ->I_CMPQ
| "cmovc"|"CMOVC"    ->I_CMOVC
| "cmovcb"|"CMOVCB"    ->I_CMOVCB
| "cmovcw"|"CMOVCW"    ->I_CMOVCW
| "cmovcl"|"CMOVCL"    ->I_CMOVCL
| "cmovcq"|"CMOVCQ"    ->I_CMOVCQ
| "ret" | "RET"        ->I_RET
| "retq" | "RETQ"      ->I_RET
| "jmp"|"JMP"   ->I_JMP
| "je"|"JE"    ->I_JE
| "jne"|"JNE"    ->I_JNE
| "jle"|"JLE"    ->I_JLE
| "jlt"|"JLT"    ->I_JLT
| "jgt"|"JGT"    ->I_JGT
| "jge"|"JGE"    ->I_JGE
| "js"|"JS"    ->I_JS
| "jns"|"JNS"    ->I_JNS
| "lock"|"LOCK"   ->I_LOCK
| "xchg"|"XCHG"    ->I_XCHG
| "xchgb"|"XCHGB"    ->I_XCHGB
| "xchgw"|"XCHGW"    ->I_XCHGW
| "xchgl"|"XCHGL"    ->I_XCHGL
| "xchgq"|"XCHGQ"    ->I_XCHGQ
| "uxch"|"UXCH"    ->I_UXCH
| "uxchb"|"UXCHB"    ->I_UXCHB
| "uxchw"|"UXCHW"    ->I_UXCHW
| "uxchl"|"UXCHL"    ->I_UXCHL
| "uxchq"|"UXCHQ"    ->I_UXCHQ
| "cmpxchg"|"CMPXCHG"    ->I_CMPXCHG
| "cmpxchgb"|"CMPXCHGB"    ->I_CMPXCHGB
| "cmpxchgw"|"CMPXCHGW"    ->I_CMPXCHGW
| "cmpxchgl"|"CMPXCHGL"    ->I_CMPXCHGL
| "cmpxchgq"|"CMPXCHGQ"    ->I_CMPXCHGQ
| "mfence"|"MFENCE"   ->I_MFENCE
| "lfence"|"LFENCE"   ->I_LFENCE
| "sfence"|"SFENCE"   ->I_SFENCE
| "setnb"|"SETNB"       ->I_SETNB
| "nopl" | "NOPL" -> I_NOP
| "nop" | "NOP" -> I_NOPA
| "nopw" | "NOPW" -> I_NOP
| "endbr64" -> I_ENDBR64
| "clflush"|"CLFLUSH" ->I_CLFLUSH X86_64Base.NoOpt
| "clflushopt"|"CLFLUSHOPT" ->I_CLFLUSH X86_64Base.Opt
| _ -> begin match X86_64Base.parse_any_reg name with
  | Some r -> ARCH_REG r
  | None ->
      begin match  X86_64Base.parse_xmm_reg name with
      | Some r -> XMM r
      | None -> NAME name
      end
  end


# 106 "lib/X86_64Lexer.ml"
let __ocaml_lex_tables = {
  Lexing.lex_base =
   "\000\000\239\255\080\000\160\000\241\255\242\255\243\255\244\255\
    \246\255\247\255\248\255\218\000\040\001\158\000\115\001\125\001\
    \000\000\254\255\255\255\253\255\135\001\145\001\158\001\236\001\
    \038\002";
  Lexing.lex_backtrk =
   "\017\000\255\255\015\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\015\000\255\255\003\000\255\255\
    \010\000\255\255\255\255\255\255\004\000\255\255\005\000\006\000\
    \255\255";
  Lexing.lex_default =
   "\255\255\000\000\255\255\255\255\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\000\000\000\000\000\000\255\255\255\255\255\255\255\255\
    \255\255";
  Lexing.lex_trans =
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\018\000\017\000\000\000\000\000\018\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \018\000\000\000\000\000\000\000\013\000\011\000\000\000\000\000\
    \016\000\007\000\019\000\000\000\009\000\015\000\000\000\000\000\
    \014\000\014\000\014\000\014\000\014\000\014\000\014\000\014\000\
    \014\000\014\000\004\000\010\000\000\000\000\000\000\000\000\000\
    \000\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
    \002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
    \012\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
    \002\000\002\000\002\000\006\000\000\000\005\000\000\000\003\000\
    \000\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
    \002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
    \002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
    \002\000\002\000\002\000\000\000\008\000\002\000\002\000\002\000\
    \002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
    \002\000\002\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
    \002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
    \002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
    \002\000\002\000\002\000\000\000\000\000\000\000\000\000\002\000\
    \000\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
    \002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
    \002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
    \002\000\002\000\002\000\021\000\000\000\000\000\020\000\020\000\
    \020\000\020\000\020\000\020\000\020\000\020\000\020\000\020\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
    \002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
    \002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
    \002\000\002\000\002\000\000\000\000\000\000\000\000\000\000\000\
    \001\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
    \002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
    \002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
    \002\000\002\000\002\000\023\000\023\000\023\000\023\000\023\000\
    \023\000\023\000\023\000\023\000\023\000\023\000\023\000\023\000\
    \023\000\023\000\023\000\023\000\023\000\023\000\023\000\023\000\
    \023\000\023\000\023\000\023\000\023\000\000\000\000\000\000\000\
    \000\000\024\000\000\000\023\000\023\000\023\000\023\000\023\000\
    \023\000\023\000\023\000\023\000\023\000\023\000\023\000\023\000\
    \023\000\023\000\023\000\023\000\023\000\023\000\023\000\023\000\
    \023\000\023\000\023\000\023\000\023\000\002\000\002\000\002\000\
    \022\000\022\000\022\000\022\000\022\000\022\000\022\000\022\000\
    \022\000\022\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
    \002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
    \002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
    \002\000\002\000\002\000\000\000\000\000\000\000\000\000\002\000\
    \000\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
    \002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
    \002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
    \002\000\002\000\002\000\014\000\014\000\014\000\014\000\014\000\
    \014\000\014\000\014\000\014\000\014\000\014\000\014\000\014\000\
    \014\000\014\000\014\000\014\000\014\000\014\000\014\000\020\000\
    \020\000\020\000\020\000\020\000\020\000\020\000\020\000\020\000\
    \020\000\020\000\020\000\020\000\020\000\020\000\020\000\020\000\
    \020\000\020\000\020\000\002\000\002\000\002\000\022\000\022\000\
    \022\000\022\000\022\000\022\000\022\000\022\000\022\000\022\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\002\000\
    \002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
    \002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
    \002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
    \002\000\000\000\000\000\000\000\000\000\002\000\000\000\002\000\
    \002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
    \002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
    \002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
    \002\000\023\000\023\000\023\000\023\000\023\000\023\000\023\000\
    \023\000\023\000\023\000\023\000\023\000\023\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\023\000\023\000\023\000\
    \023\000\023\000\023\000\023\000\023\000\023\000\023\000\023\000\
    \023\000\023\000\023\000\023\000\023\000\023\000\023\000\023\000\
    \023\000\023\000\023\000\023\000\023\000\023\000\023\000\000\000\
    \000\000\000\000\000\000\023\000\000\000\023\000\023\000\023\000\
    \023\000\023\000\023\000\023\000\023\000\023\000\023\000\023\000\
    \023\000\023\000\023\000\023\000\023\000\023\000\023\000\023\000\
    \023\000\023\000\023\000\023\000\023\000\023\000\023\000\023\000\
    \023\000\023\000\023\000\023\000\023\000\023\000\023\000\023\000\
    \023\000\023\000\023\000\023\000\023\000\023\000\023\000\023\000\
    \023\000\023\000\023\000\023\000\023\000\023\000\023\000\023\000\
    \023\000\000\000\000\000\000\000\000\000\000\000\000\000\023\000\
    \023\000\023\000\023\000\023\000\023\000\023\000\023\000\023\000\
    \023\000\023\000\023\000\023\000\023\000\023\000\023\000\023\000\
    \023\000\023\000\023\000\023\000\023\000\023\000\023\000\023\000\
    \023\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000";
  Lexing.lex_check =
   "\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\000\000\000\000\255\255\255\255\000\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \000\000\255\255\255\255\255\255\000\000\000\000\255\255\255\255\
    \000\000\000\000\016\000\255\255\000\000\000\000\255\255\255\255\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\255\255\255\255\255\255\255\255\
    \255\255\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\255\255\000\000\255\255\000\000\
    \255\255\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\255\255\000\000\002\000\002\000\002\000\
    \002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
    \002\000\002\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
    \002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
    \002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
    \002\000\002\000\002\000\255\255\255\255\255\255\255\255\002\000\
    \255\255\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
    \002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
    \002\000\002\000\002\000\002\000\002\000\002\000\002\000\002\000\
    \002\000\002\000\002\000\013\000\255\255\255\255\013\000\013\000\
    \013\000\013\000\013\000\013\000\013\000\013\000\013\000\013\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
    \003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
    \003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
    \003\000\003\000\003\000\255\255\255\255\255\255\255\255\255\255\
    \000\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
    \003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
    \003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
    \003\000\003\000\003\000\011\000\011\000\011\000\011\000\011\000\
    \011\000\011\000\011\000\011\000\011\000\011\000\011\000\011\000\
    \011\000\011\000\011\000\011\000\011\000\011\000\011\000\011\000\
    \011\000\011\000\011\000\011\000\011\000\255\255\255\255\255\255\
    \255\255\011\000\255\255\011\000\011\000\011\000\011\000\011\000\
    \011\000\011\000\011\000\011\000\011\000\011\000\011\000\011\000\
    \011\000\011\000\011\000\011\000\011\000\011\000\011\000\011\000\
    \011\000\011\000\011\000\011\000\011\000\012\000\012\000\012\000\
    \012\000\012\000\012\000\012\000\012\000\012\000\012\000\012\000\
    \012\000\012\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\012\000\012\000\012\000\012\000\012\000\012\000\012\000\
    \012\000\012\000\012\000\012\000\012\000\012\000\012\000\012\000\
    \012\000\012\000\012\000\012\000\012\000\012\000\012\000\012\000\
    \012\000\012\000\012\000\255\255\255\255\255\255\255\255\012\000\
    \255\255\012\000\012\000\012\000\012\000\012\000\012\000\012\000\
    \012\000\012\000\012\000\012\000\012\000\012\000\012\000\012\000\
    \012\000\012\000\012\000\012\000\012\000\012\000\012\000\012\000\
    \012\000\012\000\012\000\014\000\014\000\014\000\014\000\014\000\
    \014\000\014\000\014\000\014\000\014\000\015\000\015\000\015\000\
    \015\000\015\000\015\000\015\000\015\000\015\000\015\000\020\000\
    \020\000\020\000\020\000\020\000\020\000\020\000\020\000\020\000\
    \020\000\021\000\021\000\021\000\021\000\021\000\021\000\021\000\
    \021\000\021\000\021\000\022\000\022\000\022\000\022\000\022\000\
    \022\000\022\000\022\000\022\000\022\000\022\000\022\000\022\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\022\000\
    \022\000\022\000\022\000\022\000\022\000\022\000\022\000\022\000\
    \022\000\022\000\022\000\022\000\022\000\022\000\022\000\022\000\
    \022\000\022\000\022\000\022\000\022\000\022\000\022\000\022\000\
    \022\000\255\255\255\255\255\255\255\255\022\000\255\255\022\000\
    \022\000\022\000\022\000\022\000\022\000\022\000\022\000\022\000\
    \022\000\022\000\022\000\022\000\022\000\022\000\022\000\022\000\
    \022\000\022\000\022\000\022\000\022\000\022\000\022\000\022\000\
    \022\000\023\000\023\000\023\000\023\000\023\000\023\000\023\000\
    \023\000\023\000\023\000\023\000\023\000\023\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\023\000\023\000\023\000\
    \023\000\023\000\023\000\023\000\023\000\023\000\023\000\023\000\
    \023\000\023\000\023\000\023\000\023\000\023\000\023\000\023\000\
    \023\000\023\000\023\000\023\000\023\000\023\000\023\000\255\255\
    \255\255\255\255\255\255\023\000\255\255\023\000\023\000\023\000\
    \023\000\023\000\023\000\023\000\023\000\023\000\023\000\023\000\
    \023\000\023\000\023\000\023\000\023\000\023\000\023\000\023\000\
    \023\000\023\000\023\000\023\000\023\000\023\000\023\000\024\000\
    \024\000\024\000\024\000\024\000\024\000\024\000\024\000\024\000\
    \024\000\024\000\024\000\024\000\024\000\024\000\024\000\024\000\
    \024\000\024\000\024\000\024\000\024\000\024\000\024\000\024\000\
    \024\000\255\255\255\255\255\255\255\255\255\255\255\255\024\000\
    \024\000\024\000\024\000\024\000\024\000\024\000\024\000\024\000\
    \024\000\024\000\024\000\024\000\024\000\024\000\024\000\024\000\
    \024\000\024\000\024\000\024\000\024\000\024\000\024\000\024\000\
    \024\000\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255";
  Lexing.lex_base_code =
   "";
  Lexing.lex_backtrk_code =
   "";
  Lexing.lex_default_code =
   "";
  Lexing.lex_trans_code =
   "";
  Lexing.lex_check_code =
   "";
  Lexing.lex_code =
   "";
}

let rec token lexbuf =
   __ocaml_lex_token_rec lexbuf 0
and __ocaml_lex_token_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 126 "lib/X86_64Lexer.mll"
                ( token lexbuf )
# 348 "lib/X86_64Lexer.ml"

  | 1 ->
# 127 "lib/X86_64Lexer.mll"
            ( incr_lineno lexbuf; token lexbuf )
# 353 "lib/X86_64Lexer.ml"

  | 2 ->
# 128 "lib/X86_64Lexer.mll"
            ( LU.skip_comment lexbuf ; token lexbuf )
# 358 "lib/X86_64Lexer.ml"

  | 3 ->
let
# 129 "lib/X86_64Lexer.mll"
               x
# 364 "lib/X86_64Lexer.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos lexbuf.Lexing.lex_curr_pos in
# 129 "lib/X86_64Lexer.mll"
                 ( NUM x )
# 368 "lib/X86_64Lexer.ml"

  | 4 ->
let
# 130 "lib/X86_64Lexer.mll"
                   x
# 374 "lib/X86_64Lexer.ml"
= Lexing.sub_lexeme lexbuf (lexbuf.Lexing.lex_start_pos + 1) lexbuf.Lexing.lex_curr_pos in
# 130 "lib/X86_64Lexer.mll"
                      ( INTEL_NUM x )
# 378 "lib/X86_64Lexer.ml"

  | 5 ->
let
# 131 "lib/X86_64Lexer.mll"
              x
# 384 "lib/X86_64Lexer.ml"
= Lexing.sub_lexeme lexbuf (lexbuf.Lexing.lex_start_pos + 1) lexbuf.Lexing.lex_curr_pos in
# 132 "lib/X86_64Lexer.mll"
    ( PROC (int_of_string x) )
# 388 "lib/X86_64Lexer.ml"

  | 6 ->
let
# 133 "lib/X86_64Lexer.mll"
               name
# 394 "lib/X86_64Lexer.ml"
= Lexing.sub_lexeme lexbuf (lexbuf.Lexing.lex_start_pos + 1) lexbuf.Lexing.lex_curr_pos in
# 134 "lib/X86_64Lexer.mll"
    ( match X86_64Base.parse_any_reg name with
    | Some r -> ARCH_REG r
    | None ->
        begin match X86_64Base.parse_xmm_reg name with
        | Some xmm -> XMM xmm
        | None -> SYMB_REG name
        end )
# 404 "lib/X86_64Lexer.ml"

  | 7 ->
# 141 "lib/X86_64Lexer.mll"
      ( SEMI )
# 409 "lib/X86_64Lexer.ml"

  | 8 ->
# 142 "lib/X86_64Lexer.mll"
      ( COMMA )
# 414 "lib/X86_64Lexer.ml"

  | 9 ->
# 143 "lib/X86_64Lexer.mll"
      ( PIPE )
# 419 "lib/X86_64Lexer.ml"

  | 10 ->
# 144 "lib/X86_64Lexer.mll"
      ( LPAR )
# 424 "lib/X86_64Lexer.ml"

  | 11 ->
# 145 "lib/X86_64Lexer.mll"
      ( RPAR )
# 429 "lib/X86_64Lexer.ml"

  | 12 ->
# 146 "lib/X86_64Lexer.mll"
      ( LBRK )
# 434 "lib/X86_64Lexer.ml"

  | 13 ->
# 147 "lib/X86_64Lexer.mll"
      ( RBRK )
# 439 "lib/X86_64Lexer.ml"

  | 14 ->
# 148 "lib/X86_64Lexer.mll"
      ( COLON )
# 444 "lib/X86_64Lexer.ml"

  | 15 ->
let
# 149 "lib/X86_64Lexer.mll"
          x
# 450 "lib/X86_64Lexer.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos lexbuf.Lexing.lex_curr_pos in
# 149 "lib/X86_64Lexer.mll"
            ( check_name x )
# 454 "lib/X86_64Lexer.ml"

  | 16 ->
# 150 "lib/X86_64Lexer.mll"
      ( EOF )
# 459 "lib/X86_64Lexer.ml"

  | 17 ->
# 151 "lib/X86_64Lexer.mll"
      ( error "X86_64 Lexer" lexbuf )
# 464 "lib/X86_64Lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf;
      __ocaml_lex_token_rec lexbuf __ocaml_lex_state

;;

# 153 "lib/X86_64Lexer.mll"
 
let token lexbuf =
   let tok = token lexbuf in
   if O.debug then begin
     Printf.eprintf
       "%a: Lexed '%s'\n"
       Pos.pp_pos2
       (lexeme_start_p lexbuf,lexeme_end_p lexbuf)
       (lexeme lexbuf)
   end ;
   tok
end

# 485 "lib/X86_64Lexer.ml"
