
module Make

# 42 "asllib/Parser.mly"
          (Config : ParserConfig.CONFIG)
# 7 "asllib/Parser.ml"

= struct
  
  module MenhirBasics = struct
    
    exception Error
    
    let _eRR =
      fun _s ->
        raise Error
    
    type token = Tokens.token
    
  end
  
  include MenhirBasics
  
# 50 "asllib/Parser.mly"
  

open AST
open ASTUtils
open Desugar

let version = V1

let t_bit =
  T_Bits (E_Literal (L_Int Z.one) |> add_dummy_annotation ~version, [])

let zero = E_Literal (L_Int Z.zero) |> add_dummy_annotation ~version

let make_ldi_vars (xs, ty) =
  let make_one x =
    S_Decl (LDK_Var, LDI_Typed (LDI_Var x, ty), None)
    |> add_dummy_annotation ~version
  in
  List.map make_one xs |> stmt_from_list |> desc

let make_ty_decl_subtype (x, s) =
  let name, _fields = s.desc in
  let ty = ASTUtils.add_pos_from s (T_Named name) in
  D_TypeDecl (x, ty, Some s.desc)

let prec =
  let open AST in
  function
  | BOR | BAND | IMPL | BEQ -> 1
  | EQ_OP | NEQ -> 2
  | PLUS | MINUS | OR | EOR | AND -> 3
  | MUL | DIV | DIVRM | RDIV | MOD | SHL | SHR -> 4
  | POW -> 5
  | _ -> 0 (* Non assoc *)

let check_not_same_prec loc op op' =
  if prec op = prec op' then Error.(fatal_from loc CannotParse)

let check_not_binop_same_prec op e =
  match e.desc with
  | E_Binop (op', _, _) when op != op' -> check_not_same_prec e op op'
  | _ -> ()

let e_binop (e1, op, e2) =
  let () = check_not_binop_same_prec op e1
  and () = check_not_binop_same_prec op e2
  in
  E_Binop (op, e1, e2)

let e_call call = E_Call { call with call_type = ST_Function }
let s_call call = S_Call { call with call_type = ST_Procedure }


# 79 "asllib/Parser.ml"
  
  type ('s, 'r) _menhir_state = 
    | MenhirState0000 : ('s, _menhir_box_opn) _menhir_state
      (** State 0000.
        Stack shape : .
        Start symbol: opn. *)

    | MenhirState0001 : (('s, _menhir_box_opn) _menhir_cell1_WHILE, _menhir_box_opn) _menhir_state
      (** State 0001.
        Stack shape : WHILE.
        Start symbol: opn. *)

    | MenhirState0004 : (('s, 'r) _menhir_cell1_NOT, 'r) _menhir_state
      (** State 0004.
        Stack shape : NOT.
        Start symbol: <undetermined>. *)

    | MenhirState0005 : (('s, 'r) _menhir_cell1_MINUS, 'r) _menhir_state
      (** State 0005.
        Stack shape : MINUS.
        Start symbol: <undetermined>. *)

    | MenhirState0006 : (('s, 'r) _menhir_cell1_LPAR, 'r) _menhir_state
      (** State 0006.
        Stack shape : LPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0008 : (('s, 'r) _menhir_cell1_IF, 'r) _menhir_state
      (** State 0008.
        Stack shape : IF.
        Start symbol: <undetermined>. *)

    | MenhirState0010 : (('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_state
      (** State 0010.
        Stack shape : IDENTIFIER LPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0013 : (('s, 'r) _menhir_cell1_BNOT, 'r) _menhir_state
      (** State 0013.
        Stack shape : BNOT.
        Start symbol: <undetermined>. *)

    | MenhirState0016 : (('s, 'r) _menhir_cell1_ARBITRARY, 'r) _menhir_state
      (** State 0016.
        Stack shape : ARBITRARY.
        Start symbol: <undetermined>. *)

    | MenhirState0019 : (('s, 'r) _menhir_cell1_LPAR, 'r) _menhir_state
      (** State 0019.
        Stack shape : LPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0022 : (('s, 'r) _menhir_cell1_INTEGER _menhir_cell0_LBRACE, 'r) _menhir_state
      (** State 0022.
        Stack shape : INTEGER LBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0023 : ((('s, 'r) _menhir_cell1_INTEGER _menhir_cell0_LBRACE, 'r) _menhir_cell1_MINUS, 'r) _menhir_state
      (** State 0023.
        Stack shape : INTEGER LBRACE MINUS.
        Start symbol: <undetermined>. *)

    | MenhirState0025 : ((('s, 'r) _menhir_cell1_MINUS, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0025.
        Stack shape : MINUS expr.
        Start symbol: <undetermined>. *)

    | MenhirState0026 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LLBRACKET, 'r) _menhir_state
      (** State 0026.
        Stack shape : expr LLBRACKET.
        Start symbol: <undetermined>. *)

    | MenhirState0027 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LLBRACKET, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0027.
        Stack shape : expr LLBRACKET expr.
        Start symbol: <undetermined>. *)

    | MenhirState0028 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SHR, 'r) _menhir_state
      (** State 0028.
        Stack shape : expr SHR.
        Start symbol: <undetermined>. *)

    | MenhirState0029 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SHR, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0029.
        Stack shape : expr SHR expr.
        Start symbol: <undetermined>. *)

    | MenhirState0030 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_POW, 'r) _menhir_state
      (** State 0030.
        Stack shape : expr POW.
        Start symbol: <undetermined>. *)

    | MenhirState0031 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_POW, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0031.
        Stack shape : expr POW expr.
        Start symbol: <undetermined>. *)

    | MenhirState0032 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LBRACKET, 'r) _menhir_state
      (** State 0032.
        Stack shape : expr LBRACKET.
        Start symbol: <undetermined>. *)

    | MenhirState0033 : (('s, 'r) _menhir_cell1_COLON, 'r) _menhir_state
      (** State 0033.
        Stack shape : COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0034 : ((('s, 'r) _menhir_cell1_COLON, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0034.
        Stack shape : COLON expr.
        Start symbol: <undetermined>. *)

    | MenhirState0035 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SHL, 'r) _menhir_state
      (** State 0035.
        Stack shape : expr SHL.
        Start symbol: <undetermined>. *)

    | MenhirState0036 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SHL, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0036.
        Stack shape : expr SHL expr.
        Start symbol: <undetermined>. *)

    | MenhirState0038 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_IN _menhir_cell0_LBRACE, 'r) _menhir_state
      (** State 0038.
        Stack shape : expr IN LBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0041 : (('s, 'r) _menhir_cell1_NOT, 'r) _menhir_state
      (** State 0041.
        Stack shape : NOT.
        Start symbol: <undetermined>. *)

    | MenhirState0042 : ((('s, 'r) _menhir_cell1_NOT, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0042.
        Stack shape : NOT expr.
        Start symbol: <undetermined>. *)

    | MenhirState0044 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DOT, 'r) _menhir_state
      (** State 0044.
        Stack shape : expr DOT.
        Start symbol: <undetermined>. *)

    | MenhirState0046 : (('s, 'r) _menhir_cell1_IDENTIFIER, 'r) _menhir_state
      (** State 0046.
        Stack shape : IDENTIFIER.
        Start symbol: <undetermined>. *)

    | MenhirState0051 : (('s, 'r) _menhir_cell1_MINUS, 'r) _menhir_state
      (** State 0051.
        Stack shape : MINUS.
        Start symbol: <undetermined>. *)

    | MenhirState0052 : ((('s, 'r) _menhir_cell1_MINUS, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0052.
        Stack shape : MINUS expr.
        Start symbol: <undetermined>. *)

    | MenhirState0054 : (('s, 'r) _menhir_cell1_LPAR, 'r) _menhir_state
      (** State 0054.
        Stack shape : LPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0055 : (('s, 'r) _menhir_cell1_LEQ, 'r) _menhir_state
      (** State 0055.
        Stack shape : LEQ.
        Start symbol: <undetermined>. *)

    | MenhirState0056 : ((('s, 'r) _menhir_cell1_LEQ, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0056.
        Stack shape : LEQ expr.
        Start symbol: <undetermined>. *)

    | MenhirState0057 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_RDIV, 'r) _menhir_state
      (** State 0057.
        Stack shape : expr RDIV.
        Start symbol: <undetermined>. *)

    | MenhirState0058 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_RDIV, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0058.
        Stack shape : expr RDIV expr.
        Start symbol: <undetermined>. *)

    | MenhirState0059 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_CONCAT, 'r) _menhir_state
      (** State 0059.
        Stack shape : expr CONCAT.
        Start symbol: <undetermined>. *)

    | MenhirState0060 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_CONCAT, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0060.
        Stack shape : expr CONCAT expr.
        Start symbol: <undetermined>. *)

    | MenhirState0061 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_PLUS, 'r) _menhir_state
      (** State 0061.
        Stack shape : expr PLUS.
        Start symbol: <undetermined>. *)

    | MenhirState0062 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_PLUS, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0062.
        Stack shape : expr PLUS expr.
        Start symbol: <undetermined>. *)

    | MenhirState0063 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_MUL, 'r) _menhir_state
      (** State 0063.
        Stack shape : expr MUL.
        Start symbol: <undetermined>. *)

    | MenhirState0064 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_MUL, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0064.
        Stack shape : expr MUL expr.
        Start symbol: <undetermined>. *)

    | MenhirState0065 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_MOD, 'r) _menhir_state
      (** State 0065.
        Stack shape : expr MOD.
        Start symbol: <undetermined>. *)

    | MenhirState0066 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_MOD, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0066.
        Stack shape : expr MOD expr.
        Start symbol: <undetermined>. *)

    | MenhirState0067 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DIVRM, 'r) _menhir_state
      (** State 0067.
        Stack shape : expr DIVRM.
        Start symbol: <undetermined>. *)

    | MenhirState0068 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DIVRM, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0068.
        Stack shape : expr DIVRM expr.
        Start symbol: <undetermined>. *)

    | MenhirState0069 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DIV, 'r) _menhir_state
      (** State 0069.
        Stack shape : expr DIV.
        Start symbol: <undetermined>. *)

    | MenhirState0070 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DIV, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0070.
        Stack shape : expr DIV expr.
        Start symbol: <undetermined>. *)

    | MenhirState0071 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_OR, 'r) _menhir_state
      (** State 0071.
        Stack shape : expr OR.
        Start symbol: <undetermined>. *)

    | MenhirState0072 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_OR, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0072.
        Stack shape : expr OR expr.
        Start symbol: <undetermined>. *)

    | MenhirState0073 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_NEQ, 'r) _menhir_state
      (** State 0073.
        Stack shape : expr NEQ.
        Start symbol: <undetermined>. *)

    | MenhirState0075 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_NEQ, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0075.
        Stack shape : expr NEQ expr.
        Start symbol: <undetermined>. *)

    | MenhirState0076 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_MINUS, 'r) _menhir_state
      (** State 0076.
        Stack shape : expr MINUS.
        Start symbol: <undetermined>. *)

    | MenhirState0077 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_MINUS, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0077.
        Stack shape : expr MINUS expr.
        Start symbol: <undetermined>. *)

    | MenhirState0078 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LT, 'r) _menhir_state
      (** State 0078.
        Stack shape : expr LT.
        Start symbol: <undetermined>. *)

    | MenhirState0079 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LT, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0079.
        Stack shape : expr LT expr.
        Start symbol: <undetermined>. *)

    | MenhirState0080 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_EOR, 'r) _menhir_state
      (** State 0080.
        Stack shape : expr EOR.
        Start symbol: <undetermined>. *)

    | MenhirState0081 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_EOR, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0081.
        Stack shape : expr EOR expr.
        Start symbol: <undetermined>. *)

    | MenhirState0082 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_COLON_COLON, 'r) _menhir_state
      (** State 0082.
        Stack shape : expr COLON_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0083 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_COLON_COLON, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0083.
        Stack shape : expr COLON_COLON expr.
        Start symbol: <undetermined>. *)

    | MenhirState0084 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_AND, 'r) _menhir_state
      (** State 0084.
        Stack shape : expr AND.
        Start symbol: <undetermined>. *)

    | MenhirState0085 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_AND, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0085.
        Stack shape : expr AND expr.
        Start symbol: <undetermined>. *)

    | MenhirState0086 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LEQ, 'r) _menhir_state
      (** State 0086.
        Stack shape : expr LEQ.
        Start symbol: <undetermined>. *)

    | MenhirState0087 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LEQ, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0087.
        Stack shape : expr LEQ expr.
        Start symbol: <undetermined>. *)

    | MenhirState0088 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_GT, 'r) _menhir_state
      (** State 0088.
        Stack shape : expr GT.
        Start symbol: <undetermined>. *)

    | MenhirState0089 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_GT, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0089.
        Stack shape : expr GT expr.
        Start symbol: <undetermined>. *)

    | MenhirState0090 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_GEQ, 'r) _menhir_state
      (** State 0090.
        Stack shape : expr GEQ.
        Start symbol: <undetermined>. *)

    | MenhirState0091 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_GEQ, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0091.
        Stack shape : expr GEQ expr.
        Start symbol: <undetermined>. *)

    | MenhirState0092 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_IMPL, 'r) _menhir_state
      (** State 0092.
        Stack shape : expr IMPL.
        Start symbol: <undetermined>. *)

    | MenhirState0093 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_IMPL, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0093.
        Stack shape : expr IMPL expr.
        Start symbol: <undetermined>. *)

    | MenhirState0094 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_EQ_OP, 'r) _menhir_state
      (** State 0094.
        Stack shape : expr EQ_OP.
        Start symbol: <undetermined>. *)

    | MenhirState0096 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_EQ_OP, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0096.
        Stack shape : expr EQ_OP expr.
        Start symbol: <undetermined>. *)

    | MenhirState0097 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_BOR, 'r) _menhir_state
      (** State 0097.
        Stack shape : expr BOR.
        Start symbol: <undetermined>. *)

    | MenhirState0098 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_BOR, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0098.
        Stack shape : expr BOR expr.
        Start symbol: <undetermined>. *)

    | MenhirState0099 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_BEQ, 'r) _menhir_state
      (** State 0099.
        Stack shape : expr BEQ.
        Start symbol: <undetermined>. *)

    | MenhirState0100 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_BEQ, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0100.
        Stack shape : expr BEQ expr.
        Start symbol: <undetermined>. *)

    | MenhirState0101 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_BAND, 'r) _menhir_state
      (** State 0101.
        Stack shape : expr BAND.
        Start symbol: <undetermined>. *)

    | MenhirState0102 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_BAND, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0102.
        Stack shape : expr BAND expr.
        Start symbol: <undetermined>. *)

    | MenhirState0103 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_AS, 'r) _menhir_state
      (** State 0103.
        Stack shape : expr AS.
        Start symbol: <undetermined>. *)

    | MenhirState0104 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_AS, 'r) _menhir_cell1_LBRACE, 'r) _menhir_state
      (** State 0104.
        Stack shape : expr AS LBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0105 : ((((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_AS, 'r) _menhir_cell1_LBRACE, 'r) _menhir_cell1_MINUS, 'r) _menhir_state
      (** State 0105.
        Stack shape : expr AS LBRACE MINUS.
        Start symbol: <undetermined>. *)

    | MenhirState0109 : (('s, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0109.
        Stack shape : expr.
        Start symbol: <undetermined>. *)

    | MenhirState0110 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SLICING, 'r) _menhir_state
      (** State 0110.
        Stack shape : expr SLICING.
        Start symbol: <undetermined>. *)

    | MenhirState0111 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SLICING, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0111.
        Stack shape : expr SLICING expr.
        Start symbol: <undetermined>. *)

    | MenhirState0112 : ((((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SLICING, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_COMMA, 'r) _menhir_state
      (** State 0112.
        Stack shape : expr SLICING expr COMMA.
        Start symbol: <undetermined>. *)

    | MenhirState0114 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_COMMA, 'r) _menhir_state
      (** State 0114.
        Stack shape : expr COMMA.
        Start symbol: <undetermined>. *)

    | MenhirState0119 : (('s, 'r) _menhir_cell1_BITS _menhir_cell0_LPAR, 'r) _menhir_state
      (** State 0119.
        Stack shape : BITS LPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0120 : ((('s, 'r) _menhir_cell1_BITS _menhir_cell0_LPAR, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0120.
        Stack shape : BITS LPAR expr.
        Start symbol: <undetermined>. *)

    | MenhirState0122 : (((('s, 'r) _menhir_cell1_BITS _menhir_cell0_LPAR, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_RPAR _menhir_cell0_LBRACE, 'r) _menhir_state
      (** State 0122.
        Stack shape : BITS LPAR expr RPAR LBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0123 : (('s, 'r) _menhir_cell1_LBRACKET, 'r) _menhir_state
      (** State 0123.
        Stack shape : LBRACKET.
        Start symbol: <undetermined>. *)

    | MenhirState0127 : ((('s, 'r) _menhir_cell1_LBRACKET, 'r) _menhir_cell1_separated_nonempty_list_COMMA_slice_ _menhir_cell0_RBRACKET _menhir_cell0_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_state
      (** State 0127.
        Stack shape : LBRACKET separated_nonempty_list(COMMA,slice) RBRACKET IDENTIFIER LBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0131 : (((('s, 'r) _menhir_cell1_LBRACKET, 'r) _menhir_cell1_separated_nonempty_list_COMMA_slice_ _menhir_cell0_RBRACKET _menhir_cell0_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_loption_non_empty_trailing_list_COMMA_bitfield__ _menhir_cell0_RBRACE, 'r) _menhir_state
      (** State 0131.
        Stack shape : LBRACKET separated_nonempty_list(COMMA,slice) RBRACKET IDENTIFIER LBRACE loption(non_empty_trailing_list(COMMA,bitfield)) RBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0133 : ((('s, 'r) _menhir_cell1_LBRACKET, 'r) _menhir_cell1_separated_nonempty_list_COMMA_slice_ _menhir_cell0_RBRACKET _menhir_cell0_IDENTIFIER, 'r) _menhir_state
      (** State 0133.
        Stack shape : LBRACKET separated_nonempty_list(COMMA,slice) RBRACKET IDENTIFIER.
        Start symbol: <undetermined>. *)

    | MenhirState0135 : ((('s, 'r) _menhir_cell1_LBRACKET, 'r) _menhir_cell1_separated_nonempty_list_COMMA_slice_ _menhir_cell0_RBRACKET _menhir_cell0_IDENTIFIER, 'r) _menhir_state
      (** State 0135.
        Stack shape : LBRACKET separated_nonempty_list(COMMA,slice) RBRACKET IDENTIFIER.
        Start symbol: <undetermined>. *)

    | MenhirState0138 : (('s, 'r) _menhir_cell1_ARRAY, 'r) _menhir_state
      (** State 0138.
        Stack shape : ARRAY.
        Start symbol: <undetermined>. *)

    | MenhirState0139 : ((('s, 'r) _menhir_cell1_ARRAY, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0139.
        Stack shape : ARRAY expr.
        Start symbol: <undetermined>. *)

    | MenhirState0141 : (((('s, 'r) _menhir_cell1_ARRAY, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_RBRACKET, 'r) _menhir_state
      (** State 0141.
        Stack shape : ARRAY expr RBRACKET.
        Start symbol: <undetermined>. *)

    | MenhirState0144 : (((('s, 'r) _menhir_cell1_LBRACKET, 'r) _menhir_cell1_separated_nonempty_list_COMMA_slice_ _menhir_cell0_RBRACKET _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_ty, 'r) _menhir_state
      (** State 0144.
        Stack shape : LBRACKET separated_nonempty_list(COMMA,slice) RBRACKET IDENTIFIER ty.
        Start symbol: <undetermined>. *)

    | MenhirState0146 : ((('s, 'r) _menhir_cell1_LBRACKET, 'r) _menhir_cell1_separated_nonempty_list_COMMA_slice_ _menhir_cell0_RBRACKET _menhir_cell0_IDENTIFIER, 'r) _menhir_state
      (** State 0146.
        Stack shape : LBRACKET separated_nonempty_list(COMMA,slice) RBRACKET IDENTIFIER.
        Start symbol: <undetermined>. *)

    | MenhirState0148 : (((('s, 'r) _menhir_cell1_LBRACKET, 'r) _menhir_cell1_separated_nonempty_list_COMMA_slice_ _menhir_cell0_RBRACKET _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_ty, 'r) _menhir_state
      (** State 0148.
        Stack shape : LBRACKET separated_nonempty_list(COMMA,slice) RBRACKET IDENTIFIER ty.
        Start symbol: <undetermined>. *)

    | MenhirState0150 : (('s, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0150.
        Stack shape : expr.
        Start symbol: <undetermined>. *)

    | MenhirState0151 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_STAR_COLON, 'r) _menhir_state
      (** State 0151.
        Stack shape : expr STAR_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0152 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_STAR_COLON, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0152.
        Stack shape : expr STAR_COLON expr.
        Start symbol: <undetermined>. *)

    | MenhirState0153 : ((((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_STAR_COLON, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_COMMA, 'r) _menhir_state
      (** State 0153.
        Stack shape : expr STAR_COLON expr COMMA.
        Start symbol: <undetermined>. *)

    | MenhirState0155 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_PLUS_COLON, 'r) _menhir_state
      (** State 0155.
        Stack shape : expr PLUS_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0156 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_PLUS_COLON, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0156.
        Stack shape : expr PLUS_COLON expr.
        Start symbol: <undetermined>. *)

    | MenhirState0157 : ((((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_PLUS_COLON, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_COMMA, 'r) _menhir_state
      (** State 0157.
        Stack shape : expr PLUS_COLON expr COMMA.
        Start symbol: <undetermined>. *)

    | MenhirState0159 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_COMMA, 'r) _menhir_state
      (** State 0159.
        Stack shape : expr COMMA.
        Start symbol: <undetermined>. *)

    | MenhirState0161 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_COLON, 'r) _menhir_state
      (** State 0161.
        Stack shape : expr COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0162 : (((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_COLON, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0162.
        Stack shape : expr COLON expr.
        Start symbol: <undetermined>. *)

    | MenhirState0163 : ((((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_COLON, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_COMMA, 'r) _menhir_state
      (** State 0163.
        Stack shape : expr COLON expr COMMA.
        Start symbol: <undetermined>. *)

    | MenhirState0169 : (('s, 'r) _menhir_cell1_LBRACE, 'r) _menhir_state
      (** State 0169.
        Stack shape : LBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0171 : (('s, 'r) _menhir_cell1_IF, 'r) _menhir_state
      (** State 0171.
        Stack shape : IF.
        Start symbol: <undetermined>. *)

    | MenhirState0172 : ((('s, 'r) _menhir_cell1_IF, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0172.
        Stack shape : IF expr.
        Start symbol: <undetermined>. *)

    | MenhirState0173 : (((('s, 'r) _menhir_cell1_IF, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_state
      (** State 0173.
        Stack shape : IF expr THEN.
        Start symbol: <undetermined>. *)

    | MenhirState0174 : ((((('s, 'r) _menhir_cell1_IF, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0174.
        Stack shape : IF expr THEN expr.
        Start symbol: <undetermined>. *)

    | MenhirState0175 : ((((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_ELSIF, 'r) _menhir_state
      (** State 0175.
        Stack shape : expr THEN expr ELSIF.
        Start symbol: <undetermined>. *)

    | MenhirState0176 : (((((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_ELSIF, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0176.
        Stack shape : expr THEN expr ELSIF expr.
        Start symbol: <undetermined>. *)

    | MenhirState0177 : ((((((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_ELSIF, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_state
      (** State 0177.
        Stack shape : expr THEN expr ELSIF expr THEN.
        Start symbol: <undetermined>. *)

    | MenhirState0178 : (((((((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_ELSIF, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0178.
        Stack shape : expr THEN expr ELSIF expr THEN expr.
        Start symbol: <undetermined>. *)

    | MenhirState0179 : ((((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_ELSE, 'r) _menhir_state
      (** State 0179.
        Stack shape : expr THEN expr ELSE.
        Start symbol: <undetermined>. *)

    | MenhirState0180 : (((((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_ELSE, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0180.
        Stack shape : expr THEN expr ELSE expr.
        Start symbol: <undetermined>. *)

    | MenhirState0184 : (('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_state
      (** State 0184.
        Stack shape : IDENTIFIER LPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0188 : (('s, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0188.
        Stack shape : expr.
        Start symbol: <undetermined>. *)

    | MenhirState0189 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_COMMA, 'r) _menhir_state
      (** State 0189.
        Stack shape : expr COMMA.
        Start symbol: <undetermined>. *)

    | MenhirState0191 : (('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_state
      (** State 0191.
        Stack shape : IDENTIFIER LBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0194 : (('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_state
      (** State 0194.
        Stack shape : IDENTIFIER LBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0199 : ((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_state
      (** State 0199.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0200 : ((('s, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_cell1_LPAR, 'r) _menhir_state
      (** State 0200.
        Stack shape : separated_nonempty_list(COMMA,expr) RBRACE LPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0205 : (('s, 'r) _menhir_cell1_IDENTIFIER, 'r) _menhir_state
      (** State 0205.
        Stack shape : IDENTIFIER.
        Start symbol: <undetermined>. *)

    | MenhirState0206 : ((('s, 'r) _menhir_cell1_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0206.
        Stack shape : IDENTIFIER expr.
        Start symbol: <undetermined>. *)

    | MenhirState0207 : (((('s, 'r) _menhir_cell1_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_COMMA, 'r) _menhir_state
      (** State 0207.
        Stack shape : IDENTIFIER expr COMMA.
        Start symbol: <undetermined>. *)

    | MenhirState0213 : ((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_state
      (** State 0213.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0215 : (('s, 'r) _menhir_cell1_GEQ, 'r) _menhir_state
      (** State 0215.
        Stack shape : GEQ.
        Start symbol: <undetermined>. *)

    | MenhirState0216 : ((('s, 'r) _menhir_cell1_GEQ, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0216.
        Stack shape : GEQ expr.
        Start symbol: <undetermined>. *)

    | MenhirState0218 : (('s, 'r) _menhir_cell1_BNOT, 'r) _menhir_state
      (** State 0218.
        Stack shape : BNOT.
        Start symbol: <undetermined>. *)

    | MenhirState0219 : ((('s, 'r) _menhir_cell1_BNOT, 'r) _menhir_cell1_LBRACE, 'r) _menhir_state
      (** State 0219.
        Stack shape : BNOT LBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0222 : (('s, 'r) _menhir_cell1_ARBITRARY, 'r) _menhir_state
      (** State 0222.
        Stack shape : ARBITRARY.
        Start symbol: <undetermined>. *)

    | MenhirState0224 : (('s, 'r) _menhir_cell1_ARBITRARY, 'r) _menhir_state
      (** State 0224.
        Stack shape : ARBITRARY.
        Start symbol: <undetermined>. *)

    | MenhirState0229 : (('s, 'r) _menhir_cell1_pattern, 'r) _menhir_state
      (** State 0229.
        Stack shape : pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0232 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0232.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0233 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0233.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0234 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0234.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0235 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0235.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0236 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0236.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0237 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0237.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0238 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0238.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0239 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0239.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0240 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0240.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0241 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0241.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0242 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0242.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0243 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0243.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0244 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0244.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0245 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0245.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0246 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0246.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0248 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0248.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0249 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0249.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0250 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0250.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0251 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0251.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0252 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0252.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0253 : (('s, 'r) _menhir_cell1_expr_pattern _menhir_cell0_MINUS, 'r) _menhir_state
      (** State 0253.
        Stack shape : expr_pattern MINUS.
        Start symbol: <undetermined>. *)

    | MenhirState0254 : ((('s, 'r) _menhir_cell1_expr_pattern _menhir_cell0_MINUS, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0254.
        Stack shape : expr_pattern MINUS expr.
        Start symbol: <undetermined>. *)

    | MenhirState0255 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0255.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0256 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0256.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0257 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0257.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0258 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0258.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0260 : (('s, 'r) _menhir_cell1_expr_pattern _menhir_cell0_LEQ, 'r) _menhir_state
      (** State 0260.
        Stack shape : expr_pattern LEQ.
        Start symbol: <undetermined>. *)

    | MenhirState0261 : ((('s, 'r) _menhir_cell1_expr_pattern _menhir_cell0_LEQ, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0261.
        Stack shape : expr_pattern LEQ expr.
        Start symbol: <undetermined>. *)

    | MenhirState0262 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0262.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0266 : (('s, 'r) _menhir_cell1_expr_pattern _menhir_cell0_LBRACE, 'r) _menhir_state
      (** State 0266.
        Stack shape : expr_pattern LBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0270 : (('s, 'r) _menhir_cell1_expr_pattern _menhir_cell0_BNOT _menhir_cell0_LBRACE, 'r) _menhir_state
      (** State 0270.
        Stack shape : expr_pattern BNOT LBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0273 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0273.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0274 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0274.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0275 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0275.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0276 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0276.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0277 : (('s, 'r) _menhir_cell1_expr_pattern _menhir_cell0_GEQ, 'r) _menhir_state
      (** State 0277.
        Stack shape : expr_pattern GEQ.
        Start symbol: <undetermined>. *)

    | MenhirState0278 : ((('s, 'r) _menhir_cell1_expr_pattern _menhir_cell0_GEQ, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0278.
        Stack shape : expr_pattern GEQ expr.
        Start symbol: <undetermined>. *)

    | MenhirState0279 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0279.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0281 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0281.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0282 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0282.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0283 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0283.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0285 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0285.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0289 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0289.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0290 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0290.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0291 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0291.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0292 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0292.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0293 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0293.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0294 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0294.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0295 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0295.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0296 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0296.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0297 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0297.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0298 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0298.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0299 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0299.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0300 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0300.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0301 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0301.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0302 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0302.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0303 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0303.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0304 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_LBRACE, 'r) _menhir_state
      (** State 0304.
        Stack shape : expr_pattern LBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0305 : (((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_LBRACE, 'r) _menhir_cell1_MINUS, 'r) _menhir_state
      (** State 0305.
        Stack shape : expr_pattern LBRACE MINUS.
        Start symbol: <undetermined>. *)

    | MenhirState0310 : (('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_state
      (** State 0310.
        Stack shape : expr_pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0311 : ((('s, 'r) _menhir_cell1_expr_pattern, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0311.
        Stack shape : expr_pattern expr.
        Start symbol: <undetermined>. *)

    | MenhirState0312 : ((('s, 'r) _menhir_cell1_BNOT, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0312.
        Stack shape : BNOT expr.
        Start symbol: <undetermined>. *)

    | MenhirState0316 : ((('s, 'r) _menhir_cell1_LPAR, 'r) _menhir_cell1_pattern, 'r) _menhir_state
      (** State 0316.
        Stack shape : LPAR pattern.
        Start symbol: <undetermined>. *)

    | MenhirState0324 : ((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_IN _menhir_cell0_BNOT _menhir_cell0_LBRACE, 'r) _menhir_state
      (** State 0324.
        Stack shape : expr IN BNOT LBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0327 : (((('s, 'r) _menhir_cell1_COLON, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_COMMA, 'r) _menhir_state
      (** State 0327.
        Stack shape : COLON expr COMMA.
        Start symbol: <undetermined>. *)

    | MenhirState0335 : (('s, 'r) _menhir_cell1_ty, 'r) _menhir_state
      (** State 0335.
        Stack shape : ty.
        Start symbol: <undetermined>. *)

    | MenhirState0340 : (('s, 'r) _menhir_cell1_ARBITRARY, 'r) _menhir_state
      (** State 0340.
        Stack shape : ARBITRARY.
        Start symbol: <undetermined>. *)

    | MenhirState0342 : ((('s, 'r) _menhir_cell1_BNOT, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0342.
        Stack shape : BNOT expr.
        Start symbol: <undetermined>. *)

    | MenhirState0345 : ((('s, 'r) _menhir_cell1_IF, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0345.
        Stack shape : IF expr.
        Start symbol: <undetermined>. *)

    | MenhirState0346 : (((('s, 'r) _menhir_cell1_IF, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_state
      (** State 0346.
        Stack shape : IF expr THEN.
        Start symbol: <undetermined>. *)

    | MenhirState0347 : ((((('s, 'r) _menhir_cell1_IF, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0347.
        Stack shape : IF expr THEN expr.
        Start symbol: <undetermined>. *)

    | MenhirState0349 : ((('s, 'r) _menhir_cell1_LPAR, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0349.
        Stack shape : LPAR expr.
        Start symbol: <undetermined>. *)

    | MenhirState0351 : (((('s, 'r) _menhir_cell1_LPAR, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_COMMA, 'r) _menhir_state
      (** State 0351.
        Stack shape : LPAR expr COMMA.
        Start symbol: <undetermined>. *)

    | MenhirState0354 : ((('s, 'r) _menhir_cell1_NOT, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0354.
        Stack shape : NOT expr.
        Start symbol: <undetermined>. *)

    | MenhirState0355 : ((('s, _menhir_box_opn) _menhir_cell1_WHILE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 0355.
        Stack shape : WHILE expr.
        Start symbol: opn. *)

    | MenhirState0356 : (((('s, _menhir_box_opn) _menhir_cell1_WHILE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_state
      (** State 0356.
        Stack shape : WHILE expr LOOPLIMIT.
        Start symbol: opn. *)

    | MenhirState0357 : ((((('s, _menhir_box_opn) _menhir_cell1_WHILE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 0357.
        Stack shape : WHILE expr LOOPLIMIT expr.
        Start symbol: opn. *)

    | MenhirState0358 : (((((('s, _menhir_box_opn) _menhir_cell1_WHILE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_state
      (** State 0358.
        Stack shape : WHILE expr LOOPLIMIT expr DO.
        Start symbol: opn. *)

    | MenhirState0359 : (('s, 'r) _menhir_cell1_WHILE, 'r) _menhir_state
      (** State 0359.
        Stack shape : WHILE.
        Start symbol: <undetermined>. *)

    | MenhirState0360 : ((('s, 'r) _menhir_cell1_WHILE, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0360.
        Stack shape : WHILE expr.
        Start symbol: <undetermined>. *)

    | MenhirState0361 : (((('s, 'r) _menhir_cell1_WHILE, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LOOPLIMIT, 'r) _menhir_state
      (** State 0361.
        Stack shape : WHILE expr LOOPLIMIT.
        Start symbol: <undetermined>. *)

    | MenhirState0362 : ((((('s, 'r) _menhir_cell1_WHILE, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LOOPLIMIT, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0362.
        Stack shape : WHILE expr LOOPLIMIT expr.
        Start symbol: <undetermined>. *)

    | MenhirState0363 : (((((('s, 'r) _menhir_cell1_WHILE, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LOOPLIMIT, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_state
      (** State 0363.
        Stack shape : WHILE expr LOOPLIMIT expr DO.
        Start symbol: <undetermined>. *)

    | MenhirState0364 : (('s, 'r) _menhir_cell1_VAR, 'r) _menhir_state
      (** State 0364.
        Stack shape : VAR.
        Start symbol: <undetermined>. *)

    | MenhirState0366 : (('s, 'r) _menhir_cell1_MINUS, 'r) _menhir_state
      (** State 0366.
        Stack shape : MINUS.
        Start symbol: <undetermined>. *)

    | MenhirState0368 : (('s, 'r) _menhir_cell1_MINUS, 'r) _menhir_state
      (** State 0368.
        Stack shape : MINUS.
        Start symbol: <undetermined>. *)

    | MenhirState0370 : (('s, 'r) _menhir_cell1_LPAR, 'r) _menhir_state
      (** State 0370.
        Stack shape : LPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0372 : (('s, 'r) _menhir_cell1_IDENTIFIER, 'r) _menhir_state
      (** State 0372.
        Stack shape : IDENTIFIER.
        Start symbol: <undetermined>. *)

    | MenhirState0374 : (('s, 'r) _menhir_cell1_IDENTIFIER, 'r) _menhir_state
      (** State 0374.
        Stack shape : IDENTIFIER.
        Start symbol: <undetermined>. *)

    | MenhirState0377 : ((('s, 'r) _menhir_cell1_LPAR, 'r) _menhir_cell1_decl_item, 'r) _menhir_state
      (** State 0377.
        Stack shape : LPAR decl_item.
        Start symbol: <undetermined>. *)

    | MenhirState0380 : (((('s, 'r) _menhir_cell1_LPAR, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_separated_nonempty_list_COMMA_decl_item_ _menhir_cell0_RPAR, 'r) _menhir_state
      (** State 0380.
        Stack shape : LPAR decl_item separated_nonempty_list(COMMA,decl_item) RPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0382 : (((('s, 'r) _menhir_cell1_LPAR, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_separated_nonempty_list_COMMA_decl_item_ _menhir_cell0_RPAR, 'r) _menhir_state
      (** State 0382.
        Stack shape : LPAR decl_item separated_nonempty_list(COMMA,decl_item) RPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0385 : ((('s, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_decl_item, 'r) _menhir_state
      (** State 0385.
        Stack shape : decl_item decl_item.
        Start symbol: <undetermined>. *)

    | MenhirState0388 : ((('s, 'r) _menhir_cell1_VAR, 'r) _menhir_cell1_IDENTIFIER, 'r) _menhir_state
      (** State 0388.
        Stack shape : VAR IDENTIFIER.
        Start symbol: <undetermined>. *)

    | MenhirState0390 : (((('s, 'r) _menhir_cell1_VAR, 'r) _menhir_cell1_IDENTIFIER, 'r) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_, 'r) _menhir_state
      (** State 0390.
        Stack shape : VAR IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER).
        Start symbol: <undetermined>. *)

    | MenhirState0392 : ((((('s, 'r) _menhir_cell1_VAR, 'r) _menhir_cell1_IDENTIFIER, 'r) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_, 'r) _menhir_cell1_ty _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0392.
        Stack shape : VAR IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER) ty SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0396 : (('s, 'r) _menhir_cell1_UNREACHABLE _menhir_cell0_LPAR _menhir_cell0_RPAR _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0396.
        Stack shape : UNREACHABLE LPAR RPAR SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0397 : (('s, 'r) _menhir_cell1_TRY, 'r) _menhir_state
      (** State 0397.
        Stack shape : TRY.
        Start symbol: <undetermined>. *)

    | MenhirState0398 : (('s, 'r) _menhir_cell1_THROW, 'r) _menhir_state
      (** State 0398.
        Stack shape : THROW.
        Start symbol: <undetermined>. *)

    | MenhirState0399 : ((('s, 'r) _menhir_cell1_THROW, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0399.
        Stack shape : THROW SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0400 : (('s, 'r) _menhir_cell1_RETURN, 'r) _menhir_state
      (** State 0400.
        Stack shape : RETURN.
        Start symbol: <undetermined>. *)

    | MenhirState0401 : ((('s, 'r) _menhir_cell1_RETURN, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0401.
        Stack shape : RETURN SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0402 : (('s, 'r) _menhir_cell1_REPEAT, 'r) _menhir_state
      (** State 0402.
        Stack shape : REPEAT.
        Start symbol: <undetermined>. *)

    | MenhirState0404 : (('s, 'r) _menhir_cell1_PRINT _menhir_cell0_LPAR, 'r) _menhir_state
      (** State 0404.
        Stack shape : PRINT LPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0406 : ((('s, 'r) _menhir_cell1_PRINT _menhir_cell0_LPAR, 'r) _menhir_cell1_RPAR _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0406.
        Stack shape : PRINT LPAR RPAR SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0408 : (('s, 'r) _menhir_cell1_PRAGMA _menhir_cell0_IDENTIFIER, 'r) _menhir_state
      (** State 0408.
        Stack shape : PRAGMA IDENTIFIER.
        Start symbol: <undetermined>. *)

    | MenhirState0409 : ((('s, 'r) _menhir_cell1_PRAGMA _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0409.
        Stack shape : PRAGMA IDENTIFIER SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0411 : (('s, 'r) _menhir_cell1_PASS _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0411.
        Stack shape : PASS SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0413 : (('s, 'r) _menhir_cell1_LPAR, 'r) _menhir_state
      (** State 0413.
        Stack shape : LPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0416 : (('s, 'r) _menhir_cell1_lexpr_atom_desc, 'r) _menhir_state
      (** State 0416.
        Stack shape : lexpr_atom_desc.
        Start symbol: <undetermined>. *)

    | MenhirState0417 : ((('s, 'r) _menhir_cell1_lexpr_atom_desc, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0417.
        Stack shape : lexpr_atom_desc expr.
        Start symbol: <undetermined>. *)

    | MenhirState0419 : (('s, 'r) _menhir_cell1_lexpr_atom_desc, 'r) _menhir_state
      (** State 0419.
        Stack shape : lexpr_atom_desc.
        Start symbol: <undetermined>. *)

    | MenhirState0423 : (('s, 'r) _menhir_cell1_lexpr_atom_desc, 'r) _menhir_state
      (** State 0423.
        Stack shape : lexpr_atom_desc.
        Start symbol: <undetermined>. *)

    | MenhirState0429 : ((('s, 'r) _menhir_cell1_LPAR, 'r) _menhir_cell1_lexpr, 'r) _menhir_state
      (** State 0429.
        Stack shape : LPAR lexpr.
        Start symbol: <undetermined>. *)

    | MenhirState0433 : ((('s, 'r) _menhir_cell1_lexpr, 'r) _menhir_cell1_lexpr, 'r) _menhir_state
      (** State 0433.
        Stack shape : lexpr lexpr.
        Start symbol: <undetermined>. *)

    | MenhirState0435 : (('s, 'r) _menhir_cell1_LET, 'r) _menhir_state
      (** State 0435.
        Stack shape : LET.
        Start symbol: <undetermined>. *)

    | MenhirState0437 : ((('s, 'r) _menhir_cell1_LET, 'r) _menhir_cell1_decl_item, 'r) _menhir_state
      (** State 0437.
        Stack shape : LET decl_item.
        Start symbol: <undetermined>. *)

    | MenhirState0439 : (((('s, 'r) _menhir_cell1_LET, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_state
      (** State 0439.
        Stack shape : LET decl_item IDENTIFIER LBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0441 : ((((('s, 'r) _menhir_cell1_LET, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_RBRACE _menhir_cell0_LPAR, 'r) _menhir_state
      (** State 0441.
        Stack shape : LET decl_item IDENTIFIER LBRACE RBRACE LPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0443 : (((((('s, 'r) _menhir_cell1_LET, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_RBRACE _menhir_cell0_LPAR, 'r) _menhir_cell1_RPAR _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0443.
        Stack shape : LET decl_item IDENTIFIER LBRACE RBRACE LPAR RPAR SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0444 : (('s, 'r) _menhir_cell1_IF, 'r) _menhir_state
      (** State 0444.
        Stack shape : IF.
        Start symbol: <undetermined>. *)

    | MenhirState0445 : ((('s, 'r) _menhir_cell1_IF, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0445.
        Stack shape : IF expr.
        Start symbol: <undetermined>. *)

    | MenhirState0446 : (((('s, 'r) _menhir_cell1_IF, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_state
      (** State 0446.
        Stack shape : IF expr THEN.
        Start symbol: <undetermined>. *)

    | MenhirState0448 : (('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_state
      (** State 0448.
        Stack shape : IDENTIFIER LPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0450 : ((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_RPAR _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0450.
        Stack shape : IDENTIFIER LPAR RPAR SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0453 : (('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_state
      (** State 0453.
        Stack shape : FOR IDENTIFIER.
        Start symbol: <undetermined>. *)

    | MenhirState0454 : ((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0454.
        Stack shape : FOR IDENTIFIER expr.
        Start symbol: <undetermined>. *)

    | MenhirState0455 : (((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_TO, 'r) _menhir_state
      (** State 0455.
        Stack shape : FOR IDENTIFIER expr TO.
        Start symbol: <undetermined>. *)

    | MenhirState0456 : ((((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_TO, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0456.
        Stack shape : FOR IDENTIFIER expr TO expr.
        Start symbol: <undetermined>. *)

    | MenhirState0457 : (((((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_TO, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LOOPLIMIT, 'r) _menhir_state
      (** State 0457.
        Stack shape : FOR IDENTIFIER expr TO expr LOOPLIMIT.
        Start symbol: <undetermined>. *)

    | MenhirState0458 : ((((((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_TO, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LOOPLIMIT, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0458.
        Stack shape : FOR IDENTIFIER expr TO expr LOOPLIMIT expr.
        Start symbol: <undetermined>. *)

    | MenhirState0459 : (((((((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_TO, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LOOPLIMIT, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_state
      (** State 0459.
        Stack shape : FOR IDENTIFIER expr TO expr LOOPLIMIT expr DO.
        Start symbol: <undetermined>. *)

    | MenhirState0461 : (('s, 'r) _menhir_cell1_DEBUG _menhir_cell0_LPAR, 'r) _menhir_state
      (** State 0461.
        Stack shape : DEBUG LPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0463 : ((('s, 'r) _menhir_cell1_DEBUG _menhir_cell0_LPAR, 'r) _menhir_cell1_RPAR _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0463.
        Stack shape : DEBUG LPAR RPAR SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0464 : (('s, 'r) _menhir_cell1_CONSTANT, 'r) _menhir_state
      (** State 0464.
        Stack shape : CONSTANT.
        Start symbol: <undetermined>. *)

    | MenhirState0466 : ((('s, 'r) _menhir_cell1_CONSTANT, 'r) _menhir_cell1_decl_item, 'r) _menhir_state
      (** State 0466.
        Stack shape : CONSTANT decl_item.
        Start symbol: <undetermined>. *)

    | MenhirState0468 : (((('s, 'r) _menhir_cell1_CONSTANT, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_state
      (** State 0468.
        Stack shape : CONSTANT decl_item IDENTIFIER LBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0470 : ((((('s, 'r) _menhir_cell1_CONSTANT, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_RBRACE _menhir_cell0_LPAR, 'r) _menhir_state
      (** State 0470.
        Stack shape : CONSTANT decl_item IDENTIFIER LBRACE RBRACE LPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0472 : (((((('s, 'r) _menhir_cell1_CONSTANT, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_RBRACE _menhir_cell0_LPAR, 'r) _menhir_cell1_RPAR _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0472.
        Stack shape : CONSTANT decl_item IDENTIFIER LBRACE RBRACE LPAR RPAR SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0473 : (('s, 'r) _menhir_cell1_CASE, 'r) _menhir_state
      (** State 0473.
        Stack shape : CASE.
        Start symbol: <undetermined>. *)

    | MenhirState0474 : ((('s, 'r) _menhir_cell1_CASE, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0474.
        Stack shape : CASE expr.
        Start symbol: <undetermined>. *)

    | MenhirState0475 : (((('s, 'r) _menhir_cell1_CASE, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_OF, 'r) _menhir_state
      (** State 0475.
        Stack shape : CASE expr OF.
        Start symbol: <undetermined>. *)

    | MenhirState0476 : (('s, 'r) _menhir_cell1_WHEN, 'r) _menhir_state
      (** State 0476.
        Stack shape : WHEN.
        Start symbol: <undetermined>. *)

    | MenhirState0478 : ((('s, 'r) _menhir_cell1_WHEN, 'r) _menhir_cell1_separated_nonempty_list_COMMA_pattern_, 'r) _menhir_state
      (** State 0478.
        Stack shape : WHEN separated_nonempty_list(COMMA,pattern).
        Start symbol: <undetermined>. *)

    | MenhirState0479 : (((('s, 'r) _menhir_cell1_WHEN, 'r) _menhir_cell1_separated_nonempty_list_COMMA_pattern_, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0479.
        Stack shape : WHEN separated_nonempty_list(COMMA,pattern) expr.
        Start symbol: <undetermined>. *)

    | MenhirState0480 : ((((('s, 'r) _menhir_cell1_WHEN, 'r) _menhir_cell1_separated_nonempty_list_COMMA_pattern_, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_ARROW, 'r) _menhir_state
      (** State 0480.
        Stack shape : WHEN separated_nonempty_list(COMMA,pattern) expr ARROW.
        Start symbol: <undetermined>. *)

    | MenhirState0481 : (('s, 'r) _menhir_cell1_ASSERT, 'r) _menhir_state
      (** State 0481.
        Stack shape : ASSERT.
        Start symbol: <undetermined>. *)

    | MenhirState0482 : ((('s, 'r) _menhir_cell1_ASSERT, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0482.
        Stack shape : ASSERT expr.
        Start symbol: <undetermined>. *)

    | MenhirState0483 : (((('s, 'r) _menhir_cell1_ASSERT, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0483.
        Stack shape : ASSERT expr SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0485 : (('s, 'r) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, 'r) _menhir_state
      (** State 0485.
        Stack shape : ARROBASE_LOOPLIMIT LPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0486 : ((('s, 'r) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0486.
        Stack shape : ARROBASE_LOOPLIMIT LPAR expr.
        Start symbol: <undetermined>. *)

    | MenhirState0488 : (((('s, 'r) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_RPAR _menhir_cell0_WHILE, 'r) _menhir_state
      (** State 0488.
        Stack shape : ARROBASE_LOOPLIMIT LPAR expr RPAR WHILE.
        Start symbol: <undetermined>. *)

    | MenhirState0489 : ((((('s, 'r) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_RPAR _menhir_cell0_WHILE, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0489.
        Stack shape : ARROBASE_LOOPLIMIT LPAR expr RPAR WHILE expr.
        Start symbol: <undetermined>. *)

    | MenhirState0490 : (((((('s, 'r) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_RPAR _menhir_cell0_WHILE, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_state
      (** State 0490.
        Stack shape : ARROBASE_LOOPLIMIT LPAR expr RPAR WHILE expr DO.
        Start symbol: <undetermined>. *)

    | MenhirState0491 : ((((((('s, 'r) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_RPAR _menhir_cell0_WHILE, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_state
      (** State 0491.
        Stack shape : ARROBASE_LOOPLIMIT LPAR expr RPAR WHILE expr DO nonempty_list(stmt).
        Start symbol: <undetermined>. *)

    | MenhirState0495 : (((((((('s, 'r) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_RPAR _menhir_cell0_WHILE, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_end_semicolon, 'r) _menhir_state
      (** State 0495.
        Stack shape : ARROBASE_LOOPLIMIT LPAR expr RPAR WHILE expr DO nonempty_list(stmt) end_semicolon.
        Start symbol: <undetermined>. *)

    | MenhirState0498 : (('s, 'r) _menhir_cell1_lexpr, 'r) _menhir_state
      (** State 0498.
        Stack shape : lexpr.
        Start symbol: <undetermined>. *)

    | MenhirState0499 : ((('s, 'r) _menhir_cell1_lexpr, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0499.
        Stack shape : lexpr expr.
        Start symbol: <undetermined>. *)

    | MenhirState0500 : (((('s, 'r) _menhir_cell1_lexpr, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0500.
        Stack shape : lexpr expr SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0502 : (((('s, 'r) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_RPAR _menhir_cell0_REPEAT, 'r) _menhir_state
      (** State 0502.
        Stack shape : ARROBASE_LOOPLIMIT LPAR expr RPAR REPEAT.
        Start symbol: <undetermined>. *)

    | MenhirState0504 : ((((('s, 'r) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_RPAR _menhir_cell0_REPEAT, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_state
      (** State 0504.
        Stack shape : ARROBASE_LOOPLIMIT LPAR expr RPAR REPEAT nonempty_list(stmt).
        Start symbol: <undetermined>. *)

    | MenhirState0505 : (((((('s, 'r) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_RPAR _menhir_cell0_REPEAT, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0505.
        Stack shape : ARROBASE_LOOPLIMIT LPAR expr RPAR REPEAT nonempty_list(stmt) expr.
        Start symbol: <undetermined>. *)

    | MenhirState0506 : ((((((('s, 'r) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_RPAR _menhir_cell0_REPEAT, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0506.
        Stack shape : ARROBASE_LOOPLIMIT LPAR expr RPAR REPEAT nonempty_list(stmt) expr SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0509 : (((((('s, 'r) _menhir_cell1_WHEN, 'r) _menhir_cell1_separated_nonempty_list_COMMA_pattern_, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_ARROW, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_state
      (** State 0509.
        Stack shape : WHEN separated_nonempty_list(COMMA,pattern) expr ARROW nonempty_list(stmt).
        Start symbol: <undetermined>. *)

    | MenhirState0511 : ((((((('s, 'r) _menhir_cell1_WHEN, 'r) _menhir_cell1_separated_nonempty_list_COMMA_pattern_, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_ARROW, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_OTHERWISE, 'r) _menhir_state
      (** State 0511.
        Stack shape : WHEN separated_nonempty_list(COMMA,pattern) expr ARROW nonempty_list(stmt) OTHERWISE.
        Start symbol: <undetermined>. *)

    | MenhirState0514 : ((('s, 'r) _menhir_cell1_WHEN, 'r) _menhir_cell1_separated_nonempty_list_COMMA_pattern_, 'r) _menhir_state
      (** State 0514.
        Stack shape : WHEN separated_nonempty_list(COMMA,pattern).
        Start symbol: <undetermined>. *)

    | MenhirState0515 : (((('s, 'r) _menhir_cell1_WHEN, 'r) _menhir_cell1_separated_nonempty_list_COMMA_pattern_, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_state
      (** State 0515.
        Stack shape : WHEN separated_nonempty_list(COMMA,pattern) nonempty_list(stmt).
        Start symbol: <undetermined>. *)

    | MenhirState0517 : ((((('s, 'r) _menhir_cell1_WHEN, 'r) _menhir_cell1_separated_nonempty_list_COMMA_pattern_, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_OTHERWISE, 'r) _menhir_state
      (** State 0517.
        Stack shape : WHEN separated_nonempty_list(COMMA,pattern) nonempty_list(stmt) OTHERWISE.
        Start symbol: <undetermined>. *)

    | MenhirState0520 : ((((('s, 'r) _menhir_cell1_CASE, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_OF, 'r) _menhir_cell1_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__, 'r) _menhir_state
      (** State 0520.
        Stack shape : CASE expr OF nlist_opt_terminated(annotated(case_alt),annotated(case_otherwise)).
        Start symbol: <undetermined>. *)

    | MenhirState0521 : (((((('s, 'r) _menhir_cell1_CASE, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_OF, 'r) _menhir_cell1_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__, 'r) _menhir_cell1_end_semicolon, 'r) _menhir_state
      (** State 0521.
        Stack shape : CASE expr OF nlist_opt_terminated(annotated(case_alt),annotated(case_otherwise)) end_semicolon.
        Start symbol: <undetermined>. *)

    | MenhirState0526 : (((((('s, 'r) _menhir_cell1_CONSTANT, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_RBRACE _menhir_cell0_LPAR, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0526.
        Stack shape : CONSTANT decl_item IDENTIFIER LBRACE RBRACE LPAR separated_nonempty_list(COMMA,expr) RPAR SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0528 : ((((('s, 'r) _menhir_cell1_CONSTANT, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_COMMA, 'r) _menhir_state
      (** State 0528.
        Stack shape : CONSTANT decl_item IDENTIFIER LBRACE COMMA.
        Start symbol: <undetermined>. *)

    | MenhirState0530 : (((((('s, 'r) _menhir_cell1_CONSTANT, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_COMMA, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_state
      (** State 0530.
        Stack shape : CONSTANT decl_item IDENTIFIER LBRACE COMMA separated_nonempty_list(COMMA,expr) RBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0532 : ((((((('s, 'r) _menhir_cell1_CONSTANT, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_COMMA, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_cell1_loption_plist_expr__ _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0532.
        Stack shape : CONSTANT decl_item IDENTIFIER LBRACE COMMA separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)) SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0534 : (((('s, 'r) _menhir_cell1_CONSTANT, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0534.
        Stack shape : CONSTANT decl_item expr.
        Start symbol: <undetermined>. *)

    | MenhirState0535 : ((((('s, 'r) _menhir_cell1_CONSTANT, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0535.
        Stack shape : CONSTANT decl_item expr SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0540 : ((('s, 'r) _menhir_cell1_DEBUG _menhir_cell0_LPAR, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0540.
        Stack shape : DEBUG LPAR separated_nonempty_list(COMMA,expr) RPAR SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0542 : ((((((((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_TO, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LOOPLIMIT, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_state
      (** State 0542.
        Stack shape : FOR IDENTIFIER expr TO expr LOOPLIMIT expr DO nonempty_list(stmt).
        Start symbol: <undetermined>. *)

    | MenhirState0543 : (((((((((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_TO, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LOOPLIMIT, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_end_semicolon, 'r) _menhir_state
      (** State 0543.
        Stack shape : FOR IDENTIFIER expr TO expr LOOPLIMIT expr DO nonempty_list(stmt) end_semicolon.
        Start symbol: <undetermined>. *)

    | MenhirState0545 : (((((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_TO, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_state
      (** State 0545.
        Stack shape : FOR IDENTIFIER expr TO expr DO.
        Start symbol: <undetermined>. *)

    | MenhirState0546 : ((((((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_TO, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_state
      (** State 0546.
        Stack shape : FOR IDENTIFIER expr TO expr DO nonempty_list(stmt).
        Start symbol: <undetermined>. *)

    | MenhirState0547 : (((((((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_TO, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_end_semicolon, 'r) _menhir_state
      (** State 0547.
        Stack shape : FOR IDENTIFIER expr TO expr DO nonempty_list(stmt) end_semicolon.
        Start symbol: <undetermined>. *)

    | MenhirState0549 : (((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DOWNTO, 'r) _menhir_state
      (** State 0549.
        Stack shape : FOR IDENTIFIER expr DOWNTO.
        Start symbol: <undetermined>. *)

    | MenhirState0550 : ((((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DOWNTO, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0550.
        Stack shape : FOR IDENTIFIER expr DOWNTO expr.
        Start symbol: <undetermined>. *)

    | MenhirState0551 : (((((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DOWNTO, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LOOPLIMIT, 'r) _menhir_state
      (** State 0551.
        Stack shape : FOR IDENTIFIER expr DOWNTO expr LOOPLIMIT.
        Start symbol: <undetermined>. *)

    | MenhirState0552 : ((((((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DOWNTO, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LOOPLIMIT, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0552.
        Stack shape : FOR IDENTIFIER expr DOWNTO expr LOOPLIMIT expr.
        Start symbol: <undetermined>. *)

    | MenhirState0553 : (((((((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DOWNTO, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LOOPLIMIT, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_state
      (** State 0553.
        Stack shape : FOR IDENTIFIER expr DOWNTO expr LOOPLIMIT expr DO.
        Start symbol: <undetermined>. *)

    | MenhirState0554 : ((((((((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DOWNTO, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LOOPLIMIT, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_state
      (** State 0554.
        Stack shape : FOR IDENTIFIER expr DOWNTO expr LOOPLIMIT expr DO nonempty_list(stmt).
        Start symbol: <undetermined>. *)

    | MenhirState0555 : (((((((((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DOWNTO, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LOOPLIMIT, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_end_semicolon, 'r) _menhir_state
      (** State 0555.
        Stack shape : FOR IDENTIFIER expr DOWNTO expr LOOPLIMIT expr DO nonempty_list(stmt) end_semicolon.
        Start symbol: <undetermined>. *)

    | MenhirState0557 : (((((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DOWNTO, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_state
      (** State 0557.
        Stack shape : FOR IDENTIFIER expr DOWNTO expr DO.
        Start symbol: <undetermined>. *)

    | MenhirState0558 : ((((((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DOWNTO, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_state
      (** State 0558.
        Stack shape : FOR IDENTIFIER expr DOWNTO expr DO nonempty_list(stmt).
        Start symbol: <undetermined>. *)

    | MenhirState0559 : (((((((('s, 'r) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DOWNTO, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_end_semicolon, 'r) _menhir_state
      (** State 0559.
        Stack shape : FOR IDENTIFIER expr DOWNTO expr DO nonempty_list(stmt) end_semicolon.
        Start symbol: <undetermined>. *)

    | MenhirState0562 : ((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_RPAR, 'r) _menhir_state
      (** State 0562.
        Stack shape : IDENTIFIER LPAR RPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0563 : (((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_RPAR, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0563.
        Stack shape : IDENTIFIER LPAR RPAR expr.
        Start symbol: <undetermined>. *)

    | MenhirState0564 : ((((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_RPAR, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0564.
        Stack shape : IDENTIFIER LPAR RPAR expr SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0569 : ((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, 'r) _menhir_state
      (** State 0569.
        Stack shape : IDENTIFIER LPAR RPAR IDENTIFIER.
        Start symbol: <undetermined>. *)

    | MenhirState0572 : (((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET, 'r) _menhir_state
      (** State 0572.
        Stack shape : IDENTIFIER LPAR RPAR IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER) RBRACKET.
        Start symbol: <undetermined>. *)

    | MenhirState0573 : ((((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0573.
        Stack shape : IDENTIFIER LPAR RPAR IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER) RBRACKET expr.
        Start symbol: <undetermined>. *)

    | MenhirState0574 : (((((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0574.
        Stack shape : IDENTIFIER LPAR RPAR IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER) RBRACKET expr SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0577 : ((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, 'r) _menhir_state
      (** State 0577.
        Stack shape : IDENTIFIER LPAR RPAR IDENTIFIER.
        Start symbol: <undetermined>. *)

    | MenhirState0578 : (((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0578.
        Stack shape : IDENTIFIER LPAR RPAR IDENTIFIER expr.
        Start symbol: <undetermined>. *)

    | MenhirState0579 : ((((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0579.
        Stack shape : IDENTIFIER LPAR RPAR IDENTIFIER expr SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0583 : ((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0583.
        Stack shape : IDENTIFIER LPAR separated_nonempty_list(COMMA,expr) RPAR SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0585 : ((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR, 'r) _menhir_state
      (** State 0585.
        Stack shape : IDENTIFIER LPAR separated_nonempty_list(COMMA,expr) RPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0586 : (((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0586.
        Stack shape : IDENTIFIER LPAR separated_nonempty_list(COMMA,expr) RPAR expr.
        Start symbol: <undetermined>. *)

    | MenhirState0587 : ((((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0587.
        Stack shape : IDENTIFIER LPAR separated_nonempty_list(COMMA,expr) RPAR expr SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0592 : ((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, 'r) _menhir_state
      (** State 0592.
        Stack shape : IDENTIFIER LPAR separated_nonempty_list(COMMA,expr) RPAR IDENTIFIER.
        Start symbol: <undetermined>. *)

    | MenhirState0595 : (((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET, 'r) _menhir_state
      (** State 0595.
        Stack shape : IDENTIFIER LPAR separated_nonempty_list(COMMA,expr) RPAR IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER) RBRACKET.
        Start symbol: <undetermined>. *)

    | MenhirState0596 : ((((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0596.
        Stack shape : IDENTIFIER LPAR separated_nonempty_list(COMMA,expr) RPAR IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER) RBRACKET expr.
        Start symbol: <undetermined>. *)

    | MenhirState0597 : (((((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0597.
        Stack shape : IDENTIFIER LPAR separated_nonempty_list(COMMA,expr) RPAR IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER) RBRACKET expr SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0600 : ((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, 'r) _menhir_state
      (** State 0600.
        Stack shape : IDENTIFIER LPAR separated_nonempty_list(COMMA,expr) RPAR IDENTIFIER.
        Start symbol: <undetermined>. *)

    | MenhirState0601 : (((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0601.
        Stack shape : IDENTIFIER LPAR separated_nonempty_list(COMMA,expr) RPAR IDENTIFIER expr.
        Start symbol: <undetermined>. *)

    | MenhirState0602 : ((((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0602.
        Stack shape : IDENTIFIER LPAR separated_nonempty_list(COMMA,expr) RPAR IDENTIFIER expr SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0604 : (('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_state
      (** State 0604.
        Stack shape : IDENTIFIER LBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0606 : ((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_state
      (** State 0606.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0608 : (((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_cell1_loption_plist_expr__ _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0608.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)) SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0610 : (((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_cell1_loption_plist_expr__, 'r) _menhir_state
      (** State 0610.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)).
        Start symbol: <undetermined>. *)

    | MenhirState0611 : ((((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_cell1_loption_plist_expr__, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0611.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)) expr.
        Start symbol: <undetermined>. *)

    | MenhirState0612 : (((((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_cell1_loption_plist_expr__, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0612.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)) expr SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0617 : (((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER, 'r) _menhir_state
      (** State 0617.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)) IDENTIFIER.
        Start symbol: <undetermined>. *)

    | MenhirState0620 : ((((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET, 'r) _menhir_state
      (** State 0620.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)) IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER) RBRACKET.
        Start symbol: <undetermined>. *)

    | MenhirState0621 : (((((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0621.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)) IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER) RBRACKET expr.
        Start symbol: <undetermined>. *)

    | MenhirState0622 : ((((((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0622.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)) IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER) RBRACKET expr SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0625 : (((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER, 'r) _menhir_state
      (** State 0625.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)) IDENTIFIER.
        Start symbol: <undetermined>. *)

    | MenhirState0626 : ((((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0626.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)) IDENTIFIER expr.
        Start symbol: <undetermined>. *)

    | MenhirState0627 : (((((('s, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0627.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)) IDENTIFIER expr SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0629 : ((((('s, 'r) _menhir_cell1_IF, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_state
      (** State 0629.
        Stack shape : IF expr THEN nonempty_list(stmt).
        Start symbol: <undetermined>. *)

    | MenhirState0630 : ((((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_ELSIF, 'r) _menhir_state
      (** State 0630.
        Stack shape : expr THEN nonempty_list(stmt) ELSIF.
        Start symbol: <undetermined>. *)

    | MenhirState0631 : (((((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_ELSIF, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0631.
        Stack shape : expr THEN nonempty_list(stmt) ELSIF expr.
        Start symbol: <undetermined>. *)

    | MenhirState0632 : ((((((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_ELSIF, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_state
      (** State 0632.
        Stack shape : expr THEN nonempty_list(stmt) ELSIF expr THEN.
        Start symbol: <undetermined>. *)

    | MenhirState0633 : (((((((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_ELSIF, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_state
      (** State 0633.
        Stack shape : expr THEN nonempty_list(stmt) ELSIF expr THEN nonempty_list(stmt).
        Start symbol: <undetermined>. *)

    | MenhirState0634 : ((((('s, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_ELSE, 'r) _menhir_state
      (** State 0634.
        Stack shape : expr THEN nonempty_list(stmt) ELSE.
        Start symbol: <undetermined>. *)

    | MenhirState0637 : (((((('s, 'r) _menhir_cell1_IF, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_s_else, 'r) _menhir_state
      (** State 0637.
        Stack shape : IF expr THEN nonempty_list(stmt) s_else.
        Start symbol: <undetermined>. *)

    | MenhirState0638 : ((((((('s, 'r) _menhir_cell1_IF, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_THEN, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_s_else, 'r) _menhir_cell1_end_semicolon, 'r) _menhir_state
      (** State 0638.
        Stack shape : IF expr THEN nonempty_list(stmt) s_else end_semicolon.
        Start symbol: <undetermined>. *)

    | MenhirState0643 : (((((('s, 'r) _menhir_cell1_LET, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_RBRACE _menhir_cell0_LPAR, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0643.
        Stack shape : LET decl_item IDENTIFIER LBRACE RBRACE LPAR separated_nonempty_list(COMMA,expr) RPAR SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0645 : ((((('s, 'r) _menhir_cell1_LET, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_COMMA, 'r) _menhir_state
      (** State 0645.
        Stack shape : LET decl_item IDENTIFIER LBRACE COMMA.
        Start symbol: <undetermined>. *)

    | MenhirState0647 : (((((('s, 'r) _menhir_cell1_LET, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_COMMA, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_state
      (** State 0647.
        Stack shape : LET decl_item IDENTIFIER LBRACE COMMA separated_nonempty_list(COMMA,expr) RBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0649 : ((((((('s, 'r) _menhir_cell1_LET, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_COMMA, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_cell1_loption_plist_expr__ _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0649.
        Stack shape : LET decl_item IDENTIFIER LBRACE COMMA separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)) SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0651 : (((('s, 'r) _menhir_cell1_LET, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0651.
        Stack shape : LET decl_item expr.
        Start symbol: <undetermined>. *)

    | MenhirState0652 : ((((('s, 'r) _menhir_cell1_LET, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0652.
        Stack shape : LET decl_item expr SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0657 : ((('s, 'r) _menhir_cell1_PRAGMA _menhir_cell0_IDENTIFIER, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0657.
        Stack shape : PRAGMA IDENTIFIER separated_nonempty_list(COMMA,expr) SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0662 : ((('s, 'r) _menhir_cell1_PRINT _menhir_cell0_LPAR, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0662.
        Stack shape : PRINT LPAR separated_nonempty_list(COMMA,expr) RPAR SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0665 : ((('s, 'r) _menhir_cell1_REPEAT, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_state
      (** State 0665.
        Stack shape : REPEAT nonempty_list(stmt).
        Start symbol: <undetermined>. *)

    | MenhirState0666 : (((('s, 'r) _menhir_cell1_REPEAT, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0666.
        Stack shape : REPEAT nonempty_list(stmt) expr.
        Start symbol: <undetermined>. *)

    | MenhirState0667 : ((((('s, 'r) _menhir_cell1_REPEAT, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0667.
        Stack shape : REPEAT nonempty_list(stmt) expr SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0669 : ((((('s, 'r) _menhir_cell1_REPEAT, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LOOPLIMIT, 'r) _menhir_state
      (** State 0669.
        Stack shape : REPEAT nonempty_list(stmt) expr LOOPLIMIT.
        Start symbol: <undetermined>. *)

    | MenhirState0670 : (((((('s, 'r) _menhir_cell1_REPEAT, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LOOPLIMIT, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0670.
        Stack shape : REPEAT nonempty_list(stmt) expr LOOPLIMIT expr.
        Start symbol: <undetermined>. *)

    | MenhirState0671 : ((((((('s, 'r) _menhir_cell1_REPEAT, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LOOPLIMIT, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0671.
        Stack shape : REPEAT nonempty_list(stmt) expr LOOPLIMIT expr SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0674 : ((('s, 'r) _menhir_cell1_RETURN, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0674.
        Stack shape : RETURN expr.
        Start symbol: <undetermined>. *)

    | MenhirState0675 : (((('s, 'r) _menhir_cell1_RETURN, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0675.
        Stack shape : RETURN expr SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0678 : ((('s, 'r) _menhir_cell1_THROW, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0678.
        Stack shape : THROW expr.
        Start symbol: <undetermined>. *)

    | MenhirState0679 : (((('s, 'r) _menhir_cell1_THROW, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0679.
        Stack shape : THROW expr SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0682 : ((('s, 'r) _menhir_cell1_TRY, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_state
      (** State 0682.
        Stack shape : TRY nonempty_list(stmt).
        Start symbol: <undetermined>. *)

    | MenhirState0683 : (('s, 'r) _menhir_cell1_WHEN, 'r) _menhir_state
      (** State 0683.
        Stack shape : WHEN.
        Start symbol: <undetermined>. *)

    | MenhirState0685 : ((('s, 'r) _menhir_cell1_WHEN, 'r) _menhir_cell1_IDENTIFIER, 'r) _menhir_state
      (** State 0685.
        Stack shape : WHEN IDENTIFIER.
        Start symbol: <undetermined>. *)

    | MenhirState0687 : (((('s, 'r) _menhir_cell1_WHEN, 'r) _menhir_cell1_IDENTIFIER, 'r) _menhir_cell1_ty, 'r) _menhir_state
      (** State 0687.
        Stack shape : WHEN IDENTIFIER ty.
        Start symbol: <undetermined>. *)

    | MenhirState0688 : ((((('s, 'r) _menhir_cell1_WHEN, 'r) _menhir_cell1_IDENTIFIER, 'r) _menhir_cell1_ty, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_state
      (** State 0688.
        Stack shape : WHEN IDENTIFIER ty nonempty_list(stmt).
        Start symbol: <undetermined>. *)

    | MenhirState0691 : ((('s, 'r) _menhir_cell1_WHEN, 'r) _menhir_cell1_ty, 'r) _menhir_state
      (** State 0691.
        Stack shape : WHEN ty.
        Start symbol: <undetermined>. *)

    | MenhirState0692 : (((('s, 'r) _menhir_cell1_WHEN, 'r) _menhir_cell1_ty, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_state
      (** State 0692.
        Stack shape : WHEN ty nonempty_list(stmt).
        Start symbol: <undetermined>. *)

    | MenhirState0694 : (((('s, 'r) _menhir_cell1_TRY, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_nonempty_list_catcher_, 'r) _menhir_state
      (** State 0694.
        Stack shape : TRY nonempty_list(stmt) nonempty_list(catcher).
        Start symbol: <undetermined>. *)

    | MenhirState0696 : ((((('s, 'r) _menhir_cell1_TRY, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_nonempty_list_catcher_, 'r) _menhir_cell1_OTHERWISE, 'r) _menhir_state
      (** State 0696.
        Stack shape : TRY nonempty_list(stmt) nonempty_list(catcher) OTHERWISE.
        Start symbol: <undetermined>. *)

    | MenhirState0697 : (((((('s, 'r) _menhir_cell1_TRY, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_nonempty_list_catcher_, 'r) _menhir_cell1_OTHERWISE, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_state
      (** State 0697.
        Stack shape : TRY nonempty_list(stmt) nonempty_list(catcher) OTHERWISE nonempty_list(stmt).
        Start symbol: <undetermined>. *)

    | MenhirState0698 : ((((((('s, 'r) _menhir_cell1_TRY, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_nonempty_list_catcher_, 'r) _menhir_cell1_OTHERWISE, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_end_semicolon, 'r) _menhir_state
      (** State 0698.
        Stack shape : TRY nonempty_list(stmt) nonempty_list(catcher) OTHERWISE nonempty_list(stmt) end_semicolon.
        Start symbol: <undetermined>. *)

    | MenhirState0700 : ((((('s, 'r) _menhir_cell1_TRY, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_nonempty_list_catcher_, 'r) _menhir_cell1_end_semicolon, 'r) _menhir_state
      (** State 0700.
        Stack shape : TRY nonempty_list(stmt) nonempty_list(catcher) end_semicolon.
        Start symbol: <undetermined>. *)

    | MenhirState0704 : (((('s, 'r) _menhir_cell1_VAR, 'r) _menhir_cell1_IDENTIFIER, 'r) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_, 'r) _menhir_state
      (** State 0704.
        Stack shape : VAR IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER).
        Start symbol: <undetermined>. *)

    | MenhirState0706 : ((((('s, 'r) _menhir_cell1_VAR, 'r) _menhir_cell1_IDENTIFIER, 'r) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_, 'r) _menhir_cell1_ty _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0706.
        Stack shape : VAR IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER) ty SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0709 : ((('s, 'r) _menhir_cell1_VAR, 'r) _menhir_cell1_decl_item _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0709.
        Stack shape : VAR decl_item SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0711 : ((('s, 'r) _menhir_cell1_VAR, 'r) _menhir_cell1_decl_item, 'r) _menhir_state
      (** State 0711.
        Stack shape : VAR decl_item.
        Start symbol: <undetermined>. *)

    | MenhirState0713 : (((('s, 'r) _menhir_cell1_VAR, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_state
      (** State 0713.
        Stack shape : VAR decl_item IDENTIFIER LBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0715 : ((((('s, 'r) _menhir_cell1_VAR, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_RBRACE _menhir_cell0_LPAR, 'r) _menhir_state
      (** State 0715.
        Stack shape : VAR decl_item IDENTIFIER LBRACE RBRACE LPAR.
        Start symbol: <undetermined>. *)

    | MenhirState0717 : (((((('s, 'r) _menhir_cell1_VAR, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_RBRACE _menhir_cell0_LPAR, 'r) _menhir_cell1_RPAR _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0717.
        Stack shape : VAR decl_item IDENTIFIER LBRACE RBRACE LPAR RPAR SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0721 : (((((('s, 'r) _menhir_cell1_VAR, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_RBRACE _menhir_cell0_LPAR, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0721.
        Stack shape : VAR decl_item IDENTIFIER LBRACE RBRACE LPAR separated_nonempty_list(COMMA,expr) RPAR SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0723 : ((((('s, 'r) _menhir_cell1_VAR, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_COMMA, 'r) _menhir_state
      (** State 0723.
        Stack shape : VAR decl_item IDENTIFIER LBRACE COMMA.
        Start symbol: <undetermined>. *)

    | MenhirState0725 : (((((('s, 'r) _menhir_cell1_VAR, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_COMMA, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_state
      (** State 0725.
        Stack shape : VAR decl_item IDENTIFIER LBRACE COMMA separated_nonempty_list(COMMA,expr) RBRACE.
        Start symbol: <undetermined>. *)

    | MenhirState0727 : ((((((('s, 'r) _menhir_cell1_VAR, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, 'r) _menhir_cell1_COMMA, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, 'r) _menhir_cell1_loption_plist_expr__ _menhir_cell0_SEMI_COLON, 'r) _menhir_state
      (** State 0727.
        Stack shape : VAR decl_item IDENTIFIER LBRACE COMMA separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)) SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0729 : (((('s, 'r) _menhir_cell1_VAR, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_expr, 'r) _menhir_state
      (** State 0729.
        Stack shape : VAR decl_item expr.
        Start symbol: <undetermined>. *)

    | MenhirState0730 : ((((('s, 'r) _menhir_cell1_VAR, 'r) _menhir_cell1_decl_item, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_SEMI_COLON, 'r) _menhir_state
      (** State 0730.
        Stack shape : VAR decl_item expr SEMI_COLON.
        Start symbol: <undetermined>. *)

    | MenhirState0732 : ((((((('s, 'r) _menhir_cell1_WHILE, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LOOPLIMIT, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_state
      (** State 0732.
        Stack shape : WHILE expr LOOPLIMIT expr DO nonempty_list(stmt).
        Start symbol: <undetermined>. *)

    | MenhirState0733 : (((((((('s, 'r) _menhir_cell1_WHILE, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_LOOPLIMIT, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_end_semicolon, 'r) _menhir_state
      (** State 0733.
        Stack shape : WHILE expr LOOPLIMIT expr DO nonempty_list(stmt) end_semicolon.
        Start symbol: <undetermined>. *)

    | MenhirState0735 : (((('s, 'r) _menhir_cell1_WHILE, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_state
      (** State 0735.
        Stack shape : WHILE expr DO.
        Start symbol: <undetermined>. *)

    | MenhirState0736 : ((((('s, 'r) _menhir_cell1_WHILE, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_state
      (** State 0736.
        Stack shape : WHILE expr DO nonempty_list(stmt).
        Start symbol: <undetermined>. *)

    | MenhirState0737 : (((((('s, 'r) _menhir_cell1_WHILE, 'r) _menhir_cell1_expr, 'r) _menhir_cell1_DO, 'r) _menhir_cell1_nonempty_list_stmt_, 'r) _menhir_cell1_end_semicolon, 'r) _menhir_state
      (** State 0737.
        Stack shape : WHILE expr DO nonempty_list(stmt) end_semicolon.
        Start symbol: <undetermined>. *)

    | MenhirState0739 : ((((((('s, _menhir_box_opn) _menhir_cell1_WHILE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_state
      (** State 0739.
        Stack shape : WHILE expr LOOPLIMIT expr DO nonempty_list(stmt).
        Start symbol: opn. *)

    | MenhirState0742 : (((('s, _menhir_box_opn) _menhir_cell1_WHILE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_state
      (** State 0742.
        Stack shape : WHILE expr DO.
        Start symbol: opn. *)

    | MenhirState0743 : ((((('s, _menhir_box_opn) _menhir_cell1_WHILE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_state
      (** State 0743.
        Stack shape : WHILE expr DO nonempty_list(stmt).
        Start symbol: opn. *)

    | MenhirState0746 : (('s, _menhir_box_opn) _menhir_cell1_VAR, _menhir_box_opn) _menhir_state
      (** State 0746.
        Stack shape : VAR.
        Start symbol: opn. *)

    | MenhirState0748 : ((('s, _menhir_box_opn) _menhir_cell1_VAR, _menhir_box_opn) _menhir_cell1_IDENTIFIER, _menhir_box_opn) _menhir_state
      (** State 0748.
        Stack shape : VAR IDENTIFIER.
        Start symbol: opn. *)

    | MenhirState0750 : (((('s, _menhir_box_opn) _menhir_cell1_VAR, _menhir_box_opn) _menhir_cell1_IDENTIFIER, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_, _menhir_box_opn) _menhir_state
      (** State 0750.
        Stack shape : VAR IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER).
        Start symbol: opn. *)

    | MenhirState0754 : (((('s, _menhir_box_opn) _menhir_cell1_VAR, _menhir_box_opn) _menhir_cell1_IDENTIFIER, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_, _menhir_box_opn) _menhir_state
      (** State 0754.
        Stack shape : VAR IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER).
        Start symbol: opn. *)

    | MenhirState0761 : ((('s, _menhir_box_opn) _menhir_cell1_VAR, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_state
      (** State 0761.
        Stack shape : VAR decl_item.
        Start symbol: opn. *)

    | MenhirState0763 : (((('s, _menhir_box_opn) _menhir_cell1_VAR, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_state
      (** State 0763.
        Stack shape : VAR decl_item IDENTIFIER LBRACE.
        Start symbol: opn. *)

    | MenhirState0765 : ((((('s, _menhir_box_opn) _menhir_cell1_VAR, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_RBRACE _menhir_cell0_LPAR, _menhir_box_opn) _menhir_state
      (** State 0765.
        Stack shape : VAR decl_item IDENTIFIER LBRACE RBRACE LPAR.
        Start symbol: opn. *)

    | MenhirState0773 : ((((('s, _menhir_box_opn) _menhir_cell1_VAR, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_COMMA, _menhir_box_opn) _menhir_state
      (** State 0773.
        Stack shape : VAR decl_item IDENTIFIER LBRACE COMMA.
        Start symbol: opn. *)

    | MenhirState0775 : (((((('s, _menhir_box_opn) _menhir_cell1_VAR, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_COMMA, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, _menhir_box_opn) _menhir_state
      (** State 0775.
        Stack shape : VAR decl_item IDENTIFIER LBRACE COMMA separated_nonempty_list(COMMA,expr) RBRACE.
        Start symbol: opn. *)

    | MenhirState0779 : (((('s, _menhir_box_opn) _menhir_cell1_VAR, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 0779.
        Stack shape : VAR decl_item expr.
        Start symbol: opn. *)

    | MenhirState0787 : (('s, _menhir_box_opn) _menhir_cell1_TRY, _menhir_box_opn) _menhir_state
      (** State 0787.
        Stack shape : TRY.
        Start symbol: opn. *)

    | MenhirState0789 : ((('s, _menhir_box_opn) _menhir_cell1_TRY, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_state
      (** State 0789.
        Stack shape : TRY nonempty_list(stmt).
        Start symbol: opn. *)

    | MenhirState0790 : (((('s, _menhir_box_opn) _menhir_cell1_TRY, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_cell1_nonempty_list_catcher_, _menhir_box_opn) _menhir_state
      (** State 0790.
        Stack shape : TRY nonempty_list(stmt) nonempty_list(catcher).
        Start symbol: opn. *)

    | MenhirState0792 : ((((('s, _menhir_box_opn) _menhir_cell1_TRY, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_cell1_nonempty_list_catcher_, _menhir_box_opn) _menhir_cell1_OTHERWISE, _menhir_box_opn) _menhir_state
      (** State 0792.
        Stack shape : TRY nonempty_list(stmt) nonempty_list(catcher) OTHERWISE.
        Start symbol: opn. *)

    | MenhirState0793 : (((((('s, _menhir_box_opn) _menhir_cell1_TRY, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_cell1_nonempty_list_catcher_, _menhir_box_opn) _menhir_cell1_OTHERWISE, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_state
      (** State 0793.
        Stack shape : TRY nonempty_list(stmt) nonempty_list(catcher) OTHERWISE nonempty_list(stmt).
        Start symbol: opn. *)

    | MenhirState0798 : (('s, _menhir_box_opn) _menhir_cell1_THROW, _menhir_box_opn) _menhir_state
      (** State 0798.
        Stack shape : THROW.
        Start symbol: opn. *)

    | MenhirState0801 : ((('s, _menhir_box_opn) _menhir_cell1_THROW, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 0801.
        Stack shape : THROW expr.
        Start symbol: opn. *)

    | MenhirState0804 : (('s, _menhir_box_opn) _menhir_cell1_RETURN, _menhir_box_opn) _menhir_state
      (** State 0804.
        Stack shape : RETURN.
        Start symbol: opn. *)

    | MenhirState0807 : ((('s, _menhir_box_opn) _menhir_cell1_RETURN, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 0807.
        Stack shape : RETURN expr.
        Start symbol: opn. *)

    | MenhirState0810 : (('s, _menhir_box_opn) _menhir_cell1_REPEAT, _menhir_box_opn) _menhir_state
      (** State 0810.
        Stack shape : REPEAT.
        Start symbol: opn. *)

    | MenhirState0812 : ((('s, _menhir_box_opn) _menhir_cell1_REPEAT, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_state
      (** State 0812.
        Stack shape : REPEAT nonempty_list(stmt).
        Start symbol: opn. *)

    | MenhirState0813 : (((('s, _menhir_box_opn) _menhir_cell1_REPEAT, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 0813.
        Stack shape : REPEAT nonempty_list(stmt) expr.
        Start symbol: opn. *)

    | MenhirState0816 : ((((('s, _menhir_box_opn) _menhir_cell1_REPEAT, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_state
      (** State 0816.
        Stack shape : REPEAT nonempty_list(stmt) expr LOOPLIMIT.
        Start symbol: opn. *)

    | MenhirState0817 : (((((('s, _menhir_box_opn) _menhir_cell1_REPEAT, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 0817.
        Stack shape : REPEAT nonempty_list(stmt) expr LOOPLIMIT expr.
        Start symbol: opn. *)

    | MenhirState0821 : (('s, _menhir_box_opn) _menhir_cell1_PRINT _menhir_cell0_LPAR, _menhir_box_opn) _menhir_state
      (** State 0821.
        Stack shape : PRINT LPAR.
        Start symbol: opn. *)

    | MenhirState0830 : (('s, _menhir_box_opn) _menhir_cell1_PRAGMA _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_state
      (** State 0830.
        Stack shape : PRAGMA IDENTIFIER.
        Start symbol: opn. *)

    | MenhirState0839 : (('s, _menhir_box_opn) _menhir_cell1_LET, _menhir_box_opn) _menhir_state
      (** State 0839.
        Stack shape : LET.
        Start symbol: opn. *)

    | MenhirState0841 : ((('s, _menhir_box_opn) _menhir_cell1_LET, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_state
      (** State 0841.
        Stack shape : LET decl_item.
        Start symbol: opn. *)

    | MenhirState0843 : (((('s, _menhir_box_opn) _menhir_cell1_LET, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_state
      (** State 0843.
        Stack shape : LET decl_item IDENTIFIER LBRACE.
        Start symbol: opn. *)

    | MenhirState0845 : ((((('s, _menhir_box_opn) _menhir_cell1_LET, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_RBRACE _menhir_cell0_LPAR, _menhir_box_opn) _menhir_state
      (** State 0845.
        Stack shape : LET decl_item IDENTIFIER LBRACE RBRACE LPAR.
        Start symbol: opn. *)

    | MenhirState0853 : ((((('s, _menhir_box_opn) _menhir_cell1_LET, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_COMMA, _menhir_box_opn) _menhir_state
      (** State 0853.
        Stack shape : LET decl_item IDENTIFIER LBRACE COMMA.
        Start symbol: opn. *)

    | MenhirState0855 : (((((('s, _menhir_box_opn) _menhir_cell1_LET, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_COMMA, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, _menhir_box_opn) _menhir_state
      (** State 0855.
        Stack shape : LET decl_item IDENTIFIER LBRACE COMMA separated_nonempty_list(COMMA,expr) RBRACE.
        Start symbol: opn. *)

    | MenhirState0859 : (((('s, _menhir_box_opn) _menhir_cell1_LET, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 0859.
        Stack shape : LET decl_item expr.
        Start symbol: opn. *)

    | MenhirState0862 : (('s, _menhir_box_opn) _menhir_cell1_IF, _menhir_box_opn) _menhir_state
      (** State 0862.
        Stack shape : IF.
        Start symbol: opn. *)

    | MenhirState0863 : ((('s, _menhir_box_opn) _menhir_cell1_IF, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 0863.
        Stack shape : IF expr.
        Start symbol: opn. *)

    | MenhirState0864 : (((('s, _menhir_box_opn) _menhir_cell1_IF, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_THEN, _menhir_box_opn) _menhir_state
      (** State 0864.
        Stack shape : IF expr THEN.
        Start symbol: opn. *)

    | MenhirState0865 : ((((('s, _menhir_box_opn) _menhir_cell1_IF, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_THEN, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_state
      (** State 0865.
        Stack shape : IF expr THEN nonempty_list(stmt).
        Start symbol: opn. *)

    | MenhirState0866 : (((((('s, _menhir_box_opn) _menhir_cell1_IF, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_THEN, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_cell1_s_else, _menhir_box_opn) _menhir_state
      (** State 0866.
        Stack shape : IF expr THEN nonempty_list(stmt) s_else.
        Start symbol: opn. *)

    | MenhirState0870 : (('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_state
      (** State 0870.
        Stack shape : IDENTIFIER LPAR.
        Start symbol: opn. *)

    | MenhirState0874 : ((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_RPAR, _menhir_box_opn) _menhir_state
      (** State 0874.
        Stack shape : IDENTIFIER LPAR RPAR.
        Start symbol: opn. *)

    | MenhirState0875 : (((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_RPAR, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 0875.
        Stack shape : IDENTIFIER LPAR RPAR expr.
        Start symbol: opn. *)

    | MenhirState0881 : ((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_state
      (** State 0881.
        Stack shape : IDENTIFIER LPAR RPAR IDENTIFIER.
        Start symbol: opn. *)

    | MenhirState0884 : (((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET, _menhir_box_opn) _menhir_state
      (** State 0884.
        Stack shape : IDENTIFIER LPAR RPAR IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER) RBRACKET.
        Start symbol: opn. *)

    | MenhirState0885 : ((((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 0885.
        Stack shape : IDENTIFIER LPAR RPAR IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER) RBRACKET expr.
        Start symbol: opn. *)

    | MenhirState0889 : ((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_state
      (** State 0889.
        Stack shape : IDENTIFIER LPAR RPAR IDENTIFIER.
        Start symbol: opn. *)

    | MenhirState0890 : (((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 0890.
        Stack shape : IDENTIFIER LPAR RPAR IDENTIFIER expr.
        Start symbol: opn. *)

    | MenhirState0897 : ((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR, _menhir_box_opn) _menhir_state
      (** State 0897.
        Stack shape : IDENTIFIER LPAR separated_nonempty_list(COMMA,expr) RPAR.
        Start symbol: opn. *)

    | MenhirState0898 : (((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 0898.
        Stack shape : IDENTIFIER LPAR separated_nonempty_list(COMMA,expr) RPAR expr.
        Start symbol: opn. *)

    | MenhirState0904 : ((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_state
      (** State 0904.
        Stack shape : IDENTIFIER LPAR separated_nonempty_list(COMMA,expr) RPAR IDENTIFIER.
        Start symbol: opn. *)

    | MenhirState0907 : (((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET, _menhir_box_opn) _menhir_state
      (** State 0907.
        Stack shape : IDENTIFIER LPAR separated_nonempty_list(COMMA,expr) RPAR IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER) RBRACKET.
        Start symbol: opn. *)

    | MenhirState0908 : ((((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 0908.
        Stack shape : IDENTIFIER LPAR separated_nonempty_list(COMMA,expr) RPAR IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER) RBRACKET expr.
        Start symbol: opn. *)

    | MenhirState0912 : ((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_state
      (** State 0912.
        Stack shape : IDENTIFIER LPAR separated_nonempty_list(COMMA,expr) RPAR IDENTIFIER.
        Start symbol: opn. *)

    | MenhirState0913 : (((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 0913.
        Stack shape : IDENTIFIER LPAR separated_nonempty_list(COMMA,expr) RPAR IDENTIFIER expr.
        Start symbol: opn. *)

    | MenhirState0916 : (('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_state
      (** State 0916.
        Stack shape : IDENTIFIER LBRACE.
        Start symbol: opn. *)

    | MenhirState0918 : ((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, _menhir_box_opn) _menhir_state
      (** State 0918.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE.
        Start symbol: opn. *)

    | MenhirState0922 : (((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, _menhir_box_opn) _menhir_cell1_loption_plist_expr__, _menhir_box_opn) _menhir_state
      (** State 0922.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)).
        Start symbol: opn. *)

    | MenhirState0923 : ((((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, _menhir_box_opn) _menhir_cell1_loption_plist_expr__, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 0923.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)) expr.
        Start symbol: opn. *)

    | MenhirState0929 : (((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, _menhir_box_opn) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_state
      (** State 0929.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)) IDENTIFIER.
        Start symbol: opn. *)

    | MenhirState0932 : ((((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, _menhir_box_opn) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET, _menhir_box_opn) _menhir_state
      (** State 0932.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)) IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER) RBRACKET.
        Start symbol: opn. *)

    | MenhirState0933 : (((((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, _menhir_box_opn) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 0933.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)) IDENTIFIER separated_nonempty_list(COMMA,IDENTIFIER) RBRACKET expr.
        Start symbol: opn. *)

    | MenhirState0937 : (((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, _menhir_box_opn) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_state
      (** State 0937.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)) IDENTIFIER.
        Start symbol: opn. *)

    | MenhirState0938 : ((((('s, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, _menhir_box_opn) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 0938.
        Stack shape : IDENTIFIER LBRACE separated_nonempty_list(COMMA,expr) RBRACE loption(plist(expr)) IDENTIFIER expr.
        Start symbol: opn. *)

    | MenhirState0943 : (('s, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_state
      (** State 0943.
        Stack shape : FOR IDENTIFIER.
        Start symbol: opn. *)

    | MenhirState0944 : ((('s, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 0944.
        Stack shape : FOR IDENTIFIER expr.
        Start symbol: opn. *)

    | MenhirState0945 : (((('s, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_TO, _menhir_box_opn) _menhir_state
      (** State 0945.
        Stack shape : FOR IDENTIFIER expr TO.
        Start symbol: opn. *)

    | MenhirState0946 : ((((('s, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_TO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 0946.
        Stack shape : FOR IDENTIFIER expr TO expr.
        Start symbol: opn. *)

    | MenhirState0947 : (((((('s, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_TO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_state
      (** State 0947.
        Stack shape : FOR IDENTIFIER expr TO expr LOOPLIMIT.
        Start symbol: opn. *)

    | MenhirState0948 : ((((((('s, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_TO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 0948.
        Stack shape : FOR IDENTIFIER expr TO expr LOOPLIMIT expr.
        Start symbol: opn. *)

    | MenhirState0949 : (((((((('s, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_TO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_state
      (** State 0949.
        Stack shape : FOR IDENTIFIER expr TO expr LOOPLIMIT expr DO.
        Start symbol: opn. *)

    | MenhirState0950 : ((((((((('s, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_TO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_state
      (** State 0950.
        Stack shape : FOR IDENTIFIER expr TO expr LOOPLIMIT expr DO nonempty_list(stmt).
        Start symbol: opn. *)

    | MenhirState0953 : (((((('s, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_TO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_state
      (** State 0953.
        Stack shape : FOR IDENTIFIER expr TO expr DO.
        Start symbol: opn. *)

    | MenhirState0954 : ((((((('s, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_TO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_state
      (** State 0954.
        Stack shape : FOR IDENTIFIER expr TO expr DO nonempty_list(stmt).
        Start symbol: opn. *)

    | MenhirState0957 : (((('s, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DOWNTO, _menhir_box_opn) _menhir_state
      (** State 0957.
        Stack shape : FOR IDENTIFIER expr DOWNTO.
        Start symbol: opn. *)

    | MenhirState0958 : ((((('s, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DOWNTO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 0958.
        Stack shape : FOR IDENTIFIER expr DOWNTO expr.
        Start symbol: opn. *)

    | MenhirState0959 : (((((('s, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DOWNTO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_state
      (** State 0959.
        Stack shape : FOR IDENTIFIER expr DOWNTO expr LOOPLIMIT.
        Start symbol: opn. *)

    | MenhirState0960 : ((((((('s, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DOWNTO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 0960.
        Stack shape : FOR IDENTIFIER expr DOWNTO expr LOOPLIMIT expr.
        Start symbol: opn. *)

    | MenhirState0961 : (((((((('s, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DOWNTO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_state
      (** State 0961.
        Stack shape : FOR IDENTIFIER expr DOWNTO expr LOOPLIMIT expr DO.
        Start symbol: opn. *)

    | MenhirState0962 : ((((((((('s, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DOWNTO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_state
      (** State 0962.
        Stack shape : FOR IDENTIFIER expr DOWNTO expr LOOPLIMIT expr DO nonempty_list(stmt).
        Start symbol: opn. *)

    | MenhirState0965 : (((((('s, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DOWNTO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_state
      (** State 0965.
        Stack shape : FOR IDENTIFIER expr DOWNTO expr DO.
        Start symbol: opn. *)

    | MenhirState0966 : ((((((('s, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DOWNTO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_state
      (** State 0966.
        Stack shape : FOR IDENTIFIER expr DOWNTO expr DO nonempty_list(stmt).
        Start symbol: opn. *)

    | MenhirState0970 : (('s, _menhir_box_opn) _menhir_cell1_DEBUG _menhir_cell0_LPAR, _menhir_box_opn) _menhir_state
      (** State 0970.
        Stack shape : DEBUG LPAR.
        Start symbol: opn. *)

    | MenhirState0978 : (('s, _menhir_box_opn) _menhir_cell1_CONSTANT, _menhir_box_opn) _menhir_state
      (** State 0978.
        Stack shape : CONSTANT.
        Start symbol: opn. *)

    | MenhirState0980 : ((('s, _menhir_box_opn) _menhir_cell1_CONSTANT, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_state
      (** State 0980.
        Stack shape : CONSTANT decl_item.
        Start symbol: opn. *)

    | MenhirState0982 : (((('s, _menhir_box_opn) _menhir_cell1_CONSTANT, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_state
      (** State 0982.
        Stack shape : CONSTANT decl_item IDENTIFIER LBRACE.
        Start symbol: opn. *)

    | MenhirState0984 : ((((('s, _menhir_box_opn) _menhir_cell1_CONSTANT, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_RBRACE _menhir_cell0_LPAR, _menhir_box_opn) _menhir_state
      (** State 0984.
        Stack shape : CONSTANT decl_item IDENTIFIER LBRACE RBRACE LPAR.
        Start symbol: opn. *)

    | MenhirState0992 : ((((('s, _menhir_box_opn) _menhir_cell1_CONSTANT, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_COMMA, _menhir_box_opn) _menhir_state
      (** State 0992.
        Stack shape : CONSTANT decl_item IDENTIFIER LBRACE COMMA.
        Start symbol: opn. *)

    | MenhirState0994 : (((((('s, _menhir_box_opn) _menhir_cell1_CONSTANT, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_COMMA, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, _menhir_box_opn) _menhir_state
      (** State 0994.
        Stack shape : CONSTANT decl_item IDENTIFIER LBRACE COMMA separated_nonempty_list(COMMA,expr) RBRACE.
        Start symbol: opn. *)

    | MenhirState0998 : (((('s, _menhir_box_opn) _menhir_cell1_CONSTANT, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 0998.
        Stack shape : CONSTANT decl_item expr.
        Start symbol: opn. *)

    | MenhirState1001 : (('s, _menhir_box_opn) _menhir_cell1_CASE, _menhir_box_opn) _menhir_state
      (** State 1001.
        Stack shape : CASE.
        Start symbol: opn. *)

    | MenhirState1002 : ((('s, _menhir_box_opn) _menhir_cell1_CASE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 1002.
        Stack shape : CASE expr.
        Start symbol: opn. *)

    | MenhirState1003 : (((('s, _menhir_box_opn) _menhir_cell1_CASE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_OF, _menhir_box_opn) _menhir_state
      (** State 1003.
        Stack shape : CASE expr OF.
        Start symbol: opn. *)

    | MenhirState1004 : ((((('s, _menhir_box_opn) _menhir_cell1_CASE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_OF, _menhir_box_opn) _menhir_cell1_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__, _menhir_box_opn) _menhir_state
      (** State 1004.
        Stack shape : CASE expr OF nlist_opt_terminated(annotated(case_alt),annotated(case_otherwise)).
        Start symbol: opn. *)

    | MenhirState1007 : (('s, _menhir_box_opn) _menhir_cell1_ASSERT, _menhir_box_opn) _menhir_state
      (** State 1007.
        Stack shape : ASSERT.
        Start symbol: opn. *)

    | MenhirState1008 : ((('s, _menhir_box_opn) _menhir_cell1_ASSERT, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 1008.
        Stack shape : ASSERT expr.
        Start symbol: opn. *)

    | MenhirState1012 : (('s, _menhir_box_opn) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, _menhir_box_opn) _menhir_state
      (** State 1012.
        Stack shape : ARROBASE_LOOPLIMIT LPAR.
        Start symbol: opn. *)

    | MenhirState1013 : ((('s, _menhir_box_opn) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 1013.
        Stack shape : ARROBASE_LOOPLIMIT LPAR expr.
        Start symbol: opn. *)

    | MenhirState1015 : (((('s, _menhir_box_opn) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_WHILE, _menhir_box_opn) _menhir_state
      (** State 1015.
        Stack shape : ARROBASE_LOOPLIMIT LPAR expr RPAR WHILE.
        Start symbol: opn. *)

    | MenhirState1016 : ((((('s, _menhir_box_opn) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_WHILE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 1016.
        Stack shape : ARROBASE_LOOPLIMIT LPAR expr RPAR WHILE expr.
        Start symbol: opn. *)

    | MenhirState1017 : (((((('s, _menhir_box_opn) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_WHILE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_state
      (** State 1017.
        Stack shape : ARROBASE_LOOPLIMIT LPAR expr RPAR WHILE expr DO.
        Start symbol: opn. *)

    | MenhirState1018 : ((((((('s, _menhir_box_opn) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_WHILE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_state
      (** State 1018.
        Stack shape : ARROBASE_LOOPLIMIT LPAR expr RPAR WHILE expr DO nonempty_list(stmt).
        Start symbol: opn. *)

    | MenhirState1021 : (((('s, _menhir_box_opn) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_REPEAT, _menhir_box_opn) _menhir_state
      (** State 1021.
        Stack shape : ARROBASE_LOOPLIMIT LPAR expr RPAR REPEAT.
        Start symbol: opn. *)

    | MenhirState1023 : ((((('s, _menhir_box_opn) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_REPEAT, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_state
      (** State 1023.
        Stack shape : ARROBASE_LOOPLIMIT LPAR expr RPAR REPEAT nonempty_list(stmt).
        Start symbol: opn. *)

    | MenhirState1024 : (((((('s, _menhir_box_opn) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_REPEAT, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 1024.
        Stack shape : ARROBASE_LOOPLIMIT LPAR expr RPAR REPEAT nonempty_list(stmt) expr.
        Start symbol: opn. *)

    | MenhirState1029 : (('s, _menhir_box_opn) _menhir_cell1_lexpr, _menhir_box_opn) _menhir_state
      (** State 1029.
        Stack shape : lexpr.
        Start symbol: opn. *)

    | MenhirState1030 : ((('s, _menhir_box_opn) _menhir_cell1_lexpr, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_state
      (** State 1030.
        Stack shape : lexpr expr.
        Start symbol: opn. *)

    | MenhirState1033 : ('s, _menhir_box_spec) _menhir_state
      (** State 1033.
        Stack shape : .
        Start symbol: spec. *)

    | MenhirState1036 : (('s, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_spec) _menhir_state
      (** State 1036.
        Stack shape : VAR MINUS.
        Start symbol: spec. *)

    | MenhirState1037 : ((('s, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_state
      (** State 1037.
        Stack shape : VAR MINUS expr.
        Start symbol: spec. *)

    | MenhirState1038 : (((('s, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON, _menhir_box_spec) _menhir_state
      (** State 1038.
        Stack shape : VAR MINUS expr SEMI_COLON.
        Start symbol: spec. *)

    | MenhirState1042 : (('s, _menhir_box_spec) _menhir_cell1_TYPE _menhir_cell0_IDENTIFIER _menhir_cell0_SUBTYPES _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_state
      (** State 1042.
        Stack shape : TYPE IDENTIFIER SUBTYPES IDENTIFIER.
        Start symbol: spec. *)

    | MenhirState1044 : (('s _menhir_cell0_SUBTYPES _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_WITH _menhir_cell0_LBRACE, _menhir_box_spec) _menhir_state
      (** State 1044.
        Stack shape : SUBTYPES IDENTIFIER WITH LBRACE.
        Start symbol: spec. *)

    | MenhirState1046 : (('s, _menhir_box_spec) _menhir_cell1_IDENTIFIER, _menhir_box_spec) _menhir_state
      (** State 1046.
        Stack shape : IDENTIFIER.
        Start symbol: spec. *)

    | MenhirState1048 : ((('s, _menhir_box_spec) _menhir_cell1_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_state
      (** State 1048.
        Stack shape : IDENTIFIER ty.
        Start symbol: spec. *)

    | MenhirState1050 : (('s, _menhir_box_spec) _menhir_cell1_IDENTIFIER, _menhir_box_spec) _menhir_state
      (** State 1050.
        Stack shape : IDENTIFIER.
        Start symbol: spec. *)

    | MenhirState1052 : ((('s, _menhir_box_spec) _menhir_cell1_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_state
      (** State 1052.
        Stack shape : IDENTIFIER ty.
        Start symbol: spec. *)

    | MenhirState1058 : ((('s, _menhir_box_spec) _menhir_cell1_TYPE _menhir_cell0_IDENTIFIER _menhir_cell0_SUBTYPES _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption___anonymous_21_ _menhir_cell0_SEMI_COLON, _menhir_box_spec) _menhir_state
      (** State 1058.
        Stack shape : TYPE IDENTIFIER SUBTYPES IDENTIFIER loption(__anonymous_21) SEMI_COLON.
        Start symbol: spec. *)

    | MenhirState1060 : (('s, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_state
      (** State 1060.
        Stack shape : SETTER IDENTIFIER.
        Start symbol: spec. *)

    | MenhirState1061 : (('s _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_LBRACE, _menhir_box_spec) _menhir_state
      (** State 1061.
        Stack shape : IDENTIFIER LBRACE.
        Start symbol: spec. *)

    | MenhirState1063 : (('s, _menhir_box_spec) _menhir_cell1_IDENTIFIER, _menhir_box_spec) _menhir_state
      (** State 1063.
        Stack shape : IDENTIFIER.
        Start symbol: spec. *)

    | MenhirState1065 : (('s, _menhir_box_spec) _menhir_cell1_IDENTIFIER, _menhir_box_spec) _menhir_state
      (** State 1065.
        Stack shape : IDENTIFIER.
        Start symbol: spec. *)

    | MenhirState1067 : ((('s, _menhir_box_spec) _menhir_cell1_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_state
      (** State 1067.
        Stack shape : IDENTIFIER ty.
        Start symbol: spec. *)

    | MenhirState1069 : (('s, _menhir_box_spec) _menhir_cell1_IDENTIFIER, _menhir_box_spec) _menhir_state
      (** State 1069.
        Stack shape : IDENTIFIER.
        Start symbol: spec. *)

    | MenhirState1071 : ((('s, _menhir_box_spec) _menhir_cell1_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_state
      (** State 1071.
        Stack shape : IDENTIFIER ty.
        Start symbol: spec. *)

    | MenhirState1076 : ((('s, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_state
      (** State 1076.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR.
        Start symbol: spec. *)

    | MenhirState1080 : (((('s, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_state
      (** State 1080.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR IDENTIFIER.
        Start symbol: spec. *)

    | MenhirState1082 : ((((('s, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_state
      (** State 1082.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR IDENTIFIER ty BEGIN.
        Start symbol: spec. *)

    | MenhirState1083 : (((((('s, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_, _menhir_box_spec) _menhir_state
      (** State 1083.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR IDENTIFIER ty BEGIN nonempty_list(stmt).
        Start symbol: spec. *)

    | MenhirState1084 : ((((((('s, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_, _menhir_box_spec) _menhir_cell1_end_semicolon, _menhir_box_spec) _menhir_state
      (** State 1084.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR IDENTIFIER ty BEGIN nonempty_list(stmt) end_semicolon.
        Start symbol: spec. *)

    | MenhirState1086 : (('s, _menhir_box_spec) _menhir_cell1_PRAGMA _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_state
      (** State 1086.
        Stack shape : PRAGMA IDENTIFIER.
        Start symbol: spec. *)

    | MenhirState1087 : ((('s, _menhir_box_spec) _menhir_cell1_PRAGMA _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_SEMI_COLON, _menhir_box_spec) _menhir_state
      (** State 1087.
        Stack shape : PRAGMA IDENTIFIER SEMI_COLON.
        Start symbol: spec. *)

    | MenhirState1090 : (('s, _menhir_box_spec) _menhir_cell1_LET _menhir_cell0_MINUS, _menhir_box_spec) _menhir_state
      (** State 1090.
        Stack shape : LET MINUS.
        Start symbol: spec. *)

    | MenhirState1091 : ((('s, _menhir_box_spec) _menhir_cell1_LET _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_state
      (** State 1091.
        Stack shape : LET MINUS expr.
        Start symbol: spec. *)

    | MenhirState1092 : (((('s, _menhir_box_spec) _menhir_cell1_LET _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON, _menhir_box_spec) _menhir_state
      (** State 1092.
        Stack shape : LET MINUS expr SEMI_COLON.
        Start symbol: spec. *)

    | MenhirState1094 : (('s, _menhir_box_spec) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_state
      (** State 1094.
        Stack shape : GETTER IDENTIFIER.
        Start symbol: spec. *)

    | MenhirState1096 : ((('s, _menhir_box_spec) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_state
      (** State 1096.
        Stack shape : GETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR.
        Start symbol: spec. *)

    | MenhirState1098 : (((('s, _menhir_box_spec) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR, _menhir_box_spec) _menhir_state
      (** State 1098.
        Stack shape : GETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR.
        Start symbol: spec. *)

    | MenhirState1100 : ((((('s, _menhir_box_spec) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_state
      (** State 1100.
        Stack shape : GETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR ty BEGIN.
        Start symbol: spec. *)

    | MenhirState1101 : (((((('s, _menhir_box_spec) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_, _menhir_box_spec) _menhir_state
      (** State 1101.
        Stack shape : GETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR ty BEGIN nonempty_list(stmt).
        Start symbol: spec. *)

    | MenhirState1102 : ((((((('s, _menhir_box_spec) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_, _menhir_box_spec) _menhir_cell1_end_semicolon, _menhir_box_spec) _menhir_state
      (** State 1102.
        Stack shape : GETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR ty BEGIN nonempty_list(stmt) end_semicolon.
        Start symbol: spec. *)

    | MenhirState1104 : (('s, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_state
      (** State 1104.
        Stack shape : FUNC IDENTIFIER.
        Start symbol: spec. *)

    | MenhirState1106 : ((('s, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_state
      (** State 1106.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR.
        Start symbol: spec. *)

    | MenhirState1108 : (((('s, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_state
      (** State 1108.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR BEGIN.
        Start symbol: spec. *)

    | MenhirState1109 : ((((('s, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_, _menhir_box_spec) _menhir_state
      (** State 1109.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR BEGIN nonempty_list(stmt).
        Start symbol: spec. *)

    | MenhirState1110 : (((((('s, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_, _menhir_box_spec) _menhir_cell1_end_semicolon, _menhir_box_spec) _menhir_state
      (** State 1110.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR BEGIN nonempty_list(stmt) end_semicolon.
        Start symbol: spec. *)

    | MenhirState1113 : (('s, _menhir_box_spec) _menhir_cell1_CONSTANT _menhir_cell0_MINUS, _menhir_box_spec) _menhir_state
      (** State 1113.
        Stack shape : CONSTANT MINUS.
        Start symbol: spec. *)

    | MenhirState1114 : ((('s, _menhir_box_spec) _menhir_cell1_CONSTANT _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_state
      (** State 1114.
        Stack shape : CONSTANT MINUS expr.
        Start symbol: spec. *)

    | MenhirState1115 : (((('s, _menhir_box_spec) _menhir_cell1_CONSTANT _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON, _menhir_box_spec) _menhir_state
      (** State 1115.
        Stack shape : CONSTANT MINUS expr SEMI_COLON.
        Start symbol: spec. *)

    | MenhirState1118 : (('s, _menhir_box_spec) _menhir_cell1_CONFIG _menhir_cell0_MINUS, _menhir_box_spec) _menhir_state
      (** State 1118.
        Stack shape : CONFIG MINUS.
        Start symbol: spec. *)

    | MenhirState1119 : ((('s, _menhir_box_spec) _menhir_cell1_CONFIG _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_state
      (** State 1119.
        Stack shape : CONFIG MINUS expr.
        Start symbol: spec. *)

    | MenhirState1120 : (((('s, _menhir_box_spec) _menhir_cell1_CONFIG _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON, _menhir_box_spec) _menhir_state
      (** State 1120.
        Stack shape : CONFIG MINUS expr SEMI_COLON.
        Start symbol: spec. *)

    | MenhirState1122 : (('s, _menhir_box_spec) _menhir_cell1_CONFIG _menhir_cell0_MINUS, _menhir_box_spec) _menhir_state
      (** State 1122.
        Stack shape : CONFIG MINUS.
        Start symbol: spec. *)

    | MenhirState1124 : ((('s, _menhir_box_spec) _menhir_cell1_CONFIG _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_state
      (** State 1124.
        Stack shape : CONFIG MINUS ty.
        Start symbol: spec. *)

    | MenhirState1125 : (((('s, _menhir_box_spec) _menhir_cell1_CONFIG _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_state
      (** State 1125.
        Stack shape : CONFIG MINUS ty expr.
        Start symbol: spec. *)

    | MenhirState1126 : ((((('s, _menhir_box_spec) _menhir_cell1_CONFIG _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON, _menhir_box_spec) _menhir_state
      (** State 1126.
        Stack shape : CONFIG MINUS ty expr SEMI_COLON.
        Start symbol: spec. *)

    | MenhirState1128 : (('s, _menhir_box_spec) _menhir_cell1_CONFIG _menhir_cell0_MINUS, _menhir_box_spec) _menhir_state
      (** State 1128.
        Stack shape : CONFIG MINUS.
        Start symbol: spec. *)

    | MenhirState1130 : ((('s, _menhir_box_spec) _menhir_cell1_CONFIG _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_state
      (** State 1130.
        Stack shape : CONFIG MINUS ty.
        Start symbol: spec. *)

    | MenhirState1131 : (((('s, _menhir_box_spec) _menhir_cell1_CONFIG _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_state
      (** State 1131.
        Stack shape : CONFIG MINUS ty expr.
        Start symbol: spec. *)

    | MenhirState1132 : ((((('s, _menhir_box_spec) _menhir_cell1_CONFIG _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON, _menhir_box_spec) _menhir_state
      (** State 1132.
        Stack shape : CONFIG MINUS ty expr SEMI_COLON.
        Start symbol: spec. *)

    | MenhirState1135 : (('s, _menhir_box_spec) _menhir_cell1_CONFIG _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_state
      (** State 1135.
        Stack shape : CONFIG IDENTIFIER.
        Start symbol: spec. *)

    | MenhirState1136 : ((('s, _menhir_box_spec) _menhir_cell1_CONFIG _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_state
      (** State 1136.
        Stack shape : CONFIG IDENTIFIER expr.
        Start symbol: spec. *)

    | MenhirState1137 : (((('s, _menhir_box_spec) _menhir_cell1_CONFIG _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON, _menhir_box_spec) _menhir_state
      (** State 1137.
        Stack shape : CONFIG IDENTIFIER expr SEMI_COLON.
        Start symbol: spec. *)

    | MenhirState1139 : (('s, _menhir_box_spec) _menhir_cell1_CONFIG _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_state
      (** State 1139.
        Stack shape : CONFIG IDENTIFIER.
        Start symbol: spec. *)

    | MenhirState1141 : ((('s, _menhir_box_spec) _menhir_cell1_CONFIG _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_state
      (** State 1141.
        Stack shape : CONFIG IDENTIFIER ty.
        Start symbol: spec. *)

    | MenhirState1142 : (((('s, _menhir_box_spec) _menhir_cell1_CONFIG _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_state
      (** State 1142.
        Stack shape : CONFIG IDENTIFIER ty expr.
        Start symbol: spec. *)

    | MenhirState1143 : ((((('s, _menhir_box_spec) _menhir_cell1_CONFIG _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON, _menhir_box_spec) _menhir_state
      (** State 1143.
        Stack shape : CONFIG IDENTIFIER ty expr SEMI_COLON.
        Start symbol: spec. *)

    | MenhirState1145 : (('s, _menhir_box_spec) _menhir_cell1_CONFIG _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_state
      (** State 1145.
        Stack shape : CONFIG IDENTIFIER.
        Start symbol: spec. *)

    | MenhirState1147 : ((('s, _menhir_box_spec) _menhir_cell1_CONFIG _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_state
      (** State 1147.
        Stack shape : CONFIG IDENTIFIER ty.
        Start symbol: spec. *)

    | MenhirState1148 : (((('s, _menhir_box_spec) _menhir_cell1_CONFIG _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_state
      (** State 1148.
        Stack shape : CONFIG IDENTIFIER ty expr.
        Start symbol: spec. *)

    | MenhirState1149 : ((((('s, _menhir_box_spec) _menhir_cell1_CONFIG _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON, _menhir_box_spec) _menhir_state
      (** State 1149.
        Stack shape : CONFIG IDENTIFIER ty expr SEMI_COLON.
        Start symbol: spec. *)

    | MenhirState1152 : (('s, _menhir_box_spec) _menhir_cell1_CONSTANT _menhir_cell0_MINUS, _menhir_box_spec) _menhir_state
      (** State 1152.
        Stack shape : CONSTANT MINUS.
        Start symbol: spec. *)

    | MenhirState1154 : ((('s, _menhir_box_spec) _menhir_cell1_CONSTANT _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_state
      (** State 1154.
        Stack shape : CONSTANT MINUS ty.
        Start symbol: spec. *)

    | MenhirState1155 : (((('s, _menhir_box_spec) _menhir_cell1_CONSTANT _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_state
      (** State 1155.
        Stack shape : CONSTANT MINUS ty expr.
        Start symbol: spec. *)

    | MenhirState1156 : ((((('s, _menhir_box_spec) _menhir_cell1_CONSTANT _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON, _menhir_box_spec) _menhir_state
      (** State 1156.
        Stack shape : CONSTANT MINUS ty expr SEMI_COLON.
        Start symbol: spec. *)

    | MenhirState1158 : (('s, _menhir_box_spec) _menhir_cell1_CONSTANT _menhir_cell0_MINUS, _menhir_box_spec) _menhir_state
      (** State 1158.
        Stack shape : CONSTANT MINUS.
        Start symbol: spec. *)

    | MenhirState1160 : ((('s, _menhir_box_spec) _menhir_cell1_CONSTANT _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_state
      (** State 1160.
        Stack shape : CONSTANT MINUS ty.
        Start symbol: spec. *)

    | MenhirState1161 : (((('s, _menhir_box_spec) _menhir_cell1_CONSTANT _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_state
      (** State 1161.
        Stack shape : CONSTANT MINUS ty expr.
        Start symbol: spec. *)

    | MenhirState1162 : ((((('s, _menhir_box_spec) _menhir_cell1_CONSTANT _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON, _menhir_box_spec) _menhir_state
      (** State 1162.
        Stack shape : CONSTANT MINUS ty expr SEMI_COLON.
        Start symbol: spec. *)

    | MenhirState1165 : (('s, _menhir_box_spec) _menhir_cell1_CONSTANT _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_state
      (** State 1165.
        Stack shape : CONSTANT IDENTIFIER.
        Start symbol: spec. *)

    | MenhirState1166 : ((('s, _menhir_box_spec) _menhir_cell1_CONSTANT _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_state
      (** State 1166.
        Stack shape : CONSTANT IDENTIFIER expr.
        Start symbol: spec. *)

    | MenhirState1167 : (((('s, _menhir_box_spec) _menhir_cell1_CONSTANT _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON, _menhir_box_spec) _menhir_state
      (** State 1167.
        Stack shape : CONSTANT IDENTIFIER expr SEMI_COLON.
        Start symbol: spec. *)

    | MenhirState1169 : (('s, _menhir_box_spec) _menhir_cell1_CONSTANT _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_state
      (** State 1169.
        Stack shape : CONSTANT IDENTIFIER.
        Start symbol: spec. *)

    | MenhirState1171 : ((('s, _menhir_box_spec) _menhir_cell1_CONSTANT _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_state
      (** State 1171.
        Stack shape : CONSTANT IDENTIFIER ty.
        Start symbol: spec. *)

    | MenhirState1172 : (((('s, _menhir_box_spec) _menhir_cell1_CONSTANT _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_state
      (** State 1172.
        Stack shape : CONSTANT IDENTIFIER ty expr.
        Start symbol: spec. *)

    | MenhirState1173 : ((((('s, _menhir_box_spec) _menhir_cell1_CONSTANT _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON, _menhir_box_spec) _menhir_state
      (** State 1173.
        Stack shape : CONSTANT IDENTIFIER ty expr SEMI_COLON.
        Start symbol: spec. *)

    | MenhirState1175 : (('s, _menhir_box_spec) _menhir_cell1_CONSTANT _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_state
      (** State 1175.
        Stack shape : CONSTANT IDENTIFIER.
        Start symbol: spec. *)

    | MenhirState1177 : ((('s, _menhir_box_spec) _menhir_cell1_CONSTANT _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_state
      (** State 1177.
        Stack shape : CONSTANT IDENTIFIER ty.
        Start symbol: spec. *)

    | MenhirState1178 : (((('s, _menhir_box_spec) _menhir_cell1_CONSTANT _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_state
      (** State 1178.
        Stack shape : CONSTANT IDENTIFIER ty expr.
        Start symbol: spec. *)

    | MenhirState1179 : ((((('s, _menhir_box_spec) _menhir_cell1_CONSTANT _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON, _menhir_box_spec) _menhir_state
      (** State 1179.
        Stack shape : CONSTANT IDENTIFIER ty expr SEMI_COLON.
        Start symbol: spec. *)

    | MenhirState1182 : ((((('s, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_end_semicolon, _menhir_box_spec) _menhir_state
      (** State 1182.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR BEGIN end_semicolon.
        Start symbol: spec. *)

    | MenhirState1184 : (((('s, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR, _menhir_box_spec) _menhir_state
      (** State 1184.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR.
        Start symbol: spec. *)

    | MenhirState1186 : ((((('s, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_state
      (** State 1186.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR ty.
        Start symbol: spec. *)

    | MenhirState1187 : (((((('s, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_state
      (** State 1187.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR ty expr.
        Start symbol: spec. *)

    | MenhirState1188 : ((((((('s, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_BEGIN, _menhir_box_spec) _menhir_state
      (** State 1188.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR ty expr BEGIN.
        Start symbol: spec. *)

    | MenhirState1189 : (((((((('s, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_, _menhir_box_spec) _menhir_state
      (** State 1189.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR ty expr BEGIN nonempty_list(stmt).
        Start symbol: spec. *)

    | MenhirState1190 : ((((((((('s, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_, _menhir_box_spec) _menhir_cell1_end_semicolon, _menhir_box_spec) _menhir_state
      (** State 1190.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR ty expr BEGIN nonempty_list(stmt) end_semicolon.
        Start symbol: spec. *)

    | MenhirState1192 : (((((((('s, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_BEGIN, _menhir_box_spec) _menhir_cell1_end_semicolon, _menhir_box_spec) _menhir_state
      (** State 1192.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR ty expr BEGIN end_semicolon.
        Start symbol: spec. *)

    | MenhirState1194 : ((((('s, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_state
      (** State 1194.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR ty BEGIN.
        Start symbol: spec. *)

    | MenhirState1195 : (((((('s, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_, _menhir_box_spec) _menhir_state
      (** State 1195.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR ty BEGIN nonempty_list(stmt).
        Start symbol: spec. *)

    | MenhirState1196 : ((((((('s, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_, _menhir_box_spec) _menhir_cell1_end_semicolon, _menhir_box_spec) _menhir_state
      (** State 1196.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR ty BEGIN nonempty_list(stmt) end_semicolon.
        Start symbol: spec. *)

    | MenhirState1198 : (((((('s, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_end_semicolon, _menhir_box_spec) _menhir_state
      (** State 1198.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR ty BEGIN end_semicolon.
        Start symbol: spec. *)

    | MenhirState1201 : (('s, _menhir_box_spec) _menhir_cell1_IDENTIFIER, _menhir_box_spec) _menhir_state
      (** State 1201.
        Stack shape : IDENTIFIER.
        Start symbol: spec. *)

    | MenhirState1203 : ((('s, _menhir_box_spec) _menhir_cell1_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_state
      (** State 1203.
        Stack shape : IDENTIFIER ty.
        Start symbol: spec. *)

    | MenhirState1205 : (('s, _menhir_box_spec) _menhir_cell1_IDENTIFIER, _menhir_box_spec) _menhir_state
      (** State 1205.
        Stack shape : IDENTIFIER.
        Start symbol: spec. *)

    | MenhirState1207 : ((('s, _menhir_box_spec) _menhir_cell1_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_state
      (** State 1207.
        Stack shape : IDENTIFIER ty.
        Start symbol: spec. *)

    | MenhirState1211 : (((('s, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_state
      (** State 1211.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR BEGIN.
        Start symbol: spec. *)

    | MenhirState1212 : ((((('s, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_, _menhir_box_spec) _menhir_state
      (** State 1212.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR BEGIN nonempty_list(stmt).
        Start symbol: spec. *)

    | MenhirState1213 : (((((('s, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_, _menhir_box_spec) _menhir_cell1_end_semicolon, _menhir_box_spec) _menhir_state
      (** State 1213.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR BEGIN nonempty_list(stmt) end_semicolon.
        Start symbol: spec. *)

    | MenhirState1215 : ((((('s, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_end_semicolon, _menhir_box_spec) _menhir_state
      (** State 1215.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR BEGIN end_semicolon.
        Start symbol: spec. *)

    | MenhirState1217 : (((('s, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_spec) _menhir_state
      (** State 1217.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR.
        Start symbol: spec. *)

    | MenhirState1219 : ((((('s, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_state
      (** State 1219.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR ty.
        Start symbol: spec. *)

    | MenhirState1220 : (((((('s, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_state
      (** State 1220.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR ty expr.
        Start symbol: spec. *)

    | MenhirState1221 : ((((((('s, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_BEGIN, _menhir_box_spec) _menhir_state
      (** State 1221.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR ty expr BEGIN.
        Start symbol: spec. *)

    | MenhirState1222 : (((((((('s, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_, _menhir_box_spec) _menhir_state
      (** State 1222.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR ty expr BEGIN nonempty_list(stmt).
        Start symbol: spec. *)

    | MenhirState1223 : ((((((((('s, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_, _menhir_box_spec) _menhir_cell1_end_semicolon, _menhir_box_spec) _menhir_state
      (** State 1223.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR ty expr BEGIN nonempty_list(stmt) end_semicolon.
        Start symbol: spec. *)

    | MenhirState1225 : (((((((('s, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_BEGIN, _menhir_box_spec) _menhir_cell1_end_semicolon, _menhir_box_spec) _menhir_state
      (** State 1225.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR ty expr BEGIN end_semicolon.
        Start symbol: spec. *)

    | MenhirState1227 : ((((('s, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_state
      (** State 1227.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR ty BEGIN.
        Start symbol: spec. *)

    | MenhirState1228 : (((((('s, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_, _menhir_box_spec) _menhir_state
      (** State 1228.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR ty BEGIN nonempty_list(stmt).
        Start symbol: spec. *)

    | MenhirState1229 : ((((((('s, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_, _menhir_box_spec) _menhir_cell1_end_semicolon, _menhir_box_spec) _menhir_state
      (** State 1229.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR ty BEGIN nonempty_list(stmt) end_semicolon.
        Start symbol: spec. *)

    | MenhirState1231 : (((((('s, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_end_semicolon, _menhir_box_spec) _menhir_state
      (** State 1231.
        Stack shape : FUNC IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR ty BEGIN end_semicolon.
        Start symbol: spec. *)

    | MenhirState1234 : (((((('s, _menhir_box_spec) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_end_semicolon, _menhir_box_spec) _menhir_state
      (** State 1234.
        Stack shape : GETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR ty BEGIN end_semicolon.
        Start symbol: spec. *)

    | MenhirState1238 : (((('s, _menhir_box_spec) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_spec) _menhir_state
      (** State 1238.
        Stack shape : GETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR.
        Start symbol: spec. *)

    | MenhirState1240 : ((((('s, _menhir_box_spec) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_state
      (** State 1240.
        Stack shape : GETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR ty BEGIN.
        Start symbol: spec. *)

    | MenhirState1241 : (((((('s, _menhir_box_spec) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_, _menhir_box_spec) _menhir_state
      (** State 1241.
        Stack shape : GETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR ty BEGIN nonempty_list(stmt).
        Start symbol: spec. *)

    | MenhirState1242 : ((((((('s, _menhir_box_spec) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_, _menhir_box_spec) _menhir_cell1_end_semicolon, _menhir_box_spec) _menhir_state
      (** State 1242.
        Stack shape : GETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR ty BEGIN nonempty_list(stmt) end_semicolon.
        Start symbol: spec. *)

    | MenhirState1244 : (((((('s, _menhir_box_spec) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_end_semicolon, _menhir_box_spec) _menhir_state
      (** State 1244.
        Stack shape : GETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR ty BEGIN end_semicolon.
        Start symbol: spec. *)

    | MenhirState1247 : (('s, _menhir_box_spec) _menhir_cell1_LET _menhir_cell0_MINUS, _menhir_box_spec) _menhir_state
      (** State 1247.
        Stack shape : LET MINUS.
        Start symbol: spec. *)

    | MenhirState1249 : ((('s, _menhir_box_spec) _menhir_cell1_LET _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_state
      (** State 1249.
        Stack shape : LET MINUS ty.
        Start symbol: spec. *)

    | MenhirState1250 : (((('s, _menhir_box_spec) _menhir_cell1_LET _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_state
      (** State 1250.
        Stack shape : LET MINUS ty expr.
        Start symbol: spec. *)

    | MenhirState1251 : ((((('s, _menhir_box_spec) _menhir_cell1_LET _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON, _menhir_box_spec) _menhir_state
      (** State 1251.
        Stack shape : LET MINUS ty expr SEMI_COLON.
        Start symbol: spec. *)

    | MenhirState1253 : (('s, _menhir_box_spec) _menhir_cell1_LET _menhir_cell0_MINUS, _menhir_box_spec) _menhir_state
      (** State 1253.
        Stack shape : LET MINUS.
        Start symbol: spec. *)

    | MenhirState1255 : ((('s, _menhir_box_spec) _menhir_cell1_LET _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_state
      (** State 1255.
        Stack shape : LET MINUS ty.
        Start symbol: spec. *)

    | MenhirState1256 : (((('s, _menhir_box_spec) _menhir_cell1_LET _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_state
      (** State 1256.
        Stack shape : LET MINUS ty expr.
        Start symbol: spec. *)

    | MenhirState1257 : ((((('s, _menhir_box_spec) _menhir_cell1_LET _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON, _menhir_box_spec) _menhir_state
      (** State 1257.
        Stack shape : LET MINUS ty expr SEMI_COLON.
        Start symbol: spec. *)

    | MenhirState1260 : (('s, _menhir_box_spec) _menhir_cell1_LET _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_state
      (** State 1260.
        Stack shape : LET IDENTIFIER.
        Start symbol: spec. *)

    | MenhirState1261 : ((('s, _menhir_box_spec) _menhir_cell1_LET _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_state
      (** State 1261.
        Stack shape : LET IDENTIFIER expr.
        Start symbol: spec. *)

    | MenhirState1262 : (((('s, _menhir_box_spec) _menhir_cell1_LET _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON, _menhir_box_spec) _menhir_state
      (** State 1262.
        Stack shape : LET IDENTIFIER expr SEMI_COLON.
        Start symbol: spec. *)

    | MenhirState1264 : (('s, _menhir_box_spec) _menhir_cell1_LET _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_state
      (** State 1264.
        Stack shape : LET IDENTIFIER.
        Start symbol: spec. *)

    | MenhirState1266 : ((('s, _menhir_box_spec) _menhir_cell1_LET _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_state
      (** State 1266.
        Stack shape : LET IDENTIFIER ty.
        Start symbol: spec. *)

    | MenhirState1267 : (((('s, _menhir_box_spec) _menhir_cell1_LET _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_state
      (** State 1267.
        Stack shape : LET IDENTIFIER ty expr.
        Start symbol: spec. *)

    | MenhirState1268 : ((((('s, _menhir_box_spec) _menhir_cell1_LET _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON, _menhir_box_spec) _menhir_state
      (** State 1268.
        Stack shape : LET IDENTIFIER ty expr SEMI_COLON.
        Start symbol: spec. *)

    | MenhirState1270 : (('s, _menhir_box_spec) _menhir_cell1_LET _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_state
      (** State 1270.
        Stack shape : LET IDENTIFIER.
        Start symbol: spec. *)

    | MenhirState1272 : ((('s, _menhir_box_spec) _menhir_cell1_LET _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_state
      (** State 1272.
        Stack shape : LET IDENTIFIER ty.
        Start symbol: spec. *)

    | MenhirState1273 : (((('s, _menhir_box_spec) _menhir_cell1_LET _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_state
      (** State 1273.
        Stack shape : LET IDENTIFIER ty expr.
        Start symbol: spec. *)

    | MenhirState1274 : ((((('s, _menhir_box_spec) _menhir_cell1_LET _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON, _menhir_box_spec) _menhir_state
      (** State 1274.
        Stack shape : LET IDENTIFIER ty expr SEMI_COLON.
        Start symbol: spec. *)

    | MenhirState1278 : ((('s, _menhir_box_spec) _menhir_cell1_PRAGMA _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_SEMI_COLON, _menhir_box_spec) _menhir_state
      (** State 1278.
        Stack shape : PRAGMA IDENTIFIER separated_nonempty_list(COMMA,expr) SEMI_COLON.
        Start symbol: spec. *)

    | MenhirState1281 : (((((('s, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_end_semicolon, _menhir_box_spec) _menhir_state
      (** State 1281.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR IDENTIFIER ty BEGIN end_semicolon.
        Start symbol: spec. *)

    | MenhirState1283 : (((('s, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_state
      (** State 1283.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR IDENTIFIER.
        Start symbol: spec. *)

    | MenhirState1285 : ((((('s, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_state
      (** State 1285.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR IDENTIFIER ty BEGIN.
        Start symbol: spec. *)

    | MenhirState1286 : (((((('s, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_, _menhir_box_spec) _menhir_state
      (** State 1286.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR IDENTIFIER ty BEGIN nonempty_list(stmt).
        Start symbol: spec. *)

    | MenhirState1287 : ((((((('s, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_, _menhir_box_spec) _menhir_cell1_end_semicolon, _menhir_box_spec) _menhir_state
      (** State 1287.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR IDENTIFIER ty BEGIN nonempty_list(stmt) end_semicolon.
        Start symbol: spec. *)

    | MenhirState1289 : (((((('s, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_end_semicolon, _menhir_box_spec) _menhir_state
      (** State 1289.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR RPAR IDENTIFIER ty BEGIN end_semicolon.
        Start symbol: spec. *)

    | MenhirState1295 : (((('s, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_state
      (** State 1295.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR IDENTIFIER.
        Start symbol: spec. *)

    | MenhirState1297 : ((((('s, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_state
      (** State 1297.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR IDENTIFIER ty BEGIN.
        Start symbol: spec. *)

    | MenhirState1298 : (((((('s, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_, _menhir_box_spec) _menhir_state
      (** State 1298.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR IDENTIFIER ty BEGIN nonempty_list(stmt).
        Start symbol: spec. *)

    | MenhirState1299 : ((((((('s, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_, _menhir_box_spec) _menhir_cell1_end_semicolon, _menhir_box_spec) _menhir_state
      (** State 1299.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR IDENTIFIER ty BEGIN nonempty_list(stmt) end_semicolon.
        Start symbol: spec. *)

    | MenhirState1301 : (((((('s, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_end_semicolon, _menhir_box_spec) _menhir_state
      (** State 1301.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR IDENTIFIER ty BEGIN end_semicolon.
        Start symbol: spec. *)

    | MenhirState1303 : (((('s, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_state
      (** State 1303.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR IDENTIFIER.
        Start symbol: spec. *)

    | MenhirState1305 : ((((('s, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_state
      (** State 1305.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR IDENTIFIER ty BEGIN.
        Start symbol: spec. *)

    | MenhirState1306 : (((((('s, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_, _menhir_box_spec) _menhir_state
      (** State 1306.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR IDENTIFIER ty BEGIN nonempty_list(stmt).
        Start symbol: spec. *)

    | MenhirState1307 : ((((((('s, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_, _menhir_box_spec) _menhir_cell1_end_semicolon, _menhir_box_spec) _menhir_state
      (** State 1307.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR IDENTIFIER ty BEGIN nonempty_list(stmt) end_semicolon.
        Start symbol: spec. *)

    | MenhirState1309 : (((((('s, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_end_semicolon, _menhir_box_spec) _menhir_state
      (** State 1309.
        Stack shape : SETTER IDENTIFIER loption(braced(nclist(opt_typed_identifier))) LPAR separated_nonempty_list(COMMA,typed_identifier) RPAR IDENTIFIER ty BEGIN end_semicolon.
        Start symbol: spec. *)

    | MenhirState1312 : (('s, _menhir_box_spec) _menhir_cell1_TYPE _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_state
      (** State 1312.
        Stack shape : TYPE IDENTIFIER.
        Start symbol: spec. *)

    | MenhirState1314 : ((('s, _menhir_box_spec) _menhir_cell1_TYPE _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_RECORD _menhir_cell0_LBRACE, _menhir_box_spec) _menhir_state
      (** State 1314.
        Stack shape : TYPE IDENTIFIER RECORD LBRACE.
        Start symbol: spec. *)

    | MenhirState1318 : ((('s, _menhir_box_spec) _menhir_cell1_TYPE _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_EXCEPTION _menhir_cell0_LBRACE, _menhir_box_spec) _menhir_state
      (** State 1318.
        Stack shape : TYPE IDENTIFIER EXCEPTION LBRACE.
        Start symbol: spec. *)

    | MenhirState1322 : ((('s, _menhir_box_spec) _menhir_cell1_TYPE _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ENUMERATION _menhir_cell0_LBRACE, _menhir_box_spec) _menhir_state
      (** State 1322.
        Stack shape : TYPE IDENTIFIER ENUMERATION LBRACE.
        Start symbol: spec. *)

    | MenhirState1324 : (('s, _menhir_box_spec) _menhir_cell1_IDENTIFIER, _menhir_box_spec) _menhir_state
      (** State 1324.
        Stack shape : IDENTIFIER.
        Start symbol: spec. *)

    | MenhirState1330 : ((('s, _menhir_box_spec) _menhir_cell1_TYPE _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty_decl _menhir_cell0_SUBTYPES _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_state
      (** State 1330.
        Stack shape : TYPE IDENTIFIER ty_decl SUBTYPES IDENTIFIER.
        Start symbol: spec. *)

    | MenhirState1333 : ((('s, _menhir_box_spec) _menhir_cell1_TYPE _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty_decl _menhir_cell0_option_subtype_ _menhir_cell0_SEMI_COLON, _menhir_box_spec) _menhir_state
      (** State 1333.
        Stack shape : TYPE IDENTIFIER ty_decl option(subtype) SEMI_COLON.
        Start symbol: spec. *)

    | MenhirState1337 : (('s, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_spec) _menhir_state
      (** State 1337.
        Stack shape : VAR MINUS.
        Start symbol: spec. *)

    | MenhirState1339 : ((('s, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_SEMI_COLON, _menhir_box_spec) _menhir_state
      (** State 1339.
        Stack shape : VAR MINUS ty SEMI_COLON.
        Start symbol: spec. *)

    | MenhirState1341 : ((('s, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_state
      (** State 1341.
        Stack shape : VAR MINUS ty.
        Start symbol: spec. *)

    | MenhirState1342 : (((('s, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_state
      (** State 1342.
        Stack shape : VAR MINUS ty expr.
        Start symbol: spec. *)

    | MenhirState1343 : ((((('s, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON, _menhir_box_spec) _menhir_state
      (** State 1343.
        Stack shape : VAR MINUS ty expr SEMI_COLON.
        Start symbol: spec. *)

    | MenhirState1345 : (('s, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_spec) _menhir_state
      (** State 1345.
        Stack shape : VAR MINUS.
        Start symbol: spec. *)

    | MenhirState1347 : ((('s, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_SEMI_COLON, _menhir_box_spec) _menhir_state
      (** State 1347.
        Stack shape : VAR MINUS ty SEMI_COLON.
        Start symbol: spec. *)

    | MenhirState1349 : ((('s, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_state
      (** State 1349.
        Stack shape : VAR MINUS ty.
        Start symbol: spec. *)

    | MenhirState1350 : (((('s, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_state
      (** State 1350.
        Stack shape : VAR MINUS ty expr.
        Start symbol: spec. *)

    | MenhirState1351 : ((((('s, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON, _menhir_box_spec) _menhir_state
      (** State 1351.
        Stack shape : VAR MINUS ty expr SEMI_COLON.
        Start symbol: spec. *)

    | MenhirState1354 : (('s, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_state
      (** State 1354.
        Stack shape : VAR IDENTIFIER.
        Start symbol: spec. *)

    | MenhirState1355 : ((('s, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_state
      (** State 1355.
        Stack shape : VAR IDENTIFIER expr.
        Start symbol: spec. *)

    | MenhirState1356 : (((('s, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON, _menhir_box_spec) _menhir_state
      (** State 1356.
        Stack shape : VAR IDENTIFIER expr SEMI_COLON.
        Start symbol: spec. *)

    | MenhirState1358 : (('s, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_state
      (** State 1358.
        Stack shape : VAR IDENTIFIER.
        Start symbol: spec. *)

    | MenhirState1360 : ((('s, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_SEMI_COLON, _menhir_box_spec) _menhir_state
      (** State 1360.
        Stack shape : VAR IDENTIFIER ty SEMI_COLON.
        Start symbol: spec. *)

    | MenhirState1362 : ((('s, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_state
      (** State 1362.
        Stack shape : VAR IDENTIFIER ty.
        Start symbol: spec. *)

    | MenhirState1363 : (((('s, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_state
      (** State 1363.
        Stack shape : VAR IDENTIFIER ty expr.
        Start symbol: spec. *)

    | MenhirState1364 : ((((('s, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON, _menhir_box_spec) _menhir_state
      (** State 1364.
        Stack shape : VAR IDENTIFIER ty expr SEMI_COLON.
        Start symbol: spec. *)

    | MenhirState1366 : (('s, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_state
      (** State 1366.
        Stack shape : VAR IDENTIFIER.
        Start symbol: spec. *)

    | MenhirState1368 : ((('s, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_SEMI_COLON, _menhir_box_spec) _menhir_state
      (** State 1368.
        Stack shape : VAR IDENTIFIER ty SEMI_COLON.
        Start symbol: spec. *)

    | MenhirState1370 : ((('s, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_state
      (** State 1370.
        Stack shape : VAR IDENTIFIER ty.
        Start symbol: spec. *)

    | MenhirState1371 : (((('s, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_state
      (** State 1371.
        Stack shape : VAR IDENTIFIER ty expr.
        Start symbol: spec. *)

    | MenhirState1372 : ((((('s, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON, _menhir_box_spec) _menhir_state
      (** State 1372.
        Stack shape : VAR IDENTIFIER ty expr SEMI_COLON.
        Start symbol: spec. *)

    | MenhirState1377 : ('s, _menhir_box_stmts) _menhir_state
      (** State 1377.
        Stack shape : .
        Start symbol: stmts. *)

  
  and ('s, 'r) _menhir_cell1_decl_item = 
    | MenhirCell1_decl_item of 's * ('s, 'r) _menhir_state * (AST.local_decl_item)
  
  and ('s, 'r) _menhir_cell1_end_semicolon = 
    | MenhirCell1_end_semicolon of 's * ('s, 'r) _menhir_state * (unit) * Lexing.position
  
  and ('s, 'r) _menhir_cell1_expr = 
    | MenhirCell1_expr of 's * ('s, 'r) _menhir_state * (AST.expr) * Lexing.position * Lexing.position
  
  and ('s, 'r) _menhir_cell1_expr_pattern = 
    | MenhirCell1_expr_pattern of 's * ('s, 'r) _menhir_state * (AST.expr) * Lexing.position * Lexing.position
  
  and ('s, 'r) _menhir_cell1_lexpr = 
    | MenhirCell1_lexpr of 's * ('s, 'r) _menhir_state * (AST.lexpr) * Lexing.position
  
  and ('s, 'r) _menhir_cell1_lexpr_atom_desc = 
    | MenhirCell1_lexpr_atom_desc of 's * ('s, 'r) _menhir_state * (AST.lexpr_desc) * Lexing.position * Lexing.position
  
  and ('s, 'r) _menhir_cell1_loption___anonymous_21_ = 
    | MenhirCell1_loption___anonymous_21_ of 's * ('s, 'r) _menhir_state * (AST.field list) * Lexing.position
  
  and ('s, 'r) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ = 
    | MenhirCell1_loption_braced_nclist_opt_typed_identifier___ of 's * ('s, 'r) _menhir_state * ((string * AST.ty option) list)
  
  and ('s, 'r) _menhir_cell1_loption_non_empty_trailing_list_COMMA_bitfield__ = 
    | MenhirCell1_loption_non_empty_trailing_list_COMMA_bitfield__ of 's * ('s, 'r) _menhir_state * (AST.bitfield list)
  
  and ('s, 'r) _menhir_cell1_loption_plist_expr__ = 
    | MenhirCell1_loption_plist_expr__ of 's * ('s, 'r) _menhir_state * (AST.expr list) * Lexing.position
  
  and ('s, 'r) _menhir_cell1_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__ = 
    | MenhirCell1_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__ of 's * ('s, 'r) _menhir_state * (AST.case_alt list)
  
  and ('s, 'r) _menhir_cell1_nonempty_list_catcher_ = 
    | MenhirCell1_nonempty_list_catcher_ of 's * ('s, 'r) _menhir_state * (AST.catcher list)
  
  and ('s, 'r) _menhir_cell1_nonempty_list_stmt_ = 
    | MenhirCell1_nonempty_list_stmt_ of 's * ('s, 'r) _menhir_state * (AST.stmt list) * Lexing.position
  
  and 's _menhir_cell0_option_subtype_ = 
    | MenhirCell0_option_subtype_ of 's * ((string * AST.field list) option)
  
  and ('s, 'r) _menhir_cell1_pattern = 
    | MenhirCell1_pattern of 's * ('s, 'r) _menhir_state * (AST.pattern) * Lexing.position * Lexing.position
  
  and ('s, 'r) _menhir_cell1_s_else = 
    | MenhirCell1_s_else of 's * ('s, 'r) _menhir_state * (AST.stmt) * Lexing.position
  
  and ('s, 'r) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ = 
    | MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ of 's * ('s, 'r) _menhir_state * (string list)
  
  and ('s, 'r) _menhir_cell1_separated_nonempty_list_COMMA_decl_item_ = 
    | MenhirCell1_separated_nonempty_list_COMMA_decl_item_ of 's * ('s, 'r) _menhir_state * (AST.local_decl_item list)
  
  and ('s, 'r) _menhir_cell1_separated_nonempty_list_COMMA_expr_ = 
    | MenhirCell1_separated_nonempty_list_COMMA_expr_ of 's * ('s, 'r) _menhir_state * (AST.expr list)
  
  and ('s, 'r) _menhir_cell1_separated_nonempty_list_COMMA_pattern_ = 
    | MenhirCell1_separated_nonempty_list_COMMA_pattern_ of 's * ('s, 'r) _menhir_state * (AST.pattern list) * Lexing.position * Lexing.position
  
  and ('s, 'r) _menhir_cell1_separated_nonempty_list_COMMA_slice_ = 
    | MenhirCell1_separated_nonempty_list_COMMA_slice_ of 's * ('s, 'r) _menhir_state * (AST.slice list)
  
  and ('s, 'r) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ = 
    | MenhirCell1_separated_nonempty_list_COMMA_typed_identifier_ of 's * ('s, 'r) _menhir_state * (AST.typed_identifier list)
  
  and ('s, 'r) _menhir_cell1_ty = 
    | MenhirCell1_ty of 's * ('s, 'r) _menhir_state * (AST.ty) * Lexing.position
  
  and ('s, 'r) _menhir_cell1_ty_decl = 
    | MenhirCell1_ty_decl of 's * ('s, 'r) _menhir_state * (AST.ty)
  
  and ('s, 'r) _menhir_cell1_AND = 
    | MenhirCell1_AND of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_ARBITRARY = 
    | MenhirCell1_ARBITRARY of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_ARRAY = 
    | MenhirCell1_ARRAY of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_ARROBASE_LOOPLIMIT = 
    | MenhirCell1_ARROBASE_LOOPLIMIT of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_ARROW = 
    | MenhirCell1_ARROW of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_AS = 
    | MenhirCell1_AS of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_ASSERT = 
    | MenhirCell1_ASSERT of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_BAND = 
    | MenhirCell1_BAND of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_BEGIN = 
    | MenhirCell1_BEGIN of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and 's _menhir_cell0_BEGIN = 
    | MenhirCell0_BEGIN of 's * Lexing.position
  
  and ('s, 'r) _menhir_cell1_BEQ = 
    | MenhirCell1_BEQ of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_BITS = 
    | MenhirCell1_BITS of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_BNOT = 
    | MenhirCell1_BNOT of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and 's _menhir_cell0_BNOT = 
    | MenhirCell0_BNOT of 's * Lexing.position
  
  and ('s, 'r) _menhir_cell1_BOR = 
    | MenhirCell1_BOR of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_CASE = 
    | MenhirCell1_CASE of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_COLON = 
    | MenhirCell1_COLON of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_COLON_COLON = 
    | MenhirCell1_COLON_COLON of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_COMMA = 
    | MenhirCell1_COMMA of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_CONCAT = 
    | MenhirCell1_CONCAT of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_CONFIG = 
    | MenhirCell1_CONFIG of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_CONSTANT = 
    | MenhirCell1_CONSTANT of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_DEBUG = 
    | MenhirCell1_DEBUG of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_DIV = 
    | MenhirCell1_DIV of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_DIVRM = 
    | MenhirCell1_DIVRM of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_DO = 
    | MenhirCell1_DO of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_DOT = 
    | MenhirCell1_DOT of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_DOWNTO = 
    | MenhirCell1_DOWNTO of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_ELSE = 
    | MenhirCell1_ELSE of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_ELSIF = 
    | MenhirCell1_ELSIF of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_END = 
    | MenhirCell1_END of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_ENUMERATION = 
    | MenhirCell1_ENUMERATION of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_EOR = 
    | MenhirCell1_EOR of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_EQ_OP = 
    | MenhirCell1_EQ_OP of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_EXCEPTION = 
    | MenhirCell1_EXCEPTION of 's * ('s, 'r) _menhir_state * Lexing.position * Lexing.position
  
  and ('s, 'r) _menhir_cell1_FOR = 
    | MenhirCell1_FOR of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_FUNC = 
    | MenhirCell1_FUNC of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_GEQ = 
    | MenhirCell1_GEQ of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and 's _menhir_cell0_GEQ = 
    | MenhirCell0_GEQ of 's * Lexing.position
  
  and ('s, 'r) _menhir_cell1_GETTER = 
    | MenhirCell1_GETTER of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_GT = 
    | MenhirCell1_GT of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_IDENTIFIER = 
    | MenhirCell1_IDENTIFIER of 's * ('s, 'r) _menhir_state * (
# 40 "asllib/Tokens.mly"
       (string)
# 3867 "asllib/Parser.ml"
  ) * Lexing.position * Lexing.position
  
  and 's _menhir_cell0_IDENTIFIER = 
    | MenhirCell0_IDENTIFIER of 's * (
# 40 "asllib/Tokens.mly"
       (string)
# 3874 "asllib/Parser.ml"
  ) * Lexing.position * Lexing.position
  
  and ('s, 'r) _menhir_cell1_IF = 
    | MenhirCell1_IF of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_IMPL = 
    | MenhirCell1_IMPL of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_IN = 
    | MenhirCell1_IN of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_INTEGER = 
    | MenhirCell1_INTEGER of 's * ('s, 'r) _menhir_state * Lexing.position * Lexing.position
  
  and ('s, 'r) _menhir_cell1_LBRACE = 
    | MenhirCell1_LBRACE of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and 's _menhir_cell0_LBRACE = 
    | MenhirCell0_LBRACE of 's * Lexing.position
  
  and ('s, 'r) _menhir_cell1_LBRACKET = 
    | MenhirCell1_LBRACKET of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_LEQ = 
    | MenhirCell1_LEQ of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and 's _menhir_cell0_LEQ = 
    | MenhirCell0_LEQ of 's * Lexing.position
  
  and ('s, 'r) _menhir_cell1_LET = 
    | MenhirCell1_LET of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_LLBRACKET = 
    | MenhirCell1_LLBRACKET of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_LOOPLIMIT = 
    | MenhirCell1_LOOPLIMIT of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_LPAR = 
    | MenhirCell1_LPAR of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and 's _menhir_cell0_LPAR = 
    | MenhirCell0_LPAR of 's * Lexing.position
  
  and ('s, 'r) _menhir_cell1_LT = 
    | MenhirCell1_LT of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_MINUS = 
    | MenhirCell1_MINUS of 's * ('s, 'r) _menhir_state * Lexing.position * Lexing.position
  
  and 's _menhir_cell0_MINUS = 
    | MenhirCell0_MINUS of 's * Lexing.position * Lexing.position
  
  and ('s, 'r) _menhir_cell1_MOD = 
    | MenhirCell1_MOD of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_MUL = 
    | MenhirCell1_MUL of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_NEQ = 
    | MenhirCell1_NEQ of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_NOT = 
    | MenhirCell1_NOT of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_OF = 
    | MenhirCell1_OF of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_OR = 
    | MenhirCell1_OR of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_OTHERWISE = 
    | MenhirCell1_OTHERWISE of 's * ('s, 'r) _menhir_state * Lexing.position * Lexing.position
  
  and ('s, 'r) _menhir_cell1_PASS = 
    | MenhirCell1_PASS of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_PLUS = 
    | MenhirCell1_PLUS of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_PLUS_COLON = 
    | MenhirCell1_PLUS_COLON of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_POW = 
    | MenhirCell1_POW of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_PRAGMA = 
    | MenhirCell1_PRAGMA of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_PRINT = 
    | MenhirCell1_PRINT of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_RBRACE = 
    | MenhirCell1_RBRACE of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and 's _menhir_cell0_RBRACE = 
    | MenhirCell0_RBRACE of 's * Lexing.position
  
  and ('s, 'r) _menhir_cell1_RBRACKET = 
    | MenhirCell1_RBRACKET of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and 's _menhir_cell0_RBRACKET = 
    | MenhirCell0_RBRACKET of 's * Lexing.position
  
  and ('s, 'r) _menhir_cell1_RDIV = 
    | MenhirCell1_RDIV of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_RECORD = 
    | MenhirCell1_RECORD of 's * ('s, 'r) _menhir_state * Lexing.position * Lexing.position
  
  and ('s, 'r) _menhir_cell1_REPEAT = 
    | MenhirCell1_REPEAT of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and 's _menhir_cell0_REPEAT = 
    | MenhirCell0_REPEAT of 's * Lexing.position
  
  and ('s, 'r) _menhir_cell1_RETURN = 
    | MenhirCell1_RETURN of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_RPAR = 
    | MenhirCell1_RPAR of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and 's _menhir_cell0_RPAR = 
    | MenhirCell0_RPAR of 's * Lexing.position
  
  and ('s, 'r) _menhir_cell1_SEMI_COLON = 
    | MenhirCell1_SEMI_COLON of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and 's _menhir_cell0_SEMI_COLON = 
    | MenhirCell0_SEMI_COLON of 's * Lexing.position
  
  and ('s, 'r) _menhir_cell1_SETTER = 
    | MenhirCell1_SETTER of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_SHL = 
    | MenhirCell1_SHL of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_SHR = 
    | MenhirCell1_SHR of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_SLICING = 
    | MenhirCell1_SLICING of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_STAR_COLON = 
    | MenhirCell1_STAR_COLON of 's * ('s, 'r) _menhir_state
  
  and 's _menhir_cell0_SUBTYPES = 
    | MenhirCell0_SUBTYPES of 's * Lexing.position
  
  and ('s, 'r) _menhir_cell1_THEN = 
    | MenhirCell1_THEN of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_THROW = 
    | MenhirCell1_THROW of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_TO = 
    | MenhirCell1_TO of 's * ('s, 'r) _menhir_state
  
  and ('s, 'r) _menhir_cell1_TRY = 
    | MenhirCell1_TRY of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_TYPE = 
    | MenhirCell1_TYPE of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_UNREACHABLE = 
    | MenhirCell1_UNREACHABLE of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_VAR = 
    | MenhirCell1_VAR of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_WHEN = 
    | MenhirCell1_WHEN of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and ('s, 'r) _menhir_cell1_WHILE = 
    | MenhirCell1_WHILE of 's * ('s, 'r) _menhir_state * Lexing.position
  
  and 's _menhir_cell0_WHILE = 
    | MenhirCell0_WHILE of 's * Lexing.position
  
  and ('s, 'r) _menhir_cell1_WITH = 
    | MenhirCell1_WITH of 's * ('s, 'r) _menhir_state
  
  and _menhir_box_stmts = 
    | MenhirBox_stmts of (AST.stmt) [@@unboxed]
  
  and _menhir_box_spec = 
    | MenhirBox_spec of (AST.t) [@@unboxed]
  
  and _menhir_box_opn = 
    | MenhirBox_opn of (AST.t) [@@unboxed]
  
  let _menhir_action_003 =
    fun _1 _1_inlined1 _2 ->
      let as_ty =
        let _1 = _1_inlined1 in
        _2
      in
      let untyped_decl_item =
        let _1_1 = _1 in
        
# 417 "asllib/Parser.mly"
                              LDI_Var      
# 4077 "asllib/Parser.ml"
         _1_1
      in
      (
# 413 "asllib/Parser.mly"
                                      LDI_Typed    
# 4083 "asllib/Parser.ml"
       (untyped_decl_item, as_ty) : (AST.local_decl_item))
  
  let _menhir_action_004 =
    fun _1 _1_inlined1 _2 ->
      let as_ty =
        let _1 = _1_inlined1 in
        _2
      in
      let untyped_decl_item =
        let _1_1 = _1 in
        
# 417 "asllib/Parser.mly"
                              LDI_Var      
# 4097 "asllib/Parser.ml"
         _1_1
      in
      (
# 413 "asllib/Parser.mly"
                                      LDI_Typed    
# 4103 "asllib/Parser.ml"
       (untyped_decl_item, as_ty) : (AST.local_decl_item))
  
  let _menhir_action_005 =
    fun _1 _1_inlined1 _2 ->
      let as_ty =
        let _1 = _1_inlined1 in
        _2
      in
      let untyped_decl_item = 
# 418 "asllib/Parser.mly"
                            ( LDI_Discard )
# 4115 "asllib/Parser.ml"
       in
      (
# 413 "asllib/Parser.mly"
                                      LDI_Typed    
# 4120 "asllib/Parser.ml"
       (untyped_decl_item, as_ty) : (AST.local_decl_item))
  
  let _menhir_action_006 =
    fun _1 _1_inlined1 _2 ->
      let as_ty =
        let _1 = _1_inlined1 in
        _2
      in
      let untyped_decl_item = 
# 418 "asllib/Parser.mly"
                            ( LDI_Discard )
# 4132 "asllib/Parser.ml"
       in
      (
# 413 "asllib/Parser.mly"
                                      LDI_Typed    
# 4137 "asllib/Parser.ml"
       (untyped_decl_item, as_ty) : (AST.local_decl_item))
  
  let _menhir_action_007 =
    fun _1_inlined1 _2 _2_inlined1 li x ->
      let as_ty =
        let (_2, _1) = (_2_inlined1, _1_inlined1) in
        _2
      in
      let untyped_decl_item =
        let _1 =
          let x = 
# 177 "asllib/Parser.mly"
                                                    ( x :: li )
# 4151 "asllib/Parser.ml"
           in
          
# 205 "<standard.mly>"
    ( x )
# 4156 "asllib/Parser.ml"
          
        in
        let _1_1 = _1 in
        
# 419 "asllib/Parser.mly"
                              LDI_Tuple    
# 4163 "asllib/Parser.ml"
         _1_1
      in
      (
# 413 "asllib/Parser.mly"
                                      LDI_Typed    
# 4169 "asllib/Parser.ml"
       (untyped_decl_item, as_ty) : (AST.local_decl_item))
  
  let _menhir_action_008 =
    fun _1_inlined1 _2 _2_inlined1 li x ->
      let as_ty =
        let (_2, _1) = (_2_inlined1, _1_inlined1) in
        _2
      in
      let untyped_decl_item =
        let _1 =
          let x = 
# 177 "asllib/Parser.mly"
                                                    ( x :: li )
# 4183 "asllib/Parser.ml"
           in
          
# 205 "<standard.mly>"
    ( x )
# 4188 "asllib/Parser.ml"
          
        in
        let _1_1 = _1 in
        
# 419 "asllib/Parser.mly"
                              LDI_Tuple    
# 4195 "asllib/Parser.ml"
         _1_1
      in
      (
# 413 "asllib/Parser.mly"
                                      LDI_Typed    
# 4201 "asllib/Parser.ml"
       (untyped_decl_item, as_ty) : (AST.local_decl_item))
  
  let _menhir_action_009 =
    fun _1 ->
      let _1_1 = _1 in
      (
# 417 "asllib/Parser.mly"
                              LDI_Var      
# 4210 "asllib/Parser.ml"
       _1_1 : (AST.local_decl_item))
  
  let _menhir_action_010 =
    fun _1 ->
      (
# 418 "asllib/Parser.mly"
                            ( LDI_Discard )
# 4218 "asllib/Parser.ml"
       : (AST.local_decl_item))
  
  let _menhir_action_011 =
    fun _2 li x ->
      let _1 =
        let x = 
# 177 "asllib/Parser.mly"
                                                    ( x :: li )
# 4227 "asllib/Parser.ml"
         in
        
# 205 "<standard.mly>"
    ( x )
# 4232 "asllib/Parser.ml"
        
      in
      let _1_1 = _1 in
      (
# 419 "asllib/Parser.mly"
                              LDI_Tuple    
# 4239 "asllib/Parser.ml"
       _1_1 : (AST.local_decl_item))
  
  let _menhir_action_012 =
    fun _1 _2 ->
      (_2 : (AST.expr))
  
  let _menhir_action_013 =
    fun _1 _3 _endpos_e_else_ _startpos__1_ c e e_else ->
      let desc = 
# 252 "asllib/Parser.mly"
                                                        E_Cond 
# 4251 "asllib/Parser.ml"
       (c, e, e_else) in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e_else_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4259 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_014 =
    fun _1 _endpos_opt_ _startpos__1_ opt ->
      let _endpos = _endpos_opt_ in
      let _startpos = _startpos__1_ in
      (
# 187 "asllib/Parser.mly"
                                                  (
      if Option.is_none opt && not Config.allow_no_end_semicolon then
          Error.fatal_here _startpos _endpos @@ Error.ObsoleteSyntax "Missing ';' after 'end' keyword.";
  )
# 4272 "asllib/Parser.ml"
       : (unit))
  
  let _menhir_action_015 =
    fun _endpos_i_ _startpos_i_ i ->
      let desc =
        let value = 
# 200 "asllib/Parser.mly"
                        L_Int          
# 4281 "asllib/Parser.ml"
         i in
        
# 264 "asllib/Parser.mly"
                                                                    E_Literal             
# 4286 "asllib/Parser.ml"
         value
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_i_, _startpos_i_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4295 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_016 =
    fun _endpos_b_ _startpos_b_ b ->
      let desc =
        let value = 
# 201 "asllib/Parser.mly"
                        L_Bool         
# 4304 "asllib/Parser.ml"
         b in
        
# 264 "asllib/Parser.mly"
                                                                    E_Literal             
# 4309 "asllib/Parser.ml"
         value
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_b_, _startpos_b_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4318 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_017 =
    fun _endpos_r_ _startpos_r_ r ->
      let desc =
        let value = 
# 202 "asllib/Parser.mly"
                        L_Real         
# 4327 "asllib/Parser.ml"
         r in
        
# 264 "asllib/Parser.mly"
                                                                    E_Literal             
# 4332 "asllib/Parser.ml"
         value
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_r_, _startpos_r_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4341 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_018 =
    fun _endpos_b_ _startpos_b_ b ->
      let desc =
        let value = 
# 203 "asllib/Parser.mly"
                        L_BitVector    
# 4350 "asllib/Parser.ml"
         b in
        
# 264 "asllib/Parser.mly"
                                                                    E_Literal             
# 4355 "asllib/Parser.ml"
         value
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_b_, _startpos_b_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4364 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_019 =
    fun _endpos_s_ _startpos_s_ s ->
      let desc =
        let value = 
# 204 "asllib/Parser.mly"
                        L_String       
# 4373 "asllib/Parser.ml"
         s in
        
# 264 "asllib/Parser.mly"
                                                                    E_Literal             
# 4378 "asllib/Parser.ml"
         value
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_s_, _startpos_s_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4387 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_020 =
    fun _1 _endpos__1_ _startpos__1_ ->
      let desc =
        let _1_1 = _1 in
        
# 265 "asllib/Parser.mly"
                                                                    E_Var                 
# 4397 "asllib/Parser.ml"
         _1_1
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4406 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_021 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 214 "asllib/Parser.mly"
                  ( AND    )
# 4415 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4420 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4429 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_022 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 215 "asllib/Parser.mly"
                  ( BAND   )
# 4438 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4443 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4452 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_023 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 216 "asllib/Parser.mly"
                  ( BOR    )
# 4461 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4466 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4475 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_024 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 217 "asllib/Parser.mly"
                  ( EQ_OP  )
# 4484 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4489 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4498 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_025 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 218 "asllib/Parser.mly"
                  ( DIV    )
# 4507 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4512 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4521 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_026 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 219 "asllib/Parser.mly"
                  ( DIVRM  )
# 4530 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4535 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4544 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_027 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 220 "asllib/Parser.mly"
                  ( EOR    )
# 4553 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4558 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4567 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_028 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 221 "asllib/Parser.mly"
                  ( EQ_OP  )
# 4576 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4581 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4590 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_029 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 222 "asllib/Parser.mly"
                  ( NEQ    )
# 4599 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4604 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4613 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_030 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 223 "asllib/Parser.mly"
                  ( GT     )
# 4622 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4627 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4636 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_031 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 224 "asllib/Parser.mly"
                  ( GEQ    )
# 4645 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4650 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4659 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_032 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 225 "asllib/Parser.mly"
                  ( IMPL   )
# 4668 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4673 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4682 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_033 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 226 "asllib/Parser.mly"
                  ( LT     )
# 4691 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4696 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4705 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_034 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 227 "asllib/Parser.mly"
                  ( LEQ    )
# 4714 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4719 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4728 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_035 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 228 "asllib/Parser.mly"
                  ( PLUS   )
# 4737 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4742 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4751 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_036 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 229 "asllib/Parser.mly"
                  ( MINUS  )
# 4760 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4765 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4774 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_037 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 230 "asllib/Parser.mly"
                  ( MOD    )
# 4783 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4788 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4797 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_038 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 231 "asllib/Parser.mly"
                  ( MUL    )
# 4806 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4811 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4820 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_039 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 232 "asllib/Parser.mly"
                  ( OR     )
# 4829 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4834 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4843 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_040 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 233 "asllib/Parser.mly"
                  ( RDIV   )
# 4852 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4857 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4866 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_041 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 234 "asllib/Parser.mly"
                  ( SHL    )
# 4875 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4880 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4889 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_042 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 235 "asllib/Parser.mly"
                  ( SHR    )
# 4898 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4903 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4912 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_043 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 236 "asllib/Parser.mly"
                  ( POW    )
# 4921 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4926 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4935 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_044 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 237 "asllib/Parser.mly"
                  ( BV_CONCAT )
# 4944 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4949 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4958 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_045 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op =
          let _1 = () in
          
# 211 "asllib/Parser.mly"
                                  ( PLUS )
# 4969 "asllib/Parser.ml"
          
        in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 4975 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 4984 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_046 =
    fun _1 _endpos_e_ _startpos__1_ e ->
      let desc =
        let op = 
# 207 "asllib/Parser.mly"
            ( BNOT )
# 4993 "asllib/Parser.ml"
         in
        
# 267 "asllib/Parser.mly"
                                                                    E_Unop                
# 4998 "asllib/Parser.ml"
         (op, e)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5007 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_047 =
    fun _1 _endpos_e_ _startpos__1_ e ->
      let desc =
        let op = 
# 208 "asllib/Parser.mly"
            ( NEG )
# 5016 "asllib/Parser.ml"
         in
        
# 267 "asllib/Parser.mly"
                                                                    E_Unop                
# 5021 "asllib/Parser.ml"
         (op, e)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5030 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_048 =
    fun _1 _endpos_e_ _startpos__1_ e ->
      let desc =
        let op = 
# 209 "asllib/Parser.mly"
            ( NOT )
# 5039 "asllib/Parser.ml"
         in
        
# 267 "asllib/Parser.mly"
                                                                    E_Unop                
# 5044 "asllib/Parser.ml"
         (op, e)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5053 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_049 =
    fun _1 _3 _endpos_e_else_ _startpos__1_ e1 e2 e_else ->
      let desc = 
# 268 "asllib/Parser.mly"
                                                                    E_Cond                
# 5061 "asllib/Parser.ml"
       (e1, e2, e_else) in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e_else_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5069 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_050 =
    fun _endpos__3_ _startpos_name_ name ->
      let desc =
        let call =
          let args =
            let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 5080 "asllib/Parser.ml"
             in
            
# 205 "<standard.mly>"
    ( x )
# 5085 "asllib/Parser.ml"
            
          in
          
# 525 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 5091 "asllib/Parser.ml"
          
        in
        
# 269 "asllib/Parser.mly"
                                                                    e_call                
# 5097 "asllib/Parser.ml"
         call
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5106 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_051 =
    fun _1_inlined1 _endpos__3_ _startpos_name_ name ->
      let desc =
        let call =
          let args =
            let x = _1_inlined1 in
            
# 205 "<standard.mly>"
    ( x )
# 5118 "asllib/Parser.ml"
            
          in
          
# 525 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 5124 "asllib/Parser.ml"
          
        in
        
# 269 "asllib/Parser.mly"
                                                                    e_call                
# 5130 "asllib/Parser.ml"
         call
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5139 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_052 =
    fun _1_inlined1 _endpos_args_ _startpos_name_ args name ->
      let desc =
        let call =
          let params =
            let x = _1_inlined1 in
            
# 205 "<standard.mly>"
    ( x )
# 5151 "asllib/Parser.ml"
            
          in
          
# 527 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 5157 "asllib/Parser.ml"
          
        in
        
# 269 "asllib/Parser.mly"
                                                                    e_call                
# 5163 "asllib/Parser.ml"
         call
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5172 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_053 =
    fun _1_inlined1 _endpos__3_ _startpos_e_ e ->
      let desc =
        let slices =
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 5183 "asllib/Parser.ml"
          
        in
        
# 270 "asllib/Parser.mly"
                                                                    E_Slice               
# 5189 "asllib/Parser.ml"
         (e, slices)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_e_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5198 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_054 =
    fun _2 _4 _endpos__4_ _startpos_e1_ e1 e2 ->
      let desc = 
# 271 "asllib/Parser.mly"
                                                                    E_GetArray            
# 5206 "asllib/Parser.ml"
       (e1, e2) in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__4_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5214 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_055 =
    fun _2 _endpos_x_ _startpos_e_ e x ->
      let desc = 
# 272 "asllib/Parser.mly"
                                                                    E_GetField            
# 5222 "asllib/Parser.ml"
       (e, x) in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_x_, _startpos_e_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5230 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_056 =
    fun _1_inlined1 _2 _endpos__3_ _startpos_e_ e ->
      let desc =
        let fs =
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 5241 "asllib/Parser.ml"
          
        in
        
# 273 "asllib/Parser.mly"
                                                                    E_GetFields           
# 5247 "asllib/Parser.ml"
         (e, fs)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_e_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5256 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_057 =
    fun _2 _endpos_ty_ _startpos_sub_expr_ sub_expr ty ->
      let desc = 
# 275 "asllib/Parser.mly"
                                                                    E_ATC                 
# 5264 "asllib/Parser.ml"
       (sub_expr, ty) in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_ty_, _startpos_sub_expr_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5272 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_058 =
    fun _1_inlined1 _2 _endpos__3_ _startpos__1_ _startpos_sub_expr_ sub_expr ->
      let desc =
        let implicit_t_int =
          let desc =
            let constraint_kind =
              let _1 =
                let x = _1_inlined1 in
                
# 205 "<standard.mly>"
    ( x )
# 5286 "asllib/Parser.ml"
                
              in
              let _1_1 = _1 in
              
# 301 "asllib/Parser.mly"
                                        WellConstrained  
# 5293 "asllib/Parser.ml"
               _1_1
            in
            
# 382 "asllib/Parser.mly"
                                                       T_Int 
# 5299 "asllib/Parser.ml"
             constraint_kind
          in
          let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
          let _endpos = _endpos_desc_ in
          let _symbolstartpos = _startpos_desc_ in
          
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5308 "asllib/Parser.ml"
          
        in
        
# 276 "asllib/Parser.mly"
                                                                    E_ATC                 
# 5314 "asllib/Parser.ml"
         (sub_expr, implicit_t_int)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_sub_expr_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5323 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_059 =
    fun _2 _endpos__3_ _startpos__1_ _startpos_sub_expr_ sub_expr x ->
      let desc =
        let implicit_t_int =
          let desc =
            let constraint_kind =
              let _1 = 
# 205 "<standard.mly>"
    ( x )
# 5335 "asllib/Parser.ml"
               in
              
# 302 "asllib/Parser.mly"
                   ( PendingConstrained )
# 5340 "asllib/Parser.ml"
              
            in
            
# 382 "asllib/Parser.mly"
                                                       T_Int 
# 5346 "asllib/Parser.ml"
             constraint_kind
          in
          let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
          let _endpos = _endpos_desc_ in
          let _symbolstartpos = _startpos_desc_ in
          
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5355 "asllib/Parser.ml"
          
        in
        
# 276 "asllib/Parser.mly"
                                                                    E_ATC                 
# 5361 "asllib/Parser.ml"
         (sub_expr, implicit_t_int)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_sub_expr_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5370 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_060 =
    fun _1_inlined1 _2 _endpos__1_inlined1_ _endpos__3_ _startpos__1_inlined1_ _startpos_sub_expr_ sub_expr ->
      let desc =
        let pattern_set =
          let x =
            let (_endpos__1_, _startpos__1_, _1) = (_endpos__1_inlined1_, _startpos__1_inlined1_, _1_inlined1) in
            let desc =
              let _1_1 = _1 in
              
# 311 "asllib/Parser.mly"
                                                   Pattern_Any  
# 5384 "asllib/Parser.ml"
               _1_1
            in
            let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
            let _endpos = _endpos_desc_ in
            let _symbolstartpos = _startpos_desc_ in
            
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5393 "asllib/Parser.ml"
            
          in
          
# 205 "<standard.mly>"
    ( x )
# 5399 "asllib/Parser.ml"
          
        in
        
# 278 "asllib/Parser.mly"
                                                                    E_Pattern             
# 5405 "asllib/Parser.ml"
         (sub_expr, pattern_set)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_sub_expr_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5414 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_061 =
    fun _1 _1_inlined2 _2 _endpos__1_inlined2_ _endpos__3_ _startpos__1_ _startpos__1_inlined2_ _startpos_sub_expr_ sub_expr ->
      let desc =
        let pattern_set =
          let desc =
            let _2 =
              let (_endpos__1_inlined1_, _startpos__1_inlined1_, _1_inlined1) = (_endpos__1_inlined2_, _startpos__1_inlined2_, _1_inlined2) in
              let x =
                let (_endpos__1_, _startpos__1_, _1) = (_endpos__1_inlined1_, _startpos__1_inlined1_, _1_inlined1) in
                let desc =
                  let _1_1 = _1 in
                  
# 311 "asllib/Parser.mly"
                                                   Pattern_Any  
# 5431 "asllib/Parser.ml"
                   _1_1
                in
                let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
                let _endpos = _endpos_desc_ in
                let _symbolstartpos = _startpos_desc_ in
                
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5440 "asllib/Parser.ml"
                
              in
              
# 205 "<standard.mly>"
    ( x )
# 5446 "asllib/Parser.ml"
              
            in
            let _2_1 = _2 in
            
# 328 "asllib/Parser.mly"
                                      Pattern_Not  
# 5453 "asllib/Parser.ml"
             _2_1
          in
          let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
          let _endpos = _endpos_desc_ in
          let _symbolstartpos = _startpos_desc_ in
          
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5462 "asllib/Parser.ml"
          
        in
        
# 278 "asllib/Parser.mly"
                                                                    E_Pattern             
# 5468 "asllib/Parser.ml"
         (sub_expr, pattern_set)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_sub_expr_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5477 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_062 =
    fun _1 _2 _endpos__1_ _startpos__1_ _startpos_sub_expr_ sub_expr ->
      let desc =
        let pattern_mask =
          let desc =
            let _1_1 = _1 in
            
# 310 "asllib/Parser.mly"
                                            Pattern_Mask  
# 5489 "asllib/Parser.ml"
             _1_1
          in
          let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
          let _endpos = _endpos_desc_ in
          let _symbolstartpos = _startpos_desc_ in
          
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5498 "asllib/Parser.ml"
          
        in
        
# 279 "asllib/Parser.mly"
                                                                    E_Pattern             
# 5504 "asllib/Parser.ml"
         (sub_expr, pattern_mask)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos_sub_expr_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5513 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_063 =
    fun _1 _2 _endpos__1_ _startpos__1_ _startpos_e_ e ->
      let desc =
        let p =
          let desc =
            let _1_1 = _1 in
            
# 310 "asllib/Parser.mly"
                                            Pattern_Mask  
# 5525 "asllib/Parser.ml"
             _1_1
          in
          let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
          let _endpos = _endpos_desc_ in
          let _symbolstartpos = _startpos_desc_ in
          
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5534 "asllib/Parser.ml"
          
        in
        
# 280 "asllib/Parser.mly"
                                                                  ( E_Pattern (e, Pattern_Not (p) |> add_pos_from p) )
# 5540 "asllib/Parser.ml"
        
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos_e_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5549 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_064 =
    fun _1 _1_inlined1 _endpos_ty_ _startpos__1_ ty ->
      let desc =
        let _2 = _1_inlined1 in
        
# 282 "asllib/Parser.mly"
                                                                      E_Arbitrary         
# 5559 "asllib/Parser.ml"
         ty
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_ty_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5568 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_065 =
    fun _1 _1_inlined1 _endpos_ty_ _startpos__1_ ty ->
      let desc =
        let _2 = _1_inlined1 in
        
# 282 "asllib/Parser.mly"
                                                                      E_Arbitrary         
# 5578 "asllib/Parser.ml"
         ty
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_ty_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5587 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_066 =
    fun _endpos__3_ _startpos__1_ x ->
      let desc =
        let e = 
# 205 "<standard.mly>"
    ( x )
# 5596 "asllib/Parser.ml"
         in
        
# 283 "asllib/Parser.mly"
                                                                  ( E_Tuple [ e ]        )
# 5601 "asllib/Parser.ml"
        
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5610 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_067 =
    fun _endpos__3_ _endpos_desc_ _startpos_desc_ desc ->
      let desc =
        let fields =
          let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 5620 "asllib/Parser.ml"
           in
          
# 205 "<standard.mly>"
    ( x )
# 5625 "asllib/Parser.ml"
          
        in
        let t =
          let _endpos = _endpos_desc_ in
          let _symbolstartpos = _startpos_desc_ in
          
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5634 "asllib/Parser.ml"
          
        in
        
# 286 "asllib/Parser.mly"
        ( E_Record (add_pos_from t (T_Named t.desc), fields) )
# 5640 "asllib/Parser.ml"
        
      in
      let _endpos_desc_ = _endpos__3_ in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5649 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_068 =
    fun _1_inlined1 _endpos__3_ _endpos_desc_ _startpos_desc_ desc ->
      let desc =
        let fields =
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 5660 "asllib/Parser.ml"
          
        in
        let t =
          let _endpos = _endpos_desc_ in
          let _symbolstartpos = _startpos_desc_ in
          
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5669 "asllib/Parser.ml"
          
        in
        
# 286 "asllib/Parser.mly"
        ( E_Record (add_pos_from t (T_Named t.desc), fields) )
# 5675 "asllib/Parser.ml"
        
      in
      let _endpos_desc_ = _endpos__3_ in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5684 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_069 =
    fun _2 _endpos__3_ _startpos__1_ li x ->
      let desc =
        let _1 =
          let x = 
# 177 "asllib/Parser.mly"
                                                    ( x :: li )
# 5694 "asllib/Parser.ml"
           in
          
# 205 "<standard.mly>"
    ( x )
# 5699 "asllib/Parser.ml"
          
        in
        let _1_1 = _1 in
        
# 257 "asllib/Parser.mly"
                                                                    E_Tuple               
# 5706 "asllib/Parser.ml"
         _1_1
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5715 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_070 =
    fun _endpos_i_ _startpos_i_ i ->
      let desc =
        let value = 
# 200 "asllib/Parser.mly"
                        L_Int          
# 5724 "asllib/Parser.ml"
         i in
        
# 264 "asllib/Parser.mly"
                                                                    E_Literal             
# 5729 "asllib/Parser.ml"
         value
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_i_, _startpos_i_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5738 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_071 =
    fun _endpos_b_ _startpos_b_ b ->
      let desc =
        let value = 
# 201 "asllib/Parser.mly"
                        L_Bool         
# 5747 "asllib/Parser.ml"
         b in
        
# 264 "asllib/Parser.mly"
                                                                    E_Literal             
# 5752 "asllib/Parser.ml"
         value
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_b_, _startpos_b_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5761 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_072 =
    fun _endpos_r_ _startpos_r_ r ->
      let desc =
        let value = 
# 202 "asllib/Parser.mly"
                        L_Real         
# 5770 "asllib/Parser.ml"
         r in
        
# 264 "asllib/Parser.mly"
                                                                    E_Literal             
# 5775 "asllib/Parser.ml"
         value
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_r_, _startpos_r_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5784 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_073 =
    fun _endpos_b_ _startpos_b_ b ->
      let desc =
        let value = 
# 203 "asllib/Parser.mly"
                        L_BitVector    
# 5793 "asllib/Parser.ml"
         b in
        
# 264 "asllib/Parser.mly"
                                                                    E_Literal             
# 5798 "asllib/Parser.ml"
         value
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_b_, _startpos_b_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5807 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_074 =
    fun _endpos_s_ _startpos_s_ s ->
      let desc =
        let value = 
# 204 "asllib/Parser.mly"
                        L_String       
# 5816 "asllib/Parser.ml"
         s in
        
# 264 "asllib/Parser.mly"
                                                                    E_Literal             
# 5821 "asllib/Parser.ml"
         value
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_s_, _startpos_s_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5830 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_075 =
    fun _1 _endpos__1_ _startpos__1_ ->
      let desc =
        let _1_1 = _1 in
        
# 265 "asllib/Parser.mly"
                                                                    E_Var                 
# 5840 "asllib/Parser.ml"
         _1_1
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5849 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_076 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 214 "asllib/Parser.mly"
                  ( AND    )
# 5858 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 5863 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5872 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_077 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 215 "asllib/Parser.mly"
                  ( BAND   )
# 5881 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 5886 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5895 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_078 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 216 "asllib/Parser.mly"
                  ( BOR    )
# 5904 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 5909 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5918 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_079 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 217 "asllib/Parser.mly"
                  ( EQ_OP  )
# 5927 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 5932 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5941 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_080 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 218 "asllib/Parser.mly"
                  ( DIV    )
# 5950 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 5955 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5964 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_081 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 219 "asllib/Parser.mly"
                  ( DIVRM  )
# 5973 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 5978 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 5987 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_082 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 220 "asllib/Parser.mly"
                  ( EOR    )
# 5996 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 6001 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6010 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_083 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 221 "asllib/Parser.mly"
                  ( EQ_OP  )
# 6019 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 6024 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6033 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_084 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 222 "asllib/Parser.mly"
                  ( NEQ    )
# 6042 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 6047 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6056 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_085 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 223 "asllib/Parser.mly"
                  ( GT     )
# 6065 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 6070 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6079 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_086 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 224 "asllib/Parser.mly"
                  ( GEQ    )
# 6088 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 6093 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6102 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_087 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 225 "asllib/Parser.mly"
                  ( IMPL   )
# 6111 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 6116 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6125 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_088 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 226 "asllib/Parser.mly"
                  ( LT     )
# 6134 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 6139 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6148 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_089 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 227 "asllib/Parser.mly"
                  ( LEQ    )
# 6157 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 6162 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6171 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_090 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 228 "asllib/Parser.mly"
                  ( PLUS   )
# 6180 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 6185 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6194 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_091 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 229 "asllib/Parser.mly"
                  ( MINUS  )
# 6203 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 6208 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6217 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_092 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 230 "asllib/Parser.mly"
                  ( MOD    )
# 6226 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 6231 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6240 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_093 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 231 "asllib/Parser.mly"
                  ( MUL    )
# 6249 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 6254 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6263 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_094 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 232 "asllib/Parser.mly"
                  ( OR     )
# 6272 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 6277 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6286 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_095 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 233 "asllib/Parser.mly"
                  ( RDIV   )
# 6295 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 6300 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6309 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_096 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 234 "asllib/Parser.mly"
                  ( SHL    )
# 6318 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 6323 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6332 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_097 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 235 "asllib/Parser.mly"
                  ( SHR    )
# 6341 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 6346 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6355 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_098 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 236 "asllib/Parser.mly"
                  ( POW    )
# 6364 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 6369 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6378 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_099 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op = 
# 237 "asllib/Parser.mly"
                  ( BV_CONCAT )
# 6387 "asllib/Parser.ml"
         in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 6392 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6401 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_100 =
    fun _1 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc =
        let op =
          let _1 = () in
          
# 211 "asllib/Parser.mly"
                                  ( PLUS )
# 6412 "asllib/Parser.ml"
          
        in
        
# 266 "asllib/Parser.mly"
                                                                    e_binop               
# 6418 "asllib/Parser.ml"
         (e1, op, e2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6427 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_101 =
    fun _1 _endpos_e_ _startpos__1_ e ->
      let desc =
        let op = 
# 207 "asllib/Parser.mly"
            ( BNOT )
# 6436 "asllib/Parser.ml"
         in
        
# 267 "asllib/Parser.mly"
                                                                    E_Unop                
# 6441 "asllib/Parser.ml"
         (op, e)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6450 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_102 =
    fun _1 _endpos_e_ _startpos__1_ e ->
      let desc =
        let op = 
# 208 "asllib/Parser.mly"
            ( NEG )
# 6459 "asllib/Parser.ml"
         in
        
# 267 "asllib/Parser.mly"
                                                                    E_Unop                
# 6464 "asllib/Parser.ml"
         (op, e)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6473 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_103 =
    fun _1 _endpos_e_ _startpos__1_ e ->
      let desc =
        let op = 
# 209 "asllib/Parser.mly"
            ( NOT )
# 6482 "asllib/Parser.ml"
         in
        
# 267 "asllib/Parser.mly"
                                                                    E_Unop                
# 6487 "asllib/Parser.ml"
         (op, e)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6496 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_104 =
    fun _1 _3 _endpos_e_else_ _startpos__1_ e1 e2 e_else ->
      let desc = 
# 268 "asllib/Parser.mly"
                                                                    E_Cond                
# 6504 "asllib/Parser.ml"
       (e1, e2, e_else) in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e_else_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6512 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_105 =
    fun _endpos__3_ _startpos_name_ name ->
      let desc =
        let call =
          let args =
            let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 6523 "asllib/Parser.ml"
             in
            
# 205 "<standard.mly>"
    ( x )
# 6528 "asllib/Parser.ml"
            
          in
          
# 525 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 6534 "asllib/Parser.ml"
          
        in
        
# 269 "asllib/Parser.mly"
                                                                    e_call                
# 6540 "asllib/Parser.ml"
         call
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6549 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_106 =
    fun _1_inlined1 _endpos__3_ _startpos_name_ name ->
      let desc =
        let call =
          let args =
            let x = _1_inlined1 in
            
# 205 "<standard.mly>"
    ( x )
# 6561 "asllib/Parser.ml"
            
          in
          
# 525 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 6567 "asllib/Parser.ml"
          
        in
        
# 269 "asllib/Parser.mly"
                                                                    e_call                
# 6573 "asllib/Parser.ml"
         call
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6582 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_107 =
    fun _1_inlined1 _endpos_args_ _startpos_name_ args name ->
      let desc =
        let call =
          let params =
            let x = _1_inlined1 in
            
# 205 "<standard.mly>"
    ( x )
# 6594 "asllib/Parser.ml"
            
          in
          
# 527 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 6600 "asllib/Parser.ml"
          
        in
        
# 269 "asllib/Parser.mly"
                                                                    e_call                
# 6606 "asllib/Parser.ml"
         call
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6615 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_108 =
    fun _1_inlined1 _endpos__3_ _startpos_e_ e ->
      let desc =
        let slices =
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 6626 "asllib/Parser.ml"
          
        in
        
# 270 "asllib/Parser.mly"
                                                                    E_Slice               
# 6632 "asllib/Parser.ml"
         (e, slices)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_e_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6641 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_109 =
    fun _2 _4 _endpos__4_ _startpos_e1_ e1 e2 ->
      let desc = 
# 271 "asllib/Parser.mly"
                                                                    E_GetArray            
# 6649 "asllib/Parser.ml"
       (e1, e2) in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__4_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6657 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_110 =
    fun _2 _endpos_x_ _startpos_e_ e x ->
      let desc = 
# 272 "asllib/Parser.mly"
                                                                    E_GetField            
# 6665 "asllib/Parser.ml"
       (e, x) in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_x_, _startpos_e_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6673 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_111 =
    fun _1_inlined1 _2 _endpos__3_ _startpos_e_ e ->
      let desc =
        let fs =
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 6684 "asllib/Parser.ml"
          
        in
        
# 273 "asllib/Parser.mly"
                                                                    E_GetFields           
# 6690 "asllib/Parser.ml"
         (e, fs)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_e_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6699 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_112 =
    fun _2 _endpos_ty_ _startpos_sub_expr_ sub_expr ty ->
      let desc = 
# 275 "asllib/Parser.mly"
                                                                    E_ATC                 
# 6707 "asllib/Parser.ml"
       (sub_expr, ty) in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_ty_, _startpos_sub_expr_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6715 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_113 =
    fun _1_inlined1 _2 _endpos__3_ _startpos__1_ _startpos_sub_expr_ sub_expr ->
      let desc =
        let implicit_t_int =
          let desc =
            let constraint_kind =
              let _1 =
                let x = _1_inlined1 in
                
# 205 "<standard.mly>"
    ( x )
# 6729 "asllib/Parser.ml"
                
              in
              let _1_1 = _1 in
              
# 301 "asllib/Parser.mly"
                                        WellConstrained  
# 6736 "asllib/Parser.ml"
               _1_1
            in
            
# 382 "asllib/Parser.mly"
                                                       T_Int 
# 6742 "asllib/Parser.ml"
             constraint_kind
          in
          let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
          let _endpos = _endpos_desc_ in
          let _symbolstartpos = _startpos_desc_ in
          
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6751 "asllib/Parser.ml"
          
        in
        
# 276 "asllib/Parser.mly"
                                                                    E_ATC                 
# 6757 "asllib/Parser.ml"
         (sub_expr, implicit_t_int)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_sub_expr_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6766 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_114 =
    fun _2 _endpos__3_ _startpos__1_ _startpos_sub_expr_ sub_expr x ->
      let desc =
        let implicit_t_int =
          let desc =
            let constraint_kind =
              let _1 = 
# 205 "<standard.mly>"
    ( x )
# 6778 "asllib/Parser.ml"
               in
              
# 302 "asllib/Parser.mly"
                   ( PendingConstrained )
# 6783 "asllib/Parser.ml"
              
            in
            
# 382 "asllib/Parser.mly"
                                                       T_Int 
# 6789 "asllib/Parser.ml"
             constraint_kind
          in
          let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
          let _endpos = _endpos_desc_ in
          let _symbolstartpos = _startpos_desc_ in
          
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6798 "asllib/Parser.ml"
          
        in
        
# 276 "asllib/Parser.mly"
                                                                    E_ATC                 
# 6804 "asllib/Parser.ml"
         (sub_expr, implicit_t_int)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_sub_expr_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6813 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_115 =
    fun _1_inlined1 _2 _endpos__1_inlined1_ _endpos__3_ _startpos__1_inlined1_ _startpos_sub_expr_ sub_expr ->
      let desc =
        let pattern_set =
          let x =
            let (_endpos__1_, _startpos__1_, _1) = (_endpos__1_inlined1_, _startpos__1_inlined1_, _1_inlined1) in
            let desc =
              let _1_1 = _1 in
              
# 311 "asllib/Parser.mly"
                                                   Pattern_Any  
# 6827 "asllib/Parser.ml"
               _1_1
            in
            let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
            let _endpos = _endpos_desc_ in
            let _symbolstartpos = _startpos_desc_ in
            
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6836 "asllib/Parser.ml"
            
          in
          
# 205 "<standard.mly>"
    ( x )
# 6842 "asllib/Parser.ml"
          
        in
        
# 278 "asllib/Parser.mly"
                                                                    E_Pattern             
# 6848 "asllib/Parser.ml"
         (sub_expr, pattern_set)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_sub_expr_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6857 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_116 =
    fun _1 _1_inlined2 _2 _endpos__1_inlined2_ _endpos__3_ _startpos__1_ _startpos__1_inlined2_ _startpos_sub_expr_ sub_expr ->
      let desc =
        let pattern_set =
          let desc =
            let _2 =
              let (_endpos__1_inlined1_, _startpos__1_inlined1_, _1_inlined1) = (_endpos__1_inlined2_, _startpos__1_inlined2_, _1_inlined2) in
              let x =
                let (_endpos__1_, _startpos__1_, _1) = (_endpos__1_inlined1_, _startpos__1_inlined1_, _1_inlined1) in
                let desc =
                  let _1_1 = _1 in
                  
# 311 "asllib/Parser.mly"
                                                   Pattern_Any  
# 6874 "asllib/Parser.ml"
                   _1_1
                in
                let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
                let _endpos = _endpos_desc_ in
                let _symbolstartpos = _startpos_desc_ in
                
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6883 "asllib/Parser.ml"
                
              in
              
# 205 "<standard.mly>"
    ( x )
# 6889 "asllib/Parser.ml"
              
            in
            let _2_1 = _2 in
            
# 328 "asllib/Parser.mly"
                                      Pattern_Not  
# 6896 "asllib/Parser.ml"
             _2_1
          in
          let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
          let _endpos = _endpos_desc_ in
          let _symbolstartpos = _startpos_desc_ in
          
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6905 "asllib/Parser.ml"
          
        in
        
# 278 "asllib/Parser.mly"
                                                                    E_Pattern             
# 6911 "asllib/Parser.ml"
         (sub_expr, pattern_set)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_sub_expr_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6920 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_117 =
    fun _1 _2 _endpos__1_ _startpos__1_ _startpos_sub_expr_ sub_expr ->
      let desc =
        let pattern_mask =
          let desc =
            let _1_1 = _1 in
            
# 310 "asllib/Parser.mly"
                                            Pattern_Mask  
# 6932 "asllib/Parser.ml"
             _1_1
          in
          let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
          let _endpos = _endpos_desc_ in
          let _symbolstartpos = _startpos_desc_ in
          
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6941 "asllib/Parser.ml"
          
        in
        
# 279 "asllib/Parser.mly"
                                                                    E_Pattern             
# 6947 "asllib/Parser.ml"
         (sub_expr, pattern_mask)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos_sub_expr_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6956 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_118 =
    fun _1 _2 _endpos__1_ _startpos__1_ _startpos_e_ e ->
      let desc =
        let p =
          let desc =
            let _1_1 = _1 in
            
# 310 "asllib/Parser.mly"
                                            Pattern_Mask  
# 6968 "asllib/Parser.ml"
             _1_1
          in
          let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
          let _endpos = _endpos_desc_ in
          let _symbolstartpos = _startpos_desc_ in
          
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6977 "asllib/Parser.ml"
          
        in
        
# 280 "asllib/Parser.mly"
                                                                  ( E_Pattern (e, Pattern_Not (p) |> add_pos_from p) )
# 6983 "asllib/Parser.ml"
        
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos_e_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 6992 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_119 =
    fun _1 _1_inlined1 _endpos_ty_ _startpos__1_ ty ->
      let desc =
        let _2 = _1_inlined1 in
        
# 282 "asllib/Parser.mly"
                                                                      E_Arbitrary         
# 7002 "asllib/Parser.ml"
         ty
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_ty_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7011 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_120 =
    fun _1 _1_inlined1 _endpos_ty_ _startpos__1_ ty ->
      let desc =
        let _2 = _1_inlined1 in
        
# 282 "asllib/Parser.mly"
                                                                      E_Arbitrary         
# 7021 "asllib/Parser.ml"
         ty
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_ty_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7030 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_121 =
    fun _endpos__3_ _startpos__1_ x ->
      let desc =
        let e = 
# 205 "<standard.mly>"
    ( x )
# 7039 "asllib/Parser.ml"
         in
        
# 283 "asllib/Parser.mly"
                                                                  ( E_Tuple [ e ]        )
# 7044 "asllib/Parser.ml"
        
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7053 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_122 =
    fun _endpos__3_ _endpos_desc_ _startpos_desc_ desc ->
      let desc =
        let fields =
          let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 7063 "asllib/Parser.ml"
           in
          
# 205 "<standard.mly>"
    ( x )
# 7068 "asllib/Parser.ml"
          
        in
        let t =
          let _endpos = _endpos_desc_ in
          let _symbolstartpos = _startpos_desc_ in
          
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7077 "asllib/Parser.ml"
          
        in
        
# 286 "asllib/Parser.mly"
        ( E_Record (add_pos_from t (T_Named t.desc), fields) )
# 7083 "asllib/Parser.ml"
        
      in
      let _endpos_desc_ = _endpos__3_ in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7092 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_123 =
    fun _1_inlined1 _endpos__3_ _endpos_desc_ _startpos_desc_ desc ->
      let desc =
        let fields =
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 7103 "asllib/Parser.ml"
          
        in
        let t =
          let _endpos = _endpos_desc_ in
          let _symbolstartpos = _startpos_desc_ in
          
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7112 "asllib/Parser.ml"
          
        in
        
# 286 "asllib/Parser.mly"
        ( E_Record (add_pos_from t (T_Named t.desc), fields) )
# 7118 "asllib/Parser.ml"
        
      in
      let _endpos_desc_ = _endpos__3_ in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7127 "asllib/Parser.ml"
       : (AST.expr))
  
  let _menhir_action_124 =
    fun _endpos_desc_ _startpos_desc_ desc ->
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7137 "asllib/Parser.ml"
       : (AST.lexpr))
  
  let _menhir_action_125 =
    fun _1 _endpos__1_ _startpos__1_ ->
      let desc = 
# 394 "asllib/Parser.mly"
                      ( LE_Discard )
# 7145 "asllib/Parser.ml"
       in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7153 "asllib/Parser.ml"
       : (AST.lexpr))
  
  let _menhir_action_126 =
    fun _1_inlined1 _2 _endpos__3_ _startpos__1_ x ->
      let desc =
        let _1 =
          let x =
            let _1 = _1_inlined1 in
            let li = _1 in
            
# 164 "asllib/Parser.mly"
                                           ( x :: li )
# 7166 "asllib/Parser.ml"
            
          in
          
# 205 "<standard.mly>"
    ( x )
# 7172 "asllib/Parser.ml"
          
        in
        let _1_1 = _1 in
        
# 396 "asllib/Parser.mly"
                               LE_Destructuring 
# 7179 "asllib/Parser.ml"
         _1_1
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7188 "asllib/Parser.ml"
       : (AST.lexpr))
  
  let _menhir_action_127 =
    fun _1 ->
      let _1_1 = _1 in
      (
# 400 "asllib/Parser.mly"
                    LE_Var 
# 7197 "asllib/Parser.ml"
       _1_1 : (AST.lexpr_desc))
  
  let _menhir_action_128 =
    fun _1_inlined1 _endpos_desc_ _startpos_desc_ desc ->
      let slices =
        let x = _1_inlined1 in
        
# 205 "<standard.mly>"
    ( x )
# 7207 "asllib/Parser.ml"
        
      in
      let le =
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7216 "asllib/Parser.ml"
        
      in
      (
# 401 "asllib/Parser.mly"
                              LE_Slice 
# 7222 "asllib/Parser.ml"
       (le, slices) : (AST.lexpr_desc))
  
  let _menhir_action_129 =
    fun _2 _4 _endpos_desc_ _startpos_desc_ desc expr ->
      let le =
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7233 "asllib/Parser.ml"
        
      in
      (
# 402 "asllib/Parser.mly"
                                                  LE_SetArray 
# 7239 "asllib/Parser.ml"
       (le, expr) : (AST.lexpr_desc))
  
  let _menhir_action_130 =
    fun _2 _endpos_desc_ _startpos_desc_ desc field ->
      let le =
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7250 "asllib/Parser.ml"
        
      in
      (
# 403 "asllib/Parser.mly"
                                           LE_SetField 
# 7256 "asllib/Parser.ml"
       (le, field) : (AST.lexpr_desc))
  
  let _menhir_action_131 =
    fun _2 _endpos_desc_ _startpos_desc_ desc ->
      let li =
        let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 7265 "asllib/Parser.ml"
         in
        
# 205 "<standard.mly>"
    ( x )
# 7270 "asllib/Parser.ml"
        
      in
      let le =
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7279 "asllib/Parser.ml"
        
      in
      (
# 404 "asllib/Parser.mly"
                                                         ( LE_SetFields (le, li, []) )
# 7285 "asllib/Parser.ml"
       : (AST.lexpr_desc))
  
  let _menhir_action_132 =
    fun _1_inlined1 _2 _endpos_desc_ _startpos_desc_ desc ->
      let li =
        let x = _1_inlined1 in
        
# 205 "<standard.mly>"
    ( x )
# 7295 "asllib/Parser.ml"
        
      in
      let le =
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7304 "asllib/Parser.ml"
        
      in
      (
# 404 "asllib/Parser.mly"
                                                         ( LE_SetFields (le, li, []) )
# 7310 "asllib/Parser.ml"
       : (AST.lexpr_desc))
  
  let _menhir_action_133 =
    fun () ->
      (
# 216 "<standard.mly>"
    ( [] )
# 7318 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_134 =
    fun _1 _1_inlined1 _1_inlined3 _1_inlined5 _2 _endpos__3_inlined1_ _startpos__1_ name xs ->
      let x =
        let desc =
          let body =
            let _1_inlined1 = _1_inlined5 in
            let x =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 7333 "asllib/Parser.ml"
               _1_1
            in
            
# 205 "<standard.mly>"
    ( x )
# 7339 "asllib/Parser.ml"
            
          in
          let recurse_limit = 
# 123 "<standard.mly>"
    ( None )
# 7345 "asllib/Parser.ml"
           in
          let return_type =
            let _1 = _1_inlined3 in
            _2
          in
          let func_args =
            let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 7355 "asllib/Parser.ml"
             in
            
# 205 "<standard.mly>"
    ( x )
# 7360 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 545 "asllib/Parser.mly"
        (
          D_Func {
            name;
            parameters = params_opt;
            args = func_args;
            body = SB_ASL body;
            return_type = Some return_type;
            subprogram_type = ST_Function;
            recurse_limit;
            builtin = false;
          }
        )
# 7378 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7387 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 7393 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_135 =
    fun _1 _1_inlined1 _1_inlined3 _2 _endpos__1_inlined4_ _endpos__3_inlined1_ _startpos__1_ name xs ->
      let x =
        let desc =
          let body =
            let _endpos__1_ = _endpos__1_inlined4_ in
            let x =
              let _endpos__0_ = _endpos__1_ in
              let desc = 
# 534 "asllib/Parser.mly"
                                                   ( S_Pass )
# 7407 "asllib/Parser.ml"
               in
              let _endpos_desc_ = _endpos__0_ in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _endpos in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7415 "asllib/Parser.ml"
              
            in
            
# 205 "<standard.mly>"
    ( x )
# 7421 "asllib/Parser.ml"
            
          in
          let recurse_limit = 
# 123 "<standard.mly>"
    ( None )
# 7427 "asllib/Parser.ml"
           in
          let return_type =
            let _1 = _1_inlined3 in
            _2
          in
          let func_args =
            let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 7437 "asllib/Parser.ml"
             in
            
# 205 "<standard.mly>"
    ( x )
# 7442 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 545 "asllib/Parser.mly"
        (
          D_Func {
            name;
            parameters = params_opt;
            args = func_args;
            body = SB_ASL body;
            return_type = Some return_type;
            subprogram_type = ST_Function;
            recurse_limit;
            builtin = false;
          }
        )
# 7460 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7469 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 7475 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_136 =
    fun _1 _1_inlined1 _1_inlined3 _1_inlined4 _1_inlined6 _2 _2_inlined1 _endpos__3_inlined1_ _startpos__1_ name xs ->
      let x =
        let desc =
          let body =
            let _1_inlined1 = _1_inlined6 in
            let x =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 7490 "asllib/Parser.ml"
               _1_1
            in
            
# 205 "<standard.mly>"
    ( x )
# 7496 "asllib/Parser.ml"
            
          in
          let recurse_limit =
            let (_2, _1) = (_2_inlined1, _1_inlined4) in
            let x = _2 in
            
# 126 "<standard.mly>"
    ( Some x )
# 7505 "asllib/Parser.ml"
            
          in
          let return_type =
            let _1 = _1_inlined3 in
            _2
          in
          let func_args =
            let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 7516 "asllib/Parser.ml"
             in
            
# 205 "<standard.mly>"
    ( x )
# 7521 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 545 "asllib/Parser.mly"
        (
          D_Func {
            name;
            parameters = params_opt;
            args = func_args;
            body = SB_ASL body;
            return_type = Some return_type;
            subprogram_type = ST_Function;
            recurse_limit;
            builtin = false;
          }
        )
# 7539 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7548 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 7554 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_137 =
    fun _1 _1_inlined1 _1_inlined3 _1_inlined4 _2 _2_inlined1 _endpos__1_inlined5_ _endpos__3_inlined1_ _startpos__1_ name xs ->
      let x =
        let desc =
          let body =
            let _endpos__1_ = _endpos__1_inlined5_ in
            let x =
              let _endpos__0_ = _endpos__1_ in
              let desc = 
# 534 "asllib/Parser.mly"
                                                   ( S_Pass )
# 7568 "asllib/Parser.ml"
               in
              let _endpos_desc_ = _endpos__0_ in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _endpos in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7576 "asllib/Parser.ml"
              
            in
            
# 205 "<standard.mly>"
    ( x )
# 7582 "asllib/Parser.ml"
            
          in
          let recurse_limit =
            let (_2, _1) = (_2_inlined1, _1_inlined4) in
            let x = _2 in
            
# 126 "<standard.mly>"
    ( Some x )
# 7591 "asllib/Parser.ml"
            
          in
          let return_type =
            let _1 = _1_inlined3 in
            _2
          in
          let func_args =
            let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 7602 "asllib/Parser.ml"
             in
            
# 205 "<standard.mly>"
    ( x )
# 7607 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 545 "asllib/Parser.mly"
        (
          D_Func {
            name;
            parameters = params_opt;
            args = func_args;
            body = SB_ASL body;
            return_type = Some return_type;
            subprogram_type = ST_Function;
            recurse_limit;
            builtin = false;
          }
        )
# 7625 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7634 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 7640 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_138 =
    fun _1 _1_inlined1 _1_inlined3 _1_inlined4 _1_inlined6 _2 _endpos__3_inlined1_ _startpos__1_ name xs ->
      let x =
        let desc =
          let body =
            let _1_inlined1 = _1_inlined6 in
            let x =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 7655 "asllib/Parser.ml"
               _1_1
            in
            
# 205 "<standard.mly>"
    ( x )
# 7661 "asllib/Parser.ml"
            
          in
          let recurse_limit = 
# 123 "<standard.mly>"
    ( None )
# 7667 "asllib/Parser.ml"
           in
          let return_type =
            let _1 = _1_inlined4 in
            _2
          in
          let func_args =
            let _1_inlined1 = _1_inlined3 in
            let x = _1_inlined1 in
            
# 205 "<standard.mly>"
    ( x )
# 7679 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 545 "asllib/Parser.mly"
        (
          D_Func {
            name;
            parameters = params_opt;
            args = func_args;
            body = SB_ASL body;
            return_type = Some return_type;
            subprogram_type = ST_Function;
            recurse_limit;
            builtin = false;
          }
        )
# 7697 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7706 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 7712 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_139 =
    fun _1 _1_inlined1 _1_inlined3 _1_inlined4 _2 _endpos__1_inlined5_ _endpos__3_inlined1_ _startpos__1_ name xs ->
      let x =
        let desc =
          let body =
            let _endpos__1_ = _endpos__1_inlined5_ in
            let x =
              let _endpos__0_ = _endpos__1_ in
              let desc = 
# 534 "asllib/Parser.mly"
                                                   ( S_Pass )
# 7726 "asllib/Parser.ml"
               in
              let _endpos_desc_ = _endpos__0_ in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _endpos in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7734 "asllib/Parser.ml"
              
            in
            
# 205 "<standard.mly>"
    ( x )
# 7740 "asllib/Parser.ml"
            
          in
          let recurse_limit = 
# 123 "<standard.mly>"
    ( None )
# 7746 "asllib/Parser.ml"
           in
          let return_type =
            let _1 = _1_inlined4 in
            _2
          in
          let func_args =
            let _1_inlined1 = _1_inlined3 in
            let x = _1_inlined1 in
            
# 205 "<standard.mly>"
    ( x )
# 7758 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 545 "asllib/Parser.mly"
        (
          D_Func {
            name;
            parameters = params_opt;
            args = func_args;
            body = SB_ASL body;
            return_type = Some return_type;
            subprogram_type = ST_Function;
            recurse_limit;
            builtin = false;
          }
        )
# 7776 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7785 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 7791 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_140 =
    fun _1 _1_inlined1 _1_inlined3 _1_inlined4 _1_inlined5 _1_inlined7 _2 _2_inlined1 _endpos__3_inlined1_ _startpos__1_ name xs ->
      let x =
        let desc =
          let body =
            let _1_inlined1 = _1_inlined7 in
            let x =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 7806 "asllib/Parser.ml"
               _1_1
            in
            
# 205 "<standard.mly>"
    ( x )
# 7812 "asllib/Parser.ml"
            
          in
          let recurse_limit =
            let (_2, _1) = (_2_inlined1, _1_inlined5) in
            let x = _2 in
            
# 126 "<standard.mly>"
    ( Some x )
# 7821 "asllib/Parser.ml"
            
          in
          let return_type =
            let _1 = _1_inlined4 in
            _2
          in
          let func_args =
            let _1_inlined1 = _1_inlined3 in
            let x = _1_inlined1 in
            
# 205 "<standard.mly>"
    ( x )
# 7834 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 545 "asllib/Parser.mly"
        (
          D_Func {
            name;
            parameters = params_opt;
            args = func_args;
            body = SB_ASL body;
            return_type = Some return_type;
            subprogram_type = ST_Function;
            recurse_limit;
            builtin = false;
          }
        )
# 7852 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7861 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 7867 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_141 =
    fun _1 _1_inlined1 _1_inlined3 _1_inlined4 _1_inlined5 _2 _2_inlined1 _endpos__1_inlined6_ _endpos__3_inlined1_ _startpos__1_ name xs ->
      let x =
        let desc =
          let body =
            let _endpos__1_ = _endpos__1_inlined6_ in
            let x =
              let _endpos__0_ = _endpos__1_ in
              let desc = 
# 534 "asllib/Parser.mly"
                                                   ( S_Pass )
# 7881 "asllib/Parser.ml"
               in
              let _endpos_desc_ = _endpos__0_ in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _endpos in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7889 "asllib/Parser.ml"
              
            in
            
# 205 "<standard.mly>"
    ( x )
# 7895 "asllib/Parser.ml"
            
          in
          let recurse_limit =
            let (_2, _1) = (_2_inlined1, _1_inlined5) in
            let x = _2 in
            
# 126 "<standard.mly>"
    ( Some x )
# 7904 "asllib/Parser.ml"
            
          in
          let return_type =
            let _1 = _1_inlined4 in
            _2
          in
          let func_args =
            let _1_inlined1 = _1_inlined3 in
            let x = _1_inlined1 in
            
# 205 "<standard.mly>"
    ( x )
# 7917 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 545 "asllib/Parser.mly"
        (
          D_Func {
            name;
            parameters = params_opt;
            args = func_args;
            body = SB_ASL body;
            return_type = Some return_type;
            subprogram_type = ST_Function;
            recurse_limit;
            builtin = false;
          }
        )
# 7935 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 7944 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 7950 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_142 =
    fun _1 _1_inlined1 _1_inlined4 _endpos__3_inlined1_ _startpos__1_ name xs ->
      let x =
        let desc =
          let body =
            let _1_inlined1 = _1_inlined4 in
            let x =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 7965 "asllib/Parser.ml"
               _1_1
            in
            
# 205 "<standard.mly>"
    ( x )
# 7971 "asllib/Parser.ml"
            
          in
          let func_args =
            let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 7978 "asllib/Parser.ml"
             in
            
# 205 "<standard.mly>"
    ( x )
# 7983 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 560 "asllib/Parser.mly"
        (
          D_Func {
            name;
            parameters = params_opt;
            args = func_args;
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        )
# 8001 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8010 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 8016 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_143 =
    fun _1 _1_inlined1 _endpos__1_inlined3_ _endpos__3_inlined1_ _startpos__1_ name xs ->
      let x =
        let desc =
          let body =
            let _endpos__1_ = _endpos__1_inlined3_ in
            let x =
              let _endpos__0_ = _endpos__1_ in
              let desc = 
# 534 "asllib/Parser.mly"
                                                   ( S_Pass )
# 8030 "asllib/Parser.ml"
               in
              let _endpos_desc_ = _endpos__0_ in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _endpos in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8038 "asllib/Parser.ml"
              
            in
            
# 205 "<standard.mly>"
    ( x )
# 8044 "asllib/Parser.ml"
            
          in
          let func_args =
            let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 8051 "asllib/Parser.ml"
             in
            
# 205 "<standard.mly>"
    ( x )
# 8056 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 560 "asllib/Parser.mly"
        (
          D_Func {
            name;
            parameters = params_opt;
            args = func_args;
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        )
# 8074 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8083 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 8089 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_144 =
    fun _1 _1_inlined1 _1_inlined3 _1_inlined5 _endpos__3_inlined1_ _startpos__1_ name xs ->
      let x =
        let desc =
          let body =
            let _1_inlined1 = _1_inlined5 in
            let x =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 8104 "asllib/Parser.ml"
               _1_1
            in
            
# 205 "<standard.mly>"
    ( x )
# 8110 "asllib/Parser.ml"
            
          in
          let func_args =
            let _1_inlined1 = _1_inlined3 in
            let x = _1_inlined1 in
            
# 205 "<standard.mly>"
    ( x )
# 8119 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 560 "asllib/Parser.mly"
        (
          D_Func {
            name;
            parameters = params_opt;
            args = func_args;
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        )
# 8137 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8146 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 8152 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_145 =
    fun _1 _1_inlined1 _1_inlined3 _endpos__1_inlined4_ _endpos__3_inlined1_ _startpos__1_ name xs ->
      let x =
        let desc =
          let body =
            let _endpos__1_ = _endpos__1_inlined4_ in
            let x =
              let _endpos__0_ = _endpos__1_ in
              let desc = 
# 534 "asllib/Parser.mly"
                                                   ( S_Pass )
# 8166 "asllib/Parser.ml"
               in
              let _endpos_desc_ = _endpos__0_ in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _endpos in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8174 "asllib/Parser.ml"
              
            in
            
# 205 "<standard.mly>"
    ( x )
# 8180 "asllib/Parser.ml"
            
          in
          let func_args =
            let _1_inlined1 = _1_inlined3 in
            let x = _1_inlined1 in
            
# 205 "<standard.mly>"
    ( x )
# 8189 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 560 "asllib/Parser.mly"
        (
          D_Func {
            name;
            parameters = params_opt;
            args = func_args;
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        )
# 8207 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8216 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 8222 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_146 =
    fun _1 _1_inlined1 _1_inlined3 _1_inlined5 _2 _endpos__3_inlined1_ _startpos__1_ name xs ->
      let x =
        let desc =
          let func_body =
            let _1_inlined1 = _1_inlined5 in
            let x =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 8237 "asllib/Parser.ml"
               _1_1
            in
            
# 205 "<standard.mly>"
    ( x )
# 8243 "asllib/Parser.ml"
            
          in
          let return_type =
            let _1 = _1_inlined3 in
            _2
          in
          let func_args =
            let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 8254 "asllib/Parser.ml"
             in
            
# 205 "<standard.mly>"
    ( x )
# 8259 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 576 "asllib/Parser.mly"
        (
          D_Func
            {
              name;
              parameters = params_opt;
              args = func_args;
              return_type = Some return_type;
              body = SB_ASL func_body;
              subprogram_type = ST_Getter;
              recurse_limit = None;
              builtin = false;
            }
        )
# 8278 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8287 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 8293 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_147 =
    fun _1 _1_inlined1 _1_inlined3 _2 _endpos__1_inlined4_ _endpos__3_inlined1_ _startpos__1_ name xs ->
      let x =
        let desc =
          let func_body =
            let _endpos__1_ = _endpos__1_inlined4_ in
            let x =
              let _endpos__0_ = _endpos__1_ in
              let desc = 
# 534 "asllib/Parser.mly"
                                                   ( S_Pass )
# 8307 "asllib/Parser.ml"
               in
              let _endpos_desc_ = _endpos__0_ in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _endpos in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8315 "asllib/Parser.ml"
              
            in
            
# 205 "<standard.mly>"
    ( x )
# 8321 "asllib/Parser.ml"
            
          in
          let return_type =
            let _1 = _1_inlined3 in
            _2
          in
          let func_args =
            let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 8332 "asllib/Parser.ml"
             in
            
# 205 "<standard.mly>"
    ( x )
# 8337 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 576 "asllib/Parser.mly"
        (
          D_Func
            {
              name;
              parameters = params_opt;
              args = func_args;
              return_type = Some return_type;
              body = SB_ASL func_body;
              subprogram_type = ST_Getter;
              recurse_limit = None;
              builtin = false;
            }
        )
# 8356 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8365 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 8371 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_148 =
    fun _1 _1_inlined1 _1_inlined3 _1_inlined4 _1_inlined6 _2 _endpos__3_inlined1_ _startpos__1_ name xs ->
      let x =
        let desc =
          let func_body =
            let _1_inlined1 = _1_inlined6 in
            let x =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 8386 "asllib/Parser.ml"
               _1_1
            in
            
# 205 "<standard.mly>"
    ( x )
# 8392 "asllib/Parser.ml"
            
          in
          let return_type =
            let _1 = _1_inlined4 in
            _2
          in
          let func_args =
            let _1_inlined1 = _1_inlined3 in
            let x = _1_inlined1 in
            
# 205 "<standard.mly>"
    ( x )
# 8405 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 576 "asllib/Parser.mly"
        (
          D_Func
            {
              name;
              parameters = params_opt;
              args = func_args;
              return_type = Some return_type;
              body = SB_ASL func_body;
              subprogram_type = ST_Getter;
              recurse_limit = None;
              builtin = false;
            }
        )
# 8424 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8433 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 8439 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_149 =
    fun _1 _1_inlined1 _1_inlined3 _1_inlined4 _2 _endpos__1_inlined5_ _endpos__3_inlined1_ _startpos__1_ name xs ->
      let x =
        let desc =
          let func_body =
            let _endpos__1_ = _endpos__1_inlined5_ in
            let x =
              let _endpos__0_ = _endpos__1_ in
              let desc = 
# 534 "asllib/Parser.mly"
                                                   ( S_Pass )
# 8453 "asllib/Parser.ml"
               in
              let _endpos_desc_ = _endpos__0_ in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _endpos in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8461 "asllib/Parser.ml"
              
            in
            
# 205 "<standard.mly>"
    ( x )
# 8467 "asllib/Parser.ml"
            
          in
          let return_type =
            let _1 = _1_inlined4 in
            _2
          in
          let func_args =
            let _1_inlined1 = _1_inlined3 in
            let x = _1_inlined1 in
            
# 205 "<standard.mly>"
    ( x )
# 8480 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 576 "asllib/Parser.mly"
        (
          D_Func
            {
              name;
              parameters = params_opt;
              args = func_args;
              return_type = Some return_type;
              body = SB_ASL func_body;
              subprogram_type = ST_Getter;
              recurse_limit = None;
              builtin = false;
            }
        )
# 8499 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8508 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 8514 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_150 =
    fun _1 _1_inlined1 _1_inlined3 _1_inlined5 _2 _5 _endpos__3_inlined1_ _startpos__1_ name x xs ->
      let x =
        let desc =
          let func_body =
            let _1_inlined1 = _1_inlined5 in
            let x =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 8529 "asllib/Parser.ml"
               _1_1
            in
            
# 205 "<standard.mly>"
    ( x )
# 8535 "asllib/Parser.ml"
            
          in
          let v =
            let _1 = _1_inlined3 in
            let y = _2 in
            
# 172 "<standard.mly>"
    ( (x, y) )
# 8544 "asllib/Parser.ml"
            
          in
          let func_args =
            let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 8551 "asllib/Parser.ml"
             in
            
# 205 "<standard.mly>"
    ( x )
# 8556 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 593 "asllib/Parser.mly"
        (
          D_Func
            {
              name;
              parameters = params_opt;
              args = v :: func_args;
              return_type = None;
              body = SB_ASL func_body;
              subprogram_type = ST_Setter;
              recurse_limit = None;
              builtin = false;
            }
        )
# 8575 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8584 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 8590 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_151 =
    fun _1 _1_inlined1 _1_inlined3 _2 _5 _endpos__1_inlined4_ _endpos__3_inlined1_ _startpos__1_ name x xs ->
      let x =
        let desc =
          let func_body =
            let _endpos__1_ = _endpos__1_inlined4_ in
            let x =
              let _endpos__0_ = _endpos__1_ in
              let desc = 
# 534 "asllib/Parser.mly"
                                                   ( S_Pass )
# 8604 "asllib/Parser.ml"
               in
              let _endpos_desc_ = _endpos__0_ in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _endpos in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8612 "asllib/Parser.ml"
              
            in
            
# 205 "<standard.mly>"
    ( x )
# 8618 "asllib/Parser.ml"
            
          in
          let v =
            let _1 = _1_inlined3 in
            let y = _2 in
            
# 172 "<standard.mly>"
    ( (x, y) )
# 8627 "asllib/Parser.ml"
            
          in
          let func_args =
            let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 8634 "asllib/Parser.ml"
             in
            
# 205 "<standard.mly>"
    ( x )
# 8639 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 593 "asllib/Parser.mly"
        (
          D_Func
            {
              name;
              parameters = params_opt;
              args = v :: func_args;
              return_type = None;
              body = SB_ASL func_body;
              subprogram_type = ST_Setter;
              recurse_limit = None;
              builtin = false;
            }
        )
# 8658 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8667 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 8673 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_152 =
    fun _1 _1_inlined1 _1_inlined3 _1_inlined5 _2 _5 _endpos__3_inlined1_ _startpos__1_ name x xs ->
      let x =
        let desc =
          let func_body =
            let _1_inlined1 = _1_inlined5 in
            let x =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 8688 "asllib/Parser.ml"
               _1_1
            in
            
# 205 "<standard.mly>"
    ( x )
# 8694 "asllib/Parser.ml"
            
          in
          let v =
            let _1 = _1_inlined3 in
            let y = _2 in
            
# 172 "<standard.mly>"
    ( (x, y) )
# 8703 "asllib/Parser.ml"
            
          in
          let func_args =
            let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 8710 "asllib/Parser.ml"
             in
            
# 205 "<standard.mly>"
    ( x )
# 8715 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 593 "asllib/Parser.mly"
        (
          D_Func
            {
              name;
              parameters = params_opt;
              args = v :: func_args;
              return_type = None;
              body = SB_ASL func_body;
              subprogram_type = ST_Setter;
              recurse_limit = None;
              builtin = false;
            }
        )
# 8734 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8743 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 8749 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_153 =
    fun _1 _1_inlined1 _1_inlined3 _2 _5 _endpos__1_inlined4_ _endpos__3_inlined1_ _startpos__1_ name x xs ->
      let x =
        let desc =
          let func_body =
            let _endpos__1_ = _endpos__1_inlined4_ in
            let x =
              let _endpos__0_ = _endpos__1_ in
              let desc = 
# 534 "asllib/Parser.mly"
                                                   ( S_Pass )
# 8763 "asllib/Parser.ml"
               in
              let _endpos_desc_ = _endpos__0_ in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _endpos in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8771 "asllib/Parser.ml"
              
            in
            
# 205 "<standard.mly>"
    ( x )
# 8777 "asllib/Parser.ml"
            
          in
          let v =
            let _1 = _1_inlined3 in
            let y = _2 in
            
# 172 "<standard.mly>"
    ( (x, y) )
# 8786 "asllib/Parser.ml"
            
          in
          let func_args =
            let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 8793 "asllib/Parser.ml"
             in
            
# 205 "<standard.mly>"
    ( x )
# 8798 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 593 "asllib/Parser.mly"
        (
          D_Func
            {
              name;
              parameters = params_opt;
              args = v :: func_args;
              return_type = None;
              body = SB_ASL func_body;
              subprogram_type = ST_Setter;
              recurse_limit = None;
              builtin = false;
            }
        )
# 8817 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8826 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 8832 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_154 =
    fun _1 _1_inlined1 _1_inlined3 _1_inlined4 _1_inlined6 _2 _5 _endpos__3_inlined1_ _startpos__1_ name x xs ->
      let x =
        let desc =
          let func_body =
            let _1_inlined1 = _1_inlined6 in
            let x =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 8847 "asllib/Parser.ml"
               _1_1
            in
            
# 205 "<standard.mly>"
    ( x )
# 8853 "asllib/Parser.ml"
            
          in
          let v =
            let _1 = _1_inlined4 in
            let y = _2 in
            
# 172 "<standard.mly>"
    ( (x, y) )
# 8862 "asllib/Parser.ml"
            
          in
          let func_args =
            let _1_inlined1 = _1_inlined3 in
            let x = _1_inlined1 in
            
# 205 "<standard.mly>"
    ( x )
# 8871 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 593 "asllib/Parser.mly"
        (
          D_Func
            {
              name;
              parameters = params_opt;
              args = v :: func_args;
              return_type = None;
              body = SB_ASL func_body;
              subprogram_type = ST_Setter;
              recurse_limit = None;
              builtin = false;
            }
        )
# 8890 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8899 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 8905 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_155 =
    fun _1 _1_inlined1 _1_inlined3 _1_inlined4 _2 _5 _endpos__1_inlined5_ _endpos__3_inlined1_ _startpos__1_ name x xs ->
      let x =
        let desc =
          let func_body =
            let _endpos__1_ = _endpos__1_inlined5_ in
            let x =
              let _endpos__0_ = _endpos__1_ in
              let desc = 
# 534 "asllib/Parser.mly"
                                                   ( S_Pass )
# 8919 "asllib/Parser.ml"
               in
              let _endpos_desc_ = _endpos__0_ in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _endpos in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8927 "asllib/Parser.ml"
              
            in
            
# 205 "<standard.mly>"
    ( x )
# 8933 "asllib/Parser.ml"
            
          in
          let v =
            let _1 = _1_inlined4 in
            let y = _2 in
            
# 172 "<standard.mly>"
    ( (x, y) )
# 8942 "asllib/Parser.ml"
            
          in
          let func_args =
            let _1_inlined1 = _1_inlined3 in
            let x = _1_inlined1 in
            
# 205 "<standard.mly>"
    ( x )
# 8951 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 593 "asllib/Parser.mly"
        (
          D_Func
            {
              name;
              parameters = params_opt;
              args = v :: func_args;
              return_type = None;
              body = SB_ASL func_body;
              subprogram_type = ST_Setter;
              recurse_limit = None;
              builtin = false;
            }
        )
# 8970 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 8979 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 8985 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_156 =
    fun _1 _1_inlined1 _1_inlined3 _1_inlined4 _1_inlined6 _2 _5 _endpos__3_inlined1_ _startpos__1_ name x xs ->
      let x =
        let desc =
          let func_body =
            let _1_inlined1 = _1_inlined6 in
            let x =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 9000 "asllib/Parser.ml"
               _1_1
            in
            
# 205 "<standard.mly>"
    ( x )
# 9006 "asllib/Parser.ml"
            
          in
          let v =
            let _1 = _1_inlined4 in
            let y = _2 in
            
# 172 "<standard.mly>"
    ( (x, y) )
# 9015 "asllib/Parser.ml"
            
          in
          let func_args =
            let _1_inlined1 = _1_inlined3 in
            let x = _1_inlined1 in
            
# 205 "<standard.mly>"
    ( x )
# 9024 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 593 "asllib/Parser.mly"
        (
          D_Func
            {
              name;
              parameters = params_opt;
              args = v :: func_args;
              return_type = None;
              body = SB_ASL func_body;
              subprogram_type = ST_Setter;
              recurse_limit = None;
              builtin = false;
            }
        )
# 9043 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9052 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 9058 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_157 =
    fun _1 _1_inlined1 _1_inlined3 _1_inlined4 _2 _5 _endpos__1_inlined5_ _endpos__3_inlined1_ _startpos__1_ name x xs ->
      let x =
        let desc =
          let func_body =
            let _endpos__1_ = _endpos__1_inlined5_ in
            let x =
              let _endpos__0_ = _endpos__1_ in
              let desc = 
# 534 "asllib/Parser.mly"
                                                   ( S_Pass )
# 9072 "asllib/Parser.ml"
               in
              let _endpos_desc_ = _endpos__0_ in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _endpos in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9080 "asllib/Parser.ml"
              
            in
            
# 205 "<standard.mly>"
    ( x )
# 9086 "asllib/Parser.ml"
            
          in
          let v =
            let _1 = _1_inlined4 in
            let y = _2 in
            
# 172 "<standard.mly>"
    ( (x, y) )
# 9095 "asllib/Parser.ml"
            
          in
          let func_args =
            let _1_inlined1 = _1_inlined3 in
            let x = _1_inlined1 in
            
# 205 "<standard.mly>"
    ( x )
# 9104 "asllib/Parser.ml"
            
          in
          let params_opt = _1_inlined1 in
          
# 593 "asllib/Parser.mly"
        (
          D_Func
            {
              name;
              parameters = params_opt;
              args = v :: func_args;
              return_type = None;
              body = SB_ASL func_body;
              subprogram_type = ST_Setter;
              recurse_limit = None;
              builtin = false;
            }
        )
# 9123 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9132 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 9138 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_158 =
    fun _1 _1_inlined1 _3 _endpos__2_ _startpos__1_ t x xs ->
      let x =
        let desc =
          let x =
            let subtype_opt = _1_inlined1 in
            
# 609 "asllib/Parser.mly"
                                                            D_TypeDecl            
# 9150 "asllib/Parser.ml"
             (x, t, subtype_opt)
          in
          
# 196 "<standard.mly>"
    ( x )
# 9156 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9165 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 9171 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_159 =
    fun _1 _1_inlined1 _2_inlined1 _3 _endpos__2_ _endpos__3_ _startpos__1_ _startpos__1_inlined1_ x xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let s =
              let (_startpos__1_, _1) = (_startpos__1_inlined1_, _1_inlined1) in
              let desc =
                let _2_1 = _2 in
                let _3_1 = _3 in
                (_2_1, _3_1)
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9193 "asllib/Parser.ml"
              
            in
            
# 612 "asllib/Parser.mly"
                                                            make_ty_decl_subtype  
# 9199 "asllib/Parser.ml"
             (x, s)
          in
          
# 196 "<standard.mly>"
    ( x )
# 9205 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9214 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 9220 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_160 =
    fun _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 9231 "asllib/Parser.ml"
             x in
            let ty = 
# 123 "<standard.mly>"
    ( None )
# 9236 "asllib/Parser.ml"
             in
            let name =
              let _1 = _1_inlined1 in
              
# 538 "asllib/Parser.mly"
           ( global_ignored () )
# 9243 "asllib/Parser.ml"
              
            in
            let keyword = 
# 432 "asllib/Parser.mly"
                ( GDK_Let      )
# 9249 "asllib/Parser.ml"
             in
            
# 617 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 9254 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 9260 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9269 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 9275 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_161 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 9287 "asllib/Parser.ml"
             x in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 9295 "asllib/Parser.ml"
              
            in
            let name =
              let _1 = _1_inlined1 in
              
# 538 "asllib/Parser.mly"
           ( global_ignored () )
# 9303 "asllib/Parser.ml"
              
            in
            let keyword = 
# 432 "asllib/Parser.mly"
                ( GDK_Let      )
# 9309 "asllib/Parser.ml"
             in
            
# 617 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 9314 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 9320 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9329 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 9335 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_162 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 9347 "asllib/Parser.ml"
             x in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 9355 "asllib/Parser.ml"
              
            in
            let name =
              let _1 = _1_inlined1 in
              
# 538 "asllib/Parser.mly"
           ( global_ignored () )
# 9363 "asllib/Parser.ml"
              
            in
            let keyword = 
# 432 "asllib/Parser.mly"
                ( GDK_Let      )
# 9369 "asllib/Parser.ml"
             in
            
# 617 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 9374 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 9380 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9389 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 9395 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_163 =
    fun _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 9406 "asllib/Parser.ml"
             x in
            let ty = 
# 123 "<standard.mly>"
    ( None )
# 9411 "asllib/Parser.ml"
             in
            let name = _1_inlined1 in
            let keyword = 
# 432 "asllib/Parser.mly"
                ( GDK_Let      )
# 9417 "asllib/Parser.ml"
             in
            
# 617 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 9422 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 9428 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9437 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 9443 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_164 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 9455 "asllib/Parser.ml"
             x in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 9463 "asllib/Parser.ml"
              
            in
            let name = _1_inlined1 in
            let keyword = 
# 432 "asllib/Parser.mly"
                ( GDK_Let      )
# 9470 "asllib/Parser.ml"
             in
            
# 617 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 9475 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 9481 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9490 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 9496 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_165 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 9508 "asllib/Parser.ml"
             x in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 9516 "asllib/Parser.ml"
              
            in
            let name = _1_inlined1 in
            let keyword = 
# 432 "asllib/Parser.mly"
                ( GDK_Let      )
# 9523 "asllib/Parser.ml"
             in
            
# 617 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 9528 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 9534 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9543 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 9549 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_166 =
    fun _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 9560 "asllib/Parser.ml"
             x in
            let ty = 
# 123 "<standard.mly>"
    ( None )
# 9565 "asllib/Parser.ml"
             in
            let name =
              let _1 = _1_inlined1 in
              
# 538 "asllib/Parser.mly"
           ( global_ignored () )
# 9572 "asllib/Parser.ml"
              
            in
            let keyword = 
# 433 "asllib/Parser.mly"
                ( GDK_Constant )
# 9578 "asllib/Parser.ml"
             in
            
# 617 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 9583 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 9589 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9598 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 9604 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_167 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 9616 "asllib/Parser.ml"
             x in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 9624 "asllib/Parser.ml"
              
            in
            let name =
              let _1 = _1_inlined1 in
              
# 538 "asllib/Parser.mly"
           ( global_ignored () )
# 9632 "asllib/Parser.ml"
              
            in
            let keyword = 
# 433 "asllib/Parser.mly"
                ( GDK_Constant )
# 9638 "asllib/Parser.ml"
             in
            
# 617 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 9643 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 9649 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9658 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 9664 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_168 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 9676 "asllib/Parser.ml"
             x in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 9684 "asllib/Parser.ml"
              
            in
            let name =
              let _1 = _1_inlined1 in
              
# 538 "asllib/Parser.mly"
           ( global_ignored () )
# 9692 "asllib/Parser.ml"
              
            in
            let keyword = 
# 433 "asllib/Parser.mly"
                ( GDK_Constant )
# 9698 "asllib/Parser.ml"
             in
            
# 617 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 9703 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 9709 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9718 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 9724 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_169 =
    fun _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 9735 "asllib/Parser.ml"
             x in
            let ty = 
# 123 "<standard.mly>"
    ( None )
# 9740 "asllib/Parser.ml"
             in
            let name = _1_inlined1 in
            let keyword = 
# 433 "asllib/Parser.mly"
                ( GDK_Constant )
# 9746 "asllib/Parser.ml"
             in
            
# 617 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 9751 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 9757 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9766 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 9772 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_170 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 9784 "asllib/Parser.ml"
             x in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 9792 "asllib/Parser.ml"
              
            in
            let name = _1_inlined1 in
            let keyword = 
# 433 "asllib/Parser.mly"
                ( GDK_Constant )
# 9799 "asllib/Parser.ml"
             in
            
# 617 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 9804 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 9810 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9819 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 9825 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_171 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 9837 "asllib/Parser.ml"
             x in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 9845 "asllib/Parser.ml"
              
            in
            let name = _1_inlined1 in
            let keyword = 
# 433 "asllib/Parser.mly"
                ( GDK_Constant )
# 9852 "asllib/Parser.ml"
             in
            
# 617 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 9857 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 9863 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9872 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 9878 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_172 =
    fun _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 9889 "asllib/Parser.ml"
             x in
            let ty = 
# 123 "<standard.mly>"
    ( None )
# 9894 "asllib/Parser.ml"
             in
            let name =
              let _1 = _1_inlined1 in
              
# 538 "asllib/Parser.mly"
           ( global_ignored () )
# 9901 "asllib/Parser.ml"
              
            in
            let keyword = 
# 434 "asllib/Parser.mly"
                ( GDK_Var      )
# 9907 "asllib/Parser.ml"
             in
            
# 617 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 9912 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 9918 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9927 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 9933 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_173 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 9945 "asllib/Parser.ml"
             x in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 9953 "asllib/Parser.ml"
              
            in
            let name =
              let _1 = _1_inlined1 in
              
# 538 "asllib/Parser.mly"
           ( global_ignored () )
# 9961 "asllib/Parser.ml"
              
            in
            let keyword = 
# 434 "asllib/Parser.mly"
                ( GDK_Var      )
# 9967 "asllib/Parser.ml"
             in
            
# 617 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 9972 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 9978 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 9987 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 9993 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_174 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 10005 "asllib/Parser.ml"
             x in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 10013 "asllib/Parser.ml"
              
            in
            let name =
              let _1 = _1_inlined1 in
              
# 538 "asllib/Parser.mly"
           ( global_ignored () )
# 10021 "asllib/Parser.ml"
              
            in
            let keyword = 
# 434 "asllib/Parser.mly"
                ( GDK_Var      )
# 10027 "asllib/Parser.ml"
             in
            
# 617 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 10032 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 10038 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 10047 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 10053 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_175 =
    fun _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 10064 "asllib/Parser.ml"
             x in
            let ty = 
# 123 "<standard.mly>"
    ( None )
# 10069 "asllib/Parser.ml"
             in
            let name = _1_inlined1 in
            let keyword = 
# 434 "asllib/Parser.mly"
                ( GDK_Var      )
# 10075 "asllib/Parser.ml"
             in
            
# 617 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 10080 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 10086 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 10095 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 10101 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_176 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 10113 "asllib/Parser.ml"
             x in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 10121 "asllib/Parser.ml"
              
            in
            let name = _1_inlined1 in
            let keyword = 
# 434 "asllib/Parser.mly"
                ( GDK_Var      )
# 10128 "asllib/Parser.ml"
             in
            
# 617 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 10133 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 10139 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 10148 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 10154 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_177 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 10166 "asllib/Parser.ml"
             x in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 10174 "asllib/Parser.ml"
              
            in
            let name = _1_inlined1 in
            let keyword = 
# 434 "asllib/Parser.mly"
                ( GDK_Var      )
# 10181 "asllib/Parser.ml"
             in
            
# 617 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 10186 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 10192 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 10201 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 10207 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_178 =
    fun _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 10218 "asllib/Parser.ml"
             x in
            let ty = 
# 123 "<standard.mly>"
    ( None )
# 10223 "asllib/Parser.ml"
             in
            let name =
              let _1 = _1_inlined1 in
              
# 538 "asllib/Parser.mly"
           ( global_ignored () )
# 10230 "asllib/Parser.ml"
              
            in
            let keyword = 
# 435 "asllib/Parser.mly"
                ( GDK_Config   )
# 10236 "asllib/Parser.ml"
             in
            
# 617 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 10241 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 10247 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 10256 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 10262 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_179 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 10274 "asllib/Parser.ml"
             x in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 10282 "asllib/Parser.ml"
              
            in
            let name =
              let _1 = _1_inlined1 in
              
# 538 "asllib/Parser.mly"
           ( global_ignored () )
# 10290 "asllib/Parser.ml"
              
            in
            let keyword = 
# 435 "asllib/Parser.mly"
                ( GDK_Config   )
# 10296 "asllib/Parser.ml"
             in
            
# 617 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 10301 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 10307 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 10316 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 10322 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_180 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 10334 "asllib/Parser.ml"
             x in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 10342 "asllib/Parser.ml"
              
            in
            let name =
              let _1 = _1_inlined1 in
              
# 538 "asllib/Parser.mly"
           ( global_ignored () )
# 10350 "asllib/Parser.ml"
              
            in
            let keyword = 
# 435 "asllib/Parser.mly"
                ( GDK_Config   )
# 10356 "asllib/Parser.ml"
             in
            
# 617 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 10361 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 10367 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 10376 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 10382 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_181 =
    fun _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 10393 "asllib/Parser.ml"
             x in
            let ty = 
# 123 "<standard.mly>"
    ( None )
# 10398 "asllib/Parser.ml"
             in
            let name = _1_inlined1 in
            let keyword = 
# 435 "asllib/Parser.mly"
                ( GDK_Config   )
# 10404 "asllib/Parser.ml"
             in
            
# 617 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 10409 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 10415 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 10424 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 10430 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_182 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 10442 "asllib/Parser.ml"
             x in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 10450 "asllib/Parser.ml"
              
            in
            let name = _1_inlined1 in
            let keyword = 
# 435 "asllib/Parser.mly"
                ( GDK_Config   )
# 10457 "asllib/Parser.ml"
             in
            
# 617 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 10462 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 10468 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 10477 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 10483 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_183 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let initial_value = 
# 133 "asllib/Parser.mly"
                        Some 
# 10495 "asllib/Parser.ml"
             x in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 10503 "asllib/Parser.ml"
              
            in
            let name = _1_inlined1 in
            let keyword = 
# 435 "asllib/Parser.mly"
                ( GDK_Config   )
# 10510 "asllib/Parser.ml"
             in
            
# 617 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword; name; ty; initial_value } )
# 10515 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 10521 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 10530 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 10536 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_184 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _endpos__2_ _startpos__1_ xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 133 "asllib/Parser.mly"
                        Some 
# 10551 "asllib/Parser.ml"
               x
            in
            let name =
              let _1 = _1_inlined1 in
              
# 538 "asllib/Parser.mly"
           ( global_ignored () )
# 10559 "asllib/Parser.ml"
              
            in
            
# 621 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword=GDK_Var; name; ty; initial_value=None})
# 10565 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 10571 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 10580 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 10586 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_185 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _endpos__2_ _startpos__1_ xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 133 "asllib/Parser.mly"
                        Some 
# 10601 "asllib/Parser.ml"
               x
            in
            let name =
              let _1 = _1_inlined1 in
              
# 538 "asllib/Parser.mly"
           ( global_ignored () )
# 10609 "asllib/Parser.ml"
              
            in
            
# 621 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword=GDK_Var; name; ty; initial_value=None})
# 10615 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 10621 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 10630 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 10636 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_186 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _endpos__2_ _startpos__1_ xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 133 "asllib/Parser.mly"
                        Some 
# 10651 "asllib/Parser.ml"
               x
            in
            let name = _1_inlined1 in
            
# 621 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword=GDK_Var; name; ty; initial_value=None})
# 10658 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 10664 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 10673 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 10679 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_187 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _endpos__2_ _startpos__1_ xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let ty =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 133 "asllib/Parser.mly"
                        Some 
# 10694 "asllib/Parser.ml"
               x
            in
            let name = _1_inlined1 in
            
# 621 "asllib/Parser.mly"
        ( D_GlobalStorage { keyword=GDK_Var; name; ty; initial_value=None})
# 10701 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 10707 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 10716 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 10722 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_188 =
    fun _1 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let e = 
# 161 "asllib/Parser.mly"
                ( [] )
# 10733 "asllib/Parser.ml"
             in
            
# 624 "asllib/Parser.mly"
                                               D_Pragma  
# 10738 "asllib/Parser.ml"
             (x, e)
          in
          
# 196 "<standard.mly>"
    ( x )
# 10744 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 10753 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 10759 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_189 =
    fun _1 _1_inlined1 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let e = _1_inlined1 in
            
# 624 "asllib/Parser.mly"
                                               D_Pragma  
# 10771 "asllib/Parser.ml"
             (x, e)
          in
          
# 196 "<standard.mly>"
    ( x )
# 10777 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 10786 "asllib/Parser.ml"
        
      in
      (
# 219 "<standard.mly>"
    ( x :: xs )
# 10792 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_190 =
    fun () ->
      (
# 145 "<standard.mly>"
    ( [] )
# 10800 "asllib/Parser.ml"
       : (AST.field list))
  
  let _menhir_action_191 =
    fun _1 _1_inlined2 ->
      let x =
        let _1_inlined1 = _1_inlined2 in
        let x = _1_inlined1 in
        
# 205 "<standard.mly>"
    ( x )
# 10811 "asllib/Parser.ml"
        
      in
      (
# 148 "<standard.mly>"
    ( x )
# 10817 "asllib/Parser.ml"
       : (AST.field list))
  
  let _menhir_action_192 =
    fun () ->
      (
# 145 "<standard.mly>"
    ( [] )
# 10825 "asllib/Parser.ml"
       : (AST.bitfield list))
  
  let _menhir_action_193 =
    fun _1_inlined1 ->
      let x =
        let x = _1_inlined1 in
        
# 205 "<standard.mly>"
    ( x )
# 10835 "asllib/Parser.ml"
        
      in
      (
# 148 "<standard.mly>"
    ( x )
# 10841 "asllib/Parser.ml"
       : (AST.bitfield list))
  
  let _menhir_action_194 =
    fun () ->
      (
# 145 "<standard.mly>"
    ( [] )
# 10849 "asllib/Parser.ml"
       : ((string * AST.ty option) list))
  
  let _menhir_action_195 =
    fun _1_inlined1 ->
      let x =
        let x = _1_inlined1 in
        
# 205 "<standard.mly>"
    ( x )
# 10859 "asllib/Parser.ml"
        
      in
      (
# 148 "<standard.mly>"
    ( x )
# 10865 "asllib/Parser.ml"
       : ((string * AST.ty option) list))
  
  let _menhir_action_196 =
    fun () ->
      (
# 145 "<standard.mly>"
    ( [] )
# 10873 "asllib/Parser.ml"
       : (AST.bitfield list))
  
  let _menhir_action_197 =
    fun x ->
      (
# 148 "<standard.mly>"
    ( x )
# 10881 "asllib/Parser.ml"
       : (AST.bitfield list))
  
  let _menhir_action_198 =
    fun () ->
      (
# 145 "<standard.mly>"
    ( [] )
# 10889 "asllib/Parser.ml"
       : (AST.field list))
  
  let _menhir_action_199 =
    fun x ->
      (
# 148 "<standard.mly>"
    ( x )
# 10897 "asllib/Parser.ml"
       : (AST.field list))
  
  let _menhir_action_200 =
    fun () ->
      (
# 145 "<standard.mly>"
    ( [] )
# 10905 "asllib/Parser.ml"
       : (AST.expr list))
  
  let _menhir_action_201 =
    fun () ->
      let x =
        let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 10914 "asllib/Parser.ml"
         in
        
# 205 "<standard.mly>"
    ( x )
# 10919 "asllib/Parser.ml"
        
      in
      (
# 148 "<standard.mly>"
    ( x )
# 10925 "asllib/Parser.ml"
       : (AST.expr list))
  
  let _menhir_action_202 =
    fun _1_inlined1 ->
      let x =
        let x = _1_inlined1 in
        
# 205 "<standard.mly>"
    ( x )
# 10935 "asllib/Parser.ml"
        
      in
      (
# 148 "<standard.mly>"
    ( x )
# 10941 "asllib/Parser.ml"
       : (AST.expr list))
  
  let _menhir_action_203 =
    fun _1 _1_inlined1 _1_inlined2 _4 _endpos__1_inlined1_ _endpos__1_inlined2_ _startpos__1_ _startpos__1_inlined1_ ->
      let x =
        let desc =
          let stmt =
            let _1 = _1_inlined2 in
            let _1_1 = _1 in
            
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 10954 "asllib/Parser.ml"
             _1_1
          in
          let where = 
# 123 "<standard.mly>"
    ( None )
# 10960 "asllib/Parser.ml"
           in
          let pattern =
            let (_endpos__1_, _startpos__1_, _1) = (_endpos__1_inlined1_, _startpos__1_inlined1_, _1_inlined1) in
            let desc =
              let _1_1 = _1 in
              
# 311 "asllib/Parser.mly"
                                                   Pattern_Any  
# 10969 "asllib/Parser.ml"
               _1_1
            in
            let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
            let _endpos = _endpos_desc_ in
            let _symbolstartpos = _startpos_desc_ in
            
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 10978 "asllib/Parser.ml"
            
          in
          
# 443 "asllib/Parser.mly"
      ( { pattern; where; stmt } )
# 10984 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__1_inlined2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 10993 "asllib/Parser.ml"
        
      in
      (
# 183 "asllib/Parser.mly"
         ( [ x ] )
# 10999 "asllib/Parser.ml"
       : (AST.case_alt list))
  
  let _menhir_action_204 =
    fun _1 _1_inlined1 _1_inlined2 _1_inlined3 _2 _4 _endpos__1_inlined1_ _endpos__1_inlined3_ _startpos__1_ _startpos__1_inlined1_ ->
      let x =
        let desc =
          let stmt =
            let _1 = _1_inlined3 in
            let _1_1 = _1 in
            
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 11012 "asllib/Parser.ml"
             _1_1
          in
          let where =
            let _1 = _1_inlined2 in
            let x = _2 in
            
# 126 "<standard.mly>"
    ( Some x )
# 11021 "asllib/Parser.ml"
            
          in
          let pattern =
            let (_endpos__1_, _startpos__1_, _1) = (_endpos__1_inlined1_, _startpos__1_inlined1_, _1_inlined1) in
            let desc =
              let _1_1 = _1 in
              
# 311 "asllib/Parser.mly"
                                                   Pattern_Any  
# 11031 "asllib/Parser.ml"
               _1_1
            in
            let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
            let _endpos = _endpos_desc_ in
            let _symbolstartpos = _startpos_desc_ in
            
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 11040 "asllib/Parser.ml"
            
          in
          
# 443 "asllib/Parser.mly"
      ( { pattern; where; stmt } )
# 11046 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__1_inlined3_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 11055 "asllib/Parser.ml"
        
      in
      (
# 183 "asllib/Parser.mly"
         ( [ x ] )
# 11061 "asllib/Parser.ml"
       : (AST.case_alt list))
  
  let _menhir_action_205 =
    fun _1 _1_inlined1 _1_inlined2 _1_inlined3 _2 _4 _endpos__1_inlined1_ _endpos__1_inlined2_ _endpos__1_inlined3_ _endpos_desc_ _startpos__1_ _startpos__1_inlined1_ _startpos_desc_ desc ->
      let y =
        let (_endpos__1_, _1) = (_endpos__1_inlined3_, _1_inlined3) in
        let desc =
          let stmt =
            let _1_1 = _1 in
            
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 11074 "asllib/Parser.ml"
             _1_1
          in
          let loc =
            let _endpos = _endpos_desc_ in
            let _symbolstartpos = _startpos_desc_ in
            
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 11083 "asllib/Parser.ml"
            
          in
          
# 447 "asllib/Parser.mly"
  ( { pattern = add_pos_from loc Pattern_All; where = None; stmt } )
# 11089 "asllib/Parser.ml"
          
        in
        let _endpos_desc_ = _endpos__1_ in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 11098 "asllib/Parser.ml"
        
      in
      let x =
        let desc =
          let stmt =
            let _1 = _1_inlined2 in
            let _1_1 = _1 in
            
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 11109 "asllib/Parser.ml"
             _1_1
          in
          let where = 
# 123 "<standard.mly>"
    ( None )
# 11115 "asllib/Parser.ml"
           in
          let pattern =
            let (_endpos__1_, _startpos__1_, _1) = (_endpos__1_inlined1_, _startpos__1_inlined1_, _1_inlined1) in
            let desc =
              let _1_1 = _1 in
              
# 311 "asllib/Parser.mly"
                                                   Pattern_Any  
# 11124 "asllib/Parser.ml"
               _1_1
            in
            let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
            let _endpos = _endpos_desc_ in
            let _symbolstartpos = _startpos_desc_ in
            
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 11133 "asllib/Parser.ml"
            
          in
          
# 443 "asllib/Parser.mly"
      ( { pattern; where; stmt } )
# 11139 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__1_inlined2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 11148 "asllib/Parser.ml"
        
      in
      (
# 184 "asllib/Parser.mly"
              ( [ x; y ] )
# 11154 "asllib/Parser.ml"
       : (AST.case_alt list))
  
  let _menhir_action_206 =
    fun _1 _1_inlined1 _1_inlined2 _1_inlined3 _1_inlined4 _2 _2_inlined1 _4 _endpos__1_inlined1_ _endpos__1_inlined3_ _endpos__1_inlined4_ _endpos_desc_ _startpos__1_ _startpos__1_inlined1_ _startpos_desc_ desc ->
      let y =
        let (_endpos__1_, _1, _2) = (_endpos__1_inlined4_, _1_inlined4, _2_inlined1) in
        let desc =
          let stmt =
            let _1_1 = _1 in
            
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 11167 "asllib/Parser.ml"
             _1_1
          in
          let loc =
            let _endpos = _endpos_desc_ in
            let _symbolstartpos = _startpos_desc_ in
            
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 11176 "asllib/Parser.ml"
            
          in
          
# 447 "asllib/Parser.mly"
  ( { pattern = add_pos_from loc Pattern_All; where = None; stmt } )
# 11182 "asllib/Parser.ml"
          
        in
        let _endpos_desc_ = _endpos__1_ in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 11191 "asllib/Parser.ml"
        
      in
      let x =
        let desc =
          let stmt =
            let _1 = _1_inlined3 in
            let _1_1 = _1 in
            
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 11202 "asllib/Parser.ml"
             _1_1
          in
          let where =
            let _1 = _1_inlined2 in
            let x = _2 in
            
# 126 "<standard.mly>"
    ( Some x )
# 11211 "asllib/Parser.ml"
            
          in
          let pattern =
            let (_endpos__1_, _startpos__1_, _1) = (_endpos__1_inlined1_, _startpos__1_inlined1_, _1_inlined1) in
            let desc =
              let _1_1 = _1 in
              
# 311 "asllib/Parser.mly"
                                                   Pattern_Any  
# 11221 "asllib/Parser.ml"
               _1_1
            in
            let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
            let _endpos = _endpos_desc_ in
            let _symbolstartpos = _startpos_desc_ in
            
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 11230 "asllib/Parser.ml"
            
          in
          
# 443 "asllib/Parser.mly"
      ( { pattern; where; stmt } )
# 11236 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__1_inlined3_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 11245 "asllib/Parser.ml"
        
      in
      (
# 184 "asllib/Parser.mly"
              ( [ x; y ] )
# 11251 "asllib/Parser.ml"
       : (AST.case_alt list))
  
  let _menhir_action_207 =
    fun _1 _1_inlined1 _1_inlined2 _4 _endpos__1_inlined1_ _endpos__1_inlined2_ _startpos__1_ _startpos__1_inlined1_ l ->
      let x =
        let desc =
          let stmt =
            let _1 = _1_inlined2 in
            let _1_1 = _1 in
            
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 11264 "asllib/Parser.ml"
             _1_1
          in
          let where = 
# 123 "<standard.mly>"
    ( None )
# 11270 "asllib/Parser.ml"
           in
          let pattern =
            let (_endpos__1_, _startpos__1_, _1) = (_endpos__1_inlined1_, _startpos__1_inlined1_, _1_inlined1) in
            let desc =
              let _1_1 = _1 in
              
# 311 "asllib/Parser.mly"
                                                   Pattern_Any  
# 11279 "asllib/Parser.ml"
               _1_1
            in
            let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
            let _endpos = _endpos_desc_ in
            let _symbolstartpos = _startpos_desc_ in
            
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 11288 "asllib/Parser.ml"
            
          in
          
# 443 "asllib/Parser.mly"
      ( { pattern; where; stmt } )
# 11294 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__1_inlined2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 11303 "asllib/Parser.ml"
        
      in
      (
# 185 "asllib/Parser.mly"
                                       ( x :: l )
# 11309 "asllib/Parser.ml"
       : (AST.case_alt list))
  
  let _menhir_action_208 =
    fun _1 _1_inlined1 _1_inlined2 _1_inlined3 _2 _4 _endpos__1_inlined1_ _endpos__1_inlined3_ _startpos__1_ _startpos__1_inlined1_ l ->
      let x =
        let desc =
          let stmt =
            let _1 = _1_inlined3 in
            let _1_1 = _1 in
            
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 11322 "asllib/Parser.ml"
             _1_1
          in
          let where =
            let _1 = _1_inlined2 in
            let x = _2 in
            
# 126 "<standard.mly>"
    ( Some x )
# 11331 "asllib/Parser.ml"
            
          in
          let pattern =
            let (_endpos__1_, _startpos__1_, _1) = (_endpos__1_inlined1_, _startpos__1_inlined1_, _1_inlined1) in
            let desc =
              let _1_1 = _1 in
              
# 311 "asllib/Parser.mly"
                                                   Pattern_Any  
# 11341 "asllib/Parser.ml"
               _1_1
            in
            let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
            let _endpos = _endpos_desc_ in
            let _symbolstartpos = _startpos_desc_ in
            
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 11350 "asllib/Parser.ml"
            
          in
          
# 443 "asllib/Parser.mly"
      ( { pattern; where; stmt } )
# 11356 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__1_inlined3_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 11365 "asllib/Parser.ml"
        
      in
      (
# 185 "asllib/Parser.mly"
                                       ( x :: l )
# 11371 "asllib/Parser.ml"
       : (AST.case_alt list))
  
  let _menhir_action_209 =
    fun x ->
      let _2 = 
# 123 "<standard.mly>"
    ( None )
# 11379 "asllib/Parser.ml"
       in
      (
# 154 "asllib/Parser.mly"
                       ( [ x ] )
# 11384 "asllib/Parser.ml"
       : (string list))
  
  let _menhir_action_210 =
    fun x x_inlined1 ->
      let _2 =
        let x = x_inlined1 in
        
# 126 "<standard.mly>"
    ( Some x )
# 11394 "asllib/Parser.ml"
        
      in
      (
# 154 "asllib/Parser.mly"
                       ( [ x ] )
# 11400 "asllib/Parser.ml"
       : (string list))
  
  let _menhir_action_211 =
    fun _2 h t ->
      (
# 155 "asllib/Parser.mly"
                                                 ( h :: t )
# 11408 "asllib/Parser.ml"
       : (string list))
  
  let _menhir_action_212 =
    fun _1_inlined1 x ->
      let _2 = 
# 123 "<standard.mly>"
    ( None )
# 11416 "asllib/Parser.ml"
       in
      let x =
        let s =
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 11424 "asllib/Parser.ml"
          
        in
        
# 347 "asllib/Parser.mly"
                                              ( BitField_Simple (x, s)     )
# 11430 "asllib/Parser.ml"
        
      in
      (
# 154 "asllib/Parser.mly"
                       ( [ x ] )
# 11436 "asllib/Parser.ml"
       : (AST.bitfield list))
  
  let _menhir_action_213 =
    fun _1_inlined1 x x_inlined1 ->
      let _2 =
        let x = x_inlined1 in
        
# 126 "<standard.mly>"
    ( Some x )
# 11446 "asllib/Parser.ml"
        
      in
      let x =
        let s =
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 11455 "asllib/Parser.ml"
          
        in
        
# 347 "asllib/Parser.mly"
                                              ( BitField_Simple (x, s)     )
# 11461 "asllib/Parser.ml"
        
      in
      (
# 154 "asllib/Parser.mly"
                       ( [ x ] )
# 11467 "asllib/Parser.ml"
       : (AST.bitfield list))
  
  let _menhir_action_214 =
    fun _1_inlined1 _1_inlined3 x ->
      let _2 = 
# 123 "<standard.mly>"
    ( None )
# 11475 "asllib/Parser.ml"
       in
      let x =
        let bf =
          let _1_inlined1 = _1_inlined3 in
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 11484 "asllib/Parser.ml"
          
        in
        let s =
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 11492 "asllib/Parser.ml"
          
        in
        
# 348 "asllib/Parser.mly"
                                              ( BitField_Nested (x, s, bf) )
# 11498 "asllib/Parser.ml"
        
      in
      (
# 154 "asllib/Parser.mly"
                       ( [ x ] )
# 11504 "asllib/Parser.ml"
       : (AST.bitfield list))
  
  let _menhir_action_215 =
    fun _1_inlined1 _1_inlined3 x x_inlined1 ->
      let _2 =
        let x = x_inlined1 in
        
# 126 "<standard.mly>"
    ( Some x )
# 11514 "asllib/Parser.ml"
        
      in
      let x =
        let bf =
          let _1_inlined1 = _1_inlined3 in
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 11524 "asllib/Parser.ml"
          
        in
        let s =
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 11532 "asllib/Parser.ml"
          
        in
        
# 348 "asllib/Parser.mly"
                                              ( BitField_Nested (x, s, bf) )
# 11538 "asllib/Parser.ml"
        
      in
      (
# 154 "asllib/Parser.mly"
                       ( [ x ] )
# 11544 "asllib/Parser.ml"
       : (AST.bitfield list))
  
  let _menhir_action_216 =
    fun _1_inlined1 _1_inlined2 _2_inlined1 x ->
      let _2 = 
# 123 "<standard.mly>"
    ( None )
# 11552 "asllib/Parser.ml"
       in
      let x =
        let _2 = _2_inlined1 in
        let ty =
          let _1 = _1_inlined2 in
          _2
        in
        let s =
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 11565 "asllib/Parser.ml"
          
        in
        
# 349 "asllib/Parser.mly"
                                              ( BitField_Type   (x, s, ty) )
# 11571 "asllib/Parser.ml"
        
      in
      (
# 154 "asllib/Parser.mly"
                       ( [ x ] )
# 11577 "asllib/Parser.ml"
       : (AST.bitfield list))
  
  let _menhir_action_217 =
    fun _1_inlined1 _1_inlined2 _2_inlined1 x x_inlined1 ->
      let _2 =
        let x = x_inlined1 in
        
# 126 "<standard.mly>"
    ( Some x )
# 11587 "asllib/Parser.ml"
        
      in
      let x =
        let _2 = _2_inlined1 in
        let ty =
          let _1 = _1_inlined2 in
          _2
        in
        let s =
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 11601 "asllib/Parser.ml"
          
        in
        
# 349 "asllib/Parser.mly"
                                              ( BitField_Type   (x, s, ty) )
# 11607 "asllib/Parser.ml"
        
      in
      (
# 154 "asllib/Parser.mly"
                       ( [ x ] )
# 11613 "asllib/Parser.ml"
       : (AST.bitfield list))
  
  let _menhir_action_218 =
    fun _1_inlined1 _1_inlined2 _2_inlined1 x ->
      let _2 = 
# 123 "<standard.mly>"
    ( None )
# 11621 "asllib/Parser.ml"
       in
      let x =
        let _2 = _2_inlined1 in
        let ty =
          let _1 = _1_inlined2 in
          _2
        in
        let s =
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 11634 "asllib/Parser.ml"
          
        in
        
# 349 "asllib/Parser.mly"
                                              ( BitField_Type   (x, s, ty) )
# 11640 "asllib/Parser.ml"
        
      in
      (
# 154 "asllib/Parser.mly"
                       ( [ x ] )
# 11646 "asllib/Parser.ml"
       : (AST.bitfield list))
  
  let _menhir_action_219 =
    fun _1_inlined1 _1_inlined2 _2_inlined1 x x_inlined1 ->
      let _2 =
        let x = x_inlined1 in
        
# 126 "<standard.mly>"
    ( Some x )
# 11656 "asllib/Parser.ml"
        
      in
      let x =
        let _2 = _2_inlined1 in
        let ty =
          let _1 = _1_inlined2 in
          _2
        in
        let s =
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 11670 "asllib/Parser.ml"
          
        in
        
# 349 "asllib/Parser.mly"
                                              ( BitField_Type   (x, s, ty) )
# 11676 "asllib/Parser.ml"
        
      in
      (
# 154 "asllib/Parser.mly"
                       ( [ x ] )
# 11682 "asllib/Parser.ml"
       : (AST.bitfield list))
  
  let _menhir_action_220 =
    fun _1_inlined1 _2 t x ->
      let h =
        let s =
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 11693 "asllib/Parser.ml"
          
        in
        
# 347 "asllib/Parser.mly"
                                              ( BitField_Simple (x, s)     )
# 11699 "asllib/Parser.ml"
        
      in
      (
# 155 "asllib/Parser.mly"
                                                 ( h :: t )
# 11705 "asllib/Parser.ml"
       : (AST.bitfield list))
  
  let _menhir_action_221 =
    fun _1_inlined1 _1_inlined3 _2 t x ->
      let h =
        let bf =
          let _1_inlined1 = _1_inlined3 in
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 11717 "asllib/Parser.ml"
          
        in
        let s =
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 11725 "asllib/Parser.ml"
          
        in
        
# 348 "asllib/Parser.mly"
                                              ( BitField_Nested (x, s, bf) )
# 11731 "asllib/Parser.ml"
        
      in
      (
# 155 "asllib/Parser.mly"
                                                 ( h :: t )
# 11737 "asllib/Parser.ml"
       : (AST.bitfield list))
  
  let _menhir_action_222 =
    fun _1_inlined1 _1_inlined2 _2 _2_inlined1 t x ->
      let h =
        let _2 = _2_inlined1 in
        let ty =
          let _1 = _1_inlined2 in
          _2
        in
        let s =
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 11753 "asllib/Parser.ml"
          
        in
        
# 349 "asllib/Parser.mly"
                                              ( BitField_Type   (x, s, ty) )
# 11759 "asllib/Parser.ml"
        
      in
      (
# 155 "asllib/Parser.mly"
                                                 ( h :: t )
# 11765 "asllib/Parser.ml"
       : (AST.bitfield list))
  
  let _menhir_action_223 =
    fun _1_inlined1 _1_inlined2 _2 _2_inlined1 t x ->
      let h =
        let _2 = _2_inlined1 in
        let ty =
          let _1 = _1_inlined2 in
          _2
        in
        let s =
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 11781 "asllib/Parser.ml"
          
        in
        
# 349 "asllib/Parser.mly"
                                              ( BitField_Type   (x, s, ty) )
# 11787 "asllib/Parser.ml"
        
      in
      (
# 155 "asllib/Parser.mly"
                                                 ( h :: t )
# 11793 "asllib/Parser.ml"
       : (AST.bitfield list))
  
  let _menhir_action_224 =
    fun _1 _2_inlined1 x ->
      let _2 = 
# 123 "<standard.mly>"
    ( None )
# 11801 "asllib/Parser.ml"
       in
      let x =
        let _2 = _2_inlined1 in
        let y = _2 in
        
# 172 "<standard.mly>"
    ( (x, y) )
# 11809 "asllib/Parser.ml"
        
      in
      (
# 154 "asllib/Parser.mly"
                       ( [ x ] )
# 11815 "asllib/Parser.ml"
       : (AST.field list))
  
  let _menhir_action_225 =
    fun _1 _2_inlined1 x x_inlined1 ->
      let _2 =
        let x = x_inlined1 in
        
# 126 "<standard.mly>"
    ( Some x )
# 11825 "asllib/Parser.ml"
        
      in
      let x =
        let _2 = _2_inlined1 in
        let y = _2 in
        
# 172 "<standard.mly>"
    ( (x, y) )
# 11834 "asllib/Parser.ml"
        
      in
      (
# 154 "asllib/Parser.mly"
                       ( [ x ] )
# 11840 "asllib/Parser.ml"
       : (AST.field list))
  
  let _menhir_action_226 =
    fun _1 _2_inlined1 x ->
      let _2 = 
# 123 "<standard.mly>"
    ( None )
# 11848 "asllib/Parser.ml"
       in
      let x =
        let _2 = _2_inlined1 in
        let y = _2 in
        
# 172 "<standard.mly>"
    ( (x, y) )
# 11856 "asllib/Parser.ml"
        
      in
      (
# 154 "asllib/Parser.mly"
                       ( [ x ] )
# 11862 "asllib/Parser.ml"
       : (AST.field list))
  
  let _menhir_action_227 =
    fun _1 _2_inlined1 x x_inlined1 ->
      let _2 =
        let x = x_inlined1 in
        
# 126 "<standard.mly>"
    ( Some x )
# 11872 "asllib/Parser.ml"
        
      in
      let x =
        let _2 = _2_inlined1 in
        let y = _2 in
        
# 172 "<standard.mly>"
    ( (x, y) )
# 11881 "asllib/Parser.ml"
        
      in
      (
# 154 "asllib/Parser.mly"
                       ( [ x ] )
# 11887 "asllib/Parser.ml"
       : (AST.field list))
  
  let _menhir_action_228 =
    fun _1 _2 _2_inlined1 t x ->
      let h =
        let _2 = _2_inlined1 in
        let y = _2 in
        
# 172 "<standard.mly>"
    ( (x, y) )
# 11898 "asllib/Parser.ml"
        
      in
      (
# 155 "asllib/Parser.mly"
                                                 ( h :: t )
# 11904 "asllib/Parser.ml"
       : (AST.field list))
  
  let _menhir_action_229 =
    fun _1 _2 _2_inlined1 t x ->
      let h =
        let _2 = _2_inlined1 in
        let y = _2 in
        
# 172 "<standard.mly>"
    ( (x, y) )
# 11915 "asllib/Parser.ml"
        
      in
      (
# 155 "asllib/Parser.mly"
                                                 ( h :: t )
# 11921 "asllib/Parser.ml"
       : (AST.field list))
  
  let _menhir_action_230 =
    fun _1 _1_inlined1 _4 ty ->
      let x =
        let stmt_list =
          let _1 = _1_inlined1 in
          let _1_1 = _1 in
          
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 11933 "asllib/Parser.ml"
           _1_1
        in
        let _2 = 
# 123 "<standard.mly>"
    ( None )
# 11939 "asllib/Parser.ml"
         in
        let _2_1 = _2 in
        (_2_1, ty, stmt_list)
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 11947 "asllib/Parser.ml"
       : (AST.catcher list))
  
  let _menhir_action_231 =
    fun _1 _1_inlined1 _4 ty x ->
      let x =
        let stmt_list =
          let _1 = _1_inlined1 in
          let _1_1 = _1 in
          
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 11959 "asllib/Parser.ml"
           _1_1
        in
        let _2 =
          let x = 
# 196 "<standard.mly>"
    ( x )
# 11966 "asllib/Parser.ml"
           in
          
# 126 "<standard.mly>"
    ( Some x )
# 11971 "asllib/Parser.ml"
          
        in
        let _2_1 = _2 in
        (_2_1, ty, stmt_list)
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 11980 "asllib/Parser.ml"
       : (AST.catcher list))
  
  let _menhir_action_232 =
    fun _1 _1_inlined1 _4 ty xs ->
      let x =
        let stmt_list =
          let _1 = _1_inlined1 in
          let _1_1 = _1 in
          
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 11992 "asllib/Parser.ml"
           _1_1
        in
        let _2 = 
# 123 "<standard.mly>"
    ( None )
# 11998 "asllib/Parser.ml"
         in
        let _2_1 = _2 in
        (_2_1, ty, stmt_list)
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 12006 "asllib/Parser.ml"
       : (AST.catcher list))
  
  let _menhir_action_233 =
    fun _1 _1_inlined1 _4 ty x xs ->
      let x =
        let stmt_list =
          let _1 = _1_inlined1 in
          let _1_1 = _1 in
          
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 12018 "asllib/Parser.ml"
           _1_1
        in
        let _2 =
          let x = 
# 196 "<standard.mly>"
    ( x )
# 12025 "asllib/Parser.ml"
           in
          
# 126 "<standard.mly>"
    ( Some x )
# 12030 "asllib/Parser.ml"
          
        in
        let _2_1 = _2 in
        (_2_1, ty, stmt_list)
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 12039 "asllib/Parser.ml"
       : (AST.catcher list))
  
  let _menhir_action_234 =
    fun _1 _1_inlined1 _3 _endpos__2_ _startpos__1_ e s2 ->
      let x =
        let desc =
          let x =
            let s1 =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 12053 "asllib/Parser.ml"
               _1_1
            in
            
# 460 "asllib/Parser.mly"
                                                       S_Cond 
# 12059 "asllib/Parser.ml"
             (e, s1, s2)
          in
          
# 196 "<standard.mly>"
    ( x )
# 12065 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12074 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12080 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_235 =
    fun _1 _1_inlined1 _3 _endpos__2_ _startpos__1_ expr ->
      let x =
        let desc =
          let x =
            let alt = _1_inlined1 in
            
# 461 "asllib/Parser.mly"
                                                       S_Case 
# 12092 "asllib/Parser.ml"
             (expr, alt)
          in
          
# 196 "<standard.mly>"
    ( x )
# 12098 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12107 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12113 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_236 =
    fun _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ expr ->
      let x =
        let desc =
          let x =
            let stmt_list =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 12127 "asllib/Parser.ml"
               _1_1
            in
            let loop_limit = 
# 123 "<standard.mly>"
    ( None )
# 12133 "asllib/Parser.ml"
             in
            
# 462 "asllib/Parser.mly"
                                                       S_While 
# 12138 "asllib/Parser.ml"
             (expr, loop_limit, stmt_list)
          in
          
# 196 "<standard.mly>"
    ( x )
# 12144 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12153 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12159 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_237 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ expr ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let stmt_list =
              let _1 = _1_inlined2 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 12174 "asllib/Parser.ml"
               _1_1
            in
            let loop_limit =
              let _1 = _1_inlined1 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 12183 "asllib/Parser.ml"
              
            in
            
# 462 "asllib/Parser.mly"
                                                       S_While 
# 12189 "asllib/Parser.ml"
             (expr, loop_limit, stmt_list)
          in
          
# 196 "<standard.mly>"
    ( x )
# 12195 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12204 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12210 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_238 =
    fun _1 _1_inlined1 _1_inlined2 _3 _8 _endpos__2_ _startpos__1_ end_e index_name start_e ->
      let x =
        let desc =
          let x =
            let body =
              let _1 = _1_inlined2 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 12224 "asllib/Parser.ml"
               _1_1
            in
            let limit = 
# 123 "<standard.mly>"
    ( None )
# 12230 "asllib/Parser.ml"
             in
            let dir =
              let _1 = _1_inlined1 in
              
# 439 "asllib/Parser.mly"
                       ( AST.Up )
# 12237 "asllib/Parser.ml"
              
            in
            
# 465 "asllib/Parser.mly"
          ( S_For { index_name; start_e; end_e; dir; body; limit } )
# 12243 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 12249 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12258 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12264 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_239 =
    fun _1 _1_inlined1 _1_inlined2 _1_inlined3 _2_inlined1 _3 _8 _endpos__2_ _startpos__1_ end_e index_name start_e ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let body =
              let _1 = _1_inlined3 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 12279 "asllib/Parser.ml"
               _1_1
            in
            let limit =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 12288 "asllib/Parser.ml"
              
            in
            let dir =
              let _1 = _1_inlined1 in
              
# 439 "asllib/Parser.mly"
                       ( AST.Up )
# 12296 "asllib/Parser.ml"
              
            in
            
# 465 "asllib/Parser.mly"
          ( S_For { index_name; start_e; end_e; dir; body; limit } )
# 12302 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 12308 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12317 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12323 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_240 =
    fun _1 _1_inlined1 _1_inlined2 _3 _8 _endpos__2_ _startpos__1_ end_e index_name start_e ->
      let x =
        let desc =
          let x =
            let body =
              let _1 = _1_inlined2 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 12337 "asllib/Parser.ml"
               _1_1
            in
            let limit = 
# 123 "<standard.mly>"
    ( None )
# 12343 "asllib/Parser.ml"
             in
            let dir =
              let _1 = _1_inlined1 in
              
# 439 "asllib/Parser.mly"
                                            ( AST.Down )
# 12350 "asllib/Parser.ml"
              
            in
            
# 465 "asllib/Parser.mly"
          ( S_For { index_name; start_e; end_e; dir; body; limit } )
# 12356 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 12362 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12371 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12377 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_241 =
    fun _1 _1_inlined1 _1_inlined2 _1_inlined3 _2_inlined1 _3 _8 _endpos__2_ _startpos__1_ end_e index_name start_e ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let body =
              let _1 = _1_inlined3 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 12392 "asllib/Parser.ml"
               _1_1
            in
            let limit =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 12401 "asllib/Parser.ml"
              
            in
            let dir =
              let _1 = _1_inlined1 in
              
# 439 "asllib/Parser.mly"
                                            ( AST.Down )
# 12409 "asllib/Parser.ml"
              
            in
            
# 465 "asllib/Parser.mly"
          ( S_For { index_name; start_e; end_e; dir; body; limit } )
# 12415 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 12421 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12430 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12436 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_242 =
    fun _1 _1_inlined1 _3 _endpos__2_ _startpos__1_ c ->
      let x =
        let desc =
          let x =
            let o = 
# 123 "<standard.mly>"
    ( None )
# 12447 "asllib/Parser.ml"
             in
            let s =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 12455 "asllib/Parser.ml"
               _1_1
            in
            
# 466 "asllib/Parser.mly"
                                                                              S_Try  
# 12461 "asllib/Parser.ml"
             (s, c, o)
          in
          
# 196 "<standard.mly>"
    ( x )
# 12467 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12476 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12482 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_243 =
    fun _1 _1_inlined1 _1_inlined2 _1_inlined3 _2_inlined1 _3 _endpos__2_ _startpos__1_ c ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let o =
              let (_1_inlined1, _1) = (_1_inlined3, _1_inlined2) in
              let x =
                let _1 = _1_inlined1 in
                let _1_1 = _1 in
                
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 12499 "asllib/Parser.ml"
                 _1_1
              in
              
# 126 "<standard.mly>"
    ( Some x )
# 12505 "asllib/Parser.ml"
              
            in
            let s =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 12514 "asllib/Parser.ml"
               _1_1
            in
            
# 466 "asllib/Parser.mly"
                                                                              S_Try  
# 12520 "asllib/Parser.ml"
             (s, c, o)
          in
          
# 196 "<standard.mly>"
    ( x )
# 12526 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12535 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12541 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_244 =
    fun _1 _1_inlined2 _3 _5 _endpos__2_ _startpos__1_ cond x ->
      let x =
        let desc =
          let x =
            let body =
              let _1 = _1_inlined2 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 12555 "asllib/Parser.ml"
               _1_1
            in
            let looplimit = 
# 205 "<standard.mly>"
    ( x )
# 12561 "asllib/Parser.ml"
             in
            
# 468 "asllib/Parser.mly"
          ( S_While (cond, Some looplimit, body) )
# 12566 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 12572 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12581 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12587 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_245 =
    fun _1 _endpos__2_ _startpos__1_ ->
      let x =
        let desc =
          let x = 
# 437 "asllib/Parser.mly"
            ( S_Pass )
# 12597 "asllib/Parser.ml"
           in
          
# 196 "<standard.mly>"
    ( x )
# 12602 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12611 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12617 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_246 =
    fun _1 _endpos__2_ _startpos__1_ ->
      let x =
        let desc =
          let x =
            let _2 = 
# 123 "<standard.mly>"
    ( None )
# 12628 "asllib/Parser.ml"
             in
            let _2_1 = _2 in
            
# 472 "asllib/Parser.mly"
                                                               S_Return  
# 12634 "asllib/Parser.ml"
             _2_1
          in
          
# 196 "<standard.mly>"
    ( x )
# 12640 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12649 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12655 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_247 =
    fun _1 _endpos__2_ _startpos__1_ x ->
      let x =
        let desc =
          let x =
            let _2 = 
# 126 "<standard.mly>"
    ( Some x )
# 12666 "asllib/Parser.ml"
             in
            let _2_1 = _2 in
            
# 472 "asllib/Parser.mly"
                                                               S_Return  
# 12672 "asllib/Parser.ml"
             _2_1
          in
          
# 196 "<standard.mly>"
    ( x )
# 12678 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12687 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12693 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_248 =
    fun _endpos__2_ _startpos_name_ name ->
      let x =
        let desc =
          let x =
            let call =
              let args =
                let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 12706 "asllib/Parser.ml"
                 in
                
# 205 "<standard.mly>"
    ( x )
# 12711 "asllib/Parser.ml"
                
              in
              
# 525 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 12717 "asllib/Parser.ml"
              
            in
            
# 473 "asllib/Parser.mly"
                                                               s_call  
# 12723 "asllib/Parser.ml"
             call
          in
          
# 196 "<standard.mly>"
    ( x )
# 12729 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12738 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12744 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_249 =
    fun _1_inlined1 _endpos__2_ _startpos_name_ name ->
      let x =
        let desc =
          let x =
            let call =
              let args =
                let x = _1_inlined1 in
                
# 205 "<standard.mly>"
    ( x )
# 12758 "asllib/Parser.ml"
                
              in
              
# 525 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 12764 "asllib/Parser.ml"
              
            in
            
# 473 "asllib/Parser.mly"
                                                               s_call  
# 12770 "asllib/Parser.ml"
             call
          in
          
# 196 "<standard.mly>"
    ( x )
# 12776 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12785 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12791 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_250 =
    fun _1_inlined1 _endpos__2_ _startpos_name_ args name ->
      let x =
        let desc =
          let x =
            let call =
              let params =
                let x = _1_inlined1 in
                
# 205 "<standard.mly>"
    ( x )
# 12805 "asllib/Parser.ml"
                
              in
              
# 527 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 12811 "asllib/Parser.ml"
              
            in
            
# 473 "asllib/Parser.mly"
                                                               s_call  
# 12817 "asllib/Parser.ml"
             call
          in
          
# 196 "<standard.mly>"
    ( x )
# 12823 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12832 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12838 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_251 =
    fun _1 _endpos__2_ _startpos__1_ e ->
      let x =
        let desc =
          let x = 
# 474 "asllib/Parser.mly"
                                                               S_Assert  
# 12848 "asllib/Parser.ml"
           e in
          
# 196 "<standard.mly>"
    ( x )
# 12853 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12862 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12868 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_252 =
    fun _1 _3 _endpos__2_ _startpos__1_ decl_item x ->
      let x =
        let desc =
          let x =
            let _4 = 
# 133 "asllib/Parser.mly"
                        Some 
# 12879 "asllib/Parser.ml"
             x in
            let local_decl_keyword = 
# 425 "asllib/Parser.mly"
                ( LDK_Let       )
# 12884 "asllib/Parser.ml"
             in
            let _4_1 = _4 in
            
# 475 "asllib/Parser.mly"
                                                               S_Decl    
# 12890 "asllib/Parser.ml"
             (local_decl_keyword, decl_item, _4_1)
          in
          
# 196 "<standard.mly>"
    ( x )
# 12896 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12905 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12911 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_253 =
    fun _1 _3 _endpos__2_ _startpos__1_ decl_item x ->
      let x =
        let desc =
          let x =
            let _4 = 
# 133 "asllib/Parser.mly"
                        Some 
# 12922 "asllib/Parser.ml"
             x in
            let local_decl_keyword = 
# 426 "asllib/Parser.mly"
                ( LDK_Constant  )
# 12927 "asllib/Parser.ml"
             in
            let _4_1 = _4 in
            
# 475 "asllib/Parser.mly"
                                                               S_Decl    
# 12933 "asllib/Parser.ml"
             (local_decl_keyword, decl_item, _4_1)
          in
          
# 196 "<standard.mly>"
    ( x )
# 12939 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12948 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12954 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_254 =
    fun _2_inlined1 _endpos__2_ _startpos_le_ e le ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            
# 476 "asllib/Parser.mly"
                                                               S_Assign  
# 12966 "asllib/Parser.ml"
             (le, e)
          in
          
# 196 "<standard.mly>"
    ( x )
# 12972 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_le_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 12981 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 12987 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_255 =
    fun _2_inlined1 _endpos__2_ _endpos__3_ _startpos_name_ name rhs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let desc =
                let args =
                  let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 13002 "asllib/Parser.ml"
                   in
                  
# 205 "<standard.mly>"
    ( x )
# 13007 "asllib/Parser.ml"
                  
                in
                
# 525 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 13013 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13022 "asllib/Parser.ml"
              
            in
            
# 478 "asllib/Parser.mly"
        ( desugar_setter call [] rhs )
# 13028 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 13034 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13043 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 13049 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_256 =
    fun _1_inlined1 _2_inlined1 _endpos__2_ _endpos__3_ _startpos_name_ name rhs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let desc =
                let args =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 13065 "asllib/Parser.ml"
                  
                in
                
# 525 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 13071 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13080 "asllib/Parser.ml"
              
            in
            
# 478 "asllib/Parser.mly"
        ( desugar_setter call [] rhs )
# 13086 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 13092 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13101 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 13107 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_257 =
    fun _1_inlined1 _2_inlined1 _endpos__2_ _endpos_args_ _startpos_name_ args name rhs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let desc =
                let params =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 13123 "asllib/Parser.ml"
                  
                in
                
# 527 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 13129 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13138 "asllib/Parser.ml"
              
            in
            
# 478 "asllib/Parser.mly"
        ( desugar_setter call [] rhs )
# 13144 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 13150 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13159 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 13165 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_258 =
    fun _2_inlined1 _4 _endpos__2_ _endpos__3_ _startpos_name_ fld name rhs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let desc =
                let args =
                  let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 13180 "asllib/Parser.ml"
                   in
                  
# 205 "<standard.mly>"
    ( x )
# 13185 "asllib/Parser.ml"
                  
                in
                
# 525 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 13191 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13200 "asllib/Parser.ml"
              
            in
            
# 480 "asllib/Parser.mly"
        ( desugar_setter call [fld] rhs )
# 13206 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 13212 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13221 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 13227 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_259 =
    fun _1_inlined1 _2_inlined1 _4 _endpos__2_ _endpos__3_ _startpos_name_ fld name rhs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let desc =
                let args =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 13243 "asllib/Parser.ml"
                  
                in
                
# 525 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 13249 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13258 "asllib/Parser.ml"
              
            in
            
# 480 "asllib/Parser.mly"
        ( desugar_setter call [fld] rhs )
# 13264 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 13270 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13279 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 13285 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_260 =
    fun _1_inlined1 _2_inlined1 _4 _endpos__2_ _endpos_args_ _startpos_name_ args fld name rhs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let desc =
                let params =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 13301 "asllib/Parser.ml"
                  
                in
                
# 527 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 13307 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13316 "asllib/Parser.ml"
              
            in
            
# 480 "asllib/Parser.mly"
        ( desugar_setter call [fld] rhs )
# 13322 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 13328 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13337 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 13343 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_261 =
    fun _1_inlined2 _2_inlined1 _2_inlined2 _4 _endpos__2_ _endpos__3_ _startpos_name_ name rhs x ->
      let x =
        let desc =
          let x =
            let (_2_inlined1, _2) = (_2_inlined2, _2_inlined1) in
            let flds =
              let (_1_inlined1, _2) = (_1_inlined2, _2_inlined1) in
              let x =
                let _1 = _1_inlined1 in
                let li = _1 in
                
# 164 "asllib/Parser.mly"
                                           ( x :: li )
# 13360 "asllib/Parser.ml"
                
              in
              
# 205 "<standard.mly>"
    ( x )
# 13366 "asllib/Parser.ml"
              
            in
            let call =
              let desc =
                let args =
                  let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 13375 "asllib/Parser.ml"
                   in
                  
# 205 "<standard.mly>"
    ( x )
# 13380 "asllib/Parser.ml"
                  
                in
                
# 525 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 13386 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13395 "asllib/Parser.ml"
              
            in
            
# 482 "asllib/Parser.mly"
        ( desugar_setter call flds rhs )
# 13401 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 13407 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13416 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 13422 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_262 =
    fun _1_inlined1 _1_inlined3 _2_inlined1 _2_inlined2 _4 _endpos__2_ _endpos__3_ _startpos_name_ name rhs x ->
      let x =
        let desc =
          let x =
            let (_2_inlined1, _2) = (_2_inlined2, _2_inlined1) in
            let flds =
              let (_1_inlined1, _2) = (_1_inlined3, _2_inlined1) in
              let x =
                let _1 = _1_inlined1 in
                let li = _1 in
                
# 164 "asllib/Parser.mly"
                                           ( x :: li )
# 13439 "asllib/Parser.ml"
                
              in
              
# 205 "<standard.mly>"
    ( x )
# 13445 "asllib/Parser.ml"
              
            in
            let call =
              let desc =
                let args =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 13455 "asllib/Parser.ml"
                  
                in
                
# 525 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 13461 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13470 "asllib/Parser.ml"
              
            in
            
# 482 "asllib/Parser.mly"
        ( desugar_setter call flds rhs )
# 13476 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 13482 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13491 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 13497 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_263 =
    fun _1_inlined1 _1_inlined3 _2_inlined1 _2_inlined2 _4 _endpos__2_ _endpos_args_ _startpos_name_ args name rhs x ->
      let x =
        let desc =
          let x =
            let (_2_inlined1, _2) = (_2_inlined2, _2_inlined1) in
            let flds =
              let (_1_inlined1, _2) = (_1_inlined3, _2_inlined1) in
              let x =
                let _1 = _1_inlined1 in
                let li = _1 in
                
# 164 "asllib/Parser.mly"
                                           ( x :: li )
# 13514 "asllib/Parser.ml"
                
              in
              
# 205 "<standard.mly>"
    ( x )
# 13520 "asllib/Parser.ml"
              
            in
            let call =
              let desc =
                let params =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 13530 "asllib/Parser.ml"
                  
                in
                
# 527 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 13536 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13545 "asllib/Parser.ml"
              
            in
            
# 482 "asllib/Parser.mly"
        ( desugar_setter call flds rhs )
# 13551 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 13557 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13566 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 13572 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_264 =
    fun _1 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_endpos__3_inlined1_, _3) = (_endpos__3_inlined2_, _3_inlined1) in
              let desc =
                let args =
                  let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 13588 "asllib/Parser.ml"
                   in
                  
# 205 "<standard.mly>"
    ( x )
# 13593 "asllib/Parser.ml"
                  
                in
                
# 530 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 13599 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13608 "asllib/Parser.ml"
              
            in
            let ldk = 
# 425 "asllib/Parser.mly"
                ( LDK_Let       )
# 13614 "asllib/Parser.ml"
             in
            
# 484 "asllib/Parser.mly"
        ( desugar_elided_parameter ldk lhs call)
# 13619 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 13625 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13634 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 13640 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_265 =
    fun _1 _1_inlined2 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_endpos__3_inlined1_, _1_inlined1, _3) = (_endpos__3_inlined2_, _1_inlined2, _3_inlined1) in
              let desc =
                let args =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 13657 "asllib/Parser.ml"
                  
                in
                
# 530 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 13663 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13672 "asllib/Parser.ml"
              
            in
            let ldk = 
# 425 "asllib/Parser.mly"
                ( LDK_Let       )
# 13678 "asllib/Parser.ml"
             in
            
# 484 "asllib/Parser.mly"
        ( desugar_elided_parameter ldk lhs call)
# 13683 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 13689 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13698 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 13704 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_266 =
    fun _1 _1_inlined1 _2_inlined1 _3 _3_inlined1 _5 _endpos__2_ _endpos_args_ _startpos__1_ _startpos_name_ args lhs name ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_1, _3) = (_1_inlined1, _3_inlined1) in
              let desc =
                let params = _1 in
                
# 532 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 13720 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13729 "asllib/Parser.ml"
              
            in
            let ldk = 
# 425 "asllib/Parser.mly"
                ( LDK_Let       )
# 13735 "asllib/Parser.ml"
             in
            
# 484 "asllib/Parser.mly"
        ( desugar_elided_parameter ldk lhs call)
# 13740 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 13746 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13755 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 13761 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_267 =
    fun _1 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_endpos__3_inlined1_, _3) = (_endpos__3_inlined2_, _3_inlined1) in
              let desc =
                let args =
                  let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 13777 "asllib/Parser.ml"
                   in
                  
# 205 "<standard.mly>"
    ( x )
# 13782 "asllib/Parser.ml"
                  
                in
                
# 530 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 13788 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13797 "asllib/Parser.ml"
              
            in
            let ldk = 
# 426 "asllib/Parser.mly"
                ( LDK_Constant  )
# 13803 "asllib/Parser.ml"
             in
            
# 484 "asllib/Parser.mly"
        ( desugar_elided_parameter ldk lhs call)
# 13808 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 13814 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13823 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 13829 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_268 =
    fun _1 _1_inlined2 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_endpos__3_inlined1_, _1_inlined1, _3) = (_endpos__3_inlined2_, _1_inlined2, _3_inlined1) in
              let desc =
                let args =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 13846 "asllib/Parser.ml"
                  
                in
                
# 530 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 13852 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13861 "asllib/Parser.ml"
              
            in
            let ldk = 
# 426 "asllib/Parser.mly"
                ( LDK_Constant  )
# 13867 "asllib/Parser.ml"
             in
            
# 484 "asllib/Parser.mly"
        ( desugar_elided_parameter ldk lhs call)
# 13872 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 13878 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13887 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 13893 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_269 =
    fun _1 _1_inlined1 _2_inlined1 _3 _3_inlined1 _5 _endpos__2_ _endpos_args_ _startpos__1_ _startpos_name_ args lhs name ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_1, _3) = (_1_inlined1, _3_inlined1) in
              let desc =
                let params = _1 in
                
# 532 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 13909 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13918 "asllib/Parser.ml"
              
            in
            let ldk = 
# 426 "asllib/Parser.mly"
                ( LDK_Constant  )
# 13924 "asllib/Parser.ml"
             in
            
# 484 "asllib/Parser.mly"
        ( desugar_elided_parameter ldk lhs call)
# 13929 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 13935 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13944 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 13950 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_270 =
    fun _1 _endpos__2_ _startpos__1_ ldi ->
      let x =
        let desc =
          let x =
            let e = 
# 123 "<standard.mly>"
    ( None )
# 13961 "asllib/Parser.ml"
             in
            
# 485 "asllib/Parser.mly"
                                                             ( S_Decl (LDK_Var, ldi, e) )
# 13966 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 13972 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 13981 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 13987 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_271 =
    fun _1 _1_inlined1 _2_inlined1 _endpos__2_ _startpos__1_ ldi ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let e =
              let _1 = _1_inlined1 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 14002 "asllib/Parser.ml"
              
            in
            
# 485 "asllib/Parser.mly"
                                                             ( S_Decl (LDK_Var, ldi, e) )
# 14008 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 14014 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14023 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 14029 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_272 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _2_inlined2 _endpos__2_ _startpos__1_ x ->
      let x =
        let desc =
          let x =
            let (_2_inlined1, _2) = (_2_inlined2, _2_inlined1) in
            let as_ty =
              let (_2, _1) = (_2_inlined1, _1_inlined2) in
              _2
            in
            let _2 =
              let _1 = _1_inlined1 in
              let li = _1 in
              
# 164 "asllib/Parser.mly"
                                           ( x :: li )
# 14048 "asllib/Parser.ml"
              
            in
            let _2_1 = _2 in
            
# 486 "asllib/Parser.mly"
                                                               make_ldi_vars  
# 14055 "asllib/Parser.ml"
             (_2_1, as_ty)
          in
          
# 196 "<standard.mly>"
    ( x )
# 14061 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14070 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 14076 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_273 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _2_inlined2 _endpos__2_ _startpos__1_ x ->
      let x =
        let desc =
          let x =
            let (_2_inlined1, _2) = (_2_inlined2, _2_inlined1) in
            let as_ty =
              let (_2, _1) = (_2_inlined1, _1_inlined2) in
              _2
            in
            let _2 =
              let _1 = _1_inlined1 in
              let li = _1 in
              
# 164 "asllib/Parser.mly"
                                           ( x :: li )
# 14095 "asllib/Parser.ml"
              
            in
            let _2_1 = _2 in
            
# 486 "asllib/Parser.mly"
                                                               make_ldi_vars  
# 14102 "asllib/Parser.ml"
             (_2_1, as_ty)
          in
          
# 196 "<standard.mly>"
    ( x )
# 14108 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14117 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 14123 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_274 =
    fun _1 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_endpos__3_inlined1_, _3) = (_endpos__3_inlined2_, _3_inlined1) in
              let desc =
                let args =
                  let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 14139 "asllib/Parser.ml"
                   in
                  
# 205 "<standard.mly>"
    ( x )
# 14144 "asllib/Parser.ml"
                  
                in
                
# 530 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 14150 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14159 "asllib/Parser.ml"
              
            in
            
# 488 "asllib/Parser.mly"
        ( desugar_elided_parameter LDK_Var lhs call)
# 14165 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 14171 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14180 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 14186 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_275 =
    fun _1 _1_inlined2 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_endpos__3_inlined1_, _1_inlined1, _3) = (_endpos__3_inlined2_, _1_inlined2, _3_inlined1) in
              let desc =
                let args =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 14203 "asllib/Parser.ml"
                  
                in
                
# 530 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 14209 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14218 "asllib/Parser.ml"
              
            in
            
# 488 "asllib/Parser.mly"
        ( desugar_elided_parameter LDK_Var lhs call)
# 14224 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 14230 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14239 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 14245 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_276 =
    fun _1 _1_inlined1 _2_inlined1 _3 _3_inlined1 _5 _endpos__2_ _endpos_args_ _startpos__1_ _startpos_name_ args lhs name ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_1, _3) = (_1_inlined1, _3_inlined1) in
              let desc =
                let params = _1 in
                
# 532 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 14261 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14270 "asllib/Parser.ml"
              
            in
            
# 488 "asllib/Parser.mly"
        ( desugar_elided_parameter LDK_Var lhs call)
# 14276 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 14282 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14291 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 14297 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_277 =
    fun _1 _endpos__2_ _startpos__1_ ->
      let x =
        let desc =
          let x =
            let args =
              let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 14309 "asllib/Parser.ml"
               in
              
# 205 "<standard.mly>"
    ( x )
# 14314 "asllib/Parser.ml"
              
            in
            
# 489 "asllib/Parser.mly"
                                                             ( S_Print { args; debug = false } )
# 14320 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 14326 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14335 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 14341 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_278 =
    fun _1 _1_inlined2 _endpos__2_ _startpos__1_ ->
      let x =
        let desc =
          let x =
            let args =
              let _1_inlined1 = _1_inlined2 in
              let x = _1_inlined1 in
              
# 205 "<standard.mly>"
    ( x )
# 14355 "asllib/Parser.ml"
              
            in
            
# 489 "asllib/Parser.mly"
                                                             ( S_Print { args; debug = false } )
# 14361 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 14367 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14376 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 14382 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_279 =
    fun _1 _endpos__2_ _startpos__1_ ->
      let x =
        let desc =
          let x =
            let args =
              let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 14394 "asllib/Parser.ml"
               in
              
# 205 "<standard.mly>"
    ( x )
# 14399 "asllib/Parser.ml"
              
            in
            
# 490 "asllib/Parser.mly"
                                                             ( S_Print { args; debug = true } )
# 14405 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 14411 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14420 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 14426 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_280 =
    fun _1 _1_inlined2 _endpos__2_ _startpos__1_ ->
      let x =
        let desc =
          let x =
            let args =
              let _1_inlined1 = _1_inlined2 in
              let x = _1_inlined1 in
              
# 205 "<standard.mly>"
    ( x )
# 14440 "asllib/Parser.ml"
              
            in
            
# 490 "asllib/Parser.mly"
                                                             ( S_Print { args; debug = true } )
# 14446 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 14452 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14461 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 14467 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_281 =
    fun _1 _2_inlined1 _3 _endpos__2_ _startpos__1_ ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            
# 491 "asllib/Parser.mly"
                                                             ( S_Unreachable )
# 14479 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 14485 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14494 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 14500 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_282 =
    fun _1 _1_inlined1 _3 _endpos__2_ _startpos__1_ expr ->
      let x =
        let desc =
          let x =
            let loop_limit = 
# 123 "<standard.mly>"
    ( None )
# 14511 "asllib/Parser.ml"
             in
            let stmt_list =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 14519 "asllib/Parser.ml"
               _1_1
            in
            
# 492 "asllib/Parser.mly"
                                                               S_Repeat  
# 14525 "asllib/Parser.ml"
             (stmt_list, expr, loop_limit)
          in
          
# 196 "<standard.mly>"
    ( x )
# 14531 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14540 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 14546 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_283 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _3 _endpos__2_ _startpos__1_ expr ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let loop_limit =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 14561 "asllib/Parser.ml"
              
            in
            let stmt_list =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 14570 "asllib/Parser.ml"
               _1_1
            in
            
# 492 "asllib/Parser.mly"
                                                               S_Repeat  
# 14576 "asllib/Parser.ml"
             (stmt_list, expr, loop_limit)
          in
          
# 196 "<standard.mly>"
    ( x )
# 14582 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14591 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 14597 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_284 =
    fun _1 _1_inlined2 _3 _5 _endpos__2_ _startpos__1_ cond x ->
      let x =
        let desc =
          let x =
            let body =
              let _1 = _1_inlined2 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 14611 "asllib/Parser.ml"
               _1_1
            in
            let looplimit = 
# 205 "<standard.mly>"
    ( x )
# 14617 "asllib/Parser.ml"
             in
            
# 494 "asllib/Parser.mly"
          ( S_Repeat (body, cond, Some looplimit) )
# 14622 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 14628 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14637 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 14643 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_285 =
    fun _1 _endpos__2_ _startpos__1_ e ->
      let x =
        let desc =
          let x = 
# 495 "asllib/Parser.mly"
                                                             ( S_Throw (Some (e, None)) )
# 14653 "asllib/Parser.ml"
           in
          
# 196 "<standard.mly>"
    ( x )
# 14658 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14667 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 14673 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_286 =
    fun _1 _endpos__2_ _startpos__1_ ->
      let x =
        let desc =
          let x = 
# 496 "asllib/Parser.mly"
                                                             ( S_Throw None             )
# 14683 "asllib/Parser.ml"
           in
          
# 196 "<standard.mly>"
    ( x )
# 14688 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14697 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 14703 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_287 =
    fun _1 _endpos__2_ _startpos__1_ x ->
      let x =
        let desc =
          let x =
            let e = 
# 161 "asllib/Parser.mly"
                ( [] )
# 14714 "asllib/Parser.ml"
             in
            
# 497 "asllib/Parser.mly"
                                                               S_Pragma  
# 14719 "asllib/Parser.ml"
             (x, e)
          in
          
# 196 "<standard.mly>"
    ( x )
# 14725 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14734 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 14740 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_288 =
    fun _1 _1_inlined1 _endpos__2_ _startpos__1_ x ->
      let x =
        let desc =
          let x =
            let e = _1_inlined1 in
            
# 497 "asllib/Parser.mly"
                                                               S_Pragma  
# 14752 "asllib/Parser.ml"
             (x, e)
          in
          
# 196 "<standard.mly>"
    ( x )
# 14758 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14767 "asllib/Parser.ml"
        
      in
      (
# 228 "<standard.mly>"
    ( [ x ] )
# 14773 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_289 =
    fun _1 _1_inlined1 _3 _endpos__2_ _startpos__1_ e s2 xs ->
      let x =
        let desc =
          let x =
            let s1 =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 14787 "asllib/Parser.ml"
               _1_1
            in
            
# 460 "asllib/Parser.mly"
                                                       S_Cond 
# 14793 "asllib/Parser.ml"
             (e, s1, s2)
          in
          
# 196 "<standard.mly>"
    ( x )
# 14799 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14808 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 14814 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_290 =
    fun _1 _1_inlined1 _3 _endpos__2_ _startpos__1_ expr xs ->
      let x =
        let desc =
          let x =
            let alt = _1_inlined1 in
            
# 461 "asllib/Parser.mly"
                                                       S_Case 
# 14826 "asllib/Parser.ml"
             (expr, alt)
          in
          
# 196 "<standard.mly>"
    ( x )
# 14832 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14841 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 14847 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_291 =
    fun _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ expr xs ->
      let x =
        let desc =
          let x =
            let stmt_list =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 14861 "asllib/Parser.ml"
               _1_1
            in
            let loop_limit = 
# 123 "<standard.mly>"
    ( None )
# 14867 "asllib/Parser.ml"
             in
            
# 462 "asllib/Parser.mly"
                                                       S_While 
# 14872 "asllib/Parser.ml"
             (expr, loop_limit, stmt_list)
          in
          
# 196 "<standard.mly>"
    ( x )
# 14878 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14887 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 14893 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_292 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ expr xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let stmt_list =
              let _1 = _1_inlined2 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 14908 "asllib/Parser.ml"
               _1_1
            in
            let loop_limit =
              let _1 = _1_inlined1 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 14917 "asllib/Parser.ml"
              
            in
            
# 462 "asllib/Parser.mly"
                                                       S_While 
# 14923 "asllib/Parser.ml"
             (expr, loop_limit, stmt_list)
          in
          
# 196 "<standard.mly>"
    ( x )
# 14929 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14938 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 14944 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_293 =
    fun _1 _1_inlined1 _1_inlined2 _3 _8 _endpos__2_ _startpos__1_ end_e index_name start_e xs ->
      let x =
        let desc =
          let x =
            let body =
              let _1 = _1_inlined2 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 14958 "asllib/Parser.ml"
               _1_1
            in
            let limit = 
# 123 "<standard.mly>"
    ( None )
# 14964 "asllib/Parser.ml"
             in
            let dir =
              let _1 = _1_inlined1 in
              
# 439 "asllib/Parser.mly"
                       ( AST.Up )
# 14971 "asllib/Parser.ml"
              
            in
            
# 465 "asllib/Parser.mly"
          ( S_For { index_name; start_e; end_e; dir; body; limit } )
# 14977 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 14983 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 14992 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 14998 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_294 =
    fun _1 _1_inlined1 _1_inlined2 _1_inlined3 _2_inlined1 _3 _8 _endpos__2_ _startpos__1_ end_e index_name start_e xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let body =
              let _1 = _1_inlined3 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 15013 "asllib/Parser.ml"
               _1_1
            in
            let limit =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 15022 "asllib/Parser.ml"
              
            in
            let dir =
              let _1 = _1_inlined1 in
              
# 439 "asllib/Parser.mly"
                       ( AST.Up )
# 15030 "asllib/Parser.ml"
              
            in
            
# 465 "asllib/Parser.mly"
          ( S_For { index_name; start_e; end_e; dir; body; limit } )
# 15036 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 15042 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15051 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15057 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_295 =
    fun _1 _1_inlined1 _1_inlined2 _3 _8 _endpos__2_ _startpos__1_ end_e index_name start_e xs ->
      let x =
        let desc =
          let x =
            let body =
              let _1 = _1_inlined2 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 15071 "asllib/Parser.ml"
               _1_1
            in
            let limit = 
# 123 "<standard.mly>"
    ( None )
# 15077 "asllib/Parser.ml"
             in
            let dir =
              let _1 = _1_inlined1 in
              
# 439 "asllib/Parser.mly"
                                            ( AST.Down )
# 15084 "asllib/Parser.ml"
              
            in
            
# 465 "asllib/Parser.mly"
          ( S_For { index_name; start_e; end_e; dir; body; limit } )
# 15090 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 15096 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15105 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15111 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_296 =
    fun _1 _1_inlined1 _1_inlined2 _1_inlined3 _2_inlined1 _3 _8 _endpos__2_ _startpos__1_ end_e index_name start_e xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let body =
              let _1 = _1_inlined3 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 15126 "asllib/Parser.ml"
               _1_1
            in
            let limit =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 15135 "asllib/Parser.ml"
              
            in
            let dir =
              let _1 = _1_inlined1 in
              
# 439 "asllib/Parser.mly"
                                            ( AST.Down )
# 15143 "asllib/Parser.ml"
              
            in
            
# 465 "asllib/Parser.mly"
          ( S_For { index_name; start_e; end_e; dir; body; limit } )
# 15149 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 15155 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15164 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15170 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_297 =
    fun _1 _1_inlined1 _3 _endpos__2_ _startpos__1_ c xs ->
      let x =
        let desc =
          let x =
            let o = 
# 123 "<standard.mly>"
    ( None )
# 15181 "asllib/Parser.ml"
             in
            let s =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 15189 "asllib/Parser.ml"
               _1_1
            in
            
# 466 "asllib/Parser.mly"
                                                                              S_Try  
# 15195 "asllib/Parser.ml"
             (s, c, o)
          in
          
# 196 "<standard.mly>"
    ( x )
# 15201 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15210 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15216 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_298 =
    fun _1 _1_inlined1 _1_inlined2 _1_inlined3 _2_inlined1 _3 _endpos__2_ _startpos__1_ c xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let o =
              let (_1_inlined1, _1) = (_1_inlined3, _1_inlined2) in
              let x =
                let _1 = _1_inlined1 in
                let _1_1 = _1 in
                
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 15233 "asllib/Parser.ml"
                 _1_1
              in
              
# 126 "<standard.mly>"
    ( Some x )
# 15239 "asllib/Parser.ml"
              
            in
            let s =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 15248 "asllib/Parser.ml"
               _1_1
            in
            
# 466 "asllib/Parser.mly"
                                                                              S_Try  
# 15254 "asllib/Parser.ml"
             (s, c, o)
          in
          
# 196 "<standard.mly>"
    ( x )
# 15260 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15269 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15275 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_299 =
    fun _1 _1_inlined2 _3 _5 _endpos__2_ _startpos__1_ cond x xs ->
      let x =
        let desc =
          let x =
            let body =
              let _1 = _1_inlined2 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 15289 "asllib/Parser.ml"
               _1_1
            in
            let looplimit = 
# 205 "<standard.mly>"
    ( x )
# 15295 "asllib/Parser.ml"
             in
            
# 468 "asllib/Parser.mly"
          ( S_While (cond, Some looplimit, body) )
# 15300 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 15306 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15315 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15321 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_300 =
    fun _1 _endpos__2_ _startpos__1_ xs ->
      let x =
        let desc =
          let x = 
# 437 "asllib/Parser.mly"
            ( S_Pass )
# 15331 "asllib/Parser.ml"
           in
          
# 196 "<standard.mly>"
    ( x )
# 15336 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15345 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15351 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_301 =
    fun _1 _endpos__2_ _startpos__1_ xs ->
      let x =
        let desc =
          let x =
            let _2 = 
# 123 "<standard.mly>"
    ( None )
# 15362 "asllib/Parser.ml"
             in
            let _2_1 = _2 in
            
# 472 "asllib/Parser.mly"
                                                               S_Return  
# 15368 "asllib/Parser.ml"
             _2_1
          in
          
# 196 "<standard.mly>"
    ( x )
# 15374 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15383 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15389 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_302 =
    fun _1 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let _2 = 
# 126 "<standard.mly>"
    ( Some x )
# 15400 "asllib/Parser.ml"
             in
            let _2_1 = _2 in
            
# 472 "asllib/Parser.mly"
                                                               S_Return  
# 15406 "asllib/Parser.ml"
             _2_1
          in
          
# 196 "<standard.mly>"
    ( x )
# 15412 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15421 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15427 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_303 =
    fun _endpos__2_ _startpos_name_ name xs ->
      let x =
        let desc =
          let x =
            let call =
              let args =
                let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 15440 "asllib/Parser.ml"
                 in
                
# 205 "<standard.mly>"
    ( x )
# 15445 "asllib/Parser.ml"
                
              in
              
# 525 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 15451 "asllib/Parser.ml"
              
            in
            
# 473 "asllib/Parser.mly"
                                                               s_call  
# 15457 "asllib/Parser.ml"
             call
          in
          
# 196 "<standard.mly>"
    ( x )
# 15463 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15472 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15478 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_304 =
    fun _1_inlined1 _endpos__2_ _startpos_name_ name xs ->
      let x =
        let desc =
          let x =
            let call =
              let args =
                let x = _1_inlined1 in
                
# 205 "<standard.mly>"
    ( x )
# 15492 "asllib/Parser.ml"
                
              in
              
# 525 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 15498 "asllib/Parser.ml"
              
            in
            
# 473 "asllib/Parser.mly"
                                                               s_call  
# 15504 "asllib/Parser.ml"
             call
          in
          
# 196 "<standard.mly>"
    ( x )
# 15510 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15519 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15525 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_305 =
    fun _1_inlined1 _endpos__2_ _startpos_name_ args name xs ->
      let x =
        let desc =
          let x =
            let call =
              let params =
                let x = _1_inlined1 in
                
# 205 "<standard.mly>"
    ( x )
# 15539 "asllib/Parser.ml"
                
              in
              
# 527 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 15545 "asllib/Parser.ml"
              
            in
            
# 473 "asllib/Parser.mly"
                                                               s_call  
# 15551 "asllib/Parser.ml"
             call
          in
          
# 196 "<standard.mly>"
    ( x )
# 15557 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15566 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15572 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_306 =
    fun _1 _endpos__2_ _startpos__1_ e xs ->
      let x =
        let desc =
          let x = 
# 474 "asllib/Parser.mly"
                                                               S_Assert  
# 15582 "asllib/Parser.ml"
           e in
          
# 196 "<standard.mly>"
    ( x )
# 15587 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15596 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15602 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_307 =
    fun _1 _3 _endpos__2_ _startpos__1_ decl_item x xs ->
      let x =
        let desc =
          let x =
            let _4 = 
# 133 "asllib/Parser.mly"
                        Some 
# 15613 "asllib/Parser.ml"
             x in
            let local_decl_keyword = 
# 425 "asllib/Parser.mly"
                ( LDK_Let       )
# 15618 "asllib/Parser.ml"
             in
            let _4_1 = _4 in
            
# 475 "asllib/Parser.mly"
                                                               S_Decl    
# 15624 "asllib/Parser.ml"
             (local_decl_keyword, decl_item, _4_1)
          in
          
# 196 "<standard.mly>"
    ( x )
# 15630 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15639 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15645 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_308 =
    fun _1 _3 _endpos__2_ _startpos__1_ decl_item x xs ->
      let x =
        let desc =
          let x =
            let _4 = 
# 133 "asllib/Parser.mly"
                        Some 
# 15656 "asllib/Parser.ml"
             x in
            let local_decl_keyword = 
# 426 "asllib/Parser.mly"
                ( LDK_Constant  )
# 15661 "asllib/Parser.ml"
             in
            let _4_1 = _4 in
            
# 475 "asllib/Parser.mly"
                                                               S_Decl    
# 15667 "asllib/Parser.ml"
             (local_decl_keyword, decl_item, _4_1)
          in
          
# 196 "<standard.mly>"
    ( x )
# 15673 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15682 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15688 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_309 =
    fun _2_inlined1 _endpos__2_ _startpos_le_ e le xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            
# 476 "asllib/Parser.mly"
                                                               S_Assign  
# 15700 "asllib/Parser.ml"
             (le, e)
          in
          
# 196 "<standard.mly>"
    ( x )
# 15706 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_le_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15715 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15721 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_310 =
    fun _2_inlined1 _endpos__2_ _endpos__3_ _startpos_name_ name rhs xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let desc =
                let args =
                  let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 15736 "asllib/Parser.ml"
                   in
                  
# 205 "<standard.mly>"
    ( x )
# 15741 "asllib/Parser.ml"
                  
                in
                
# 525 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 15747 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15756 "asllib/Parser.ml"
              
            in
            
# 478 "asllib/Parser.mly"
        ( desugar_setter call [] rhs )
# 15762 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 15768 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15777 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15783 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_311 =
    fun _1_inlined1 _2_inlined1 _endpos__2_ _endpos__3_ _startpos_name_ name rhs xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let desc =
                let args =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 15799 "asllib/Parser.ml"
                  
                in
                
# 525 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 15805 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15814 "asllib/Parser.ml"
              
            in
            
# 478 "asllib/Parser.mly"
        ( desugar_setter call [] rhs )
# 15820 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 15826 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15835 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15841 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_312 =
    fun _1_inlined1 _2_inlined1 _endpos__2_ _endpos_args_ _startpos_name_ args name rhs xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let desc =
                let params =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 15857 "asllib/Parser.ml"
                  
                in
                
# 527 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 15863 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15872 "asllib/Parser.ml"
              
            in
            
# 478 "asllib/Parser.mly"
        ( desugar_setter call [] rhs )
# 15878 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 15884 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15893 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15899 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_313 =
    fun _2_inlined1 _4 _endpos__2_ _endpos__3_ _startpos_name_ fld name rhs xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let desc =
                let args =
                  let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 15914 "asllib/Parser.ml"
                   in
                  
# 205 "<standard.mly>"
    ( x )
# 15919 "asllib/Parser.ml"
                  
                in
                
# 525 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 15925 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15934 "asllib/Parser.ml"
              
            in
            
# 480 "asllib/Parser.mly"
        ( desugar_setter call [fld] rhs )
# 15940 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 15946 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15955 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 15961 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_314 =
    fun _1_inlined1 _2_inlined1 _4 _endpos__2_ _endpos__3_ _startpos_name_ fld name rhs xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let desc =
                let args =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 15977 "asllib/Parser.ml"
                  
                in
                
# 525 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 15983 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 15992 "asllib/Parser.ml"
              
            in
            
# 480 "asllib/Parser.mly"
        ( desugar_setter call [fld] rhs )
# 15998 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 16004 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16013 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 16019 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_315 =
    fun _1_inlined1 _2_inlined1 _4 _endpos__2_ _endpos_args_ _startpos_name_ args fld name rhs xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let desc =
                let params =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 16035 "asllib/Parser.ml"
                  
                in
                
# 527 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 16041 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16050 "asllib/Parser.ml"
              
            in
            
# 480 "asllib/Parser.mly"
        ( desugar_setter call [fld] rhs )
# 16056 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 16062 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16071 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 16077 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_316 =
    fun _1_inlined2 _2_inlined1 _2_inlined2 _4 _endpos__2_ _endpos__3_ _startpos_name_ name rhs x xs ->
      let x =
        let desc =
          let x =
            let (_2_inlined1, _2) = (_2_inlined2, _2_inlined1) in
            let flds =
              let (_1_inlined1, _2) = (_1_inlined2, _2_inlined1) in
              let x =
                let _1 = _1_inlined1 in
                let li = _1 in
                
# 164 "asllib/Parser.mly"
                                           ( x :: li )
# 16094 "asllib/Parser.ml"
                
              in
              
# 205 "<standard.mly>"
    ( x )
# 16100 "asllib/Parser.ml"
              
            in
            let call =
              let desc =
                let args =
                  let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 16109 "asllib/Parser.ml"
                   in
                  
# 205 "<standard.mly>"
    ( x )
# 16114 "asllib/Parser.ml"
                  
                in
                
# 525 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 16120 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16129 "asllib/Parser.ml"
              
            in
            
# 482 "asllib/Parser.mly"
        ( desugar_setter call flds rhs )
# 16135 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 16141 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16150 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 16156 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_317 =
    fun _1_inlined1 _1_inlined3 _2_inlined1 _2_inlined2 _4 _endpos__2_ _endpos__3_ _startpos_name_ name rhs x xs ->
      let x =
        let desc =
          let x =
            let (_2_inlined1, _2) = (_2_inlined2, _2_inlined1) in
            let flds =
              let (_1_inlined1, _2) = (_1_inlined3, _2_inlined1) in
              let x =
                let _1 = _1_inlined1 in
                let li = _1 in
                
# 164 "asllib/Parser.mly"
                                           ( x :: li )
# 16173 "asllib/Parser.ml"
                
              in
              
# 205 "<standard.mly>"
    ( x )
# 16179 "asllib/Parser.ml"
              
            in
            let call =
              let desc =
                let args =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 16189 "asllib/Parser.ml"
                  
                in
                
# 525 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 16195 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16204 "asllib/Parser.ml"
              
            in
            
# 482 "asllib/Parser.mly"
        ( desugar_setter call flds rhs )
# 16210 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 16216 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16225 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 16231 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_318 =
    fun _1_inlined1 _1_inlined3 _2_inlined1 _2_inlined2 _4 _endpos__2_ _endpos_args_ _startpos_name_ args name rhs x xs ->
      let x =
        let desc =
          let x =
            let (_2_inlined1, _2) = (_2_inlined2, _2_inlined1) in
            let flds =
              let (_1_inlined1, _2) = (_1_inlined3, _2_inlined1) in
              let x =
                let _1 = _1_inlined1 in
                let li = _1 in
                
# 164 "asllib/Parser.mly"
                                           ( x :: li )
# 16248 "asllib/Parser.ml"
                
              in
              
# 205 "<standard.mly>"
    ( x )
# 16254 "asllib/Parser.ml"
              
            in
            let call =
              let desc =
                let params =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 16264 "asllib/Parser.ml"
                  
                in
                
# 527 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 16270 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16279 "asllib/Parser.ml"
              
            in
            
# 482 "asllib/Parser.mly"
        ( desugar_setter call flds rhs )
# 16285 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 16291 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16300 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 16306 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_319 =
    fun _1 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_endpos__3_inlined1_, _3) = (_endpos__3_inlined2_, _3_inlined1) in
              let desc =
                let args =
                  let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 16322 "asllib/Parser.ml"
                   in
                  
# 205 "<standard.mly>"
    ( x )
# 16327 "asllib/Parser.ml"
                  
                in
                
# 530 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 16333 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16342 "asllib/Parser.ml"
              
            in
            let ldk = 
# 425 "asllib/Parser.mly"
                ( LDK_Let       )
# 16348 "asllib/Parser.ml"
             in
            
# 484 "asllib/Parser.mly"
        ( desugar_elided_parameter ldk lhs call)
# 16353 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 16359 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16368 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 16374 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_320 =
    fun _1 _1_inlined2 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_endpos__3_inlined1_, _1_inlined1, _3) = (_endpos__3_inlined2_, _1_inlined2, _3_inlined1) in
              let desc =
                let args =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 16391 "asllib/Parser.ml"
                  
                in
                
# 530 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 16397 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16406 "asllib/Parser.ml"
              
            in
            let ldk = 
# 425 "asllib/Parser.mly"
                ( LDK_Let       )
# 16412 "asllib/Parser.ml"
             in
            
# 484 "asllib/Parser.mly"
        ( desugar_elided_parameter ldk lhs call)
# 16417 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 16423 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16432 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 16438 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_321 =
    fun _1 _1_inlined1 _2_inlined1 _3 _3_inlined1 _5 _endpos__2_ _endpos_args_ _startpos__1_ _startpos_name_ args lhs name xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_1, _3) = (_1_inlined1, _3_inlined1) in
              let desc =
                let params = _1 in
                
# 532 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 16454 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16463 "asllib/Parser.ml"
              
            in
            let ldk = 
# 425 "asllib/Parser.mly"
                ( LDK_Let       )
# 16469 "asllib/Parser.ml"
             in
            
# 484 "asllib/Parser.mly"
        ( desugar_elided_parameter ldk lhs call)
# 16474 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 16480 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16489 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 16495 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_322 =
    fun _1 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_endpos__3_inlined1_, _3) = (_endpos__3_inlined2_, _3_inlined1) in
              let desc =
                let args =
                  let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 16511 "asllib/Parser.ml"
                   in
                  
# 205 "<standard.mly>"
    ( x )
# 16516 "asllib/Parser.ml"
                  
                in
                
# 530 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 16522 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16531 "asllib/Parser.ml"
              
            in
            let ldk = 
# 426 "asllib/Parser.mly"
                ( LDK_Constant  )
# 16537 "asllib/Parser.ml"
             in
            
# 484 "asllib/Parser.mly"
        ( desugar_elided_parameter ldk lhs call)
# 16542 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 16548 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16557 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 16563 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_323 =
    fun _1 _1_inlined2 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_endpos__3_inlined1_, _1_inlined1, _3) = (_endpos__3_inlined2_, _1_inlined2, _3_inlined1) in
              let desc =
                let args =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 16580 "asllib/Parser.ml"
                  
                in
                
# 530 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 16586 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16595 "asllib/Parser.ml"
              
            in
            let ldk = 
# 426 "asllib/Parser.mly"
                ( LDK_Constant  )
# 16601 "asllib/Parser.ml"
             in
            
# 484 "asllib/Parser.mly"
        ( desugar_elided_parameter ldk lhs call)
# 16606 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 16612 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16621 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 16627 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_324 =
    fun _1 _1_inlined1 _2_inlined1 _3 _3_inlined1 _5 _endpos__2_ _endpos_args_ _startpos__1_ _startpos_name_ args lhs name xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_1, _3) = (_1_inlined1, _3_inlined1) in
              let desc =
                let params = _1 in
                
# 532 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 16643 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16652 "asllib/Parser.ml"
              
            in
            let ldk = 
# 426 "asllib/Parser.mly"
                ( LDK_Constant  )
# 16658 "asllib/Parser.ml"
             in
            
# 484 "asllib/Parser.mly"
        ( desugar_elided_parameter ldk lhs call)
# 16663 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 16669 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16678 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 16684 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_325 =
    fun _1 _endpos__2_ _startpos__1_ ldi xs ->
      let x =
        let desc =
          let x =
            let e = 
# 123 "<standard.mly>"
    ( None )
# 16695 "asllib/Parser.ml"
             in
            
# 485 "asllib/Parser.mly"
                                                             ( S_Decl (LDK_Var, ldi, e) )
# 16700 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 16706 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16715 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 16721 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_326 =
    fun _1 _1_inlined1 _2_inlined1 _endpos__2_ _startpos__1_ ldi xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let e =
              let _1 = _1_inlined1 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 16736 "asllib/Parser.ml"
              
            in
            
# 485 "asllib/Parser.mly"
                                                             ( S_Decl (LDK_Var, ldi, e) )
# 16742 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 16748 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16757 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 16763 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_327 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _2_inlined2 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let (_2_inlined1, _2) = (_2_inlined2, _2_inlined1) in
            let as_ty =
              let (_2, _1) = (_2_inlined1, _1_inlined2) in
              _2
            in
            let _2 =
              let _1 = _1_inlined1 in
              let li = _1 in
              
# 164 "asllib/Parser.mly"
                                           ( x :: li )
# 16782 "asllib/Parser.ml"
              
            in
            let _2_1 = _2 in
            
# 486 "asllib/Parser.mly"
                                                               make_ldi_vars  
# 16789 "asllib/Parser.ml"
             (_2_1, as_ty)
          in
          
# 196 "<standard.mly>"
    ( x )
# 16795 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16804 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 16810 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_328 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _2_inlined2 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let (_2_inlined1, _2) = (_2_inlined2, _2_inlined1) in
            let as_ty =
              let (_2, _1) = (_2_inlined1, _1_inlined2) in
              _2
            in
            let _2 =
              let _1 = _1_inlined1 in
              let li = _1 in
              
# 164 "asllib/Parser.mly"
                                           ( x :: li )
# 16829 "asllib/Parser.ml"
              
            in
            let _2_1 = _2 in
            
# 486 "asllib/Parser.mly"
                                                               make_ldi_vars  
# 16836 "asllib/Parser.ml"
             (_2_1, as_ty)
          in
          
# 196 "<standard.mly>"
    ( x )
# 16842 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16851 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 16857 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_329 =
    fun _1 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_endpos__3_inlined1_, _3) = (_endpos__3_inlined2_, _3_inlined1) in
              let desc =
                let args =
                  let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 16873 "asllib/Parser.ml"
                   in
                  
# 205 "<standard.mly>"
    ( x )
# 16878 "asllib/Parser.ml"
                  
                in
                
# 530 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 16884 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16893 "asllib/Parser.ml"
              
            in
            
# 488 "asllib/Parser.mly"
        ( desugar_elided_parameter LDK_Var lhs call)
# 16899 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 16905 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16914 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 16920 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_330 =
    fun _1 _1_inlined2 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_endpos__3_inlined1_, _1_inlined1, _3) = (_endpos__3_inlined2_, _1_inlined2, _3_inlined1) in
              let desc =
                let args =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 16937 "asllib/Parser.ml"
                  
                in
                
# 530 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 16943 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16952 "asllib/Parser.ml"
              
            in
            
# 488 "asllib/Parser.mly"
        ( desugar_elided_parameter LDK_Var lhs call)
# 16958 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 16964 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 16973 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 16979 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_331 =
    fun _1 _1_inlined1 _2_inlined1 _3 _3_inlined1 _5 _endpos__2_ _endpos_args_ _startpos__1_ _startpos_name_ args lhs name xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_1, _3) = (_1_inlined1, _3_inlined1) in
              let desc =
                let params = _1 in
                
# 532 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 16995 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17004 "asllib/Parser.ml"
              
            in
            
# 488 "asllib/Parser.mly"
        ( desugar_elided_parameter LDK_Var lhs call)
# 17010 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 17016 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17025 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 17031 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_332 =
    fun _1 _endpos__2_ _startpos__1_ xs ->
      let x =
        let desc =
          let x =
            let args =
              let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 17043 "asllib/Parser.ml"
               in
              
# 205 "<standard.mly>"
    ( x )
# 17048 "asllib/Parser.ml"
              
            in
            
# 489 "asllib/Parser.mly"
                                                             ( S_Print { args; debug = false } )
# 17054 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 17060 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17069 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 17075 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_333 =
    fun _1 _1_inlined2 _endpos__2_ _startpos__1_ xs ->
      let x =
        let desc =
          let x =
            let args =
              let _1_inlined1 = _1_inlined2 in
              let x = _1_inlined1 in
              
# 205 "<standard.mly>"
    ( x )
# 17089 "asllib/Parser.ml"
              
            in
            
# 489 "asllib/Parser.mly"
                                                             ( S_Print { args; debug = false } )
# 17095 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 17101 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17110 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 17116 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_334 =
    fun _1 _endpos__2_ _startpos__1_ xs ->
      let x =
        let desc =
          let x =
            let args =
              let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 17128 "asllib/Parser.ml"
               in
              
# 205 "<standard.mly>"
    ( x )
# 17133 "asllib/Parser.ml"
              
            in
            
# 490 "asllib/Parser.mly"
                                                             ( S_Print { args; debug = true } )
# 17139 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 17145 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17154 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 17160 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_335 =
    fun _1 _1_inlined2 _endpos__2_ _startpos__1_ xs ->
      let x =
        let desc =
          let x =
            let args =
              let _1_inlined1 = _1_inlined2 in
              let x = _1_inlined1 in
              
# 205 "<standard.mly>"
    ( x )
# 17174 "asllib/Parser.ml"
              
            in
            
# 490 "asllib/Parser.mly"
                                                             ( S_Print { args; debug = true } )
# 17180 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 17186 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17195 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 17201 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_336 =
    fun _1 _2_inlined1 _3 _endpos__2_ _startpos__1_ xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            
# 491 "asllib/Parser.mly"
                                                             ( S_Unreachable )
# 17213 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 17219 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17228 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 17234 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_337 =
    fun _1 _1_inlined1 _3 _endpos__2_ _startpos__1_ expr xs ->
      let x =
        let desc =
          let x =
            let loop_limit = 
# 123 "<standard.mly>"
    ( None )
# 17245 "asllib/Parser.ml"
             in
            let stmt_list =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 17253 "asllib/Parser.ml"
               _1_1
            in
            
# 492 "asllib/Parser.mly"
                                                               S_Repeat  
# 17259 "asllib/Parser.ml"
             (stmt_list, expr, loop_limit)
          in
          
# 196 "<standard.mly>"
    ( x )
# 17265 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17274 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 17280 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_338 =
    fun _1 _1_inlined1 _1_inlined2 _2_inlined1 _3 _endpos__2_ _startpos__1_ expr xs ->
      let x =
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let loop_limit =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 17295 "asllib/Parser.ml"
              
            in
            let stmt_list =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 17304 "asllib/Parser.ml"
               _1_1
            in
            
# 492 "asllib/Parser.mly"
                                                               S_Repeat  
# 17310 "asllib/Parser.ml"
             (stmt_list, expr, loop_limit)
          in
          
# 196 "<standard.mly>"
    ( x )
# 17316 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17325 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 17331 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_339 =
    fun _1 _1_inlined2 _3 _5 _endpos__2_ _startpos__1_ cond x xs ->
      let x =
        let desc =
          let x =
            let body =
              let _1 = _1_inlined2 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 17345 "asllib/Parser.ml"
               _1_1
            in
            let looplimit = 
# 205 "<standard.mly>"
    ( x )
# 17351 "asllib/Parser.ml"
             in
            
# 494 "asllib/Parser.mly"
          ( S_Repeat (body, cond, Some looplimit) )
# 17356 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 17362 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17371 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 17377 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_340 =
    fun _1 _endpos__2_ _startpos__1_ e xs ->
      let x =
        let desc =
          let x = 
# 495 "asllib/Parser.mly"
                                                             ( S_Throw (Some (e, None)) )
# 17387 "asllib/Parser.ml"
           in
          
# 196 "<standard.mly>"
    ( x )
# 17392 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17401 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 17407 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_341 =
    fun _1 _endpos__2_ _startpos__1_ xs ->
      let x =
        let desc =
          let x = 
# 496 "asllib/Parser.mly"
                                                             ( S_Throw None             )
# 17417 "asllib/Parser.ml"
           in
          
# 196 "<standard.mly>"
    ( x )
# 17422 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17431 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 17437 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_342 =
    fun _1 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let e = 
# 161 "asllib/Parser.mly"
                ( [] )
# 17448 "asllib/Parser.ml"
             in
            
# 497 "asllib/Parser.mly"
                                                               S_Pragma  
# 17453 "asllib/Parser.ml"
             (x, e)
          in
          
# 196 "<standard.mly>"
    ( x )
# 17459 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17468 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 17474 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_343 =
    fun _1 _1_inlined1 _endpos__2_ _startpos__1_ x xs ->
      let x =
        let desc =
          let x =
            let e = _1_inlined1 in
            
# 497 "asllib/Parser.mly"
                                                               S_Pragma  
# 17486 "asllib/Parser.ml"
             (x, e)
          in
          
# 196 "<standard.mly>"
    ( x )
# 17492 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17501 "asllib/Parser.ml"
        
      in
      (
# 231 "<standard.mly>"
    ( x :: xs )
# 17507 "asllib/Parser.ml"
       : (AST.stmt list))
  
  let _menhir_action_344 =
    fun _1 _1_inlined1 _2 _3 _endpos__2_inlined1_ _startpos__1_ e s2 ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let s1 =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 17522 "asllib/Parser.ml"
               _1_1
            in
            
# 460 "asllib/Parser.mly"
                                                       S_Cond 
# 17528 "asllib/Parser.ml"
             (e, s1, s2)
          in
          
# 196 "<standard.mly>"
    ( x )
# 17534 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17543 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 17564 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_345 =
    fun _1 _1_inlined1 _2 _3 _endpos__2_inlined1_ _startpos__1_ expr ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let alt = _1_inlined1 in
            
# 461 "asllib/Parser.mly"
                                                       S_Case 
# 17577 "asllib/Parser.ml"
             (expr, alt)
          in
          
# 196 "<standard.mly>"
    ( x )
# 17583 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17592 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 17613 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_346 =
    fun _1 _1_inlined1 _2 _4 _endpos__2_inlined1_ _startpos__1_ expr ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let stmt_list =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 17628 "asllib/Parser.ml"
               _1_1
            in
            let loop_limit = 
# 123 "<standard.mly>"
    ( None )
# 17634 "asllib/Parser.ml"
             in
            
# 462 "asllib/Parser.mly"
                                                       S_While 
# 17639 "asllib/Parser.ml"
             (expr, loop_limit, stmt_list)
          in
          
# 196 "<standard.mly>"
    ( x )
# 17645 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17654 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 17675 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_347 =
    fun _1 _1_inlined1 _1_inlined2 _2 _2_inlined1 _4 _endpos__2_inlined2_ _startpos__1_ expr ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let stmt_list =
              let _1 = _1_inlined2 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 17691 "asllib/Parser.ml"
               _1_1
            in
            let loop_limit =
              let _1 = _1_inlined1 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 17700 "asllib/Parser.ml"
              
            in
            
# 462 "asllib/Parser.mly"
                                                       S_While 
# 17706 "asllib/Parser.ml"
             (expr, loop_limit, stmt_list)
          in
          
# 196 "<standard.mly>"
    ( x )
# 17712 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17721 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 17742 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_348 =
    fun _1 _1_inlined1 _1_inlined2 _2 _3 _8 _endpos__2_inlined1_ _startpos__1_ end_e index_name start_e ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let body =
              let _1 = _1_inlined2 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 17757 "asllib/Parser.ml"
               _1_1
            in
            let limit = 
# 123 "<standard.mly>"
    ( None )
# 17763 "asllib/Parser.ml"
             in
            let dir =
              let _1 = _1_inlined1 in
              
# 439 "asllib/Parser.mly"
                       ( AST.Up )
# 17770 "asllib/Parser.ml"
              
            in
            
# 465 "asllib/Parser.mly"
          ( S_For { index_name; start_e; end_e; dir; body; limit } )
# 17776 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 17782 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17791 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 17812 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_349 =
    fun _1 _1_inlined1 _1_inlined2 _1_inlined3 _2 _2_inlined1 _3 _8 _endpos__2_inlined2_ _startpos__1_ end_e index_name start_e ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let body =
              let _1 = _1_inlined3 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 17828 "asllib/Parser.ml"
               _1_1
            in
            let limit =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 17837 "asllib/Parser.ml"
              
            in
            let dir =
              let _1 = _1_inlined1 in
              
# 439 "asllib/Parser.mly"
                       ( AST.Up )
# 17845 "asllib/Parser.ml"
              
            in
            
# 465 "asllib/Parser.mly"
          ( S_For { index_name; start_e; end_e; dir; body; limit } )
# 17851 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 17857 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17866 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 17887 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_350 =
    fun _1 _1_inlined1 _1_inlined2 _2 _3 _8 _endpos__2_inlined1_ _startpos__1_ end_e index_name start_e ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let body =
              let _1 = _1_inlined2 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 17902 "asllib/Parser.ml"
               _1_1
            in
            let limit = 
# 123 "<standard.mly>"
    ( None )
# 17908 "asllib/Parser.ml"
             in
            let dir =
              let _1 = _1_inlined1 in
              
# 439 "asllib/Parser.mly"
                                            ( AST.Down )
# 17915 "asllib/Parser.ml"
              
            in
            
# 465 "asllib/Parser.mly"
          ( S_For { index_name; start_e; end_e; dir; body; limit } )
# 17921 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 17927 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 17936 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 17957 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_351 =
    fun _1 _1_inlined1 _1_inlined2 _1_inlined3 _2 _2_inlined1 _3 _8 _endpos__2_inlined2_ _startpos__1_ end_e index_name start_e ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let body =
              let _1 = _1_inlined3 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 17973 "asllib/Parser.ml"
               _1_1
            in
            let limit =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 17982 "asllib/Parser.ml"
              
            in
            let dir =
              let _1 = _1_inlined1 in
              
# 439 "asllib/Parser.mly"
                                            ( AST.Down )
# 17990 "asllib/Parser.ml"
              
            in
            
# 465 "asllib/Parser.mly"
          ( S_For { index_name; start_e; end_e; dir; body; limit } )
# 17996 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 18002 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 18011 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 18032 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_352 =
    fun _1 _1_inlined1 _2 _3 _endpos__2_inlined1_ _startpos__1_ c ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let o = 
# 123 "<standard.mly>"
    ( None )
# 18044 "asllib/Parser.ml"
             in
            let s =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 18052 "asllib/Parser.ml"
               _1_1
            in
            
# 466 "asllib/Parser.mly"
                                                                              S_Try  
# 18058 "asllib/Parser.ml"
             (s, c, o)
          in
          
# 196 "<standard.mly>"
    ( x )
# 18064 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 18073 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 18094 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_353 =
    fun _1 _1_inlined1 _1_inlined2 _1_inlined3 _2 _2_inlined1 _3 _endpos__2_inlined2_ _startpos__1_ c ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let o =
              let (_1_inlined1, _1) = (_1_inlined3, _1_inlined2) in
              let x =
                let _1 = _1_inlined1 in
                let _1_1 = _1 in
                
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 18112 "asllib/Parser.ml"
                 _1_1
              in
              
# 126 "<standard.mly>"
    ( Some x )
# 18118 "asllib/Parser.ml"
              
            in
            let s =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 18127 "asllib/Parser.ml"
               _1_1
            in
            
# 466 "asllib/Parser.mly"
                                                                              S_Try  
# 18133 "asllib/Parser.ml"
             (s, c, o)
          in
          
# 196 "<standard.mly>"
    ( x )
# 18139 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 18148 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 18169 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_354 =
    fun _1 _1_inlined2 _2 _3 _5 _endpos__2_inlined1_ _startpos__1_ cond x ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let body =
              let _1 = _1_inlined2 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 18184 "asllib/Parser.ml"
               _1_1
            in
            let looplimit = 
# 205 "<standard.mly>"
    ( x )
# 18190 "asllib/Parser.ml"
             in
            
# 468 "asllib/Parser.mly"
          ( S_While (cond, Some looplimit, body) )
# 18195 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 18201 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 18210 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 18231 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_355 =
    fun _1 _2 _endpos__2_inlined1_ _startpos__1_ ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x = 
# 437 "asllib/Parser.mly"
            ( S_Pass )
# 18242 "asllib/Parser.ml"
           in
          
# 196 "<standard.mly>"
    ( x )
# 18247 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 18256 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 18277 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_356 =
    fun _1 _2 _endpos__2_inlined1_ _startpos__1_ ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let _2 = 
# 123 "<standard.mly>"
    ( None )
# 18289 "asllib/Parser.ml"
             in
            let _2_1 = _2 in
            
# 472 "asllib/Parser.mly"
                                                               S_Return  
# 18295 "asllib/Parser.ml"
             _2_1
          in
          
# 196 "<standard.mly>"
    ( x )
# 18301 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 18310 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 18331 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_357 =
    fun _1 _2 _endpos__2_inlined1_ _startpos__1_ x ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let _2 = 
# 126 "<standard.mly>"
    ( Some x )
# 18343 "asllib/Parser.ml"
             in
            let _2_1 = _2 in
            
# 472 "asllib/Parser.mly"
                                                               S_Return  
# 18349 "asllib/Parser.ml"
             _2_1
          in
          
# 196 "<standard.mly>"
    ( x )
# 18355 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 18364 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 18385 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_358 =
    fun _2 _endpos__2_inlined1_ _startpos_name_ name ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let call =
              let args =
                let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 18399 "asllib/Parser.ml"
                 in
                
# 205 "<standard.mly>"
    ( x )
# 18404 "asllib/Parser.ml"
                
              in
              
# 525 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 18410 "asllib/Parser.ml"
              
            in
            
# 473 "asllib/Parser.mly"
                                                               s_call  
# 18416 "asllib/Parser.ml"
             call
          in
          
# 196 "<standard.mly>"
    ( x )
# 18422 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 18431 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 18452 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_359 =
    fun _1_inlined1 _2 _endpos__2_inlined1_ _startpos_name_ name ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let call =
              let args =
                let x = _1_inlined1 in
                
# 205 "<standard.mly>"
    ( x )
# 18467 "asllib/Parser.ml"
                
              in
              
# 525 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 18473 "asllib/Parser.ml"
              
            in
            
# 473 "asllib/Parser.mly"
                                                               s_call  
# 18479 "asllib/Parser.ml"
             call
          in
          
# 196 "<standard.mly>"
    ( x )
# 18485 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 18494 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 18515 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_360 =
    fun _1_inlined1 _2 _endpos__2_inlined1_ _startpos_name_ args name ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let call =
              let params =
                let x = _1_inlined1 in
                
# 205 "<standard.mly>"
    ( x )
# 18530 "asllib/Parser.ml"
                
              in
              
# 527 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 18536 "asllib/Parser.ml"
              
            in
            
# 473 "asllib/Parser.mly"
                                                               s_call  
# 18542 "asllib/Parser.ml"
             call
          in
          
# 196 "<standard.mly>"
    ( x )
# 18548 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 18557 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 18578 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_361 =
    fun _1 _2 _endpos__2_inlined1_ _startpos__1_ e ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x = 
# 474 "asllib/Parser.mly"
                                                               S_Assert  
# 18589 "asllib/Parser.ml"
           e in
          
# 196 "<standard.mly>"
    ( x )
# 18594 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 18603 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 18624 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_362 =
    fun _1 _2 _3 _endpos__2_inlined1_ _startpos__1_ decl_item x ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let _4 = 
# 133 "asllib/Parser.mly"
                        Some 
# 18636 "asllib/Parser.ml"
             x in
            let local_decl_keyword = 
# 425 "asllib/Parser.mly"
                ( LDK_Let       )
# 18641 "asllib/Parser.ml"
             in
            let _4_1 = _4 in
            
# 475 "asllib/Parser.mly"
                                                               S_Decl    
# 18647 "asllib/Parser.ml"
             (local_decl_keyword, decl_item, _4_1)
          in
          
# 196 "<standard.mly>"
    ( x )
# 18653 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 18662 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 18683 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_363 =
    fun _1 _2 _3 _endpos__2_inlined1_ _startpos__1_ decl_item x ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let _4 = 
# 133 "asllib/Parser.mly"
                        Some 
# 18695 "asllib/Parser.ml"
             x in
            let local_decl_keyword = 
# 426 "asllib/Parser.mly"
                ( LDK_Constant  )
# 18700 "asllib/Parser.ml"
             in
            let _4_1 = _4 in
            
# 475 "asllib/Parser.mly"
                                                               S_Decl    
# 18706 "asllib/Parser.ml"
             (local_decl_keyword, decl_item, _4_1)
          in
          
# 196 "<standard.mly>"
    ( x )
# 18712 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 18721 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 18742 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_364 =
    fun _2 _2_inlined1 _endpos__2_inlined2_ _startpos_le_ e le ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            
# 476 "asllib/Parser.mly"
                                                               S_Assign  
# 18755 "asllib/Parser.ml"
             (le, e)
          in
          
# 196 "<standard.mly>"
    ( x )
# 18761 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_le_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 18770 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 18791 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_365 =
    fun _2 _2_inlined1 _endpos__2_inlined2_ _endpos__3_ _startpos_name_ name rhs ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let desc =
                let args =
                  let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 18807 "asllib/Parser.ml"
                   in
                  
# 205 "<standard.mly>"
    ( x )
# 18812 "asllib/Parser.ml"
                  
                in
                
# 525 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 18818 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 18827 "asllib/Parser.ml"
              
            in
            
# 478 "asllib/Parser.mly"
        ( desugar_setter call [] rhs )
# 18833 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 18839 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 18848 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 18869 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_366 =
    fun _1_inlined1 _2 _2_inlined1 _endpos__2_inlined2_ _endpos__3_ _startpos_name_ name rhs ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let desc =
                let args =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 18886 "asllib/Parser.ml"
                  
                in
                
# 525 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 18892 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 18901 "asllib/Parser.ml"
              
            in
            
# 478 "asllib/Parser.mly"
        ( desugar_setter call [] rhs )
# 18907 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 18913 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 18922 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 18943 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_367 =
    fun _1_inlined1 _2 _2_inlined1 _endpos__2_inlined2_ _endpos_args_ _startpos_name_ args name rhs ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let desc =
                let params =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 18960 "asllib/Parser.ml"
                  
                in
                
# 527 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 18966 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 18975 "asllib/Parser.ml"
              
            in
            
# 478 "asllib/Parser.mly"
        ( desugar_setter call [] rhs )
# 18981 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 18987 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 18996 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 19017 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_368 =
    fun _2 _2_inlined1 _4 _endpos__2_inlined2_ _endpos__3_ _startpos_name_ fld name rhs ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let desc =
                let args =
                  let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 19033 "asllib/Parser.ml"
                   in
                  
# 205 "<standard.mly>"
    ( x )
# 19038 "asllib/Parser.ml"
                  
                in
                
# 525 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 19044 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19053 "asllib/Parser.ml"
              
            in
            
# 480 "asllib/Parser.mly"
        ( desugar_setter call [fld] rhs )
# 19059 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 19065 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19074 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 19095 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_369 =
    fun _1_inlined1 _2 _2_inlined1 _4 _endpos__2_inlined2_ _endpos__3_ _startpos_name_ fld name rhs ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let desc =
                let args =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 19112 "asllib/Parser.ml"
                  
                in
                
# 525 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 19118 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19127 "asllib/Parser.ml"
              
            in
            
# 480 "asllib/Parser.mly"
        ( desugar_setter call [fld] rhs )
# 19133 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 19139 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19148 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 19169 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_370 =
    fun _1_inlined1 _2 _2_inlined1 _4 _endpos__2_inlined2_ _endpos_args_ _startpos_name_ args fld name rhs ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let desc =
                let params =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 19186 "asllib/Parser.ml"
                  
                in
                
# 527 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 19192 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19201 "asllib/Parser.ml"
              
            in
            
# 480 "asllib/Parser.mly"
        ( desugar_setter call [fld] rhs )
# 19207 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 19213 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19222 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 19243 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_371 =
    fun _1_inlined2 _2 _2_inlined1 _2_inlined2 _4 _endpos__2_inlined3_ _endpos__3_ _startpos_name_ name rhs x ->
      let body =
        let _endpos__2_ = _endpos__2_inlined3_ in
        let desc =
          let x =
            let (_2_inlined1, _2) = (_2_inlined2, _2_inlined1) in
            let flds =
              let (_1_inlined1, _2) = (_1_inlined2, _2_inlined1) in
              let x =
                let _1 = _1_inlined1 in
                let li = _1 in
                
# 164 "asllib/Parser.mly"
                                           ( x :: li )
# 19261 "asllib/Parser.ml"
                
              in
              
# 205 "<standard.mly>"
    ( x )
# 19267 "asllib/Parser.ml"
              
            in
            let call =
              let desc =
                let args =
                  let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 19276 "asllib/Parser.ml"
                   in
                  
# 205 "<standard.mly>"
    ( x )
# 19281 "asllib/Parser.ml"
                  
                in
                
# 525 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 19287 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19296 "asllib/Parser.ml"
              
            in
            
# 482 "asllib/Parser.mly"
        ( desugar_setter call flds rhs )
# 19302 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 19308 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19317 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 19338 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_372 =
    fun _1_inlined1 _1_inlined3 _2 _2_inlined1 _2_inlined2 _4 _endpos__2_inlined3_ _endpos__3_ _startpos_name_ name rhs x ->
      let body =
        let _endpos__2_ = _endpos__2_inlined3_ in
        let desc =
          let x =
            let (_2_inlined1, _2) = (_2_inlined2, _2_inlined1) in
            let flds =
              let (_1_inlined1, _2) = (_1_inlined3, _2_inlined1) in
              let x =
                let _1 = _1_inlined1 in
                let li = _1 in
                
# 164 "asllib/Parser.mly"
                                           ( x :: li )
# 19356 "asllib/Parser.ml"
                
              in
              
# 205 "<standard.mly>"
    ( x )
# 19362 "asllib/Parser.ml"
              
            in
            let call =
              let desc =
                let args =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 19372 "asllib/Parser.ml"
                  
                in
                
# 525 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 19378 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19387 "asllib/Parser.ml"
              
            in
            
# 482 "asllib/Parser.mly"
        ( desugar_setter call flds rhs )
# 19393 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 19399 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19408 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 19429 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_373 =
    fun _1_inlined1 _1_inlined3 _2 _2_inlined1 _2_inlined2 _4 _endpos__2_inlined3_ _endpos_args_ _startpos_name_ args name rhs x ->
      let body =
        let _endpos__2_ = _endpos__2_inlined3_ in
        let desc =
          let x =
            let (_2_inlined1, _2) = (_2_inlined2, _2_inlined1) in
            let flds =
              let (_1_inlined1, _2) = (_1_inlined3, _2_inlined1) in
              let x =
                let _1 = _1_inlined1 in
                let li = _1 in
                
# 164 "asllib/Parser.mly"
                                           ( x :: li )
# 19447 "asllib/Parser.ml"
                
              in
              
# 205 "<standard.mly>"
    ( x )
# 19453 "asllib/Parser.ml"
              
            in
            let call =
              let desc =
                let params =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 19463 "asllib/Parser.ml"
                  
                in
                
# 527 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 19469 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19478 "asllib/Parser.ml"
              
            in
            
# 482 "asllib/Parser.mly"
        ( desugar_setter call flds rhs )
# 19484 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 19490 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos_name_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19499 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 19520 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_374 =
    fun _1 _2 _2_inlined1 _3 _3_inlined1 _endpos__2_inlined2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_endpos__3_inlined1_, _3) = (_endpos__3_inlined2_, _3_inlined1) in
              let desc =
                let args =
                  let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 19537 "asllib/Parser.ml"
                   in
                  
# 205 "<standard.mly>"
    ( x )
# 19542 "asllib/Parser.ml"
                  
                in
                
# 530 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 19548 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19557 "asllib/Parser.ml"
              
            in
            let ldk = 
# 425 "asllib/Parser.mly"
                ( LDK_Let       )
# 19563 "asllib/Parser.ml"
             in
            
# 484 "asllib/Parser.mly"
        ( desugar_elided_parameter ldk lhs call)
# 19568 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 19574 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19583 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 19604 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_375 =
    fun _1 _1_inlined2 _2 _2_inlined1 _3 _3_inlined1 _endpos__2_inlined2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_endpos__3_inlined1_, _1_inlined1, _3) = (_endpos__3_inlined2_, _1_inlined2, _3_inlined1) in
              let desc =
                let args =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 19622 "asllib/Parser.ml"
                  
                in
                
# 530 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 19628 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19637 "asllib/Parser.ml"
              
            in
            let ldk = 
# 425 "asllib/Parser.mly"
                ( LDK_Let       )
# 19643 "asllib/Parser.ml"
             in
            
# 484 "asllib/Parser.mly"
        ( desugar_elided_parameter ldk lhs call)
# 19648 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 19654 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19663 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 19684 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_376 =
    fun _1 _1_inlined1 _2 _2_inlined1 _3 _3_inlined1 _5 _endpos__2_inlined2_ _endpos_args_ _startpos__1_ _startpos_name_ args lhs name ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_1, _3) = (_1_inlined1, _3_inlined1) in
              let desc =
                let params = _1 in
                
# 532 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 19701 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19710 "asllib/Parser.ml"
              
            in
            let ldk = 
# 425 "asllib/Parser.mly"
                ( LDK_Let       )
# 19716 "asllib/Parser.ml"
             in
            
# 484 "asllib/Parser.mly"
        ( desugar_elided_parameter ldk lhs call)
# 19721 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 19727 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19736 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 19757 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_377 =
    fun _1 _2 _2_inlined1 _3 _3_inlined1 _endpos__2_inlined2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_endpos__3_inlined1_, _3) = (_endpos__3_inlined2_, _3_inlined1) in
              let desc =
                let args =
                  let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 19774 "asllib/Parser.ml"
                   in
                  
# 205 "<standard.mly>"
    ( x )
# 19779 "asllib/Parser.ml"
                  
                in
                
# 530 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 19785 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19794 "asllib/Parser.ml"
              
            in
            let ldk = 
# 426 "asllib/Parser.mly"
                ( LDK_Constant  )
# 19800 "asllib/Parser.ml"
             in
            
# 484 "asllib/Parser.mly"
        ( desugar_elided_parameter ldk lhs call)
# 19805 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 19811 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19820 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 19841 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_378 =
    fun _1 _1_inlined2 _2 _2_inlined1 _3 _3_inlined1 _endpos__2_inlined2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_endpos__3_inlined1_, _1_inlined1, _3) = (_endpos__3_inlined2_, _1_inlined2, _3_inlined1) in
              let desc =
                let args =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 19859 "asllib/Parser.ml"
                  
                in
                
# 530 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 19865 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19874 "asllib/Parser.ml"
              
            in
            let ldk = 
# 426 "asllib/Parser.mly"
                ( LDK_Constant  )
# 19880 "asllib/Parser.ml"
             in
            
# 484 "asllib/Parser.mly"
        ( desugar_elided_parameter ldk lhs call)
# 19885 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 19891 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19900 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 19921 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_379 =
    fun _1 _1_inlined1 _2 _2_inlined1 _3 _3_inlined1 _5 _endpos__2_inlined2_ _endpos_args_ _startpos__1_ _startpos_name_ args lhs name ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_1, _3) = (_1_inlined1, _3_inlined1) in
              let desc =
                let params = _1 in
                
# 532 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 19938 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19947 "asllib/Parser.ml"
              
            in
            let ldk = 
# 426 "asllib/Parser.mly"
                ( LDK_Constant  )
# 19953 "asllib/Parser.ml"
             in
            
# 484 "asllib/Parser.mly"
        ( desugar_elided_parameter ldk lhs call)
# 19958 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 19964 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 19973 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 19994 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_380 =
    fun _1 _2 _endpos__2_inlined1_ _startpos__1_ ldi ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let e = 
# 123 "<standard.mly>"
    ( None )
# 20006 "asllib/Parser.ml"
             in
            
# 485 "asllib/Parser.mly"
                                                             ( S_Decl (LDK_Var, ldi, e) )
# 20011 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 20017 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20026 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 20047 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_381 =
    fun _1 _1_inlined1 _2 _2_inlined1 _endpos__2_inlined2_ _startpos__1_ ldi ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let e =
              let _1 = _1_inlined1 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 20063 "asllib/Parser.ml"
              
            in
            
# 485 "asllib/Parser.mly"
                                                             ( S_Decl (LDK_Var, ldi, e) )
# 20069 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 20075 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20084 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 20105 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_382 =
    fun _1 _1_inlined1 _1_inlined2 _2 _2_inlined1 _2_inlined2 _endpos__2_inlined3_ _startpos__1_ x ->
      let body =
        let _endpos__2_ = _endpos__2_inlined3_ in
        let desc =
          let x =
            let (_2_inlined1, _2) = (_2_inlined2, _2_inlined1) in
            let as_ty =
              let (_2, _1) = (_2_inlined1, _1_inlined2) in
              _2
            in
            let _2 =
              let _1 = _1_inlined1 in
              let li = _1 in
              
# 164 "asllib/Parser.mly"
                                           ( x :: li )
# 20125 "asllib/Parser.ml"
              
            in
            let _2_1 = _2 in
            
# 486 "asllib/Parser.mly"
                                                               make_ldi_vars  
# 20132 "asllib/Parser.ml"
             (_2_1, as_ty)
          in
          
# 196 "<standard.mly>"
    ( x )
# 20138 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20147 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 20168 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_383 =
    fun _1 _1_inlined1 _1_inlined2 _2 _2_inlined1 _2_inlined2 _endpos__2_inlined3_ _startpos__1_ x ->
      let body =
        let _endpos__2_ = _endpos__2_inlined3_ in
        let desc =
          let x =
            let (_2_inlined1, _2) = (_2_inlined2, _2_inlined1) in
            let as_ty =
              let (_2, _1) = (_2_inlined1, _1_inlined2) in
              _2
            in
            let _2 =
              let _1 = _1_inlined1 in
              let li = _1 in
              
# 164 "asllib/Parser.mly"
                                           ( x :: li )
# 20188 "asllib/Parser.ml"
              
            in
            let _2_1 = _2 in
            
# 486 "asllib/Parser.mly"
                                                               make_ldi_vars  
# 20195 "asllib/Parser.ml"
             (_2_1, as_ty)
          in
          
# 196 "<standard.mly>"
    ( x )
# 20201 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20210 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 20231 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_384 =
    fun _1 _2 _2_inlined1 _3 _3_inlined1 _endpos__2_inlined2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_endpos__3_inlined1_, _3) = (_endpos__3_inlined2_, _3_inlined1) in
              let desc =
                let args =
                  let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 20248 "asllib/Parser.ml"
                   in
                  
# 205 "<standard.mly>"
    ( x )
# 20253 "asllib/Parser.ml"
                  
                in
                
# 530 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 20259 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20268 "asllib/Parser.ml"
              
            in
            
# 488 "asllib/Parser.mly"
        ( desugar_elided_parameter LDK_Var lhs call)
# 20274 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 20280 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20289 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 20310 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_385 =
    fun _1 _1_inlined2 _2 _2_inlined1 _3 _3_inlined1 _endpos__2_inlined2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_endpos__3_inlined1_, _1_inlined1, _3) = (_endpos__3_inlined2_, _1_inlined2, _3_inlined1) in
              let desc =
                let args =
                  let x = _1_inlined1 in
                  
# 205 "<standard.mly>"
    ( x )
# 20328 "asllib/Parser.ml"
                  
                in
                
# 530 "asllib/Parser.mly"
    ( { name; params=[]; args; call_type = ST_Function } )
# 20334 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos__3_inlined1_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20343 "asllib/Parser.ml"
              
            in
            
# 488 "asllib/Parser.mly"
        ( desugar_elided_parameter LDK_Var lhs call)
# 20349 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 20355 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20364 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 20385 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_386 =
    fun _1 _1_inlined1 _2 _2_inlined1 _3 _3_inlined1 _5 _endpos__2_inlined2_ _endpos_args_ _startpos__1_ _startpos_name_ args lhs name ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let call =
              let (_1, _3) = (_1_inlined1, _3_inlined1) in
              let desc =
                let params = _1 in
                
# 532 "asllib/Parser.mly"
    ( { name; params; args; call_type = ST_Function } )
# 20402 "asllib/Parser.ml"
                
              in
              let (_endpos_desc_, _startpos_desc_) = (_endpos_args_, _startpos_name_) in
              let _endpos = _endpos_desc_ in
              let _symbolstartpos = _startpos_desc_ in
              
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20411 "asllib/Parser.ml"
              
            in
            
# 488 "asllib/Parser.mly"
        ( desugar_elided_parameter LDK_Var lhs call)
# 20417 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 20423 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20432 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 20453 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_387 =
    fun _1 _2 _endpos__2_inlined1_ _startpos__1_ ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let args =
              let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 20466 "asllib/Parser.ml"
               in
              
# 205 "<standard.mly>"
    ( x )
# 20471 "asllib/Parser.ml"
              
            in
            
# 489 "asllib/Parser.mly"
                                                             ( S_Print { args; debug = false } )
# 20477 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 20483 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20492 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 20513 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_388 =
    fun _1 _1_inlined2 _2 _endpos__2_inlined1_ _startpos__1_ ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let args =
              let _1_inlined1 = _1_inlined2 in
              let x = _1_inlined1 in
              
# 205 "<standard.mly>"
    ( x )
# 20528 "asllib/Parser.ml"
              
            in
            
# 489 "asllib/Parser.mly"
                                                             ( S_Print { args; debug = false } )
# 20534 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 20540 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20549 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 20570 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_389 =
    fun _1 _2 _endpos__2_inlined1_ _startpos__1_ ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let args =
              let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 20583 "asllib/Parser.ml"
               in
              
# 205 "<standard.mly>"
    ( x )
# 20588 "asllib/Parser.ml"
              
            in
            
# 490 "asllib/Parser.mly"
                                                             ( S_Print { args; debug = true } )
# 20594 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 20600 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20609 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 20630 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_390 =
    fun _1 _1_inlined2 _2 _endpos__2_inlined1_ _startpos__1_ ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let args =
              let _1_inlined1 = _1_inlined2 in
              let x = _1_inlined1 in
              
# 205 "<standard.mly>"
    ( x )
# 20645 "asllib/Parser.ml"
              
            in
            
# 490 "asllib/Parser.mly"
                                                             ( S_Print { args; debug = true } )
# 20651 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 20657 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20666 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 20687 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_391 =
    fun _1 _2 _2_inlined1 _3 _endpos__2_inlined2_ _startpos__1_ ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            
# 491 "asllib/Parser.mly"
                                                             ( S_Unreachable )
# 20700 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 20706 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20715 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 20736 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_392 =
    fun _1 _1_inlined1 _2 _3 _endpos__2_inlined1_ _startpos__1_ expr ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let loop_limit = 
# 123 "<standard.mly>"
    ( None )
# 20748 "asllib/Parser.ml"
             in
            let stmt_list =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 20756 "asllib/Parser.ml"
               _1_1
            in
            
# 492 "asllib/Parser.mly"
                                                               S_Repeat  
# 20762 "asllib/Parser.ml"
             (stmt_list, expr, loop_limit)
          in
          
# 196 "<standard.mly>"
    ( x )
# 20768 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20777 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 20798 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_393 =
    fun _1 _1_inlined1 _1_inlined2 _2 _2_inlined1 _3 _endpos__2_inlined2_ _startpos__1_ expr ->
      let body =
        let _endpos__2_ = _endpos__2_inlined2_ in
        let desc =
          let x =
            let _2 = _2_inlined1 in
            let loop_limit =
              let _1 = _1_inlined2 in
              let x = _2 in
              
# 126 "<standard.mly>"
    ( Some x )
# 20814 "asllib/Parser.ml"
              
            in
            let stmt_list =
              let _1 = _1_inlined1 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 20823 "asllib/Parser.ml"
               _1_1
            in
            
# 492 "asllib/Parser.mly"
                                                               S_Repeat  
# 20829 "asllib/Parser.ml"
             (stmt_list, expr, loop_limit)
          in
          
# 196 "<standard.mly>"
    ( x )
# 20835 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20844 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 20865 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_394 =
    fun _1 _1_inlined2 _2 _3 _5 _endpos__2_inlined1_ _startpos__1_ cond x ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let body =
              let _1 = _1_inlined2 in
              let _1_1 = _1 in
              
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 20880 "asllib/Parser.ml"
               _1_1
            in
            let looplimit = 
# 205 "<standard.mly>"
    ( x )
# 20886 "asllib/Parser.ml"
             in
            
# 494 "asllib/Parser.mly"
          ( S_Repeat (body, cond, Some looplimit) )
# 20891 "asllib/Parser.ml"
            
          in
          
# 196 "<standard.mly>"
    ( x )
# 20897 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20906 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 20927 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_395 =
    fun _1 _2 _endpos__2_inlined1_ _startpos__1_ e ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x = 
# 495 "asllib/Parser.mly"
                                                             ( S_Throw (Some (e, None)) )
# 20938 "asllib/Parser.ml"
           in
          
# 196 "<standard.mly>"
    ( x )
# 20943 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20952 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 20973 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_396 =
    fun _1 _2 _endpos__2_inlined1_ _startpos__1_ ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x = 
# 496 "asllib/Parser.mly"
                                                             ( S_Throw None             )
# 20984 "asllib/Parser.ml"
           in
          
# 196 "<standard.mly>"
    ( x )
# 20989 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 20998 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 21019 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_397 =
    fun _1 _2 _endpos__2_inlined1_ _startpos__1_ x ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let e = 
# 161 "asllib/Parser.mly"
                ( [] )
# 21031 "asllib/Parser.ml"
             in
            
# 497 "asllib/Parser.mly"
                                                               S_Pragma  
# 21036 "asllib/Parser.ml"
             (x, e)
          in
          
# 196 "<standard.mly>"
    ( x )
# 21042 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 21051 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 21072 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_398 =
    fun _1 _1_inlined1 _2 _endpos__2_inlined1_ _startpos__1_ x ->
      let body =
        let _endpos__2_ = _endpos__2_inlined1_ in
        let desc =
          let x =
            let e = _1_inlined1 in
            
# 497 "asllib/Parser.mly"
                                                               S_Pragma  
# 21085 "asllib/Parser.ml"
             (x, e)
          in
          
# 196 "<standard.mly>"
    ( x )
# 21091 "asllib/Parser.ml"
          
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__2_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 21100 "asllib/Parser.ml"
        
      in
      (
# 634 "asllib/Parser.mly"
    (
      [
        D_Func
          {
            name = "main";
            args = [];
            parameters = [];
            body = SB_ASL body;
            return_type = None;
            subprogram_type = ST_Procedure;
            recurse_limit = None;
            builtin = false;
          }
        |> ASTUtils.add_pos_from body
      ]
    )
# 21121 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_399 =
    fun () ->
      (
# 111 "<standard.mly>"
    ( None )
# 21129 "asllib/Parser.ml"
       : (unit option))
  
  let _menhir_action_400 =
    fun x ->
      (
# 114 "<standard.mly>"
    ( Some x )
# 21137 "asllib/Parser.ml"
       : (unit option))
  
  let _menhir_action_401 =
    fun () ->
      (
# 111 "<standard.mly>"
    ( None )
# 21145 "asllib/Parser.ml"
       : ((string * AST.field list) option))
  
  let _menhir_action_402 =
    fun _1 _2 _3 ->
      let x =
        let _2_1 = _2 in
        let _3_1 = _3 in
        (_2_1, _3_1)
      in
      (
# 114 "<standard.mly>"
    ( Some x )
# 21158 "asllib/Parser.ml"
       : ((string * AST.field list) option))
  
  let _menhir_action_403 =
    fun _endpos_expr_pattern_ _startpos_expr_pattern_ expr_pattern ->
      let desc = 
# 315 "asllib/Parser.mly"
                        Pattern_Single  
# 21166 "asllib/Parser.ml"
       expr_pattern in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_expr_pattern_, _startpos_expr_pattern_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 21174 "asllib/Parser.ml"
       : (AST.pattern))
  
  let _menhir_action_404 =
    fun _2 _endpos_e2_ _startpos_e1_ e1 e2 ->
      let desc = 
# 316 "asllib/Parser.mly"
                                           Pattern_Range  
# 21182 "asllib/Parser.ml"
       (e1, e2) in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_e2_, _startpos_e1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 21190 "asllib/Parser.ml"
       : (AST.pattern))
  
  let _menhir_action_405 =
    fun _1 _endpos__1_ _startpos__1_ ->
      let desc = 
# 317 "asllib/Parser.mly"
             ( Pattern_All )
# 21198 "asllib/Parser.ml"
       in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 21206 "asllib/Parser.ml"
       : (AST.pattern))
  
  let _menhir_action_406 =
    fun _1 _endpos_expr_ _startpos__1_ expr ->
      let desc = 
# 318 "asllib/Parser.mly"
                     Pattern_Leq  
# 21214 "asllib/Parser.ml"
       expr in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_expr_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 21222 "asllib/Parser.ml"
       : (AST.pattern))
  
  let _menhir_action_407 =
    fun _1 _endpos_expr_ _startpos__1_ expr ->
      let desc = 
# 319 "asllib/Parser.mly"
                     Pattern_Geq  
# 21230 "asllib/Parser.ml"
       expr in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_expr_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 21238 "asllib/Parser.ml"
       : (AST.pattern))
  
  let _menhir_action_408 =
    fun _2 _endpos__3_ _startpos__1_ li x ->
      let desc =
        let _1 =
          let x = 
# 177 "asllib/Parser.mly"
                                                    ( x :: li )
# 21248 "asllib/Parser.ml"
           in
          
# 205 "<standard.mly>"
    ( x )
# 21253 "asllib/Parser.ml"
          
        in
        let _1_1 = _1 in
        
# 320 "asllib/Parser.mly"
                           Pattern_Tuple  
# 21260 "asllib/Parser.ml"
         _1_1
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 21269 "asllib/Parser.ml"
       : (AST.pattern))
  
  let _menhir_action_409 =
    fun _1 _endpos__1_ _startpos__1_ ->
      let desc =
        let _1_1 = _1 in
        
# 310 "asllib/Parser.mly"
                                            Pattern_Mask  
# 21279 "asllib/Parser.ml"
         _1_1
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 21288 "asllib/Parser.ml"
       : (AST.pattern))
  
  let _menhir_action_410 =
    fun _1_inlined1 _endpos__1_inlined1_ _startpos__1_inlined1_ ->
      let x =
        let (_endpos__1_, _startpos__1_, _1) = (_endpos__1_inlined1_, _startpos__1_inlined1_, _1_inlined1) in
        let desc =
          let _1_1 = _1 in
          
# 311 "asllib/Parser.mly"
                                                   Pattern_Any  
# 21300 "asllib/Parser.ml"
           _1_1
        in
        let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
        let _endpos = _endpos_desc_ in
        let _symbolstartpos = _startpos_desc_ in
        
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 21309 "asllib/Parser.ml"
        
      in
      (
# 205 "<standard.mly>"
    ( x )
# 21315 "asllib/Parser.ml"
       : (AST.pattern))
  
  let _menhir_action_411 =
    fun _1 _1_inlined2 _endpos__1_inlined2_ _endpos__3_ _startpos__1_ _startpos__1_inlined2_ ->
      let desc =
        let _2 =
          let (_endpos__1_inlined1_, _startpos__1_inlined1_, _1_inlined1) = (_endpos__1_inlined2_, _startpos__1_inlined2_, _1_inlined2) in
          let x =
            let (_endpos__1_, _startpos__1_, _1) = (_endpos__1_inlined1_, _startpos__1_inlined1_, _1_inlined1) in
            let desc =
              let _1_1 = _1 in
              
# 311 "asllib/Parser.mly"
                                                   Pattern_Any  
# 21330 "asllib/Parser.ml"
               _1_1
            in
            let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
            let _endpos = _endpos_desc_ in
            let _symbolstartpos = _startpos_desc_ in
            
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 21339 "asllib/Parser.ml"
            
          in
          
# 205 "<standard.mly>"
    ( x )
# 21345 "asllib/Parser.ml"
          
        in
        let _2_1 = _2 in
        
# 328 "asllib/Parser.mly"
                                      Pattern_Not  
# 21352 "asllib/Parser.ml"
         _2_1
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 21361 "asllib/Parser.ml"
       : (AST.pattern))
  
  let _menhir_action_412 =
    fun _1 _1_inlined1 _3 _endpos_s2_ _startpos__1_ e s2 ->
      let desc =
        let s1 =
          let _1 = _1_inlined1 in
          let _1_1 = _1 in
          
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 21373 "asllib/Parser.ml"
           _1_1
        in
        
# 505 "asllib/Parser.mly"
                                                     S_Cond 
# 21379 "asllib/Parser.ml"
         (e, s1, s2)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_s2_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = if _startpos_desc_ != _endpos_desc_ then
        _startpos_desc_
      else
        _endpos in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 21391 "asllib/Parser.ml"
       : (AST.stmt))
  
  let _menhir_action_413 =
    fun _endpos__0_ ->
      let desc = 
# 437 "asllib/Parser.mly"
            ( S_Pass )
# 21399 "asllib/Parser.ml"
       in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__0_, _endpos__0_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = if _startpos_desc_ != _endpos_desc_ then
        _startpos_desc_
      else
        _endpos in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 21410 "asllib/Parser.ml"
       : (AST.stmt))
  
  let _menhir_action_414 =
    fun _1 _1_inlined1 ->
      let _1 = _1_inlined1 in
      let _1_1 = _1 in
      (
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 21420 "asllib/Parser.ml"
       _1_1 : (AST.stmt))
  
  let _menhir_action_415 =
    fun x ->
      (
# 250 "<standard.mly>"
    ( [ x ] )
# 21428 "asllib/Parser.ml"
       : (string list))
  
  let _menhir_action_416 =
    fun x xs ->
      (
# 253 "<standard.mly>"
    ( x :: xs )
# 21436 "asllib/Parser.ml"
       : (string list))
  
  let _menhir_action_417 =
    fun x ->
      (
# 250 "<standard.mly>"
    ( [ x ] )
# 21444 "asllib/Parser.ml"
       : (AST.local_decl_item list))
  
  let _menhir_action_418 =
    fun x xs ->
      (
# 253 "<standard.mly>"
    ( x :: xs )
# 21452 "asllib/Parser.ml"
       : (AST.local_decl_item list))
  
  let _menhir_action_419 =
    fun x ->
      (
# 250 "<standard.mly>"
    ( [ x ] )
# 21460 "asllib/Parser.ml"
       : (AST.expr list))
  
  let _menhir_action_420 =
    fun x xs ->
      (
# 253 "<standard.mly>"
    ( x :: xs )
# 21468 "asllib/Parser.ml"
       : (AST.expr list))
  
  let _menhir_action_421 =
    fun x y ->
      let x = 
# 180 "<standard.mly>"
    ( (x, y) )
# 21476 "asllib/Parser.ml"
       in
      (
# 250 "<standard.mly>"
    ( [ x ] )
# 21481 "asllib/Parser.ml"
       : ((string * AST.expr) list))
  
  let _menhir_action_422 =
    fun x xs y ->
      let x = 
# 180 "<standard.mly>"
    ( (x, y) )
# 21489 "asllib/Parser.ml"
       in
      (
# 253 "<standard.mly>"
    ( x :: xs )
# 21494 "asllib/Parser.ml"
       : ((string * AST.expr) list))
  
  let _menhir_action_423 =
    fun expr ->
      let x = 
# 305 "asllib/Parser.mly"
                                  Constraint_Exact  
# 21502 "asllib/Parser.ml"
       expr in
      (
# 250 "<standard.mly>"
    ( [ x ] )
# 21507 "asllib/Parser.ml"
       : (AST.int_constraint list))
  
  let _menhir_action_424 =
    fun _2 e1 e2 ->
      let x = 
# 306 "asllib/Parser.mly"
                                  Constraint_Range  
# 21515 "asllib/Parser.ml"
       (e1, e2) in
      (
# 250 "<standard.mly>"
    ( [ x ] )
# 21520 "asllib/Parser.ml"
       : (AST.int_constraint list))
  
  let _menhir_action_425 =
    fun expr xs ->
      let x = 
# 305 "asllib/Parser.mly"
                                  Constraint_Exact  
# 21528 "asllib/Parser.ml"
       expr in
      (
# 253 "<standard.mly>"
    ( x :: xs )
# 21533 "asllib/Parser.ml"
       : (AST.int_constraint list))
  
  let _menhir_action_426 =
    fun _2_inlined1 e1 e2 xs ->
      let x =
        let _2 = _2_inlined1 in
        
# 306 "asllib/Parser.mly"
                                  Constraint_Range  
# 21543 "asllib/Parser.ml"
         (e1, e2)
      in
      (
# 253 "<standard.mly>"
    ( x :: xs )
# 21549 "asllib/Parser.ml"
       : (AST.int_constraint list))
  
  let _menhir_action_427 =
    fun x ->
      (
# 250 "<standard.mly>"
    ( [ x ] )
# 21557 "asllib/Parser.ml"
       : (AST.lexpr list))
  
  let _menhir_action_428 =
    fun x xs ->
      (
# 253 "<standard.mly>"
    ( x :: xs )
# 21565 "asllib/Parser.ml"
       : (AST.lexpr list))
  
  let _menhir_action_429 =
    fun x ->
      let x =
        let y = 
# 123 "<standard.mly>"
    ( None )
# 21574 "asllib/Parser.ml"
         in
        
# 172 "<standard.mly>"
    ( (x, y) )
# 21579 "asllib/Parser.ml"
        
      in
      (
# 250 "<standard.mly>"
    ( [ x ] )
# 21585 "asllib/Parser.ml"
       : ((string * AST.ty option) list))
  
  let _menhir_action_430 =
    fun _1 _2 x ->
      let x =
        let y =
          let x = _2 in
          
# 126 "<standard.mly>"
    ( Some x )
# 21596 "asllib/Parser.ml"
          
        in
        
# 172 "<standard.mly>"
    ( (x, y) )
# 21602 "asllib/Parser.ml"
        
      in
      (
# 250 "<standard.mly>"
    ( [ x ] )
# 21608 "asllib/Parser.ml"
       : ((string * AST.ty option) list))
  
  let _menhir_action_431 =
    fun _1 _2 x ->
      let x =
        let y =
          let x = _2 in
          
# 126 "<standard.mly>"
    ( Some x )
# 21619 "asllib/Parser.ml"
          
        in
        
# 172 "<standard.mly>"
    ( (x, y) )
# 21625 "asllib/Parser.ml"
        
      in
      (
# 250 "<standard.mly>"
    ( [ x ] )
# 21631 "asllib/Parser.ml"
       : ((string * AST.ty option) list))
  
  let _menhir_action_432 =
    fun x xs ->
      let x =
        let y = 
# 123 "<standard.mly>"
    ( None )
# 21640 "asllib/Parser.ml"
         in
        
# 172 "<standard.mly>"
    ( (x, y) )
# 21645 "asllib/Parser.ml"
        
      in
      (
# 253 "<standard.mly>"
    ( x :: xs )
# 21651 "asllib/Parser.ml"
       : ((string * AST.ty option) list))
  
  let _menhir_action_433 =
    fun _1 _2_inlined1 x xs ->
      let x =
        let _2 = _2_inlined1 in
        let y =
          let x = _2 in
          
# 126 "<standard.mly>"
    ( Some x )
# 21663 "asllib/Parser.ml"
          
        in
        
# 172 "<standard.mly>"
    ( (x, y) )
# 21669 "asllib/Parser.ml"
        
      in
      (
# 253 "<standard.mly>"
    ( x :: xs )
# 21675 "asllib/Parser.ml"
       : ((string * AST.ty option) list))
  
  let _menhir_action_434 =
    fun _1 _2_inlined1 x xs ->
      let x =
        let _2 = _2_inlined1 in
        let y =
          let x = _2 in
          
# 126 "<standard.mly>"
    ( Some x )
# 21687 "asllib/Parser.ml"
          
        in
        
# 172 "<standard.mly>"
    ( (x, y) )
# 21693 "asllib/Parser.ml"
        
      in
      (
# 253 "<standard.mly>"
    ( x :: xs )
# 21699 "asllib/Parser.ml"
       : ((string * AST.ty option) list))
  
  let _menhir_action_435 =
    fun x ->
      (
# 250 "<standard.mly>"
    ( [ x ] )
# 21707 "asllib/Parser.ml"
       : (AST.pattern list))
  
  let _menhir_action_436 =
    fun x xs ->
      (
# 253 "<standard.mly>"
    ( x :: xs )
# 21715 "asllib/Parser.ml"
       : (AST.pattern list))
  
  let _menhir_action_437 =
    fun expr ->
      let x = 
# 337 "asllib/Parser.mly"
                                    Slice_Single   
# 21723 "asllib/Parser.ml"
       expr in
      (
# 250 "<standard.mly>"
    ( [ x ] )
# 21728 "asllib/Parser.ml"
       : (AST.slice list))
  
  let _menhir_action_438 =
    fun _2 e1 e2 ->
      let x = 
# 338 "asllib/Parser.mly"
                                    Slice_Range    
# 21736 "asllib/Parser.ml"
       (e1, e2) in
      (
# 250 "<standard.mly>"
    ( [ x ] )
# 21741 "asllib/Parser.ml"
       : (AST.slice list))
  
  let _menhir_action_439 =
    fun _2 e1 e2 ->
      let x = 
# 339 "asllib/Parser.mly"
                                    Slice_Length   
# 21749 "asllib/Parser.ml"
       (e1, e2) in
      (
# 250 "<standard.mly>"
    ( [ x ] )
# 21754 "asllib/Parser.ml"
       : (AST.slice list))
  
  let _menhir_action_440 =
    fun _1 e ->
      let x = 
# 340 "asllib/Parser.mly"
                                  ( Slice_Length(zero, e) )
# 21762 "asllib/Parser.ml"
       in
      (
# 250 "<standard.mly>"
    ( [ x ] )
# 21767 "asllib/Parser.ml"
       : (AST.slice list))
  
  let _menhir_action_441 =
    fun _2 e1 e2 ->
      let x = 
# 341 "asllib/Parser.mly"
                                    Slice_Star     
# 21775 "asllib/Parser.ml"
       (e1, e2) in
      (
# 250 "<standard.mly>"
    ( [ x ] )
# 21780 "asllib/Parser.ml"
       : (AST.slice list))
  
  let _menhir_action_442 =
    fun expr xs ->
      let x = 
# 337 "asllib/Parser.mly"
                                    Slice_Single   
# 21788 "asllib/Parser.ml"
       expr in
      (
# 253 "<standard.mly>"
    ( x :: xs )
# 21793 "asllib/Parser.ml"
       : (AST.slice list))
  
  let _menhir_action_443 =
    fun _2_inlined1 e1 e2 xs ->
      let x =
        let _2 = _2_inlined1 in
        
# 338 "asllib/Parser.mly"
                                    Slice_Range    
# 21803 "asllib/Parser.ml"
         (e1, e2)
      in
      (
# 253 "<standard.mly>"
    ( x :: xs )
# 21809 "asllib/Parser.ml"
       : (AST.slice list))
  
  let _menhir_action_444 =
    fun _2_inlined1 e1 e2 xs ->
      let x =
        let _2 = _2_inlined1 in
        
# 339 "asllib/Parser.mly"
                                    Slice_Length   
# 21819 "asllib/Parser.ml"
         (e1, e2)
      in
      (
# 253 "<standard.mly>"
    ( x :: xs )
# 21825 "asllib/Parser.ml"
       : (AST.slice list))
  
  let _menhir_action_445 =
    fun _1 e xs ->
      let x = 
# 340 "asllib/Parser.mly"
                                  ( Slice_Length(zero, e) )
# 21833 "asllib/Parser.ml"
       in
      (
# 253 "<standard.mly>"
    ( x :: xs )
# 21838 "asllib/Parser.ml"
       : (AST.slice list))
  
  let _menhir_action_446 =
    fun _2_inlined1 e1 e2 xs ->
      let x =
        let _2 = _2_inlined1 in
        
# 341 "asllib/Parser.mly"
                                    Slice_Star     
# 21848 "asllib/Parser.ml"
         (e1, e2)
      in
      (
# 253 "<standard.mly>"
    ( x :: xs )
# 21854 "asllib/Parser.ml"
       : (AST.slice list))
  
  let _menhir_action_447 =
    fun x ->
      (
# 250 "<standard.mly>"
    ( [ x ] )
# 21862 "asllib/Parser.ml"
       : (AST.ty list))
  
  let _menhir_action_448 =
    fun x xs ->
      (
# 253 "<standard.mly>"
    ( x :: xs )
# 21870 "asllib/Parser.ml"
       : (AST.ty list))
  
  let _menhir_action_449 =
    fun _1 _2 x ->
      let x =
        let y = _2 in
        
# 172 "<standard.mly>"
    ( (x, y) )
# 21880 "asllib/Parser.ml"
        
      in
      (
# 250 "<standard.mly>"
    ( [ x ] )
# 21886 "asllib/Parser.ml"
       : (AST.typed_identifier list))
  
  let _menhir_action_450 =
    fun _1 _2 x ->
      let x =
        let y = _2 in
        
# 172 "<standard.mly>"
    ( (x, y) )
# 21896 "asllib/Parser.ml"
        
      in
      (
# 250 "<standard.mly>"
    ( [ x ] )
# 21902 "asllib/Parser.ml"
       : (AST.typed_identifier list))
  
  let _menhir_action_451 =
    fun _1 _2_inlined1 x xs ->
      let x =
        let _2 = _2_inlined1 in
        let y = _2 in
        
# 172 "<standard.mly>"
    ( (x, y) )
# 21913 "asllib/Parser.ml"
        
      in
      (
# 253 "<standard.mly>"
    ( x :: xs )
# 21919 "asllib/Parser.ml"
       : (AST.typed_identifier list))
  
  let _menhir_action_452 =
    fun _1 _2_inlined1 x xs ->
      let x =
        let _2 = _2_inlined1 in
        let y = _2 in
        
# 172 "<standard.mly>"
    ( (x, y) )
# 21930 "asllib/Parser.ml"
        
      in
      (
# 253 "<standard.mly>"
    ( x :: xs )
# 21936 "asllib/Parser.ml"
       : (AST.typed_identifier list))
  
  let _menhir_action_453 =
    fun x ->
      (
# 196 "<standard.mly>"
    ( x )
# 21944 "asllib/Parser.ml"
       : (AST.t))
  
  let _menhir_action_454 =
    fun _1 ->
      let x =
        let _1_1 = _1 in
        
# 501 "asllib/Parser.mly"
                                            stmt_from_list 
# 21954 "asllib/Parser.ml"
         _1_1
      in
      (
# 196 "<standard.mly>"
    ( x )
# 21960 "asllib/Parser.ml"
       : (AST.stmt))
  
  let _menhir_action_455 =
    fun _1 _1_inlined2 _endpos__3_ _startpos__1_ ->
      let desc =
        let c =
          let _1_inlined1 = _1_inlined2 in
          let _1 =
            let x = _1_inlined1 in
            
# 205 "<standard.mly>"
    ( x )
# 21973 "asllib/Parser.ml"
            
          in
          let _1_1 = _1 in
          
# 301 "asllib/Parser.mly"
                                        WellConstrained  
# 21980 "asllib/Parser.ml"
           _1_1
        in
        
# 354 "asllib/Parser.mly"
                                                          T_Int        
# 21986 "asllib/Parser.ml"
         c
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 21995 "asllib/Parser.ml"
       : (AST.ty))
  
  let _menhir_action_456 =
    fun _1 _endpos__3_ _startpos__1_ x ->
      let desc =
        let c =
          let _1 = 
# 205 "<standard.mly>"
    ( x )
# 22005 "asllib/Parser.ml"
           in
          
# 302 "asllib/Parser.mly"
                   ( PendingConstrained )
# 22010 "asllib/Parser.ml"
          
        in
        
# 354 "asllib/Parser.mly"
                                                          T_Int        
# 22016 "asllib/Parser.ml"
         c
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 22025 "asllib/Parser.ml"
       : (AST.ty))
  
  let _menhir_action_457 =
    fun _1 _endpos__1_ _startpos__1_ ->
      let desc =
        let c = 
# 299 "asllib/Parser.mly"
                                             ( UnConstrained )
# 22034 "asllib/Parser.ml"
         in
        
# 354 "asllib/Parser.mly"
                                                          T_Int        
# 22039 "asllib/Parser.ml"
         c
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 22048 "asllib/Parser.ml"
       : (AST.ty))
  
  let _menhir_action_458 =
    fun _1 _endpos__1_ _startpos__1_ ->
      let desc = 
# 355 "asllib/Parser.mly"
                                                        ( T_Real      )
# 22056 "asllib/Parser.ml"
       in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 22064 "asllib/Parser.ml"
       : (AST.ty))
  
  let _menhir_action_459 =
    fun _1 _endpos__1_ _startpos__1_ ->
      let desc = 
# 356 "asllib/Parser.mly"
                                                        ( T_Bool      )
# 22072 "asllib/Parser.ml"
       in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 22080 "asllib/Parser.ml"
       : (AST.ty))
  
  let _menhir_action_460 =
    fun _1 _endpos__1_ _startpos__1_ ->
      let desc = 
# 357 "asllib/Parser.mly"
                                                        ( T_String    )
# 22088 "asllib/Parser.ml"
       in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 22096 "asllib/Parser.ml"
       : (AST.ty))
  
  let _menhir_action_461 =
    fun _1 _endpos__1_ _startpos__1_ ->
      let desc = 
# 358 "asllib/Parser.mly"
                                                        ( t_bit       )
# 22104 "asllib/Parser.ml"
       in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 22112 "asllib/Parser.ml"
       : (AST.ty))
  
  let _menhir_action_462 =
    fun _1 _1_inlined2 _endpos__1_inlined2_ _startpos__1_ x ->
      let desc =
        let bitfields_opt = _1_inlined2 in
        let _2 = 
# 205 "<standard.mly>"
    ( x )
# 22122 "asllib/Parser.ml"
         in
        let _2_1 = _2 in
        
# 359 "asllib/Parser.mly"
                                                          T_Bits       
# 22128 "asllib/Parser.ml"
         (_2_1, bitfields_opt)
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__1_inlined2_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 22137 "asllib/Parser.ml"
       : (AST.ty))
  
  let _menhir_action_463 =
    fun _endpos__3_ _startpos__1_ ->
      let desc =
        let l =
          let x = 
# 161 "asllib/Parser.mly"
                ( [] )
# 22147 "asllib/Parser.ml"
           in
          
# 205 "<standard.mly>"
    ( x )
# 22152 "asllib/Parser.ml"
          
        in
        
# 360 "asllib/Parser.mly"
                                                          T_Tuple      
# 22158 "asllib/Parser.ml"
         l
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 22167 "asllib/Parser.ml"
       : (AST.ty))
  
  let _menhir_action_464 =
    fun _1_inlined1 _endpos__3_ _startpos__1_ ->
      let desc =
        let l =
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 22178 "asllib/Parser.ml"
          
        in
        
# 360 "asllib/Parser.mly"
                                                          T_Tuple      
# 22184 "asllib/Parser.ml"
         l
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 22193 "asllib/Parser.ml"
       : (AST.ty))
  
  let _menhir_action_465 =
    fun _endpos_name_ _startpos_name_ name ->
      let desc = 
# 361 "asllib/Parser.mly"
                                                          T_Named      
# 22201 "asllib/Parser.ml"
       name in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_name_, _startpos_name_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 22209 "asllib/Parser.ml"
       : (AST.ty))
  
  let _menhir_action_466 =
    fun _1 _3 _endpos_t_ _startpos__1_ t x ->
      let desc =
        let e = 
# 205 "<standard.mly>"
    ( x )
# 22218 "asllib/Parser.ml"
         in
        
# 362 "asllib/Parser.mly"
                                                        ( T_Array (ArrayLength_Expr e, t) )
# 22223 "asllib/Parser.ml"
        
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos_t_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 22232 "asllib/Parser.ml"
       : (AST.ty))
  
  let _menhir_action_467 =
    fun _1 ->
      (_1 : (AST.ty))
  
  let _menhir_action_468 =
    fun _1 _1_inlined2 _endpos__3_ _startpos__1_ ->
      let desc =
        let l =
          let _1_inlined1 = _1_inlined2 in
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 22248 "asllib/Parser.ml"
          
        in
        
# 367 "asllib/Parser.mly"
                                                          T_Enum       
# 22254 "asllib/Parser.ml"
         l
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 22263 "asllib/Parser.ml"
       : (AST.ty))
  
  let _menhir_action_469 =
    fun _1 _endpos__1_ _startpos__1_ ->
      let desc =
        let l = 
# 332 "asllib/Parser.mly"
                  ( [] )
# 22272 "asllib/Parser.ml"
         in
        
# 368 "asllib/Parser.mly"
                                                          T_Record     
# 22277 "asllib/Parser.ml"
         l
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 22286 "asllib/Parser.ml"
       : (AST.ty))
  
  let _menhir_action_470 =
    fun _1 _1_inlined2 _endpos__3_ _startpos__1_ ->
      let desc =
        let l =
          let _1_inlined1 = _1_inlined2 in
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 22298 "asllib/Parser.ml"
          
        in
        
# 368 "asllib/Parser.mly"
                                                          T_Record     
# 22304 "asllib/Parser.ml"
         l
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 22313 "asllib/Parser.ml"
       : (AST.ty))
  
  let _menhir_action_471 =
    fun _1 _endpos__1_ _startpos__1_ ->
      let desc =
        let l = 
# 332 "asllib/Parser.mly"
                  ( [] )
# 22322 "asllib/Parser.ml"
         in
        
# 369 "asllib/Parser.mly"
                                                          T_Exception  
# 22327 "asllib/Parser.ml"
         l
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__1_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 22336 "asllib/Parser.ml"
       : (AST.ty))
  
  let _menhir_action_472 =
    fun _1 _1_inlined2 _endpos__3_ _startpos__1_ ->
      let desc =
        let l =
          let _1_inlined1 = _1_inlined2 in
          let x = _1_inlined1 in
          
# 205 "<standard.mly>"
    ( x )
# 22348 "asllib/Parser.ml"
          
        in
        
# 369 "asllib/Parser.mly"
                                                          T_Exception  
# 22354 "asllib/Parser.ml"
         l
      in
      let (_endpos_desc_, _startpos_desc_) = (_endpos__3_, _startpos__1_) in
      let _endpos = _endpos_desc_ in
      let _symbolstartpos = _startpos_desc_ in
      (
# 140 "asllib/Parser.mly"
                              ( { desc; pos_start=_symbolstartpos; pos_end=_endpos; version } )
# 22363 "asllib/Parser.ml"
       : (AST.ty))
  
  let _menhir_print_token : token -> string =
    fun _tok ->
      match _tok with
      | Tokens.AND ->
          "AND"
      | Tokens.ARBITRARY ->
          "ARBITRARY"
      | Tokens.ARRAY ->
          "ARRAY"
      | Tokens.ARROBASE_LOOPLIMIT ->
          "ARROBASE_LOOPLIMIT"
      | Tokens.ARROW ->
          "ARROW"
      | Tokens.AS ->
          "AS"
      | Tokens.ASSERT ->
          "ASSERT"
      | Tokens.BAND ->
          "BAND"
      | Tokens.BEGIN ->
          "BEGIN"
      | Tokens.BEQ ->
          "BEQ"
      | Tokens.BIT ->
          "BIT"
      | Tokens.BITS ->
          "BITS"
      | Tokens.BITVECTOR_LIT _ ->
          "BITVECTOR_LIT"
      | Tokens.BNOT ->
          "BNOT"
      | Tokens.BOOLEAN ->
          "BOOLEAN"
      | Tokens.BOOL_LIT _ ->
          "BOOL_LIT"
      | Tokens.BOR ->
          "BOR"
      | Tokens.CASE ->
          "CASE"
      | Tokens.CATCH ->
          "CATCH"
      | Tokens.COLON ->
          "COLON"
      | Tokens.COLON_COLON ->
          "COLON_COLON"
      | Tokens.COMMA ->
          "COMMA"
      | Tokens.CONCAT ->
          "CONCAT"
      | Tokens.CONFIG ->
          "CONFIG"
      | Tokens.CONSTANT ->
          "CONSTANT"
      | Tokens.DEBUG ->
          "DEBUG"
      | Tokens.DIV ->
          "DIV"
      | Tokens.DIVRM ->
          "DIVRM"
      | Tokens.DO ->
          "DO"
      | Tokens.DOT ->
          "DOT"
      | Tokens.DOWNTO ->
          "DOWNTO"
      | Tokens.ELSE ->
          "ELSE"
      | Tokens.ELSIF ->
          "ELSIF"
      | Tokens.END ->
          "END"
      | Tokens.ENUMERATION ->
          "ENUMERATION"
      | Tokens.EOF ->
          "EOF"
      | Tokens.EOR ->
          "EOR"
      | Tokens.EQ ->
          "EQ"
      | Tokens.EQ_OP ->
          "EQ_OP"
      | Tokens.EXCEPTION ->
          "EXCEPTION"
      | Tokens.FOR ->
          "FOR"
      | Tokens.FUNC ->
          "FUNC"
      | Tokens.GEQ ->
          "GEQ"
      | Tokens.GETTER ->
          "GETTER"
      | Tokens.GT ->
          "GT"
      | Tokens.IDENTIFIER _ ->
          "IDENTIFIER"
      | Tokens.IF ->
          "IF"
      | Tokens.IMPL ->
          "IMPL"
      | Tokens.IN ->
          "IN"
      | Tokens.INTEGER ->
          "INTEGER"
      | Tokens.INT_LIT _ ->
          "INT_LIT"
      | Tokens.LBRACE ->
          "LBRACE"
      | Tokens.LBRACKET ->
          "LBRACKET"
      | Tokens.LEQ ->
          "LEQ"
      | Tokens.LET ->
          "LET"
      | Tokens.LLBRACKET ->
          "LLBRACKET"
      | Tokens.LOOPLIMIT ->
          "LOOPLIMIT"
      | Tokens.LPAR ->
          "LPAR"
      | Tokens.LT ->
          "LT"
      | Tokens.MASK_LIT _ ->
          "MASK_LIT"
      | Tokens.MINUS ->
          "MINUS"
      | Tokens.MOD ->
          "MOD"
      | Tokens.MUL ->
          "MUL"
      | Tokens.NEQ ->
          "NEQ"
      | Tokens.NOT ->
          "NOT"
      | Tokens.OF ->
          "OF"
      | Tokens.OR ->
          "OR"
      | Tokens.OTHERWISE ->
          "OTHERWISE"
      | Tokens.PASS ->
          "PASS"
      | Tokens.PLUS ->
          "PLUS"
      | Tokens.PLUS_COLON ->
          "PLUS_COLON"
      | Tokens.POW ->
          "POW"
      | Tokens.PRAGMA ->
          "PRAGMA"
      | Tokens.PRINT ->
          "PRINT"
      | Tokens.RBRACE ->
          "RBRACE"
      | Tokens.RBRACKET ->
          "RBRACKET"
      | Tokens.RDIV ->
          "RDIV"
      | Tokens.REAL ->
          "REAL"
      | Tokens.REAL_LIT _ ->
          "REAL_LIT"
      | Tokens.RECORD ->
          "RECORD"
      | Tokens.RECURSELIMIT ->
          "RECURSELIMIT"
      | Tokens.REPEAT ->
          "REPEAT"
      | Tokens.RETURN ->
          "RETURN"
      | Tokens.RPAR ->
          "RPAR"
      | Tokens.RRBRACKET ->
          "RRBRACKET"
      | Tokens.SEMI_COLON ->
          "SEMI_COLON"
      | Tokens.SETTER ->
          "SETTER"
      | Tokens.SHL ->
          "SHL"
      | Tokens.SHR ->
          "SHR"
      | Tokens.SLICING ->
          "SLICING"
      | Tokens.STAR_COLON ->
          "STAR_COLON"
      | Tokens.STRING ->
          "STRING"
      | Tokens.STRING_LIT _ ->
          "STRING_LIT"
      | Tokens.SUBTYPES ->
          "SUBTYPES"
      | Tokens.THEN ->
          "THEN"
      | Tokens.THROW ->
          "THROW"
      | Tokens.TO ->
          "TO"
      | Tokens.TRY ->
          "TRY"
      | Tokens.TYPE ->
          "TYPE"
      | Tokens.UNREACHABLE ->
          "UNREACHABLE"
      | Tokens.UNTIL ->
          "UNTIL"
      | Tokens.VAR ->
          "VAR"
      | Tokens.WHEN ->
          "WHEN"
      | Tokens.WHERE ->
          "WHERE"
      | Tokens.WHILE ->
          "WHILE"
      | Tokens.WITH ->
          "WITH"
  
  let _menhir_fail : unit -> 'a =
    fun () ->
      Printf.eprintf "Internal failure -- please contact the parser generator's developers.\n%!";
      assert false
  
  include struct
    
    [@@@ocaml.warning "-4-37"]
    
    let _menhir_goto_opn : type  ttv_stack. ttv_stack -> _ -> _menhir_box_opn =
      fun _menhir_stack _v ->
        MenhirBox_opn _v
    
    let _menhir_run_1379 : type  ttv_stack. ttv_stack -> _ -> _ -> _menhir_box_stmts =
      fun _menhir_stack _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.EOF ->
            let _1 = _v in
            let _v = _menhir_action_454 _1 in
            MenhirBox_stmts _v
        | _ ->
            _eRR ()
    
    let _menhir_run_1375 : type  ttv_stack. ttv_stack -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let x = _v in
        let _v = _menhir_action_453 x in
        MenhirBox_spec _v
    
    let rec _menhir_run_1276 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_PRAGMA _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
        let MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs) = ((), _v) in
        let _v = _menhir_action_188 _1 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_goto_list_decl_ : type  ttv_stack. ttv_stack -> _ -> (ttv_stack, _menhir_box_spec) _menhir_state -> _menhir_box_spec =
      fun _menhir_stack _v _menhir_s ->
        match _menhir_s with
        | MenhirState1033 ->
            _menhir_run_1375 _menhir_stack _v
        | MenhirState1372 ->
            _menhir_run_1373 _menhir_stack _v
        | MenhirState1368 ->
            _menhir_run_1369 _menhir_stack _v
        | MenhirState1364 ->
            _menhir_run_1365 _menhir_stack _v
        | MenhirState1360 ->
            _menhir_run_1361 _menhir_stack _v
        | MenhirState1356 ->
            _menhir_run_1357 _menhir_stack _v
        | MenhirState1351 ->
            _menhir_run_1352 _menhir_stack _v
        | MenhirState1347 ->
            _menhir_run_1348 _menhir_stack _v
        | MenhirState1343 ->
            _menhir_run_1344 _menhir_stack _v
        | MenhirState1339 ->
            _menhir_run_1340 _menhir_stack _v
        | MenhirState1038 ->
            _menhir_run_1336 _menhir_stack _v
        | MenhirState1333 ->
            _menhir_run_1334 _menhir_stack _v
        | MenhirState1058 ->
            _menhir_run_1311 _menhir_stack _v
        | MenhirState1309 ->
            _menhir_run_1310 _menhir_stack _v
        | MenhirState1307 ->
            _menhir_run_1308 _menhir_stack _v
        | MenhirState1301 ->
            _menhir_run_1302 _menhir_stack _v
        | MenhirState1299 ->
            _menhir_run_1300 _menhir_stack _v
        | MenhirState1289 ->
            _menhir_run_1290 _menhir_stack _v
        | MenhirState1287 ->
            _menhir_run_1288 _menhir_stack _v
        | MenhirState1281 ->
            _menhir_run_1282 _menhir_stack _v
        | MenhirState1084 ->
            _menhir_run_1280 _menhir_stack _v
        | MenhirState1278 ->
            _menhir_run_1279 _menhir_stack _v
        | MenhirState1087 ->
            _menhir_run_1276 _menhir_stack _v
        | MenhirState1274 ->
            _menhir_run_1275 _menhir_stack _v
        | MenhirState1268 ->
            _menhir_run_1269 _menhir_stack _v
        | MenhirState1262 ->
            _menhir_run_1263 _menhir_stack _v
        | MenhirState1257 ->
            _menhir_run_1258 _menhir_stack _v
        | MenhirState1251 ->
            _menhir_run_1252 _menhir_stack _v
        | MenhirState1092 ->
            _menhir_run_1246 _menhir_stack _v
        | MenhirState1244 ->
            _menhir_run_1245 _menhir_stack _v
        | MenhirState1242 ->
            _menhir_run_1243 _menhir_stack _v
        | MenhirState1234 ->
            _menhir_run_1235 _menhir_stack _v
        | MenhirState1102 ->
            _menhir_run_1233 _menhir_stack _v
        | MenhirState1231 ->
            _menhir_run_1232 _menhir_stack _v
        | MenhirState1229 ->
            _menhir_run_1230 _menhir_stack _v
        | MenhirState1225 ->
            _menhir_run_1226 _menhir_stack _v
        | MenhirState1223 ->
            _menhir_run_1224 _menhir_stack _v
        | MenhirState1215 ->
            _menhir_run_1216 _menhir_stack _v
        | MenhirState1213 ->
            _menhir_run_1214 _menhir_stack _v
        | MenhirState1198 ->
            _menhir_run_1199 _menhir_stack _v
        | MenhirState1196 ->
            _menhir_run_1197 _menhir_stack _v
        | MenhirState1192 ->
            _menhir_run_1193 _menhir_stack _v
        | MenhirState1190 ->
            _menhir_run_1191 _menhir_stack _v
        | MenhirState1182 ->
            _menhir_run_1183 _menhir_stack _v
        | MenhirState1110 ->
            _menhir_run_1181 _menhir_stack _v
        | MenhirState1179 ->
            _menhir_run_1180 _menhir_stack _v
        | MenhirState1173 ->
            _menhir_run_1174 _menhir_stack _v
        | MenhirState1167 ->
            _menhir_run_1168 _menhir_stack _v
        | MenhirState1162 ->
            _menhir_run_1163 _menhir_stack _v
        | MenhirState1156 ->
            _menhir_run_1157 _menhir_stack _v
        | MenhirState1115 ->
            _menhir_run_1151 _menhir_stack _v
        | MenhirState1149 ->
            _menhir_run_1150 _menhir_stack _v
        | MenhirState1143 ->
            _menhir_run_1144 _menhir_stack _v
        | MenhirState1137 ->
            _menhir_run_1138 _menhir_stack _v
        | MenhirState1132 ->
            _menhir_run_1133 _menhir_stack _v
        | MenhirState1126 ->
            _menhir_run_1127 _menhir_stack _v
        | MenhirState1120 ->
            _menhir_run_1121 _menhir_stack _v
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_1373 : type  ttv_stack. ((((ttv_stack, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, _1_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined2) = ((), _v, (), ()) in
        let _v = _menhir_action_176 _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1369 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_SEMI_COLON -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, _1_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined2) = ((), _v, ()) in
        let _v = _menhir_action_186 _1 _1_inlined1 _1_inlined2 _2_inlined1 _endpos__2_ _startpos__1_ xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1365 : type  ttv_stack. ((((ttv_stack, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, _1_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined2) = ((), _v, (), ()) in
        let _v = _menhir_action_177 _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1361 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_SEMI_COLON -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, _1_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined2) = ((), _v, ()) in
        let _v = _menhir_action_187 _1 _1_inlined1 _1_inlined2 _2_inlined1 _endpos__2_ _startpos__1_ xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1357 : type  ttv_stack. (((ttv_stack, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, _1_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4) = ((), _v, ()) in
        let _v = _menhir_action_175 _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1352 : type  ttv_stack. ((((ttv_stack, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_MINUS (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined2, _1_inlined1) = ((), _v, (), (), ()) in
        let _v = _menhir_action_173 _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1348 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_SEMI_COLON -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_MINUS (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined2, _1_inlined1) = ((), _v, (), ()) in
        let _v = _menhir_action_184 _1 _1_inlined1 _1_inlined2 _2_inlined1 _endpos__2_ _startpos__1_ xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1344 : type  ttv_stack. ((((ttv_stack, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_MINUS (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined2, _1_inlined1) = ((), _v, (), (), ()) in
        let _v = _menhir_action_174 _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1340 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_SEMI_COLON -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_MINUS (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined2, _1_inlined1) = ((), _v, (), ()) in
        let _v = _menhir_action_185 _1 _1_inlined1 _1_inlined2 _2_inlined1 _endpos__2_ _startpos__1_ xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1336 : type  ttv_stack. (((ttv_stack, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell0_MINUS (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined1) = ((), _v, (), ()) in
        let _v = _menhir_action_172 _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1334 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_TYPE _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty_decl _menhir_cell0_option_subtype_ _menhir_cell0_SEMI_COLON -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell0_option_subtype_ (_menhir_stack, _1_inlined1) = _menhir_stack in
        let MenhirCell1_ty_decl (_menhir_stack, _, t) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
        let MenhirCell1_TYPE (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _3) = ((), _v, ()) in
        let _v = _menhir_action_158 _1 _1_inlined1 _3 _endpos__2_ _startpos__1_ t x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1311 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_TYPE _menhir_cell0_IDENTIFIER _menhir_cell0_SUBTYPES _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption___anonymous_21_ _menhir_cell0_SEMI_COLON -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_loption___anonymous_21_ (_menhir_stack, _, _3, _endpos__3_) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, _2_inlined1, _, _) = _menhir_stack in
        let MenhirCell0_SUBTYPES (_menhir_stack, _startpos__1_inlined1_) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
        let MenhirCell1_TYPE (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined1) = ((), _v, ()) in
        let _v = _menhir_action_159 _1 _1_inlined1 _2_inlined1 _3 _endpos__2_ _endpos__3_ _startpos__1_ _startpos__1_inlined1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1310 : type  ttv_stack. (((((ttv_stack, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_end_semicolon -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _endpos__1_inlined5_) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_typed_identifier_ (_menhir_stack, _, _1_inlined3) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_SETTER (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined4, _5) = ((), _v, (), ()) in
        let _v = _menhir_action_155 _1 _1_inlined1 _1_inlined3 _1_inlined4 _2 _5 _endpos__1_inlined5_ _endpos__3_inlined1_ _startpos__1_ name x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1308 : type  ttv_stack. ((((((ttv_stack, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_, _menhir_box_spec) _menhir_cell1_end_semicolon -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined6, _) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_typed_identifier_ (_menhir_stack, _, _1_inlined3) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_SETTER (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined4, _5) = ((), _v, (), ()) in
        let _v = _menhir_action_154 _1 _1_inlined1 _1_inlined3 _1_inlined4 _1_inlined6 _2 _5 _endpos__3_inlined1_ _startpos__1_ name x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1302 : type  ttv_stack. (((((ttv_stack, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_end_semicolon -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _endpos__1_inlined5_) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_typed_identifier_ (_menhir_stack, _, _1_inlined3) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_SETTER (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined4, _5) = ((), _v, (), ()) in
        let _v = _menhir_action_157 _1 _1_inlined1 _1_inlined3 _1_inlined4 _2 _5 _endpos__1_inlined5_ _endpos__3_inlined1_ _startpos__1_ name x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1300 : type  ttv_stack. ((((((ttv_stack, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_, _menhir_box_spec) _menhir_cell1_end_semicolon -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined6, _) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_typed_identifier_ (_menhir_stack, _, _1_inlined3) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_SETTER (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined4, _5) = ((), _v, (), ()) in
        let _v = _menhir_action_156 _1 _1_inlined1 _1_inlined3 _1_inlined4 _1_inlined6 _2 _5 _endpos__3_inlined1_ _startpos__1_ name x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1290 : type  ttv_stack. (((((ttv_stack, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_end_semicolon -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _endpos__1_inlined4_) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_SETTER (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _5, _1_inlined3) = ((), _v, (), ()) in
        let _v = _menhir_action_151 _1 _1_inlined1 _1_inlined3 _2 _5 _endpos__1_inlined4_ _endpos__3_inlined1_ _startpos__1_ name x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1288 : type  ttv_stack. ((((((ttv_stack, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_, _menhir_box_spec) _menhir_cell1_end_semicolon -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined5, _) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_SETTER (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _5, _1_inlined3) = ((), _v, (), ()) in
        let _v = _menhir_action_150 _1 _1_inlined1 _1_inlined3 _1_inlined5 _2 _5 _endpos__3_inlined1_ _startpos__1_ name x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1282 : type  ttv_stack. (((((ttv_stack, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_end_semicolon -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _endpos__1_inlined4_) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_SETTER (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _5, _1_inlined3) = ((), _v, (), ()) in
        let _v = _menhir_action_153 _1 _1_inlined1 _1_inlined3 _2 _5 _endpos__1_inlined4_ _endpos__3_inlined1_ _startpos__1_ name x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1280 : type  ttv_stack. ((((((ttv_stack, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_, _menhir_box_spec) _menhir_cell1_end_semicolon -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined5, _) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_SETTER (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _5, _1_inlined3) = ((), _v, (), ()) in
        let _v = _menhir_action_152 _1 _1_inlined1 _1_inlined3 _1_inlined5 _2 _5 _endpos__3_inlined1_ _startpos__1_ name x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1279 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_PRAGMA _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_SEMI_COLON -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
        let MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs) = ((), _v) in
        let _v = _menhir_action_189 _1 _1_inlined1 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1275 : type  ttv_stack. ((((ttv_stack, _menhir_box_spec) _menhir_cell1_LET _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, _1_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_LET (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined2) = ((), _v, (), ()) in
        let _v = _menhir_action_164 _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1269 : type  ttv_stack. ((((ttv_stack, _menhir_box_spec) _menhir_cell1_LET _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, _1_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_LET (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined2) = ((), _v, (), ()) in
        let _v = _menhir_action_165 _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1263 : type  ttv_stack. (((ttv_stack, _menhir_box_spec) _menhir_cell1_LET _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, _1_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_LET (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4) = ((), _v, ()) in
        let _v = _menhir_action_163 _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1258 : type  ttv_stack. ((((ttv_stack, _menhir_box_spec) _menhir_cell1_LET _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_MINUS (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_LET (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined2, _1_inlined1) = ((), _v, (), (), ()) in
        let _v = _menhir_action_161 _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1252 : type  ttv_stack. ((((ttv_stack, _menhir_box_spec) _menhir_cell1_LET _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_MINUS (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_LET (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined2, _1_inlined1) = ((), _v, (), (), ()) in
        let _v = _menhir_action_162 _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1246 : type  ttv_stack. (((ttv_stack, _menhir_box_spec) _menhir_cell1_LET _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell0_MINUS (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_LET (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined1) = ((), _v, (), ()) in
        let _v = _menhir_action_160 _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1245 : type  ttv_stack. (((((ttv_stack, _menhir_box_spec) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_end_semicolon -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _endpos__1_inlined5_) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_typed_identifier_ (_menhir_stack, _, _1_inlined3) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_GETTER (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined4) = ((), _v, ()) in
        let _v = _menhir_action_149 _1 _1_inlined1 _1_inlined3 _1_inlined4 _2 _endpos__1_inlined5_ _endpos__3_inlined1_ _startpos__1_ name xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1243 : type  ttv_stack. ((((((ttv_stack, _menhir_box_spec) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_, _menhir_box_spec) _menhir_cell1_end_semicolon -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined6, _) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_typed_identifier_ (_menhir_stack, _, _1_inlined3) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_GETTER (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined4) = ((), _v, ()) in
        let _v = _menhir_action_148 _1 _1_inlined1 _1_inlined3 _1_inlined4 _1_inlined6 _2 _endpos__3_inlined1_ _startpos__1_ name xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1235 : type  ttv_stack. (((((ttv_stack, _menhir_box_spec) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_end_semicolon -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _endpos__1_inlined4_) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_GETTER (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined3) = ((), _v, ()) in
        let _v = _menhir_action_147 _1 _1_inlined1 _1_inlined3 _2 _endpos__1_inlined4_ _endpos__3_inlined1_ _startpos__1_ name xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1233 : type  ttv_stack. ((((((ttv_stack, _menhir_box_spec) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_, _menhir_box_spec) _menhir_cell1_end_semicolon -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined5, _) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_GETTER (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined3) = ((), _v, ()) in
        let _v = _menhir_action_146 _1 _1_inlined1 _1_inlined3 _1_inlined5 _2 _endpos__3_inlined1_ _startpos__1_ name xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1232 : type  ttv_stack. (((((ttv_stack, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_end_semicolon -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _endpos__1_inlined5_) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_typed_identifier_ (_menhir_stack, _, _1_inlined3) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_FUNC (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined4) = ((), _v, ()) in
        let _v = _menhir_action_139 _1 _1_inlined1 _1_inlined3 _1_inlined4 _2 _endpos__1_inlined5_ _endpos__3_inlined1_ _startpos__1_ name xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1230 : type  ttv_stack. ((((((ttv_stack, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_, _menhir_box_spec) _menhir_cell1_end_semicolon -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined6, _) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_typed_identifier_ (_menhir_stack, _, _1_inlined3) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_FUNC (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined4) = ((), _v, ()) in
        let _v = _menhir_action_138 _1 _1_inlined1 _1_inlined3 _1_inlined4 _1_inlined6 _2 _endpos__3_inlined1_ _startpos__1_ name xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1226 : type  ttv_stack. (((((((ttv_stack, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_BEGIN, _menhir_box_spec) _menhir_cell1_end_semicolon -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell1_BEGIN (_menhir_stack, _, _endpos__1_inlined6_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, _2_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_typed_identifier_ (_menhir_stack, _, _1_inlined3) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_FUNC (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined5, _1_inlined4) = ((), _v, (), ()) in
        let _v = _menhir_action_141 _1 _1_inlined1 _1_inlined3 _1_inlined4 _1_inlined5 _2 _2_inlined1 _endpos__1_inlined6_ _endpos__3_inlined1_ _startpos__1_ name xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1224 : type  ttv_stack. ((((((((ttv_stack, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_, _menhir_box_spec) _menhir_cell1_end_semicolon -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined7, _) = _menhir_stack in
        let MenhirCell1_BEGIN (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, _2_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_typed_identifier_ (_menhir_stack, _, _1_inlined3) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_FUNC (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined5, _1_inlined4) = ((), _v, (), ()) in
        let _v = _menhir_action_140 _1 _1_inlined1 _1_inlined3 _1_inlined4 _1_inlined5 _1_inlined7 _2 _2_inlined1 _endpos__3_inlined1_ _startpos__1_ name xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1216 : type  ttv_stack. ((((ttv_stack, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_end_semicolon -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _endpos__1_inlined4_) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_typed_identifier_ (_menhir_stack, _, _1_inlined3) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_FUNC (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs) = ((), _v) in
        let _v = _menhir_action_145 _1 _1_inlined1 _1_inlined3 _endpos__1_inlined4_ _endpos__3_inlined1_ _startpos__1_ name xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1214 : type  ttv_stack. (((((ttv_stack, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_, _menhir_box_spec) _menhir_cell1_end_semicolon -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined5, _) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_typed_identifier_ (_menhir_stack, _, _1_inlined3) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_FUNC (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs) = ((), _v) in
        let _v = _menhir_action_144 _1 _1_inlined1 _1_inlined3 _1_inlined5 _endpos__3_inlined1_ _startpos__1_ name xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1199 : type  ttv_stack. (((((ttv_stack, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_end_semicolon -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _endpos__1_inlined4_) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_FUNC (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined3) = ((), _v, ()) in
        let _v = _menhir_action_135 _1 _1_inlined1 _1_inlined3 _2 _endpos__1_inlined4_ _endpos__3_inlined1_ _startpos__1_ name xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1197 : type  ttv_stack. ((((((ttv_stack, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_, _menhir_box_spec) _menhir_cell1_end_semicolon -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined5, _) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_FUNC (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined3) = ((), _v, ()) in
        let _v = _menhir_action_134 _1 _1_inlined1 _1_inlined3 _1_inlined5 _2 _endpos__3_inlined1_ _startpos__1_ name xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1193 : type  ttv_stack. (((((((ttv_stack, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_BEGIN, _menhir_box_spec) _menhir_cell1_end_semicolon -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell1_BEGIN (_menhir_stack, _, _endpos__1_inlined5_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, _2_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_FUNC (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined4, _1_inlined3) = ((), _v, (), ()) in
        let _v = _menhir_action_137 _1 _1_inlined1 _1_inlined3 _1_inlined4 _2 _2_inlined1 _endpos__1_inlined5_ _endpos__3_inlined1_ _startpos__1_ name xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1191 : type  ttv_stack. ((((((((ttv_stack, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_, _menhir_box_spec) _menhir_cell1_end_semicolon -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined6, _) = _menhir_stack in
        let MenhirCell1_BEGIN (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, _2_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2, _) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_FUNC (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _1_inlined4, _1_inlined3) = ((), _v, (), ()) in
        let _v = _menhir_action_136 _1 _1_inlined1 _1_inlined3 _1_inlined4 _1_inlined6 _2 _2_inlined1 _endpos__3_inlined1_ _startpos__1_ name xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1183 : type  ttv_stack. ((((ttv_stack, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_end_semicolon -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _endpos__1_inlined3_) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_FUNC (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs) = ((), _v) in
        let _v = _menhir_action_143 _1 _1_inlined1 _endpos__1_inlined3_ _endpos__3_inlined1_ _startpos__1_ name xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1181 : type  ttv_stack. (((((ttv_stack, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_, _menhir_box_spec) _menhir_cell1_end_semicolon -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__3_inlined1_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined4, _) = _menhir_stack in
        let MenhirCell0_BEGIN (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, name, _, _) = _menhir_stack in
        let MenhirCell1_FUNC (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs) = ((), _v) in
        let _v = _menhir_action_142 _1 _1_inlined1 _1_inlined4 _endpos__3_inlined1_ _startpos__1_ name xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1180 : type  ttv_stack. ((((ttv_stack, _menhir_box_spec) _menhir_cell1_CONSTANT _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, _1_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_CONSTANT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined2) = ((), _v, (), ()) in
        let _v = _menhir_action_170 _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1174 : type  ttv_stack. ((((ttv_stack, _menhir_box_spec) _menhir_cell1_CONSTANT _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, _1_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_CONSTANT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined2) = ((), _v, (), ()) in
        let _v = _menhir_action_171 _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1168 : type  ttv_stack. (((ttv_stack, _menhir_box_spec) _menhir_cell1_CONSTANT _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, _1_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_CONSTANT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4) = ((), _v, ()) in
        let _v = _menhir_action_169 _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1163 : type  ttv_stack. ((((ttv_stack, _menhir_box_spec) _menhir_cell1_CONSTANT _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_MINUS (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_CONSTANT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined2, _1_inlined1) = ((), _v, (), (), ()) in
        let _v = _menhir_action_167 _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1157 : type  ttv_stack. ((((ttv_stack, _menhir_box_spec) _menhir_cell1_CONSTANT _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_MINUS (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_CONSTANT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined2, _1_inlined1) = ((), _v, (), (), ()) in
        let _v = _menhir_action_168 _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1151 : type  ttv_stack. (((ttv_stack, _menhir_box_spec) _menhir_cell1_CONSTANT _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell0_MINUS (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_CONSTANT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined1) = ((), _v, (), ()) in
        let _v = _menhir_action_166 _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1150 : type  ttv_stack. ((((ttv_stack, _menhir_box_spec) _menhir_cell1_CONFIG _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, _1_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_CONFIG (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined2) = ((), _v, (), ()) in
        let _v = _menhir_action_182 _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1144 : type  ttv_stack. ((((ttv_stack, _menhir_box_spec) _menhir_cell1_CONFIG _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, _1_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_CONFIG (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined2) = ((), _v, (), ()) in
        let _v = _menhir_action_183 _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1138 : type  ttv_stack. (((ttv_stack, _menhir_box_spec) _menhir_cell1_CONFIG _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, _1_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_CONFIG (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4) = ((), _v, ()) in
        let _v = _menhir_action_181 _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1133 : type  ttv_stack. ((((ttv_stack, _menhir_box_spec) _menhir_cell1_CONFIG _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_MINUS (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_CONFIG (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined2, _1_inlined1) = ((), _v, (), (), ()) in
        let _v = _menhir_action_179 _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1127 : type  ttv_stack. ((((ttv_stack, _menhir_box_spec) _menhir_cell1_CONFIG _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_MINUS (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_CONFIG (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined2, _1_inlined1) = ((), _v, (), (), ()) in
        let _v = _menhir_action_180 _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    and _menhir_run_1121 : type  ttv_stack. (((ttv_stack, _menhir_box_spec) _menhir_cell1_CONFIG _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_SEMI_COLON -> _ -> _menhir_box_spec =
      fun _menhir_stack _v ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell0_MINUS (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_CONFIG (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, xs, _4, _1_inlined1) = ((), _v, (), ()) in
        let _v = _menhir_action_178 _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_list_decl_ _menhir_stack _v _menhir_s
    
    let _menhir_run_1019 : type  ttv_stack. ((((((ttv_stack, _menhir_box_opn) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_WHILE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _endpos _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.EOF ->
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined2, _) = _menhir_stack in
            let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, cond, _, _) = _menhir_stack in
            let MenhirCell0_WHILE (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
            let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_ARROBASE_LOOPLIMIT (_menhir_stack, _, _startpos__1_) = _menhir_stack in
            let (_1, _2, _endpos__2_inlined1_, _3, _5) = ((), (), _endpos, (), ()) in
            let _v = _menhir_action_354 _1 _1_inlined2 _2 _3 _5 _endpos__2_inlined1_ _startpos__1_ cond x in
            _menhir_goto_opn _menhir_stack _v
        | _ ->
            _eRR ()
    
    let _menhir_run_1005 : type  ttv_stack. ((((ttv_stack, _menhir_box_opn) _menhir_cell1_CASE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_OF, _menhir_box_opn) _menhir_cell1_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _endpos _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.EOF ->
            let MenhirCell1_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
            let MenhirCell1_OF (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, expr, _, _) = _menhir_stack in
            let MenhirCell1_CASE (_menhir_stack, _, _startpos__1_) = _menhir_stack in
            let (_1, _2, _endpos__2_inlined1_, _3) = ((), (), _endpos, ()) in
            let _v = _menhir_action_345 _1 _1_inlined1 _2 _3 _endpos__2_inlined1_ _startpos__1_ expr in
            _menhir_goto_opn _menhir_stack _v
        | _ ->
            _eRR ()
    
    let _menhir_run_0967 : type  ttv_stack. ((((((ttv_stack, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DOWNTO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _endpos _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.EOF ->
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined2, _) = _menhir_stack in
            let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, end_e, _, _) = _menhir_stack in
            let MenhirCell1_DOWNTO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, start_e, _, _) = _menhir_stack in
            let MenhirCell0_IDENTIFIER (_menhir_stack, index_name, _, _) = _menhir_stack in
            let MenhirCell1_FOR (_menhir_stack, _, _startpos__1_) = _menhir_stack in
            let (_1, _2, _endpos__2_inlined1_, _1_inlined1, _3, _8) = ((), (), _endpos, (), (), ()) in
            let _v = _menhir_action_350 _1 _1_inlined1 _1_inlined2 _2 _3 _8 _endpos__2_inlined1_ _startpos__1_ end_e index_name start_e in
            _menhir_goto_opn _menhir_stack _v
        | _ ->
            _eRR ()
    
    let _menhir_run_0963 : type  ttv_stack. ((((((((ttv_stack, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DOWNTO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _endpos _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.EOF ->
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined3, _) = _menhir_stack in
            let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, _2_inlined1, _, _) = _menhir_stack in
            let MenhirCell1_LOOPLIMIT (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, end_e, _, _) = _menhir_stack in
            let MenhirCell1_DOWNTO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, start_e, _, _) = _menhir_stack in
            let MenhirCell0_IDENTIFIER (_menhir_stack, index_name, _, _) = _menhir_stack in
            let MenhirCell1_FOR (_menhir_stack, _, _startpos__1_) = _menhir_stack in
            let (_1, _2, _1_inlined2, _1_inlined1, _3, _endpos__2_inlined2_, _8) = ((), (), (), (), (), _endpos, ()) in
            let _v = _menhir_action_351 _1 _1_inlined1 _1_inlined2 _1_inlined3 _2 _2_inlined1 _3 _8 _endpos__2_inlined2_ _startpos__1_ end_e index_name start_e in
            _menhir_goto_opn _menhir_stack _v
        | _ ->
            _eRR ()
    
    let _menhir_run_0955 : type  ttv_stack. ((((((ttv_stack, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_TO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _endpos _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.EOF ->
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined2, _) = _menhir_stack in
            let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, end_e, _, _) = _menhir_stack in
            let MenhirCell1_TO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, start_e, _, _) = _menhir_stack in
            let MenhirCell0_IDENTIFIER (_menhir_stack, index_name, _, _) = _menhir_stack in
            let MenhirCell1_FOR (_menhir_stack, _, _startpos__1_) = _menhir_stack in
            let (_1, _2, _endpos__2_inlined1_, _1_inlined1, _3, _8) = ((), (), _endpos, (), (), ()) in
            let _v = _menhir_action_348 _1 _1_inlined1 _1_inlined2 _2 _3 _8 _endpos__2_inlined1_ _startpos__1_ end_e index_name start_e in
            _menhir_goto_opn _menhir_stack _v
        | _ ->
            _eRR ()
    
    let _menhir_run_0951 : type  ttv_stack. ((((((((ttv_stack, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_TO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _endpos _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.EOF ->
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined3, _) = _menhir_stack in
            let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, _2_inlined1, _, _) = _menhir_stack in
            let MenhirCell1_LOOPLIMIT (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, end_e, _, _) = _menhir_stack in
            let MenhirCell1_TO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, start_e, _, _) = _menhir_stack in
            let MenhirCell0_IDENTIFIER (_menhir_stack, index_name, _, _) = _menhir_stack in
            let MenhirCell1_FOR (_menhir_stack, _, _startpos__1_) = _menhir_stack in
            let (_1, _2, _1_inlined2, _1_inlined1, _3, _endpos__2_inlined2_, _8) = ((), (), (), (), (), _endpos, ()) in
            let _v = _menhir_action_349 _1 _1_inlined1 _1_inlined2 _1_inlined3 _2 _2_inlined1 _3 _8 _endpos__2_inlined2_ _startpos__1_ end_e index_name start_e in
            _menhir_goto_opn _menhir_stack _v
        | _ ->
            _eRR ()
    
    let _menhir_run_0867 : type  ttv_stack. (((((ttv_stack, _menhir_box_opn) _menhir_cell1_IF, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_THEN, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_cell1_s_else -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _endpos _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.EOF ->
            let MenhirCell1_s_else (_menhir_stack, _, s2, _) = _menhir_stack in
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
            let MenhirCell1_THEN (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, e, _, _) = _menhir_stack in
            let MenhirCell1_IF (_menhir_stack, _, _startpos__1_) = _menhir_stack in
            let (_1, _2, _endpos__2_inlined1_, _3) = ((), (), _endpos, ()) in
            let _v = _menhir_action_344 _1 _1_inlined1 _2 _3 _endpos__2_inlined1_ _startpos__1_ e s2 in
            _menhir_goto_opn _menhir_stack _v
        | _ ->
            _eRR ()
    
    let _menhir_run_0796 : type  ttv_stack. (((ttv_stack, _menhir_box_opn) _menhir_cell1_TRY, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_cell1_nonempty_list_catcher_ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _endpos _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.EOF ->
            let MenhirCell1_nonempty_list_catcher_ (_menhir_stack, _, c) = _menhir_stack in
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
            let MenhirCell1_TRY (_menhir_stack, _, _startpos__1_) = _menhir_stack in
            let (_1, _2, _endpos__2_inlined1_, _3) = ((), (), _endpos, ()) in
            let _v = _menhir_action_352 _1 _1_inlined1 _2 _3 _endpos__2_inlined1_ _startpos__1_ c in
            _menhir_goto_opn _menhir_stack _v
        | _ ->
            _eRR ()
    
    let _menhir_run_0794 : type  ttv_stack. (((((ttv_stack, _menhir_box_opn) _menhir_cell1_TRY, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_cell1_nonempty_list_catcher_, _menhir_box_opn) _menhir_cell1_OTHERWISE, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _endpos _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.EOF ->
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined3, _) = _menhir_stack in
            let MenhirCell1_OTHERWISE (_menhir_stack, _, _, _) = _menhir_stack in
            let MenhirCell1_nonempty_list_catcher_ (_menhir_stack, _, c) = _menhir_stack in
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
            let MenhirCell1_TRY (_menhir_stack, _, _startpos__1_) = _menhir_stack in
            let (_1, _2, _2_inlined1, _1_inlined2, _3, _endpos__2_inlined2_) = ((), (), (), (), (), _endpos) in
            let _v = _menhir_action_353 _1 _1_inlined1 _1_inlined2 _1_inlined3 _2 _2_inlined1 _3 _endpos__2_inlined2_ _startpos__1_ c in
            _menhir_goto_opn _menhir_stack _v
        | _ ->
            _eRR ()
    
    let _menhir_run_0744 : type  ttv_stack. ((((ttv_stack, _menhir_box_opn) _menhir_cell1_WHILE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _endpos _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.EOF ->
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
            let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, expr, _, _) = _menhir_stack in
            let MenhirCell1_WHILE (_menhir_stack, _, _startpos__1_) = _menhir_stack in
            let (_1, _2, _endpos__2_inlined1_, _4) = ((), (), _endpos, ()) in
            let _v = _menhir_action_346 _1 _1_inlined1 _2 _4 _endpos__2_inlined1_ _startpos__1_ expr in
            _menhir_goto_opn _menhir_stack _v
        | _ ->
            _eRR ()
    
    let _menhir_run_0740 : type  ttv_stack. ((((((ttv_stack, _menhir_box_opn) _menhir_cell1_WHILE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _endpos _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.EOF ->
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined2, _) = _menhir_stack in
            let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, _2_inlined1, _, _) = _menhir_stack in
            let MenhirCell1_LOOPLIMIT (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, expr, _, _) = _menhir_stack in
            let MenhirCell1_WHILE (_menhir_stack, _, _startpos__1_) = _menhir_stack in
            let (_1, _2, _4, _1_inlined1, _endpos__2_inlined2_) = ((), (), (), (), _endpos) in
            let _v = _menhir_action_347 _1 _1_inlined1 _1_inlined2 _2 _2_inlined1 _4 _endpos__2_inlined2_ _startpos__1_ expr in
            _menhir_goto_opn _menhir_stack _v
        | _ ->
            _eRR ()
    
    let _menhir_run_0755 : type  ttv_stack. (((ttv_stack, _menhir_box_opn) _menhir_cell1_VAR, _menhir_box_opn) _menhir_cell1_IDENTIFIER, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ -> _ -> _ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, x, _, _) = _menhir_stack in
                let MenhirCell1_VAR (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                let (_1, _2, _2_inlined1, _1_inlined2, _2_inlined2, _endpos__2_inlined3_) = ((), (), (), (), _v, _endpos_0) in
                let _v = _menhir_action_382 _1 _1_inlined1 _1_inlined2 _2 _2_inlined1 _2_inlined2 _endpos__2_inlined3_ _startpos__1_ x in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    let _menhir_run_0751 : type  ttv_stack. (((ttv_stack, _menhir_box_opn) _menhir_cell1_VAR, _menhir_box_opn) _menhir_cell1_IDENTIFIER, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ -> _ -> _ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, x, _, _) = _menhir_stack in
                let MenhirCell1_VAR (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                let (_1, _2, _2_inlined1, _1_inlined2, _2_inlined2, _endpos__2_inlined3_) = ((), (), (), (), _v, _endpos_0) in
                let _v = _menhir_action_383 _1 _1_inlined1 _1_inlined2 _2 _2_inlined1 _2_inlined2 _endpos__2_inlined3_ _startpos__1_ x in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    let _menhir_run_0995 : type  ttv_stack. (((((ttv_stack, _menhir_box_opn) _menhir_cell1_CONSTANT, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_COMMA, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE -> _ -> _ -> _ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
                let MenhirCell1_COMMA (_menhir_stack, _) = _menhir_stack in
                let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
                let MenhirCell1_CONSTANT (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                let (_1, _2, _2_inlined1, _3, _3_inlined1, _5, _endpos__2_inlined2_, _endpos_args_, args) = ((), (), (), (), (), (), _endpos_0, _endpos, _v) in
                let _v = _menhir_action_379 _1 _1_inlined1 _2 _2_inlined1 _3 _3_inlined1 _5 _endpos__2_inlined2_ _endpos_args_ _startpos__1_ _startpos_name_ args lhs name in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    let _menhir_run_0856 : type  ttv_stack. (((((ttv_stack, _menhir_box_opn) _menhir_cell1_LET, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_COMMA, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE -> _ -> _ -> _ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
                let MenhirCell1_COMMA (_menhir_stack, _) = _menhir_stack in
                let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
                let MenhirCell1_LET (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                let (_1, _2, _2_inlined1, _3, _3_inlined1, _5, _endpos__2_inlined2_, _endpos_args_, args) = ((), (), (), (), (), (), _endpos_0, _endpos, _v) in
                let _v = _menhir_action_376 _1 _1_inlined1 _2 _2_inlined1 _3 _3_inlined1 _5 _endpos__2_inlined2_ _endpos_args_ _startpos__1_ _startpos_name_ args lhs name in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    let _menhir_run_0776 : type  ttv_stack. (((((ttv_stack, _menhir_box_opn) _menhir_cell1_VAR, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_COMMA, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE -> _ -> _ -> _ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
                let MenhirCell1_COMMA (_menhir_stack, _) = _menhir_stack in
                let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
                let MenhirCell1_VAR (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                let (_1, _2, _2_inlined1, _3, _3_inlined1, _5, _endpos__2_inlined2_, _endpos_args_, args) = ((), (), (), (), (), (), _endpos_0, _endpos, _v) in
                let _v = _menhir_action_386 _1 _1_inlined1 _2 _2_inlined1 _3 _3_inlined1 _5 _endpos__2_inlined2_ _endpos_args_ _startpos__1_ _startpos_name_ args lhs name in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    let _menhir_run_0988 : type  ttv_stack. ((((ttv_stack, _menhir_box_opn) _menhir_cell1_CONSTANT, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_RBRACE _menhir_cell0_LPAR -> _ -> _ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RPAR ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.SEMI_COLON ->
                let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.EOF ->
                    let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                    let MenhirCell1_RBRACE (_menhir_stack, _, _) = _menhir_stack in
                    let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                    let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                    let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
                    let MenhirCell1_CONSTANT (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                    let (_1, _2, _2_inlined1, _1_inlined2, _3, _3_inlined1, _endpos__2_inlined2_, _endpos__3_inlined2_) = ((), (), (), _v, (), (), _endpos_0, _endpos) in
                    let _v = _menhir_action_378 _1 _1_inlined2 _2 _2_inlined1 _3 _3_inlined1 _endpos__2_inlined2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name in
                    _menhir_goto_opn _menhir_stack _v
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    let _menhir_run_0974 : type  ttv_stack. (ttv_stack, _menhir_box_opn) _menhir_cell1_DEBUG _menhir_cell0_LPAR -> _ -> _ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RPAR ->
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.SEMI_COLON ->
                let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.EOF ->
                    let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                    let MenhirCell1_DEBUG (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                    let (_1, _2, _endpos__2_inlined1_, _1_inlined2) = ((), (), _endpos_0, _v) in
                    let _v = _menhir_action_390 _1 _1_inlined2 _2 _endpos__2_inlined1_ _startpos__1_ in
                    _menhir_goto_opn _menhir_stack _v
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    let _menhir_run_0849 : type  ttv_stack. ((((ttv_stack, _menhir_box_opn) _menhir_cell1_LET, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_RBRACE _menhir_cell0_LPAR -> _ -> _ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RPAR ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.SEMI_COLON ->
                let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.EOF ->
                    let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                    let MenhirCell1_RBRACE (_menhir_stack, _, _) = _menhir_stack in
                    let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                    let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                    let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
                    let MenhirCell1_LET (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                    let (_1, _2, _2_inlined1, _1_inlined2, _3, _3_inlined1, _endpos__2_inlined2_, _endpos__3_inlined2_) = ((), (), (), _v, (), (), _endpos_0, _endpos) in
                    let _v = _menhir_action_375 _1 _1_inlined2 _2 _2_inlined1 _3 _3_inlined1 _endpos__2_inlined2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name in
                    _menhir_goto_opn _menhir_stack _v
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    let _menhir_run_0833 : type  ttv_stack. (ttv_stack, _menhir_box_opn) _menhir_cell1_PRAGMA _menhir_cell0_IDENTIFIER -> _ -> _ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
                let MenhirCell1_PRAGMA (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                let (_1, _2, _endpos__2_inlined1_, _1_inlined1) = ((), (), _endpos, _v) in
                let _v = _menhir_action_398 _1 _1_inlined1 _2 _endpos__2_inlined1_ _startpos__1_ x in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    let _menhir_run_0825 : type  ttv_stack. (ttv_stack, _menhir_box_opn) _menhir_cell1_PRINT _menhir_cell0_LPAR -> _ -> _ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RPAR ->
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.SEMI_COLON ->
                let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.EOF ->
                    let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                    let MenhirCell1_PRINT (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                    let (_1, _2, _endpos__2_inlined1_, _1_inlined2) = ((), (), _endpos_0, _v) in
                    let _v = _menhir_action_388 _1 _1_inlined2 _2 _endpos__2_inlined1_ _startpos__1_ in
                    _menhir_goto_opn _menhir_stack _v
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    let _menhir_run_0769 : type  ttv_stack. ((((ttv_stack, _menhir_box_opn) _menhir_cell1_VAR, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_RBRACE _menhir_cell0_LPAR -> _ -> _ -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RPAR ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.SEMI_COLON ->
                let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.EOF ->
                    let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                    let MenhirCell1_RBRACE (_menhir_stack, _, _) = _menhir_stack in
                    let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                    let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                    let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
                    let MenhirCell1_VAR (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                    let (_1, _2, _2_inlined1, _1_inlined2, _3, _3_inlined1, _endpos__2_inlined2_, _endpos__3_inlined2_) = ((), (), (), _v, (), (), _endpos_0, _endpos) in
                    let _v = _menhir_action_385 _1 _1_inlined2 _2 _2_inlined1 _3 _3_inlined1 _endpos__2_inlined2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name in
                    _menhir_goto_opn _menhir_stack _v
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    let rec _menhir_run_0002 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let (_endpos_s_, _startpos_s_, s) = (_endpos, _startpos, _v) in
        let _v = _menhir_action_019 _endpos_s_ _startpos_s_ s in
        _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_s_ _startpos_s_ _v _menhir_s _tok
    
    and _menhir_goto_expr : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match _menhir_s with
        | MenhirState1370 ->
            _menhir_run_1371 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1362 ->
            _menhir_run_1363 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1354 ->
            _menhir_run_1355 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1349 ->
            _menhir_run_1350 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1341 ->
            _menhir_run_1342 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1272 ->
            _menhir_run_1273 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1266 ->
            _menhir_run_1267 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1260 ->
            _menhir_run_1261 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1255 ->
            _menhir_run_1256 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1249 ->
            _menhir_run_1250 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1219 ->
            _menhir_run_1220 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1186 ->
            _menhir_run_1187 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1177 ->
            _menhir_run_1178 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1171 ->
            _menhir_run_1172 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1165 ->
            _menhir_run_1166 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1160 ->
            _menhir_run_1161 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1154 ->
            _menhir_run_1155 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1147 ->
            _menhir_run_1148 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1141 ->
            _menhir_run_1142 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1135 ->
            _menhir_run_1136 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1130 ->
            _menhir_run_1131 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1124 ->
            _menhir_run_1125 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1118 ->
            _menhir_run_1119 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1113 ->
            _menhir_run_1114 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1090 ->
            _menhir_run_1091 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1036 ->
            _menhir_run_1037 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1029 ->
            _menhir_run_1030 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1023 ->
            _menhir_run_1024 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1015 ->
            _menhir_run_1016 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1012 ->
            _menhir_run_1013 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1007 ->
            _menhir_run_1008 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1001 ->
            _menhir_run_1002 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0980 ->
            _menhir_run_0998 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0959 ->
            _menhir_run_0960 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0957 ->
            _menhir_run_0958 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0947 ->
            _menhir_run_0948 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0945 ->
            _menhir_run_0946 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0943 ->
            _menhir_run_0944 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0937 ->
            _menhir_run_0938 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0932 ->
            _menhir_run_0933 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0922 ->
            _menhir_run_0923 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0912 ->
            _menhir_run_0913 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0907 ->
            _menhir_run_0908 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0897 ->
            _menhir_run_0898 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0889 ->
            _menhir_run_0890 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0884 ->
            _menhir_run_0885 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0874 ->
            _menhir_run_0875 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0862 ->
            _menhir_run_0863 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0841 ->
            _menhir_run_0859 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0816 ->
            _menhir_run_0817 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0812 ->
            _menhir_run_0813 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0804 ->
            _menhir_run_0807 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0798 ->
            _menhir_run_0801 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0761 ->
            _menhir_run_0779 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0711 ->
            _menhir_run_0729 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0398 ->
            _menhir_run_0678 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0400 ->
            _menhir_run_0674 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0669 ->
            _menhir_run_0670 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0665 ->
            _menhir_run_0666 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0437 ->
            _menhir_run_0651 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0630 ->
            _menhir_run_0631 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0625 ->
            _menhir_run_0626 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0620 ->
            _menhir_run_0621 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0610 ->
            _menhir_run_0611 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0600 ->
            _menhir_run_0601 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0595 ->
            _menhir_run_0596 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0585 ->
            _menhir_run_0586 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0577 ->
            _menhir_run_0578 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0572 ->
            _menhir_run_0573 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0562 ->
            _menhir_run_0563 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0551 ->
            _menhir_run_0552 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0549 ->
            _menhir_run_0550 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0466 ->
            _menhir_run_0534 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0504 ->
            _menhir_run_0505 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0498 ->
            _menhir_run_0499 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0488 ->
            _menhir_run_0489 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0485 ->
            _menhir_run_0486 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0481 ->
            _menhir_run_0482 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0478 ->
            _menhir_run_0479 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0473 ->
            _menhir_run_0474 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0457 ->
            _menhir_run_0458 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0455 ->
            _menhir_run_0456 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0453 ->
            _menhir_run_0454 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0444 ->
            _menhir_run_0445 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0416 ->
            _menhir_run_0417 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0361 ->
            _menhir_run_0362 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0359 ->
            _menhir_run_0360 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0356 ->
            _menhir_run_0357 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0001 ->
            _menhir_run_0355 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0004 ->
            _menhir_run_0354 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0006 ->
            _menhir_run_0349 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0346 ->
            _menhir_run_0347 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0008 ->
            _menhir_run_0345 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0013 ->
            _menhir_run_0342 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0218 ->
            _menhir_run_0312 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0310 ->
            _menhir_run_0311 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0301 ->
            _menhir_run_0302 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0299 ->
            _menhir_run_0300 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0297 ->
            _menhir_run_0298 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0295 ->
            _menhir_run_0296 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0293 ->
            _menhir_run_0294 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0291 ->
            _menhir_run_0292 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0289 ->
            _menhir_run_0290 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0282 ->
            _menhir_run_0283 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0279 ->
            _menhir_run_0281 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0277 ->
            _menhir_run_0278 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0275 ->
            _menhir_run_0276 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0273 ->
            _menhir_run_0274 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0260 ->
            _menhir_run_0261 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0257 ->
            _menhir_run_0258 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0255 ->
            _menhir_run_0256 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0253 ->
            _menhir_run_0254 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0251 ->
            _menhir_run_0252 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0249 ->
            _menhir_run_0250 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0246 ->
            _menhir_run_0248 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0244 ->
            _menhir_run_0245 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0242 ->
            _menhir_run_0243 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0240 ->
            _menhir_run_0241 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0238 ->
            _menhir_run_0239 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0236 ->
            _menhir_run_0237 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0234 ->
            _menhir_run_0235 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0232 ->
            _menhir_run_0233 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0215 ->
            _menhir_run_0216 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0205 ->
            _menhir_run_0206 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState1086 ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0982 ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0992 ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0984 ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0970 ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0916 ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0870 ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0843 ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0853 ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0845 ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0830 ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0821 ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0763 ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0773 ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0765 ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0713 ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0723 ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0715 ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0404 ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0408 ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0439 ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0645 ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0441 ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0604 ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0448 ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0461 ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0468 ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0528 ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0470 ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0351 ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0010 ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0191 ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0194 ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0200 ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0189 ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0184 ->
            _menhir_run_0188 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0179 ->
            _menhir_run_0180 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0177 ->
            _menhir_run_0178 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0175 ->
            _menhir_run_0176 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0173 ->
            _menhir_run_0174 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0171 ->
            _menhir_run_0172 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0161 ->
            _menhir_run_0162 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0155 ->
            _menhir_run_0156 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0151 ->
            _menhir_run_0152 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0419 ->
            _menhir_run_0150 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0032 ->
            _menhir_run_0150 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0327 ->
            _menhir_run_0150 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0262 ->
            _menhir_run_0150 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0163 ->
            _menhir_run_0150 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0159 ->
            _menhir_run_0150 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0157 ->
            _menhir_run_0150 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0153 ->
            _menhir_run_0150 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0123 ->
            _menhir_run_0150 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0138 ->
            _menhir_run_0139 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0119 ->
            _menhir_run_0120 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0110 ->
            _menhir_run_0111 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0022 ->
            _menhir_run_0109 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0304 ->
            _menhir_run_0109 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0114 ->
            _menhir_run_0109 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0112 ->
            _menhir_run_0109 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0104 ->
            _menhir_run_0109 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0101 ->
            _menhir_run_0102 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0099 ->
            _menhir_run_0100 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0097 ->
            _menhir_run_0098 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0094 ->
            _menhir_run_0096 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0092 ->
            _menhir_run_0093 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0090 ->
            _menhir_run_0091 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0088 ->
            _menhir_run_0089 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0086 ->
            _menhir_run_0087 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0084 ->
            _menhir_run_0085 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0082 ->
            _menhir_run_0083 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0080 ->
            _menhir_run_0081 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0078 ->
            _menhir_run_0079 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0076 ->
            _menhir_run_0077 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0073 ->
            _menhir_run_0075 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0071 ->
            _menhir_run_0072 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0069 ->
            _menhir_run_0070 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0067 ->
            _menhir_run_0068 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0065 ->
            _menhir_run_0066 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0063 ->
            _menhir_run_0064 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0061 ->
            _menhir_run_0062 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0059 ->
            _menhir_run_0060 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0057 ->
            _menhir_run_0058 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0055 ->
            _menhir_run_0056 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0051 ->
            _menhir_run_0052 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0041 ->
            _menhir_run_0042 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0035 ->
            _menhir_run_0036 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0033 ->
            _menhir_run_0034 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0030 ->
            _menhir_run_0031 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0028 ->
            _menhir_run_0029 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0026 ->
            _menhir_run_0027 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0005 ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0305 ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0105 ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0023 ->
            _menhir_run_0025 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_1371 : type  ttv_stack. (((ttv_stack, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState1371, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1372
            | Tokens.TYPE ->
                _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1372
            | Tokens.SETTER ->
                _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1372
            | Tokens.PRAGMA ->
                _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1372
            | Tokens.LET ->
                _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1372
            | Tokens.GETTER ->
                _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1372
            | Tokens.FUNC ->
                _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1372
            | Tokens.CONSTANT ->
                _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1372
            | Tokens.CONFIG ->
                _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1372
            | Tokens.EOF ->
                let _v_1 = _menhir_action_133 () in
                _menhir_run_1373 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1371
        | _ ->
            _eRR ()
    
    and _menhir_run_0028 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_SHR (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0028 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0003 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let (_endpos_r_, _startpos_r_, r) = (_endpos, _startpos, _v) in
        let _v = _menhir_action_017 _endpos_r_ _startpos_r_ r in
        _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_r_ _startpos_r_ _v _menhir_s _tok
    
    and _menhir_run_0004 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_NOT (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0004 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0005 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, _menhir_s, _startpos, _endpos) in
        let _menhir_s = MenhirState0005 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0006 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0006 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0007 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let (_endpos_i_, _startpos_i_, i) = (_endpos, _startpos, _v) in
        let _v = _menhir_action_015 _endpos_i_ _startpos_i_ i in
        _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_i_ _startpos_i_ _v _menhir_s _tok
    
    and _menhir_run_0008 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_IF (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0008 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0009 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.LBRACE ->
            let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0194 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DO | Tokens.DOT | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.IN | Tokens.LBRACKET | Tokens.LEQ | Tokens.LLBRACKET | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.POW | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RDIV | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let (_endpos__1_, _startpos__1_, _1) = (_endpos, _startpos, _v) in
            let _v = _menhir_action_020 _1 _endpos__1_ _startpos__1_ in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__1_ _startpos__1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0010 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos) in
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0010
        | Tokens.RPAR ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
            let _endpos__3_ = _endpos in
            let _v = _menhir_action_050 _endpos__3_ _startpos_name_ name in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_name_ _v _menhir_s _tok
        | Tokens.REAL_LIT _v ->
            let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos) in
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0010
        | Tokens.NOT ->
            let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos) in
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0010
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos) in
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0010
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos) in
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0010
        | Tokens.INT_LIT _v ->
            let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos) in
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0010
        | Tokens.IF ->
            let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos) in
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0010
        | Tokens.IDENTIFIER _v ->
            let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos) in
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0010
        | Tokens.BOOL_LIT _v ->
            let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos) in
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0010
        | Tokens.BNOT ->
            let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos) in
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0010
        | Tokens.BITVECTOR_LIT _v ->
            let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos) in
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0010
        | Tokens.ARBITRARY ->
            let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos) in
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0010
        | _ ->
            _eRR ()
    
    and _menhir_run_0012 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let (_endpos_b_, _startpos_b_, b) = (_endpos, _startpos, _v) in
        let _v = _menhir_action_016 _endpos_b_ _startpos_b_ b in
        _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_b_ _startpos_b_ _v _menhir_s _tok
    
    and _menhir_run_0013 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_BNOT (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0013 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0014 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let (_endpos_b_, _startpos_b_, b) = (_endpos, _startpos, _v) in
        let _v = _menhir_action_018 _endpos_b_ _startpos_b_ b in
        _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_b_ _startpos_b_ _v _menhir_s _tok
    
    and _menhir_run_0015 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_ARBITRARY (_menhir_stack, _menhir_s, _startpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.COLON_COLON ->
            let _menhir_s = MenhirState0016 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON ->
            let _menhir_s = MenhirState0340 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0017 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let (_endpos__1_, _startpos__1_, _1) = (_endpos, _startpos, ()) in
        let _v = _menhir_action_460 _1 _endpos__1_ _startpos__1_ in
        _menhir_goto_ty _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__1_ _v _menhir_s _tok
    
    and _menhir_goto_ty : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match _menhir_s with
        | MenhirState1366 ->
            _menhir_run_1367 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1358 ->
            _menhir_run_1359 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1345 ->
            _menhir_run_1346 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1337 ->
            _menhir_run_1338 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1312 ->
            _menhir_run_1335 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState1303 ->
            _menhir_run_1304 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1295 ->
            _menhir_run_1296 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1283 ->
            _menhir_run_1284 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1270 ->
            _menhir_run_1271 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1264 ->
            _menhir_run_1265 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1253 ->
            _menhir_run_1254 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1247 ->
            _menhir_run_1248 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1238 ->
            _menhir_run_1239 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1217 ->
            _menhir_run_1218 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1205 ->
            _menhir_run_1206 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1201 ->
            _menhir_run_1202 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1184 ->
            _menhir_run_1185 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1175 ->
            _menhir_run_1176 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1169 ->
            _menhir_run_1170 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1158 ->
            _menhir_run_1159 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1152 ->
            _menhir_run_1153 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1145 ->
            _menhir_run_1146 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1139 ->
            _menhir_run_1140 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1128 ->
            _menhir_run_1129 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1122 ->
            _menhir_run_1123 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1098 ->
            _menhir_run_1099 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1080 ->
            _menhir_run_1081 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1069 ->
            _menhir_run_1070 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1065 ->
            _menhir_run_1066 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1050 ->
            _menhir_run_1051 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1046 ->
            _menhir_run_1047 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0754 ->
            _menhir_run_0755 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0750 ->
            _menhir_run_0751 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0704 ->
            _menhir_run_0705 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0683 ->
            _menhir_run_0690 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0685 ->
            _menhir_run_0686 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0390 ->
            _menhir_run_0391 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0382 ->
            _menhir_run_0383 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0380 ->
            _menhir_run_0381 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0374 ->
            _menhir_run_0375 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0372 ->
            _menhir_run_0373 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0368 ->
            _menhir_run_0369 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0366 ->
            _menhir_run_0367 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0340 ->
            _menhir_run_0341 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0016 ->
            _menhir_run_0339 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0335 ->
            _menhir_run_0334 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0019 ->
            _menhir_run_0334 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0303 ->
            _menhir_run_0309 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0224 ->
            _menhir_run_0225 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0222 ->
            _menhir_run_0223 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0103 ->
            _menhir_run_0168 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0146 ->
            _menhir_run_0147 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0135 ->
            _menhir_run_0143 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0141 ->
            _menhir_run_0142 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_1367 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1368
            | Tokens.TYPE ->
                _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1368
            | Tokens.SETTER ->
                _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1368
            | Tokens.PRAGMA ->
                _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1368
            | Tokens.LET ->
                _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1368
            | Tokens.GETTER ->
                _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1368
            | Tokens.FUNC ->
                _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1368
            | Tokens.CONSTANT ->
                _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1368
            | Tokens.CONFIG ->
                _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1368
            | Tokens.EOF ->
                let _v_1 = _menhir_action_133 () in
                _menhir_run_1369 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.EQ ->
            let _menhir_s = MenhirState1370 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1034 : type  ttv_stack. ttv_stack -> _ -> _ -> (ttv_stack, _menhir_box_spec) _menhir_state -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.MINUS ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_MINUS (_menhir_stack, _startpos, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EQ ->
                let _menhir_s = MenhirState1036 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.ARBITRARY ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.COLON_COLON ->
                let _menhir_s = MenhirState1337 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.COLON ->
                let _menhir_s = MenhirState1345 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.IDENTIFIER _v ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EQ ->
                let _menhir_s = MenhirState1354 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.ARBITRARY ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.COLON_COLON ->
                let _menhir_s = MenhirState1358 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.COLON ->
                let _menhir_s = MenhirState1366 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0018 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let (_endpos__1_, _startpos__1_, _1) = (_endpos, _startpos, ()) in
        let _v = _menhir_action_458 _1 _endpos__1_ _startpos__1_ in
        _menhir_goto_ty _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__1_ _v _menhir_s _tok
    
    and _menhir_run_0019 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0019
        | Tokens.RPAR ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let (_startpos__1_, _endpos__3_) = (_startpos, _endpos) in
            let _v = _menhir_action_463 _endpos__3_ _startpos__1_ in
            _menhir_goto_ty _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _v _menhir_s _tok
        | Tokens.REAL ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0019
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0019
        | Tokens.INTEGER ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0019
        | Tokens.IDENTIFIER _v ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0019
        | Tokens.BOOLEAN ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0019
        | Tokens.BITS ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0019
        | Tokens.BIT ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0019
        | Tokens.ARRAY ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0019
        | _ ->
            _eRR ()
    
    and _menhir_run_0021 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.LBRACE ->
            let _startpos_0 = _menhir_lexbuf.Lexing.lex_start_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                let _menhir_stack = MenhirCell1_INTEGER (_menhir_stack, _menhir_s, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_0) in
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0022
            | Tokens.REAL_LIT _v ->
                let _menhir_stack = MenhirCell1_INTEGER (_menhir_stack, _menhir_s, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_0) in
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0022
            | Tokens.NOT ->
                let _menhir_stack = MenhirCell1_INTEGER (_menhir_stack, _menhir_s, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_0) in
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0022
            | Tokens.MINUS ->
                let _startpos_1 = _menhir_lexbuf.Lexing.lex_start_p in
                let _endpos_2 = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING_LIT _v ->
                    let _menhir_stack = MenhirCell1_INTEGER (_menhir_stack, _menhir_s, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_0) in
                    let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, MenhirState0022, _startpos_1, _endpos_2) in
                    _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0023
                | Tokens.REAL_LIT _v ->
                    let _menhir_stack = MenhirCell1_INTEGER (_menhir_stack, _menhir_s, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_0) in
                    let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, MenhirState0022, _startpos_1, _endpos_2) in
                    _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0023
                | Tokens.RBRACE ->
                    let _endpos_3 = _menhir_lexbuf.Lexing.lex_curr_p in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    let (_startpos__1_, _1, x, _endpos__3_) = (_startpos, (), (), _endpos_3) in
                    let _v = _menhir_action_456 _1 _endpos__3_ _startpos__1_ x in
                    _menhir_goto_ty _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _v _menhir_s _tok
                | Tokens.NOT ->
                    let _menhir_stack = MenhirCell1_INTEGER (_menhir_stack, _menhir_s, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_0) in
                    let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, MenhirState0022, _startpos_1, _endpos_2) in
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0023
                | Tokens.MINUS ->
                    let _menhir_stack = MenhirCell1_INTEGER (_menhir_stack, _menhir_s, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_0) in
                    let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, MenhirState0022, _startpos_1, _endpos_2) in
                    _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0023
                | Tokens.LPAR ->
                    let _menhir_stack = MenhirCell1_INTEGER (_menhir_stack, _menhir_s, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_0) in
                    let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, MenhirState0022, _startpos_1, _endpos_2) in
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0023
                | Tokens.INT_LIT _v ->
                    let _menhir_stack = MenhirCell1_INTEGER (_menhir_stack, _menhir_s, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_0) in
                    let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, MenhirState0022, _startpos_1, _endpos_2) in
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0023
                | Tokens.IF ->
                    let _menhir_stack = MenhirCell1_INTEGER (_menhir_stack, _menhir_s, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_0) in
                    let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, MenhirState0022, _startpos_1, _endpos_2) in
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0023
                | Tokens.IDENTIFIER _v ->
                    let _menhir_stack = MenhirCell1_INTEGER (_menhir_stack, _menhir_s, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_0) in
                    let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, MenhirState0022, _startpos_1, _endpos_2) in
                    _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0023
                | Tokens.BOOL_LIT _v ->
                    let _menhir_stack = MenhirCell1_INTEGER (_menhir_stack, _menhir_s, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_0) in
                    let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, MenhirState0022, _startpos_1, _endpos_2) in
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0023
                | Tokens.BNOT ->
                    let _menhir_stack = MenhirCell1_INTEGER (_menhir_stack, _menhir_s, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_0) in
                    let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, MenhirState0022, _startpos_1, _endpos_2) in
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0023
                | Tokens.BITVECTOR_LIT _v ->
                    let _menhir_stack = MenhirCell1_INTEGER (_menhir_stack, _menhir_s, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_0) in
                    let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, MenhirState0022, _startpos_1, _endpos_2) in
                    _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0023
                | Tokens.ARBITRARY ->
                    let _menhir_stack = MenhirCell1_INTEGER (_menhir_stack, _menhir_s, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_0) in
                    let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, MenhirState0022, _startpos_1, _endpos_2) in
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0023
                | _ ->
                    _eRR ())
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_INTEGER (_menhir_stack, _menhir_s, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_0) in
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0022
            | Tokens.INT_LIT _v ->
                let _menhir_stack = MenhirCell1_INTEGER (_menhir_stack, _menhir_s, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_0) in
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0022
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_INTEGER (_menhir_stack, _menhir_s, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_0) in
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0022
            | Tokens.IDENTIFIER _v ->
                let _menhir_stack = MenhirCell1_INTEGER (_menhir_stack, _menhir_s, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_0) in
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0022
            | Tokens.BOOL_LIT _v ->
                let _menhir_stack = MenhirCell1_INTEGER (_menhir_stack, _menhir_s, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_0) in
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0022
            | Tokens.BNOT ->
                let _menhir_stack = MenhirCell1_INTEGER (_menhir_stack, _menhir_s, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_0) in
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0022
            | Tokens.BITVECTOR_LIT _v ->
                let _menhir_stack = MenhirCell1_INTEGER (_menhir_stack, _menhir_s, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_0) in
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0022
            | Tokens.ARBITRARY ->
                let _menhir_stack = MenhirCell1_INTEGER (_menhir_stack, _menhir_s, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_0) in
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0022
            | _ ->
                _eRR ())
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DO | Tokens.DOT | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.IN | Tokens.LBRACKET | Tokens.LEQ | Tokens.LLBRACKET | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.POW | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RDIV | Tokens.RECURSELIMIT | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.STAR_COLON | Tokens.SUBTYPES | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let (_endpos__1_, _startpos__1_, _1) = (_endpos, _startpos, ()) in
            let _v = _menhir_action_457 _1 _endpos__1_ _startpos__1_ in
            _menhir_goto_ty _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0116 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let (_endpos_name_, _startpos_name_, name) = (_endpos, _startpos, _v) in
        let _v = _menhir_action_465 _endpos_name_ _startpos_name_ name in
        _menhir_goto_ty _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_name_ _v _menhir_s _tok
    
    and _menhir_run_0117 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let (_endpos__1_, _startpos__1_, _1) = (_endpos, _startpos, ()) in
        let _v = _menhir_action_459 _1 _endpos__1_ _startpos__1_ in
        _menhir_goto_ty _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__1_ _v _menhir_s _tok
    
    and _menhir_run_0118 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_BITS (_menhir_stack, _menhir_s, _startpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.LPAR ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos) in
            let _menhir_s = MenhirState0119 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0136 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let (_endpos__1_, _startpos__1_, _1) = (_endpos, _startpos, ()) in
        let _v = _menhir_action_461 _1 _endpos__1_ _startpos__1_ in
        _menhir_goto_ty _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__1_ _v _menhir_s _tok
    
    and _menhir_run_0137 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_ARRAY (_menhir_stack, _menhir_s, _startpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.LBRACKET ->
            let _menhir_s = MenhirState0138 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1039 : type  ttv_stack. ttv_stack -> _ -> _ -> (ttv_stack, _menhir_box_spec) _menhir_state -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_TYPE (_menhir_stack, _menhir_s, _startpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.IDENTIFIER _v ->
            let _startpos_0 = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.SUBTYPES ->
                let _startpos_1 = _menhir_lexbuf.Lexing.lex_start_p in
                let _menhir_stack = MenhirCell0_SUBTYPES (_menhir_stack, _startpos_1) in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.IDENTIFIER _v_2 ->
                    let _startpos_3 = _menhir_lexbuf.Lexing.lex_start_p in
                    let _endpos_4 = _menhir_lexbuf.Lexing.lex_curr_p in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v_2, _startpos_3, _endpos_4) in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.WITH ->
                        _menhir_run_1043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1042
                    | Tokens.SEMI_COLON ->
                        let _v_5 = _menhir_action_190 () in
                        _menhir_run_1057 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_4 _v_5 MenhirState1042 _tok
                    | _ ->
                        _eRR ())
                | _ ->
                    _eRR ())
            | Tokens.OF ->
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1312
                | Tokens.RECORD ->
                    let _startpos_6 = _menhir_lexbuf.Lexing.lex_start_p in
                    let _endpos_7 = _menhir_lexbuf.Lexing.lex_curr_p in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.LBRACE ->
                        let _menhir_stack = MenhirCell1_RECORD (_menhir_stack, MenhirState1312, _startpos_6, _endpos_7) in
                        let _startpos_8 = _menhir_lexbuf.Lexing.lex_start_p in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_8) in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.IDENTIFIER _v_9 ->
                            _menhir_run_1045 _menhir_stack _menhir_lexbuf _menhir_lexer _v_9 MenhirState1314
                        | Tokens.RBRACE ->
                            let _v_10 = _menhir_action_198 () in
                            _menhir_run_1315 _menhir_stack _menhir_lexbuf _menhir_lexer _v_10
                        | _ ->
                            _eRR ())
                    | Tokens.SEMI_COLON | Tokens.SUBTYPES ->
                        let (_menhir_s, _endpos__1_, _startpos__1_, _1) = (MenhirState1312, _endpos_7, _startpos_6, ()) in
                        let _v = _menhir_action_469 _1 _endpos__1_ _startpos__1_ in
                        _menhir_goto_ty_decl _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
                    | _ ->
                        _eRR ())
                | Tokens.REAL ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1312
                | Tokens.LPAR ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1312
                | Tokens.INTEGER ->
                    _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1312
                | Tokens.IDENTIFIER _v_12 ->
                    _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v_12 MenhirState1312
                | Tokens.EXCEPTION ->
                    let _startpos_13 = _menhir_lexbuf.Lexing.lex_start_p in
                    let _endpos_14 = _menhir_lexbuf.Lexing.lex_curr_p in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.LBRACE ->
                        let _menhir_stack = MenhirCell1_EXCEPTION (_menhir_stack, MenhirState1312, _startpos_13, _endpos_14) in
                        let _startpos_15 = _menhir_lexbuf.Lexing.lex_start_p in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_15) in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.IDENTIFIER _v_16 ->
                            _menhir_run_1045 _menhir_stack _menhir_lexbuf _menhir_lexer _v_16 MenhirState1318
                        | Tokens.RBRACE ->
                            let _v_17 = _menhir_action_198 () in
                            _menhir_run_1319 _menhir_stack _menhir_lexbuf _menhir_lexer _v_17
                        | _ ->
                            _eRR ())
                    | Tokens.SEMI_COLON | Tokens.SUBTYPES ->
                        let (_menhir_s, _endpos__1_, _startpos__1_, _1) = (MenhirState1312, _endpos_14, _startpos_13, ()) in
                        let _v = _menhir_action_471 _1 _endpos__1_ _startpos__1_ in
                        _menhir_goto_ty_decl _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
                    | _ ->
                        _eRR ())
                | Tokens.ENUMERATION ->
                    let _menhir_s = MenhirState1312 in
                    let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                    let _menhir_stack = MenhirCell1_ENUMERATION (_menhir_stack, _menhir_s, _startpos) in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.LBRACE ->
                        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
                        let _menhir_s = MenhirState1322 in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.IDENTIFIER _v ->
                            _menhir_run_1323 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | _ ->
                            _eRR ())
                    | _ ->
                        _eRR ())
                | Tokens.BOOLEAN ->
                    _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1312
                | Tokens.BITS ->
                    _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1312
                | Tokens.BIT ->
                    _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1312
                | Tokens.ARRAY ->
                    _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1312
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1043 : type  ttv_stack. (ttv_stack _menhir_cell0_SUBTYPES _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_WITH (_menhir_stack, _menhir_s) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.LBRACE ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.IDENTIFIER _v ->
                _menhir_run_1045 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState1044
            | Tokens.RBRACE ->
                let _v = _menhir_action_198 () in
                _menhir_run_1055 _menhir_stack _menhir_lexbuf _menhir_lexer _v
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1045 : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_spec) _menhir_state -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.COLON_COLON ->
            let _menhir_s = MenhirState1046 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON ->
            let _menhir_s = MenhirState1050 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1055 : type  ttv_stack. (ttv_stack _menhir_cell0_SUBTYPES _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_WITH _menhir_cell0_LBRACE -> _ -> _ -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_WITH (_menhir_stack, _menhir_s) = _menhir_stack in
        let (_1, _1_inlined2, _endpos__3_) = ((), _v, _endpos) in
        let _v = _menhir_action_191 _1 _1_inlined2 in
        _menhir_goto_loption___anonymous_21_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _v _menhir_s _tok
    
    and _menhir_goto_loption___anonymous_21_ : type  ttv_stack. (ttv_stack _menhir_cell0_SUBTYPES _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match _menhir_s with
        | MenhirState1330 ->
            _menhir_run_1331 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState1042 ->
            _menhir_run_1057 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_1331 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_TYPE _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty_decl _menhir_cell0_SUBTYPES _menhir_cell0_IDENTIFIER -> _ -> _ -> _ -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        let MenhirCell0_IDENTIFIER (_menhir_stack, _2, _, _) = _menhir_stack in
        let MenhirCell0_SUBTYPES (_menhir_stack, _) = _menhir_stack in
        let (_1, _3) = ((), _v) in
        let _v = _menhir_action_402 _1 _2 _3 in
        _menhir_goto_option_subtype_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
    
    and _menhir_goto_option_subtype_ : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_TYPE _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty_decl -> _ -> _ -> _ -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        let _menhir_stack = MenhirCell0_option_subtype_ (_menhir_stack, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1333
            | Tokens.TYPE ->
                _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1333
            | Tokens.SETTER ->
                _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1333
            | Tokens.PRAGMA ->
                _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1333
            | Tokens.LET ->
                _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1333
            | Tokens.GETTER ->
                _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1333
            | Tokens.FUNC ->
                _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1333
            | Tokens.CONSTANT ->
                _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1333
            | Tokens.CONFIG ->
                _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1333
            | Tokens.EOF ->
                let _v_0 = _menhir_action_133 () in
                _menhir_run_1334 _menhir_stack _v_0
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1059 : type  ttv_stack. ttv_stack -> _ -> _ -> (ttv_stack, _menhir_box_spec) _menhir_state -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_SETTER (_menhir_stack, _menhir_s, _startpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.IDENTIFIER _v ->
            let _startpos_0 = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LBRACE ->
                _menhir_run_1061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1060
            | Tokens.LPAR ->
                let _v_1 = _menhir_action_194 () in
                _menhir_run_1075 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState1060 _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1061 : type  ttv_stack. (ttv_stack _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState1061 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.IDENTIFIER _v ->
            _menhir_run_1062 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_1062 : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_spec) _menhir_state -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_s = MenhirState1063 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.IDENTIFIER _v ->
                _menhir_run_1062 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_s = MenhirState1065 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON ->
            let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_s = MenhirState1069 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.RBRACE ->
            let x = _v in
            let _v = _menhir_action_429 x in
            _menhir_goto_separated_nonempty_list_COMMA_opt_typed_identifier_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_goto_separated_nonempty_list_COMMA_opt_typed_identifier_ : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_spec) _menhir_state -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        match _menhir_s with
        | MenhirState1061 ->
            _menhir_run_1073 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState1071 ->
            _menhir_run_1072 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState1067 ->
            _menhir_run_1068 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState1063 ->
            _menhir_run_1064 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_1073 : type  ttv_stack. (ttv_stack _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_LBRACE -> _ -> _ -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let _tok = _menhir_lexer _menhir_lexbuf in
        let MenhirCell1_LBRACE (_menhir_stack, _menhir_s, _) = _menhir_stack in
        let _1_inlined1 = _v in
        let _v = _menhir_action_195 _1_inlined1 in
        _menhir_goto_loption_braced_nclist_opt_typed_identifier___ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
    
    and _menhir_goto_loption_braced_nclist_opt_typed_identifier___ : type  ttv_stack. (ttv_stack _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        match _menhir_s with
        | MenhirState1104 ->
            _menhir_run_1105 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState1094 ->
            _menhir_run_1095 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState1060 ->
            _menhir_run_1075 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_1105 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.LPAR ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos) in
            let _menhir_s = MenhirState1106 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.RPAR ->
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, _menhir_s, _endpos) in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.BEGIN ->
                    let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                    let _menhir_stack = MenhirCell0_BEGIN (_menhir_stack, _endpos) in
                    let _menhir_s = MenhirState1108 in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.WHILE ->
                        _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.VAR ->
                        _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.UNREACHABLE ->
                        _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.TRY ->
                        _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.THROW ->
                        _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.RETURN ->
                        _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.REPEAT ->
                        _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.PRINT ->
                        _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.PRAGMA ->
                        _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.PASS ->
                        _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.MINUS ->
                        _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.LPAR ->
                        _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.LET ->
                        _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.IF ->
                        _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.IDENTIFIER _v ->
                        _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.FOR ->
                        _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.END ->
                        _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.DEBUG ->
                        _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.CONSTANT ->
                        _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.CASE ->
                        _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.ASSERT ->
                        _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.ARROBASE_LOOPLIMIT ->
                        _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | _ ->
                        _eRR ())
                | Tokens.ARROW ->
                    let _menhir_s = MenhirState1184 in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.STRING ->
                        _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.REAL ->
                        _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.LPAR ->
                        _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.INTEGER ->
                        _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.IDENTIFIER _v ->
                        _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.BOOLEAN ->
                        _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.BITS ->
                        _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.BIT ->
                        _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.ARRAY ->
                        _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | _ ->
                        _eRR ())
                | _ ->
                    _eRR ())
            | Tokens.IDENTIFIER _v ->
                _menhir_run_1200 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0359 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_WHILE (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0359 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0364 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.MINUS ->
            _menhir_run_0365 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0364
        | Tokens.LPAR ->
            _menhir_run_0370 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0364
        | Tokens.IDENTIFIER _v ->
            let _startpos_0 = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.COMMA ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0364, _v, _startpos_0, _endpos) in
                let _menhir_s = MenhirState0388 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0045 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.COLON_COLON ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0364, _v, _startpos_0, _endpos) in
                _menhir_run_0372 _menhir_stack _menhir_lexbuf _menhir_lexer
            | Tokens.COLON ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0364, _v, _startpos_0, _endpos) in
                _menhir_run_0374 _menhir_stack _menhir_lexbuf _menhir_lexer
            | Tokens.EQ | Tokens.SEMI_COLON ->
                let _v =
                  let _1 = _v in
                  _menhir_action_009 _1
                in
                _menhir_run_0708 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0364 _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0365 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, _menhir_s, _startpos, _endpos) in
            let _menhir_s = MenhirState0366 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON ->
            let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, _menhir_s, _startpos, _endpos) in
            let _menhir_s = MenhirState0368 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COMMA | Tokens.EQ | Tokens.RPAR | Tokens.SEMI_COLON ->
            let _1 = () in
            let _v = _menhir_action_010 _1 in
            _menhir_goto_decl_item _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_goto_decl_item : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        match _menhir_s with
        | MenhirState0978 ->
            _menhir_run_0979 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0839 ->
            _menhir_run_0840 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0746 ->
            _menhir_run_0758 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0364 ->
            _menhir_run_0708 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0464 ->
            _menhir_run_0465 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0435 ->
            _menhir_run_0436 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0385 ->
            _menhir_run_0384 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0377 ->
            _menhir_run_0384 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0370 ->
            _menhir_run_0376 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0979 : type  ttv_stack. ((ttv_stack, _menhir_box_opn) _menhir_cell1_CONSTANT as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.EQ ->
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v_0 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState0980
            | Tokens.REAL_LIT _v_1 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0980
            | Tokens.NOT ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0980
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0980
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0980
            | Tokens.INT_LIT _v_2 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v_2 MenhirState0980
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0980
            | Tokens.IDENTIFIER _v_3 ->
                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.LPAR ->
                    let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0980, _v_3, _startpos, _endpos) in
                    _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer
                | Tokens.LBRACE ->
                    let _startpos_4 = _menhir_lexbuf.Lexing.lex_start_p in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.STRING_LIT _v_5 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0980, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v_5 MenhirState0982
                    | Tokens.REAL_LIT _v_6 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0980, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v_6 MenhirState0982
                    | Tokens.RBRACE ->
                        let _endpos_7 = _menhir_lexbuf.Lexing.lex_curr_p in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.LPAR ->
                            let _startpos_8 = _menhir_lexbuf.Lexing.lex_start_p in
                            let _tok = _menhir_lexer _menhir_lexbuf in
                            (match (_tok : MenhirBasics.token) with
                            | Tokens.STRING_LIT _v_9 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0980, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0982, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v_9 MenhirState0984
                            | Tokens.RPAR ->
                                let _endpos_10 = _menhir_lexbuf.Lexing.lex_curr_p in
                                let _tok = _menhir_lexer _menhir_lexbuf in
                                (match (_tok : MenhirBasics.token) with
                                | Tokens.SEMI_COLON ->
                                    let _endpos_11 = _menhir_lexbuf.Lexing.lex_curr_p in
                                    let _tok = _menhir_lexer _menhir_lexbuf in
                                    (match (_tok : MenhirBasics.token) with
                                    | Tokens.EOF ->
                                        let MenhirCell1_CONSTANT (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                                        let (_1, _2, _2_inlined1, _3, _3_inlined1, _startpos_name_, name, _endpos__2_inlined2_, lhs, _endpos__3_inlined2_) = ((), (), (), (), (), _startpos, _v_3, _endpos_11, _v, _endpos_10) in
                                        let _v = _menhir_action_377 _1 _2 _2_inlined1 _3 _3_inlined1 _endpos__2_inlined2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name in
                                        _menhir_goto_opn _menhir_stack _v
                                    | _ ->
                                        _eRR ())
                                | _ ->
                                    _eRR ())
                            | Tokens.REAL_LIT _v_12 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0980, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0982, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v_12 MenhirState0984
                            | Tokens.NOT ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0980, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0982, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0984
                            | Tokens.MINUS ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0980, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0982, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0984
                            | Tokens.LPAR ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0980, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0982, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0984
                            | Tokens.INT_LIT _v_13 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0980, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0982, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v_13 MenhirState0984
                            | Tokens.IF ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0980, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0982, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0984
                            | Tokens.IDENTIFIER _v_14 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0980, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0982, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v_14 MenhirState0984
                            | Tokens.BOOL_LIT _v_15 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0980, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0982, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_15 MenhirState0984
                            | Tokens.BNOT ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0980, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0982, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0984
                            | Tokens.BITVECTOR_LIT _v_16 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0980, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0982, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v_16 MenhirState0984
                            | Tokens.ARBITRARY ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0980, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0982, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0984
                            | _ ->
                                _eRR ())
                        | Tokens.AND | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DOT | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.IN | Tokens.LBRACKET | Tokens.LEQ | Tokens.LLBRACKET | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RDIV | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR ->
                            let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                            let _v_17 =
                              let (_endpos__3_, _endpos_desc_, _startpos_desc_, desc) = (_endpos_7, _endpos, _startpos, _v_3) in
                              _menhir_action_067 _endpos__3_ _endpos_desc_ _startpos_desc_ desc
                            in
                            _menhir_run_0998 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_7 _startpos_4 _v_17 MenhirState0980 _tok
                        | _ ->
                            _eRR ())
                    | Tokens.NOT ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0980, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0982
                    | Tokens.MINUS ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0980, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0982
                    | Tokens.LPAR ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0980, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0982
                    | Tokens.INT_LIT _v_18 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0980, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v_18 MenhirState0982
                    | Tokens.IF ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0980, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0982
                    | Tokens.IDENTIFIER _v_19 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0980, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _v_19 MenhirState0982
                    | Tokens.COMMA ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0980, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        let _menhir_stack = MenhirCell1_COMMA (_menhir_stack, MenhirState0982) in
                        let _menhir_s = MenhirState0992 in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.STRING_LIT _v ->
                            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.REAL_LIT _v ->
                            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.NOT ->
                            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.MINUS ->
                            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.LPAR ->
                            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.INT_LIT _v ->
                            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.IF ->
                            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.IDENTIFIER _v ->
                            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.BOOL_LIT _v ->
                            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.BNOT ->
                            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.BITVECTOR_LIT _v ->
                            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.ARBITRARY ->
                            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | _ ->
                            _eRR ())
                    | Tokens.BOOL_LIT _v_26 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0980, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_26 MenhirState0982
                    | Tokens.BNOT ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0980, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0982
                    | Tokens.BITVECTOR_LIT _v_27 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0980, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v_27 MenhirState0982
                    | Tokens.ARBITRARY ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0980, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0982
                    | _ ->
                        _eRR ())
                | Tokens.AND | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DOT | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.IN | Tokens.LBRACKET | Tokens.LEQ | Tokens.LLBRACKET | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RDIV | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR ->
                    let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                    let _v_28 =
                      let (_endpos__1_, _startpos__1_, _1) = (_endpos, _startpos, _v_3) in
                      _menhir_action_020 _1 _endpos__1_ _startpos__1_
                    in
                    _menhir_run_0998 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v_28 MenhirState0980 _tok
                | _ ->
                    _eRR ())
            | Tokens.BOOL_LIT _v_29 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_29 MenhirState0980
            | Tokens.BNOT ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0980
            | Tokens.BITVECTOR_LIT _v_30 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v_30 MenhirState0980
            | Tokens.ARBITRARY ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0980
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0998 : type  ttv_stack. (((ttv_stack, _menhir_box_opn) _menhir_cell1_CONSTANT, _menhir_box_opn) _menhir_cell1_decl_item as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell1_decl_item (_menhir_stack, _, decl_item) = _menhir_stack in
                let MenhirCell1_CONSTANT (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                let (_1, _2, _endpos__2_inlined1_, x, _3) = ((), (), _endpos_0, _v, ()) in
                let _v = _menhir_action_363 _1 _2 _3 _endpos__2_inlined1_ _startpos__1_ decl_item x in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0998
        | _ ->
            _eRR ()
    
    and _menhir_run_0035 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_SHL (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0035 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0057 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_RDIV (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0057 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0030 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_POW (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0030 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0061 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_PLUS (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0061 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0071 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_OR (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0071 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0073 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            let _menhir_stack = MenhirCell1_NEQ (_menhir_stack, _menhir_s) in
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0073
        | Tokens.REAL_LIT _v ->
            let _menhir_stack = MenhirCell1_NEQ (_menhir_stack, _menhir_s) in
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0073
        | Tokens.NOT ->
            let _menhir_stack = MenhirCell1_NEQ (_menhir_stack, _menhir_s) in
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0073
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_NEQ (_menhir_stack, _menhir_s) in
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0073
        | Tokens.MASK_LIT _v ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e, _startpos_e_, _) = _menhir_stack in
            let (_endpos__1_, _startpos__1_, _1, _2) = (_endpos, _startpos, _v, ()) in
            let _v = _menhir_action_063 _1 _2 _endpos__1_ _startpos__1_ _startpos_e_ e in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__1_ _startpos_e_ _v _menhir_s _tok
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_NEQ (_menhir_stack, _menhir_s) in
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0073
        | Tokens.INT_LIT _v ->
            let _menhir_stack = MenhirCell1_NEQ (_menhir_stack, _menhir_s) in
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0073
        | Tokens.IF ->
            let _menhir_stack = MenhirCell1_NEQ (_menhir_stack, _menhir_s) in
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0073
        | Tokens.IDENTIFIER _v ->
            let _menhir_stack = MenhirCell1_NEQ (_menhir_stack, _menhir_s) in
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0073
        | Tokens.BOOL_LIT _v ->
            let _menhir_stack = MenhirCell1_NEQ (_menhir_stack, _menhir_s) in
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0073
        | Tokens.BNOT ->
            let _menhir_stack = MenhirCell1_NEQ (_menhir_stack, _menhir_s) in
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0073
        | Tokens.BITVECTOR_LIT _v ->
            let _menhir_stack = MenhirCell1_NEQ (_menhir_stack, _menhir_s) in
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0073
        | Tokens.ARBITRARY ->
            let _menhir_stack = MenhirCell1_NEQ (_menhir_stack, _menhir_s) in
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0073
        | _ ->
            _eRR ()
    
    and _menhir_run_0063 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_MUL (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0063 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0065 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_MOD (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0065 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0076 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, _menhir_s, _startpos, _endpos) in
        let _menhir_s = MenhirState0076 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0078 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_LT (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0078 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0026 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_LLBRACKET (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0026 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0086 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_LEQ (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0086 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0032 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_LBRACKET (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0032 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.COLON ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0033 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_COLON (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0033 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0037 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_IN (_menhir_stack, _menhir_s) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.LBRACE ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
            let _menhir_s = MenhirState0038 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0039 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0040 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0041 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0051 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MASK_LIT _v ->
                _menhir_run_0053 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0054 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LEQ ->
                _menhir_run_0055 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LBRACE ->
                _menhir_run_0169 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0170 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0171 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0183 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.GEQ ->
                _menhir_run_0215 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0217 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0218 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0220 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0221 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.BNOT ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell0_BNOT (_menhir_stack, _startpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LBRACE ->
                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
                let _menhir_s = MenhirState0324 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0039 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0040 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0041 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0051 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MASK_LIT _v ->
                    _menhir_run_0053 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0054 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LEQ ->
                    _menhir_run_0055 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LBRACE ->
                    _menhir_run_0169 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0170 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0171 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0183 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.GEQ ->
                    _menhir_run_0215 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0217 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0218 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0220 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.ARBITRARY ->
                    _menhir_run_0221 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0039 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let (_endpos_s_, _startpos_s_, s) = (_endpos, _startpos, _v) in
        let _v = _menhir_action_074 _endpos_s_ _startpos_s_ s in
        _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_s_ _startpos_s_ _v _menhir_s _tok
    
    and _menhir_goto_expr_pattern : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match _menhir_s with
        | MenhirState0054 ->
            _menhir_run_0319 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0476 ->
            _menhir_run_0231 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0324 ->
            _menhir_run_0231 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0038 ->
            _menhir_run_0231 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0316 ->
            _menhir_run_0231 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0169 ->
            _menhir_run_0231 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0219 ->
            _menhir_run_0231 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0270 ->
            _menhir_run_0231 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0266 ->
            _menhir_run_0231 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0229 ->
            _menhir_run_0231 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0319 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LPAR as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SLICING ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0232 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0234 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0236 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.RPAR ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (x, _endpos__3_) = (_v, _endpos_0) in
            let _v = _menhir_action_121 _endpos__3_ _startpos__1_ x in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos__1_ _v _menhir_s _tok
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0238 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0240 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0242 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0244 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0246 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0249 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0251 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0253 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0255 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0257 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0260 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0262 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0265 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0273 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0275 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0277 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0279 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0282 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0284 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0289 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0291 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0293 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0295 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0297 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0299 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0301 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0303 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0310 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.COMMA ->
            let (_endpos_expr_pattern_, _startpos_expr_pattern_, expr_pattern) = (_endpos, _startpos, _v) in
            let _v = _menhir_action_403 _endpos_expr_pattern_ _startpos_expr_pattern_ expr_pattern in
            _menhir_goto_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_expr_pattern_ _startpos_expr_pattern_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0232 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0232 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0234 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0234 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0236 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0236 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0238 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0238 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0240 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0240 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0242 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0242 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0244 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0244 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0246 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0246
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0246
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0246
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0246
        | Tokens.MASK_LIT _v ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e, _startpos_e_, _) = _menhir_stack in
            let (_endpos__1_, _startpos__1_, _1, _2) = (_endpos, _startpos, _v, ()) in
            let _v = _menhir_action_118 _1 _2 _endpos__1_ _startpos__1_ _startpos_e_ e in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__1_ _startpos_e_ _v _menhir_s _tok
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0246
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0246
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0246
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0246
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0246
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0246
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0246
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0246
        | _ ->
            _eRR ()
    
    and _menhir_run_0249 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0249 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0251 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0251 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0253 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _menhir_stack = MenhirCell0_MINUS (_menhir_stack, _startpos, _endpos) in
        let _menhir_s = MenhirState0253 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0255 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0255 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0257 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0257 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0260 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell0_LEQ (_menhir_stack, _startpos) in
        let _menhir_s = MenhirState0260 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0262 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0262 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.COLON ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0265 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.LBRACE ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
            let _menhir_s = MenhirState0266 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0039 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0040 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0041 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0051 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MASK_LIT _v ->
                _menhir_run_0053 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0054 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LEQ ->
                _menhir_run_0055 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LBRACE ->
                _menhir_run_0169 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0170 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0171 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0183 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.GEQ ->
                _menhir_run_0215 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0217 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0218 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0220 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0221 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.BNOT ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell0_BNOT (_menhir_stack, _startpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LBRACE ->
                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
                let _menhir_s = MenhirState0270 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0039 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0040 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0041 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0051 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MASK_LIT _v ->
                    _menhir_run_0053 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0054 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LEQ ->
                    _menhir_run_0055 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LBRACE ->
                    _menhir_run_0169 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0170 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0171 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0183 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.GEQ ->
                    _menhir_run_0215 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0217 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0218 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0220 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.ARBITRARY ->
                    _menhir_run_0221 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0040 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let (_endpos_r_, _startpos_r_, r) = (_endpos, _startpos, _v) in
        let _v = _menhir_action_072 _endpos_r_ _startpos_r_ r in
        _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_r_ _startpos_r_ _v _menhir_s _tok
    
    and _menhir_run_0041 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_NOT (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0041 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0051 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, _menhir_s, _startpos, _endpos) in
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0051
        | Tokens.REAL_LIT _v ->
            let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, _menhir_s, _startpos, _endpos) in
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0051
        | Tokens.NOT ->
            let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, _menhir_s, _startpos, _endpos) in
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0051
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, _menhir_s, _startpos, _endpos) in
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0051
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, _menhir_s, _startpos, _endpos) in
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0051
        | Tokens.INT_LIT _v ->
            let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, _menhir_s, _startpos, _endpos) in
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0051
        | Tokens.IF ->
            let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, _menhir_s, _startpos, _endpos) in
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0051
        | Tokens.IDENTIFIER _v ->
            let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, _menhir_s, _startpos, _endpos) in
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0051
        | Tokens.BOOL_LIT _v ->
            let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, _menhir_s, _startpos, _endpos) in
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0051
        | Tokens.BNOT ->
            let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, _menhir_s, _startpos, _endpos) in
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0051
        | Tokens.BITVECTOR_LIT _v ->
            let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, _menhir_s, _startpos, _endpos) in
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0051
        | Tokens.ARBITRARY ->
            let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, _menhir_s, _startpos, _endpos) in
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0051
        | Tokens.ARROW | Tokens.COMMA | Tokens.RBRACE | Tokens.RPAR | Tokens.WHERE ->
            let (_endpos__1_, _startpos__1_, _1) = (_endpos, _startpos, ()) in
            let _v = _menhir_action_405 _1 _endpos__1_ _startpos__1_ in
            _menhir_goto_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__1_ _startpos__1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_goto_pattern : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match _menhir_s with
        | MenhirState0054 ->
            _menhir_run_0315 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0476 ->
            _menhir_run_0228 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0324 ->
            _menhir_run_0228 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0038 ->
            _menhir_run_0228 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0316 ->
            _menhir_run_0228 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0169 ->
            _menhir_run_0228 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0270 ->
            _menhir_run_0228 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0266 ->
            _menhir_run_0228 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0229 ->
            _menhir_run_0228 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0219 ->
            _menhir_run_0228 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0315 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LPAR as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.COMMA ->
            let _menhir_s = MenhirState0316 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0039 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0040 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0041 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0051 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MASK_LIT _v ->
                _menhir_run_0053 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0054 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LEQ ->
                _menhir_run_0055 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LBRACE ->
                _menhir_run_0169 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0170 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0171 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0183 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.GEQ ->
                _menhir_run_0215 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0217 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0218 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0220 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0221 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0053 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let (_endpos__1_, _startpos__1_, _1) = (_endpos, _startpos, _v) in
        let _v = _menhir_action_409 _1 _endpos__1_ _startpos__1_ in
        _menhir_goto_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__1_ _startpos__1_ _v _menhir_s _tok
    
    and _menhir_run_0054 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0054 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0039 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0040 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0041 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0051 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MASK_LIT _v ->
            _menhir_run_0053 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0054 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LEQ ->
            _menhir_run_0055 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LBRACE ->
            _menhir_run_0169 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0170 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0171 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0183 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.GEQ ->
            _menhir_run_0215 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0217 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0218 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0220 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0221 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0055 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_LEQ (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0055 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0169 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0169 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0039 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0040 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0041 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0051 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MASK_LIT _v ->
            _menhir_run_0053 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0054 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LEQ ->
            _menhir_run_0055 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LBRACE ->
            _menhir_run_0169 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0170 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0171 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0183 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.GEQ ->
            _menhir_run_0215 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0217 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0218 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0220 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0221 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0170 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let (_endpos_i_, _startpos_i_, i) = (_endpos, _startpos, _v) in
        let _v = _menhir_action_070 _endpos_i_ _startpos_i_ i in
        _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_i_ _startpos_i_ _v _menhir_s _tok
    
    and _menhir_run_0171 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_IF (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0171 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0183 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.LPAR ->
            let _startpos_0 = _menhir_lexbuf.Lexing.lex_start_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v_1 ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0184
            | Tokens.RPAR ->
                let _endpos_2 = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                let (_endpos__3_, _startpos_name_, name) = (_endpos_2, _startpos, _v) in
                let _v = _menhir_action_105 _endpos__3_ _startpos_name_ name in
                _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_name_ _v _menhir_s _tok
            | Tokens.REAL_LIT _v_3 ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v_3 MenhirState0184
            | Tokens.NOT ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0184
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0184
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0184
            | Tokens.INT_LIT _v_4 ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v_4 MenhirState0184
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0184
            | Tokens.IDENTIFIER _v_5 ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v_5 MenhirState0184
            | Tokens.BOOL_LIT _v_6 ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_6 MenhirState0184
            | Tokens.BNOT ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0184
            | Tokens.BITVECTOR_LIT _v_7 ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v_7 MenhirState0184
            | Tokens.ARBITRARY ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0184
            | _ ->
                _eRR ())
        | Tokens.LBRACE ->
            let _startpos_8 = _menhir_lexbuf.Lexing.lex_start_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v_9 ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_8) in
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v_9 MenhirState0191
            | Tokens.REAL_LIT _v_10 ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_8) in
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v_10 MenhirState0191
            | Tokens.RBRACE ->
                let _endpos_11 = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                let (_endpos__3_, _endpos_desc_, _startpos_desc_, desc) = (_endpos_11, _endpos, _startpos, _v) in
                let _v = _menhir_action_122 _endpos__3_ _endpos_desc_ _startpos_desc_ desc in
                _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_desc_ _v _menhir_s _tok
            | Tokens.NOT ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_8) in
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0191
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_8) in
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0191
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_8) in
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0191
            | Tokens.INT_LIT _v_12 ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_8) in
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v_12 MenhirState0191
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_8) in
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0191
            | Tokens.IDENTIFIER _v_13 ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_8) in
                _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _v_13 MenhirState0191
            | Tokens.BOOL_LIT _v_14 ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_8) in
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_14 MenhirState0191
            | Tokens.BNOT ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_8) in
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0191
            | Tokens.BITVECTOR_LIT _v_15 ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_8) in
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v_15 MenhirState0191
            | Tokens.ARBITRARY ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_8) in
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0191
            | _ ->
                _eRR ())
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DOT | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.IN | Tokens.LBRACKET | Tokens.LEQ | Tokens.LLBRACKET | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RBRACE | Tokens.RDIV | Tokens.RPAR | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.WHERE ->
            let (_endpos__1_, _startpos__1_, _1) = (_endpos, _startpos, _v) in
            let _v = _menhir_action_075 _1 _endpos__1_ _startpos__1_ in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__1_ _startpos__1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0193 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.LBRACE ->
            let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0194 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.EQ ->
            let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0205 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.AND | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DOT | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.IN | Tokens.LBRACKET | Tokens.LEQ | Tokens.LLBRACKET | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RBRACE | Tokens.RDIV | Tokens.SHL | Tokens.SHR ->
            let (_endpos__1_, _startpos__1_, _1) = (_endpos, _startpos, _v) in
            let _v = _menhir_action_020 _1 _endpos__1_ _startpos__1_ in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__1_ _startpos__1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0194 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0194
        | Tokens.REAL_LIT _v ->
            let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0194
        | Tokens.RBRACE ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, desc, _startpos_desc_, _endpos_desc_) = _menhir_stack in
            let _endpos__3_ = _endpos in
            let _v = _menhir_action_067 _endpos__3_ _endpos_desc_ _startpos_desc_ desc in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_desc_ _v _menhir_s _tok
        | Tokens.NOT ->
            let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0194
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0194
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0194
        | Tokens.INT_LIT _v ->
            let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0194
        | Tokens.IF ->
            let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0194
        | Tokens.IDENTIFIER _v ->
            let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
            _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0194
        | Tokens.BOOL_LIT _v ->
            let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0194
        | Tokens.BNOT ->
            let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0194
        | Tokens.BITVECTOR_LIT _v ->
            let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0194
        | Tokens.ARBITRARY ->
            let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0194
        | _ ->
            _eRR ()
    
    and _menhir_run_0205 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0205 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0215 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_GEQ (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0215 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0217 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let (_endpos_b_, _startpos_b_, b) = (_endpos, _startpos, _v) in
        let _v = _menhir_action_071 _endpos_b_ _startpos_b_ b in
        _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_b_ _startpos_b_ _v _menhir_s _tok
    
    and _menhir_run_0218 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_BNOT (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0218 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LBRACE ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, _menhir_s, _startpos) in
            let _menhir_s = MenhirState0219 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0039 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0040 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0041 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0051 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MASK_LIT _v ->
                _menhir_run_0053 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0054 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LEQ ->
                _menhir_run_0055 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LBRACE ->
                _menhir_run_0169 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0170 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0171 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0183 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.GEQ ->
                _menhir_run_0215 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0217 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0218 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0220 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0221 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0220 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let (_endpos_b_, _startpos_b_, b) = (_endpos, _startpos, _v) in
        let _v = _menhir_action_073 _endpos_b_ _startpos_b_ b in
        _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_b_ _startpos_b_ _v _menhir_s _tok
    
    and _menhir_run_0221 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_ARBITRARY (_menhir_stack, _menhir_s, _startpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.COLON_COLON ->
            let _menhir_s = MenhirState0222 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON ->
            let _menhir_s = MenhirState0224 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0228 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_s = MenhirState0229 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0039 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0040 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0041 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0051 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MASK_LIT _v ->
                _menhir_run_0053 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0054 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LEQ ->
                _menhir_run_0055 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LBRACE ->
                _menhir_run_0169 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0170 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0171 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0183 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.GEQ ->
                _menhir_run_0215 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0217 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0218 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0220 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0221 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.ARROW | Tokens.RBRACE | Tokens.RPAR | Tokens.WHERE ->
            let (_endpos_x_, _startpos_x_, x) = (_endpos, _startpos, _v) in
            let _v = _menhir_action_435 x in
            _menhir_goto_separated_nonempty_list_COMMA_pattern_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_x_ _startpos_x_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_goto_separated_nonempty_list_COMMA_pattern_ : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match _menhir_s with
        | MenhirState0476 ->
            _menhir_run_0477 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok
        | MenhirState0324 ->
            _menhir_run_0325 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _tok
        | MenhirState0038 ->
            _menhir_run_0321 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _tok
        | MenhirState0316 ->
            _menhir_run_0317 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0169 ->
            _menhir_run_0313 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _tok
        | MenhirState0270 ->
            _menhir_run_0271 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _tok
        | MenhirState0266 ->
            _menhir_run_0267 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _tok
        | MenhirState0229 ->
            _menhir_run_0230 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0219 ->
            _menhir_run_0226 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _tok
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0477 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_WHEN as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_pattern_ (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.WHERE ->
            let _menhir_s = MenhirState0478 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.ARROW ->
            let _menhir_s = MenhirState0514 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0393 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.LPAR ->
            let _startpos_0 = _menhir_lexbuf.Lexing.lex_start_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.RPAR ->
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.SEMI_COLON ->
                    let _endpos_1 = _menhir_lexbuf.Lexing.lex_curr_p in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.WHILE ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0396
                    | Tokens.VAR ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0396
                    | Tokens.UNREACHABLE ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0396
                    | Tokens.TRY ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0396
                    | Tokens.THROW ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0396
                    | Tokens.RETURN ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0396
                    | Tokens.REPEAT ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0396
                    | Tokens.PRINT ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0396
                    | Tokens.PRAGMA ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0396
                    | Tokens.PASS ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0396
                    | Tokens.MINUS ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0396
                    | Tokens.LPAR ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0396
                    | Tokens.LET ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0396
                    | Tokens.IF ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0396
                    | Tokens.IDENTIFIER _v ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0396
                    | Tokens.FOR ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0396
                    | Tokens.DEBUG ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0396
                    | Tokens.CONSTANT ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0396
                    | Tokens.CASE ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0396
                    | Tokens.ASSERT ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0396
                    | Tokens.ARROBASE_LOOPLIMIT ->
                        let _menhir_stack = MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0396
                    | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                        let (_startpos__1_, _1, _endpos__2_, _2_inlined1, _3) = (_startpos, (), _endpos_1, (), ()) in
                        let _v = _menhir_action_281 _1 _2_inlined1 _3 _endpos__2_ _startpos__1_ in
                        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
                    | _ ->
                        _eRR ())
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0397 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_TRY (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0397 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.WHILE ->
            _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.VAR ->
            _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.UNREACHABLE ->
            _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.TRY ->
            _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.THROW ->
            _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.RETURN ->
            _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.REPEAT ->
            _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.PRINT ->
            _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.PRAGMA ->
            _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.PASS ->
            _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LET ->
            _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IF ->
            _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.FOR ->
            _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.DEBUG ->
            _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.CONSTANT ->
            _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.CASE ->
            _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.ASSERT ->
            _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.ARROBASE_LOOPLIMIT ->
            _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0398 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0398
        | Tokens.SEMI_COLON ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0398, _endpos) in
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0399
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0398, _endpos) in
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0399
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0398, _endpos) in
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0399
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0398, _endpos) in
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0399
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0398, _endpos) in
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0399
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0398, _endpos) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0399
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0398, _endpos) in
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0399
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0398, _endpos) in
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0399
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0398, _endpos) in
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0399
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0398, _endpos) in
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0399
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0398, _endpos) in
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0399
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0398, _endpos) in
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0399
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0398, _endpos) in
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0399
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0398, _endpos) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0399
            | Tokens.IDENTIFIER _v ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0398, _endpos) in
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0399
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0398, _endpos) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0399
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0398, _endpos) in
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0399
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0398, _endpos) in
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0399
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0398, _endpos) in
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0399
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0398, _endpos) in
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0399
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0398, _endpos) in
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0399
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let (_startpos__1_, _1, _endpos__2_) = (_startpos, (), _endpos) in
                let _v = _menhir_action_286 _1 _endpos__2_ _startpos__1_ in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.REAL_LIT _v ->
            let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0398
        | Tokens.NOT ->
            let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0398
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0398
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0398
        | Tokens.INT_LIT _v ->
            let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0398
        | Tokens.IF ->
            let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0398
        | Tokens.IDENTIFIER _v ->
            let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0398
        | Tokens.BOOL_LIT _v ->
            let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0398
        | Tokens.BNOT ->
            let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0398
        | Tokens.BITVECTOR_LIT _v ->
            let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0398
        | Tokens.ARBITRARY ->
            let _menhir_stack = MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0398
        | _ ->
            _eRR ()
    
    and _menhir_run_0400 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0400
        | Tokens.SEMI_COLON ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0400, _endpos) in
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0400, _endpos) in
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0400, _endpos) in
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0400, _endpos) in
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0400, _endpos) in
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0400, _endpos) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0400, _endpos) in
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0400, _endpos) in
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0400, _endpos) in
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0400, _endpos) in
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0400, _endpos) in
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0400, _endpos) in
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0400, _endpos) in
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0400, _endpos) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
            | Tokens.IDENTIFIER _v ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0400, _endpos) in
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0401
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0400, _endpos) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0400, _endpos) in
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0400, _endpos) in
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0400, _endpos) in
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0400, _endpos) in
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0400, _endpos) in
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0401
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let (_startpos__1_, _1, _endpos__2_) = (_startpos, (), _endpos) in
                let _v = _menhir_action_246 _1 _endpos__2_ _startpos__1_ in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.REAL_LIT _v ->
            let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0400
        | Tokens.NOT ->
            let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0400
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0400
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0400
        | Tokens.INT_LIT _v ->
            let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0400
        | Tokens.IF ->
            let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0400
        | Tokens.IDENTIFIER _v ->
            let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0400
        | Tokens.BOOL_LIT _v ->
            let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0400
        | Tokens.BNOT ->
            let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0400
        | Tokens.BITVECTOR_LIT _v ->
            let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0400
        | Tokens.ARBITRARY ->
            let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0400
        | _ ->
            _eRR ()
    
    and _menhir_run_0402 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_REPEAT (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0402 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.WHILE ->
            _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.VAR ->
            _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.UNREACHABLE ->
            _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.TRY ->
            _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.THROW ->
            _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.RETURN ->
            _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.REPEAT ->
            _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.PRINT ->
            _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.PRAGMA ->
            _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.PASS ->
            _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LET ->
            _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IF ->
            _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.FOR ->
            _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.DEBUG ->
            _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.CONSTANT ->
            _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.CASE ->
            _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.ASSERT ->
            _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.ARROBASE_LOOPLIMIT ->
            _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0403 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.LPAR ->
            let _startpos_0 = _menhir_lexbuf.Lexing.lex_start_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0404
            | Tokens.RPAR ->
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.SEMI_COLON ->
                    let _endpos_1 = _menhir_lexbuf.Lexing.lex_curr_p in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.WHILE ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0404, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0406
                    | Tokens.VAR ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0404, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0406
                    | Tokens.UNREACHABLE ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0404, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0406
                    | Tokens.TRY ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0404, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0406
                    | Tokens.THROW ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0404, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0406
                    | Tokens.RETURN ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0404, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0406
                    | Tokens.REPEAT ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0404, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0406
                    | Tokens.PRINT ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0404, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0406
                    | Tokens.PRAGMA ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0404, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0406
                    | Tokens.PASS ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0404, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0406
                    | Tokens.MINUS ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0404, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0406
                    | Tokens.LPAR ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0404, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0406
                    | Tokens.LET ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0404, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0406
                    | Tokens.IF ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0404, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0406
                    | Tokens.IDENTIFIER _v ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0404, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0406
                    | Tokens.FOR ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0404, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0406
                    | Tokens.DEBUG ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0404, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0406
                    | Tokens.CONSTANT ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0404, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0406
                    | Tokens.CASE ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0404, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0406
                    | Tokens.ASSERT ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0404, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0406
                    | Tokens.ARROBASE_LOOPLIMIT ->
                        let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0404, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0406
                    | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                        let (_startpos__1_, _1, _endpos__2_) = (_startpos, (), _endpos_1) in
                        let _v = _menhir_action_277 _1 _endpos__2_ _startpos__1_ in
                        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
                    | _ ->
                        _eRR ())
                | _ ->
                    _eRR ())
            | Tokens.REAL_LIT _v ->
                let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0404
            | Tokens.NOT ->
                let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0404
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0404
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0404
            | Tokens.INT_LIT _v ->
                let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0404
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0404
            | Tokens.IDENTIFIER _v ->
                let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0404
            | Tokens.BOOL_LIT _v ->
                let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0404
            | Tokens.BNOT ->
                let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0404
            | Tokens.BITVECTOR_LIT _v ->
                let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0404
            | Tokens.ARBITRARY ->
                let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0404
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0407 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.IDENTIFIER _v ->
            let _startpos_0 = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v_1 ->
                let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0408
            | Tokens.SEMI_COLON ->
                let _endpos_2 = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.WHILE ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0408, _endpos_2) in
                    _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0409
                | Tokens.VAR ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0408, _endpos_2) in
                    _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0409
                | Tokens.UNREACHABLE ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0408, _endpos_2) in
                    _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0409
                | Tokens.TRY ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0408, _endpos_2) in
                    _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0409
                | Tokens.THROW ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0408, _endpos_2) in
                    _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0409
                | Tokens.RETURN ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0408, _endpos_2) in
                    _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0409
                | Tokens.REPEAT ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0408, _endpos_2) in
                    _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0409
                | Tokens.PRINT ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0408, _endpos_2) in
                    _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0409
                | Tokens.PRAGMA ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0408, _endpos_2) in
                    _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0409
                | Tokens.PASS ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0408, _endpos_2) in
                    _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0409
                | Tokens.MINUS ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0408, _endpos_2) in
                    _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0409
                | Tokens.LPAR ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0408, _endpos_2) in
                    _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0409
                | Tokens.LET ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0408, _endpos_2) in
                    _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0409
                | Tokens.IF ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0408, _endpos_2) in
                    _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0409
                | Tokens.IDENTIFIER _v_3 ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0408, _endpos_2) in
                    _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_3 MenhirState0409
                | Tokens.FOR ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0408, _endpos_2) in
                    _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0409
                | Tokens.DEBUG ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0408, _endpos_2) in
                    _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0409
                | Tokens.CONSTANT ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0408, _endpos_2) in
                    _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0409
                | Tokens.CASE ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0408, _endpos_2) in
                    _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0409
                | Tokens.ASSERT ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0408, _endpos_2) in
                    _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0409
                | Tokens.ARROBASE_LOOPLIMIT ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                    let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0408, _endpos_2) in
                    _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0409
                | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                    let (_startpos__1_, _1, _endpos__2_, x) = (_startpos, (), _endpos_2, _v) in
                    let _v = _menhir_action_287 _1 _endpos__2_ _startpos__1_ x in
                    _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
                | _ ->
                    _eRR ())
            | Tokens.REAL_LIT _v_4 ->
                let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v_4 MenhirState0408
            | Tokens.NOT ->
                let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0408
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0408
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0408
            | Tokens.INT_LIT _v_5 ->
                let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v_5 MenhirState0408
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0408
            | Tokens.IDENTIFIER _v_6 ->
                let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v_6 MenhirState0408
            | Tokens.BOOL_LIT _v_7 ->
                let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_7 MenhirState0408
            | Tokens.BNOT ->
                let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0408
            | Tokens.BITVECTOR_LIT _v_8 ->
                let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v_8 MenhirState0408
            | Tokens.ARBITRARY ->
                let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0408
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0410 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0411
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0411
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0411
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0411
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0411
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0411
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0411
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0411
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0411
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0411
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0411
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0411
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0411
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0411
            | Tokens.IDENTIFIER _v ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0411
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0411
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0411
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0411
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0411
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0411
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0411
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let (_startpos__1_, _1, _endpos__2_) = (_startpos, (), _endpos) in
                let _v = _menhir_action_245 _1 _endpos__2_ _startpos__1_ in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0412 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let (_endpos__1_, _startpos__1_, _1) = (_endpos, _startpos, ()) in
        let _v = _menhir_action_125 _1 _endpos__1_ _startpos__1_ in
        _menhir_goto_lexpr _menhir_stack _menhir_lexbuf _menhir_lexer _startpos__1_ _v _menhir_s _tok
    
    and _menhir_goto_lexpr : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok ->
        match _menhir_s with
        | MenhirState0000 ->
            _menhir_run_1028 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState1377 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState1305 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState1297 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState1285 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState1082 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState1240 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState1100 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState1227 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState1221 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState1211 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState1194 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState1188 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState1108 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState1021 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState1017 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0965 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0961 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0953 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0949 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0864 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0810 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0787 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0792 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0742 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0358 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0735 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0737 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0363 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0733 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0730 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0727 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0721 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0717 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0709 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0706 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0392 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0396 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0397 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0700 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0696 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0698 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0691 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0687 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0679 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0399 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0675 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0401 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0402 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0671 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0667 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0662 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0406 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0657 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0409 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0411 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0652 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0649 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0643 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0443 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0446 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0638 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0632 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0634 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0627 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0622 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0612 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0608 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0602 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0597 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0587 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0583 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0579 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0574 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0564 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0450 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0557 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0559 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0553 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0555 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0545 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0547 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0459 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0543 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0540 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0463 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0535 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0532 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0526 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0472 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0521 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0514 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0517 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0480 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0511 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0483 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0502 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0506 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0490 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0500 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0495 ->
            _menhir_run_0497 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0433 ->
            _menhir_run_0432 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0429 ->
            _menhir_run_0432 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | MenhirState0413 ->
            _menhir_run_0428 _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_1028 : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> _ -> (ttv_stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_lexpr (_menhir_stack, _menhir_s, _v, _startpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.EQ ->
            let _menhir_s = MenhirState1029 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0497 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_lexpr (_menhir_stack, _menhir_s, _v, _startpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.EQ ->
            let _menhir_s = MenhirState0498 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0432 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_lexpr as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_lexpr (_menhir_stack, _menhir_s, _v, _startpos) in
            let _menhir_s = MenhirState0433 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.MINUS ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0414 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.RPAR ->
            let x = _v in
            let _v = _menhir_action_427 x in
            _menhir_goto_separated_nonempty_list_COMMA_lexpr_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0413 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0413 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.MINUS ->
            _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0414 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0414 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let (_endpos__1_, _startpos__1_, _1) = (_endpos, _startpos, _v) in
        let _v = _menhir_action_127 _1 in
        _menhir_goto_lexpr_atom_desc _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__1_ _startpos__1_ _v _menhir_s _tok
    
    and _menhir_goto_lexpr_atom_desc : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_lexpr_atom_desc (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_s = MenhirState0416 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_lexpr_atom_desc (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_s = MenhirState0419 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.COLON ->
                _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DOT ->
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LBRACKET ->
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.RBRACKET ->
                    let _endpos_12 = _menhir_lexbuf.Lexing.lex_curr_p in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    let (_2, _endpos__3_, _endpos_desc_, _startpos_desc_, desc) = ((), _endpos_12, _endpos, _startpos, _v) in
                    let _v = _menhir_action_131 _2 _endpos_desc_ _startpos_desc_ desc in
                    _menhir_goto_lexpr_atom_desc _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_desc_ _v _menhir_s _tok
                | Tokens.IDENTIFIER _v_13 ->
                    let _menhir_stack = MenhirCell1_lexpr_atom_desc (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                    _menhir_run_0045 _menhir_stack _menhir_lexbuf _menhir_lexer _v_13 MenhirState0423
                | _ ->
                    _eRR ())
            | Tokens.IDENTIFIER _v_14 ->
                let _endpos_16 = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                let (_2, _endpos_desc_, _startpos_desc_, desc, _endpos_field_, field) = ((), _endpos, _startpos, _v, _endpos_16, _v_14) in
                let _v = _menhir_action_130 _2 _endpos_desc_ _startpos_desc_ desc field in
                _menhir_goto_lexpr_atom_desc _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_field_ _startpos_desc_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.COMMA | Tokens.EQ | Tokens.RPAR ->
            let (_endpos_desc_, _startpos_desc_, desc) = (_endpos, _startpos, _v) in
            let _v = _menhir_action_124 _endpos_desc_ _startpos_desc_ desc in
            _menhir_goto_lexpr _menhir_stack _menhir_lexbuf _menhir_lexer _startpos_desc_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0045 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_s = MenhirState0046 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0045 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON | Tokens.COLON_COLON | Tokens.RBRACKET ->
            let x = _v in
            let _v = _menhir_action_415 x in
            _menhir_goto_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_goto_separated_nonempty_list_COMMA_IDENTIFIER_ : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        match _menhir_s with
        | MenhirState0929 ->
            _menhir_run_0930 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0904 ->
            _menhir_run_0905 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0881 ->
            _menhir_run_0882 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0748 ->
            _menhir_run_0749 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0617 ->
            _menhir_run_0618 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0592 ->
            _menhir_run_0593 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0569 ->
            _menhir_run_0570 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0423 ->
            _menhir_run_0425 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0388 ->
            _menhir_run_0389 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0285 ->
            _menhir_run_0286 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0044 ->
            _menhir_run_0048 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0046 ->
            _menhir_run_0047 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0930 : type  ttv_stack. ((((ttv_stack, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, _menhir_box_opn) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACKET ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_RBRACKET (_menhir_stack, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EQ ->
                let _menhir_s = MenhirState0932 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.ARBITRARY ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0905 : type  ttv_stack. (((ttv_stack, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACKET ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_RBRACKET (_menhir_stack, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EQ ->
                let _menhir_s = MenhirState0907 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.ARBITRARY ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0882 : type  ttv_stack. (((ttv_stack, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACKET ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_RBRACKET (_menhir_stack, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EQ ->
                let _menhir_s = MenhirState0884 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.ARBITRARY ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0749 : type  ttv_stack. (((ttv_stack, _menhir_box_opn) _menhir_cell1_VAR, _menhir_box_opn) _menhir_cell1_IDENTIFIER as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.COLON_COLON ->
            let _menhir_s = MenhirState0750 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON ->
            let _menhir_s = MenhirState0754 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0618 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, ttv_result) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACKET ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_RBRACKET (_menhir_stack, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EQ ->
                let _menhir_s = MenhirState0620 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.ARBITRARY ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0593 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACKET ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_RBRACKET (_menhir_stack, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EQ ->
                let _menhir_s = MenhirState0595 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.ARBITRARY ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0570 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, ttv_result) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACKET ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_RBRACKET (_menhir_stack, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EQ ->
                let _menhir_s = MenhirState0572 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.ARBITRARY ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0425 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_lexpr_atom_desc -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACKET ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_lexpr_atom_desc (_menhir_stack, _menhir_s, desc, _startpos_desc_, _endpos_desc_) = _menhir_stack in
            let (_2, _1_inlined1, _endpos__3_) = ((), _v, _endpos) in
            let _v = _menhir_action_132 _1_inlined1 _2 _endpos_desc_ _startpos_desc_ desc in
            _menhir_goto_lexpr_atom_desc _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_desc_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0389 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_VAR, ttv_result) _menhir_cell1_IDENTIFIER as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.COLON_COLON ->
            let _menhir_s = MenhirState0390 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON ->
            let _menhir_s = MenhirState0704 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0286 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACKET ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e, _startpos_e_, _) = _menhir_stack in
            let (_2, _1_inlined1, _endpos__3_) = ((), _v, _endpos) in
            let _v = _menhir_action_111 _1_inlined1 _2 _endpos__3_ _startpos_e_ e in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_e_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0048 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DOT -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACKET ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_DOT (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e, _startpos_e_, _) = _menhir_stack in
            let (_2, _1_inlined1, _endpos__3_) = ((), _v, _endpos) in
            let _v = _menhir_action_056 _1_inlined1 _2 _endpos__3_ _startpos_e_ e in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_e_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0047 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, x, _, _) = _menhir_stack in
        let xs = _v in
        let _v = _menhir_action_416 x xs in
        _menhir_goto_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
    
    and _menhir_goto_separated_nonempty_list_COMMA_lexpr_ : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_lexpr as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        match _menhir_s with
        | MenhirState0433 ->
            _menhir_run_0434 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0429 ->
            _menhir_run_0430 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0434 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_lexpr, ttv_result) _menhir_cell1_lexpr -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_lexpr (_menhir_stack, _menhir_s, x, _) = _menhir_stack in
        let xs = _v in
        let _v = _menhir_action_428 x xs in
        _menhir_goto_separated_nonempty_list_COMMA_lexpr_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_0430 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LPAR, ttv_result) _menhir_cell1_lexpr -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let MenhirCell1_lexpr (_menhir_stack, _, x, _) = _menhir_stack in
        let MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_2, _1_inlined1, _endpos__3_) = ((), _v, _endpos) in
        let _v = _menhir_action_126 _1_inlined1 _2 _endpos__3_ _startpos__1_ x in
        _menhir_goto_lexpr _menhir_stack _menhir_lexbuf _menhir_lexer _startpos__1_ _v _menhir_s _tok
    
    and _menhir_run_0428 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LPAR as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_lexpr (_menhir_stack, _menhir_s, _v, _startpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.COMMA ->
            let _menhir_s = MenhirState0429 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.MINUS ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0414 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0435 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_LET (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0435 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.MINUS ->
            _menhir_run_0365 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0370 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0371 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0370 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0370 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.MINUS ->
            _menhir_run_0365 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0370 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0371 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0371 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0372 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.COLON ->
            let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0374 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.COMMA | Tokens.EQ | Tokens.RPAR ->
            let _1 = _v in
            let _v = _menhir_action_009 _1 in
            _menhir_goto_decl_item _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0372 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0372 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.REAL ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INTEGER ->
            _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOLEAN ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITS ->
            _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BIT ->
            _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.ARRAY ->
            _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0374 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0374 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.REAL ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INTEGER ->
            _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOLEAN ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITS ->
            _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BIT ->
            _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.ARRAY ->
            _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0444 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_IF (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0444 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0447 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.LPAR ->
            let _startpos_0 = _menhir_lexbuf.Lexing.lex_start_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v_1 ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0448
            | Tokens.RPAR ->
                let _endpos_2 = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.SEMI_COLON ->
                    let _endpos_3 = _menhir_lexbuf.Lexing.lex_curr_p in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.WHILE ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0448, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                    | Tokens.VAR ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0448, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                    | Tokens.UNREACHABLE ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0448, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                    | Tokens.TRY ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0448, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                    | Tokens.THROW ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0448, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                    | Tokens.RETURN ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0448, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                    | Tokens.REPEAT ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0448, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                    | Tokens.PRINT ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0448, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                    | Tokens.PRAGMA ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0448, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                    | Tokens.PASS ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0448, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                    | Tokens.MINUS ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0448, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                    | Tokens.LPAR ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0448, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                    | Tokens.LET ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0448, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                    | Tokens.IF ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0448, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                    | Tokens.IDENTIFIER _v_4 ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0448, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_4 MenhirState0450
                    | Tokens.FOR ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0448, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                    | Tokens.DEBUG ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0448, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                    | Tokens.CONSTANT ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0448, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                    | Tokens.CASE ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0448, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                    | Tokens.ASSERT ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0448, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                    | Tokens.ARROBASE_LOOPLIMIT ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0448, _endpos_2) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_3) in
                        _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0450
                    | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                        let (_endpos__2_, _startpos_name_, name) = (_endpos_3, _startpos, _v) in
                        let _v = _menhir_action_248 _endpos__2_ _startpos_name_ name in
                        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
                    | _ ->
                        _eRR ())
                | Tokens.EQ ->
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                    let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0448, _endpos_2) in
                    let _menhir_s = MenhirState0562 in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.STRING_LIT _v ->
                        _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.REAL_LIT _v ->
                        _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.NOT ->
                        _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.MINUS ->
                        _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.LPAR ->
                        _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.INT_LIT _v ->
                        _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.IF ->
                        _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.IDENTIFIER _v ->
                        _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.BOOL_LIT _v ->
                        _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.BNOT ->
                        _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.BITVECTOR_LIT _v ->
                        _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.ARBITRARY ->
                        _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | _ ->
                        _eRR ())
                | Tokens.DOT ->
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                    let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0448, _endpos_2) in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.LBRACKET ->
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.IDENTIFIER _v ->
                            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                            let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
                            let _tok = _menhir_lexer _menhir_lexbuf in
                            (match (_tok : MenhirBasics.token) with
                            | Tokens.COMMA ->
                                let _menhir_s = MenhirState0569 in
                                let _tok = _menhir_lexer _menhir_lexbuf in
                                (match (_tok : MenhirBasics.token) with
                                | Tokens.IDENTIFIER _v ->
                                    _menhir_run_0045 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                                | _ ->
                                    _eRR ())
                            | _ ->
                                _eRR ())
                        | _ ->
                            _eRR ())
                    | Tokens.IDENTIFIER _v ->
                        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                        let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.EQ ->
                            let _menhir_s = MenhirState0577 in
                            let _tok = _menhir_lexer _menhir_lexbuf in
                            (match (_tok : MenhirBasics.token) with
                            | Tokens.STRING_LIT _v ->
                                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                            | Tokens.REAL_LIT _v ->
                                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                            | Tokens.NOT ->
                                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.MINUS ->
                                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.LPAR ->
                                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.INT_LIT _v ->
                                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                            | Tokens.IF ->
                                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.IDENTIFIER _v ->
                                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                            | Tokens.BOOL_LIT _v ->
                                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                            | Tokens.BNOT ->
                                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.BITVECTOR_LIT _v ->
                                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                            | Tokens.ARBITRARY ->
                                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | _ ->
                                _eRR ())
                        | _ ->
                            _eRR ())
                    | _ ->
                        _eRR ())
                | _ ->
                    _eRR ())
            | Tokens.REAL_LIT _v_24 ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v_24 MenhirState0448
            | Tokens.NOT ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0448
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0448
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0448
            | Tokens.INT_LIT _v_25 ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v_25 MenhirState0448
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0448
            | Tokens.IDENTIFIER _v_26 ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v_26 MenhirState0448
            | Tokens.BOOL_LIT _v_27 ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_27 MenhirState0448
            | Tokens.BNOT ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0448
            | Tokens.BITVECTOR_LIT _v_28 ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v_28 MenhirState0448
            | Tokens.ARBITRARY ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0448
            | _ ->
                _eRR ())
        | Tokens.LBRACE ->
            let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
            let _menhir_s = MenhirState0604 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DOT | Tokens.EQ | Tokens.LBRACKET | Tokens.LLBRACKET ->
            let (_endpos__1_, _startpos__1_, _1) = (_endpos, _startpos, _v) in
            let _v = _menhir_action_127 _1 in
            _menhir_goto_lexpr_atom_desc _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__1_ _startpos__1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0451 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_FOR (_menhir_stack, _menhir_s, _startpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.IDENTIFIER _v ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EQ ->
                let _menhir_s = MenhirState0453 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.ARBITRARY ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0460 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.LPAR ->
            let _startpos_0 = _menhir_lexbuf.Lexing.lex_start_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0461
            | Tokens.RPAR ->
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.SEMI_COLON ->
                    let _endpos_1 = _menhir_lexbuf.Lexing.lex_curr_p in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.WHILE ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0461, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0463
                    | Tokens.VAR ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0461, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0463
                    | Tokens.UNREACHABLE ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0461, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0463
                    | Tokens.TRY ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0461, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0463
                    | Tokens.THROW ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0461, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0463
                    | Tokens.RETURN ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0461, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0463
                    | Tokens.REPEAT ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0461, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0463
                    | Tokens.PRINT ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0461, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0463
                    | Tokens.PRAGMA ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0461, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0463
                    | Tokens.PASS ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0461, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0463
                    | Tokens.MINUS ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0461, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0463
                    | Tokens.LPAR ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0461, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0463
                    | Tokens.LET ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0461, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0463
                    | Tokens.IF ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0461, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0463
                    | Tokens.IDENTIFIER _v ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0461, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0463
                    | Tokens.FOR ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0461, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0463
                    | Tokens.DEBUG ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0461, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0463
                    | Tokens.CONSTANT ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0461, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0463
                    | Tokens.CASE ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0461, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0463
                    | Tokens.ASSERT ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0461, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0463
                    | Tokens.ARROBASE_LOOPLIMIT ->
                        let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0461, _endpos) in
                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_1) in
                        _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0463
                    | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                        let (_startpos__1_, _1, _endpos__2_) = (_startpos, (), _endpos_1) in
                        let _v = _menhir_action_279 _1 _endpos__2_ _startpos__1_ in
                        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
                    | _ ->
                        _eRR ())
                | _ ->
                    _eRR ())
            | Tokens.REAL_LIT _v ->
                let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0461
            | Tokens.NOT ->
                let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0461
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0461
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0461
            | Tokens.INT_LIT _v ->
                let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0461
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0461
            | Tokens.IDENTIFIER _v ->
                let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0461
            | Tokens.BOOL_LIT _v ->
                let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0461
            | Tokens.BNOT ->
                let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0461
            | Tokens.BITVECTOR_LIT _v ->
                let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0461
            | Tokens.ARBITRARY ->
                let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos) in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_0) in
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0461
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0464 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_CONSTANT (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0464 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.MINUS ->
            _menhir_run_0365 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0370 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0371 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0473 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_CASE (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0473 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0481 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_ASSERT (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0481 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0484 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_ARROBASE_LOOPLIMIT (_menhir_stack, _menhir_s, _startpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.LPAR ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos) in
            let _menhir_s = MenhirState0485 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_goto_nonempty_list_stmt_ : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match _menhir_s with
        | MenhirState1377 ->
            _menhir_run_1379 _menhir_stack _v _tok
        | MenhirState1305 ->
            _menhir_run_1306 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1297 ->
            _menhir_run_1298 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1285 ->
            _menhir_run_1286 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1240 ->
            _menhir_run_1241 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1227 ->
            _menhir_run_1228 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1221 ->
            _menhir_run_1222 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1211 ->
            _menhir_run_1212 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1194 ->
            _menhir_run_1195 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1188 ->
            _menhir_run_1189 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1108 ->
            _menhir_run_1109 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1100 ->
            _menhir_run_1101 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1082 ->
            _menhir_run_1083 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1021 ->
            _menhir_run_1022 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1017 ->
            _menhir_run_1018 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0965 ->
            _menhir_run_0966 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0961 ->
            _menhir_run_0962 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0953 ->
            _menhir_run_0954 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0949 ->
            _menhir_run_0950 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0864 ->
            _menhir_run_0865 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0810 ->
            _menhir_run_0811 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0792 ->
            _menhir_run_0793 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0787 ->
            _menhir_run_0788 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0742 ->
            _menhir_run_0743 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0358 ->
            _menhir_run_0739 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0737 ->
            _menhir_run_0738 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0735 ->
            _menhir_run_0736 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0733 ->
            _menhir_run_0734 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0363 ->
            _menhir_run_0732 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0730 ->
            _menhir_run_0731 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0727 ->
            _menhir_run_0728 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0721 ->
            _menhir_run_0722 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0717 ->
            _menhir_run_0718 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0709 ->
            _menhir_run_0710 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0706 ->
            _menhir_run_0707 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0392 ->
            _menhir_run_0703 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0396 ->
            _menhir_run_0702 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0700 ->
            _menhir_run_0701 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0698 ->
            _menhir_run_0699 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0696 ->
            _menhir_run_0697 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0691 ->
            _menhir_run_0692 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0687 ->
            _menhir_run_0688 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0397 ->
            _menhir_run_0681 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0679 ->
            _menhir_run_0680 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0399 ->
            _menhir_run_0677 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0675 ->
            _menhir_run_0676 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0401 ->
            _menhir_run_0673 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0671 ->
            _menhir_run_0672 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0667 ->
            _menhir_run_0668 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0402 ->
            _menhir_run_0664 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0662 ->
            _menhir_run_0663 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0406 ->
            _menhir_run_0659 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0657 ->
            _menhir_run_0658 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0409 ->
            _menhir_run_0655 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0411 ->
            _menhir_run_0654 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0652 ->
            _menhir_run_0653 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0649 ->
            _menhir_run_0650 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0643 ->
            _menhir_run_0644 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0443 ->
            _menhir_run_0640 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0638 ->
            _menhir_run_0639 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0634 ->
            _menhir_run_0635 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0632 ->
            _menhir_run_0633 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0446 ->
            _menhir_run_0629 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0627 ->
            _menhir_run_0628 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0622 ->
            _menhir_run_0623 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0612 ->
            _menhir_run_0613 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0608 ->
            _menhir_run_0609 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0602 ->
            _menhir_run_0603 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0597 ->
            _menhir_run_0598 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0587 ->
            _menhir_run_0588 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0583 ->
            _menhir_run_0584 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0579 ->
            _menhir_run_0580 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0574 ->
            _menhir_run_0575 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0564 ->
            _menhir_run_0565 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0450 ->
            _menhir_run_0561 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0559 ->
            _menhir_run_0560 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0557 ->
            _menhir_run_0558 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0555 ->
            _menhir_run_0556 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0553 ->
            _menhir_run_0554 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0547 ->
            _menhir_run_0548 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0545 ->
            _menhir_run_0546 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0543 ->
            _menhir_run_0544 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0459 ->
            _menhir_run_0542 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0540 ->
            _menhir_run_0541 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0463 ->
            _menhir_run_0537 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0535 ->
            _menhir_run_0536 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0532 ->
            _menhir_run_0533 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0526 ->
            _menhir_run_0527 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0472 ->
            _menhir_run_0523 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0521 ->
            _menhir_run_0522 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0517 ->
            _menhir_run_0518 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0514 ->
            _menhir_run_0515 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0511 ->
            _menhir_run_0512 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0480 ->
            _menhir_run_0509 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0483 ->
            _menhir_run_0508 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0506 ->
            _menhir_run_0507 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0502 ->
            _menhir_run_0503 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0500 ->
            _menhir_run_0501 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0495 ->
            _menhir_run_0496 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0490 ->
            _menhir_run_0491 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_1306 : type  ttv_stack. (((((ttv_stack, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1306
        | _ ->
            _eRR ()
    
    and _menhir_run_0492 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _menhir_stack = MenhirCell1_END (_menhir_stack, _menhir_s, _startpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let (_endpos_x_, x) = (_endpos, ()) in
            let _v = _menhir_action_400 x in
            _menhir_goto_option_SEMI_COLON_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_x_ _v _tok
        | Tokens.ARROBASE_LOOPLIMIT | Tokens.ASSERT | Tokens.CASE | Tokens.CATCH | Tokens.CONFIG | Tokens.CONSTANT | Tokens.DEBUG | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.FOR | Tokens.FUNC | Tokens.GETTER | Tokens.IDENTIFIER _ | Tokens.IF | Tokens.LET | Tokens.LPAR | Tokens.MINUS | Tokens.OTHERWISE | Tokens.PASS | Tokens.PRAGMA | Tokens.PRINT | Tokens.REPEAT | Tokens.RETURN | Tokens.SETTER | Tokens.THROW | Tokens.TRY | Tokens.TYPE | Tokens.UNREACHABLE | Tokens.UNTIL | Tokens.VAR | Tokens.WHEN | Tokens.WHILE ->
            let _v = _menhir_action_399 () in
            _menhir_goto_option_SEMI_COLON_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | _ ->
            _eRR ()
    
    and _menhir_goto_option_SEMI_COLON_ : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_END -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_END (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_opt_, opt) = ((), _endpos, _v) in
        let _v = _menhir_action_014 _1 _endpos_opt_ _startpos__1_ opt in
        _menhir_goto_end_semicolon _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_opt_ _v _menhir_s _tok
    
    and _menhir_goto_end_semicolon : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match _menhir_s with
        | MenhirState1305 ->
            _menhir_run_1309 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1306 ->
            _menhir_run_1307 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1297 ->
            _menhir_run_1301 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1298 ->
            _menhir_run_1299 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1285 ->
            _menhir_run_1289 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1286 ->
            _menhir_run_1287 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1082 ->
            _menhir_run_1281 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1240 ->
            _menhir_run_1244 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1241 ->
            _menhir_run_1242 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1100 ->
            _menhir_run_1234 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1227 ->
            _menhir_run_1231 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1228 ->
            _menhir_run_1229 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1221 ->
            _menhir_run_1225 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1222 ->
            _menhir_run_1223 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1211 ->
            _menhir_run_1215 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1212 ->
            _menhir_run_1213 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1194 ->
            _menhir_run_1198 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1195 ->
            _menhir_run_1196 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1188 ->
            _menhir_run_1192 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1189 ->
            _menhir_run_1190 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1108 ->
            _menhir_run_1182 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1109 ->
            _menhir_run_1110 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1101 ->
            _menhir_run_1102 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1083 ->
            _menhir_run_1084 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState1018 ->
            _menhir_run_1019 _menhir_stack _endpos _tok
        | MenhirState1004 ->
            _menhir_run_1005 _menhir_stack _endpos _tok
        | MenhirState0966 ->
            _menhir_run_0967 _menhir_stack _endpos _tok
        | MenhirState0962 ->
            _menhir_run_0963 _menhir_stack _endpos _tok
        | MenhirState0954 ->
            _menhir_run_0955 _menhir_stack _endpos _tok
        | MenhirState0950 ->
            _menhir_run_0951 _menhir_stack _endpos _tok
        | MenhirState0866 ->
            _menhir_run_0867 _menhir_stack _endpos _tok
        | MenhirState0790 ->
            _menhir_run_0796 _menhir_stack _endpos _tok
        | MenhirState0793 ->
            _menhir_run_0794 _menhir_stack _endpos _tok
        | MenhirState0743 ->
            _menhir_run_0744 _menhir_stack _endpos _tok
        | MenhirState0739 ->
            _menhir_run_0740 _menhir_stack _endpos _tok
        | MenhirState0736 ->
            _menhir_run_0737 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0732 ->
            _menhir_run_0733 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0694 ->
            _menhir_run_0700 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0697 ->
            _menhir_run_0698 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0637 ->
            _menhir_run_0638 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0558 ->
            _menhir_run_0559 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0554 ->
            _menhir_run_0555 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0546 ->
            _menhir_run_0547 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0542 ->
            _menhir_run_0543 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0520 ->
            _menhir_run_0521 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0491 ->
            _menhir_run_0495 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_1309 : type  ttv_stack. (((((ttv_stack, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1309
        | Tokens.TYPE ->
            _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1309
        | Tokens.SETTER ->
            _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1309
        | Tokens.PRAGMA ->
            _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1309
        | Tokens.LET ->
            _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1309
        | Tokens.GETTER ->
            _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1309
        | Tokens.FUNC ->
            _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1309
        | Tokens.CONSTANT ->
            _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1309
        | Tokens.CONFIG ->
            _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1309
        | Tokens.EOF ->
            let _v_0 = _menhir_action_133 () in
            _menhir_run_1310 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_1085 : type  ttv_stack. ttv_stack -> _ -> _ -> (ttv_stack, _menhir_box_spec) _menhir_state -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.IDENTIFIER _v ->
            let _startpos_0 = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v_1 ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState1086
            | Tokens.SEMI_COLON ->
                let _endpos_2 = _menhir_lexbuf.Lexing.lex_curr_p in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState1086, _endpos_2) in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.VAR ->
                    _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1087
                | Tokens.TYPE ->
                    _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1087
                | Tokens.SETTER ->
                    _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1087
                | Tokens.PRAGMA ->
                    _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1087
                | Tokens.LET ->
                    _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1087
                | Tokens.GETTER ->
                    _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1087
                | Tokens.FUNC ->
                    _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1087
                | Tokens.CONSTANT ->
                    _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1087
                | Tokens.CONFIG ->
                    _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1087
                | Tokens.EOF ->
                    let _v_3 = _menhir_action_133 () in
                    _menhir_run_1276 _menhir_stack _v_3
                | _ ->
                    _eRR ())
            | Tokens.REAL_LIT _v_4 ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v_4 MenhirState1086
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1086
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1086
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1086
            | Tokens.INT_LIT _v_5 ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v_5 MenhirState1086
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1086
            | Tokens.IDENTIFIER _v_6 ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v_6 MenhirState1086
            | Tokens.BOOL_LIT _v_7 ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_7 MenhirState1086
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1086
            | Tokens.BITVECTOR_LIT _v_8 ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v_8 MenhirState1086
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1086
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1088 : type  ttv_stack. ttv_stack -> _ -> _ -> (ttv_stack, _menhir_box_spec) _menhir_state -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_LET (_menhir_stack, _menhir_s, _startpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.MINUS ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_MINUS (_menhir_stack, _startpos, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EQ ->
                let _menhir_s = MenhirState1090 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.ARBITRARY ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.COLON_COLON ->
                let _menhir_s = MenhirState1247 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.COLON ->
                let _menhir_s = MenhirState1253 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.IDENTIFIER _v ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EQ ->
                let _menhir_s = MenhirState1260 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.ARBITRARY ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.COLON_COLON ->
                let _menhir_s = MenhirState1264 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.COLON ->
                let _menhir_s = MenhirState1270 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1093 : type  ttv_stack. ttv_stack -> _ -> _ -> (ttv_stack, _menhir_box_spec) _menhir_state -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_GETTER (_menhir_stack, _menhir_s, _startpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.IDENTIFIER _v ->
            let _startpos_0 = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LBRACE ->
                _menhir_run_1061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1094
            | Tokens.LPAR ->
                let _v_1 = _menhir_action_194 () in
                _menhir_run_1095 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState1094 _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1095 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.LPAR ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos) in
            let _menhir_s = MenhirState1096 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.RPAR ->
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, _menhir_s, _endpos) in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.ARROW ->
                    let _menhir_s = MenhirState1098 in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.STRING ->
                        _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.REAL ->
                        _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.LPAR ->
                        _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.INTEGER ->
                        _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.IDENTIFIER _v ->
                        _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.BOOLEAN ->
                        _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.BITS ->
                        _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.BIT ->
                        _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.ARRAY ->
                        _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | _ ->
                        _eRR ())
                | _ ->
                    _eRR ())
            | Tokens.IDENTIFIER _v ->
                _menhir_run_1200 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1200 : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_spec) _menhir_state -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.COLON_COLON ->
            let _menhir_s = MenhirState1201 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON ->
            let _menhir_s = MenhirState1205 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1103 : type  ttv_stack. ttv_stack -> _ -> _ -> (ttv_stack, _menhir_box_spec) _menhir_state -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_FUNC (_menhir_stack, _menhir_s, _startpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.IDENTIFIER _v ->
            let _startpos_0 = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_0, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LBRACE ->
                _menhir_run_1061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1104
            | Tokens.LPAR ->
                let _v_1 = _menhir_action_194 () in
                _menhir_run_1105 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState1104 _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1111 : type  ttv_stack. ttv_stack -> _ -> _ -> (ttv_stack, _menhir_box_spec) _menhir_state -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_CONSTANT (_menhir_stack, _menhir_s, _startpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.MINUS ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_MINUS (_menhir_stack, _startpos, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EQ ->
                let _menhir_s = MenhirState1113 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.ARBITRARY ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.COLON_COLON ->
                let _menhir_s = MenhirState1152 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.COLON ->
                let _menhir_s = MenhirState1158 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.IDENTIFIER _v ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EQ ->
                let _menhir_s = MenhirState1165 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.ARBITRARY ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.COLON_COLON ->
                let _menhir_s = MenhirState1169 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.COLON ->
                let _menhir_s = MenhirState1175 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1116 : type  ttv_stack. ttv_stack -> _ -> _ -> (ttv_stack, _menhir_box_spec) _menhir_state -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_CONFIG (_menhir_stack, _menhir_s, _startpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.MINUS ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_MINUS (_menhir_stack, _startpos, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EQ ->
                let _menhir_s = MenhirState1118 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.ARBITRARY ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.COLON_COLON ->
                let _menhir_s = MenhirState1122 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.COLON ->
                let _menhir_s = MenhirState1128 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.IDENTIFIER _v ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EQ ->
                let _menhir_s = MenhirState1135 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.ARBITRARY ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.COLON_COLON ->
                let _menhir_s = MenhirState1139 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.COLON ->
                let _menhir_s = MenhirState1145 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1307 : type  ttv_stack. ((((((ttv_stack, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1307
        | Tokens.TYPE ->
            _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1307
        | Tokens.SETTER ->
            _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1307
        | Tokens.PRAGMA ->
            _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1307
        | Tokens.LET ->
            _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1307
        | Tokens.GETTER ->
            _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1307
        | Tokens.FUNC ->
            _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1307
        | Tokens.CONSTANT ->
            _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1307
        | Tokens.CONFIG ->
            _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1307
        | Tokens.EOF ->
            let _v_0 = _menhir_action_133 () in
            _menhir_run_1308 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_1301 : type  ttv_stack. (((((ttv_stack, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | Tokens.TYPE ->
            _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | Tokens.SETTER ->
            _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | Tokens.PRAGMA ->
            _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | Tokens.LET ->
            _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | Tokens.GETTER ->
            _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | Tokens.FUNC ->
            _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | Tokens.CONSTANT ->
            _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | Tokens.CONFIG ->
            _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1301
        | Tokens.EOF ->
            let _v_0 = _menhir_action_133 () in
            _menhir_run_1302 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_1299 : type  ttv_stack. ((((((ttv_stack, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | Tokens.TYPE ->
            _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | Tokens.SETTER ->
            _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | Tokens.PRAGMA ->
            _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | Tokens.LET ->
            _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | Tokens.GETTER ->
            _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | Tokens.FUNC ->
            _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | Tokens.CONSTANT ->
            _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | Tokens.CONFIG ->
            _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1299
        | Tokens.EOF ->
            let _v_0 = _menhir_action_133 () in
            _menhir_run_1300 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_1289 : type  ttv_stack. (((((ttv_stack, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | Tokens.TYPE ->
            _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | Tokens.SETTER ->
            _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | Tokens.PRAGMA ->
            _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | Tokens.LET ->
            _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | Tokens.GETTER ->
            _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | Tokens.FUNC ->
            _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | Tokens.CONSTANT ->
            _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | Tokens.CONFIG ->
            _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1289
        | Tokens.EOF ->
            let _v_0 = _menhir_action_133 () in
            _menhir_run_1290 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_1287 : type  ttv_stack. ((((((ttv_stack, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | Tokens.TYPE ->
            _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | Tokens.SETTER ->
            _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | Tokens.PRAGMA ->
            _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | Tokens.LET ->
            _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | Tokens.GETTER ->
            _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | Tokens.FUNC ->
            _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | Tokens.CONSTANT ->
            _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | Tokens.CONFIG ->
            _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1287
        | Tokens.EOF ->
            let _v_0 = _menhir_action_133 () in
            _menhir_run_1288 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_1281 : type  ttv_stack. (((((ttv_stack, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1281
        | Tokens.TYPE ->
            _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1281
        | Tokens.SETTER ->
            _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1281
        | Tokens.PRAGMA ->
            _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1281
        | Tokens.LET ->
            _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1281
        | Tokens.GETTER ->
            _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1281
        | Tokens.FUNC ->
            _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1281
        | Tokens.CONSTANT ->
            _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1281
        | Tokens.CONFIG ->
            _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1281
        | Tokens.EOF ->
            let _v_0 = _menhir_action_133 () in
            _menhir_run_1282 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_1244 : type  ttv_stack. (((((ttv_stack, _menhir_box_spec) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1244
        | Tokens.TYPE ->
            _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1244
        | Tokens.SETTER ->
            _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1244
        | Tokens.PRAGMA ->
            _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1244
        | Tokens.LET ->
            _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1244
        | Tokens.GETTER ->
            _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1244
        | Tokens.FUNC ->
            _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1244
        | Tokens.CONSTANT ->
            _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1244
        | Tokens.CONFIG ->
            _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1244
        | Tokens.EOF ->
            let _v_0 = _menhir_action_133 () in
            _menhir_run_1245 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_1242 : type  ttv_stack. ((((((ttv_stack, _menhir_box_spec) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1242
        | Tokens.TYPE ->
            _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1242
        | Tokens.SETTER ->
            _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1242
        | Tokens.PRAGMA ->
            _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1242
        | Tokens.LET ->
            _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1242
        | Tokens.GETTER ->
            _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1242
        | Tokens.FUNC ->
            _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1242
        | Tokens.CONSTANT ->
            _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1242
        | Tokens.CONFIG ->
            _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1242
        | Tokens.EOF ->
            let _v_0 = _menhir_action_133 () in
            _menhir_run_1243 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_1234 : type  ttv_stack. (((((ttv_stack, _menhir_box_spec) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1234
        | Tokens.TYPE ->
            _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1234
        | Tokens.SETTER ->
            _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1234
        | Tokens.PRAGMA ->
            _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1234
        | Tokens.LET ->
            _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1234
        | Tokens.GETTER ->
            _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1234
        | Tokens.FUNC ->
            _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1234
        | Tokens.CONSTANT ->
            _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1234
        | Tokens.CONFIG ->
            _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1234
        | Tokens.EOF ->
            let _v_0 = _menhir_action_133 () in
            _menhir_run_1235 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_1231 : type  ttv_stack. (((((ttv_stack, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | Tokens.TYPE ->
            _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | Tokens.SETTER ->
            _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | Tokens.PRAGMA ->
            _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | Tokens.LET ->
            _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | Tokens.GETTER ->
            _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | Tokens.FUNC ->
            _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | Tokens.CONSTANT ->
            _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | Tokens.CONFIG ->
            _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1231
        | Tokens.EOF ->
            let _v_0 = _menhir_action_133 () in
            _menhir_run_1232 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_1229 : type  ttv_stack. ((((((ttv_stack, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1229
        | Tokens.TYPE ->
            _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1229
        | Tokens.SETTER ->
            _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1229
        | Tokens.PRAGMA ->
            _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1229
        | Tokens.LET ->
            _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1229
        | Tokens.GETTER ->
            _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1229
        | Tokens.FUNC ->
            _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1229
        | Tokens.CONSTANT ->
            _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1229
        | Tokens.CONFIG ->
            _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1229
        | Tokens.EOF ->
            let _v_0 = _menhir_action_133 () in
            _menhir_run_1230 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_1225 : type  ttv_stack. (((((((ttv_stack, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1225
        | Tokens.TYPE ->
            _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1225
        | Tokens.SETTER ->
            _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1225
        | Tokens.PRAGMA ->
            _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1225
        | Tokens.LET ->
            _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1225
        | Tokens.GETTER ->
            _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1225
        | Tokens.FUNC ->
            _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1225
        | Tokens.CONSTANT ->
            _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1225
        | Tokens.CONFIG ->
            _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1225
        | Tokens.EOF ->
            let _v_0 = _menhir_action_133 () in
            _menhir_run_1226 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_1223 : type  ttv_stack. ((((((((ttv_stack, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1223
        | Tokens.TYPE ->
            _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1223
        | Tokens.SETTER ->
            _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1223
        | Tokens.PRAGMA ->
            _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1223
        | Tokens.LET ->
            _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1223
        | Tokens.GETTER ->
            _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1223
        | Tokens.FUNC ->
            _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1223
        | Tokens.CONSTANT ->
            _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1223
        | Tokens.CONFIG ->
            _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1223
        | Tokens.EOF ->
            let _v_0 = _menhir_action_133 () in
            _menhir_run_1224 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_1215 : type  ttv_stack. ((((ttv_stack, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1215
        | Tokens.TYPE ->
            _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1215
        | Tokens.SETTER ->
            _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1215
        | Tokens.PRAGMA ->
            _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1215
        | Tokens.LET ->
            _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1215
        | Tokens.GETTER ->
            _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1215
        | Tokens.FUNC ->
            _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1215
        | Tokens.CONSTANT ->
            _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1215
        | Tokens.CONFIG ->
            _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1215
        | Tokens.EOF ->
            let _v_0 = _menhir_action_133 () in
            _menhir_run_1216 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_1213 : type  ttv_stack. (((((ttv_stack, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1213
        | Tokens.TYPE ->
            _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1213
        | Tokens.SETTER ->
            _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1213
        | Tokens.PRAGMA ->
            _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1213
        | Tokens.LET ->
            _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1213
        | Tokens.GETTER ->
            _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1213
        | Tokens.FUNC ->
            _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1213
        | Tokens.CONSTANT ->
            _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1213
        | Tokens.CONFIG ->
            _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1213
        | Tokens.EOF ->
            let _v_0 = _menhir_action_133 () in
            _menhir_run_1214 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_1198 : type  ttv_stack. (((((ttv_stack, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1198
        | Tokens.TYPE ->
            _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1198
        | Tokens.SETTER ->
            _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1198
        | Tokens.PRAGMA ->
            _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1198
        | Tokens.LET ->
            _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1198
        | Tokens.GETTER ->
            _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1198
        | Tokens.FUNC ->
            _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1198
        | Tokens.CONSTANT ->
            _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1198
        | Tokens.CONFIG ->
            _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1198
        | Tokens.EOF ->
            let _v_0 = _menhir_action_133 () in
            _menhir_run_1199 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_1196 : type  ttv_stack. ((((((ttv_stack, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1196
        | Tokens.TYPE ->
            _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1196
        | Tokens.SETTER ->
            _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1196
        | Tokens.PRAGMA ->
            _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1196
        | Tokens.LET ->
            _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1196
        | Tokens.GETTER ->
            _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1196
        | Tokens.FUNC ->
            _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1196
        | Tokens.CONSTANT ->
            _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1196
        | Tokens.CONFIG ->
            _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1196
        | Tokens.EOF ->
            let _v_0 = _menhir_action_133 () in
            _menhir_run_1197 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_1192 : type  ttv_stack. (((((((ttv_stack, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1192
        | Tokens.TYPE ->
            _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1192
        | Tokens.SETTER ->
            _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1192
        | Tokens.PRAGMA ->
            _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1192
        | Tokens.LET ->
            _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1192
        | Tokens.GETTER ->
            _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1192
        | Tokens.FUNC ->
            _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1192
        | Tokens.CONSTANT ->
            _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1192
        | Tokens.CONFIG ->
            _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1192
        | Tokens.EOF ->
            let _v_0 = _menhir_action_133 () in
            _menhir_run_1193 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_1190 : type  ttv_stack. ((((((((ttv_stack, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1190
        | Tokens.TYPE ->
            _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1190
        | Tokens.SETTER ->
            _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1190
        | Tokens.PRAGMA ->
            _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1190
        | Tokens.LET ->
            _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1190
        | Tokens.GETTER ->
            _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1190
        | Tokens.FUNC ->
            _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1190
        | Tokens.CONSTANT ->
            _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1190
        | Tokens.CONFIG ->
            _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1190
        | Tokens.EOF ->
            let _v_0 = _menhir_action_133 () in
            _menhir_run_1191 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_1182 : type  ttv_stack. ((((ttv_stack, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1182
        | Tokens.TYPE ->
            _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1182
        | Tokens.SETTER ->
            _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1182
        | Tokens.PRAGMA ->
            _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1182
        | Tokens.LET ->
            _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1182
        | Tokens.GETTER ->
            _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1182
        | Tokens.FUNC ->
            _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1182
        | Tokens.CONSTANT ->
            _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1182
        | Tokens.CONFIG ->
            _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1182
        | Tokens.EOF ->
            let _v_0 = _menhir_action_133 () in
            _menhir_run_1183 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_1110 : type  ttv_stack. (((((ttv_stack, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1110
        | Tokens.TYPE ->
            _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1110
        | Tokens.SETTER ->
            _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1110
        | Tokens.PRAGMA ->
            _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1110
        | Tokens.LET ->
            _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1110
        | Tokens.GETTER ->
            _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1110
        | Tokens.FUNC ->
            _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1110
        | Tokens.CONSTANT ->
            _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1110
        | Tokens.CONFIG ->
            _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1110
        | Tokens.EOF ->
            let _v_0 = _menhir_action_133 () in
            _menhir_run_1181 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_1102 : type  ttv_stack. ((((((ttv_stack, _menhir_box_spec) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1102
        | Tokens.TYPE ->
            _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1102
        | Tokens.SETTER ->
            _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1102
        | Tokens.PRAGMA ->
            _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1102
        | Tokens.LET ->
            _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1102
        | Tokens.GETTER ->
            _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1102
        | Tokens.FUNC ->
            _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1102
        | Tokens.CONSTANT ->
            _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1102
        | Tokens.CONFIG ->
            _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1102
        | Tokens.EOF ->
            let _v_0 = _menhir_action_133 () in
            _menhir_run_1233 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_1084 : type  ttv_stack. ((((((ttv_stack, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN, _menhir_box_spec) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1084
        | Tokens.TYPE ->
            _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1084
        | Tokens.SETTER ->
            _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1084
        | Tokens.PRAGMA ->
            _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1084
        | Tokens.LET ->
            _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1084
        | Tokens.GETTER ->
            _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1084
        | Tokens.FUNC ->
            _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1084
        | Tokens.CONSTANT ->
            _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1084
        | Tokens.CONFIG ->
            _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1084
        | Tokens.EOF ->
            let _v_0 = _menhir_action_133 () in
            _menhir_run_1280 _menhir_stack _v_0
        | _ ->
            _eRR ()
    
    and _menhir_run_0737 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_WHILE, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO, ttv_result) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.WHILE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0737
        | Tokens.VAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0737
        | Tokens.UNREACHABLE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0737
        | Tokens.TRY ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0737
        | Tokens.THROW ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0737
        | Tokens.RETURN ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0737
        | Tokens.REPEAT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0737
        | Tokens.PRINT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0737
        | Tokens.PRAGMA ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0737
        | Tokens.PASS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0737
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0737
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0737
        | Tokens.LET ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0737
        | Tokens.IF ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0737
        | Tokens.IDENTIFIER _v_0 ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState0737
        | Tokens.FOR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0737
        | Tokens.DEBUG ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0737
        | Tokens.CONSTANT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0737
        | Tokens.CASE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0737
        | Tokens.ASSERT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0737
        | Tokens.ARROBASE_LOOPLIMIT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0737
        | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
            let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, expr, _, _) = _menhir_stack in
            let MenhirCell1_WHILE (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _endpos__2_, _4) = ((), _endpos, ()) in
            let _v = _menhir_action_236 _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ expr in
            _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0733 : type  ttv_stack ttv_result. (((((((ttv_stack, ttv_result) _menhir_cell1_WHILE, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_LOOPLIMIT, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO, ttv_result) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.WHILE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0733
        | Tokens.VAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0733
        | Tokens.UNREACHABLE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0733
        | Tokens.TRY ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0733
        | Tokens.THROW ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0733
        | Tokens.RETURN ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0733
        | Tokens.REPEAT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0733
        | Tokens.PRINT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0733
        | Tokens.PRAGMA ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0733
        | Tokens.PASS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0733
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0733
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0733
        | Tokens.LET ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0733
        | Tokens.IF ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0733
        | Tokens.IDENTIFIER _v_0 ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState0733
        | Tokens.FOR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0733
        | Tokens.DEBUG ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0733
        | Tokens.CONSTANT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0733
        | Tokens.CASE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0733
        | Tokens.ASSERT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0733
        | Tokens.ARROBASE_LOOPLIMIT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0733
        | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined2, _) = _menhir_stack in
            let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, _2_inlined1, _, _) = _menhir_stack in
            let MenhirCell1_LOOPLIMIT (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, expr, _, _) = _menhir_stack in
            let MenhirCell1_WHILE (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _endpos__2_, _4, _1_inlined1) = ((), _endpos, (), ()) in
            let _v = _menhir_action_237 _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ expr in
            _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0700 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_TRY, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_nonempty_list_catcher_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.WHILE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0700
        | Tokens.VAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0700
        | Tokens.UNREACHABLE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0700
        | Tokens.TRY ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0700
        | Tokens.THROW ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0700
        | Tokens.RETURN ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0700
        | Tokens.REPEAT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0700
        | Tokens.PRINT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0700
        | Tokens.PRAGMA ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0700
        | Tokens.PASS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0700
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0700
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0700
        | Tokens.LET ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0700
        | Tokens.IF ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0700
        | Tokens.IDENTIFIER _v_0 ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState0700
        | Tokens.FOR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0700
        | Tokens.DEBUG ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0700
        | Tokens.CONSTANT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0700
        | Tokens.CASE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0700
        | Tokens.ASSERT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0700
        | Tokens.ARROBASE_LOOPLIMIT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0700
        | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
            let MenhirCell1_nonempty_list_catcher_ (_menhir_stack, _, c) = _menhir_stack in
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
            let MenhirCell1_TRY (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _endpos__2_, _3) = ((), _endpos, ()) in
            let _v = _menhir_action_242 _1 _1_inlined1 _3 _endpos__2_ _startpos__1_ c in
            _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0698 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_TRY, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_nonempty_list_catcher_, ttv_result) _menhir_cell1_OTHERWISE, ttv_result) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.WHILE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
        | Tokens.VAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
        | Tokens.UNREACHABLE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
        | Tokens.TRY ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
        | Tokens.THROW ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
        | Tokens.RETURN ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
        | Tokens.REPEAT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
        | Tokens.PRINT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
        | Tokens.PRAGMA ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
        | Tokens.PASS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
        | Tokens.LET ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
        | Tokens.IF ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
        | Tokens.IDENTIFIER _v_0 ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState0698
        | Tokens.FOR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
        | Tokens.DEBUG ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
        | Tokens.CONSTANT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
        | Tokens.CASE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
        | Tokens.ASSERT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
        | Tokens.ARROBASE_LOOPLIMIT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0698
        | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined3, _) = _menhir_stack in
            let MenhirCell1_OTHERWISE (_menhir_stack, _, _, _) = _menhir_stack in
            let MenhirCell1_nonempty_list_catcher_ (_menhir_stack, _, c) = _menhir_stack in
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
            let MenhirCell1_TRY (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _endpos__2_, _2_inlined1, _1_inlined2, _3) = ((), _endpos, (), (), ()) in
            let _v = _menhir_action_243 _1 _1_inlined1 _1_inlined2 _1_inlined3 _2_inlined1 _3 _endpos__2_ _startpos__1_ c in
            _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0638 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_IF, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_s_else as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.WHILE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
        | Tokens.VAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
        | Tokens.UNREACHABLE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
        | Tokens.TRY ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
        | Tokens.THROW ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
        | Tokens.RETURN ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
        | Tokens.REPEAT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
        | Tokens.PRINT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
        | Tokens.PRAGMA ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
        | Tokens.PASS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
        | Tokens.LET ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
        | Tokens.IF ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
        | Tokens.IDENTIFIER _v_0 ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState0638
        | Tokens.FOR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
        | Tokens.DEBUG ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
        | Tokens.CONSTANT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
        | Tokens.CASE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
        | Tokens.ASSERT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
        | Tokens.ARROBASE_LOOPLIMIT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0638
        | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
            let MenhirCell1_s_else (_menhir_stack, _, s2, _) = _menhir_stack in
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
            let MenhirCell1_THEN (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, e, _, _) = _menhir_stack in
            let MenhirCell1_IF (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _endpos__2_, _3) = ((), _endpos, ()) in
            let _v = _menhir_action_234 _1 _1_inlined1 _3 _endpos__2_ _startpos__1_ e s2 in
            _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0559 : type  ttv_stack ttv_result. (((((((ttv_stack, ttv_result) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DOWNTO, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO, ttv_result) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.WHILE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0559
        | Tokens.VAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0559
        | Tokens.UNREACHABLE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0559
        | Tokens.TRY ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0559
        | Tokens.THROW ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0559
        | Tokens.RETURN ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0559
        | Tokens.REPEAT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0559
        | Tokens.PRINT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0559
        | Tokens.PRAGMA ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0559
        | Tokens.PASS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0559
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0559
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0559
        | Tokens.LET ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0559
        | Tokens.IF ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0559
        | Tokens.IDENTIFIER _v_0 ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState0559
        | Tokens.FOR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0559
        | Tokens.DEBUG ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0559
        | Tokens.CONSTANT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0559
        | Tokens.CASE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0559
        | Tokens.ASSERT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0559
        | Tokens.ARROBASE_LOOPLIMIT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0559
        | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined2, _) = _menhir_stack in
            let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, end_e, _, _) = _menhir_stack in
            let MenhirCell1_DOWNTO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, start_e, _, _) = _menhir_stack in
            let MenhirCell0_IDENTIFIER (_menhir_stack, index_name, _, _) = _menhir_stack in
            let MenhirCell1_FOR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _endpos__2_, _1_inlined1, _3, _8) = ((), _endpos, (), (), ()) in
            let _v = _menhir_action_240 _1 _1_inlined1 _1_inlined2 _3 _8 _endpos__2_ _startpos__1_ end_e index_name start_e in
            _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0555 : type  ttv_stack ttv_result. (((((((((ttv_stack, ttv_result) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DOWNTO, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_LOOPLIMIT, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO, ttv_result) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.WHILE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0555
        | Tokens.VAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0555
        | Tokens.UNREACHABLE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0555
        | Tokens.TRY ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0555
        | Tokens.THROW ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0555
        | Tokens.RETURN ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0555
        | Tokens.REPEAT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0555
        | Tokens.PRINT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0555
        | Tokens.PRAGMA ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0555
        | Tokens.PASS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0555
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0555
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0555
        | Tokens.LET ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0555
        | Tokens.IF ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0555
        | Tokens.IDENTIFIER _v_0 ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState0555
        | Tokens.FOR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0555
        | Tokens.DEBUG ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0555
        | Tokens.CONSTANT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0555
        | Tokens.CASE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0555
        | Tokens.ASSERT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0555
        | Tokens.ARROBASE_LOOPLIMIT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0555
        | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined3, _) = _menhir_stack in
            let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, _2_inlined1, _, _) = _menhir_stack in
            let MenhirCell1_LOOPLIMIT (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, end_e, _, _) = _menhir_stack in
            let MenhirCell1_DOWNTO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, start_e, _, _) = _menhir_stack in
            let MenhirCell0_IDENTIFIER (_menhir_stack, index_name, _, _) = _menhir_stack in
            let MenhirCell1_FOR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _endpos__2_, _1_inlined2, _1_inlined1, _3, _8) = ((), _endpos, (), (), (), ()) in
            let _v = _menhir_action_241 _1 _1_inlined1 _1_inlined2 _1_inlined3 _2_inlined1 _3 _8 _endpos__2_ _startpos__1_ end_e index_name start_e in
            _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0547 : type  ttv_stack ttv_result. (((((((ttv_stack, ttv_result) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_TO, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO, ttv_result) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.WHILE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
        | Tokens.VAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
        | Tokens.UNREACHABLE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
        | Tokens.TRY ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
        | Tokens.THROW ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
        | Tokens.RETURN ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
        | Tokens.REPEAT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
        | Tokens.PRINT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
        | Tokens.PRAGMA ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
        | Tokens.PASS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
        | Tokens.LET ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
        | Tokens.IF ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
        | Tokens.IDENTIFIER _v_0 ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState0547
        | Tokens.FOR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
        | Tokens.DEBUG ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
        | Tokens.CONSTANT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
        | Tokens.CASE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
        | Tokens.ASSERT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
        | Tokens.ARROBASE_LOOPLIMIT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0547
        | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined2, _) = _menhir_stack in
            let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, end_e, _, _) = _menhir_stack in
            let MenhirCell1_TO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, start_e, _, _) = _menhir_stack in
            let MenhirCell0_IDENTIFIER (_menhir_stack, index_name, _, _) = _menhir_stack in
            let MenhirCell1_FOR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _endpos__2_, _1_inlined1, _3, _8) = ((), _endpos, (), (), ()) in
            let _v = _menhir_action_238 _1 _1_inlined1 _1_inlined2 _3 _8 _endpos__2_ _startpos__1_ end_e index_name start_e in
            _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0543 : type  ttv_stack ttv_result. (((((((((ttv_stack, ttv_result) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_TO, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_LOOPLIMIT, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO, ttv_result) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.WHILE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0543
        | Tokens.VAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0543
        | Tokens.UNREACHABLE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0543
        | Tokens.TRY ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0543
        | Tokens.THROW ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0543
        | Tokens.RETURN ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0543
        | Tokens.REPEAT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0543
        | Tokens.PRINT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0543
        | Tokens.PRAGMA ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0543
        | Tokens.PASS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0543
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0543
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0543
        | Tokens.LET ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0543
        | Tokens.IF ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0543
        | Tokens.IDENTIFIER _v_0 ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState0543
        | Tokens.FOR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0543
        | Tokens.DEBUG ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0543
        | Tokens.CONSTANT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0543
        | Tokens.CASE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0543
        | Tokens.ASSERT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0543
        | Tokens.ARROBASE_LOOPLIMIT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0543
        | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined3, _) = _menhir_stack in
            let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, _2_inlined1, _, _) = _menhir_stack in
            let MenhirCell1_LOOPLIMIT (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, end_e, _, _) = _menhir_stack in
            let MenhirCell1_TO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, start_e, _, _) = _menhir_stack in
            let MenhirCell0_IDENTIFIER (_menhir_stack, index_name, _, _) = _menhir_stack in
            let MenhirCell1_FOR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _endpos__2_, _1_inlined2, _1_inlined1, _3, _8) = ((), _endpos, (), (), (), ()) in
            let _v = _menhir_action_239 _1 _1_inlined1 _1_inlined2 _1_inlined3 _2_inlined1 _3 _8 _endpos__2_ _startpos__1_ end_e index_name start_e in
            _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0521 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_CASE, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_OF, ttv_result) _menhir_cell1_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.WHILE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0521
        | Tokens.VAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0521
        | Tokens.UNREACHABLE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0521
        | Tokens.TRY ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0521
        | Tokens.THROW ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0521
        | Tokens.RETURN ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0521
        | Tokens.REPEAT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0521
        | Tokens.PRINT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0521
        | Tokens.PRAGMA ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0521
        | Tokens.PASS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0521
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0521
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0521
        | Tokens.LET ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0521
        | Tokens.IF ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0521
        | Tokens.IDENTIFIER _v_0 ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState0521
        | Tokens.FOR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0521
        | Tokens.DEBUG ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0521
        | Tokens.CONSTANT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0521
        | Tokens.CASE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0521
        | Tokens.ASSERT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0521
        | Tokens.ARROBASE_LOOPLIMIT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0521
        | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
            let MenhirCell1_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
            let MenhirCell1_OF (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, expr, _, _) = _menhir_stack in
            let MenhirCell1_CASE (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _endpos__2_, _3) = ((), _endpos, ()) in
            let _v = _menhir_action_235 _1 _1_inlined1 _3 _endpos__2_ _startpos__1_ expr in
            _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0495 : type  ttv_stack ttv_result. (((((((ttv_stack, ttv_result) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_RPAR _menhir_cell0_WHILE, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO, ttv_result) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.WHILE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.VAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.UNREACHABLE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.TRY ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.THROW ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.RETURN ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.REPEAT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.PRINT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.PRAGMA ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.PASS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.LET ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.IF ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.IDENTIFIER _v_0 ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState0495
        | Tokens.FOR ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.DEBUG ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.CONSTANT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.CASE ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.ASSERT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.ARROBASE_LOOPLIMIT ->
            let _menhir_stack = MenhirCell1_end_semicolon (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0495
        | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
            let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined2, _) = _menhir_stack in
            let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, cond, _, _) = _menhir_stack in
            let MenhirCell0_WHILE (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
            let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_ARROBASE_LOOPLIMIT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _endpos__2_, _3, _5) = ((), _endpos, (), ()) in
            let _v = _menhir_action_244 _1 _1_inlined2 _3 _5 _endpos__2_ _startpos__1_ cond x in
            _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_1298 : type  ttv_stack. (((((ttv_stack, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1298
        | _ ->
            _eRR ()
    
    and _menhir_run_1286 : type  ttv_stack. (((((ttv_stack, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1286
        | _ ->
            _eRR ()
    
    and _menhir_run_1241 : type  ttv_stack. (((((ttv_stack, _menhir_box_spec) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1241
        | _ ->
            _eRR ()
    
    and _menhir_run_1228 : type  ttv_stack. (((((ttv_stack, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1228
        | _ ->
            _eRR ()
    
    and _menhir_run_1222 : type  ttv_stack. (((((((ttv_stack, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1222
        | _ ->
            _eRR ()
    
    and _menhir_run_1212 : type  ttv_stack. ((((ttv_stack, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1212
        | _ ->
            _eRR ()
    
    and _menhir_run_1195 : type  ttv_stack. (((((ttv_stack, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1195
        | _ ->
            _eRR ()
    
    and _menhir_run_1189 : type  ttv_stack. (((((((ttv_stack, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR, _menhir_box_spec) _menhir_cell1_ty, _menhir_box_spec) _menhir_cell1_expr, _menhir_box_spec) _menhir_cell1_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1189
        | _ ->
            _eRR ()
    
    and _menhir_run_1109 : type  ttv_stack. ((((ttv_stack, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1109
        | _ ->
            _eRR ()
    
    and _menhir_run_1101 : type  ttv_stack. (((((ttv_stack, _menhir_box_spec) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1101
        | _ ->
            _eRR ()
    
    and _menhir_run_1083 : type  ttv_stack. (((((ttv_stack, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty _menhir_cell0_BEGIN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1083
        | _ ->
            _eRR ()
    
    and _menhir_run_1022 : type  ttv_stack. ((((ttv_stack, _menhir_box_opn) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_REPEAT as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNTIL ->
            let _menhir_s = MenhirState1023 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1018 : type  ttv_stack. ((((((ttv_stack, _menhir_box_opn) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_WHILE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1018
        | _ ->
            _eRR ()
    
    and _menhir_run_0966 : type  ttv_stack. ((((((ttv_stack, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DOWNTO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0966
        | _ ->
            _eRR ()
    
    and _menhir_run_0962 : type  ttv_stack. ((((((((ttv_stack, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DOWNTO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0962
        | _ ->
            _eRR ()
    
    and _menhir_run_0954 : type  ttv_stack. ((((((ttv_stack, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_TO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0954
        | _ ->
            _eRR ()
    
    and _menhir_run_0950 : type  ttv_stack. ((((((((ttv_stack, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_TO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0950
        | _ ->
            _eRR ()
    
    and _menhir_run_0865 : type  ttv_stack. ((((ttv_stack, _menhir_box_opn) _menhir_cell1_IF, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_THEN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.ELSIF ->
            _menhir_run_0630 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0865
        | Tokens.ELSE ->
            _menhir_run_0634 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0865
        | Tokens.END ->
            let _v_0 =
              let _endpos__0_ = _endpos in
              _menhir_action_413 _endpos__0_
            in
            _menhir_run_0866 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v_0 MenhirState0865 _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0630 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_ELSIF (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0630 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0634 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_ELSE (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0634 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.WHILE ->
            _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.VAR ->
            _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.UNREACHABLE ->
            _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.TRY ->
            _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.THROW ->
            _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.RETURN ->
            _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.REPEAT ->
            _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.PRINT ->
            _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.PRAGMA ->
            _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.PASS ->
            _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LET ->
            _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IF ->
            _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.FOR ->
            _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.DEBUG ->
            _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.CONSTANT ->
            _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.CASE ->
            _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.ASSERT ->
            _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.ARROBASE_LOOPLIMIT ->
            _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0866 : type  ttv_stack. (((((ttv_stack, _menhir_box_opn) _menhir_cell1_IF, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_THEN, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_s_else (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0866
        | _ ->
            _eRR ()
    
    and _menhir_run_0811 : type  ttv_stack. ((ttv_stack, _menhir_box_opn) _menhir_cell1_REPEAT as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNTIL ->
            let _menhir_s = MenhirState0812 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0793 : type  ttv_stack. (((((ttv_stack, _menhir_box_opn) _menhir_cell1_TRY, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_cell1_nonempty_list_catcher_, _menhir_box_opn) _menhir_cell1_OTHERWISE as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0793
        | _ ->
            _eRR ()
    
    and _menhir_run_0788 : type  ttv_stack. ((ttv_stack, _menhir_box_opn) _menhir_cell1_TRY as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.CATCH ->
            let _menhir_s = MenhirState0789 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHEN ->
                _menhir_run_0683 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0683 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_WHEN (_menhir_stack, _menhir_s, _startpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0683
        | Tokens.REAL ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0683
        | Tokens.LPAR ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0683
        | Tokens.INTEGER ->
            _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0683
        | Tokens.IDENTIFIER _v ->
            let _startpos_0 = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.COLON ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0683, _v, _startpos_0, _endpos) in
                let _menhir_s = MenhirState0685 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.ARROW ->
                let _v =
                  let (_endpos_name_, _startpos_name_, name) = (_endpos, _startpos_0, _v) in
                  _menhir_action_465 _endpos_name_ _startpos_name_ name
                in
                _menhir_run_0690 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v MenhirState0683 _tok
            | _ ->
                _eRR ())
        | Tokens.BOOLEAN ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0683
        | Tokens.BITS ->
            _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0683
        | Tokens.BIT ->
            _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0683
        | Tokens.ARRAY ->
            _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0683
        | _ ->
            _eRR ()
    
    and _menhir_run_0690 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_WHEN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.ARROW ->
            let _menhir_s = MenhirState0691 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0743 : type  ttv_stack. ((((ttv_stack, _menhir_box_opn) _menhir_cell1_WHILE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0743
        | _ ->
            _eRR ()
    
    and _menhir_run_0739 : type  ttv_stack. ((((((ttv_stack, _menhir_box_opn) _menhir_cell1_WHILE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DO as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0739
        | _ ->
            _eRR ()
    
    and _menhir_run_0738 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_WHILE, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_end_semicolon -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
        let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, expr, _, _) = _menhir_stack in
        let MenhirCell1_WHILE (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _4) = ((), _endpos, _v, ()) in
        let _v = _menhir_action_291 _1 _1_inlined1 _4 _endpos__2_ _startpos__1_ expr xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0736 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_WHILE, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0736
        | _ ->
            _eRR ()
    
    and _menhir_run_0734 : type  ttv_stack ttv_result. (((((((ttv_stack, ttv_result) _menhir_cell1_WHILE, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_LOOPLIMIT, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_end_semicolon -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined2, _) = _menhir_stack in
        let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, _2_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_LOOPLIMIT (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, expr, _, _) = _menhir_stack in
        let MenhirCell1_WHILE (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _4, _1_inlined1) = ((), _endpos, _v, (), ()) in
        let _v = _menhir_action_292 _1 _1_inlined1 _1_inlined2 _2_inlined1 _4 _endpos__2_ _startpos__1_ expr xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0732 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_WHILE, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_LOOPLIMIT, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0732
        | _ ->
            _eRR ()
    
    and _menhir_run_0731 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_VAR, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, _2_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_decl_item (_menhir_stack, _, ldi) = _menhir_stack in
        let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _1_inlined1) = ((), _endpos, _v, ()) in
        let _v = _menhir_action_326 _1 _1_inlined1 _2_inlined1 _endpos__2_ _startpos__1_ ldi xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0728 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_VAR, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_COMMA, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, ttv_result) _menhir_cell1_loption_plist_expr__ _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_loption_plist_expr__ (_menhir_stack, _, args, _endpos_args_) = _menhir_stack in
        let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell1_COMMA (_menhir_stack, _) = _menhir_stack in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
        let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
        let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _2_inlined1, _endpos_xs_, xs, _3, _3_inlined1, _5) = ((), (), _endpos, _v, (), (), ()) in
        let _v = _menhir_action_331 _1 _1_inlined1 _2_inlined1 _3 _3_inlined1 _5 _endpos__2_ _endpos_args_ _startpos__1_ _startpos_name_ args lhs name xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0722 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_VAR, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_RBRACE _menhir_cell0_LPAR, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _endpos__3_inlined2_) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined2) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_RBRACE (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
        let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
        let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _2_inlined1, _endpos_xs_, xs, _3, _3_inlined1) = ((), (), _endpos, _v, (), ()) in
        let _v = _menhir_action_330 _1 _1_inlined2 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0718 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_VAR, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_RBRACE _menhir_cell0_LPAR, ttv_result) _menhir_cell1_RPAR _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _endpos__3_inlined2_) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_RBRACE (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
        let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
        let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _2_inlined1, _endpos_xs_, xs, _3, _3_inlined1) = ((), (), _endpos, _v, (), ()) in
        let _v = _menhir_action_329 _1 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0710 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_VAR, ttv_result) _menhir_cell1_decl_item _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_decl_item (_menhir_stack, _, ldi) = _menhir_stack in
        let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs) = ((), _endpos, _v) in
        let _v = _menhir_action_325 _1 _endpos__2_ _startpos__1_ ldi xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0707 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_VAR, ttv_result) _menhir_cell1_IDENTIFIER, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_, ttv_result) _menhir_cell1_ty _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined2, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _2_inlined1, _endpos_xs_, xs, _1_inlined2) = ((), (), _endpos, _v, ()) in
        let _v = _menhir_action_327 _1 _1_inlined1 _1_inlined2 _2_inlined1 _2_inlined2 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0703 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_VAR, ttv_result) _menhir_cell1_IDENTIFIER, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_, ttv_result) _menhir_cell1_ty _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined2, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _2_inlined1, _endpos_xs_, xs, _1_inlined2) = ((), (), _endpos, _v, ()) in
        let _v = _menhir_action_328 _1 _1_inlined1 _1_inlined2 _2_inlined1 _2_inlined2 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0702 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_UNREACHABLE _menhir_cell0_LPAR _menhir_cell0_RPAR _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_UNREACHABLE (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _2_inlined1, _endpos_xs_, xs, _3) = ((), (), _endpos, _v, ()) in
        let _v = _menhir_action_336 _1 _2_inlined1 _3 _endpos__2_ _startpos__1_ xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0701 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_TRY, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_nonempty_list_catcher_, ttv_result) _menhir_cell1_end_semicolon -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_nonempty_list_catcher_ (_menhir_stack, _, c) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
        let MenhirCell1_TRY (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _3) = ((), _endpos, _v, ()) in
        let _v = _menhir_action_297 _1 _1_inlined1 _3 _endpos__2_ _startpos__1_ c xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0699 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_TRY, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_nonempty_list_catcher_, ttv_result) _menhir_cell1_OTHERWISE, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_end_semicolon -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined3, _) = _menhir_stack in
        let MenhirCell1_OTHERWISE (_menhir_stack, _, _, _) = _menhir_stack in
        let MenhirCell1_nonempty_list_catcher_ (_menhir_stack, _, c) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
        let MenhirCell1_TRY (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _2_inlined1, _endpos_xs_, xs, _1_inlined2, _3) = ((), (), _endpos, _v, (), ()) in
        let _v = _menhir_action_298 _1 _1_inlined1 _1_inlined2 _1_inlined3 _2_inlined1 _3 _endpos__2_ _startpos__1_ c xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0697 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_TRY, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_nonempty_list_catcher_, ttv_result) _menhir_cell1_OTHERWISE as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0697
        | _ ->
            _eRR ()
    
    and _menhir_run_0692 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_WHEN, ttv_result) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.WHEN ->
            let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0683 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0692
        | Tokens.END | Tokens.OTHERWISE ->
            let MenhirCell1_ty (_menhir_stack, _, ty, _) = _menhir_stack in
            let MenhirCell1_WHEN (_menhir_stack, _menhir_s, _) = _menhir_stack in
            let (_1, _4, _1_inlined1) = ((), (), _v) in
            let _v = _menhir_action_230 _1 _1_inlined1 _4 ty in
            _menhir_goto_nonempty_list_catcher_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_goto_nonempty_list_catcher_ : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        match _menhir_s with
        | MenhirState0789 ->
            _menhir_run_0790 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0682 ->
            _menhir_run_0694 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0692 ->
            _menhir_run_0693 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0688 ->
            _menhir_run_0689 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0790 : type  ttv_stack. (((ttv_stack, _menhir_box_opn) _menhir_cell1_TRY, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_catcher_ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.OTHERWISE ->
            let _menhir_s = MenhirState0790 in
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_OTHERWISE (_menhir_stack, _menhir_s, _startpos, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.ARROW ->
                let _menhir_s = MenhirState0792 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.WHILE ->
                    _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.VAR ->
                    _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.UNREACHABLE ->
                    _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.TRY ->
                    _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.THROW ->
                    _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.RETURN ->
                    _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REPEAT ->
                    _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.PRINT ->
                    _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.PRAGMA ->
                    _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.PASS ->
                    _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LET ->
                    _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IF ->
                    _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.FOR ->
                    _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.DEBUG ->
                    _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.CONSTANT ->
                    _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.CASE ->
                    _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ASSERT ->
                    _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARROBASE_LOOPLIMIT ->
                    _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.END ->
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0790
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0694 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_TRY, ttv_result) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_catcher_ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.OTHERWISE ->
            let _menhir_s = MenhirState0694 in
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_OTHERWISE (_menhir_stack, _menhir_s, _startpos, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.ARROW ->
                let _menhir_s = MenhirState0696 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.WHILE ->
                    _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.VAR ->
                    _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.UNREACHABLE ->
                    _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.TRY ->
                    _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.THROW ->
                    _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.RETURN ->
                    _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REPEAT ->
                    _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.PRINT ->
                    _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.PRAGMA ->
                    _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.PASS ->
                    _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LET ->
                    _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IF ->
                    _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.FOR ->
                    _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.DEBUG ->
                    _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.CONSTANT ->
                    _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.CASE ->
                    _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ASSERT ->
                    _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARROBASE_LOOPLIMIT ->
                    _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.END ->
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0694
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0693 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_WHEN, ttv_result) _menhir_cell1_ty, ttv_result) _menhir_cell1_nonempty_list_stmt_ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, ty, _) = _menhir_stack in
        let MenhirCell1_WHEN (_menhir_stack, _menhir_s, _) = _menhir_stack in
        let (_1, xs, _4) = ((), _v, ()) in
        let _v = _menhir_action_232 _1 _1_inlined1 _4 ty xs in
        _menhir_goto_nonempty_list_catcher_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
    
    and _menhir_run_0689 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_WHEN, ttv_result) _menhir_cell1_IDENTIFIER, ttv_result) _menhir_cell1_ty, ttv_result) _menhir_cell1_nonempty_list_stmt_ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
        let MenhirCell1_ty (_menhir_stack, _, ty, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_WHEN (_menhir_stack, _menhir_s, _) = _menhir_stack in
        let (_1, xs, _4) = ((), _v, ()) in
        let _v = _menhir_action_233 _1 _1_inlined1 _4 ty x xs in
        _menhir_goto_nonempty_list_catcher_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
    
    and _menhir_run_0688 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_WHEN, ttv_result) _menhir_cell1_IDENTIFIER, ttv_result) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.WHEN ->
            let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0683 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0688
        | Tokens.END | Tokens.OTHERWISE ->
            let MenhirCell1_ty (_menhir_stack, _, ty, _) = _menhir_stack in
            let MenhirCell1_IDENTIFIER (_menhir_stack, _, x, _, _) = _menhir_stack in
            let MenhirCell1_WHEN (_menhir_stack, _menhir_s, _) = _menhir_stack in
            let (_1, _4, _1_inlined1) = ((), (), _v) in
            let _v = _menhir_action_231 _1 _1_inlined1 _4 ty x in
            _menhir_goto_nonempty_list_catcher_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0681 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_TRY as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.CATCH ->
            let _menhir_s = MenhirState0682 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHEN ->
                _menhir_run_0683 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0680 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_THROW, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, e, _, _) = _menhir_stack in
        let MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs) = ((), _endpos, _v) in
        let _v = _menhir_action_340 _1 _endpos__2_ _startpos__1_ e xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0677 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_THROW, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs) = ((), _endpos, _v) in
        let _v = _menhir_action_341 _1 _endpos__2_ _startpos__1_ xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0676 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_RETURN, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs) = ((), _endpos, _v) in
        let _v = _menhir_action_302 _1 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0673 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_RETURN, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs) = ((), _endpos, _v) in
        let _v = _menhir_action_301 _1 _endpos__2_ _startpos__1_ xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0672 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_REPEAT, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_LOOPLIMIT, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, _2_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_LOOPLIMIT (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, expr, _, _) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
        let MenhirCell1_REPEAT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _1_inlined2, _3) = ((), _endpos, _v, (), ()) in
        let _v = _menhir_action_338 _1 _1_inlined1 _1_inlined2 _2_inlined1 _3 _endpos__2_ _startpos__1_ expr xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0668 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_REPEAT, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, expr, _, _) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
        let MenhirCell1_REPEAT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _3) = ((), _endpos, _v, ()) in
        let _v = _menhir_action_337 _1 _1_inlined1 _3 _endpos__2_ _startpos__1_ expr xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0664 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_REPEAT as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNTIL ->
            let _menhir_s = MenhirState0665 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0663 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_PRINT _menhir_cell0_LPAR, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined2) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs) = ((), _endpos, _v) in
        let _v = _menhir_action_333 _1 _1_inlined2 _endpos__2_ _startpos__1_ xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0659 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_PRINT _menhir_cell0_LPAR, ttv_result) _menhir_cell1_RPAR _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs) = ((), _endpos, _v) in
        let _v = _menhir_action_332 _1 _endpos__2_ _startpos__1_ xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0658 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_PRAGMA _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
        let MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs) = ((), _endpos, _v) in
        let _v = _menhir_action_343 _1 _1_inlined1 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0655 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_PRAGMA _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
        let MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs) = ((), _endpos, _v) in
        let _v = _menhir_action_342 _1 _endpos__2_ _startpos__1_ x xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0654 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_PASS _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_PASS (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs) = ((), _endpos, _v) in
        let _v = _menhir_action_300 _1 _endpos__2_ _startpos__1_ xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0653 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_LET, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_decl_item (_menhir_stack, _, decl_item) = _menhir_stack in
        let MenhirCell1_LET (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _3) = ((), _endpos, _v, ()) in
        let _v = _menhir_action_307 _1 _3 _endpos__2_ _startpos__1_ decl_item x xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0650 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_LET, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_COMMA, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, ttv_result) _menhir_cell1_loption_plist_expr__ _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_loption_plist_expr__ (_menhir_stack, _, args, _endpos_args_) = _menhir_stack in
        let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell1_COMMA (_menhir_stack, _) = _menhir_stack in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
        let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
        let MenhirCell1_LET (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _2_inlined1, _endpos_xs_, xs, _3, _3_inlined1, _5) = ((), (), _endpos, _v, (), (), ()) in
        let _v = _menhir_action_321 _1 _1_inlined1 _2_inlined1 _3 _3_inlined1 _5 _endpos__2_ _endpos_args_ _startpos__1_ _startpos_name_ args lhs name xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0644 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_LET, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_RBRACE _menhir_cell0_LPAR, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _endpos__3_inlined2_) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined2) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_RBRACE (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
        let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
        let MenhirCell1_LET (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _2_inlined1, _endpos_xs_, xs, _3, _3_inlined1) = ((), (), _endpos, _v, (), ()) in
        let _v = _menhir_action_320 _1 _1_inlined2 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0640 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_LET, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_RBRACE _menhir_cell0_LPAR, ttv_result) _menhir_cell1_RPAR _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _endpos__3_inlined2_) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_RBRACE (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
        let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
        let MenhirCell1_LET (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _2_inlined1, _endpos_xs_, xs, _3, _3_inlined1) = ((), (), _endpos, _v, (), ()) in
        let _v = _menhir_action_319 _1 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0639 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_IF, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_s_else, ttv_result) _menhir_cell1_end_semicolon -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_s_else (_menhir_stack, _, s2, _) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
        let MenhirCell1_THEN (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, e, _, _) = _menhir_stack in
        let MenhirCell1_IF (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _3) = ((), _endpos, _v, ()) in
        let _v = _menhir_action_289 _1 _1_inlined1 _3 _endpos__2_ _startpos__1_ e s2 xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0635 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_ELSE -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_ELSE (_menhir_stack, _menhir_s) = _menhir_stack in
        let (_1, _endpos__1_inlined1_, _1_inlined1) = ((), _endpos, _v) in
        let _v = _menhir_action_414 _1 _1_inlined1 in
        _menhir_goto_s_else _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__1_inlined1_ _v _menhir_s _tok
    
    and _menhir_goto_s_else : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match _menhir_s with
        | MenhirState0865 ->
            _menhir_run_0866 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0629 ->
            _menhir_run_0637 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0633 ->
            _menhir_run_0636 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0637 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_IF, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_s_else (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0637
        | _ ->
            _eRR ()
    
    and _menhir_run_0636 : type  ttv_stack ttv_result. (((((((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_ELSIF, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_nonempty_list_stmt_ -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
        let MenhirCell1_THEN (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, e, _, _) = _menhir_stack in
        let MenhirCell1_ELSIF (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _3, _endpos_s2_, s2) = ((), (), _endpos, _v) in
        let _v = _menhir_action_412 _1 _1_inlined1 _3 _endpos_s2_ _startpos__1_ e s2 in
        _menhir_goto_s_else _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_s2_ _v _menhir_s _tok
    
    and _menhir_run_0633 : type  ttv_stack ttv_result. (((((((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_ELSIF, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.ELSIF ->
            _menhir_run_0630 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0633
        | Tokens.ELSE ->
            _menhir_run_0634 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0633
        | Tokens.END ->
            let _v_0 =
              let _endpos__0_ = _endpos in
              _menhir_action_413 _endpos__0_
            in
            _menhir_run_0636 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v_0 _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0629 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_IF, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.ELSIF ->
            _menhir_run_0630 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0629
        | Tokens.ELSE ->
            _menhir_run_0634 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0629
        | Tokens.END ->
            let _v_0 =
              let _endpos__0_ = _endpos in
              _menhir_action_413 _endpos__0_
            in
            _menhir_run_0637 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v_0 MenhirState0629 _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0628 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, ttv_result) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, rhs, _, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, fld, _, _) = _menhir_stack in
        let MenhirCell1_loption_plist_expr__ (_menhir_stack, _, args, _endpos_args_) = _menhir_stack in
        let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
        let (_2_inlined1, _endpos_xs_, xs, _4) = ((), _endpos, _v, ()) in
        let _v = _menhir_action_315 _1_inlined1 _2_inlined1 _4 _endpos__2_ _endpos_args_ _startpos_name_ args fld name rhs xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0623 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, ttv_result) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, rhs, _, _) = _menhir_stack in
        let MenhirCell0_RBRACKET (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _, _1_inlined3) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
        let MenhirCell1_loption_plist_expr__ (_menhir_stack, _, args, _endpos_args_) = _menhir_stack in
        let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
        let (_2_inlined1, _endpos_xs_, xs, _4, _2_inlined2) = ((), _endpos, _v, (), ()) in
        let _v = _menhir_action_318 _1_inlined1 _1_inlined3 _2_inlined1 _2_inlined2 _4 _endpos__2_ _endpos_args_ _startpos_name_ args name rhs x xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0613 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, ttv_result) _menhir_cell1_loption_plist_expr__, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, rhs, _, _) = _menhir_stack in
        let MenhirCell1_loption_plist_expr__ (_menhir_stack, _, args, _endpos_args_) = _menhir_stack in
        let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
        let (_2_inlined1, _endpos_xs_, xs) = ((), _endpos, _v) in
        let _v = _menhir_action_312 _1_inlined1 _2_inlined1 _endpos__2_ _endpos_args_ _startpos_name_ args name rhs xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0609 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, ttv_result) _menhir_cell1_loption_plist_expr__ _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_loption_plist_expr__ (_menhir_stack, _, args, _) = _menhir_stack in
        let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
        let (_endpos_xs_, xs) = (_endpos, _v) in
        let _v = _menhir_action_305 _1_inlined1 _endpos__2_ _startpos_name_ args name xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0603 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, rhs, _, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, fld, _, _) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _endpos__3_) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
        let (_2_inlined1, _endpos_xs_, xs, _4) = ((), _endpos, _v, ()) in
        let _v = _menhir_action_314 _1_inlined1 _2_inlined1 _4 _endpos__2_ _endpos__3_ _startpos_name_ fld name rhs xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0598 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, rhs, _, _) = _menhir_stack in
        let MenhirCell0_RBRACKET (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _, _1_inlined3) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _endpos__3_) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
        let (_2_inlined1, _endpos_xs_, xs, _4, _2_inlined2) = ((), _endpos, _v, (), ()) in
        let _v = _menhir_action_317 _1_inlined1 _1_inlined3 _2_inlined1 _2_inlined2 _4 _endpos__2_ _endpos__3_ _startpos_name_ name rhs x xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0588 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, rhs, _, _) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _endpos__3_) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
        let (_2_inlined1, _endpos_xs_, xs) = ((), _endpos, _v) in
        let _v = _menhir_action_311 _1_inlined1 _2_inlined1 _endpos__2_ _endpos__3_ _startpos_name_ name rhs xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0584 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
        let (_endpos_xs_, xs) = (_endpos, _v) in
        let _v = _menhir_action_304 _1_inlined1 _endpos__2_ _startpos_name_ name xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0580 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, ttv_result) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, rhs, _, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, fld, _, _) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _endpos__3_) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
        let (_2_inlined1, _endpos_xs_, xs, _4) = ((), _endpos, _v, ()) in
        let _v = _menhir_action_313 _2_inlined1 _4 _endpos__2_ _endpos__3_ _startpos_name_ fld name rhs xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0575 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, ttv_result) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, rhs, _, _) = _menhir_stack in
        let MenhirCell0_RBRACKET (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _, _1_inlined2) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _endpos__3_) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
        let (_2_inlined1, _endpos_xs_, xs, _4, _2_inlined2) = ((), _endpos, _v, (), ()) in
        let _v = _menhir_action_316 _1_inlined2 _2_inlined1 _2_inlined2 _4 _endpos__2_ _endpos__3_ _startpos_name_ name rhs x xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0565 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, ttv_result) _menhir_cell1_RPAR, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, rhs, _, _) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _endpos__3_) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
        let (_2_inlined1, _endpos_xs_, xs) = ((), _endpos, _v) in
        let _v = _menhir_action_310 _2_inlined1 _endpos__2_ _endpos__3_ _startpos_name_ name rhs xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0561 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, ttv_result) _menhir_cell1_RPAR _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
        let (_endpos_xs_, xs) = (_endpos, _v) in
        let _v = _menhir_action_303 _endpos__2_ _startpos_name_ name xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0560 : type  ttv_stack ttv_result. (((((((ttv_stack, ttv_result) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DOWNTO, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_end_semicolon -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined2, _) = _menhir_stack in
        let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, end_e, _, _) = _menhir_stack in
        let MenhirCell1_DOWNTO (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, start_e, _, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, index_name, _, _) = _menhir_stack in
        let MenhirCell1_FOR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _1_inlined1, _3, _8) = ((), _endpos, _v, (), (), ()) in
        let _v = _menhir_action_295 _1 _1_inlined1 _1_inlined2 _3 _8 _endpos__2_ _startpos__1_ end_e index_name start_e xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0558 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DOWNTO, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0558
        | _ ->
            _eRR ()
    
    and _menhir_run_0556 : type  ttv_stack ttv_result. (((((((((ttv_stack, ttv_result) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DOWNTO, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_LOOPLIMIT, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_end_semicolon -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined3, _) = _menhir_stack in
        let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, _2_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_LOOPLIMIT (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, end_e, _, _) = _menhir_stack in
        let MenhirCell1_DOWNTO (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, start_e, _, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, index_name, _, _) = _menhir_stack in
        let MenhirCell1_FOR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _1_inlined2, _1_inlined1, _3, _8) = ((), _endpos, _v, (), (), (), ()) in
        let _v = _menhir_action_296 _1 _1_inlined1 _1_inlined2 _1_inlined3 _2_inlined1 _3 _8 _endpos__2_ _startpos__1_ end_e index_name start_e xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0554 : type  ttv_stack ttv_result. ((((((((ttv_stack, ttv_result) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DOWNTO, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_LOOPLIMIT, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0554
        | _ ->
            _eRR ()
    
    and _menhir_run_0548 : type  ttv_stack ttv_result. (((((((ttv_stack, ttv_result) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_TO, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_end_semicolon -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined2, _) = _menhir_stack in
        let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, end_e, _, _) = _menhir_stack in
        let MenhirCell1_TO (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, start_e, _, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, index_name, _, _) = _menhir_stack in
        let MenhirCell1_FOR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _1_inlined1, _3, _8) = ((), _endpos, _v, (), (), ()) in
        let _v = _menhir_action_293 _1 _1_inlined1 _1_inlined2 _3 _8 _endpos__2_ _startpos__1_ end_e index_name start_e xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0546 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_TO, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0546
        | _ ->
            _eRR ()
    
    and _menhir_run_0544 : type  ttv_stack ttv_result. (((((((((ttv_stack, ttv_result) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_TO, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_LOOPLIMIT, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_end_semicolon -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined3, _) = _menhir_stack in
        let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, _2_inlined1, _, _) = _menhir_stack in
        let MenhirCell1_LOOPLIMIT (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, end_e, _, _) = _menhir_stack in
        let MenhirCell1_TO (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, start_e, _, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, index_name, _, _) = _menhir_stack in
        let MenhirCell1_FOR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _1_inlined2, _1_inlined1, _3, _8) = ((), _endpos, _v, (), (), (), ()) in
        let _v = _menhir_action_294 _1 _1_inlined1 _1_inlined2 _1_inlined3 _2_inlined1 _3 _8 _endpos__2_ _startpos__1_ end_e index_name start_e xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0542 : type  ttv_stack ttv_result. ((((((((ttv_stack, ttv_result) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_TO, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_LOOPLIMIT, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0542
        | _ ->
            _eRR ()
    
    and _menhir_run_0541 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_DEBUG _menhir_cell0_LPAR, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined2) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs) = ((), _endpos, _v) in
        let _v = _menhir_action_335 _1 _1_inlined2 _endpos__2_ _startpos__1_ xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0537 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_DEBUG _menhir_cell0_LPAR, ttv_result) _menhir_cell1_RPAR _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs) = ((), _endpos, _v) in
        let _v = _menhir_action_334 _1 _endpos__2_ _startpos__1_ xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0536 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_CONSTANT, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_decl_item (_menhir_stack, _, decl_item) = _menhir_stack in
        let MenhirCell1_CONSTANT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _3) = ((), _endpos, _v, ()) in
        let _v = _menhir_action_308 _1 _3 _endpos__2_ _startpos__1_ decl_item x xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0533 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_CONSTANT, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_COMMA, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, ttv_result) _menhir_cell1_loption_plist_expr__ _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_loption_plist_expr__ (_menhir_stack, _, args, _endpos_args_) = _menhir_stack in
        let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell1_COMMA (_menhir_stack, _) = _menhir_stack in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
        let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
        let MenhirCell1_CONSTANT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _2_inlined1, _endpos_xs_, xs, _3, _3_inlined1, _5) = ((), (), _endpos, _v, (), (), ()) in
        let _v = _menhir_action_324 _1 _1_inlined1 _2_inlined1 _3 _3_inlined1 _5 _endpos__2_ _endpos_args_ _startpos__1_ _startpos_name_ args lhs name xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0527 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_CONSTANT, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_RBRACE _menhir_cell0_LPAR, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell0_RPAR (_menhir_stack, _endpos__3_inlined2_) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined2) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_RBRACE (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
        let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
        let MenhirCell1_CONSTANT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _2_inlined1, _endpos_xs_, xs, _3, _3_inlined1) = ((), (), _endpos, _v, (), ()) in
        let _v = _menhir_action_323 _1 _1_inlined2 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0523 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_CONSTANT, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_RBRACE _menhir_cell0_LPAR, ttv_result) _menhir_cell1_RPAR _menhir_cell0_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_SEMI_COLON (_menhir_stack, _endpos__2_) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _endpos__3_inlined2_) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_RBRACE (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
        let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
        let MenhirCell1_CONSTANT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _2_inlined1, _endpos_xs_, xs, _3, _3_inlined1) = ((), (), _endpos, _v, (), ()) in
        let _v = _menhir_action_322 _1 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0522 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_CASE, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_OF, ttv_result) _menhir_cell1_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__, ttv_result) _menhir_cell1_end_semicolon -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell1_OF (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, expr, _, _) = _menhir_stack in
        let MenhirCell1_CASE (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _3) = ((), _endpos, _v, ()) in
        let _v = _menhir_action_290 _1 _1_inlined1 _3 _endpos__2_ _startpos__1_ expr xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0518 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_WHEN, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_pattern_, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_OTHERWISE -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_OTHERWISE (_menhir_stack, _, _startpos_desc_, _endpos_desc_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined2, _endpos__1_inlined2_) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_pattern_ (_menhir_stack, _, _1_inlined1, _startpos__1_inlined1_, _endpos__1_inlined1_) = _menhir_stack in
        let MenhirCell1_WHEN (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _2, _4, _endpos__1_inlined3_, _1_inlined3, desc) = ((), (), (), _endpos, _v, ()) in
        let _v = _menhir_action_205 _1 _1_inlined1 _1_inlined2 _1_inlined3 _2 _4 _endpos__1_inlined1_ _endpos__1_inlined2_ _endpos__1_inlined3_ _endpos_desc_ _startpos__1_ _startpos__1_inlined1_ _startpos_desc_ desc in
        _menhir_goto_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
    
    and _menhir_goto_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__ : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        match _menhir_s with
        | MenhirState1003 ->
            _menhir_run_1004 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0475 ->
            _menhir_run_0520 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0515 ->
            _menhir_run_0519 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0509 ->
            _menhir_run_0513 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_1004 : type  ttv_stack. ((((ttv_stack, _menhir_box_opn) _menhir_cell1_CASE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_OF as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1004
        | _ ->
            _eRR ()
    
    and _menhir_run_0520 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_CASE, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_OF as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0520
        | _ ->
            _eRR ()
    
    and _menhir_run_0519 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_WHEN, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_pattern_, ttv_result) _menhir_cell1_nonempty_list_stmt_ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined2, _endpos__1_inlined2_) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_pattern_ (_menhir_stack, _, _1_inlined1, _startpos__1_inlined1_, _endpos__1_inlined1_) = _menhir_stack in
        let MenhirCell1_WHEN (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _4, l) = ((), (), _v) in
        let _v = _menhir_action_207 _1 _1_inlined1 _1_inlined2 _4 _endpos__1_inlined1_ _endpos__1_inlined2_ _startpos__1_ _startpos__1_inlined1_ l in
        _menhir_goto_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
    
    and _menhir_run_0513 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_WHEN, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_pattern_, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_ARROW, ttv_result) _menhir_cell1_nonempty_list_stmt_ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined3, _endpos__1_inlined3_) = _menhir_stack in
        let MenhirCell1_ARROW (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, _2, _, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_pattern_ (_menhir_stack, _, _1_inlined1, _startpos__1_inlined1_, _endpos__1_inlined1_) = _menhir_stack in
        let MenhirCell1_WHEN (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _4, _1_inlined2, l) = ((), (), (), _v) in
        let _v = _menhir_action_208 _1 _1_inlined1 _1_inlined2 _1_inlined3 _2 _4 _endpos__1_inlined1_ _endpos__1_inlined3_ _startpos__1_ _startpos__1_inlined1_ l in
        _menhir_goto_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
    
    and _menhir_run_0515 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_WHEN, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_pattern_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.WHEN ->
            let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0515
        | Tokens.OTHERWISE ->
            let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
            let _menhir_s = MenhirState0515 in
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_OTHERWISE (_menhir_stack, _menhir_s, _startpos, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.ARROW ->
                let _menhir_s = MenhirState0517 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.WHILE ->
                    _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.VAR ->
                    _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.UNREACHABLE ->
                    _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.TRY ->
                    _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.THROW ->
                    _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.RETURN ->
                    _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REPEAT ->
                    _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.PRINT ->
                    _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.PRAGMA ->
                    _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.PASS ->
                    _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LET ->
                    _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IF ->
                    _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.FOR ->
                    _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.DEBUG ->
                    _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.CONSTANT ->
                    _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.CASE ->
                    _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ASSERT ->
                    _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARROBASE_LOOPLIMIT ->
                    _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.END ->
            let MenhirCell1_separated_nonempty_list_COMMA_pattern_ (_menhir_stack, _, _1_inlined1, _startpos__1_inlined1_, _endpos__1_inlined1_) = _menhir_stack in
            let MenhirCell1_WHEN (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _4, _1_inlined2, _endpos__1_inlined2_) = ((), (), _v, _endpos) in
            let _v = _menhir_action_203 _1 _1_inlined1 _1_inlined2 _4 _endpos__1_inlined1_ _endpos__1_inlined2_ _startpos__1_ _startpos__1_inlined1_ in
            _menhir_goto_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0476 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_WHEN (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0476 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0039 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0040 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0041 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0051 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MASK_LIT _v ->
            _menhir_run_0053 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0054 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LEQ ->
            _menhir_run_0055 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LBRACE ->
            _menhir_run_0169 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0170 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0171 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0183 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.GEQ ->
            _menhir_run_0215 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0217 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0218 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0220 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0221 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0512 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_WHEN, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_pattern_, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_ARROW, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_OTHERWISE -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_OTHERWISE (_menhir_stack, _, _startpos_desc_, _endpos_desc_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined3, _endpos__1_inlined3_) = _menhir_stack in
        let MenhirCell1_ARROW (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, _2, _, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_pattern_ (_menhir_stack, _, _1_inlined1, _startpos__1_inlined1_, _endpos__1_inlined1_) = _menhir_stack in
        let MenhirCell1_WHEN (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _2_inlined1, _4, _1_inlined2, _1_inlined4, _endpos__1_inlined4_, desc) = ((), (), (), (), _v, _endpos, ()) in
        let _v = _menhir_action_206 _1 _1_inlined1 _1_inlined2 _1_inlined3 _1_inlined4 _2 _2_inlined1 _4 _endpos__1_inlined1_ _endpos__1_inlined3_ _endpos__1_inlined4_ _endpos_desc_ _startpos__1_ _startpos__1_inlined1_ _startpos_desc_ desc in
        _menhir_goto_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
    
    and _menhir_run_0509 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_WHEN, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_pattern_, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_ARROW as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.WHEN ->
            let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
            _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0509
        | Tokens.OTHERWISE ->
            let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
            let _menhir_s = MenhirState0509 in
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_OTHERWISE (_menhir_stack, _menhir_s, _startpos, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.ARROW ->
                let _menhir_s = MenhirState0511 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.WHILE ->
                    _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.VAR ->
                    _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.UNREACHABLE ->
                    _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.TRY ->
                    _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.THROW ->
                    _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.RETURN ->
                    _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REPEAT ->
                    _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.PRINT ->
                    _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.PRAGMA ->
                    _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.PASS ->
                    _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LET ->
                    _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IF ->
                    _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.FOR ->
                    _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.DEBUG ->
                    _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.CONSTANT ->
                    _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.CASE ->
                    _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ASSERT ->
                    _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARROBASE_LOOPLIMIT ->
                    _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.END ->
            let MenhirCell1_ARROW (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, _2, _, _) = _menhir_stack in
            let MenhirCell1_separated_nonempty_list_COMMA_pattern_ (_menhir_stack, _, _1_inlined1, _startpos__1_inlined1_, _endpos__1_inlined1_) = _menhir_stack in
            let MenhirCell1_WHEN (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _4, _1_inlined2, _endpos__1_inlined3_, _1_inlined3) = ((), (), (), _endpos, _v) in
            let _v = _menhir_action_204 _1 _1_inlined1 _1_inlined2 _1_inlined3 _2 _4 _endpos__1_inlined1_ _endpos__1_inlined3_ _startpos__1_ _startpos__1_inlined1_ in
            _menhir_goto_nlist_opt_terminated_annotated_case_alt__annotated_case_otherwise__ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0508 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_ASSERT, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, e, _, _) = _menhir_stack in
        let MenhirCell1_ASSERT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs) = ((), _endpos, _v) in
        let _v = _menhir_action_306 _1 _endpos__2_ _startpos__1_ e xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0507 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_RPAR _menhir_cell0_REPEAT, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, cond, _, _) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined2, _) = _menhir_stack in
        let MenhirCell0_REPEAT (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_ARROBASE_LOOPLIMIT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _3, _5) = ((), _endpos, _v, (), ()) in
        let _v = _menhir_action_339 _1 _1_inlined2 _3 _5 _endpos__2_ _startpos__1_ cond x xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0503 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_RPAR _menhir_cell0_REPEAT as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.UNTIL ->
            let _menhir_s = MenhirState0504 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0501 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_lexpr, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SEMI_COLON -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_SEMI_COLON (_menhir_stack, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, e, _, _) = _menhir_stack in
        let MenhirCell1_lexpr (_menhir_stack, _menhir_s, le, _startpos_le_) = _menhir_stack in
        let (_2_inlined1, _endpos_xs_, xs) = ((), _endpos, _v) in
        let _v = _menhir_action_309 _2_inlined1 _endpos__2_ _startpos_le_ e le xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0496 : type  ttv_stack ttv_result. (((((((ttv_stack, ttv_result) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_RPAR _menhir_cell0_WHILE, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_end_semicolon -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_end_semicolon (_menhir_stack, _, _, _endpos__2_) = _menhir_stack in
        let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined2, _) = _menhir_stack in
        let MenhirCell1_DO (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, cond, _, _) = _menhir_stack in
        let MenhirCell0_WHILE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_ARROBASE_LOOPLIMIT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_xs_, xs, _3, _5) = ((), _endpos, _v, (), ()) in
        let _v = _menhir_action_299 _1 _1_inlined2 _3 _5 _endpos__2_ _startpos__1_ cond x xs in
        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _v _menhir_s _tok
    
    and _menhir_run_0491 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_RPAR _menhir_cell0_WHILE, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DO as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.END ->
            _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0491
        | _ ->
            _eRR ()
    
    and _menhir_run_0325 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_IN _menhir_cell0_BNOT _menhir_cell0_LBRACE -> _ -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACE ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
            let MenhirCell0_BNOT (_menhir_stack, _startpos__1_) = _menhir_stack in
            let MenhirCell1_IN (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, sub_expr, _startpos_sub_expr_, _) = _menhir_stack in
            let (_1, _2, _1_inlined2, _endpos__3_, _endpos__1_inlined2_, _startpos__1_inlined2_) = ((), (), _v, _endpos_0, _endpos, _startpos) in
            let _v = _menhir_action_061 _1 _1_inlined2 _2 _endpos__1_inlined2_ _endpos__3_ _startpos__1_ _startpos__1_inlined2_ _startpos_sub_expr_ sub_expr in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_sub_expr_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0321 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_IN _menhir_cell0_LBRACE -> _ -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACE ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_IN (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, sub_expr, _startpos_sub_expr_, _) = _menhir_stack in
            let (_2, _endpos__1_inlined1_, _startpos__1_inlined1_, _1_inlined1, _endpos__3_) = ((), _endpos, _startpos, _v, _endpos_0) in
            let _v = _menhir_action_060 _1_inlined1 _2 _endpos__1_inlined1_ _endpos__3_ _startpos__1_inlined1_ _startpos_sub_expr_ sub_expr in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_sub_expr_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0317 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LPAR, ttv_result) _menhir_cell1_pattern -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RPAR ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_pattern (_menhir_stack, _, x, _, _) = _menhir_stack in
            let MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_2, _endpos__3_, li) = ((), _endpos_0, _v) in
            let _v = _menhir_action_408 _2 _endpos__3_ _startpos__1_ li x in
            _menhir_goto_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos__1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0313 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_LBRACE -> _ -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACE ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_LBRACE (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_endpos__1_inlined1_, _startpos__1_inlined1_, _1_inlined1, _endpos__3_) = (_endpos, _startpos, _v, _endpos_0) in
            let _v = _menhir_action_410 _1_inlined1 _endpos__1_inlined1_ _startpos__1_inlined1_ in
            _menhir_goto_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos__1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0271 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern _menhir_cell0_BNOT _menhir_cell0_LBRACE -> _ -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACE ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
            let MenhirCell0_BNOT (_menhir_stack, _startpos__1_) = _menhir_stack in
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, sub_expr, _startpos_sub_expr_, _) = _menhir_stack in
            let (_1, _2, _1_inlined2, _endpos__3_, _endpos__1_inlined2_, _startpos__1_inlined2_) = ((), (), _v, _endpos_0, _endpos, _startpos) in
            let _v = _menhir_action_116 _1 _1_inlined2 _2 _endpos__1_inlined2_ _endpos__3_ _startpos__1_ _startpos__1_inlined2_ _startpos_sub_expr_ sub_expr in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_sub_expr_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0267 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern _menhir_cell0_LBRACE -> _ -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACE ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, sub_expr, _startpos_sub_expr_, _) = _menhir_stack in
            let (_2, _endpos__1_inlined1_, _startpos__1_inlined1_, _1_inlined1, _endpos__3_) = ((), _endpos, _startpos, _v, _endpos_0) in
            let _v = _menhir_action_115 _1_inlined1 _2 _endpos__1_inlined1_ _endpos__3_ _startpos__1_inlined1_ _startpos_sub_expr_ sub_expr in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_sub_expr_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0230 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_pattern -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_pattern (_menhir_stack, _menhir_s, x, _startpos_x_, _) = _menhir_stack in
        let (_endpos_xs_, xs) = (_endpos, _v) in
        let _v = _menhir_action_436 x xs in
        _menhir_goto_separated_nonempty_list_COMMA_pattern_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_xs_ _startpos_x_ _v _menhir_s _tok
    
    and _menhir_run_0226 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_BNOT, ttv_result) _menhir_cell1_LBRACE -> _ -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACE ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_LBRACE (_menhir_stack, _, _) = _menhir_stack in
            let MenhirCell1_BNOT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _1_inlined2, _endpos__3_, _endpos__1_inlined2_, _startpos__1_inlined2_) = ((), _v, _endpos_0, _endpos, _startpos) in
            let _v = _menhir_action_411 _1 _1_inlined2 _endpos__1_inlined2_ _endpos__3_ _startpos__1_ _startpos__1_inlined2_ in
            _menhir_goto_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos__1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0273 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0273 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0275 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0275 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0277 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell0_GEQ (_menhir_stack, _startpos) in
        let _menhir_s = MenhirState0277 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0279 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0279
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0279
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0279
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0279
        | Tokens.MASK_LIT _v ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, sub_expr, _startpos_sub_expr_, _) = _menhir_stack in
            let (_endpos__1_, _startpos__1_, _1, _2) = (_endpos, _startpos, _v, ()) in
            let _v = _menhir_action_117 _1 _2 _endpos__1_ _startpos__1_ _startpos_sub_expr_ sub_expr in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__1_ _startpos_sub_expr_ _v _menhir_s _tok
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0279
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0279
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0279
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0279
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0279
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0279
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0279
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0279
        | _ ->
            _eRR ()
    
    and _menhir_run_0282 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0282 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0284 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.LBRACKET ->
            let _menhir_s = MenhirState0285 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0045 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.IDENTIFIER _v ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e, _startpos_e_, _) = _menhir_stack in
            let (_2, _endpos_x_, x) = ((), _endpos, _v) in
            let _v = _menhir_action_110 _2 _endpos_x_ _startpos_e_ e x in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_x_ _startpos_e_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0289 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0289 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0291 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0291 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0293 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0293 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0295 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0295 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0297 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0297 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0299 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0299 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0301 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0301 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0303 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING ->
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0303
        | Tokens.REAL ->
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0303
        | Tokens.LPAR ->
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0303
        | Tokens.LBRACE ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0303, _startpos) in
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0304
            | Tokens.REAL_LIT _v ->
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0303, _startpos) in
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0304
            | Tokens.NOT ->
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0303, _startpos) in
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0304
            | Tokens.MINUS ->
                let _startpos_0 = _menhir_lexbuf.Lexing.lex_start_p in
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING_LIT _v ->
                    let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0303, _startpos) in
                    let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, MenhirState0304, _startpos_0, _endpos) in
                    _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0305
                | Tokens.REAL_LIT _v ->
                    let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0303, _startpos) in
                    let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, MenhirState0304, _startpos_0, _endpos) in
                    _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0305
                | Tokens.RBRACE ->
                    let _endpos_1 = _menhir_lexbuf.Lexing.lex_curr_p in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, sub_expr, _startpos_sub_expr_, _) = _menhir_stack in
                    let (_startpos__1_, _2, x, _endpos__3_) = (_startpos, (), (), _endpos_1) in
                    let _v = _menhir_action_114 _2 _endpos__3_ _startpos__1_ _startpos_sub_expr_ sub_expr x in
                    _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_sub_expr_ _v _menhir_s _tok
                | Tokens.NOT ->
                    let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0303, _startpos) in
                    let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, MenhirState0304, _startpos_0, _endpos) in
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0305
                | Tokens.MINUS ->
                    let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0303, _startpos) in
                    let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, MenhirState0304, _startpos_0, _endpos) in
                    _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0305
                | Tokens.LPAR ->
                    let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0303, _startpos) in
                    let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, MenhirState0304, _startpos_0, _endpos) in
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0305
                | Tokens.INT_LIT _v ->
                    let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0303, _startpos) in
                    let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, MenhirState0304, _startpos_0, _endpos) in
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0305
                | Tokens.IF ->
                    let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0303, _startpos) in
                    let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, MenhirState0304, _startpos_0, _endpos) in
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0305
                | Tokens.IDENTIFIER _v ->
                    let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0303, _startpos) in
                    let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, MenhirState0304, _startpos_0, _endpos) in
                    _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0305
                | Tokens.BOOL_LIT _v ->
                    let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0303, _startpos) in
                    let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, MenhirState0304, _startpos_0, _endpos) in
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0305
                | Tokens.BNOT ->
                    let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0303, _startpos) in
                    let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, MenhirState0304, _startpos_0, _endpos) in
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0305
                | Tokens.BITVECTOR_LIT _v ->
                    let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0303, _startpos) in
                    let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, MenhirState0304, _startpos_0, _endpos) in
                    _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0305
                | Tokens.ARBITRARY ->
                    let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0303, _startpos) in
                    let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, MenhirState0304, _startpos_0, _endpos) in
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0305
                | _ ->
                    _eRR ())
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0303, _startpos) in
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0304
            | Tokens.INT_LIT _v ->
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0303, _startpos) in
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0304
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0303, _startpos) in
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0304
            | Tokens.IDENTIFIER _v ->
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0303, _startpos) in
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0304
            | Tokens.BOOL_LIT _v ->
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0303, _startpos) in
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0304
            | Tokens.BNOT ->
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0303, _startpos) in
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0304
            | Tokens.BITVECTOR_LIT _v ->
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0303, _startpos) in
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0304
            | Tokens.ARBITRARY ->
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0303, _startpos) in
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0304
            | _ ->
                _eRR ())
        | Tokens.INTEGER ->
            _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0303
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0303
        | Tokens.BOOLEAN ->
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0303
        | Tokens.BITS ->
            _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0303
        | Tokens.BIT ->
            _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0303
        | Tokens.ARRAY ->
            _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0303
        | _ ->
            _eRR ()
    
    and _menhir_run_0310 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState0310 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0231 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SLICING ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0232 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0234 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0236 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0238 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0240 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0242 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0244 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0246 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0249 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0251 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0253 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0255 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0257 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0260 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0262 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0265 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0273 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0275 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0277 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0279 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0282 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0284 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0289 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0291 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0293 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0295 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0297 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0299 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0301 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0303 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0310 _menhir_stack _menhir_lexbuf _menhir_lexer
        | Tokens.ARROW | Tokens.COMMA | Tokens.RBRACE | Tokens.RPAR | Tokens.WHERE ->
            let (_endpos_expr_pattern_, _startpos_expr_pattern_, expr_pattern) = (_endpos, _startpos, _v) in
            let _v = _menhir_action_403 _endpos_expr_pattern_ _startpos_expr_pattern_ expr_pattern in
            _menhir_goto_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_expr_pattern_ _startpos_expr_pattern_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0092 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_IMPL (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0092 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0088 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_GT (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0088 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0090 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_GEQ (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0090 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0094 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            let _menhir_stack = MenhirCell1_EQ_OP (_menhir_stack, _menhir_s) in
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0094
        | Tokens.REAL_LIT _v ->
            let _menhir_stack = MenhirCell1_EQ_OP (_menhir_stack, _menhir_s) in
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0094
        | Tokens.NOT ->
            let _menhir_stack = MenhirCell1_EQ_OP (_menhir_stack, _menhir_s) in
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0094
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_EQ_OP (_menhir_stack, _menhir_s) in
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0094
        | Tokens.MASK_LIT _v ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, sub_expr, _startpos_sub_expr_, _) = _menhir_stack in
            let (_endpos__1_, _startpos__1_, _1, _2) = (_endpos, _startpos, _v, ()) in
            let _v = _menhir_action_062 _1 _2 _endpos__1_ _startpos__1_ _startpos_sub_expr_ sub_expr in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__1_ _startpos_sub_expr_ _v _menhir_s _tok
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_EQ_OP (_menhir_stack, _menhir_s) in
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0094
        | Tokens.INT_LIT _v ->
            let _menhir_stack = MenhirCell1_EQ_OP (_menhir_stack, _menhir_s) in
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0094
        | Tokens.IF ->
            let _menhir_stack = MenhirCell1_EQ_OP (_menhir_stack, _menhir_s) in
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0094
        | Tokens.IDENTIFIER _v ->
            let _menhir_stack = MenhirCell1_EQ_OP (_menhir_stack, _menhir_s) in
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0094
        | Tokens.BOOL_LIT _v ->
            let _menhir_stack = MenhirCell1_EQ_OP (_menhir_stack, _menhir_s) in
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0094
        | Tokens.BNOT ->
            let _menhir_stack = MenhirCell1_EQ_OP (_menhir_stack, _menhir_s) in
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0094
        | Tokens.BITVECTOR_LIT _v ->
            let _menhir_stack = MenhirCell1_EQ_OP (_menhir_stack, _menhir_s) in
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0094
        | Tokens.ARBITRARY ->
            let _menhir_stack = MenhirCell1_EQ_OP (_menhir_stack, _menhir_s) in
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0094
        | _ ->
            _eRR ()
    
    and _menhir_run_0080 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_EOR (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0080 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0043 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_DOT (_menhir_stack, _menhir_s) in
            let _menhir_s = MenhirState0044 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0045 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.IDENTIFIER _v ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e, _startpos_e_, _) = _menhir_stack in
            let (_2, _endpos_x_, x) = ((), _endpos, _v) in
            let _v = _menhir_action_055 _2 _endpos_x_ _startpos_e_ e x in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_x_ _startpos_e_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0067 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_DIVRM (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0067 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0069 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_DIV (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0069 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0059 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_CONCAT (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0059 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0082 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_COLON_COLON (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0082 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0097 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_BOR (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0097 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0099 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_BEQ (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0099 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0101 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_BAND (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0101 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0103 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING ->
            let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
            _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0103
        | Tokens.REAL ->
            let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
            _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0103
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
            _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0103
        | Tokens.LBRACE ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0103, _startpos) in
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0104
            | Tokens.REAL_LIT _v ->
                let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0103, _startpos) in
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0104
            | Tokens.NOT ->
                let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0103, _startpos) in
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0104
            | Tokens.MINUS ->
                let _startpos_0 = _menhir_lexbuf.Lexing.lex_start_p in
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING_LIT _v ->
                    let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
                    let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0103, _startpos) in
                    let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, MenhirState0104, _startpos_0, _endpos) in
                    _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0105
                | Tokens.REAL_LIT _v ->
                    let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
                    let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0103, _startpos) in
                    let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, MenhirState0104, _startpos_0, _endpos) in
                    _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0105
                | Tokens.RBRACE ->
                    let _endpos_1 = _menhir_lexbuf.Lexing.lex_curr_p in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    let MenhirCell1_expr (_menhir_stack, _menhir_s, sub_expr, _startpos_sub_expr_, _) = _menhir_stack in
                    let (_startpos__1_, _2, x, _endpos__3_) = (_startpos, (), (), _endpos_1) in
                    let _v = _menhir_action_059 _2 _endpos__3_ _startpos__1_ _startpos_sub_expr_ sub_expr x in
                    _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_sub_expr_ _v _menhir_s _tok
                | Tokens.NOT ->
                    let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
                    let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0103, _startpos) in
                    let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, MenhirState0104, _startpos_0, _endpos) in
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0105
                | Tokens.MINUS ->
                    let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
                    let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0103, _startpos) in
                    let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, MenhirState0104, _startpos_0, _endpos) in
                    _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0105
                | Tokens.LPAR ->
                    let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
                    let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0103, _startpos) in
                    let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, MenhirState0104, _startpos_0, _endpos) in
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0105
                | Tokens.INT_LIT _v ->
                    let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
                    let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0103, _startpos) in
                    let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, MenhirState0104, _startpos_0, _endpos) in
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0105
                | Tokens.IF ->
                    let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
                    let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0103, _startpos) in
                    let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, MenhirState0104, _startpos_0, _endpos) in
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0105
                | Tokens.IDENTIFIER _v ->
                    let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
                    let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0103, _startpos) in
                    let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, MenhirState0104, _startpos_0, _endpos) in
                    _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0105
                | Tokens.BOOL_LIT _v ->
                    let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
                    let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0103, _startpos) in
                    let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, MenhirState0104, _startpos_0, _endpos) in
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0105
                | Tokens.BNOT ->
                    let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
                    let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0103, _startpos) in
                    let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, MenhirState0104, _startpos_0, _endpos) in
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0105
                | Tokens.BITVECTOR_LIT _v ->
                    let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
                    let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0103, _startpos) in
                    let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, MenhirState0104, _startpos_0, _endpos) in
                    _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0105
                | Tokens.ARBITRARY ->
                    let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
                    let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0103, _startpos) in
                    let _menhir_stack = MenhirCell1_MINUS (_menhir_stack, MenhirState0104, _startpos_0, _endpos) in
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0105
                | _ ->
                    _eRR ())
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0103, _startpos) in
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0104
            | Tokens.INT_LIT _v ->
                let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0103, _startpos) in
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0104
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0103, _startpos) in
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0104
            | Tokens.IDENTIFIER _v ->
                let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0103, _startpos) in
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0104
            | Tokens.BOOL_LIT _v ->
                let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0103, _startpos) in
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0104
            | Tokens.BNOT ->
                let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0103, _startpos) in
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0104
            | Tokens.BITVECTOR_LIT _v ->
                let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0103, _startpos) in
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0104
            | Tokens.ARBITRARY ->
                let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
                let _menhir_stack = MenhirCell1_LBRACE (_menhir_stack, MenhirState0103, _startpos) in
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0104
            | _ ->
                _eRR ())
        | Tokens.INTEGER ->
            let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
            _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0103
        | Tokens.IDENTIFIER _v ->
            let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
            _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0103
        | Tokens.BOOLEAN ->
            let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
            _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0103
        | Tokens.BITS ->
            let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
            _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0103
        | Tokens.BIT ->
            let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
            _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0103
        | Tokens.ARRAY ->
            let _menhir_stack = MenhirCell1_AS (_menhir_stack, _menhir_s) in
            _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0103
        | _ ->
            _eRR ()
    
    and _menhir_run_0084 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_AND (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0084 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0840 : type  ttv_stack. ((ttv_stack, _menhir_box_opn) _menhir_cell1_LET as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.EQ ->
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v_0 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState0841
            | Tokens.REAL_LIT _v_1 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0841
            | Tokens.NOT ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0841
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0841
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0841
            | Tokens.INT_LIT _v_2 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v_2 MenhirState0841
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0841
            | Tokens.IDENTIFIER _v_3 ->
                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.LPAR ->
                    let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0841, _v_3, _startpos, _endpos) in
                    _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer
                | Tokens.LBRACE ->
                    let _startpos_4 = _menhir_lexbuf.Lexing.lex_start_p in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.STRING_LIT _v_5 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0841, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v_5 MenhirState0843
                    | Tokens.REAL_LIT _v_6 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0841, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v_6 MenhirState0843
                    | Tokens.RBRACE ->
                        let _endpos_7 = _menhir_lexbuf.Lexing.lex_curr_p in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.LPAR ->
                            let _startpos_8 = _menhir_lexbuf.Lexing.lex_start_p in
                            let _tok = _menhir_lexer _menhir_lexbuf in
                            (match (_tok : MenhirBasics.token) with
                            | Tokens.STRING_LIT _v_9 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0841, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0843, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v_9 MenhirState0845
                            | Tokens.RPAR ->
                                let _endpos_10 = _menhir_lexbuf.Lexing.lex_curr_p in
                                let _tok = _menhir_lexer _menhir_lexbuf in
                                (match (_tok : MenhirBasics.token) with
                                | Tokens.SEMI_COLON ->
                                    let _endpos_11 = _menhir_lexbuf.Lexing.lex_curr_p in
                                    let _tok = _menhir_lexer _menhir_lexbuf in
                                    (match (_tok : MenhirBasics.token) with
                                    | Tokens.EOF ->
                                        let MenhirCell1_LET (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                                        let (_1, _2, _2_inlined1, _3, _3_inlined1, _startpos_name_, name, _endpos__2_inlined2_, lhs, _endpos__3_inlined2_) = ((), (), (), (), (), _startpos, _v_3, _endpos_11, _v, _endpos_10) in
                                        let _v = _menhir_action_374 _1 _2 _2_inlined1 _3 _3_inlined1 _endpos__2_inlined2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name in
                                        _menhir_goto_opn _menhir_stack _v
                                    | _ ->
                                        _eRR ())
                                | _ ->
                                    _eRR ())
                            | Tokens.REAL_LIT _v_12 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0841, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0843, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v_12 MenhirState0845
                            | Tokens.NOT ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0841, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0843, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0845
                            | Tokens.MINUS ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0841, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0843, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0845
                            | Tokens.LPAR ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0841, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0843, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0845
                            | Tokens.INT_LIT _v_13 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0841, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0843, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v_13 MenhirState0845
                            | Tokens.IF ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0841, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0843, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0845
                            | Tokens.IDENTIFIER _v_14 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0841, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0843, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v_14 MenhirState0845
                            | Tokens.BOOL_LIT _v_15 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0841, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0843, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_15 MenhirState0845
                            | Tokens.BNOT ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0841, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0843, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0845
                            | Tokens.BITVECTOR_LIT _v_16 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0841, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0843, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v_16 MenhirState0845
                            | Tokens.ARBITRARY ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0841, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0843, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0845
                            | _ ->
                                _eRR ())
                        | Tokens.AND | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DOT | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.IN | Tokens.LBRACKET | Tokens.LEQ | Tokens.LLBRACKET | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RDIV | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR ->
                            let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                            let _v_17 =
                              let (_endpos__3_, _endpos_desc_, _startpos_desc_, desc) = (_endpos_7, _endpos, _startpos, _v_3) in
                              _menhir_action_067 _endpos__3_ _endpos_desc_ _startpos_desc_ desc
                            in
                            _menhir_run_0859 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_7 _startpos_4 _v_17 MenhirState0841 _tok
                        | _ ->
                            _eRR ())
                    | Tokens.NOT ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0841, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0843
                    | Tokens.MINUS ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0841, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0843
                    | Tokens.LPAR ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0841, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0843
                    | Tokens.INT_LIT _v_18 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0841, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v_18 MenhirState0843
                    | Tokens.IF ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0841, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0843
                    | Tokens.IDENTIFIER _v_19 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0841, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _v_19 MenhirState0843
                    | Tokens.COMMA ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0841, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        let _menhir_stack = MenhirCell1_COMMA (_menhir_stack, MenhirState0843) in
                        let _menhir_s = MenhirState0853 in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.STRING_LIT _v ->
                            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.REAL_LIT _v ->
                            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.NOT ->
                            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.MINUS ->
                            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.LPAR ->
                            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.INT_LIT _v ->
                            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.IF ->
                            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.IDENTIFIER _v ->
                            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.BOOL_LIT _v ->
                            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.BNOT ->
                            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.BITVECTOR_LIT _v ->
                            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.ARBITRARY ->
                            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | _ ->
                            _eRR ())
                    | Tokens.BOOL_LIT _v_26 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0841, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_26 MenhirState0843
                    | Tokens.BNOT ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0841, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0843
                    | Tokens.BITVECTOR_LIT _v_27 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0841, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v_27 MenhirState0843
                    | Tokens.ARBITRARY ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0841, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0843
                    | _ ->
                        _eRR ())
                | Tokens.AND | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DOT | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.IN | Tokens.LBRACKET | Tokens.LEQ | Tokens.LLBRACKET | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RDIV | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR ->
                    let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                    let _v_28 =
                      let (_endpos__1_, _startpos__1_, _1) = (_endpos, _startpos, _v_3) in
                      _menhir_action_020 _1 _endpos__1_ _startpos__1_
                    in
                    _menhir_run_0859 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v_28 MenhirState0841 _tok
                | _ ->
                    _eRR ())
            | Tokens.BOOL_LIT _v_29 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_29 MenhirState0841
            | Tokens.BNOT ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0841
            | Tokens.BITVECTOR_LIT _v_30 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v_30 MenhirState0841
            | Tokens.ARBITRARY ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0841
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0859 : type  ttv_stack. (((ttv_stack, _menhir_box_opn) _menhir_cell1_LET, _menhir_box_opn) _menhir_cell1_decl_item as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell1_decl_item (_menhir_stack, _, decl_item) = _menhir_stack in
                let MenhirCell1_LET (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                let (_1, _2, _endpos__2_inlined1_, x, _3) = ((), (), _endpos_0, _v, ()) in
                let _v = _menhir_action_362 _1 _2 _3 _endpos__2_inlined1_ _startpos__1_ decl_item x in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0859
        | _ ->
            _eRR ()
    
    and _menhir_run_0758 : type  ttv_stack. ((ttv_stack, _menhir_box_opn) _menhir_cell1_VAR as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell1_VAR (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                let (_1, _2, _endpos__2_inlined1_, ldi) = ((), (), _endpos, _v) in
                let _v = _menhir_action_380 _1 _2 _endpos__2_inlined1_ _startpos__1_ ldi in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.EQ ->
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v_0 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState0761
            | Tokens.REAL_LIT _v_1 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0761
            | Tokens.NOT ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0761
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0761
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0761
            | Tokens.INT_LIT _v_2 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v_2 MenhirState0761
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0761
            | Tokens.IDENTIFIER _v_3 ->
                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.LPAR ->
                    let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0761, _v_3, _startpos, _endpos) in
                    _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer
                | Tokens.LBRACE ->
                    let _startpos_4 = _menhir_lexbuf.Lexing.lex_start_p in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.STRING_LIT _v_5 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0761, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v_5 MenhirState0763
                    | Tokens.REAL_LIT _v_6 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0761, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v_6 MenhirState0763
                    | Tokens.RBRACE ->
                        let _endpos_7 = _menhir_lexbuf.Lexing.lex_curr_p in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.LPAR ->
                            let _startpos_8 = _menhir_lexbuf.Lexing.lex_start_p in
                            let _tok = _menhir_lexer _menhir_lexbuf in
                            (match (_tok : MenhirBasics.token) with
                            | Tokens.STRING_LIT _v_9 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0761, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0763, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v_9 MenhirState0765
                            | Tokens.RPAR ->
                                let _endpos_10 = _menhir_lexbuf.Lexing.lex_curr_p in
                                let _tok = _menhir_lexer _menhir_lexbuf in
                                (match (_tok : MenhirBasics.token) with
                                | Tokens.SEMI_COLON ->
                                    let _endpos_11 = _menhir_lexbuf.Lexing.lex_curr_p in
                                    let _tok = _menhir_lexer _menhir_lexbuf in
                                    (match (_tok : MenhirBasics.token) with
                                    | Tokens.EOF ->
                                        let MenhirCell1_VAR (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                                        let (_1, _2, _2_inlined1, _3, _3_inlined1, _startpos_name_, name, _endpos__2_inlined2_, lhs, _endpos__3_inlined2_) = ((), (), (), (), (), _startpos, _v_3, _endpos_11, _v, _endpos_10) in
                                        let _v = _menhir_action_384 _1 _2 _2_inlined1 _3 _3_inlined1 _endpos__2_inlined2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name in
                                        _menhir_goto_opn _menhir_stack _v
                                    | _ ->
                                        _eRR ())
                                | _ ->
                                    _eRR ())
                            | Tokens.REAL_LIT _v_12 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0761, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0763, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v_12 MenhirState0765
                            | Tokens.NOT ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0761, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0763, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0765
                            | Tokens.MINUS ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0761, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0763, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0765
                            | Tokens.LPAR ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0761, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0763, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0765
                            | Tokens.INT_LIT _v_13 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0761, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0763, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v_13 MenhirState0765
                            | Tokens.IF ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0761, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0763, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0765
                            | Tokens.IDENTIFIER _v_14 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0761, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0763, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v_14 MenhirState0765
                            | Tokens.BOOL_LIT _v_15 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0761, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0763, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_15 MenhirState0765
                            | Tokens.BNOT ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0761, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0763, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0765
                            | Tokens.BITVECTOR_LIT _v_16 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0761, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0763, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v_16 MenhirState0765
                            | Tokens.ARBITRARY ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0761, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0763, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0765
                            | _ ->
                                _eRR ())
                        | Tokens.AND | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DOT | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.IN | Tokens.LBRACKET | Tokens.LEQ | Tokens.LLBRACKET | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RDIV | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR ->
                            let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                            let _v_17 =
                              let (_endpos__3_, _endpos_desc_, _startpos_desc_, desc) = (_endpos_7, _endpos, _startpos, _v_3) in
                              _menhir_action_067 _endpos__3_ _endpos_desc_ _startpos_desc_ desc
                            in
                            _menhir_run_0779 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_7 _startpos_4 _v_17 MenhirState0761 _tok
                        | _ ->
                            _eRR ())
                    | Tokens.NOT ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0761, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0763
                    | Tokens.MINUS ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0761, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0763
                    | Tokens.LPAR ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0761, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0763
                    | Tokens.INT_LIT _v_18 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0761, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v_18 MenhirState0763
                    | Tokens.IF ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0761, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0763
                    | Tokens.IDENTIFIER _v_19 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0761, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _v_19 MenhirState0763
                    | Tokens.COMMA ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0761, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        let _menhir_stack = MenhirCell1_COMMA (_menhir_stack, MenhirState0763) in
                        let _menhir_s = MenhirState0773 in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.STRING_LIT _v ->
                            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.REAL_LIT _v ->
                            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.NOT ->
                            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.MINUS ->
                            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.LPAR ->
                            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.INT_LIT _v ->
                            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.IF ->
                            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.IDENTIFIER _v ->
                            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.BOOL_LIT _v ->
                            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.BNOT ->
                            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.BITVECTOR_LIT _v ->
                            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.ARBITRARY ->
                            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | _ ->
                            _eRR ())
                    | Tokens.BOOL_LIT _v_26 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0761, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_26 MenhirState0763
                    | Tokens.BNOT ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0761, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0763
                    | Tokens.BITVECTOR_LIT _v_27 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0761, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v_27 MenhirState0763
                    | Tokens.ARBITRARY ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0761, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0763
                    | _ ->
                        _eRR ())
                | Tokens.AND | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DOT | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.IN | Tokens.LBRACKET | Tokens.LEQ | Tokens.LLBRACKET | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RDIV | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR ->
                    let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                    let _v_28 =
                      let (_endpos__1_, _startpos__1_, _1) = (_endpos, _startpos, _v_3) in
                      _menhir_action_020 _1 _endpos__1_ _startpos__1_
                    in
                    _menhir_run_0779 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v_28 MenhirState0761 _tok
                | _ ->
                    _eRR ())
            | Tokens.BOOL_LIT _v_29 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_29 MenhirState0761
            | Tokens.BNOT ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0761
            | Tokens.BITVECTOR_LIT _v_30 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v_30 MenhirState0761
            | Tokens.ARBITRARY ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0761
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0779 : type  ttv_stack. (((ttv_stack, _menhir_box_opn) _menhir_cell1_VAR, _menhir_box_opn) _menhir_cell1_decl_item as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0779
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0779
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell1_decl_item (_menhir_stack, _, ldi) = _menhir_stack in
                let MenhirCell1_VAR (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                let (_1, _2, _2_inlined1, _1_inlined1, _endpos__2_inlined2_) = ((), (), _v, (), _endpos_0) in
                let _v = _menhir_action_381 _1 _1_inlined1 _2 _2_inlined1 _endpos__2_inlined2_ _startpos__1_ ldi in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0779
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0779
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0779
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0779
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0779
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0779
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0779
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0779
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0779
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0779
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0779
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0779
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0779
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0779
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0779
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0779
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0779
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0779
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0779
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0779
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0779
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0779
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0779
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0779
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0779
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0779
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0779
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0779
        | _ ->
            _eRR ()
    
    and _menhir_run_0708 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_VAR as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0709
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0709
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0709
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0709
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0709
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0709
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0709
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0709
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0709
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0709
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0709
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0709
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0709
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0709
            | Tokens.IDENTIFIER _v_0 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState0709
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0709
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0709
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0709
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0709
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0709
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0709
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                let (_1, _endpos__2_, ldi) = ((), _endpos, _v) in
                let _v = _menhir_action_270 _1 _endpos__2_ _startpos__1_ ldi in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.EQ ->
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v_1 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0711
            | Tokens.REAL_LIT _v_2 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v_2 MenhirState0711
            | Tokens.NOT ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0711
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0711
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0711
            | Tokens.INT_LIT _v_3 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v_3 MenhirState0711
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0711
            | Tokens.IDENTIFIER _v_4 ->
                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.LPAR ->
                    let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                    _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer
                | Tokens.LBRACE ->
                    let _startpos_5 = _menhir_lexbuf.Lexing.lex_start_p in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.STRING_LIT _v_6 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                        _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v_6 MenhirState0713
                    | Tokens.REAL_LIT _v_7 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                        _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v_7 MenhirState0713
                    | Tokens.RBRACE ->
                        let _endpos_8 = _menhir_lexbuf.Lexing.lex_curr_p in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.LPAR ->
                            let _startpos_9 = _menhir_lexbuf.Lexing.lex_start_p in
                            let _tok = _menhir_lexer _menhir_lexbuf in
                            (match (_tok : MenhirBasics.token) with
                            | Tokens.STRING_LIT _v_10 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0713, _endpos_8) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v_10 MenhirState0715
                            | Tokens.RPAR ->
                                let _endpos_11 = _menhir_lexbuf.Lexing.lex_curr_p in
                                let _tok = _menhir_lexer _menhir_lexbuf in
                                (match (_tok : MenhirBasics.token) with
                                | Tokens.SEMI_COLON ->
                                    let _endpos_12 = _menhir_lexbuf.Lexing.lex_curr_p in
                                    let _tok = _menhir_lexer _menhir_lexbuf in
                                    (match (_tok : MenhirBasics.token) with
                                    | Tokens.WHILE ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0713, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0715, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0717
                                    | Tokens.VAR ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0713, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0715, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0717
                                    | Tokens.UNREACHABLE ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0713, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0715, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0717
                                    | Tokens.TRY ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0713, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0715, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0717
                                    | Tokens.THROW ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0713, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0715, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0717
                                    | Tokens.RETURN ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0713, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0715, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0717
                                    | Tokens.REPEAT ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0713, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0715, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0717
                                    | Tokens.PRINT ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0713, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0715, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0717
                                    | Tokens.PRAGMA ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0713, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0715, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0717
                                    | Tokens.PASS ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0713, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0715, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0717
                                    | Tokens.MINUS ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0713, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0715, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0717
                                    | Tokens.LPAR ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0713, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0715, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0717
                                    | Tokens.LET ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0713, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0715, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0717
                                    | Tokens.IF ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0713, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0715, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0717
                                    | Tokens.IDENTIFIER _v_13 ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0713, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0715, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_13 MenhirState0717
                                    | Tokens.FOR ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0713, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0715, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0717
                                    | Tokens.DEBUG ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0713, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0715, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0717
                                    | Tokens.CONSTANT ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0713, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0715, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0717
                                    | Tokens.CASE ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0713, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0715, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0717
                                    | Tokens.ASSERT ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0713, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0715, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0717
                                    | Tokens.ARROBASE_LOOPLIMIT ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0713, _endpos_8) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0715, _endpos_11) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_12) in
                                        _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0717
                                    | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                                        let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                                        let (_1, _endpos__2_, _2_inlined1, _3, _3_inlined1, _startpos_name_, name, lhs, _endpos__3_inlined2_) = ((), _endpos_12, (), (), (), _startpos, _v_4, _v, _endpos_11) in
                                        let _v = _menhir_action_274 _1 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name in
                                        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
                                    | _ ->
                                        _eRR ())
                                | _ ->
                                    _eRR ())
                            | Tokens.REAL_LIT _v_14 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0713, _endpos_8) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v_14 MenhirState0715
                            | Tokens.NOT ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0713, _endpos_8) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0715
                            | Tokens.MINUS ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0713, _endpos_8) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0715
                            | Tokens.LPAR ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0713, _endpos_8) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0715
                            | Tokens.INT_LIT _v_15 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0713, _endpos_8) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v_15 MenhirState0715
                            | Tokens.IF ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0713, _endpos_8) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0715
                            | Tokens.IDENTIFIER _v_16 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0713, _endpos_8) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v_16 MenhirState0715
                            | Tokens.BOOL_LIT _v_17 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0713, _endpos_8) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_17 MenhirState0715
                            | Tokens.BNOT ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0713, _endpos_8) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0715
                            | Tokens.BITVECTOR_LIT _v_18 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0713, _endpos_8) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v_18 MenhirState0715
                            | Tokens.ARBITRARY ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0713, _endpos_8) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_9) in
                                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0715
                            | _ ->
                                _eRR ())
                        | Tokens.AND | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DOT | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.IN | Tokens.LBRACKET | Tokens.LEQ | Tokens.LLBRACKET | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RDIV | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR ->
                            let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                            let _v_19 =
                              let (_endpos__3_, _endpos_desc_, _startpos_desc_, desc) = (_endpos_8, _endpos, _startpos, _v_4) in
                              _menhir_action_067 _endpos__3_ _endpos_desc_ _startpos_desc_ desc
                            in
                            _menhir_run_0729 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_8 _startpos_5 _v_19 MenhirState0711 _tok
                        | _ ->
                            _eRR ())
                    | Tokens.NOT ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                        _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0713
                    | Tokens.MINUS ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                        _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0713
                    | Tokens.LPAR ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                        _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0713
                    | Tokens.INT_LIT _v_20 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                        _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v_20 MenhirState0713
                    | Tokens.IF ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                        _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0713
                    | Tokens.IDENTIFIER _v_21 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                        _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _v_21 MenhirState0713
                    | Tokens.COMMA ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                        let _menhir_stack = MenhirCell1_COMMA (_menhir_stack, MenhirState0713) in
                        let _menhir_s = MenhirState0723 in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.STRING_LIT _v ->
                            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.REAL_LIT _v ->
                            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.NOT ->
                            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.MINUS ->
                            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.LPAR ->
                            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.INT_LIT _v ->
                            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.IF ->
                            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.IDENTIFIER _v ->
                            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.BOOL_LIT _v ->
                            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.BNOT ->
                            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.BITVECTOR_LIT _v ->
                            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.ARBITRARY ->
                            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | _ ->
                            _eRR ())
                    | Tokens.BOOL_LIT _v_28 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                        _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_28 MenhirState0713
                    | Tokens.BNOT ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                        _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0713
                    | Tokens.BITVECTOR_LIT _v_29 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                        _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v_29 MenhirState0713
                    | Tokens.ARBITRARY ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0711, _v_4, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_5) in
                        _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0713
                    | _ ->
                        _eRR ())
                | Tokens.AND | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DOT | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.IN | Tokens.LBRACKET | Tokens.LEQ | Tokens.LLBRACKET | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RDIV | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR ->
                    let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                    let _v_30 =
                      let (_endpos__1_, _startpos__1_, _1) = (_endpos, _startpos, _v_4) in
                      _menhir_action_020 _1 _endpos__1_ _startpos__1_
                    in
                    _menhir_run_0729 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v_30 MenhirState0711 _tok
                | _ ->
                    _eRR ())
            | Tokens.BOOL_LIT _v_31 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_31 MenhirState0711
            | Tokens.BNOT ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0711
            | Tokens.BITVECTOR_LIT _v_32 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v_32 MenhirState0711
            | Tokens.ARBITRARY ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0711
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0729 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_VAR, ttv_result) _menhir_cell1_decl_item as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0729
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0729
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0729, _endpos_0) in
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0730
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0729, _endpos_0) in
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0730
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0729, _endpos_0) in
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0730
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0729, _endpos_0) in
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0730
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0729, _endpos_0) in
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0730
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0729, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0730
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0729, _endpos_0) in
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0730
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0729, _endpos_0) in
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0730
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0729, _endpos_0) in
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0730
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0729, _endpos_0) in
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0730
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0729, _endpos_0) in
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0730
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0729, _endpos_0) in
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0730
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0729, _endpos_0) in
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0730
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0729, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0730
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0729, _endpos_0) in
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0730
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0729, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0730
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0729, _endpos_0) in
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0730
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0729, _endpos_0) in
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0730
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0729, _endpos_0) in
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0730
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0729, _endpos_0) in
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0730
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0729, _endpos_0) in
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0730
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell1_decl_item (_menhir_stack, _, ldi) = _menhir_stack in
                let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                let (_1, _endpos__2_, _2_inlined1, _1_inlined1) = ((), _endpos_0, _v, ()) in
                let _v = _menhir_action_271 _1 _1_inlined1 _2_inlined1 _endpos__2_ _startpos__1_ ldi in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0729
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0729
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0729
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0729
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0729
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0729
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0729
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0729
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0729
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0729
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0729
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0729
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0729
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0729
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0729
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0729
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0729
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0729
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0729
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0729
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0729
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0729
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0729
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0729
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0729
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0729
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0729
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0729
        | _ ->
            _eRR ()
    
    and _menhir_run_0465 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_CONSTANT as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.EQ ->
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v_0 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState0466
            | Tokens.REAL_LIT _v_1 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0466
            | Tokens.NOT ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0466
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0466
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0466
            | Tokens.INT_LIT _v_2 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v_2 MenhirState0466
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0466
            | Tokens.IDENTIFIER _v_3 ->
                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.LPAR ->
                    let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                    _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer
                | Tokens.LBRACE ->
                    let _startpos_4 = _menhir_lexbuf.Lexing.lex_start_p in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.STRING_LIT _v_5 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v_5 MenhirState0468
                    | Tokens.REAL_LIT _v_6 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v_6 MenhirState0468
                    | Tokens.RBRACE ->
                        let _endpos_7 = _menhir_lexbuf.Lexing.lex_curr_p in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.LPAR ->
                            let _startpos_8 = _menhir_lexbuf.Lexing.lex_start_p in
                            let _tok = _menhir_lexer _menhir_lexbuf in
                            (match (_tok : MenhirBasics.token) with
                            | Tokens.STRING_LIT _v_9 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0468, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v_9 MenhirState0470
                            | Tokens.RPAR ->
                                let _endpos_10 = _menhir_lexbuf.Lexing.lex_curr_p in
                                let _tok = _menhir_lexer _menhir_lexbuf in
                                (match (_tok : MenhirBasics.token) with
                                | Tokens.SEMI_COLON ->
                                    let _endpos_11 = _menhir_lexbuf.Lexing.lex_curr_p in
                                    let _tok = _menhir_lexer _menhir_lexbuf in
                                    (match (_tok : MenhirBasics.token) with
                                    | Tokens.WHILE ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0468, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0470, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0472
                                    | Tokens.VAR ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0468, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0470, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0472
                                    | Tokens.UNREACHABLE ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0468, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0470, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0472
                                    | Tokens.TRY ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0468, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0470, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0472
                                    | Tokens.THROW ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0468, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0470, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0472
                                    | Tokens.RETURN ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0468, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0470, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0472
                                    | Tokens.REPEAT ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0468, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0470, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0472
                                    | Tokens.PRINT ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0468, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0470, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0472
                                    | Tokens.PRAGMA ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0468, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0470, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0472
                                    | Tokens.PASS ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0468, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0470, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0472
                                    | Tokens.MINUS ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0468, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0470, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0472
                                    | Tokens.LPAR ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0468, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0470, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0472
                                    | Tokens.LET ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0468, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0470, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0472
                                    | Tokens.IF ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0468, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0470, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0472
                                    | Tokens.IDENTIFIER _v_12 ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0468, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0470, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_12 MenhirState0472
                                    | Tokens.FOR ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0468, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0470, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0472
                                    | Tokens.DEBUG ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0468, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0470, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0472
                                    | Tokens.CONSTANT ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0468, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0470, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0472
                                    | Tokens.CASE ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0468, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0470, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0472
                                    | Tokens.ASSERT ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0468, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0470, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0472
                                    | Tokens.ARROBASE_LOOPLIMIT ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0468, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0470, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0472
                                    | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                                        let MenhirCell1_CONSTANT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                                        let (_1, _endpos__2_, _2_inlined1, _3, _3_inlined1, _startpos_name_, name, lhs, _endpos__3_inlined2_) = ((), _endpos_11, (), (), (), _startpos, _v_3, _v, _endpos_10) in
                                        let _v = _menhir_action_267 _1 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name in
                                        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
                                    | _ ->
                                        _eRR ())
                                | _ ->
                                    _eRR ())
                            | Tokens.REAL_LIT _v_13 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0468, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v_13 MenhirState0470
                            | Tokens.NOT ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0468, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0470
                            | Tokens.MINUS ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0468, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0470
                            | Tokens.LPAR ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0468, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0470
                            | Tokens.INT_LIT _v_14 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0468, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v_14 MenhirState0470
                            | Tokens.IF ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0468, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0470
                            | Tokens.IDENTIFIER _v_15 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0468, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v_15 MenhirState0470
                            | Tokens.BOOL_LIT _v_16 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0468, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_16 MenhirState0470
                            | Tokens.BNOT ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0468, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0470
                            | Tokens.BITVECTOR_LIT _v_17 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0468, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v_17 MenhirState0470
                            | Tokens.ARBITRARY ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0468, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0470
                            | _ ->
                                _eRR ())
                        | Tokens.AND | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DOT | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.IN | Tokens.LBRACKET | Tokens.LEQ | Tokens.LLBRACKET | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RDIV | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR ->
                            let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                            let _v_18 =
                              let (_endpos__3_, _endpos_desc_, _startpos_desc_, desc) = (_endpos_7, _endpos, _startpos, _v_3) in
                              _menhir_action_067 _endpos__3_ _endpos_desc_ _startpos_desc_ desc
                            in
                            _menhir_run_0534 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_7 _startpos_4 _v_18 MenhirState0466 _tok
                        | _ ->
                            _eRR ())
                    | Tokens.NOT ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0468
                    | Tokens.MINUS ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0468
                    | Tokens.LPAR ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0468
                    | Tokens.INT_LIT _v_19 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v_19 MenhirState0468
                    | Tokens.IF ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0468
                    | Tokens.IDENTIFIER _v_20 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _v_20 MenhirState0468
                    | Tokens.COMMA ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        let _menhir_stack = MenhirCell1_COMMA (_menhir_stack, MenhirState0468) in
                        let _menhir_s = MenhirState0528 in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.STRING_LIT _v ->
                            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.REAL_LIT _v ->
                            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.NOT ->
                            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.MINUS ->
                            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.LPAR ->
                            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.INT_LIT _v ->
                            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.IF ->
                            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.IDENTIFIER _v ->
                            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.BOOL_LIT _v ->
                            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.BNOT ->
                            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.BITVECTOR_LIT _v ->
                            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.ARBITRARY ->
                            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | _ ->
                            _eRR ())
                    | Tokens.BOOL_LIT _v_27 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_27 MenhirState0468
                    | Tokens.BNOT ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0468
                    | Tokens.BITVECTOR_LIT _v_28 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v_28 MenhirState0468
                    | Tokens.ARBITRARY ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0466, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0468
                    | _ ->
                        _eRR ())
                | Tokens.AND | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DOT | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.IN | Tokens.LBRACKET | Tokens.LEQ | Tokens.LLBRACKET | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RDIV | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR ->
                    let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                    let _v_29 =
                      let (_endpos__1_, _startpos__1_, _1) = (_endpos, _startpos, _v_3) in
                      _menhir_action_020 _1 _endpos__1_ _startpos__1_
                    in
                    _menhir_run_0534 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v_29 MenhirState0466 _tok
                | _ ->
                    _eRR ())
            | Tokens.BOOL_LIT _v_30 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_30 MenhirState0466
            | Tokens.BNOT ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0466
            | Tokens.BITVECTOR_LIT _v_31 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v_31 MenhirState0466
            | Tokens.ARBITRARY ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0466
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0534 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_CONSTANT, ttv_result) _menhir_cell1_decl_item as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0534
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0534
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0534, _endpos_0) in
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0535
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0534, _endpos_0) in
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0535
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0534, _endpos_0) in
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0535
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0534, _endpos_0) in
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0535
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0534, _endpos_0) in
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0535
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0534, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0535
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0534, _endpos_0) in
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0535
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0534, _endpos_0) in
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0535
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0534, _endpos_0) in
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0535
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0534, _endpos_0) in
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0535
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0534, _endpos_0) in
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0535
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0534, _endpos_0) in
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0535
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0534, _endpos_0) in
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0535
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0534, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0535
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0534, _endpos_0) in
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0535
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0534, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0535
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0534, _endpos_0) in
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0535
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0534, _endpos_0) in
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0535
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0534, _endpos_0) in
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0535
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0534, _endpos_0) in
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0535
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0534, _endpos_0) in
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0535
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell1_decl_item (_menhir_stack, _, decl_item) = _menhir_stack in
                let MenhirCell1_CONSTANT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                let (_1, _endpos__2_, x, _3) = ((), _endpos_0, _v, ()) in
                let _v = _menhir_action_253 _1 _3 _endpos__2_ _startpos__1_ decl_item x in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0534
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0534
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0534
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0534
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0534
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0534
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0534
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0534
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0534
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0534
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0534
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0534
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0534
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0534
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0534
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0534
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0534
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0534
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0534
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0534
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0534
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0534
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0534
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0534
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0534
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0534
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0534
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0534
        | _ ->
            _eRR ()
    
    and _menhir_run_0436 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LET as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.EQ ->
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v_0 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState0437
            | Tokens.REAL_LIT _v_1 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0437
            | Tokens.NOT ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0437
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0437
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0437
            | Tokens.INT_LIT _v_2 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v_2 MenhirState0437
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0437
            | Tokens.IDENTIFIER _v_3 ->
                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.LPAR ->
                    let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                    _menhir_run_0010 _menhir_stack _menhir_lexbuf _menhir_lexer
                | Tokens.LBRACE ->
                    let _startpos_4 = _menhir_lexbuf.Lexing.lex_start_p in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.STRING_LIT _v_5 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v_5 MenhirState0439
                    | Tokens.REAL_LIT _v_6 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v_6 MenhirState0439
                    | Tokens.RBRACE ->
                        let _endpos_7 = _menhir_lexbuf.Lexing.lex_curr_p in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.LPAR ->
                            let _startpos_8 = _menhir_lexbuf.Lexing.lex_start_p in
                            let _tok = _menhir_lexer _menhir_lexbuf in
                            (match (_tok : MenhirBasics.token) with
                            | Tokens.STRING_LIT _v_9 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0439, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v_9 MenhirState0441
                            | Tokens.RPAR ->
                                let _endpos_10 = _menhir_lexbuf.Lexing.lex_curr_p in
                                let _tok = _menhir_lexer _menhir_lexbuf in
                                (match (_tok : MenhirBasics.token) with
                                | Tokens.SEMI_COLON ->
                                    let _endpos_11 = _menhir_lexbuf.Lexing.lex_curr_p in
                                    let _tok = _menhir_lexer _menhir_lexbuf in
                                    (match (_tok : MenhirBasics.token) with
                                    | Tokens.WHILE ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0439, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0441, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0443
                                    | Tokens.VAR ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0439, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0441, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0443
                                    | Tokens.UNREACHABLE ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0439, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0441, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0443
                                    | Tokens.TRY ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0439, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0441, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0443
                                    | Tokens.THROW ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0439, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0441, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0443
                                    | Tokens.RETURN ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0439, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0441, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0443
                                    | Tokens.REPEAT ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0439, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0441, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0443
                                    | Tokens.PRINT ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0439, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0441, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0443
                                    | Tokens.PRAGMA ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0439, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0441, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0443
                                    | Tokens.PASS ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0439, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0441, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0443
                                    | Tokens.MINUS ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0439, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0441, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0443
                                    | Tokens.LPAR ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0439, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0441, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0443
                                    | Tokens.LET ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0439, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0441, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0443
                                    | Tokens.IF ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0439, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0441, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0443
                                    | Tokens.IDENTIFIER _v_12 ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0439, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0441, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_12 MenhirState0443
                                    | Tokens.FOR ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0439, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0441, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0443
                                    | Tokens.DEBUG ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0439, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0441, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0443
                                    | Tokens.CONSTANT ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0439, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0441, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0443
                                    | Tokens.CASE ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0439, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0441, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0443
                                    | Tokens.ASSERT ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0439, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0441, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0443
                                    | Tokens.ARROBASE_LOOPLIMIT ->
                                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                        let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0439, _endpos_7) in
                                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0441, _endpos_10) in
                                        let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_11) in
                                        _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0443
                                    | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                                        let MenhirCell1_LET (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                                        let (_1, _endpos__2_, _2_inlined1, _3, _3_inlined1, _startpos_name_, name, lhs, _endpos__3_inlined2_) = ((), _endpos_11, (), (), (), _startpos, _v_3, _v, _endpos_10) in
                                        let _v = _menhir_action_264 _1 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name in
                                        _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
                                    | _ ->
                                        _eRR ())
                                | _ ->
                                    _eRR ())
                            | Tokens.REAL_LIT _v_13 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0439, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v_13 MenhirState0441
                            | Tokens.NOT ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0439, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0441
                            | Tokens.MINUS ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0439, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0441
                            | Tokens.LPAR ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0439, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0441
                            | Tokens.INT_LIT _v_14 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0439, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v_14 MenhirState0441
                            | Tokens.IF ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0439, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0441
                            | Tokens.IDENTIFIER _v_15 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0439, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v_15 MenhirState0441
                            | Tokens.BOOL_LIT _v_16 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0439, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_16 MenhirState0441
                            | Tokens.BNOT ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0439, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0441
                            | Tokens.BITVECTOR_LIT _v_17 ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0439, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v_17 MenhirState0441
                            | Tokens.ARBITRARY ->
                                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                                let _menhir_stack = MenhirCell1_RBRACE (_menhir_stack, MenhirState0439, _endpos_7) in
                                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_8) in
                                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0441
                            | _ ->
                                _eRR ())
                        | Tokens.AND | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DOT | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.IN | Tokens.LBRACKET | Tokens.LEQ | Tokens.LLBRACKET | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RDIV | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR ->
                            let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                            let _v_18 =
                              let (_endpos__3_, _endpos_desc_, _startpos_desc_, desc) = (_endpos_7, _endpos, _startpos, _v_3) in
                              _menhir_action_067 _endpos__3_ _endpos_desc_ _startpos_desc_ desc
                            in
                            _menhir_run_0651 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_7 _startpos_4 _v_18 MenhirState0437 _tok
                        | _ ->
                            _eRR ())
                    | Tokens.NOT ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0439
                    | Tokens.MINUS ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0439
                    | Tokens.LPAR ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0439
                    | Tokens.INT_LIT _v_19 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v_19 MenhirState0439
                    | Tokens.IF ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0439
                    | Tokens.IDENTIFIER _v_20 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0193 _menhir_stack _menhir_lexbuf _menhir_lexer _v_20 MenhirState0439
                    | Tokens.COMMA ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        let _menhir_stack = MenhirCell1_COMMA (_menhir_stack, MenhirState0439) in
                        let _menhir_s = MenhirState0645 in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.STRING_LIT _v ->
                            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.REAL_LIT _v ->
                            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.NOT ->
                            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.MINUS ->
                            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.LPAR ->
                            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.INT_LIT _v ->
                            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.IF ->
                            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.IDENTIFIER _v ->
                            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.BOOL_LIT _v ->
                            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.BNOT ->
                            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.BITVECTOR_LIT _v ->
                            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.ARBITRARY ->
                            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | _ ->
                            _eRR ())
                    | Tokens.BOOL_LIT _v_27 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_27 MenhirState0439
                    | Tokens.BNOT ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0439
                    | Tokens.BITVECTOR_LIT _v_28 ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v_28 MenhirState0439
                    | Tokens.ARBITRARY ->
                        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0437, _v_3, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_4) in
                        _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0439
                    | _ ->
                        _eRR ())
                | Tokens.AND | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DOT | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.IN | Tokens.LBRACKET | Tokens.LEQ | Tokens.LLBRACKET | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RDIV | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR ->
                    let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                    let _v_29 =
                      let (_endpos__1_, _startpos__1_, _1) = (_endpos, _startpos, _v_3) in
                      _menhir_action_020 _1 _endpos__1_ _startpos__1_
                    in
                    _menhir_run_0651 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v_29 MenhirState0437 _tok
                | _ ->
                    _eRR ())
            | Tokens.BOOL_LIT _v_30 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_30 MenhirState0437
            | Tokens.BNOT ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0437
            | Tokens.BITVECTOR_LIT _v_31 ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v_31 MenhirState0437
            | Tokens.ARBITRARY ->
                let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0437
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0651 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_LET, ttv_result) _menhir_cell1_decl_item as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0651
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0651
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0651, _endpos_0) in
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0651, _endpos_0) in
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0651, _endpos_0) in
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0651, _endpos_0) in
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0651, _endpos_0) in
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0651, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0651, _endpos_0) in
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0651, _endpos_0) in
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0651, _endpos_0) in
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0651, _endpos_0) in
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0651, _endpos_0) in
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0651, _endpos_0) in
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0651, _endpos_0) in
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0651, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0651, _endpos_0) in
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0652
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0651, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0651, _endpos_0) in
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0651, _endpos_0) in
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0651, _endpos_0) in
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0651, _endpos_0) in
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0651, _endpos_0) in
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0652
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell1_decl_item (_menhir_stack, _, decl_item) = _menhir_stack in
                let MenhirCell1_LET (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                let (_1, _endpos__2_, x, _3) = ((), _endpos_0, _v, ()) in
                let _v = _menhir_action_252 _1 _3 _endpos__2_ _startpos__1_ decl_item x in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0651
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0651
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0651
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0651
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0651
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0651
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0651
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0651
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0651
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0651
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0651
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0651
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0651
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0651
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0651
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0651
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0651
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0651
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0651
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0651
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0651
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0651
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0651
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0651
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0651
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0651
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0651
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0651
        | _ ->
            _eRR ()
    
    and _menhir_run_0384 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_decl_item as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
            let _menhir_s = MenhirState0385 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.MINUS ->
                _menhir_run_0365 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0370 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0371 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.RPAR ->
            let x = _v in
            let _v = _menhir_action_417 x in
            _menhir_goto_separated_nonempty_list_COMMA_decl_item_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_goto_separated_nonempty_list_COMMA_decl_item_ : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_decl_item as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        match _menhir_s with
        | MenhirState0385 ->
            _menhir_run_0386 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0377 ->
            _menhir_run_0378 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0386 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_decl_item -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_decl_item (_menhir_stack, _menhir_s, x) = _menhir_stack in
        let xs = _v in
        let _v = _menhir_action_418 x xs in
        _menhir_goto_separated_nonempty_list_COMMA_decl_item_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_0378 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_LPAR, ttv_result) _menhir_cell1_decl_item as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_decl_item_ (_menhir_stack, _menhir_s, _v) in
            let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
            let _menhir_s = MenhirState0380 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON ->
            let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_decl_item_ (_menhir_stack, _menhir_s, _v) in
            let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
            let _menhir_s = MenhirState0382 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COMMA | Tokens.EQ | Tokens.RPAR | Tokens.SEMI_COLON ->
            let MenhirCell1_decl_item (_menhir_stack, _, x) = _menhir_stack in
            let MenhirCell1_LPAR (_menhir_stack, _menhir_s, _) = _menhir_stack in
            let (_2, li) = ((), _v) in
            let _v = _menhir_action_011 _2 li x in
            _menhir_goto_decl_item _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0376 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LPAR as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_decl_item (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.COMMA ->
            let _menhir_s = MenhirState0377 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.MINUS ->
                _menhir_run_0365 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0370 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0371 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1075 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_loption_braced_nclist_opt_typed_identifier___ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.LPAR ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos) in
            let _menhir_s = MenhirState1076 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.RPAR ->
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, _menhir_s, _endpos) in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.EQ ->
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.IDENTIFIER _v ->
                        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                        let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.COLON_COLON ->
                            let _menhir_s = MenhirState1080 in
                            let _tok = _menhir_lexer _menhir_lexbuf in
                            (match (_tok : MenhirBasics.token) with
                            | Tokens.STRING ->
                                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.REAL ->
                                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.LPAR ->
                                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.INTEGER ->
                                _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.IDENTIFIER _v ->
                                _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                            | Tokens.BOOLEAN ->
                                _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.BITS ->
                                _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.BIT ->
                                _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.ARRAY ->
                                _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | _ ->
                                _eRR ())
                        | Tokens.COLON ->
                            let _menhir_s = MenhirState1283 in
                            let _tok = _menhir_lexer _menhir_lexbuf in
                            (match (_tok : MenhirBasics.token) with
                            | Tokens.STRING ->
                                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.REAL ->
                                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.LPAR ->
                                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.INTEGER ->
                                _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.IDENTIFIER _v ->
                                _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                            | Tokens.BOOLEAN ->
                                _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.BITS ->
                                _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.BIT ->
                                _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | Tokens.ARRAY ->
                                _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                            | _ ->
                                _eRR ())
                        | _ ->
                            _eRR ())
                    | _ ->
                        _eRR ())
                | _ ->
                    _eRR ())
            | Tokens.IDENTIFIER _v ->
                _menhir_run_1200 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1072 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty -> _ -> _ -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, x, _, _) = _menhir_stack in
        let (_1, xs) = ((), _v) in
        let _v = _menhir_action_433 _1 _2_inlined1 x xs in
        _menhir_goto_separated_nonempty_list_COMMA_opt_typed_identifier_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_1068 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty -> _ -> _ -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, x, _, _) = _menhir_stack in
        let (_1, xs) = ((), _v) in
        let _v = _menhir_action_434 _1 _2_inlined1 x xs in
        _menhir_goto_separated_nonempty_list_COMMA_opt_typed_identifier_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_1064 : type  ttv_stack. (ttv_stack, _menhir_box_spec) _menhir_cell1_IDENTIFIER -> _ -> _ -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, x, _, _) = _menhir_stack in
        let xs = _v in
        let _v = _menhir_action_432 x xs in
        _menhir_goto_separated_nonempty_list_COMMA_opt_typed_identifier_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_1057 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_TYPE _menhir_cell0_IDENTIFIER _menhir_cell0_SUBTYPES _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_loption___anonymous_21_ (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1058
            | Tokens.TYPE ->
                _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1058
            | Tokens.SETTER ->
                _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1058
            | Tokens.PRAGMA ->
                _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1058
            | Tokens.LET ->
                _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1058
            | Tokens.GETTER ->
                _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1058
            | Tokens.FUNC ->
                _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1058
            | Tokens.CONSTANT ->
                _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1058
            | Tokens.CONFIG ->
                _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1058
            | Tokens.EOF ->
                let _v_1 = _menhir_action_133 () in
                _menhir_run_1311 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1315 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_TYPE _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_RECORD _menhir_cell0_LBRACE -> _ -> _ -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_RECORD (_menhir_stack, _menhir_s, _startpos__1_, _) = _menhir_stack in
        let (_1, _1_inlined2, _endpos__3_) = ((), _v, _endpos) in
        let _v = _menhir_action_470 _1 _1_inlined2 _endpos__3_ _startpos__1_ in
        _menhir_goto_ty_decl _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
    
    and _menhir_goto_ty_decl : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_TYPE _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty_decl (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SUBTYPES ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell0_SUBTYPES (_menhir_stack, _startpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.IDENTIFIER _v_0 ->
                let _startpos_1 = _menhir_lexbuf.Lexing.lex_start_p in
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v_0, _startpos_1, _endpos) in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.WITH ->
                    _menhir_run_1043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1330
                | Tokens.SEMI_COLON ->
                    let _v_2 = _menhir_action_190 () in
                    _menhir_run_1331 _menhir_stack _menhir_lexbuf _menhir_lexer _v_2 _tok
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.SEMI_COLON ->
            let _v = _menhir_action_401 () in
            _menhir_goto_option_subtype_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_1319 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_TYPE _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_EXCEPTION _menhir_cell0_LBRACE -> _ -> _ -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_EXCEPTION (_menhir_stack, _menhir_s, _startpos__1_, _) = _menhir_stack in
        let (_1, _1_inlined2, _endpos__3_) = ((), _v, _endpos) in
        let _v = _menhir_action_472 _1 _1_inlined2 _endpos__3_ _startpos__1_ in
        _menhir_goto_ty_decl _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
    
    and _menhir_run_1323 : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_spec) _menhir_state -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.COMMA ->
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.IDENTIFIER _v_0 ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                _menhir_run_1323 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState1324
            | Tokens.RBRACE ->
                let (x, x_inlined1) = (_v, ()) in
                let _v = _menhir_action_210 x x_inlined1 in
                _menhir_goto_non_empty_trailing_list_COMMA_IDENTIFIER_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.RBRACE ->
            let x = _v in
            let _v = _menhir_action_209 x in
            _menhir_goto_non_empty_trailing_list_COMMA_IDENTIFIER_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_goto_non_empty_trailing_list_COMMA_IDENTIFIER_ : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_spec) _menhir_state -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        match _menhir_s with
        | MenhirState1322 ->
            _menhir_run_1326 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState1324 ->
            _menhir_run_1325 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_1326 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_TYPE _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ENUMERATION _menhir_cell0_LBRACE -> _ -> _ -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_ENUMERATION (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _1_inlined2, _endpos__3_) = ((), _v, _endpos) in
        let _v = _menhir_action_468 _1 _1_inlined2 _endpos__3_ _startpos__1_ in
        _menhir_goto_ty_decl _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
    
    and _menhir_run_1325 : type  ttv_stack. (ttv_stack, _menhir_box_spec) _menhir_cell1_IDENTIFIER -> _ -> _ -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, h, _, _) = _menhir_stack in
        let (_2, t) = ((), _v) in
        let _v = _menhir_action_211 _2 h t in
        _menhir_goto_non_empty_trailing_list_COMMA_IDENTIFIER_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_1359 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1360
            | Tokens.TYPE ->
                _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1360
            | Tokens.SETTER ->
                _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1360
            | Tokens.PRAGMA ->
                _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1360
            | Tokens.LET ->
                _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1360
            | Tokens.GETTER ->
                _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1360
            | Tokens.FUNC ->
                _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1360
            | Tokens.CONSTANT ->
                _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1360
            | Tokens.CONFIG ->
                _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1360
            | Tokens.EOF ->
                let _v_1 = _menhir_action_133 () in
                _menhir_run_1361 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.EQ ->
            let _menhir_s = MenhirState1362 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1346 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_MINUS as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1347
            | Tokens.TYPE ->
                _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1347
            | Tokens.SETTER ->
                _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1347
            | Tokens.PRAGMA ->
                _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1347
            | Tokens.LET ->
                _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1347
            | Tokens.GETTER ->
                _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1347
            | Tokens.FUNC ->
                _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1347
            | Tokens.CONSTANT ->
                _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1347
            | Tokens.CONFIG ->
                _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1347
            | Tokens.EOF ->
                let _v_1 = _menhir_action_133 () in
                _menhir_run_1348 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.EQ ->
            let _menhir_s = MenhirState1349 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1338 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_MINUS as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
            | Tokens.TYPE ->
                _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
            | Tokens.SETTER ->
                _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
            | Tokens.PRAGMA ->
                _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
            | Tokens.LET ->
                _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
            | Tokens.GETTER ->
                _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
            | Tokens.FUNC ->
                _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
            | Tokens.CONSTANT ->
                _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
            | Tokens.CONFIG ->
                _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1339
            | Tokens.EOF ->
                let _v_1 = _menhir_action_133 () in
                _menhir_run_1340 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.EQ ->
            let _menhir_s = MenhirState1341 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1335 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_TYPE _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _1 = _v in
        let _v = _menhir_action_467 _1 in
        _menhir_goto_ty_decl _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
    
    and _menhir_run_1304 : type  ttv_stack. ((((ttv_stack, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.BEGIN ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_BEGIN (_menhir_stack, _endpos) in
            let _menhir_s = MenhirState1305 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.END ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1296 : type  ttv_stack. ((((ttv_stack, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.BEGIN ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_BEGIN (_menhir_stack, _endpos) in
            let _menhir_s = MenhirState1297 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.END ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1284 : type  ttv_stack. ((((ttv_stack, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.BEGIN ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_BEGIN (_menhir_stack, _endpos) in
            let _menhir_s = MenhirState1285 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.END ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1271 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_LET _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.EQ ->
            let _menhir_s = MenhirState1272 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1265 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_LET _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.EQ ->
            let _menhir_s = MenhirState1266 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1254 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_LET _menhir_cell0_MINUS as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.EQ ->
            let _menhir_s = MenhirState1255 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1248 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_LET _menhir_cell0_MINUS as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.EQ ->
            let _menhir_s = MenhirState1249 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1239 : type  ttv_stack. ((((ttv_stack, _menhir_box_spec) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.BEGIN ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_BEGIN (_menhir_stack, _endpos) in
            let _menhir_s = MenhirState1240 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.END ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1218 : type  ttv_stack. ((((ttv_stack, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.RECURSELIMIT ->
            let _menhir_s = MenhirState1219 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.BEGIN ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_BEGIN (_menhir_stack, _endpos) in
            let _menhir_s = MenhirState1227 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.END ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1206 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
            let _menhir_s = MenhirState1207 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.IDENTIFIER _v ->
                _menhir_run_1200 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.RPAR ->
            let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, x, _, _) = _menhir_stack in
            let (_1, _2) = ((), _v) in
            let _v = _menhir_action_449 _1 _2 x in
            _menhir_goto_separated_nonempty_list_COMMA_typed_identifier_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_goto_separated_nonempty_list_COMMA_typed_identifier_ : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_spec) _menhir_state -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        match _menhir_s with
        | MenhirState1076 ->
            _menhir_run_1291 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | MenhirState1096 ->
            _menhir_run_1236 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | MenhirState1106 ->
            _menhir_run_1209 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | MenhirState1207 ->
            _menhir_run_1208 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState1203 ->
            _menhir_run_1204 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_1291 : type  ttv_stack. (((ttv_stack, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_typed_identifier_ (_menhir_stack, _menhir_s, _v) in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.EQ ->
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.IDENTIFIER _v ->
                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.COLON_COLON ->
                    let _menhir_s = MenhirState1295 in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.STRING ->
                        _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.REAL ->
                        _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.LPAR ->
                        _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.INTEGER ->
                        _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.IDENTIFIER _v ->
                        _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.BOOLEAN ->
                        _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.BITS ->
                        _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.BIT ->
                        _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.ARRAY ->
                        _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | _ ->
                        _eRR ())
                | Tokens.COLON ->
                    let _menhir_s = MenhirState1303 in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.STRING ->
                        _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.REAL ->
                        _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.LPAR ->
                        _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.INTEGER ->
                        _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.IDENTIFIER _v ->
                        _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.BOOLEAN ->
                        _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.BITS ->
                        _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.BIT ->
                        _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.ARRAY ->
                        _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | _ ->
                        _eRR ())
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1236 : type  ttv_stack. (((ttv_stack, _menhir_box_spec) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_typed_identifier_ (_menhir_stack, _menhir_s, _v) in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.ARROW ->
            let _menhir_s = MenhirState1238 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1209 : type  ttv_stack. (((ttv_stack, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_typed_identifier_ (_menhir_stack, _menhir_s, _v) in
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.BEGIN ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_BEGIN (_menhir_stack, _endpos) in
            let _menhir_s = MenhirState1211 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.END ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.ARROW ->
            let _menhir_s = MenhirState1217 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1208 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty -> _ -> _ -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, x, _, _) = _menhir_stack in
        let (_1, xs) = ((), _v) in
        let _v = _menhir_action_451 _1 _2_inlined1 x xs in
        _menhir_goto_separated_nonempty_list_COMMA_typed_identifier_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_1204 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty -> _ -> _ -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, x, _, _) = _menhir_stack in
        let (_1, xs) = ((), _v) in
        let _v = _menhir_action_452 _1 _2_inlined1 x xs in
        _menhir_goto_separated_nonempty_list_COMMA_typed_identifier_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_1202 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
            let _menhir_s = MenhirState1203 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.IDENTIFIER _v ->
                _menhir_run_1200 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.RPAR ->
            let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, x, _, _) = _menhir_stack in
            let (_1, _2) = ((), _v) in
            let _v = _menhir_action_450 _1 _2 x in
            _menhir_goto_separated_nonempty_list_COMMA_typed_identifier_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_1185 : type  ttv_stack. ((((ttv_stack, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.RECURSELIMIT ->
            let _menhir_s = MenhirState1186 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.BEGIN ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_BEGIN (_menhir_stack, _endpos) in
            let _menhir_s = MenhirState1194 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.END ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1176 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_CONSTANT _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.EQ ->
            let _menhir_s = MenhirState1177 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1170 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_CONSTANT _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.EQ ->
            let _menhir_s = MenhirState1171 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1159 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_CONSTANT _menhir_cell0_MINUS as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.EQ ->
            let _menhir_s = MenhirState1160 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1153 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_CONSTANT _menhir_cell0_MINUS as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.EQ ->
            let _menhir_s = MenhirState1154 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1146 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_CONFIG _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.EQ ->
            let _menhir_s = MenhirState1147 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1140 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_CONFIG _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.EQ ->
            let _menhir_s = MenhirState1141 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1129 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_CONFIG _menhir_cell0_MINUS as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.EQ ->
            let _menhir_s = MenhirState1130 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1123 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_CONFIG _menhir_cell0_MINUS as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.EQ ->
            let _menhir_s = MenhirState1124 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1099 : type  ttv_stack. ((((ttv_stack, _menhir_box_spec) _menhir_cell1_GETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.BEGIN ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_BEGIN (_menhir_stack, _endpos) in
            let _menhir_s = MenhirState1100 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.END ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1081 : type  ttv_stack. ((((ttv_stack, _menhir_box_spec) _menhir_cell1_SETTER _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.BEGIN ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_BEGIN (_menhir_stack, _endpos) in
            let _menhir_s = MenhirState1082 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.END ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_1070 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
            let _menhir_s = MenhirState1071 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.IDENTIFIER _v ->
                _menhir_run_1062 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.RBRACE ->
            let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, x, _, _) = _menhir_stack in
            let (_1, _2) = ((), _v) in
            let _v = _menhir_action_430 _1 _2 x in
            _menhir_goto_separated_nonempty_list_COMMA_opt_typed_identifier_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_1066 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
            let _menhir_s = MenhirState1067 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.IDENTIFIER _v ->
                _menhir_run_1062 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.RBRACE ->
            let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, x, _, _) = _menhir_stack in
            let (_1, _2) = ((), _v) in
            let _v = _menhir_action_431 _1 _2 x in
            _menhir_goto_separated_nonempty_list_COMMA_opt_typed_identifier_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_1051 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.COMMA ->
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.IDENTIFIER _v_0 ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                _menhir_run_1045 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState1052
            | Tokens.RBRACE ->
                let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, x, _, _) = _menhir_stack in
                let (_1, _2_inlined1, x_inlined1) = ((), _v, ()) in
                let _v = _menhir_action_225 _1 _2_inlined1 x x_inlined1 in
                _menhir_goto_non_empty_trailing_list_COMMA_typed_identifier_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.RBRACE ->
            let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, x, _, _) = _menhir_stack in
            let (_1, _2_inlined1) = ((), _v) in
            let _v = _menhir_action_224 _1 _2_inlined1 x in
            _menhir_goto_non_empty_trailing_list_COMMA_typed_identifier_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_goto_non_empty_trailing_list_COMMA_typed_identifier_ : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_spec) _menhir_state -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        match _menhir_s with
        | MenhirState1318 ->
            _menhir_run_1054 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | MenhirState1314 ->
            _menhir_run_1054 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | MenhirState1044 ->
            _menhir_run_1054 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | MenhirState1052 ->
            _menhir_run_1053 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState1048 ->
            _menhir_run_1049 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_1054 : type  ttv_stack. (ttv_stack _menhir_cell0_LBRACE as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let x = _v in
        let _v = _menhir_action_199 x in
        _menhir_goto_loption_non_empty_trailing_list_COMMA_typed_identifier__ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_goto_loption_non_empty_trailing_list_COMMA_typed_identifier__ : type  ttv_stack. (ttv_stack _menhir_cell0_LBRACE as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        match _menhir_s with
        | MenhirState1318 ->
            _menhir_run_1319 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState1314 ->
            _menhir_run_1315 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState1044 ->
            _menhir_run_1055 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_1053 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty -> _ -> _ -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, x, _, _) = _menhir_stack in
        let (_1, _2, t) = ((), (), _v) in
        let _v = _menhir_action_228 _1 _2 _2_inlined1 t x in
        _menhir_goto_non_empty_trailing_list_COMMA_typed_identifier_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_1049 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty -> _ -> _ -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, x, _, _) = _menhir_stack in
        let (_1, _2, t) = ((), (), _v) in
        let _v = _menhir_action_229 _1 _2 _2_inlined1 t x in
        _menhir_goto_non_empty_trailing_list_COMMA_typed_identifier_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_1047 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.COMMA ->
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.IDENTIFIER _v_0 ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                _menhir_run_1045 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState1048
            | Tokens.RBRACE ->
                let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, x, _, _) = _menhir_stack in
                let (_1, _2_inlined1, x_inlined1) = ((), _v, ()) in
                let _v = _menhir_action_227 _1 _2_inlined1 x x_inlined1 in
                _menhir_goto_non_empty_trailing_list_COMMA_typed_identifier_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.RBRACE ->
            let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, x, _, _) = _menhir_stack in
            let (_1, _2_inlined1) = ((), _v) in
            let _v = _menhir_action_226 _1 _2_inlined1 x in
            _menhir_goto_non_empty_trailing_list_COMMA_typed_identifier_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0705 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_VAR, ttv_result) _menhir_cell1_IDENTIFIER, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0706
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0706
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0706
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0706
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0706
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0706
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0706
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0706
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0706
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0706
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0706
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0706
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0706
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0706
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0706
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0706
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0706
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0706
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0706
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0706
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0706
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, x, _, _) = _menhir_stack in
                let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                let (_1, _endpos__2_, _2_inlined1, _1_inlined2, _2_inlined2) = ((), _endpos_0, (), (), _v) in
                let _v = _menhir_action_272 _1 _1_inlined1 _1_inlined2 _2_inlined1 _2_inlined2 _endpos__2_ _startpos__1_ x in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0686 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_WHEN, ttv_result) _menhir_cell1_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.ARROW ->
            let _menhir_s = MenhirState0687 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0391 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_VAR, ttv_result) _menhir_cell1_IDENTIFIER, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0392
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0392
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0392
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0392
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0392
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0392
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0392
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0392
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0392
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0392
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0392
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0392
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0392
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0392
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0392
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0392
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0392
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0392
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0392
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0392
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0392
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, x, _, _) = _menhir_stack in
                let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                let (_1, _endpos__2_, _2_inlined1, _1_inlined2, _2_inlined2) = ((), _endpos_0, (), (), _v) in
                let _v = _menhir_action_273 _1 _1_inlined1 _1_inlined2 _2_inlined1 _2_inlined2 _endpos__2_ _startpos__1_ x in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0383 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_LPAR, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_decl_item_ _menhir_cell0_RPAR -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        let MenhirCell0_RPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_decl_item_ (_menhir_stack, _, li) = _menhir_stack in
        let MenhirCell1_decl_item (_menhir_stack, _, x) = _menhir_stack in
        let MenhirCell1_LPAR (_menhir_stack, _menhir_s, _) = _menhir_stack in
        let (_2, _2_inlined1, _1_inlined1) = ((), _v, ()) in
        let _v = _menhir_action_007 _1_inlined1 _2 _2_inlined1 li x in
        _menhir_goto_decl_item _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
    
    and _menhir_run_0381 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_LPAR, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_decl_item_ _menhir_cell0_RPAR -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        let MenhirCell0_RPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_decl_item_ (_menhir_stack, _, li) = _menhir_stack in
        let MenhirCell1_decl_item (_menhir_stack, _, x) = _menhir_stack in
        let MenhirCell1_LPAR (_menhir_stack, _menhir_s, _) = _menhir_stack in
        let (_2, _2_inlined1, _1_inlined1) = ((), _v, ()) in
        let _v = _menhir_action_008 _1_inlined1 _2 _2_inlined1 li x in
        _menhir_goto_decl_item _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
    
    and _menhir_run_0375 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _1, _, _) = _menhir_stack in
        let (_2, _1_inlined1) = (_v, ()) in
        let _v = _menhir_action_003 _1 _1_inlined1 _2 in
        _menhir_goto_decl_item _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
    
    and _menhir_run_0373 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _1, _, _) = _menhir_stack in
        let (_2, _1_inlined1) = (_v, ()) in
        let _v = _menhir_action_004 _1 _1_inlined1 _2 in
        _menhir_goto_decl_item _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
    
    and _menhir_run_0369 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_MINUS -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        let MenhirCell1_MINUS (_menhir_stack, _menhir_s, _, _) = _menhir_stack in
        let (_1, _2, _1_inlined1) = ((), _v, ()) in
        let _v = _menhir_action_005 _1 _1_inlined1 _2 in
        _menhir_goto_decl_item _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
    
    and _menhir_run_0367 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_MINUS -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        let MenhirCell1_MINUS (_menhir_stack, _menhir_s, _, _) = _menhir_stack in
        let (_1, _2, _1_inlined1) = ((), _v, ()) in
        let _v = _menhir_action_006 _1 _1_inlined1 _2 in
        _menhir_goto_decl_item _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
    
    and _menhir_run_0341 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_ARBITRARY -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_ARBITRARY (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _1_inlined1, _endpos_ty_, ty) = ((), (), _endpos, _v) in
        let _v = _menhir_action_064 _1 _1_inlined1 _endpos_ty_ _startpos__1_ ty in
        _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_ty_ _startpos__1_ _v _menhir_s _tok
    
    and _menhir_run_0339 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_ARBITRARY -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_ARBITRARY (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _1_inlined1, _endpos_ty_, ty) = ((), (), _endpos, _v) in
        let _v = _menhir_action_065 _1 _1_inlined1 _endpos_ty_ _startpos__1_ ty in
        _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_ty_ _startpos__1_ _v _menhir_s _tok
    
    and _menhir_run_0334 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
            let _menhir_s = MenhirState0335 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING ->
                _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REAL ->
                _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INTEGER ->
                _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOLEAN ->
                _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITS ->
                _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BIT ->
                _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARRAY ->
                _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.RPAR ->
            let x = _v in
            let _v = _menhir_action_447 x in
            _menhir_goto_separated_nonempty_list_COMMA_ty_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_goto_separated_nonempty_list_COMMA_ty_ : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        match _menhir_s with
        | MenhirState0019 ->
            _menhir_run_0337 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0335 ->
            _menhir_run_0336 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0337 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_LPAR -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1_inlined1, _endpos__3_) = (_v, _endpos) in
        let _v = _menhir_action_464 _1_inlined1 _endpos__3_ _startpos__1_ in
        _menhir_goto_ty _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _v _menhir_s _tok
    
    and _menhir_run_0336 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_ty -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_ty (_menhir_stack, _menhir_s, x, _) = _menhir_stack in
        let xs = _v in
        let _v = _menhir_action_448 x xs in
        _menhir_goto_separated_nonempty_list_COMMA_ty_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_0309 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, sub_expr, _startpos_sub_expr_, _) = _menhir_stack in
        let (_2, _endpos_ty_, ty) = ((), _endpos, _v) in
        let _v = _menhir_action_112 _2 _endpos_ty_ _startpos_sub_expr_ sub_expr ty in
        _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_ty_ _startpos_sub_expr_ _v _menhir_s _tok
    
    and _menhir_run_0225 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_ARBITRARY -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_ARBITRARY (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _1_inlined1, _endpos_ty_, ty) = ((), (), _endpos, _v) in
        let _v = _menhir_action_119 _1 _1_inlined1 _endpos_ty_ _startpos__1_ ty in
        _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_ty_ _startpos__1_ _v _menhir_s _tok
    
    and _menhir_run_0223 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_ARBITRARY -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_ARBITRARY (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _1_inlined1, _endpos_ty_, ty) = ((), (), _endpos, _v) in
        let _v = _menhir_action_120 _1 _1_inlined1 _endpos_ty_ _startpos__1_ ty in
        _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_ty_ _startpos__1_ _v _menhir_s _tok
    
    and _menhir_run_0168 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_AS -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_AS (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _menhir_s, sub_expr, _startpos_sub_expr_, _) = _menhir_stack in
        let (_2, _endpos_ty_, ty) = ((), _endpos, _v) in
        let _v = _menhir_action_057 _2 _endpos_ty_ _startpos_sub_expr_ sub_expr ty in
        _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_ty_ _startpos_sub_expr_ _v _menhir_s _tok
    
    and _menhir_run_0147 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_LBRACKET, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_slice_ _menhir_cell0_RBRACKET _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.COMMA ->
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LBRACKET ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0148
            | Tokens.RBRACE ->
                let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
                let MenhirCell0_RBRACKET (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_slice_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
                let MenhirCell1_LBRACKET (_menhir_stack, _menhir_s) = _menhir_stack in
                let (_2_inlined1, _1_inlined2, x_inlined1) = (_v, (), ()) in
                let _v = _menhir_action_217 _1_inlined1 _1_inlined2 _2_inlined1 x x_inlined1 in
                _menhir_goto_non_empty_trailing_list_COMMA_bitfield_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.RBRACE ->
            let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
            let MenhirCell0_RBRACKET (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_separated_nonempty_list_COMMA_slice_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
            let MenhirCell1_LBRACKET (_menhir_stack, _menhir_s) = _menhir_stack in
            let (_2_inlined1, _1_inlined2) = (_v, ()) in
            let _v = _menhir_action_216 _1_inlined1 _1_inlined2 _2_inlined1 x in
            _menhir_goto_non_empty_trailing_list_COMMA_bitfield_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0123 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_LBRACKET (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0123 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.COLON ->
            _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_goto_non_empty_trailing_list_COMMA_bitfield_ : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        match _menhir_s with
        | MenhirState0148 ->
            _menhir_run_0149 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0144 ->
            _menhir_run_0145 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0133 ->
            _menhir_run_0134 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0131 ->
            _menhir_run_0132 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0122 ->
            _menhir_run_0128 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | MenhirState0127 ->
            _menhir_run_0128 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0149 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_LBRACKET, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_slice_ _menhir_cell0_RBRACKET _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_ty -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
        let MenhirCell0_RBRACKET (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_slice_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell1_LBRACKET (_menhir_stack, _menhir_s) = _menhir_stack in
        let (_2, _1_inlined2, t) = ((), (), _v) in
        let _v = _menhir_action_222 _1_inlined1 _1_inlined2 _2 _2_inlined1 t x in
        _menhir_goto_non_empty_trailing_list_COMMA_bitfield_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_0145 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_LBRACKET, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_slice_ _menhir_cell0_RBRACKET _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_ty -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_ty (_menhir_stack, _, _2_inlined1, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
        let MenhirCell0_RBRACKET (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_slice_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell1_LBRACKET (_menhir_stack, _menhir_s) = _menhir_stack in
        let (_2, _1_inlined2, t) = ((), (), _v) in
        let _v = _menhir_action_223 _1_inlined1 _1_inlined2 _2 _2_inlined1 t x in
        _menhir_goto_non_empty_trailing_list_COMMA_bitfield_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_0134 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LBRACKET, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_slice_ _menhir_cell0_RBRACKET _menhir_cell0_IDENTIFIER -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
        let MenhirCell0_RBRACKET (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_slice_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell1_LBRACKET (_menhir_stack, _menhir_s) = _menhir_stack in
        let (_2, t) = ((), _v) in
        let _v = _menhir_action_220 _1_inlined1 _2 t x in
        _menhir_goto_non_empty_trailing_list_COMMA_bitfield_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_0132 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_LBRACKET, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_slice_ _menhir_cell0_RBRACKET _menhir_cell0_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_loption_non_empty_trailing_list_COMMA_bitfield__ _menhir_cell0_RBRACE -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_loption_non_empty_trailing_list_COMMA_bitfield__ (_menhir_stack, _, _1_inlined3) = _menhir_stack in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
        let MenhirCell0_RBRACKET (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_slice_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell1_LBRACKET (_menhir_stack, _menhir_s) = _menhir_stack in
        let (_2, t) = ((), _v) in
        let _v = _menhir_action_221 _1_inlined1 _1_inlined3 _2 t x in
        _menhir_goto_non_empty_trailing_list_COMMA_bitfield_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_0128 : type  ttv_stack ttv_result. (ttv_stack _menhir_cell0_LBRACE as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let x = _v in
        let _v = _menhir_action_197 x in
        _menhir_goto_loption_non_empty_trailing_list_COMMA_bitfield__ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_goto_loption_non_empty_trailing_list_COMMA_bitfield__ : type  ttv_stack ttv_result. (ttv_stack _menhir_cell0_LBRACE as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        match _menhir_s with
        | MenhirState0122 ->
            _menhir_run_0165 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0127 ->
            _menhir_run_0129 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0165 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_BITS _menhir_cell0_LPAR, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_RPAR _menhir_cell0_LBRACE -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let (_1_inlined1, _endpos__3_) = (_v, _endpos) in
        let _v = _menhir_action_193 _1_inlined1 in
        _menhir_goto_loption_bitfields_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _v _tok
    
    and _menhir_goto_loption_bitfields_ : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_BITS _menhir_cell0_LPAR, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_RPAR -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_BITS (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _1_inlined2, _endpos__1_inlined2_) = ((), _v, _endpos) in
        let _v = _menhir_action_462 _1 _1_inlined2 _endpos__1_inlined2_ _startpos__1_ x in
        _menhir_goto_ty _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__1_inlined2_ _v _menhir_s _tok
    
    and _menhir_run_0129 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_LBRACKET, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_slice_ _menhir_cell0_RBRACKET _menhir_cell0_IDENTIFIER _menhir_cell0_LBRACE as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.COMMA ->
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LBRACKET ->
                let _menhir_stack = MenhirCell1_loption_non_empty_trailing_list_COMMA_bitfield__ (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_RBRACE (_menhir_stack, _endpos) in
                _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0131
            | Tokens.RBRACE ->
                let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
                let MenhirCell0_RBRACKET (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_slice_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
                let MenhirCell1_LBRACKET (_menhir_stack, _menhir_s) = _menhir_stack in
                let (_1_inlined3, x_inlined1) = (_v, ()) in
                let _v = _menhir_action_215 _1_inlined1 _1_inlined3 x x_inlined1 in
                _menhir_goto_non_empty_trailing_list_COMMA_bitfield_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.RBRACE ->
            let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
            let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
            let MenhirCell0_RBRACKET (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_separated_nonempty_list_COMMA_slice_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
            let MenhirCell1_LBRACKET (_menhir_stack, _menhir_s) = _menhir_stack in
            let _1_inlined3 = _v in
            let _v = _menhir_action_214 _1_inlined1 _1_inlined3 x in
            _menhir_goto_non_empty_trailing_list_COMMA_bitfield_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0143 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_LBRACKET, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_slice_ _menhir_cell0_RBRACKET _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.COMMA ->
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LBRACKET ->
                let _menhir_stack = MenhirCell1_ty (_menhir_stack, _menhir_s, _v, _endpos) in
                _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0144
            | Tokens.RBRACE ->
                let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
                let MenhirCell0_RBRACKET (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_slice_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
                let MenhirCell1_LBRACKET (_menhir_stack, _menhir_s) = _menhir_stack in
                let (_2_inlined1, _1_inlined2, x_inlined1) = (_v, (), ()) in
                let _v = _menhir_action_219 _1_inlined1 _1_inlined2 _2_inlined1 x x_inlined1 in
                _menhir_goto_non_empty_trailing_list_COMMA_bitfield_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.RBRACE ->
            let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
            let MenhirCell0_RBRACKET (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_separated_nonempty_list_COMMA_slice_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
            let MenhirCell1_LBRACKET (_menhir_stack, _menhir_s) = _menhir_stack in
            let (_2_inlined1, _1_inlined2) = (_v, ()) in
            let _v = _menhir_action_218 _1_inlined1 _1_inlined2 _2_inlined1 x in
            _menhir_goto_non_empty_trailing_list_COMMA_bitfield_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0142 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_ARRAY, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_RBRACKET -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_RBRACKET (_menhir_stack, _, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
        let MenhirCell1_ARRAY (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _endpos_t_, t, _3) = ((), _endpos, _v, ()) in
        let _v = _menhir_action_466 _1 _3 _endpos_t_ _startpos__1_ t x in
        _menhir_goto_ty _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_t_ _v _menhir_s _tok
    
    and _menhir_run_1363 : type  ttv_stack. (((ttv_stack, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1363
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1363
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState1363, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1364
            | Tokens.TYPE ->
                _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1364
            | Tokens.SETTER ->
                _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1364
            | Tokens.PRAGMA ->
                _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1364
            | Tokens.LET ->
                _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1364
            | Tokens.GETTER ->
                _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1364
            | Tokens.FUNC ->
                _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1364
            | Tokens.CONSTANT ->
                _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1364
            | Tokens.CONFIG ->
                _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1364
            | Tokens.EOF ->
                let _v_1 = _menhir_action_133 () in
                _menhir_run_1365 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1363
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1363
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1363
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1363
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1363
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1363
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1363
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1363
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1363
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1363
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1363
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1363
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1363
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1363
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1363
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1363
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1363
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1363
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1363
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1363
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1363
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1363
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1363
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1363
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1363
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1363
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1363
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1363
        | _ ->
            _eRR ()
    
    and _menhir_run_1355 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1355
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1355
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState1355, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1356
            | Tokens.TYPE ->
                _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1356
            | Tokens.SETTER ->
                _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1356
            | Tokens.PRAGMA ->
                _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1356
            | Tokens.LET ->
                _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1356
            | Tokens.GETTER ->
                _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1356
            | Tokens.FUNC ->
                _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1356
            | Tokens.CONSTANT ->
                _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1356
            | Tokens.CONFIG ->
                _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1356
            | Tokens.EOF ->
                let _v_1 = _menhir_action_133 () in
                _menhir_run_1357 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1355
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1355
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1355
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1355
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1355
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1355
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1355
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1355
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1355
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1355
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1355
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1355
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1355
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1355
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1355
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1355
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1355
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1355
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1355
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1355
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1355
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1355
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1355
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1355
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1355
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1355
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1355
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1355
        | _ ->
            _eRR ()
    
    and _menhir_run_1350 : type  ttv_stack. (((ttv_stack, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1350
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1350
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState1350, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1351
            | Tokens.TYPE ->
                _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1351
            | Tokens.SETTER ->
                _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1351
            | Tokens.PRAGMA ->
                _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1351
            | Tokens.LET ->
                _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1351
            | Tokens.GETTER ->
                _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1351
            | Tokens.FUNC ->
                _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1351
            | Tokens.CONSTANT ->
                _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1351
            | Tokens.CONFIG ->
                _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1351
            | Tokens.EOF ->
                let _v_1 = _menhir_action_133 () in
                _menhir_run_1352 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1350
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1350
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1350
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1350
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1350
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1350
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1350
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1350
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1350
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1350
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1350
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1350
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1350
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1350
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1350
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1350
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1350
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1350
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1350
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1350
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1350
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1350
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1350
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1350
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1350
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1350
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1350
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1350
        | _ ->
            _eRR ()
    
    and _menhir_run_1342 : type  ttv_stack. (((ttv_stack, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1342
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1342
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState1342, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
            | Tokens.TYPE ->
                _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
            | Tokens.SETTER ->
                _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
            | Tokens.PRAGMA ->
                _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
            | Tokens.LET ->
                _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
            | Tokens.GETTER ->
                _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
            | Tokens.FUNC ->
                _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
            | Tokens.CONSTANT ->
                _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
            | Tokens.CONFIG ->
                _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1343
            | Tokens.EOF ->
                let _v_1 = _menhir_action_133 () in
                _menhir_run_1344 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1342
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1342
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1342
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1342
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1342
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1342
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1342
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1342
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1342
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1342
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1342
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1342
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1342
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1342
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1342
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1342
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1342
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1342
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1342
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1342
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1342
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1342
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1342
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1342
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1342
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1342
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1342
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1342
        | _ ->
            _eRR ()
    
    and _menhir_run_1273 : type  ttv_stack. (((ttv_stack, _menhir_box_spec) _menhir_cell1_LET _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1273
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1273
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState1273, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
            | Tokens.TYPE ->
                _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
            | Tokens.SETTER ->
                _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
            | Tokens.PRAGMA ->
                _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
            | Tokens.LET ->
                _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
            | Tokens.GETTER ->
                _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
            | Tokens.FUNC ->
                _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
            | Tokens.CONSTANT ->
                _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
            | Tokens.CONFIG ->
                _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1274
            | Tokens.EOF ->
                let _v_1 = _menhir_action_133 () in
                _menhir_run_1275 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1273
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1273
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1273
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1273
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1273
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1273
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1273
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1273
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1273
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1273
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1273
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1273
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1273
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1273
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1273
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1273
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1273
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1273
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1273
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1273
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1273
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1273
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1273
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1273
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1273
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1273
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1273
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1273
        | _ ->
            _eRR ()
    
    and _menhir_run_1267 : type  ttv_stack. (((ttv_stack, _menhir_box_spec) _menhir_cell1_LET _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1267
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1267
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState1267, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1268
            | Tokens.TYPE ->
                _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1268
            | Tokens.SETTER ->
                _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1268
            | Tokens.PRAGMA ->
                _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1268
            | Tokens.LET ->
                _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1268
            | Tokens.GETTER ->
                _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1268
            | Tokens.FUNC ->
                _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1268
            | Tokens.CONSTANT ->
                _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1268
            | Tokens.CONFIG ->
                _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1268
            | Tokens.EOF ->
                let _v_1 = _menhir_action_133 () in
                _menhir_run_1269 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1267
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1267
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1267
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1267
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1267
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1267
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1267
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1267
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1267
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1267
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1267
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1267
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1267
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1267
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1267
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1267
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1267
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1267
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1267
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1267
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1267
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1267
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1267
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1267
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1267
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1267
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1267
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1267
        | _ ->
            _eRR ()
    
    and _menhir_run_1261 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_LET _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1261
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1261
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState1261, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1262
            | Tokens.TYPE ->
                _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1262
            | Tokens.SETTER ->
                _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1262
            | Tokens.PRAGMA ->
                _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1262
            | Tokens.LET ->
                _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1262
            | Tokens.GETTER ->
                _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1262
            | Tokens.FUNC ->
                _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1262
            | Tokens.CONSTANT ->
                _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1262
            | Tokens.CONFIG ->
                _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1262
            | Tokens.EOF ->
                let _v_1 = _menhir_action_133 () in
                _menhir_run_1263 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1261
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1261
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1261
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1261
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1261
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1261
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1261
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1261
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1261
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1261
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1261
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1261
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1261
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1261
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1261
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1261
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1261
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1261
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1261
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1261
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1261
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1261
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1261
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1261
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1261
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1261
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1261
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1261
        | _ ->
            _eRR ()
    
    and _menhir_run_1256 : type  ttv_stack. (((ttv_stack, _menhir_box_spec) _menhir_cell1_LET _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1256
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1256
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState1256, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1257
            | Tokens.TYPE ->
                _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1257
            | Tokens.SETTER ->
                _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1257
            | Tokens.PRAGMA ->
                _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1257
            | Tokens.LET ->
                _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1257
            | Tokens.GETTER ->
                _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1257
            | Tokens.FUNC ->
                _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1257
            | Tokens.CONSTANT ->
                _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1257
            | Tokens.CONFIG ->
                _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1257
            | Tokens.EOF ->
                let _v_1 = _menhir_action_133 () in
                _menhir_run_1258 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1256
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1256
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1256
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1256
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1256
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1256
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1256
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1256
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1256
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1256
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1256
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1256
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1256
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1256
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1256
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1256
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1256
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1256
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1256
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1256
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1256
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1256
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1256
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1256
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1256
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1256
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1256
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1256
        | _ ->
            _eRR ()
    
    and _menhir_run_1250 : type  ttv_stack. (((ttv_stack, _menhir_box_spec) _menhir_cell1_LET _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1250
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1250
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState1250, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1251
            | Tokens.TYPE ->
                _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1251
            | Tokens.SETTER ->
                _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1251
            | Tokens.PRAGMA ->
                _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1251
            | Tokens.LET ->
                _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1251
            | Tokens.GETTER ->
                _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1251
            | Tokens.FUNC ->
                _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1251
            | Tokens.CONSTANT ->
                _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1251
            | Tokens.CONFIG ->
                _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1251
            | Tokens.EOF ->
                let _v_1 = _menhir_action_133 () in
                _menhir_run_1252 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1250
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1250
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1250
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1250
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1250
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1250
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1250
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1250
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1250
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1250
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1250
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1250
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1250
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1250
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1250
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1250
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1250
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1250
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1250
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1250
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1250
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1250
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1250
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1250
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1250
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1250
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1250
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1250
        | _ ->
            _eRR ()
    
    and _menhir_run_1220 : type  ttv_stack. (((((ttv_stack, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_separated_nonempty_list_COMMA_typed_identifier_ _menhir_cell0_RPAR, _menhir_box_spec) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1220
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1220
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1220
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1220
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1220
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1220
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1220
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1220
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1220
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1220
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1220
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1220
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1220
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1220
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1220
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1220
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1220
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1220
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1220
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1220
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1220
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1220
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1220
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1220
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1220
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1220
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1220
        | Tokens.BEGIN ->
            let _menhir_s = MenhirState1220 in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_BEGIN (_menhir_stack, _menhir_s, _endpos) in
            let _menhir_s = MenhirState1221 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.END ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1220
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1220
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1220
        | _ ->
            _eRR ()
    
    and _menhir_run_1187 : type  ttv_stack. (((((ttv_stack, _menhir_box_spec) _menhir_cell1_FUNC _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_loption_braced_nclist_opt_typed_identifier___ _menhir_cell0_LPAR, _menhir_box_spec) _menhir_cell1_RPAR, _menhir_box_spec) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1187
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1187
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1187
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1187
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1187
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1187
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1187
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1187
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1187
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1187
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1187
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1187
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1187
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1187
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1187
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1187
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1187
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1187
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1187
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1187
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1187
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1187
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1187
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1187
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1187
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1187
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1187
        | Tokens.BEGIN ->
            let _menhir_s = MenhirState1187 in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_BEGIN (_menhir_stack, _menhir_s, _endpos) in
            let _menhir_s = MenhirState1188 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.END ->
                _menhir_run_0492 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1187
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1187
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1187
        | _ ->
            _eRR ()
    
    and _menhir_run_1178 : type  ttv_stack. (((ttv_stack, _menhir_box_spec) _menhir_cell1_CONSTANT _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1178
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1178
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState1178, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1179
            | Tokens.TYPE ->
                _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1179
            | Tokens.SETTER ->
                _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1179
            | Tokens.PRAGMA ->
                _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1179
            | Tokens.LET ->
                _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1179
            | Tokens.GETTER ->
                _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1179
            | Tokens.FUNC ->
                _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1179
            | Tokens.CONSTANT ->
                _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1179
            | Tokens.CONFIG ->
                _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1179
            | Tokens.EOF ->
                let _v_1 = _menhir_action_133 () in
                _menhir_run_1180 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1178
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1178
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1178
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1178
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1178
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1178
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1178
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1178
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1178
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1178
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1178
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1178
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1178
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1178
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1178
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1178
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1178
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1178
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1178
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1178
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1178
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1178
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1178
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1178
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1178
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1178
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1178
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1178
        | _ ->
            _eRR ()
    
    and _menhir_run_1172 : type  ttv_stack. (((ttv_stack, _menhir_box_spec) _menhir_cell1_CONSTANT _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1172
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1172
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState1172, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1173
            | Tokens.TYPE ->
                _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1173
            | Tokens.SETTER ->
                _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1173
            | Tokens.PRAGMA ->
                _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1173
            | Tokens.LET ->
                _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1173
            | Tokens.GETTER ->
                _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1173
            | Tokens.FUNC ->
                _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1173
            | Tokens.CONSTANT ->
                _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1173
            | Tokens.CONFIG ->
                _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1173
            | Tokens.EOF ->
                let _v_1 = _menhir_action_133 () in
                _menhir_run_1174 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1172
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1172
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1172
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1172
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1172
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1172
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1172
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1172
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1172
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1172
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1172
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1172
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1172
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1172
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1172
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1172
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1172
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1172
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1172
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1172
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1172
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1172
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1172
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1172
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1172
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1172
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1172
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1172
        | _ ->
            _eRR ()
    
    and _menhir_run_1166 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_CONSTANT _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1166
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1166
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState1166, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1167
            | Tokens.TYPE ->
                _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1167
            | Tokens.SETTER ->
                _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1167
            | Tokens.PRAGMA ->
                _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1167
            | Tokens.LET ->
                _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1167
            | Tokens.GETTER ->
                _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1167
            | Tokens.FUNC ->
                _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1167
            | Tokens.CONSTANT ->
                _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1167
            | Tokens.CONFIG ->
                _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1167
            | Tokens.EOF ->
                let _v_1 = _menhir_action_133 () in
                _menhir_run_1168 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1166
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1166
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1166
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1166
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1166
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1166
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1166
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1166
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1166
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1166
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1166
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1166
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1166
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1166
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1166
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1166
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1166
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1166
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1166
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1166
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1166
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1166
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1166
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1166
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1166
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1166
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1166
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1166
        | _ ->
            _eRR ()
    
    and _menhir_run_1161 : type  ttv_stack. (((ttv_stack, _menhir_box_spec) _menhir_cell1_CONSTANT _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1161
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1161
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState1161, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1162
            | Tokens.TYPE ->
                _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1162
            | Tokens.SETTER ->
                _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1162
            | Tokens.PRAGMA ->
                _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1162
            | Tokens.LET ->
                _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1162
            | Tokens.GETTER ->
                _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1162
            | Tokens.FUNC ->
                _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1162
            | Tokens.CONSTANT ->
                _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1162
            | Tokens.CONFIG ->
                _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1162
            | Tokens.EOF ->
                let _v_1 = _menhir_action_133 () in
                _menhir_run_1163 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1161
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1161
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1161
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1161
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1161
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1161
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1161
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1161
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1161
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1161
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1161
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1161
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1161
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1161
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1161
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1161
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1161
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1161
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1161
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1161
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1161
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1161
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1161
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1161
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1161
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1161
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1161
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1161
        | _ ->
            _eRR ()
    
    and _menhir_run_1155 : type  ttv_stack. (((ttv_stack, _menhir_box_spec) _menhir_cell1_CONSTANT _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1155
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1155
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState1155, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1156
            | Tokens.TYPE ->
                _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1156
            | Tokens.SETTER ->
                _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1156
            | Tokens.PRAGMA ->
                _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1156
            | Tokens.LET ->
                _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1156
            | Tokens.GETTER ->
                _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1156
            | Tokens.FUNC ->
                _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1156
            | Tokens.CONSTANT ->
                _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1156
            | Tokens.CONFIG ->
                _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1156
            | Tokens.EOF ->
                let _v_1 = _menhir_action_133 () in
                _menhir_run_1157 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1155
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1155
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1155
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1155
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1155
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1155
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1155
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1155
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1155
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1155
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1155
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1155
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1155
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1155
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1155
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1155
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1155
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1155
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1155
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1155
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1155
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1155
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1155
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1155
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1155
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1155
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1155
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1155
        | _ ->
            _eRR ()
    
    and _menhir_run_1148 : type  ttv_stack. (((ttv_stack, _menhir_box_spec) _menhir_cell1_CONFIG _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState1148, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1149
            | Tokens.TYPE ->
                _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1149
            | Tokens.SETTER ->
                _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1149
            | Tokens.PRAGMA ->
                _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1149
            | Tokens.LET ->
                _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1149
            | Tokens.GETTER ->
                _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1149
            | Tokens.FUNC ->
                _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1149
            | Tokens.CONSTANT ->
                _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1149
            | Tokens.CONFIG ->
                _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1149
            | Tokens.EOF ->
                let _v_1 = _menhir_action_133 () in
                _menhir_run_1150 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1148
        | _ ->
            _eRR ()
    
    and _menhir_run_1142 : type  ttv_stack. (((ttv_stack, _menhir_box_spec) _menhir_cell1_CONFIG _menhir_cell0_IDENTIFIER, _menhir_box_spec) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState1142, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1143
            | Tokens.TYPE ->
                _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1143
            | Tokens.SETTER ->
                _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1143
            | Tokens.PRAGMA ->
                _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1143
            | Tokens.LET ->
                _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1143
            | Tokens.GETTER ->
                _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1143
            | Tokens.FUNC ->
                _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1143
            | Tokens.CONSTANT ->
                _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1143
            | Tokens.CONFIG ->
                _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1143
            | Tokens.EOF ->
                let _v_1 = _menhir_action_133 () in
                _menhir_run_1144 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1142
        | _ ->
            _eRR ()
    
    and _menhir_run_1136 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_CONFIG _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1136
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1136
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState1136, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1137
            | Tokens.TYPE ->
                _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1137
            | Tokens.SETTER ->
                _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1137
            | Tokens.PRAGMA ->
                _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1137
            | Tokens.LET ->
                _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1137
            | Tokens.GETTER ->
                _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1137
            | Tokens.FUNC ->
                _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1137
            | Tokens.CONSTANT ->
                _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1137
            | Tokens.CONFIG ->
                _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1137
            | Tokens.EOF ->
                let _v_1 = _menhir_action_133 () in
                _menhir_run_1138 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1136
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1136
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1136
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1136
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1136
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1136
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1136
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1136
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1136
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1136
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1136
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1136
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1136
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1136
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1136
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1136
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1136
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1136
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1136
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1136
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1136
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1136
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1136
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1136
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1136
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1136
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1136
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1136
        | _ ->
            _eRR ()
    
    and _menhir_run_1131 : type  ttv_stack. (((ttv_stack, _menhir_box_spec) _menhir_cell1_CONFIG _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1131
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1131
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState1131, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1132
            | Tokens.TYPE ->
                _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1132
            | Tokens.SETTER ->
                _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1132
            | Tokens.PRAGMA ->
                _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1132
            | Tokens.LET ->
                _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1132
            | Tokens.GETTER ->
                _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1132
            | Tokens.FUNC ->
                _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1132
            | Tokens.CONSTANT ->
                _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1132
            | Tokens.CONFIG ->
                _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1132
            | Tokens.EOF ->
                let _v_1 = _menhir_action_133 () in
                _menhir_run_1133 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1131
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1131
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1131
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1131
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1131
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1131
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1131
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1131
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1131
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1131
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1131
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1131
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1131
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1131
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1131
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1131
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1131
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1131
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1131
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1131
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1131
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1131
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1131
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1131
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1131
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1131
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1131
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1131
        | _ ->
            _eRR ()
    
    and _menhir_run_1125 : type  ttv_stack. (((ttv_stack, _menhir_box_spec) _menhir_cell1_CONFIG _menhir_cell0_MINUS, _menhir_box_spec) _menhir_cell1_ty as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1125
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1125
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState1125, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1126
            | Tokens.TYPE ->
                _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1126
            | Tokens.SETTER ->
                _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1126
            | Tokens.PRAGMA ->
                _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1126
            | Tokens.LET ->
                _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1126
            | Tokens.GETTER ->
                _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1126
            | Tokens.FUNC ->
                _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1126
            | Tokens.CONSTANT ->
                _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1126
            | Tokens.CONFIG ->
                _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1126
            | Tokens.EOF ->
                let _v_1 = _menhir_action_133 () in
                _menhir_run_1127 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1125
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1125
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1125
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1125
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1125
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1125
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1125
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1125
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1125
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1125
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1125
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1125
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1125
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1125
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1125
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1125
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1125
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1125
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1125
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1125
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1125
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1125
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1125
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1125
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1125
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1125
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1125
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1125
        | _ ->
            _eRR ()
    
    and _menhir_run_1119 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_CONFIG _menhir_cell0_MINUS as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1119
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1119
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState1119, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
            | Tokens.TYPE ->
                _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
            | Tokens.SETTER ->
                _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
            | Tokens.PRAGMA ->
                _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
            | Tokens.LET ->
                _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
            | Tokens.GETTER ->
                _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
            | Tokens.FUNC ->
                _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
            | Tokens.CONSTANT ->
                _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
            | Tokens.CONFIG ->
                _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1120
            | Tokens.EOF ->
                let _v_1 = _menhir_action_133 () in
                _menhir_run_1121 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1119
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1119
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1119
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1119
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1119
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1119
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1119
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1119
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1119
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1119
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1119
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1119
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1119
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1119
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1119
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1119
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1119
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1119
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1119
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1119
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1119
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1119
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1119
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1119
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1119
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1119
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1119
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1119
        | _ ->
            _eRR ()
    
    and _menhir_run_1114 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_CONSTANT _menhir_cell0_MINUS as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1114
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1114
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState1114, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1115
            | Tokens.TYPE ->
                _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1115
            | Tokens.SETTER ->
                _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1115
            | Tokens.PRAGMA ->
                _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1115
            | Tokens.LET ->
                _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1115
            | Tokens.GETTER ->
                _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1115
            | Tokens.FUNC ->
                _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1115
            | Tokens.CONSTANT ->
                _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1115
            | Tokens.CONFIG ->
                _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1115
            | Tokens.EOF ->
                let _v_1 = _menhir_action_133 () in
                _menhir_run_1151 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1114
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1114
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1114
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1114
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1114
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1114
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1114
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1114
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1114
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1114
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1114
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1114
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1114
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1114
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1114
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1114
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1114
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1114
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1114
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1114
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1114
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1114
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1114
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1114
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1114
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1114
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1114
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1114
        | _ ->
            _eRR ()
    
    and _menhir_run_1091 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_LET _menhir_cell0_MINUS as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1091
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1091
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState1091, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1092
            | Tokens.TYPE ->
                _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1092
            | Tokens.SETTER ->
                _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1092
            | Tokens.PRAGMA ->
                _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1092
            | Tokens.LET ->
                _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1092
            | Tokens.GETTER ->
                _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1092
            | Tokens.FUNC ->
                _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1092
            | Tokens.CONSTANT ->
                _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1092
            | Tokens.CONFIG ->
                _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1092
            | Tokens.EOF ->
                let _v_1 = _menhir_action_133 () in
                _menhir_run_1246 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1091
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1091
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1091
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1091
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1091
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1091
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1091
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1091
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1091
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1091
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1091
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1091
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1091
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1091
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1091
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1091
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1091
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1091
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1091
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1091
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1091
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1091
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1091
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1091
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1091
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1091
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1091
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1091
        | _ ->
            _eRR ()
    
    and _menhir_run_1037 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_VAR _menhir_cell0_MINUS as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1037
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1037
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState1037, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1038
            | Tokens.TYPE ->
                _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1038
            | Tokens.SETTER ->
                _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1038
            | Tokens.PRAGMA ->
                _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1038
            | Tokens.LET ->
                _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1038
            | Tokens.GETTER ->
                _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1038
            | Tokens.FUNC ->
                _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1038
            | Tokens.CONSTANT ->
                _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1038
            | Tokens.CONFIG ->
                _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1038
            | Tokens.EOF ->
                let _v_1 = _menhir_action_133 () in
                _menhir_run_1336 _menhir_stack _v_1
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1037
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1037
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1037
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1037
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1037
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1037
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1037
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1037
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1037
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1037
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1037
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1037
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1037
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1037
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1037
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1037
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1037
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1037
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1037
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1037
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1037
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1037
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1037
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1037
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1037
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1037
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1037
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1037
        | _ ->
            _eRR ()
    
    and _menhir_run_1030 : type  ttv_stack. ((ttv_stack, _menhir_box_opn) _menhir_cell1_lexpr as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell1_lexpr (_menhir_stack, _, le, _startpos_le_) = _menhir_stack in
                let (_2, _2_inlined1, e, _endpos__2_inlined2_) = ((), (), _v, _endpos_0) in
                let _v = _menhir_action_364 _2 _2_inlined1 _endpos__2_inlined2_ _startpos_le_ e le in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1030
        | _ ->
            _eRR ()
    
    and _menhir_run_1024 : type  ttv_stack. (((((ttv_stack, _menhir_box_opn) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_REPEAT, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1024
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1024
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined2, _) = _menhir_stack in
                let MenhirCell0_REPEAT (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
                let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
                let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_ARROBASE_LOOPLIMIT (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                let (_1, _2, _endpos__2_inlined1_, _3, _5, cond) = ((), (), _endpos_0, (), (), _v) in
                let _v = _menhir_action_394 _1 _1_inlined2 _2 _3 _5 _endpos__2_inlined1_ _startpos__1_ cond x in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1024
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1024
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1024
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1024
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1024
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1024
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1024
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1024
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1024
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1024
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1024
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1024
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1024
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1024
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1024
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1024
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1024
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1024
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1024
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1024
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1024
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1024
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1024
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1024
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1024
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1024
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1024
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1024
        | _ ->
            _eRR ()
    
    and _menhir_run_1016 : type  ttv_stack. ((((ttv_stack, _menhir_box_opn) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_WHILE as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1016
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1016
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1016
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1016
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1016
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1016
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1016
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1016
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1016
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1016
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1016
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1016
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1016
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1016
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1016
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1016
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1016
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1016
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1016
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1016
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1016
        | Tokens.DO ->
            let _menhir_stack = MenhirCell1_DO (_menhir_stack, MenhirState1016) in
            let _menhir_s = MenhirState1017 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1016
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1016
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1016
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1016
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1016
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1016
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1016
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1016
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1016
        | _ ->
            _eRR ()
    
    and _menhir_run_1013 : type  ttv_stack. ((ttv_stack, _menhir_box_opn) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1013
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1013
        | Tokens.RPAR ->
            let _menhir_s = MenhirState1013 in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, _menhir_s, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                let _menhir_stack = MenhirCell0_WHILE (_menhir_stack, _startpos) in
                let _menhir_s = MenhirState1015 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.ARBITRARY ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.REPEAT ->
                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                let _menhir_stack = MenhirCell0_REPEAT (_menhir_stack, _startpos) in
                let _menhir_s = MenhirState1021 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.WHILE ->
                    _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.VAR ->
                    _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.UNREACHABLE ->
                    _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.TRY ->
                    _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.THROW ->
                    _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.RETURN ->
                    _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REPEAT ->
                    _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.PRINT ->
                    _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.PRAGMA ->
                    _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.PASS ->
                    _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LET ->
                    _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IF ->
                    _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.FOR ->
                    _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.DEBUG ->
                    _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.CONSTANT ->
                    _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.CASE ->
                    _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ASSERT ->
                    _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARROBASE_LOOPLIMIT ->
                    _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1013
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1013
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1013
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1013
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1013
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1013
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1013
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1013
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1013
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1013
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1013
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1013
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1013
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1013
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1013
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1013
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1013
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1013
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1013
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1013
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1013
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1013
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1013
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1013
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1013
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1013
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1013
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1013
        | _ ->
            _eRR ()
    
    and _menhir_run_1008 : type  ttv_stack. ((ttv_stack, _menhir_box_opn) _menhir_cell1_ASSERT as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1008
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1008
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell1_ASSERT (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                let (_1, _2, _endpos__2_inlined1_, e) = ((), (), _endpos_0, _v) in
                let _v = _menhir_action_361 _1 _2 _endpos__2_inlined1_ _startpos__1_ e in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1008
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1008
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1008
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1008
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1008
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1008
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1008
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1008
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1008
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1008
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1008
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1008
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1008
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1008
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1008
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1008
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1008
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1008
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1008
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1008
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1008
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1008
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1008
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1008
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1008
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1008
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1008
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1008
        | _ ->
            _eRR ()
    
    and _menhir_run_1002 : type  ttv_stack. ((ttv_stack, _menhir_box_opn) _menhir_cell1_CASE as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1002
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1002
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1002
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1002
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1002
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1002
        | Tokens.OF ->
            let _menhir_stack = MenhirCell1_OF (_menhir_stack, MenhirState1002) in
            let _menhir_s = MenhirState1003 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHEN ->
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1002
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1002
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1002
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1002
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1002
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1002
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1002
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1002
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1002
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1002
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1002
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1002
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1002
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1002
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1002
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1002
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1002
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1002
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1002
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1002
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1002
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1002
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1002
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1002
        | _ ->
            _eRR ()
    
    and _menhir_run_0960 : type  ttv_stack. ((((((ttv_stack, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DOWNTO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0960
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0960
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0960
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0960
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0960
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0960
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0960
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0960
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0960
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0960
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0960
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0960
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0960
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0960
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0960
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0960
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0960
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0960
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0960
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0960
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0960
        | Tokens.DO ->
            let _menhir_stack = MenhirCell1_DO (_menhir_stack, MenhirState0960) in
            let _menhir_s = MenhirState0961 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0960
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0960
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0960
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0960
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0960
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0960
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0960
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0960
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0960
        | _ ->
            _eRR ()
    
    and _menhir_run_0958 : type  ttv_stack. ((((ttv_stack, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_DOWNTO as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.LOOPLIMIT ->
            let _menhir_stack = MenhirCell1_LOOPLIMIT (_menhir_stack, MenhirState0958) in
            let _menhir_s = MenhirState0959 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.DO ->
            let _menhir_stack = MenhirCell1_DO (_menhir_stack, MenhirState0958) in
            let _menhir_s = MenhirState0965 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0958
        | _ ->
            _eRR ()
    
    and _menhir_run_0948 : type  ttv_stack. ((((((ttv_stack, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_TO, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
        | Tokens.DO ->
            let _menhir_stack = MenhirCell1_DO (_menhir_stack, MenhirState0948) in
            let _menhir_s = MenhirState0949 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0948
        | _ ->
            _eRR ()
    
    and _menhir_run_0946 : type  ttv_stack. ((((ttv_stack, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_TO as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0946
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0946
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0946
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0946
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0946
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0946
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0946
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0946
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0946
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0946
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0946
        | Tokens.LOOPLIMIT ->
            let _menhir_stack = MenhirCell1_LOOPLIMIT (_menhir_stack, MenhirState0946) in
            let _menhir_s = MenhirState0947 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0946
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0946
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0946
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0946
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0946
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0946
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0946
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0946
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0946
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0946
        | Tokens.DO ->
            let _menhir_stack = MenhirCell1_DO (_menhir_stack, MenhirState0946) in
            let _menhir_s = MenhirState0953 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0946
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0946
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0946
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0946
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0946
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0946
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0946
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0946
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0946
        | _ ->
            _eRR ()
    
    and _menhir_run_0944 : type  ttv_stack. ((ttv_stack, _menhir_box_opn) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.TO ->
            let _menhir_stack = MenhirCell1_TO (_menhir_stack, MenhirState0944) in
            let _menhir_s = MenhirState0945 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0944
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0944
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0944
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0944
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0944
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0944
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0944
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0944
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0944
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0944
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0944
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0944
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0944
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0944
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0944
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0944
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0944
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0944
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0944
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0944
        | Tokens.DOWNTO ->
            let _menhir_stack = MenhirCell1_DOWNTO (_menhir_stack, MenhirState0944) in
            let _menhir_s = MenhirState0957 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0944
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0944
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0944
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0944
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0944
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0944
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0944
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0944
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0944
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0944
        | _ ->
            _eRR ()
    
    and _menhir_run_0938 : type  ttv_stack. ((((ttv_stack, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, _menhir_box_opn) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0938
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0938
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell0_IDENTIFIER (_menhir_stack, fld, _, _) = _menhir_stack in
                let MenhirCell1_loption_plist_expr__ (_menhir_stack, _, args, _endpos_args_) = _menhir_stack in
                let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
                let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                let (_2, _2_inlined1, _4, _endpos__2_inlined2_, rhs) = ((), (), (), _endpos_0, _v) in
                let _v = _menhir_action_370 _1_inlined1 _2 _2_inlined1 _4 _endpos__2_inlined2_ _endpos_args_ _startpos_name_ args fld name rhs in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0938
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0938
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0938
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0938
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0938
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0938
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0938
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0938
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0938
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0938
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0938
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0938
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0938
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0938
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0938
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0938
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0938
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0938
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0938
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0938
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0938
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0938
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0938
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0938
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0938
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0938
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0938
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0938
        | _ ->
            _eRR ()
    
    and _menhir_run_0933 : type  ttv_stack. (((((ttv_stack, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, _menhir_box_opn) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0933
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0933
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell0_RBRACKET (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _, _1_inlined3) = _menhir_stack in
                let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
                let MenhirCell1_loption_plist_expr__ (_menhir_stack, _, args, _endpos_args_) = _menhir_stack in
                let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
                let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                let (_2, _2_inlined1, _4, _2_inlined2, rhs, _endpos__2_inlined3_) = ((), (), (), (), _v, _endpos_0) in
                let _v = _menhir_action_373 _1_inlined1 _1_inlined3 _2 _2_inlined1 _2_inlined2 _4 _endpos__2_inlined3_ _endpos_args_ _startpos_name_ args name rhs x in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0933
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0933
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0933
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0933
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0933
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0933
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0933
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0933
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0933
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0933
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0933
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0933
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0933
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0933
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0933
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0933
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0933
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0933
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0933
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0933
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0933
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0933
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0933
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0933
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0933
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0933
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0933
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0933
        | _ ->
            _eRR ()
    
    and _menhir_run_0923 : type  ttv_stack. ((((ttv_stack, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, _menhir_box_opn) _menhir_cell1_loption_plist_expr__ as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0923
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0923
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell1_loption_plist_expr__ (_menhir_stack, _, args, _endpos_args_) = _menhir_stack in
                let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
                let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                let (_2, _2_inlined1, _endpos__2_inlined2_, rhs) = ((), (), _endpos_0, _v) in
                let _v = _menhir_action_367 _1_inlined1 _2 _2_inlined1 _endpos__2_inlined2_ _endpos_args_ _startpos_name_ args name rhs in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0923
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0923
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0923
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0923
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0923
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0923
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0923
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0923
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0923
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0923
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0923
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0923
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0923
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0923
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0923
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0923
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0923
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0923
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0923
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0923
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0923
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0923
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0923
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0923
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0923
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0923
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0923
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0923
        | _ ->
            _eRR ()
    
    and _menhir_run_0913 : type  ttv_stack. (((ttv_stack, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0913
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0913
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell0_IDENTIFIER (_menhir_stack, fld, _, _) = _menhir_stack in
                let MenhirCell0_RPAR (_menhir_stack, _endpos__3_) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
                let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                let (_2, _2_inlined1, _4, _endpos__2_inlined2_, rhs) = ((), (), (), _endpos_0, _v) in
                let _v = _menhir_action_369 _1_inlined1 _2 _2_inlined1 _4 _endpos__2_inlined2_ _endpos__3_ _startpos_name_ fld name rhs in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0913
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0913
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0913
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0913
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0913
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0913
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0913
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0913
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0913
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0913
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0913
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0913
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0913
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0913
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0913
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0913
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0913
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0913
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0913
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0913
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0913
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0913
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0913
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0913
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0913
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0913
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0913
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0913
        | _ ->
            _eRR ()
    
    and _menhir_run_0908 : type  ttv_stack. ((((ttv_stack, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0908
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0908
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell0_RBRACKET (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _, _1_inlined3) = _menhir_stack in
                let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
                let MenhirCell0_RPAR (_menhir_stack, _endpos__3_) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
                let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                let (_2, _2_inlined1, _4, _2_inlined2, rhs, _endpos__2_inlined3_) = ((), (), (), (), _v, _endpos_0) in
                let _v = _menhir_action_372 _1_inlined1 _1_inlined3 _2 _2_inlined1 _2_inlined2 _4 _endpos__2_inlined3_ _endpos__3_ _startpos_name_ name rhs x in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0908
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0908
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0908
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0908
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0908
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0908
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0908
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0908
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0908
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0908
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0908
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0908
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0908
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0908
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0908
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0908
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0908
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0908
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0908
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0908
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0908
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0908
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0908
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0908
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0908
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0908
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0908
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0908
        | _ ->
            _eRR ()
    
    and _menhir_run_0898 : type  ttv_stack. (((ttv_stack, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0898
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0898
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell0_RPAR (_menhir_stack, _endpos__3_) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
                let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                let (_2, _2_inlined1, _endpos__2_inlined2_, rhs) = ((), (), _endpos_0, _v) in
                let _v = _menhir_action_366 _1_inlined1 _2 _2_inlined1 _endpos__2_inlined2_ _endpos__3_ _startpos_name_ name rhs in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0898
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0898
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0898
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0898
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0898
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0898
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0898
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0898
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0898
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0898
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0898
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0898
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0898
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0898
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0898
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0898
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0898
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0898
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0898
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0898
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0898
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0898
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0898
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0898
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0898
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0898
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0898
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0898
        | _ ->
            _eRR ()
    
    and _menhir_run_0890 : type  ttv_stack. (((ttv_stack, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0890
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0890
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell0_IDENTIFIER (_menhir_stack, fld, _, _) = _menhir_stack in
                let MenhirCell1_RPAR (_menhir_stack, _, _endpos__3_) = _menhir_stack in
                let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                let (_2, _2_inlined1, _4, _endpos__2_inlined2_, rhs) = ((), (), (), _endpos_0, _v) in
                let _v = _menhir_action_368 _2 _2_inlined1 _4 _endpos__2_inlined2_ _endpos__3_ _startpos_name_ fld name rhs in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0890
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0890
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0890
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0890
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0890
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0890
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0890
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0890
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0890
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0890
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0890
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0890
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0890
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0890
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0890
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0890
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0890
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0890
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0890
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0890
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0890
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0890
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0890
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0890
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0890
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0890
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0890
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0890
        | _ ->
            _eRR ()
    
    and _menhir_run_0885 : type  ttv_stack. ((((ttv_stack, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0885
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0885
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell0_RBRACKET (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _, _1_inlined2) = _menhir_stack in
                let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
                let MenhirCell1_RPAR (_menhir_stack, _, _endpos__3_) = _menhir_stack in
                let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                let (_2, _2_inlined1, _4, _2_inlined2, rhs, _endpos__2_inlined3_) = ((), (), (), (), _v, _endpos_0) in
                let _v = _menhir_action_371 _1_inlined2 _2 _2_inlined1 _2_inlined2 _4 _endpos__2_inlined3_ _endpos__3_ _startpos_name_ name rhs x in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0885
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0885
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0885
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0885
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0885
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0885
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0885
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0885
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0885
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0885
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0885
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0885
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0885
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0885
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0885
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0885
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0885
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0885
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0885
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0885
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0885
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0885
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0885
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0885
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0885
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0885
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0885
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0885
        | _ ->
            _eRR ()
    
    and _menhir_run_0875 : type  ttv_stack. (((ttv_stack, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, _menhir_box_opn) _menhir_cell1_RPAR as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell1_RPAR (_menhir_stack, _, _endpos__3_) = _menhir_stack in
                let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                let (_2, _2_inlined1, _endpos__2_inlined2_, rhs) = ((), (), _endpos_0, _v) in
                let _v = _menhir_action_365 _2 _2_inlined1 _endpos__2_inlined2_ _endpos__3_ _startpos_name_ name rhs in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0875
        | _ ->
            _eRR ()
    
    and _menhir_run_0863 : type  ttv_stack. ((ttv_stack, _menhir_box_opn) _menhir_cell1_IF as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.THEN ->
            let _menhir_stack = MenhirCell1_THEN (_menhir_stack, MenhirState0863) in
            let _menhir_s = MenhirState0864 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0863
        | _ ->
            _eRR ()
    
    and _menhir_run_0817 : type  ttv_stack. (((((ttv_stack, _menhir_box_opn) _menhir_cell1_REPEAT, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell1_LOOPLIMIT (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_expr (_menhir_stack, _, expr, _, _) = _menhir_stack in
                let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
                let MenhirCell1_REPEAT (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                let (_1, _2, _2_inlined1, _1_inlined2, _3, _endpos__2_inlined2_) = ((), (), _v, (), (), _endpos_0) in
                let _v = _menhir_action_393 _1 _1_inlined1 _1_inlined2 _2 _2_inlined1 _3 _endpos__2_inlined2_ _startpos__1_ expr in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0817
        | _ ->
            _eRR ()
    
    and _menhir_run_0813 : type  ttv_stack. (((ttv_stack, _menhir_box_opn) _menhir_cell1_REPEAT, _menhir_box_opn) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0813
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0813
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
                let MenhirCell1_REPEAT (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                let (_1, _2, _endpos__2_inlined1_, _3, expr) = ((), (), _endpos_0, (), _v) in
                let _v = _menhir_action_392 _1 _1_inlined1 _2 _3 _endpos__2_inlined1_ _startpos__1_ expr in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0813
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0813
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0813
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0813
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0813
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0813
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0813
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0813
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0813
        | Tokens.LOOPLIMIT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_stack = MenhirCell1_LOOPLIMIT (_menhir_stack, MenhirState0813) in
            let _menhir_s = MenhirState0816 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0813
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0813
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0813
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0813
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0813
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0813
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0813
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0813
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0813
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0813
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0813
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0813
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0813
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0813
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0813
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0813
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0813
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0813
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0813
        | _ ->
            _eRR ()
    
    and _menhir_run_0807 : type  ttv_stack. ((ttv_stack, _menhir_box_opn) _menhir_cell1_RETURN as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell1_RETURN (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                let (_1, _2, _endpos__2_inlined1_, x) = ((), (), _endpos_0, _v) in
                let _v = _menhir_action_357 _1 _2 _endpos__2_inlined1_ _startpos__1_ x in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0807
        | _ ->
            _eRR ()
    
    and _menhir_run_0801 : type  ttv_stack. ((ttv_stack, _menhir_box_opn) _menhir_cell1_THROW as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0801
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0801
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell1_THROW (_menhir_stack, _, _startpos__1_) = _menhir_stack in
                let (_1, _2, _endpos__2_inlined1_, e) = ((), (), _endpos_0, _v) in
                let _v = _menhir_action_395 _1 _2 _endpos__2_inlined1_ _startpos__1_ e in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0801
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0801
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0801
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0801
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0801
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0801
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0801
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0801
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0801
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0801
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0801
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0801
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0801
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0801
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0801
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0801
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0801
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0801
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0801
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0801
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0801
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0801
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0801
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0801
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0801
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0801
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0801
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0801
        | _ ->
            _eRR ()
    
    and _menhir_run_0678 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_THROW as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0678
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0678
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0678, _endpos_0) in
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0678, _endpos_0) in
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0678, _endpos_0) in
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0678, _endpos_0) in
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0678, _endpos_0) in
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0678, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0678, _endpos_0) in
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0678, _endpos_0) in
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0678, _endpos_0) in
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0678, _endpos_0) in
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0678, _endpos_0) in
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0678, _endpos_0) in
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0678, _endpos_0) in
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0678, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0678, _endpos_0) in
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0679
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0678, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0678, _endpos_0) in
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0678, _endpos_0) in
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0678, _endpos_0) in
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0678, _endpos_0) in
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0678, _endpos_0) in
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0679
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell1_THROW (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                let (_1, _endpos__2_, e) = ((), _endpos_0, _v) in
                let _v = _menhir_action_285 _1 _endpos__2_ _startpos__1_ e in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0678
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0678
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0678
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0678
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0678
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0678
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0678
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0678
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0678
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0678
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0678
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0678
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0678
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0678
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0678
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0678
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0678
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0678
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0678
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0678
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0678
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0678
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0678
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0678
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0678
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0678
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0678
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0678
        | _ ->
            _eRR ()
    
    and _menhir_run_0674 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_RETURN as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0674
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0674
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0674, _endpos_0) in
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0675
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0674, _endpos_0) in
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0675
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0674, _endpos_0) in
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0675
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0674, _endpos_0) in
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0675
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0674, _endpos_0) in
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0675
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0674, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0675
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0674, _endpos_0) in
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0675
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0674, _endpos_0) in
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0675
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0674, _endpos_0) in
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0675
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0674, _endpos_0) in
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0675
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0674, _endpos_0) in
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0675
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0674, _endpos_0) in
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0675
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0674, _endpos_0) in
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0675
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0674, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0675
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0674, _endpos_0) in
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0675
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0674, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0675
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0674, _endpos_0) in
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0675
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0674, _endpos_0) in
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0675
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0674, _endpos_0) in
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0675
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0674, _endpos_0) in
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0675
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0674, _endpos_0) in
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0675
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell1_RETURN (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                let (_1, _endpos__2_, x) = ((), _endpos_0, _v) in
                let _v = _menhir_action_247 _1 _endpos__2_ _startpos__1_ x in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0674
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0674
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0674
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0674
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0674
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0674
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0674
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0674
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0674
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0674
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0674
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0674
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0674
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0674
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0674
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0674
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0674
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0674
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0674
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0674
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0674
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0674
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0674
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0674
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0674
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0674
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0674
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0674
        | _ ->
            _eRR ()
    
    and _menhir_run_0670 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_REPEAT, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_LOOPLIMIT as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0670
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0670
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0670, _endpos_0) in
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0671
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0670, _endpos_0) in
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0671
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0670, _endpos_0) in
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0671
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0670, _endpos_0) in
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0671
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0670, _endpos_0) in
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0671
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0670, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0671
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0670, _endpos_0) in
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0671
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0670, _endpos_0) in
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0671
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0670, _endpos_0) in
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0671
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0670, _endpos_0) in
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0671
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0670, _endpos_0) in
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0671
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0670, _endpos_0) in
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0671
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0670, _endpos_0) in
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0671
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0670, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0671
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0670, _endpos_0) in
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0671
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0670, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0671
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0670, _endpos_0) in
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0671
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0670, _endpos_0) in
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0671
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0670, _endpos_0) in
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0671
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0670, _endpos_0) in
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0671
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0670, _endpos_0) in
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0671
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell1_LOOPLIMIT (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_expr (_menhir_stack, _, expr, _, _) = _menhir_stack in
                let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
                let MenhirCell1_REPEAT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                let (_1, _endpos__2_, _2_inlined1, _1_inlined2, _3) = ((), _endpos_0, _v, (), ()) in
                let _v = _menhir_action_283 _1 _1_inlined1 _1_inlined2 _2_inlined1 _3 _endpos__2_ _startpos__1_ expr in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0670
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0670
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0670
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0670
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0670
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0670
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0670
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0670
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0670
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0670
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0670
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0670
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0670
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0670
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0670
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0670
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0670
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0670
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0670
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0670
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0670
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0670
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0670
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0670
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0670
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0670
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0670
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0670
        | _ ->
            _eRR ()
    
    and _menhir_run_0666 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_REPEAT, ttv_result) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0666
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0666
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0666, _endpos_0) in
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0666, _endpos_0) in
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0666, _endpos_0) in
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0666, _endpos_0) in
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0666, _endpos_0) in
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0666, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0666, _endpos_0) in
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0666, _endpos_0) in
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0666, _endpos_0) in
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0666, _endpos_0) in
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0666, _endpos_0) in
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0666, _endpos_0) in
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0666, _endpos_0) in
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0666, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0666, _endpos_0) in
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0667
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0666, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0666, _endpos_0) in
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0666, _endpos_0) in
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0666, _endpos_0) in
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0666, _endpos_0) in
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0666, _endpos_0) in
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0667
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined1, _) = _menhir_stack in
                let MenhirCell1_REPEAT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                let (_1, _endpos__2_, _3, expr) = ((), _endpos_0, (), _v) in
                let _v = _menhir_action_282 _1 _1_inlined1 _3 _endpos__2_ _startpos__1_ expr in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0666
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0666
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0666
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0666
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0666
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0666
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0666
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0666
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0666
        | Tokens.LOOPLIMIT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_stack = MenhirCell1_LOOPLIMIT (_menhir_stack, MenhirState0666) in
            let _menhir_s = MenhirState0669 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0666
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0666
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0666
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0666
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0666
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0666
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0666
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0666
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0666
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0666
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0666
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0666
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0666
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0666
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0666
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0666
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0666
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0666
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0666
        | _ ->
            _eRR ()
    
    and _menhir_run_0631 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_nonempty_list_stmt_, ttv_result) _menhir_cell1_ELSIF as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.THEN ->
            let _menhir_stack = MenhirCell1_THEN (_menhir_stack, MenhirState0631) in
            let _menhir_s = MenhirState0632 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0631
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0631
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0631
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0631
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0631
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0631
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0631
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0631
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0631
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0631
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0631
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0631
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0631
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0631
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0631
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0631
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0631
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0631
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0631
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0631
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0631
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0631
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0631
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0631
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0631
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0631
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0631
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0631
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0631
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0631
        | _ ->
            _eRR ()
    
    and _menhir_run_0626 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, ttv_result) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0626
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0626
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0626, _endpos_0) in
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0626, _endpos_0) in
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0626, _endpos_0) in
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0626, _endpos_0) in
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0626, _endpos_0) in
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0626, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0626, _endpos_0) in
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0626, _endpos_0) in
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0626, _endpos_0) in
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0626, _endpos_0) in
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0626, _endpos_0) in
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0626, _endpos_0) in
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0626, _endpos_0) in
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0626, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0626, _endpos_0) in
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0627
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0626, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0626, _endpos_0) in
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0626, _endpos_0) in
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0626, _endpos_0) in
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0626, _endpos_0) in
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0626, _endpos_0) in
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0627
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell0_IDENTIFIER (_menhir_stack, fld, _, _) = _menhir_stack in
                let MenhirCell1_loption_plist_expr__ (_menhir_stack, _, args, _endpos_args_) = _menhir_stack in
                let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
                let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
                let (_endpos__2_, _2_inlined1, _4, rhs) = (_endpos_0, (), (), _v) in
                let _v = _menhir_action_260 _1_inlined1 _2_inlined1 _4 _endpos__2_ _endpos_args_ _startpos_name_ args fld name rhs in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0626
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0626
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0626
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0626
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0626
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0626
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0626
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0626
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0626
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0626
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0626
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0626
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0626
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0626
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0626
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0626
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0626
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0626
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0626
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0626
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0626
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0626
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0626
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0626
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0626
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0626
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0626
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0626
        | _ ->
            _eRR ()
    
    and _menhir_run_0621 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, ttv_result) _menhir_cell1_loption_plist_expr__ _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0621
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0621
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0621, _endpos_0) in
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0622
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0621, _endpos_0) in
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0622
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0621, _endpos_0) in
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0622
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0621, _endpos_0) in
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0622
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0621, _endpos_0) in
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0622
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0621, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0622
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0621, _endpos_0) in
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0622
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0621, _endpos_0) in
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0622
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0621, _endpos_0) in
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0622
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0621, _endpos_0) in
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0622
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0621, _endpos_0) in
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0622
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0621, _endpos_0) in
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0622
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0621, _endpos_0) in
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0622
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0621, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0622
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0621, _endpos_0) in
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0622
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0621, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0622
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0621, _endpos_0) in
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0622
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0621, _endpos_0) in
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0622
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0621, _endpos_0) in
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0622
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0621, _endpos_0) in
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0622
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0621, _endpos_0) in
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0622
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell0_RBRACKET (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _, _1_inlined3) = _menhir_stack in
                let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
                let MenhirCell1_loption_plist_expr__ (_menhir_stack, _, args, _endpos_args_) = _menhir_stack in
                let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
                let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
                let (_endpos__2_, _2_inlined1, _4, _2_inlined2, rhs) = (_endpos_0, (), (), (), _v) in
                let _v = _menhir_action_263 _1_inlined1 _1_inlined3 _2_inlined1 _2_inlined2 _4 _endpos__2_ _endpos_args_ _startpos_name_ args name rhs x in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0621
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0621
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0621
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0621
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0621
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0621
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0621
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0621
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0621
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0621
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0621
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0621
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0621
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0621
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0621
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0621
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0621
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0621
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0621
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0621
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0621
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0621
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0621
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0621
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0621
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0621
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0621
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0621
        | _ ->
            _eRR ()
    
    and _menhir_run_0611 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, ttv_result) _menhir_cell1_loption_plist_expr__ as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0611
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0611
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0611, _endpos_0) in
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0612
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0611, _endpos_0) in
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0612
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0611, _endpos_0) in
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0612
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0611, _endpos_0) in
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0612
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0611, _endpos_0) in
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0612
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0611, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0612
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0611, _endpos_0) in
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0612
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0611, _endpos_0) in
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0612
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0611, _endpos_0) in
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0612
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0611, _endpos_0) in
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0612
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0611, _endpos_0) in
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0612
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0611, _endpos_0) in
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0612
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0611, _endpos_0) in
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0612
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0611, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0612
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0611, _endpos_0) in
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0612
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0611, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0612
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0611, _endpos_0) in
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0612
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0611, _endpos_0) in
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0612
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0611, _endpos_0) in
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0612
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0611, _endpos_0) in
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0612
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0611, _endpos_0) in
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0612
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell1_loption_plist_expr__ (_menhir_stack, _, args, _endpos_args_) = _menhir_stack in
                let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
                let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
                let (_endpos__2_, _2_inlined1, rhs) = (_endpos_0, (), _v) in
                let _v = _menhir_action_257 _1_inlined1 _2_inlined1 _endpos__2_ _endpos_args_ _startpos_name_ args name rhs in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0611
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0611
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0611
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0611
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0611
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0611
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0611
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0611
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0611
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0611
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0611
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0611
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0611
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0611
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0611
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0611
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0611
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0611
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0611
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0611
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0611
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0611
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0611
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0611
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0611
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0611
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0611
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0611
        | _ ->
            _eRR ()
    
    and _menhir_run_0601 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0601
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0601
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0601, _endpos_0) in
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0602
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0601, _endpos_0) in
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0602
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0601, _endpos_0) in
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0602
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0601, _endpos_0) in
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0602
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0601, _endpos_0) in
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0602
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0601, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0602
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0601, _endpos_0) in
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0602
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0601, _endpos_0) in
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0602
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0601, _endpos_0) in
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0602
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0601, _endpos_0) in
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0602
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0601, _endpos_0) in
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0602
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0601, _endpos_0) in
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0602
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0601, _endpos_0) in
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0602
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0601, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0602
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0601, _endpos_0) in
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0602
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0601, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0602
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0601, _endpos_0) in
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0602
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0601, _endpos_0) in
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0602
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0601, _endpos_0) in
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0602
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0601, _endpos_0) in
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0602
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0601, _endpos_0) in
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0602
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell0_IDENTIFIER (_menhir_stack, fld, _, _) = _menhir_stack in
                let MenhirCell0_RPAR (_menhir_stack, _endpos__3_) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
                let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
                let (_endpos__2_, _2_inlined1, _4, rhs) = (_endpos_0, (), (), _v) in
                let _v = _menhir_action_259 _1_inlined1 _2_inlined1 _4 _endpos__2_ _endpos__3_ _startpos_name_ fld name rhs in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0601
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0601
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0601
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0601
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0601
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0601
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0601
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0601
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0601
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0601
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0601
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0601
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0601
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0601
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0601
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0601
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0601
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0601
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0601
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0601
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0601
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0601
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0601
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0601
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0601
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0601
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0601
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0601
        | _ ->
            _eRR ()
    
    and _menhir_run_0596 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0596, _endpos_0) in
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0597
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0596, _endpos_0) in
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0597
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0596, _endpos_0) in
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0597
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0596, _endpos_0) in
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0597
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0596, _endpos_0) in
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0597
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0596, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0597
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0596, _endpos_0) in
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0597
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0596, _endpos_0) in
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0597
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0596, _endpos_0) in
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0597
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0596, _endpos_0) in
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0597
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0596, _endpos_0) in
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0597
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0596, _endpos_0) in
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0597
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0596, _endpos_0) in
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0597
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0596, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0597
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0596, _endpos_0) in
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0597
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0596, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0597
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0596, _endpos_0) in
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0597
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0596, _endpos_0) in
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0597
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0596, _endpos_0) in
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0597
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0596, _endpos_0) in
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0597
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0596, _endpos_0) in
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0597
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell0_RBRACKET (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _, _1_inlined3) = _menhir_stack in
                let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
                let MenhirCell0_RPAR (_menhir_stack, _endpos__3_) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
                let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
                let (_endpos__2_, _2_inlined1, _4, _2_inlined2, rhs) = (_endpos_0, (), (), (), _v) in
                let _v = _menhir_action_262 _1_inlined1 _1_inlined3 _2_inlined1 _2_inlined2 _4 _endpos__2_ _endpos__3_ _startpos_name_ name rhs x in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0596
        | _ ->
            _eRR ()
    
    and _menhir_run_0586 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RPAR as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0586
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0586
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0586, _endpos_0) in
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0587
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0586, _endpos_0) in
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0587
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0586, _endpos_0) in
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0587
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0586, _endpos_0) in
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0587
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0586, _endpos_0) in
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0587
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0586, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0587
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0586, _endpos_0) in
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0587
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0586, _endpos_0) in
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0587
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0586, _endpos_0) in
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0587
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0586, _endpos_0) in
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0587
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0586, _endpos_0) in
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0587
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0586, _endpos_0) in
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0587
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0586, _endpos_0) in
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0587
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0586, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0587
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0586, _endpos_0) in
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0587
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0586, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0587
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0586, _endpos_0) in
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0587
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0586, _endpos_0) in
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0587
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0586, _endpos_0) in
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0587
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0586, _endpos_0) in
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0587
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0586, _endpos_0) in
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0587
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell0_RPAR (_menhir_stack, _endpos__3_) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
                let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
                let (_endpos__2_, _2_inlined1, rhs) = (_endpos_0, (), _v) in
                let _v = _menhir_action_256 _1_inlined1 _2_inlined1 _endpos__2_ _endpos__3_ _startpos_name_ name rhs in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0586
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0586
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0586
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0586
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0586
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0586
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0586
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0586
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0586
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0586
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0586
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0586
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0586
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0586
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0586
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0586
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0586
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0586
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0586
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0586
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0586
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0586
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0586
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0586
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0586
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0586
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0586
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0586
        | _ ->
            _eRR ()
    
    and _menhir_run_0578 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, ttv_result) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0578
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0578
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0578, _endpos_0) in
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0579
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0578, _endpos_0) in
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0579
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0578, _endpos_0) in
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0579
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0578, _endpos_0) in
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0579
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0578, _endpos_0) in
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0579
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0578, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0579
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0578, _endpos_0) in
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0579
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0578, _endpos_0) in
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0579
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0578, _endpos_0) in
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0579
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0578, _endpos_0) in
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0579
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0578, _endpos_0) in
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0579
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0578, _endpos_0) in
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0579
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0578, _endpos_0) in
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0579
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0578, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0579
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0578, _endpos_0) in
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0579
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0578, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0579
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0578, _endpos_0) in
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0579
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0578, _endpos_0) in
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0579
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0578, _endpos_0) in
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0579
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0578, _endpos_0) in
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0579
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0578, _endpos_0) in
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0579
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell0_IDENTIFIER (_menhir_stack, fld, _, _) = _menhir_stack in
                let MenhirCell1_RPAR (_menhir_stack, _, _endpos__3_) = _menhir_stack in
                let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
                let (_endpos__2_, _2_inlined1, _4, rhs) = (_endpos_0, (), (), _v) in
                let _v = _menhir_action_258 _2_inlined1 _4 _endpos__2_ _endpos__3_ _startpos_name_ fld name rhs in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0578
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0578
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0578
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0578
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0578
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0578
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0578
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0578
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0578
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0578
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0578
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0578
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0578
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0578
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0578
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0578
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0578
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0578
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0578
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0578
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0578
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0578
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0578
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0578
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0578
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0578
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0578
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0578
        | _ ->
            _eRR ()
    
    and _menhir_run_0573 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, ttv_result) _menhir_cell1_RPAR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_IDENTIFIER_ _menhir_cell0_RBRACKET as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0573, _endpos_0) in
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0574
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0573, _endpos_0) in
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0574
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0573, _endpos_0) in
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0574
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0573, _endpos_0) in
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0574
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0573, _endpos_0) in
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0574
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0573, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0574
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0573, _endpos_0) in
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0574
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0573, _endpos_0) in
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0574
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0573, _endpos_0) in
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0574
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0573, _endpos_0) in
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0574
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0573, _endpos_0) in
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0574
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0573, _endpos_0) in
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0574
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0573, _endpos_0) in
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0574
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0573, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0574
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0573, _endpos_0) in
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0574
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0573, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0574
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0573, _endpos_0) in
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0574
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0573, _endpos_0) in
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0574
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0573, _endpos_0) in
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0574
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0573, _endpos_0) in
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0574
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0573, _endpos_0) in
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0574
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell0_RBRACKET (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_IDENTIFIER_ (_menhir_stack, _, _1_inlined2) = _menhir_stack in
                let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
                let MenhirCell1_RPAR (_menhir_stack, _, _endpos__3_) = _menhir_stack in
                let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
                let (_endpos__2_, _2_inlined1, _4, _2_inlined2, rhs) = (_endpos_0, (), (), (), _v) in
                let _v = _menhir_action_261 _1_inlined2 _2_inlined1 _2_inlined2 _4 _endpos__2_ _endpos__3_ _startpos_name_ name rhs x in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0573
        | _ ->
            _eRR ()
    
    and _menhir_run_0563 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR, ttv_result) _menhir_cell1_RPAR as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0563
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0563
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0563, _endpos_0) in
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0564
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0563, _endpos_0) in
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0564
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0563, _endpos_0) in
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0564
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0563, _endpos_0) in
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0564
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0563, _endpos_0) in
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0564
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0563, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0564
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0563, _endpos_0) in
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0564
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0563, _endpos_0) in
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0564
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0563, _endpos_0) in
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0564
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0563, _endpos_0) in
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0564
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0563, _endpos_0) in
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0564
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0563, _endpos_0) in
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0564
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0563, _endpos_0) in
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0564
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0563, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0564
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0563, _endpos_0) in
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0564
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0563, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0564
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0563, _endpos_0) in
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0564
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0563, _endpos_0) in
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0564
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0563, _endpos_0) in
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0564
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0563, _endpos_0) in
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0564
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0563, _endpos_0) in
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0564
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell1_RPAR (_menhir_stack, _, _endpos__3_) = _menhir_stack in
                let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
                let (_endpos__2_, _2_inlined1, rhs) = (_endpos_0, (), _v) in
                let _v = _menhir_action_255 _2_inlined1 _endpos__2_ _endpos__3_ _startpos_name_ name rhs in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0563
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0563
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0563
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0563
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0563
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0563
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0563
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0563
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0563
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0563
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0563
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0563
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0563
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0563
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0563
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0563
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0563
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0563
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0563
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0563
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0563
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0563
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0563
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0563
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0563
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0563
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0563
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0563
        | _ ->
            _eRR ()
    
    and _menhir_run_0552 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DOWNTO, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_LOOPLIMIT as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0552
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0552
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0552
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0552
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0552
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0552
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0552
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0552
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0552
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0552
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0552
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0552
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0552
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0552
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0552
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0552
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0552
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0552
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0552
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0552
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0552
        | Tokens.DO ->
            let _menhir_stack = MenhirCell1_DO (_menhir_stack, MenhirState0552) in
            let _menhir_s = MenhirState0553 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0552
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0552
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0552
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0552
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0552
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0552
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0552
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0552
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0552
        | _ ->
            _eRR ()
    
    and _menhir_run_0550 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DOWNTO as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0550
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0550
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0550
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0550
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0550
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0550
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0550
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0550
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0550
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0550
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0550
        | Tokens.LOOPLIMIT ->
            let _menhir_stack = MenhirCell1_LOOPLIMIT (_menhir_stack, MenhirState0550) in
            let _menhir_s = MenhirState0551 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0550
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0550
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0550
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0550
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0550
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0550
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0550
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0550
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0550
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0550
        | Tokens.DO ->
            let _menhir_stack = MenhirCell1_DO (_menhir_stack, MenhirState0550) in
            let _menhir_s = MenhirState0557 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0550
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0550
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0550
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0550
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0550
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0550
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0550
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0550
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0550
        | _ ->
            _eRR ()
    
    and _menhir_run_0505 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_RPAR _menhir_cell0_REPEAT, ttv_result) _menhir_cell1_nonempty_list_stmt_ as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0505
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0505
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0505, _endpos_0) in
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0506
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0505, _endpos_0) in
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0506
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0505, _endpos_0) in
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0506
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0505, _endpos_0) in
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0506
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0505, _endpos_0) in
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0506
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0505, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0506
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0505, _endpos_0) in
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0506
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0505, _endpos_0) in
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0506
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0505, _endpos_0) in
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0506
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0505, _endpos_0) in
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0506
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0505, _endpos_0) in
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0506
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0505, _endpos_0) in
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0506
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0505, _endpos_0) in
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0506
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0505, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0506
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0505, _endpos_0) in
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0506
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0505, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0506
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0505, _endpos_0) in
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0506
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0505, _endpos_0) in
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0506
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0505, _endpos_0) in
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0506
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0505, _endpos_0) in
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0506
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0505, _endpos_0) in
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0506
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell1_nonempty_list_stmt_ (_menhir_stack, _, _1_inlined2, _) = _menhir_stack in
                let MenhirCell0_REPEAT (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_RPAR (_menhir_stack, _, _) = _menhir_stack in
                let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
                let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_ARROBASE_LOOPLIMIT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                let (_1, _endpos__2_, _3, _5, cond) = ((), _endpos_0, (), (), _v) in
                let _v = _menhir_action_284 _1 _1_inlined2 _3 _5 _endpos__2_ _startpos__1_ cond x in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0505
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0505
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0505
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0505
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0505
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0505
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0505
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0505
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0505
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0505
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0505
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0505
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0505
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0505
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0505
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0505
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0505
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0505
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0505
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0505
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0505
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0505
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0505
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0505
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0505
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0505
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0505
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0505
        | _ ->
            _eRR ()
    
    and _menhir_run_0499 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_lexpr as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0499, _endpos_0) in
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0500
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0499, _endpos_0) in
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0500
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0499, _endpos_0) in
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0500
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0499, _endpos_0) in
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0500
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0499, _endpos_0) in
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0500
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0499, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0500
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0499, _endpos_0) in
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0500
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0499, _endpos_0) in
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0500
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0499, _endpos_0) in
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0500
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0499, _endpos_0) in
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0500
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0499, _endpos_0) in
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0500
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0499, _endpos_0) in
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0500
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0499, _endpos_0) in
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0500
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0499, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0500
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0499, _endpos_0) in
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0500
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0499, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0500
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0499, _endpos_0) in
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0500
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0499, _endpos_0) in
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0500
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0499, _endpos_0) in
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0500
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0499, _endpos_0) in
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0500
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0499, _endpos_0) in
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0500
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell1_lexpr (_menhir_stack, _menhir_s, le, _startpos_le_) = _menhir_stack in
                let (_endpos__2_, _2_inlined1, e) = (_endpos_0, (), _v) in
                let _v = _menhir_action_254 _2_inlined1 _endpos__2_ _startpos_le_ e le in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0499
        | _ ->
            _eRR ()
    
    and _menhir_run_0489 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_RPAR _menhir_cell0_WHILE as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0489
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0489
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0489
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0489
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0489
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0489
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0489
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0489
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0489
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0489
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0489
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0489
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0489
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0489
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0489
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0489
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0489
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0489
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0489
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0489
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0489
        | Tokens.DO ->
            let _menhir_stack = MenhirCell1_DO (_menhir_stack, MenhirState0489) in
            let _menhir_s = MenhirState0490 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0489
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0489
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0489
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0489
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0489
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0489
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0489
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0489
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0489
        | _ ->
            _eRR ()
    
    and _menhir_run_0486 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_ARROBASE_LOOPLIMIT _menhir_cell0_LPAR as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.RPAR ->
            let _menhir_s = MenhirState0486 in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, _menhir_s, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                let _menhir_stack = MenhirCell0_WHILE (_menhir_stack, _startpos) in
                let _menhir_s = MenhirState0488 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.ARBITRARY ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.REPEAT ->
                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                let _menhir_stack = MenhirCell0_REPEAT (_menhir_stack, _startpos) in
                let _menhir_s = MenhirState0502 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.WHILE ->
                    _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.VAR ->
                    _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.UNREACHABLE ->
                    _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.TRY ->
                    _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.THROW ->
                    _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.RETURN ->
                    _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REPEAT ->
                    _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.PRINT ->
                    _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.PRAGMA ->
                    _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.PASS ->
                    _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LET ->
                    _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IF ->
                    _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.FOR ->
                    _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.DEBUG ->
                    _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.CONSTANT ->
                    _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.CASE ->
                    _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ASSERT ->
                    _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARROBASE_LOOPLIMIT ->
                    _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0486
        | _ ->
            _eRR ()
    
    and _menhir_run_0482 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_ASSERT as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0482
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0482
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0482, _endpos_0) in
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0483
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0482, _endpos_0) in
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0483
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0482, _endpos_0) in
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0483
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0482, _endpos_0) in
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0483
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0482, _endpos_0) in
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0483
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0482, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0483
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0482, _endpos_0) in
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0483
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0482, _endpos_0) in
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0483
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0482, _endpos_0) in
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0483
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0482, _endpos_0) in
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0483
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0482, _endpos_0) in
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0483
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0482, _endpos_0) in
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0483
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0482, _endpos_0) in
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0483
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0482, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0483
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0482, _endpos_0) in
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0483
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0482, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0483
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0482, _endpos_0) in
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0483
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0482, _endpos_0) in
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0483
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0482, _endpos_0) in
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0483
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0482, _endpos_0) in
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0483
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _menhir_stack = MenhirCell1_SEMI_COLON (_menhir_stack, MenhirState0482, _endpos_0) in
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0483
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell1_ASSERT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                let (_1, _endpos__2_, e) = ((), _endpos_0, _v) in
                let _v = _menhir_action_251 _1 _endpos__2_ _startpos__1_ e in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0482
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0482
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0482
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0482
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0482
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0482
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0482
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0482
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0482
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0482
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0482
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0482
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0482
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0482
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0482
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0482
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0482
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0482
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0482
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0482
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0482
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0482
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0482
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0482
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0482
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0482
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0482
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0482
        | _ ->
            _eRR ()
    
    and _menhir_run_0479 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_WHEN, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_pattern_ as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0479
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0479
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0479
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0479
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0479
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0479
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0479
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0479
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0479
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0479
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0479
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0479
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0479
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0479
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0479
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0479
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0479
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0479
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0479
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0479
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0479
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0479
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0479
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0479
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0479
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0479
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0479
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0479
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0479
        | Tokens.ARROW ->
            let _menhir_stack = MenhirCell1_ARROW (_menhir_stack, MenhirState0479) in
            let _menhir_s = MenhirState0480 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0479
        | _ ->
            _eRR ()
    
    and _menhir_run_0474 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_CASE as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0474
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0474
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0474
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0474
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0474
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0474
        | Tokens.OF ->
            let _menhir_stack = MenhirCell1_OF (_menhir_stack, MenhirState0474) in
            let _menhir_s = MenhirState0475 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHEN ->
                _menhir_run_0476 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0474
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0474
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0474
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0474
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0474
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0474
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0474
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0474
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0474
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0474
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0474
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0474
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0474
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0474
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0474
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0474
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0474
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0474
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0474
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0474
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0474
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0474
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0474
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0474
        | _ ->
            _eRR ()
    
    and _menhir_run_0458 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_TO, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_LOOPLIMIT as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.DO ->
            let _menhir_stack = MenhirCell1_DO (_menhir_stack, MenhirState0458) in
            let _menhir_s = MenhirState0459 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0458
        | _ ->
            _eRR ()
    
    and _menhir_run_0456 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_TO as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0456
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0456
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0456
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0456
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0456
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0456
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0456
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0456
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0456
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0456
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0456
        | Tokens.LOOPLIMIT ->
            let _menhir_stack = MenhirCell1_LOOPLIMIT (_menhir_stack, MenhirState0456) in
            let _menhir_s = MenhirState0457 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0456
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0456
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0456
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0456
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0456
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0456
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0456
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0456
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0456
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0456
        | Tokens.DO ->
            let _menhir_stack = MenhirCell1_DO (_menhir_stack, MenhirState0456) in
            let _menhir_s = MenhirState0545 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0456
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0456
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0456
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0456
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0456
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0456
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0456
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0456
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0456
        | _ ->
            _eRR ()
    
    and _menhir_run_0454 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_FOR _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.TO ->
            let _menhir_stack = MenhirCell1_TO (_menhir_stack, MenhirState0454) in
            let _menhir_s = MenhirState0455 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0454
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0454
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0454
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0454
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0454
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0454
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0454
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0454
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0454
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0454
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0454
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0454
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0454
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0454
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0454
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0454
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0454
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0454
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0454
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0454
        | Tokens.DOWNTO ->
            let _menhir_stack = MenhirCell1_DOWNTO (_menhir_stack, MenhirState0454) in
            let _menhir_s = MenhirState0549 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0454
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0454
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0454
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0454
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0454
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0454
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0454
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0454
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0454
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0454
        | _ ->
            _eRR ()
    
    and _menhir_run_0445 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_IF as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.THEN ->
            let _menhir_stack = MenhirCell1_THEN (_menhir_stack, MenhirState0445) in
            let _menhir_s = MenhirState0446 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0445
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0445
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0445
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0445
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0445
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0445
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0445
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0445
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0445
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0445
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0445
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0445
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0445
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0445
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0445
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0445
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0445
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0445
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0445
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0445
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0445
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0445
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0445
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0445
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0445
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0445
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0445
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0445
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0445
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0445
        | _ ->
            _eRR ()
    
    and _menhir_run_0417 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_lexpr_atom_desc as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0417
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0417
        | Tokens.RRBRACKET ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_lexpr_atom_desc (_menhir_stack, _menhir_s, desc, _startpos_desc_, _endpos_desc_) = _menhir_stack in
            let (_2, _4, expr, _endpos__4_) = ((), (), _v, _endpos_0) in
            let _v = _menhir_action_129 _2 _4 _endpos_desc_ _startpos_desc_ desc expr in
            _menhir_goto_lexpr_atom_desc _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__4_ _startpos_desc_ _v _menhir_s _tok
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0417
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0417
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0417
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0417
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0417
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0417
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0417
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0417
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0417
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0417
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0417
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0417
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0417
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0417
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0417
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0417
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0417
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0417
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0417
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0417
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0417
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0417
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0417
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0417
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0417
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0417
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0417
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0417
        | _ ->
            _eRR ()
    
    and _menhir_run_0362 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_WHILE, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_LOOPLIMIT as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0362
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0362
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0362
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0362
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0362
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0362
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0362
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0362
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0362
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0362
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0362
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0362
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0362
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0362
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0362
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0362
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0362
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0362
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0362
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0362
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0362
        | Tokens.DO ->
            let _menhir_stack = MenhirCell1_DO (_menhir_stack, MenhirState0362) in
            let _menhir_s = MenhirState0363 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0362
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0362
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0362
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0362
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0362
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0362
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0362
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0362
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0362
        | _ ->
            _eRR ()
    
    and _menhir_run_0360 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_WHILE as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0360
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0360
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0360
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0360
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0360
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0360
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0360
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0360
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0360
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0360
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0360
        | Tokens.LOOPLIMIT ->
            let _menhir_stack = MenhirCell1_LOOPLIMIT (_menhir_stack, MenhirState0360) in
            let _menhir_s = MenhirState0361 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0360
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0360
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0360
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0360
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0360
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0360
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0360
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0360
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0360
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0360
        | Tokens.DO ->
            let _menhir_stack = MenhirCell1_DO (_menhir_stack, MenhirState0360) in
            let _menhir_s = MenhirState0735 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0360
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0360
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0360
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0360
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0360
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0360
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0360
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0360
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0360
        | _ ->
            _eRR ()
    
    and _menhir_run_0357 : type  ttv_stack. ((((ttv_stack, _menhir_box_opn) _menhir_cell1_WHILE, _menhir_box_opn) _menhir_cell1_expr, _menhir_box_opn) _menhir_cell1_LOOPLIMIT as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0357
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0357
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0357
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0357
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0357
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0357
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0357
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0357
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0357
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0357
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0357
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0357
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0357
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0357
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0357
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0357
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0357
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0357
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0357
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0357
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0357
        | Tokens.DO ->
            let _menhir_stack = MenhirCell1_DO (_menhir_stack, MenhirState0357) in
            let _menhir_s = MenhirState0358 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0357
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0357
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0357
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0357
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0357
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0357
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0357
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0357
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0357
        | _ ->
            _eRR ()
    
    and _menhir_run_0355 : type  ttv_stack. ((ttv_stack, _menhir_box_opn) _menhir_cell1_WHILE as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
        | Tokens.LOOPLIMIT ->
            let _menhir_stack = MenhirCell1_LOOPLIMIT (_menhir_stack, MenhirState0355) in
            let _menhir_s = MenhirState0356 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
        | Tokens.DO ->
            let _menhir_stack = MenhirCell1_DO (_menhir_stack, MenhirState0355) in
            let _menhir_s = MenhirState0742 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0355
        | _ ->
            _eRR ()
    
    and _menhir_run_0354 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_NOT as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0354
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0354
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0354
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0354
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.POW | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RDIV | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_NOT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _endpos_e_, e) = ((), _endpos, _v) in
            let _v = _menhir_action_048 _1 _endpos_e_ _startpos__1_ e in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e_ _startpos__1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0349 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LPAR as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0349
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0349
        | Tokens.RPAR ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (x, _endpos__3_) = (_v, _endpos_0) in
            let _v = _menhir_action_066 _endpos__3_ _startpos__1_ x in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos__1_ _v _menhir_s _tok
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0349
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0349
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0349
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0349
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0349
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0349
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0349
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0349
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0349
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0349
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0349
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0349
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0349
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0349
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0349
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0349
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0349
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0349
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0349
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0349
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0349
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0349
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_stack = MenhirCell1_COMMA (_menhir_stack, MenhirState0349) in
            let _menhir_s = MenhirState0351 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0349
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0349
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0349
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0349
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0349
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0349
        | _ ->
            _eRR ()
    
    and _menhir_run_0347 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_IF, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0347
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0347
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0347
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0347
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0347
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0347
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0347
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0347
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0347
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0347
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0347
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0347
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0347
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0347
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0347
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0347
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0347
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0347
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0347
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0347
        | Tokens.ELSIF ->
            _menhir_run_0175 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0347
        | Tokens.ELSE ->
            _menhir_run_0179 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0347
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0347
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0347
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0347
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0347
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0347
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0347
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0347
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0347
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0347
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0347
        | _ ->
            _eRR ()
    
    and _menhir_run_0175 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _menhir_stack = MenhirCell1_ELSIF (_menhir_stack, _menhir_s, _startpos) in
        let _menhir_s = MenhirState0175 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0179 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _menhir_stack = MenhirCell1_ELSE (_menhir_stack, _menhir_s) in
        let _menhir_s = MenhirState0179 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.REAL_LIT _v ->
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.NOT ->
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.INT_LIT _v ->
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.IF ->
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BOOL_LIT _v ->
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.BNOT ->
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.BITVECTOR_LIT _v ->
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.ARBITRARY ->
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0345 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_IF as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.THEN ->
            let _menhir_stack = MenhirCell1_THEN (_menhir_stack, MenhirState0345) in
            let _menhir_s = MenhirState0346 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0345
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0345
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0345
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0345
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0345
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0345
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0345
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0345
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0345
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0345
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0345
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0345
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0345
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0345
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0345
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0345
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0345
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0345
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0345
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0345
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0345
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0345
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0345
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0345
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0345
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0345
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0345
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0345
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0345
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0345
        | _ ->
            _eRR ()
    
    and _menhir_run_0342 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_BNOT as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0342
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0342
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0342
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0342
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.POW | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RDIV | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_BNOT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _endpos_e_, e) = ((), _endpos, _v) in
            let _v = _menhir_action_046 _1 _endpos_e_ _startpos__1_ e in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e_ _startpos__1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0312 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_BNOT as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0312
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0312
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0312
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0312
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RBRACE | Tokens.RDIV | Tokens.RPAR | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_BNOT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _endpos_e_, e) = ((), _endpos, _v) in
            let _v = _menhir_action_101 _1 _endpos_e_ _startpos__1_ e in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e_ _startpos__1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0311 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0311
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0311
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0311
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0311
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0311
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0311
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0311
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0311
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0311
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0311
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0311
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0311
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0311
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LT | Tokens.MINUS | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.RBRACE | Tokens.RPAR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_076 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0302 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0302
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COMMA | Tokens.IMPL | Tokens.RBRACE | Tokens.RPAR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_077 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0300 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0300
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COMMA | Tokens.IMPL | Tokens.RBRACE | Tokens.RPAR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_079 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0298 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0298
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COMMA | Tokens.IMPL | Tokens.RBRACE | Tokens.RPAR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_078 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0296 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0296
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0296
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0296
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0296
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0296
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0296
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0296
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0296
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0296
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0296
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0296
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0296
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0296
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LT | Tokens.MINUS | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.RBRACE | Tokens.RPAR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_099 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0294 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0294
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0294
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0294
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0294
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RBRACE | Tokens.RDIV | Tokens.RPAR | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_100 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0292 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0292
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0292
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0292
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0292
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0292
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0292
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DIV | Tokens.DIVRM | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.RBRACE | Tokens.RDIV | Tokens.RPAR | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_080 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0290 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0290
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0290
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0290
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0290
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0290
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0290
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DIV | Tokens.DIVRM | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.RBRACE | Tokens.RDIV | Tokens.RPAR | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_081 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0283 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0283
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0283
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0283
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0283
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0283
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0283
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0283
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0283
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0283
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0283
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0283
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0283
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0283
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LT | Tokens.MINUS | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.RBRACE | Tokens.RPAR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_082 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0281 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0281
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COMMA | Tokens.EQ_OP | Tokens.IMPL | Tokens.NEQ | Tokens.RBRACE | Tokens.RPAR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_083 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0278 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern _menhir_cell0_GEQ as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0278
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0278
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0278
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0278
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0278
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0278
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0278
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0278
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0278
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0278
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0278
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0278
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0278
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0278
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0278
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0278
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0278
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0278
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0278
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COMMA | Tokens.EQ_OP | Tokens.IMPL | Tokens.NEQ | Tokens.RBRACE | Tokens.RPAR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell0_GEQ (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_086 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0276 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0276
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0276
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0276
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0276
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0276
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0276
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0276
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0276
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0276
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0276
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0276
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0276
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0276
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0276
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0276
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0276
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0276
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0276
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0276
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COMMA | Tokens.EQ_OP | Tokens.IMPL | Tokens.NEQ | Tokens.RBRACE | Tokens.RPAR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_085 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0274 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0274
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COMMA | Tokens.IMPL | Tokens.RBRACE | Tokens.RPAR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_087 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0261 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern _menhir_cell0_LEQ as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0261
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0261
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0261
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0261
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0261
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0261
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0261
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0261
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0261
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0261
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0261
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0261
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0261
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0261
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0261
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0261
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0261
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0261
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0261
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COMMA | Tokens.EQ_OP | Tokens.IMPL | Tokens.NEQ | Tokens.RBRACE | Tokens.RPAR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell0_LEQ (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_089 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0258 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.RRBRACKET ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_2, _4, e2, _endpos__4_) = ((), (), _v, _endpos_0) in
            let _v = _menhir_action_109 _2 _4 _endpos__4_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__4_ _startpos_e1_ _v _menhir_s _tok
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0258
        | _ ->
            _eRR ()
    
    and _menhir_run_0256 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0256
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0256
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0256
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0256
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0256
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0256
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0256
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0256
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0256
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0256
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0256
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0256
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0256
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0256
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0256
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0256
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0256
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0256
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0256
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COMMA | Tokens.EQ_OP | Tokens.IMPL | Tokens.NEQ | Tokens.RBRACE | Tokens.RPAR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_088 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0254 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern _menhir_cell0_MINUS as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0254
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0254
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0254
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0254
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0254
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0254
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0254
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0254
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0254
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0254
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0254
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0254
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0254
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LT | Tokens.MINUS | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.RBRACE | Tokens.RPAR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell0_MINUS (_menhir_stack, _, _) = _menhir_stack in
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_091 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0252 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0252
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0252
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0252
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0252
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0252
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0252
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DIV | Tokens.DIVRM | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.RBRACE | Tokens.RDIV | Tokens.RPAR | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_092 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0250 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0250
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0250
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0250
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0250
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0250
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0250
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DIV | Tokens.DIVRM | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.RBRACE | Tokens.RDIV | Tokens.RPAR | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_093 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0248 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0248
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COMMA | Tokens.EQ_OP | Tokens.IMPL | Tokens.NEQ | Tokens.RBRACE | Tokens.RPAR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_084 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0245 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0245
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0245
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0245
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0245
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0245
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0245
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0245
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0245
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0245
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0245
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0245
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0245
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0245
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LT | Tokens.MINUS | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.RBRACE | Tokens.RPAR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_094 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0243 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0243
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0243
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0243
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0243
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0243
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0243
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0243
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0243
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0243
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0243
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0243
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0243
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0243
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LT | Tokens.MINUS | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.RBRACE | Tokens.RPAR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_090 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0241 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0241
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0241
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0241
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0241
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RBRACE | Tokens.RDIV | Tokens.RPAR | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_098 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0239 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0239
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0239
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0239
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0239
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0239
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0239
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DIV | Tokens.DIVRM | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.RBRACE | Tokens.RDIV | Tokens.RPAR | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_095 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0237 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0237
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0237
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0237
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0237
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0237
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0237
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DIV | Tokens.DIVRM | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.RBRACE | Tokens.RDIV | Tokens.RPAR | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_096 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0235 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0235
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0235
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0235
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0235
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0235
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0235
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DIV | Tokens.DIVRM | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.RBRACE | Tokens.RDIV | Tokens.RPAR | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_097 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0233 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0233
        | Tokens.ARROW | Tokens.COMMA | Tokens.RBRACE | Tokens.RPAR | Tokens.WHERE ->
            let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_2, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_404 _2 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0216 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_GEQ as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0216
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0216
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0216
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0216
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0216
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0216
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0216
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0216
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0216
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0216
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0216
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0216
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0216
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0216
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0216
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0216
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0216
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0216
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0216
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0216
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0216
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0216
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0216
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0216
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0216
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0216
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0216
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0216
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0216
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0216
        | Tokens.ARROW | Tokens.COMMA | Tokens.RBRACE | Tokens.RPAR | Tokens.WHERE ->
            let MenhirCell1_GEQ (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _endpos_expr_, expr) = ((), _endpos, _v) in
            let _v = _menhir_action_407 _1 _endpos_expr_ _startpos__1_ expr in
            _menhir_goto_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_expr_ _startpos__1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0206 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0206
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0206
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0206
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0206
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0206
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0206
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0206
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0206
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0206
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0206
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0206
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0206
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0206
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0206
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0206
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0206
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0206
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0206
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0206
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0206
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0206
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0206
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0206
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0206
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_stack = MenhirCell1_COMMA (_menhir_stack, MenhirState0206) in
            let _menhir_s = MenhirState0207 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.IDENTIFIER _v ->
                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.EQ ->
                    _menhir_run_0205 _menhir_stack _menhir_lexbuf _menhir_lexer
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0206
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0206
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0206
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0206
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0206
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0206
        | Tokens.RBRACE ->
            let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, x, _, _) = _menhir_stack in
            let y = _v in
            let _v = _menhir_action_421 x y in
            _menhir_goto_separated_nonempty_list_COMMA_field_assign_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_goto_separated_nonempty_list_COMMA_field_assign_ : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        match _menhir_s with
        | MenhirState0191 ->
            _menhir_run_0210 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0207 ->
            _menhir_run_0209 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0982 ->
            _menhir_run_0196 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0843 ->
            _menhir_run_0196 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0763 ->
            _menhir_run_0196 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0713 ->
            _menhir_run_0196 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0439 ->
            _menhir_run_0196 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0468 ->
            _menhir_run_0196 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0194 ->
            _menhir_run_0196 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0210 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, desc, _startpos_desc_, _endpos_desc_) = _menhir_stack in
        let (_1_inlined1, _endpos__3_) = (_v, _endpos) in
        let _v = _menhir_action_123 _1_inlined1 _endpos__3_ _endpos_desc_ _startpos_desc_ desc in
        _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_desc_ _v _menhir_s _tok
    
    and _menhir_run_0209 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_COMMA -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_COMMA (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, y, _, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, x, _, _) = _menhir_stack in
        let xs = _v in
        let _v = _menhir_action_422 x xs y in
        _menhir_goto_separated_nonempty_list_COMMA_field_assign_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_0196 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, desc, _startpos_desc_, _endpos_desc_) = _menhir_stack in
        let (_1_inlined1, _endpos__3_) = (_v, _endpos) in
        let _v = _menhir_action_068 _1_inlined1 _endpos__3_ _endpos_desc_ _startpos_desc_ desc in
        _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_desc_ _v _menhir_s _tok
    
    and _menhir_run_0188 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0188
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0188
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0188
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0188
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0188
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0188
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0188
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0188
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0188
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0188
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0188
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0188
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0188
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0188
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0188
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0188
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0188
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0188
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0188
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0188
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0188
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0188
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0188
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0188
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_stack = MenhirCell1_COMMA (_menhir_stack, MenhirState0188) in
            let _menhir_s = MenhirState0189 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0188
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0188
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0188
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0188
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0188
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0188
        | Tokens.RBRACE | Tokens.RPAR | Tokens.SEMI_COLON ->
            let x = _v in
            let _v = _menhir_action_419 x in
            _menhir_goto_separated_nonempty_list_COMMA_expr_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_goto_separated_nonempty_list_COMMA_expr_ : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        match _menhir_s with
        | MenhirState1086 ->
            _menhir_run_1277 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0992 ->
            _menhir_run_0993 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0984 ->
            _menhir_run_0988 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0970 ->
            _menhir_run_0974 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0916 ->
            _menhir_run_0917 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0870 ->
            _menhir_run_0893 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0853 ->
            _menhir_run_0854 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0845 ->
            _menhir_run_0849 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0830 ->
            _menhir_run_0833 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0821 ->
            _menhir_run_0825 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0773 ->
            _menhir_run_0774 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0765 ->
            _menhir_run_0769 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0723 ->
            _menhir_run_0724 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0715 ->
            _menhir_run_0719 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0404 ->
            _menhir_run_0660 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0408 ->
            _menhir_run_0656 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0645 ->
            _menhir_run_0646 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0441 ->
            _menhir_run_0641 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0604 ->
            _menhir_run_0605 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0448 ->
            _menhir_run_0581 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0461 ->
            _menhir_run_0538 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0528 ->
            _menhir_run_0529 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0470 ->
            _menhir_run_0524 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0351 ->
            _menhir_run_0352 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0010 ->
            _menhir_run_0343 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0191 ->
            _menhir_run_0212 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0200 ->
            _menhir_run_0202 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0982 ->
            _menhir_run_0198 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0843 ->
            _menhir_run_0198 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0763 ->
            _menhir_run_0198 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0713 ->
            _menhir_run_0198 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0439 ->
            _menhir_run_0198 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0468 ->
            _menhir_run_0198 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0194 ->
            _menhir_run_0198 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
        | MenhirState0189 ->
            _menhir_run_0190 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | MenhirState0184 ->
            _menhir_run_0186 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_1277 : type  ttv_stack. ((ttv_stack, _menhir_box_spec) _menhir_cell1_PRAGMA _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_spec) _menhir_state -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.VAR ->
                _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1278
            | Tokens.TYPE ->
                _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1278
            | Tokens.SETTER ->
                _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1278
            | Tokens.PRAGMA ->
                _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1278
            | Tokens.LET ->
                _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1278
            | Tokens.GETTER ->
                _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1278
            | Tokens.FUNC ->
                _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1278
            | Tokens.CONSTANT ->
                _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1278
            | Tokens.CONFIG ->
                _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1278
            | Tokens.EOF ->
                let _v_0 = _menhir_action_133 () in
                _menhir_run_1279 _menhir_stack _v_0
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0993 : type  ttv_stack. (((((ttv_stack, _menhir_box_opn) _menhir_cell1_CONSTANT, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_COMMA as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACE ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_RBRACE (_menhir_stack, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LPAR ->
                _menhir_run_0200 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0994
            | Tokens.SEMI_COLON ->
                let _v_0 = _menhir_action_200 () in
                _menhir_run_0995 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v_0 _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0200 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE as 'stack) -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.STRING_LIT _v ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0200
        | Tokens.RPAR ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let _endpos__3_ = _endpos in
            let _v = _menhir_action_201 () in
            _menhir_goto_loption_plist_expr__ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _v _menhir_s _tok
        | Tokens.REAL_LIT _v ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0200
        | Tokens.NOT ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0200
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0200
        | Tokens.LPAR ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0200
        | Tokens.INT_LIT _v ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0200
        | Tokens.IF ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0200
        | Tokens.IDENTIFIER _v ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0200
        | Tokens.BOOL_LIT _v ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0200
        | Tokens.BNOT ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0200
        | Tokens.BITVECTOR_LIT _v ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0200
        | Tokens.ARBITRARY ->
            let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos) in
            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0200
        | _ ->
            _eRR ()
    
    and _menhir_goto_loption_plist_expr__ : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match _menhir_s with
        | MenhirState0994 ->
            _menhir_run_0995 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0918 ->
            _menhir_run_0919 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0855 ->
            _menhir_run_0856 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0775 ->
            _menhir_run_0776 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0725 ->
            _menhir_run_0726 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0647 ->
            _menhir_run_0648 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0606 ->
            _menhir_run_0607 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0530 ->
            _menhir_run_0531 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok
        | MenhirState0213 ->
            _menhir_run_0214 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0199 ->
            _menhir_run_0204 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0919 : type  ttv_stack. (((ttv_stack, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE as 'stack) -> _ -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.EOF ->
                let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
                let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                let (_2, _endpos__2_inlined1_, args) = ((), _endpos_0, _v) in
                let _v = _menhir_action_360 _1_inlined1 _2 _endpos__2_inlined1_ _startpos_name_ args name in
                _menhir_goto_opn _menhir_stack _v
            | _ ->
                _eRR ())
        | Tokens.EQ ->
            let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
            let _menhir_s = MenhirState0922 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LBRACKET ->
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.IDENTIFIER _v ->
                    let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                    let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.COMMA ->
                        let _menhir_s = MenhirState0929 in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.IDENTIFIER _v ->
                            _menhir_run_0045 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | _ ->
                            _eRR ())
                    | _ ->
                        _eRR ())
                | _ ->
                    _eRR ())
            | Tokens.IDENTIFIER _v ->
                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.EQ ->
                    let _menhir_s = MenhirState0937 in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.STRING_LIT _v ->
                        _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.REAL_LIT _v ->
                        _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.NOT ->
                        _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.MINUS ->
                        _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.LPAR ->
                        _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.INT_LIT _v ->
                        _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.IF ->
                        _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.IDENTIFIER _v ->
                        _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.BOOL_LIT _v ->
                        _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.BNOT ->
                        _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.BITVECTOR_LIT _v ->
                        _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.ARBITRARY ->
                        _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | _ ->
                        _eRR ())
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0726 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_VAR, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_COMMA, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0727
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0727
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0727
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0727
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0727
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0727
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0727
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0727
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0727
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0727
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0727
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0727
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0727
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0727
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0727
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0727
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0727
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0727
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0727
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0727
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0727
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
                let MenhirCell1_COMMA (_menhir_stack, _) = _menhir_stack in
                let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
                let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                let (_1, _endpos__2_, _2_inlined1, _3, _3_inlined1, _5, _endpos_args_, args) = ((), _endpos_0, (), (), (), (), _endpos, _v) in
                let _v = _menhir_action_276 _1 _1_inlined1 _2_inlined1 _3 _3_inlined1 _5 _endpos__2_ _endpos_args_ _startpos__1_ _startpos_name_ args lhs name in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0648 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_LET, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_COMMA, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0649
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0649
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
                let MenhirCell1_COMMA (_menhir_stack, _) = _menhir_stack in
                let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
                let MenhirCell1_LET (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                let (_1, _endpos__2_, _2_inlined1, _3, _3_inlined1, _5, _endpos_args_, args) = ((), _endpos_0, (), (), (), (), _endpos, _v) in
                let _v = _menhir_action_266 _1 _1_inlined1 _2_inlined1 _3 _3_inlined1 _5 _endpos__2_ _endpos_args_ _startpos__1_ _startpos_name_ args lhs name in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0607 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0608
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0608
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0608
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0608
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0608
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0608
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0608
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0608
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0608
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0608
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0608
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0608
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0608
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0608
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0608
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0608
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0608
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0608
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0608
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0608
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0608
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
                let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
                let (_endpos__2_, args) = (_endpos_0, _v) in
                let _v = _menhir_action_250 _1_inlined1 _endpos__2_ _startpos_name_ args name in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.EQ ->
            let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
            let _menhir_s = MenhirState0610 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LBRACKET ->
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.IDENTIFIER _v ->
                    let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                    let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.COMMA ->
                        let _menhir_s = MenhirState0617 in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.IDENTIFIER _v ->
                            _menhir_run_0045 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | _ ->
                            _eRR ())
                    | _ ->
                        _eRR ())
                | _ ->
                    _eRR ())
            | Tokens.IDENTIFIER _v ->
                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.EQ ->
                    let _menhir_s = MenhirState0625 in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.STRING_LIT _v ->
                        _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.REAL_LIT _v ->
                        _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.NOT ->
                        _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.MINUS ->
                        _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.LPAR ->
                        _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.INT_LIT _v ->
                        _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.IF ->
                        _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.IDENTIFIER _v ->
                        _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.BOOL_LIT _v ->
                        _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.BNOT ->
                        _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.BITVECTOR_LIT _v ->
                        _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.ARBITRARY ->
                        _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | _ ->
                        _eRR ())
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0531 : type  ttv_stack ttv_result. ((((((ttv_stack, ttv_result) _menhir_cell1_CONSTANT, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_COMMA, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0532
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0532
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0532
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0532
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0532
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0532
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0532
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0532
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0532
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0532
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0532
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0532
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0532
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0532
            | Tokens.IDENTIFIER _v_1 ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0532
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0532
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0532
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0532
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0532
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0532
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_loption_plist_expr__ (_menhir_stack, _menhir_s, _v, _endpos) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0532
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
                let MenhirCell1_COMMA (_menhir_stack, _) = _menhir_stack in
                let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
                let MenhirCell1_CONSTANT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                let (_1, _endpos__2_, _2_inlined1, _3, _3_inlined1, _5, _endpos_args_, args) = ((), _endpos_0, (), (), (), (), _endpos, _v) in
                let _v = _menhir_action_269 _1 _1_inlined1 _2_inlined1 _3 _3_inlined1 _5 _endpos__2_ _endpos_args_ _startpos__1_ _startpos_name_ args lhs name in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0214 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
        let (_endpos_args_, args) = (_endpos, _v) in
        let _v = _menhir_action_107 _1_inlined1 _endpos_args_ _startpos_name_ args name in
        _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_args_ _startpos_name_ _v _menhir_s _tok
    
    and _menhir_run_0204 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell0_RBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _, _1_inlined1) = _menhir_stack in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
        let (_endpos_args_, args) = (_endpos, _v) in
        let _v = _menhir_action_052 _1_inlined1 _endpos_args_ _startpos_name_ args name in
        _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_args_ _startpos_name_ _v _menhir_s _tok
    
    and _menhir_run_0917 : type  ttv_stack. ((ttv_stack, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACE ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_RBRACE (_menhir_stack, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LPAR ->
                _menhir_run_0200 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0918
            | Tokens.DOT | Tokens.EQ | Tokens.SEMI_COLON ->
                let _v_0 = _menhir_action_200 () in
                _menhir_run_0919 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v_0 MenhirState0918 _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0893 : type  ttv_stack. ((ttv_stack, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RPAR ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.SEMI_COLON ->
                let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.EOF ->
                    let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                    let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                    let (_2, _endpos__2_inlined1_, _1_inlined1) = ((), _endpos_0, _v) in
                    let _v = _menhir_action_359 _1_inlined1 _2 _endpos__2_inlined1_ _startpos_name_ name in
                    _menhir_goto_opn _menhir_stack _v
                | _ ->
                    _eRR ())
            | Tokens.EQ ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                let _menhir_s = MenhirState0897 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.ARBITRARY ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.DOT ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.LBRACKET ->
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.IDENTIFIER _v ->
                        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                        let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.COMMA ->
                            let _menhir_s = MenhirState0904 in
                            let _tok = _menhir_lexer _menhir_lexbuf in
                            (match (_tok : MenhirBasics.token) with
                            | Tokens.IDENTIFIER _v ->
                                _menhir_run_0045 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                            | _ ->
                                _eRR ())
                        | _ ->
                            _eRR ())
                    | _ ->
                        _eRR ())
                | Tokens.IDENTIFIER _v ->
                    let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                    let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.EQ ->
                        let _menhir_s = MenhirState0912 in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.STRING_LIT _v ->
                            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.REAL_LIT _v ->
                            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.NOT ->
                            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.MINUS ->
                            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.LPAR ->
                            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.INT_LIT _v ->
                            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.IF ->
                            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.IDENTIFIER _v ->
                            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.BOOL_LIT _v ->
                            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.BNOT ->
                            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.BITVECTOR_LIT _v ->
                            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.ARBITRARY ->
                            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | _ ->
                            _eRR ())
                    | _ ->
                        _eRR ())
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0854 : type  ttv_stack. (((((ttv_stack, _menhir_box_opn) _menhir_cell1_LET, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_COMMA as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACE ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_RBRACE (_menhir_stack, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LPAR ->
                _menhir_run_0200 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0855
            | Tokens.SEMI_COLON ->
                let _v_0 = _menhir_action_200 () in
                _menhir_run_0856 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v_0 _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0774 : type  ttv_stack. (((((ttv_stack, _menhir_box_opn) _menhir_cell1_VAR, _menhir_box_opn) _menhir_cell1_decl_item, _menhir_box_opn) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, _menhir_box_opn) _menhir_cell1_COMMA as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_opn) _menhir_state -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACE ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_RBRACE (_menhir_stack, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LPAR ->
                _menhir_run_0200 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0775
            | Tokens.SEMI_COLON ->
                let _v_0 = _menhir_action_200 () in
                _menhir_run_0776 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v_0 _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0724 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_VAR, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_COMMA as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACE ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_RBRACE (_menhir_stack, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LPAR ->
                _menhir_run_0200 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0725
            | Tokens.SEMI_COLON ->
                let _v_0 = _menhir_action_200 () in
                _menhir_run_0726 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v_0 MenhirState0725 _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0719 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_VAR, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_RBRACE _menhir_cell0_LPAR as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RPAR ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.SEMI_COLON ->
                let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.WHILE ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0721
                | Tokens.VAR ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0721
                | Tokens.UNREACHABLE ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0721
                | Tokens.TRY ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0721
                | Tokens.THROW ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0721
                | Tokens.RETURN ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0721
                | Tokens.REPEAT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0721
                | Tokens.PRINT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0721
                | Tokens.PRAGMA ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0721
                | Tokens.PASS ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0721
                | Tokens.MINUS ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0721
                | Tokens.LPAR ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0721
                | Tokens.LET ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0721
                | Tokens.IF ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0721
                | Tokens.IDENTIFIER _v_1 ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0721
                | Tokens.FOR ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0721
                | Tokens.DEBUG ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0721
                | Tokens.CONSTANT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0721
                | Tokens.CASE ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0721
                | Tokens.ASSERT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0721
                | Tokens.ARROBASE_LOOPLIMIT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0721
                | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                    let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                    let MenhirCell1_RBRACE (_menhir_stack, _, _) = _menhir_stack in
                    let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                    let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                    let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
                    let MenhirCell1_VAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                    let (_1, _endpos__2_, _2_inlined1, _1_inlined2, _3, _3_inlined1, _endpos__3_inlined2_) = ((), _endpos_0, (), _v, (), (), _endpos) in
                    let _v = _menhir_action_275 _1 _1_inlined2 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name in
                    _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0660 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_PRINT _menhir_cell0_LPAR as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RPAR ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.SEMI_COLON ->
                let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.WHILE ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
                | Tokens.VAR ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
                | Tokens.UNREACHABLE ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
                | Tokens.TRY ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
                | Tokens.THROW ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
                | Tokens.RETURN ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
                | Tokens.REPEAT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
                | Tokens.PRINT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
                | Tokens.PRAGMA ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
                | Tokens.PASS ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
                | Tokens.MINUS ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
                | Tokens.LPAR ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
                | Tokens.LET ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
                | Tokens.IF ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
                | Tokens.IDENTIFIER _v_1 ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0662
                | Tokens.FOR ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
                | Tokens.DEBUG ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
                | Tokens.CONSTANT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
                | Tokens.CASE ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
                | Tokens.ASSERT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
                | Tokens.ARROBASE_LOOPLIMIT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0662
                | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                    let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                    let MenhirCell1_PRINT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                    let (_1, _endpos__2_, _1_inlined2) = ((), _endpos_0, _v) in
                    let _v = _menhir_action_278 _1 _1_inlined2 _endpos__2_ _startpos__1_ in
                    _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0656 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_PRAGMA _menhir_cell0_IDENTIFIER as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SEMI_COLON ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0657
            | Tokens.VAR ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0657
            | Tokens.UNREACHABLE ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0657
            | Tokens.TRY ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0657
            | Tokens.THROW ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0657
            | Tokens.RETURN ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0657
            | Tokens.REPEAT ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0657
            | Tokens.PRINT ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0657
            | Tokens.PRAGMA ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0657
            | Tokens.PASS ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0657
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0657
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0657
            | Tokens.LET ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0657
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0657
            | Tokens.IDENTIFIER _v_0 ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState0657
            | Tokens.FOR ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0657
            | Tokens.DEBUG ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0657
            | Tokens.CONSTANT ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0657
            | Tokens.CASE ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0657
            | Tokens.ASSERT ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0657
            | Tokens.ARROBASE_LOOPLIMIT ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos) in
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0657
            | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                let MenhirCell0_IDENTIFIER (_menhir_stack, x, _, _) = _menhir_stack in
                let MenhirCell1_PRAGMA (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                let (_1, _endpos__2_, _1_inlined1) = ((), _endpos, _v) in
                let _v = _menhir_action_288 _1 _1_inlined1 _endpos__2_ _startpos__1_ x in
                _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0646 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_LET, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_COMMA as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACE ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_RBRACE (_menhir_stack, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LPAR ->
                _menhir_run_0200 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0647
            | Tokens.SEMI_COLON ->
                let _v_0 = _menhir_action_200 () in
                _menhir_run_0648 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v_0 MenhirState0647 _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0641 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_LET, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_RBRACE _menhir_cell0_LPAR as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RPAR ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.SEMI_COLON ->
                let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.WHILE ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
                | Tokens.VAR ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
                | Tokens.UNREACHABLE ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
                | Tokens.TRY ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
                | Tokens.THROW ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
                | Tokens.RETURN ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
                | Tokens.REPEAT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
                | Tokens.PRINT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
                | Tokens.PRAGMA ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
                | Tokens.PASS ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
                | Tokens.MINUS ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
                | Tokens.LPAR ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
                | Tokens.LET ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
                | Tokens.IF ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
                | Tokens.IDENTIFIER _v_1 ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0643
                | Tokens.FOR ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
                | Tokens.DEBUG ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
                | Tokens.CONSTANT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
                | Tokens.CASE ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
                | Tokens.ASSERT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
                | Tokens.ARROBASE_LOOPLIMIT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0643
                | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                    let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                    let MenhirCell1_RBRACE (_menhir_stack, _, _) = _menhir_stack in
                    let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                    let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                    let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
                    let MenhirCell1_LET (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                    let (_1, _endpos__2_, _2_inlined1, _1_inlined2, _3, _3_inlined1, _endpos__3_inlined2_) = ((), _endpos_0, (), _v, (), (), _endpos) in
                    let _v = _menhir_action_265 _1 _1_inlined2 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name in
                    _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0605 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACE ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_RBRACE (_menhir_stack, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LPAR ->
                _menhir_run_0200 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0606
            | Tokens.DOT | Tokens.EQ | Tokens.SEMI_COLON ->
                let _v_0 = _menhir_action_200 () in
                _menhir_run_0607 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v_0 MenhirState0606 _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0581 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RPAR ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.SEMI_COLON ->
                let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.WHILE ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0583
                | Tokens.VAR ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0583
                | Tokens.UNREACHABLE ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0583
                | Tokens.TRY ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0583
                | Tokens.THROW ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0583
                | Tokens.RETURN ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0583
                | Tokens.REPEAT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0583
                | Tokens.PRINT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0583
                | Tokens.PRAGMA ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0583
                | Tokens.PASS ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0583
                | Tokens.MINUS ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0583
                | Tokens.LPAR ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0583
                | Tokens.LET ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0583
                | Tokens.IF ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0583
                | Tokens.IDENTIFIER _v_1 ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0583
                | Tokens.FOR ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0583
                | Tokens.DEBUG ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0583
                | Tokens.CONSTANT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0583
                | Tokens.CASE ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0583
                | Tokens.ASSERT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0583
                | Tokens.ARROBASE_LOOPLIMIT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0583
                | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                    let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                    let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
                    let (_endpos__2_, _1_inlined1) = (_endpos_0, _v) in
                    let _v = _menhir_action_249 _1_inlined1 _endpos__2_ _startpos_name_ name in
                    _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
                | _ ->
                    _eRR ())
            | Tokens.EQ ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                let _menhir_s = MenhirState0585 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.ARBITRARY ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.DOT ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.LBRACKET ->
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.IDENTIFIER _v ->
                        let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                        let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.COMMA ->
                            let _menhir_s = MenhirState0592 in
                            let _tok = _menhir_lexer _menhir_lexbuf in
                            (match (_tok : MenhirBasics.token) with
                            | Tokens.IDENTIFIER _v ->
                                _menhir_run_0045 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                            | _ ->
                                _eRR ())
                        | _ ->
                            _eRR ())
                    | _ ->
                        _eRR ())
                | Tokens.IDENTIFIER _v ->
                    let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                    let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.EQ ->
                        let _menhir_s = MenhirState0600 in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.STRING_LIT _v ->
                            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.REAL_LIT _v ->
                            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.NOT ->
                            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.MINUS ->
                            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.LPAR ->
                            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.INT_LIT _v ->
                            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.IF ->
                            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.IDENTIFIER _v ->
                            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.BOOL_LIT _v ->
                            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.BNOT ->
                            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.BITVECTOR_LIT _v ->
                            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.ARBITRARY ->
                            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | _ ->
                            _eRR ())
                    | _ ->
                        _eRR ())
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0538 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_DEBUG _menhir_cell0_LPAR as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RPAR ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.SEMI_COLON ->
                let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.WHILE ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
                | Tokens.VAR ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
                | Tokens.UNREACHABLE ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
                | Tokens.TRY ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
                | Tokens.THROW ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
                | Tokens.RETURN ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
                | Tokens.REPEAT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
                | Tokens.PRINT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
                | Tokens.PRAGMA ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
                | Tokens.PASS ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
                | Tokens.MINUS ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
                | Tokens.LPAR ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
                | Tokens.LET ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
                | Tokens.IF ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
                | Tokens.IDENTIFIER _v_1 ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0540
                | Tokens.FOR ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
                | Tokens.DEBUG ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
                | Tokens.CONSTANT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
                | Tokens.CASE ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
                | Tokens.ASSERT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
                | Tokens.ARROBASE_LOOPLIMIT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0540
                | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                    let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                    let MenhirCell1_DEBUG (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                    let (_1, _endpos__2_, _1_inlined2) = ((), _endpos_0, _v) in
                    let _v = _menhir_action_280 _1 _1_inlined2 _endpos__2_ _startpos__1_ in
                    _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0529 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_CONSTANT, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_COMMA as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACE ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_RBRACE (_menhir_stack, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LPAR ->
                _menhir_run_0200 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0530
            | Tokens.SEMI_COLON ->
                let _v_0 = _menhir_action_200 () in
                _menhir_run_0531 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v_0 MenhirState0530 _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0524 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_CONSTANT, ttv_result) _menhir_cell1_decl_item, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE, ttv_result) _menhir_cell1_RBRACE _menhir_cell0_LPAR as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RPAR ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.SEMI_COLON ->
                let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.WHILE ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0526
                | Tokens.VAR ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0526
                | Tokens.UNREACHABLE ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0526
                | Tokens.TRY ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0526
                | Tokens.THROW ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0526
                | Tokens.RETURN ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0526
                | Tokens.REPEAT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0526
                | Tokens.PRINT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0526
                | Tokens.PRAGMA ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0526
                | Tokens.PASS ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0526
                | Tokens.MINUS ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0526
                | Tokens.LPAR ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0526
                | Tokens.LET ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0526
                | Tokens.IF ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0526
                | Tokens.IDENTIFIER _v_1 ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState0526
                | Tokens.FOR ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0526
                | Tokens.DEBUG ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0526
                | Tokens.CONSTANT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0526
                | Tokens.CASE ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0526
                | Tokens.ASSERT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0526
                | Tokens.ARROBASE_LOOPLIMIT ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RPAR (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_SEMI_COLON (_menhir_stack, _endpos_0) in
                    _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0526
                | Tokens.CATCH | Tokens.ELSE | Tokens.ELSIF | Tokens.END | Tokens.EOF | Tokens.OTHERWISE | Tokens.UNTIL | Tokens.WHEN ->
                    let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
                    let MenhirCell1_RBRACE (_menhir_stack, _, _) = _menhir_stack in
                    let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
                    let MenhirCell1_IDENTIFIER (_menhir_stack, _, name, _startpos_name_, _) = _menhir_stack in
                    let MenhirCell1_decl_item (_menhir_stack, _, lhs) = _menhir_stack in
                    let MenhirCell1_CONSTANT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
                    let (_1, _endpos__2_, _2_inlined1, _1_inlined2, _3, _3_inlined1, _endpos__3_inlined2_) = ((), _endpos_0, (), _v, (), (), _endpos) in
                    let _v = _menhir_action_268 _1 _1_inlined2 _2_inlined1 _3 _3_inlined1 _endpos__2_ _endpos__3_inlined2_ _startpos__1_ _startpos_name_ lhs name in
                    _menhir_goto_nonempty_list_stmt_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0352 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_LPAR, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_COMMA -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RPAR ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_COMMA (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _, x, _, _) = _menhir_stack in
            let MenhirCell1_LPAR (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_2, _endpos__3_, li) = ((), _endpos, _v) in
            let _v = _menhir_action_069 _2 _endpos__3_ _startpos__1_ li x in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos__1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0343 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RPAR ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
            let (_1_inlined1, _endpos__3_) = (_v, _endpos) in
            let _v = _menhir_action_051 _1_inlined1 _endpos__3_ _startpos_name_ name in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_name_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0212 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACE ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_RBRACE (_menhir_stack, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LPAR ->
                _menhir_run_0200 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0213
            | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DOT | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.IN | Tokens.LBRACKET | Tokens.LEQ | Tokens.LLBRACKET | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RBRACE | Tokens.RDIV | Tokens.RPAR | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.WHERE ->
                let _v_0 = _menhir_action_200 () in
                _menhir_run_0214 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v_0 _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0202 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_expr_ _menhir_cell0_RBRACE, ttv_result) _menhir_cell1_LPAR -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RPAR ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_LPAR (_menhir_stack, _menhir_s, _) = _menhir_stack in
            let (_1_inlined1, _endpos__3_) = (_v, _endpos) in
            let _v = _menhir_action_202 _1_inlined1 in
            _menhir_goto_loption_plist_expr__ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0198 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LBRACE as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_expr_ (_menhir_stack, _menhir_s, _v) in
        match (_tok : MenhirBasics.token) with
        | Tokens.RBRACE ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell0_RBRACE (_menhir_stack, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LPAR ->
                _menhir_run_0200 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0199
            | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DO | Tokens.DOT | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.IN | Tokens.LBRACKET | Tokens.LEQ | Tokens.LLBRACKET | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.POW | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RDIV | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
                let _v_0 = _menhir_action_200 () in
                _menhir_run_0204 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v_0 _tok
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0190 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_COMMA -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        let MenhirCell1_COMMA (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _menhir_s, x, _, _) = _menhir_stack in
        let xs = _v in
        let _v = _menhir_action_420 x xs in
        _menhir_goto_separated_nonempty_list_COMMA_expr_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
    
    and _menhir_run_0186 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_IDENTIFIER _menhir_cell0_LPAR -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.RPAR ->
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell0_LPAR (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_IDENTIFIER (_menhir_stack, _menhir_s, name, _startpos_name_, _) = _menhir_stack in
            let (_1_inlined1, _endpos__3_) = (_v, _endpos) in
            let _v = _menhir_action_106 _1_inlined1 _endpos__3_ _startpos_name_ name in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_name_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0180 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_ELSE as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0180
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0180
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0180
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0180
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0180
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0180
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0180
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0180
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0180
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0180
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0180
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0180
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0180
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0180
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0180
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0180
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0180
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0180
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0180
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0180
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0180
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0180
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0180
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0180
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0180
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0180
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0180
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0180
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0180
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0180
        | Tokens.ARROW | Tokens.BEGIN | Tokens.COLON | Tokens.COMMA | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.LOOPLIMIT | Tokens.OF | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_ELSE (_menhir_stack, _menhir_s) = _menhir_stack in
            let (_1, _endpos__2_, _2) = ((), _endpos, _v) in
            let _v = _menhir_action_012 _1 _2 in
            _menhir_goto_e_else _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__2_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_goto_e_else : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_expr as 'stack) -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _menhir_s _tok ->
        match _menhir_s with
        | MenhirState0347 ->
            _menhir_run_0348 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0174 ->
            _menhir_run_0182 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | MenhirState0178 ->
            _menhir_run_0181 _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0348 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_IF, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_expr -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_expr (_menhir_stack, _, e2, _, _) = _menhir_stack in
        let MenhirCell1_THEN (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, e1, _, _) = _menhir_stack in
        let MenhirCell1_IF (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _3, _endpos_e_else_, e_else) = ((), (), _endpos, _v) in
        let _v = _menhir_action_049 _1 _3 _endpos_e_else_ _startpos__1_ e1 e2 e_else in
        _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e_else_ _startpos__1_ _v _menhir_s _tok
    
    and _menhir_run_0182 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_IF, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_expr -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_expr (_menhir_stack, _, e2, _, _) = _menhir_stack in
        let MenhirCell1_THEN (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, e1, _, _) = _menhir_stack in
        let MenhirCell1_IF (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _3, _endpos_e_else_, e_else) = ((), (), _endpos, _v) in
        let _v = _menhir_action_104 _1 _3 _endpos_e_else_ _startpos__1_ e1 e2 e_else in
        _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e_else_ _startpos__1_ _v _menhir_s _tok
    
    and _menhir_run_0181 : type  ttv_stack ttv_result. (((((((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_ELSIF, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_expr -> _ -> _ -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok ->
        let MenhirCell1_expr (_menhir_stack, _, e, _, _) = _menhir_stack in
        let MenhirCell1_THEN (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, c, _, _) = _menhir_stack in
        let MenhirCell1_ELSIF (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
        let (_1, _3, _endpos_e_else_, e_else) = ((), (), _endpos, _v) in
        let _v = _menhir_action_013 _1 _3 _endpos_e_else_ _startpos__1_ c e e_else in
        _menhir_goto_e_else _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e_else_ _v _menhir_s _tok
    
    and _menhir_run_0178 : type  ttv_stack ttv_result. (((((((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_ELSIF, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0178
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0178
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0178
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0178
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0178
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0178
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0178
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0178
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0178
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0178
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0178
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0178
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0178
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0178
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0178
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0178
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0178
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0178
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0178
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0178
        | Tokens.ELSIF ->
            _menhir_run_0175 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0178
        | Tokens.ELSE ->
            _menhir_run_0179 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0178
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0178
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0178
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0178
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0178
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0178
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0178
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0178
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0178
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0178
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0178
        | _ ->
            _eRR ()
    
    and _menhir_run_0176 : type  ttv_stack ttv_result. (((((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_ELSIF as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.THEN ->
            let _menhir_stack = MenhirCell1_THEN (_menhir_stack, MenhirState0176) in
            let _menhir_s = MenhirState0177 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0176
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0176
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0176
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0176
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0176
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0176
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0176
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0176
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0176
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0176
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0176
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0176
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0176
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0176
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0176
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0176
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0176
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0176
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0176
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0176
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0176
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0176
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0176
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0176
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0176
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0176
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0176
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0176
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0176
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0176
        | _ ->
            _eRR ()
    
    and _menhir_run_0174 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_IF, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_THEN as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0174
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0174
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0174
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0174
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0174
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0174
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0174
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0174
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0174
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0174
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0174
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0174
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0174
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0174
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0174
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0174
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0174
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0174
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0174
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0174
        | Tokens.ELSIF ->
            _menhir_run_0175 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0174
        | Tokens.ELSE ->
            _menhir_run_0179 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0174
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0174
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0174
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0174
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0174
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0174
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0174
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0174
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0174
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0174
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0174
        | _ ->
            _eRR ()
    
    and _menhir_run_0172 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_IF as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.THEN ->
            let _menhir_stack = MenhirCell1_THEN (_menhir_stack, MenhirState0172) in
            let _menhir_s = MenhirState0173 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0172
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0172
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0172
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0172
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0172
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0172
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0172
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0172
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0172
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0172
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0172
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0172
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0172
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0172
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0172
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0172
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0172
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0172
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0172
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0172
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0172
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0172
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0172
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0172
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0172
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0172
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0172
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0172
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0172
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0172
        | _ ->
            _eRR ()
    
    and _menhir_run_0162 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_COLON as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0162
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0162
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0162
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0162
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0162
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0162
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0162
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0162
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0162
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0162
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0162
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0162
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0162
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0162
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0162
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0162
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0162
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0162
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0162
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0162
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0162
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0162
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0162
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0162
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_stack = MenhirCell1_COMMA (_menhir_stack, MenhirState0162) in
            let _menhir_s = MenhirState0163 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.COLON ->
                _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0162
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0162
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0162
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0162
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0162
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0162
        | Tokens.RBRACKET ->
            let MenhirCell1_COLON (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _, _) = _menhir_stack in
            let (_2, e2) = ((), _v) in
            let _v = _menhir_action_438 _2 e1 e2 in
            _menhir_goto_separated_nonempty_list_COMMA_slice_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_goto_separated_nonempty_list_COMMA_slice_ : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        match _menhir_s with
        | MenhirState0419 ->
            _menhir_run_0420 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0032 ->
            _menhir_run_0329 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0327 ->
            _menhir_run_0328 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0262 ->
            _menhir_run_0263 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0163 ->
            _menhir_run_0164 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0159 ->
            _menhir_run_0160 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0157 ->
            _menhir_run_0158 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0153 ->
            _menhir_run_0154 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0123 ->
            _menhir_run_0124 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0420 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_lexpr_atom_desc -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let MenhirCell1_lexpr_atom_desc (_menhir_stack, _menhir_s, desc, _startpos_desc_, _endpos_desc_) = _menhir_stack in
        let (_1_inlined1, _endpos__3_) = (_v, _endpos) in
        let _v = _menhir_action_128 _1_inlined1 _endpos_desc_ _startpos_desc_ desc in
        _menhir_goto_lexpr_atom_desc _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_desc_ _v _menhir_s _tok
    
    and _menhir_run_0329 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_LBRACKET -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let MenhirCell1_LBRACKET (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _menhir_s, e, _startpos_e_, _) = _menhir_stack in
        let (_1_inlined1, _endpos__3_) = (_v, _endpos) in
        let _v = _menhir_action_053 _1_inlined1 _endpos__3_ _startpos_e_ e in
        _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_e_ _v _menhir_s _tok
    
    and _menhir_run_0328 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_COLON, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_COMMA -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_COMMA (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, e, _, _) = _menhir_stack in
        let MenhirCell1_COLON (_menhir_stack, _menhir_s) = _menhir_stack in
        let (_1, xs) = ((), _v) in
        let _v = _menhir_action_445 _1 e xs in
        _menhir_goto_separated_nonempty_list_COMMA_slice_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_0263 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_expr_pattern -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, e, _startpos_e_, _) = _menhir_stack in
        let (_1_inlined1, _endpos__3_) = (_v, _endpos) in
        let _v = _menhir_action_108 _1_inlined1 _endpos__3_ _startpos_e_ e in
        _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_e_ _v _menhir_s _tok
    
    and _menhir_run_0164 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_COLON, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_COMMA -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_COMMA (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, e2, _, _) = _menhir_stack in
        let MenhirCell1_COLON (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _, _) = _menhir_stack in
        let (_2_inlined1, xs) = ((), _v) in
        let _v = _menhir_action_443 _2_inlined1 e1 e2 xs in
        _menhir_goto_separated_nonempty_list_COMMA_slice_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_0160 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_COMMA -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_COMMA (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _menhir_s, expr, _, _) = _menhir_stack in
        let xs = _v in
        let _v = _menhir_action_442 expr xs in
        _menhir_goto_separated_nonempty_list_COMMA_slice_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_0158 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_PLUS_COLON, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_COMMA -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_COMMA (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, e2, _, _) = _menhir_stack in
        let MenhirCell1_PLUS_COLON (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _, _) = _menhir_stack in
        let (_2_inlined1, xs) = ((), _v) in
        let _v = _menhir_action_444 _2_inlined1 e1 e2 xs in
        _menhir_goto_separated_nonempty_list_COMMA_slice_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_0154 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_STAR_COLON, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_COMMA -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_COMMA (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, e2, _, _) = _menhir_stack in
        let MenhirCell1_STAR_COLON (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _, _) = _menhir_stack in
        let (_2_inlined1, xs) = ((), _v) in
        let _v = _menhir_action_446 _2_inlined1 e1 e2 xs in
        _menhir_goto_separated_nonempty_list_COMMA_slice_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_0124 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LBRACKET as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.IDENTIFIER _v_0 ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos_1 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LBRACE ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_slice_ (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_RBRACKET (_menhir_stack, _endpos) in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v_0, _startpos, _endpos_1) in
                let _startpos_2 = _menhir_lexbuf.Lexing.lex_start_p in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_2) in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.LBRACKET ->
                    _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0127
                | Tokens.RBRACE ->
                    let _v_3 = _menhir_action_196 () in
                    _menhir_run_0129 _menhir_stack _menhir_lexbuf _menhir_lexer _v_3 MenhirState0127
                | _ ->
                    _eRR ())
            | Tokens.COMMA ->
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.LBRACKET ->
                    let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_slice_ (_menhir_stack, _menhir_s, _v) in
                    let _menhir_stack = MenhirCell0_RBRACKET (_menhir_stack, _endpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v_0, _startpos, _endpos_1) in
                    _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0133
                | Tokens.RBRACE ->
                    let MenhirCell1_LBRACKET (_menhir_stack, _menhir_s) = _menhir_stack in
                    let (x, _1_inlined1, x_inlined1) = (_v_0, _v, ()) in
                    let _v = _menhir_action_213 _1_inlined1 x x_inlined1 in
                    _menhir_goto_non_empty_trailing_list_COMMA_bitfield_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.COLON_COLON ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_slice_ (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_RBRACKET (_menhir_stack, _endpos) in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v_0, _startpos, _endpos_1) in
                let _menhir_s = MenhirState0135 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.COLON ->
                let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_slice_ (_menhir_stack, _menhir_s, _v) in
                let _menhir_stack = MenhirCell0_RBRACKET (_menhir_stack, _endpos) in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v_0, _startpos, _endpos_1) in
                let _menhir_s = MenhirState0146 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.RBRACE ->
                let MenhirCell1_LBRACKET (_menhir_stack, _menhir_s) = _menhir_stack in
                let (x, _1_inlined1) = (_v_0, _v) in
                let _v = _menhir_action_212 _1_inlined1 x in
                _menhir_goto_non_empty_trailing_list_COMMA_bitfield_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    and _menhir_run_0156 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_PLUS_COLON as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0156
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0156
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0156
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0156
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0156
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0156
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0156
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0156
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0156
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0156
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0156
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0156
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0156
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0156
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0156
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0156
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0156
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0156
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0156
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0156
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0156
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0156
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0156
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0156
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_stack = MenhirCell1_COMMA (_menhir_stack, MenhirState0156) in
            let _menhir_s = MenhirState0157 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.COLON ->
                _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0156
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0156
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0156
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0156
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0156
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0156
        | Tokens.RBRACKET ->
            let MenhirCell1_PLUS_COLON (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _, _) = _menhir_stack in
            let (_2, e2) = ((), _v) in
            let _v = _menhir_action_439 _2 e1 e2 in
            _menhir_goto_separated_nonempty_list_COMMA_slice_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0152 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_STAR_COLON as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0152
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0152
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0152
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0152
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0152
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0152
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0152
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0152
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0152
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0152
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0152
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0152
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0152
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0152
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0152
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0152
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0152
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0152
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0152
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0152
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0152
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0152
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0152
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0152
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_stack = MenhirCell1_COMMA (_menhir_stack, MenhirState0152) in
            let _menhir_s = MenhirState0153 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.COLON ->
                _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0152
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0152
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0152
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0152
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0152
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0152
        | Tokens.RBRACKET ->
            let MenhirCell1_STAR_COLON (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _, _) = _menhir_stack in
            let (_2, e2) = ((), _v) in
            let _v = _menhir_action_441 _2 e1 e2 in
            _menhir_goto_separated_nonempty_list_COMMA_slice_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0150 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.STAR_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_stack = MenhirCell1_STAR_COLON (_menhir_stack, MenhirState0150) in
            let _menhir_s = MenhirState0151 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0150
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0150
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0150
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0150
        | Tokens.PLUS_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_stack = MenhirCell1_PLUS_COLON (_menhir_stack, MenhirState0150) in
            let _menhir_s = MenhirState0155 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0150
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0150
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0150
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0150
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0150
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0150
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0150
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0150
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0150
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0150
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0150
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0150
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0150
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0150
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0150
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0150
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0150
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0150
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0150
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0150
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_stack = MenhirCell1_COMMA (_menhir_stack, MenhirState0150) in
            let _menhir_s = MenhirState0159 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.COLON ->
                _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0150
        | Tokens.COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_stack = MenhirCell1_COLON (_menhir_stack, MenhirState0150) in
            let _menhir_s = MenhirState0161 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0150
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0150
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0150
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0150
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0150
        | Tokens.RBRACKET ->
            let expr = _v in
            let _v = _menhir_action_437 expr in
            _menhir_goto_separated_nonempty_list_COMMA_slice_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0139 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_ARRAY as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0139
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0139
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0139
        | Tokens.RBRACKET ->
            let _menhir_s = MenhirState0139 in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_RBRACKET (_menhir_stack, _menhir_s, _endpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.OF ->
                let _menhir_s = MenhirState0141 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING ->
                    _menhir_run_0017 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.REAL ->
                    _menhir_run_0018 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INTEGER ->
                    _menhir_run_0021 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOLEAN ->
                    _menhir_run_0117 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITS ->
                    _menhir_run_0118 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BIT ->
                    _menhir_run_0136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.ARRAY ->
                    _menhir_run_0137 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0139
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0139
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0139
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0139
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0139
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0139
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0139
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0139
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0139
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0139
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0139
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0139
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0139
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0139
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0139
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0139
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0139
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0139
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0139
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0139
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0139
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0139
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0139
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0139
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0139
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0139
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0139
        | _ ->
            _eRR ()
    
    and _menhir_run_0120 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_BITS _menhir_cell0_LPAR as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.SHL ->
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.RPAR ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0120, _endpos_0) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LBRACE ->
                let _startpos_1 = _menhir_lexbuf.Lexing.lex_start_p in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos_1) in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.LBRACKET ->
                    _menhir_run_0123 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0122
                | Tokens.RBRACE ->
                    let _v_2 = _menhir_action_196 () in
                    _menhir_run_0165 _menhir_stack _menhir_lexbuf _menhir_lexer _v_2
                | _ ->
                    _eRR ())
            | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DO | Tokens.DOT | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.IN | Tokens.LBRACKET | Tokens.LEQ | Tokens.LLBRACKET | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.POW | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RDIV | Tokens.RECURSELIMIT | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.STAR_COLON | Tokens.SUBTYPES | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
                let _endpos = _endpos_0 in
                let _v = _menhir_action_192 () in
                _menhir_goto_loption_bitfields_ _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _v _tok
            | _ ->
                _eRR ())
        | Tokens.RDIV ->
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.POW ->
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.PLUS ->
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.OR ->
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.NEQ ->
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.MUL ->
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.MOD ->
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.MINUS ->
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.LT ->
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.LLBRACKET ->
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.LEQ ->
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.LBRACKET ->
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.IN ->
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.IMPL ->
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.GT ->
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.GEQ ->
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.EQ_OP ->
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.EOR ->
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.DOT ->
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.DIVRM ->
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.DIV ->
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.CONCAT ->
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.COLON_COLON ->
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.BOR ->
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.BEQ ->
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.BAND ->
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.AS ->
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | Tokens.AND ->
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0120
        | _ ->
            _eRR ()
    
    and _menhir_run_0111 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SLICING as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_stack = MenhirCell1_COMMA (_menhir_stack, MenhirState0111) in
            let _menhir_s = MenhirState0112 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0111
        | Tokens.RBRACE ->
            let MenhirCell1_SLICING (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _, _) = _menhir_stack in
            let (_2, e2) = ((), _v) in
            let _v = _menhir_action_424 _2 e1 e2 in
            _menhir_goto_separated_nonempty_list_COMMA_int_constraint_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_goto_separated_nonempty_list_COMMA_int_constraint_ : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
        match _menhir_s with
        | MenhirState0022 ->
            _menhir_run_0332 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0304 ->
            _menhir_run_0307 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0114 ->
            _menhir_run_0115 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0112 ->
            _menhir_run_0113 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | MenhirState0104 ->
            _menhir_run_0107 _menhir_stack _menhir_lexbuf _menhir_lexer _v
        | _ ->
            _menhir_fail ()
    
    and _menhir_run_0332 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_INTEGER _menhir_cell0_LBRACE -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let MenhirCell0_LBRACE (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_INTEGER (_menhir_stack, _menhir_s, _startpos__1_, _) = _menhir_stack in
        let (_1, _1_inlined2, _endpos__3_) = ((), _v, _endpos) in
        let _v = _menhir_action_455 _1 _1_inlined2 _endpos__3_ _startpos__1_ in
        _menhir_goto_ty _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _v _menhir_s _tok
    
    and _menhir_run_0307 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr_pattern, ttv_result) _menhir_cell1_LBRACE -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let MenhirCell1_LBRACE (_menhir_stack, _, _startpos__1_) = _menhir_stack in
        let MenhirCell1_expr_pattern (_menhir_stack, _menhir_s, sub_expr, _startpos_sub_expr_, _) = _menhir_stack in
        let (_2, _1_inlined1, _endpos__3_) = ((), _v, _endpos) in
        let _v = _menhir_action_113 _1_inlined1 _2 _endpos__3_ _startpos__1_ _startpos_sub_expr_ sub_expr in
        _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_sub_expr_ _v _menhir_s _tok
    
    and _menhir_run_0115 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_COMMA -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_COMMA (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _menhir_s, expr, _, _) = _menhir_stack in
        let xs = _v in
        let _v = _menhir_action_425 expr xs in
        _menhir_goto_separated_nonempty_list_COMMA_int_constraint_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_0113 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SLICING, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_COMMA -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let MenhirCell1_COMMA (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _, e2, _, _) = _menhir_stack in
        let MenhirCell1_SLICING (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _, _) = _menhir_stack in
        let (_2_inlined1, xs) = ((), _v) in
        let _v = _menhir_action_426 _2_inlined1 e1 e2 xs in
        _menhir_goto_separated_nonempty_list_COMMA_int_constraint_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
    
    and _menhir_run_0107 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_AS, ttv_result) _menhir_cell1_LBRACE -> _ -> _ -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
        let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
        let _tok = _menhir_lexer _menhir_lexbuf in
        let MenhirCell1_LBRACE (_menhir_stack, _, _startpos__1_) = _menhir_stack in
        let MenhirCell1_AS (_menhir_stack, _) = _menhir_stack in
        let MenhirCell1_expr (_menhir_stack, _menhir_s, sub_expr, _startpos_sub_expr_, _) = _menhir_stack in
        let (_2, _1_inlined1, _endpos__3_) = ((), _v, _endpos) in
        let _v = _menhir_action_058 _1_inlined1 _2 _endpos__3_ _startpos__1_ _startpos_sub_expr_ sub_expr in
        _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__3_ _startpos_sub_expr_ _v _menhir_s _tok
    
    and _menhir_run_0109 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SLICING ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_stack = MenhirCell1_SLICING (_menhir_stack, MenhirState0109) in
            let _menhir_s = MenhirState0110 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_stack = MenhirCell1_COMMA (_menhir_stack, MenhirState0109) in
            let _menhir_s = MenhirState0114 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0109
        | Tokens.RBRACE ->
            let expr = _v in
            let _v = _menhir_action_423 expr in
            _menhir_goto_separated_nonempty_list_COMMA_int_constraint_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0102 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_BAND as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0102
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0102
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0102
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0102
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0102
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0102
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0102
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0102
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0102
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0102
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0102
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0102
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0102
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0102
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0102
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0102
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0102
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0102
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0102
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0102
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0102
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0102
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0102
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0102
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0102
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COMMA | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.IMPL | Tokens.LOOPLIMIT | Tokens.OF | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_BAND (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_022 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0100 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_BEQ as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0100
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0100
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0100
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0100
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0100
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0100
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0100
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0100
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0100
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0100
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0100
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0100
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0100
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0100
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0100
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0100
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0100
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0100
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0100
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0100
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0100
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0100
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0100
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0100
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0100
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COMMA | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.IMPL | Tokens.LOOPLIMIT | Tokens.OF | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_BEQ (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_024 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0098 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_BOR as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0098
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0098
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0098
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0098
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0098
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0098
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0098
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0098
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0098
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0098
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0098
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0098
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0098
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0098
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0098
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0098
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0098
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0098
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0098
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0098
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0098
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0098
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0098
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0098
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0098
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COMMA | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.IMPL | Tokens.LOOPLIMIT | Tokens.OF | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_BOR (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_023 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0096 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_EQ_OP as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0096
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0096
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0096
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0096
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0096
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0096
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0096
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0096
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0096
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0096
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0096
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0096
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0096
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0096
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0096
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0096
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0096
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0096
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0096
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0096
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0096
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0096
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0096
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COMMA | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EQ_OP | Tokens.IMPL | Tokens.LOOPLIMIT | Tokens.NEQ | Tokens.OF | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_EQ_OP (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_028 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0093 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_IMPL as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0093
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0093
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0093
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0093
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0093
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0093
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0093
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0093
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0093
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0093
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0093
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0093
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0093
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0093
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0093
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0093
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0093
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0093
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0093
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0093
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0093
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0093
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0093
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0093
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0093
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COMMA | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.IMPL | Tokens.LOOPLIMIT | Tokens.OF | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_IMPL (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_032 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0091 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_GEQ as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0091
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0091
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0091
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0091
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0091
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0091
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0091
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0091
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0091
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0091
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0091
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0091
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0091
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0091
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0091
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0091
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0091
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0091
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0091
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COMMA | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EQ_OP | Tokens.IMPL | Tokens.LOOPLIMIT | Tokens.NEQ | Tokens.OF | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_GEQ (_menhir_stack, _, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_031 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0089 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_GT as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0089
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0089
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0089
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0089
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0089
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0089
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0089
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0089
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0089
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0089
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0089
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0089
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0089
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0089
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0089
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0089
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0089
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0089
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0089
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COMMA | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EQ_OP | Tokens.IMPL | Tokens.LOOPLIMIT | Tokens.NEQ | Tokens.OF | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_GT (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_030 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0087 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_LEQ as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0087
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0087
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0087
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0087
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0087
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0087
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0087
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0087
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0087
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0087
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0087
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0087
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0087
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0087
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0087
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0087
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0087
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0087
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0087
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COMMA | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EQ_OP | Tokens.IMPL | Tokens.LOOPLIMIT | Tokens.NEQ | Tokens.OF | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_LEQ (_menhir_stack, _, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_034 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0085 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_AND as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0085
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0085
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0085
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0085
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0085
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0085
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0085
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0085
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0085
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0085
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0085
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0085
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0085
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_AND (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_021 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0083 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_COLON_COLON as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0083
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0083
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0083
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0083
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0083
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0083
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0083
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0083
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0083
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0083
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0083
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0083
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0083
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_COLON_COLON (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_044 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0081 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_EOR as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0081
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0081
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0081
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0081
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0081
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0081
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0081
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0081
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0081
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0081
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0081
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0081
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0081
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_EOR (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_027 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0079 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_LT as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0079
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0079
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0079
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0079
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0079
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0079
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0079
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0079
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0079
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0079
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0079
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0079
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0079
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0079
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0079
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0079
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0079
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0079
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0079
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COMMA | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EQ_OP | Tokens.IMPL | Tokens.LOOPLIMIT | Tokens.NEQ | Tokens.OF | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_LT (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_033 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0077 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_MINUS as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0077
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0077
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0077
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0077
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0077
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0077
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0077
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0077
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0077
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0077
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0077
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0077
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0077
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_MINUS (_menhir_stack, _, _, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_036 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0075 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_NEQ as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0075
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0075
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0075
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0075
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0075
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0075
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0075
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0075
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0075
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0075
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0075
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0075
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0075
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0075
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0075
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0075
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0075
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0075
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0075
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0075
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0075
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0075
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0075
        | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COMMA | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EQ_OP | Tokens.IMPL | Tokens.LOOPLIMIT | Tokens.NEQ | Tokens.OF | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_NEQ (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_029 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0072 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_OR as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0072
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0072
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0072
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0072
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0072
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0072
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0072
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0072
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0072
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0072
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0072
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0072
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0072
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_OR (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_039 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0070 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DIV as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0070
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0070
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0070
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0070
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0070
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0070
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DIV | Tokens.DIVRM | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RDIV | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_DIV (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_025 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0068 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_DIVRM as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0068
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0068
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0068
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0068
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0068
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0068
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DIV | Tokens.DIVRM | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RDIV | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_DIVRM (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_026 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0066 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_MOD as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0066
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0066
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0066
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0066
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0066
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0066
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DIV | Tokens.DIVRM | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RDIV | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_MOD (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_037 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0064 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_MUL as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0064
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0064
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0064
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0064
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0064
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0064
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DIV | Tokens.DIVRM | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RDIV | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_MUL (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_038 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0062 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_PLUS as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0062
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0062
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0062
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0062
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0062
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0062
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0062
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0062
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0062
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0062
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0062
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0062
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0062
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_PLUS (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_035 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0060 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_CONCAT as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0060
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0060
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0060
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0060
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.POW | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RDIV | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_CONCAT (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_045 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0058 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_RDIV as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0058
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0058
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0058
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0058
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0058
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0058
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DIV | Tokens.DIVRM | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RDIV | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_RDIV (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_040 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0056 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LEQ as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0056
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0056
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0056
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0056
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0056
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0056
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0056
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0056
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0056
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0056
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0056
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0056
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0056
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0056
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0056
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0056
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0056
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0056
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0056
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0056
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0056
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0056
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0056
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0056
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0056
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0056
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0056
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0056
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0056
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0056
        | Tokens.ARROW | Tokens.COMMA | Tokens.RBRACE | Tokens.RPAR | Tokens.WHERE ->
            let MenhirCell1_LEQ (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _endpos_expr_, expr) = ((), _endpos, _v) in
            let _v = _menhir_action_406 _1 _endpos_expr_ _startpos__1_ expr in
            _menhir_goto_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_expr_ _startpos__1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0052 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_MINUS as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0052
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0052
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0052
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0052
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RBRACE | Tokens.RDIV | Tokens.RPAR | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_MINUS (_menhir_stack, _menhir_s, _startpos__1_, _) = _menhir_stack in
            let (_1, _endpos_e_, e) = ((), _endpos, _v) in
            let _v = _menhir_action_102 _1 _endpos_e_ _startpos__1_ e in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e_ _startpos__1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0042 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_NOT as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0042
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0042
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0042
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0042
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEQ | Tokens.BOR | Tokens.COLON_COLON | Tokens.COMMA | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OR | Tokens.PLUS | Tokens.POW | Tokens.RBRACE | Tokens.RDIV | Tokens.RPAR | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.WHERE ->
            let MenhirCell1_NOT (_menhir_stack, _menhir_s, _startpos__1_) = _menhir_stack in
            let (_1, _endpos_e_, e) = ((), _endpos, _v) in
            let _v = _menhir_action_103 _1 _endpos_e_ _startpos__1_ e in
            _menhir_goto_expr_pattern _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e_ _startpos__1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0036 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SHL as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0036
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0036
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0036
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0036
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0036
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0036
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DIV | Tokens.DIVRM | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RDIV | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_SHL (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_041 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0034 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_COLON as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0034
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0034
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0034
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0034
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0034
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0034
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0034
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0034
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0034
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0034
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0034
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0034
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0034
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0034
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0034
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0034
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0034
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0034
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0034
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0034
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0034
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0034
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0034
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0034
        | Tokens.COMMA ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            let _menhir_stack = MenhirCell1_COMMA (_menhir_stack, MenhirState0034) in
            let _menhir_s = MenhirState0327 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.COLON ->
                _menhir_run_0033 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0034
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0034
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0034
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0034
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0034
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0034
        | Tokens.RBRACKET ->
            let MenhirCell1_COLON (_menhir_stack, _menhir_s) = _menhir_stack in
            let (_1, e) = ((), _v) in
            let _v = _menhir_action_440 _1 e in
            _menhir_goto_separated_nonempty_list_COMMA_slice_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | _ ->
            _eRR ()
    
    and _menhir_run_0031 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_POW as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0031
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0031
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0031
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0031
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.POW | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RDIV | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_POW (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_043 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0029 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_SHR as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0029
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0029
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0029
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0029
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0029
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0029
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.DIV | Tokens.DIVRM | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RDIV | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_SHR (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_1, _endpos_e2_, e2) = ((), _endpos, _v) in
            let _v = _menhir_action_042 _1 _endpos_e2_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e2_ _startpos_e1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    and _menhir_run_0027 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_expr, ttv_result) _menhir_cell1_LLBRACKET as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.SHR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0028 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0027
        | Tokens.SHL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0035 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0027
        | Tokens.RRBRACKET ->
            let _endpos_0 = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            let MenhirCell1_LLBRACKET (_menhir_stack, _) = _menhir_stack in
            let MenhirCell1_expr (_menhir_stack, _menhir_s, e1, _startpos_e1_, _) = _menhir_stack in
            let (_2, _4, e2, _endpos__4_) = ((), (), _v, _endpos_0) in
            let _v = _menhir_action_054 _2 _4 _endpos__4_ _startpos_e1_ e1 e2 in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__4_ _startpos_e1_ _v _menhir_s _tok
        | Tokens.RDIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0057 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0027
        | Tokens.POW ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0030 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0027
        | Tokens.PLUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0061 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0027
        | Tokens.OR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0071 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0027
        | Tokens.NEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0073 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0027
        | Tokens.MUL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0063 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0027
        | Tokens.MOD ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0065 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0027
        | Tokens.MINUS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0076 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0027
        | Tokens.LT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0078 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0027
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0027
        | Tokens.LEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0086 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0027
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0027
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0027
        | Tokens.IMPL ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0092 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0027
        | Tokens.GT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0027
        | Tokens.GEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0090 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0027
        | Tokens.EQ_OP ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0027
        | Tokens.EOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0080 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0027
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0027
        | Tokens.DIVRM ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0067 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0027
        | Tokens.DIV ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0069 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0027
        | Tokens.CONCAT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0027
        | Tokens.COLON_COLON ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0082 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0027
        | Tokens.BOR ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0097 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0027
        | Tokens.BEQ ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0099 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0027
        | Tokens.BAND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0101 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0027
        | Tokens.AS ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0027
        | Tokens.AND ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0084 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0027
        | _ ->
            _eRR ()
    
    and _menhir_run_0025 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_MINUS as 'stack) -> _ -> _ -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer _endpos _startpos _v _menhir_s _tok ->
        match (_tok : MenhirBasics.token) with
        | Tokens.LLBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0026 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0025
        | Tokens.LBRACKET ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0032 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0025
        | Tokens.IN ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0037 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0025
        | Tokens.DOT ->
            let _menhir_stack = MenhirCell1_expr (_menhir_stack, _menhir_s, _v, _startpos, _endpos) in
            _menhir_run_0043 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0025
        | Tokens.AND | Tokens.ARROW | Tokens.AS | Tokens.BAND | Tokens.BEGIN | Tokens.BEQ | Tokens.BOR | Tokens.COLON | Tokens.COLON_COLON | Tokens.COMMA | Tokens.CONCAT | Tokens.DIV | Tokens.DIVRM | Tokens.DO | Tokens.DOWNTO | Tokens.ELSE | Tokens.ELSIF | Tokens.EOR | Tokens.EQ_OP | Tokens.GEQ | Tokens.GT | Tokens.IMPL | Tokens.LEQ | Tokens.LOOPLIMIT | Tokens.LT | Tokens.MINUS | Tokens.MOD | Tokens.MUL | Tokens.NEQ | Tokens.OF | Tokens.OR | Tokens.PLUS | Tokens.PLUS_COLON | Tokens.POW | Tokens.RBRACE | Tokens.RBRACKET | Tokens.RDIV | Tokens.RPAR | Tokens.RRBRACKET | Tokens.SEMI_COLON | Tokens.SHL | Tokens.SHR | Tokens.SLICING | Tokens.STAR_COLON | Tokens.THEN | Tokens.TO | Tokens.WHERE ->
            let MenhirCell1_MINUS (_menhir_stack, _menhir_s, _startpos__1_, _) = _menhir_stack in
            let (_1, _endpos_e_, e) = ((), _endpos, _v) in
            let _v = _menhir_action_047 _1 _endpos_e_ _startpos__1_ e in
            _menhir_goto_expr _menhir_stack _menhir_lexbuf _menhir_lexer _endpos_e_ _startpos__1_ _v _menhir_s _tok
        | _ ->
            _eRR ()
    
    let _menhir_run_0000 : type  ttv_stack. ttv_stack -> _ -> _ -> _menhir_box_opn =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.WHILE ->
            let _menhir_s = MenhirState0000 in
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell1_WHILE (_menhir_stack, _menhir_s, _startpos) in
            let _menhir_s = MenhirState0001 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.VAR ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell1_VAR (_menhir_stack, MenhirState0000, _startpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.MINUS ->
                _menhir_run_0365 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0746
            | Tokens.LPAR ->
                _menhir_run_0370 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0746
            | Tokens.IDENTIFIER _v ->
                let _startpos_0 = _menhir_lexbuf.Lexing.lex_start_p in
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.COMMA ->
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0746, _v, _startpos_0, _endpos) in
                    let _menhir_s = MenhirState0748 in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.IDENTIFIER _v ->
                        _menhir_run_0045 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | _ ->
                        _eRR ())
                | Tokens.COLON_COLON ->
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0746, _v, _startpos_0, _endpos) in
                    _menhir_run_0372 _menhir_stack _menhir_lexbuf _menhir_lexer
                | Tokens.COLON ->
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0746, _v, _startpos_0, _endpos) in
                    _menhir_run_0374 _menhir_stack _menhir_lexbuf _menhir_lexer
                | Tokens.EQ | Tokens.SEMI_COLON ->
                    let _v =
                      let _1 = _v in
                      _menhir_action_009 _1
                    in
                    _menhir_run_0758 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0746 _tok
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.UNREACHABLE ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LPAR ->
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.RPAR ->
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.SEMI_COLON ->
                        let _endpos_3 = _menhir_lexbuf.Lexing.lex_curr_p in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.EOF ->
                            let (_startpos__1_, _1, _2, _2_inlined1, _3, _endpos__2_inlined2_) = (_startpos, (), (), (), (), _endpos_3) in
                            let _v = _menhir_action_391 _1 _2 _2_inlined1 _3 _endpos__2_inlined2_ _startpos__1_ in
                            _menhir_goto_opn _menhir_stack _v
                        | _ ->
                            _eRR ())
                    | _ ->
                        _eRR ())
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.TRY ->
            let _menhir_s = MenhirState0000 in
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell1_TRY (_menhir_stack, _menhir_s, _startpos) in
            let _menhir_s = MenhirState0787 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.THROW ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, MenhirState0000, _startpos) in
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0798
            | Tokens.SEMI_COLON ->
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.EOF ->
                    let (_startpos__1_, _1, _2, _endpos__2_inlined1_) = (_startpos, (), (), _endpos) in
                    let _v = _menhir_action_396 _1 _2 _endpos__2_inlined1_ _startpos__1_ in
                    _menhir_goto_opn _menhir_stack _v
                | _ ->
                    _eRR ())
            | Tokens.REAL_LIT _v ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, MenhirState0000, _startpos) in
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0798
            | Tokens.NOT ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, MenhirState0000, _startpos) in
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0798
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, MenhirState0000, _startpos) in
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0798
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, MenhirState0000, _startpos) in
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0798
            | Tokens.INT_LIT _v ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, MenhirState0000, _startpos) in
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0798
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, MenhirState0000, _startpos) in
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0798
            | Tokens.IDENTIFIER _v ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, MenhirState0000, _startpos) in
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0798
            | Tokens.BOOL_LIT _v ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, MenhirState0000, _startpos) in
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0798
            | Tokens.BNOT ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, MenhirState0000, _startpos) in
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0798
            | Tokens.BITVECTOR_LIT _v ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, MenhirState0000, _startpos) in
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0798
            | Tokens.ARBITRARY ->
                let _menhir_stack = MenhirCell1_THROW (_menhir_stack, MenhirState0000, _startpos) in
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0798
            | _ ->
                _eRR ())
        | Tokens.RETURN ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, MenhirState0000, _startpos) in
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0804
            | Tokens.SEMI_COLON ->
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.EOF ->
                    let (_startpos__1_, _1, _2, _endpos__2_inlined1_) = (_startpos, (), (), _endpos) in
                    let _v = _menhir_action_356 _1 _2 _endpos__2_inlined1_ _startpos__1_ in
                    _menhir_goto_opn _menhir_stack _v
                | _ ->
                    _eRR ())
            | Tokens.REAL_LIT _v ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, MenhirState0000, _startpos) in
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0804
            | Tokens.NOT ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, MenhirState0000, _startpos) in
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0804
            | Tokens.MINUS ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, MenhirState0000, _startpos) in
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0804
            | Tokens.LPAR ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, MenhirState0000, _startpos) in
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0804
            | Tokens.INT_LIT _v ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, MenhirState0000, _startpos) in
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0804
            | Tokens.IF ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, MenhirState0000, _startpos) in
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0804
            | Tokens.IDENTIFIER _v ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, MenhirState0000, _startpos) in
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0804
            | Tokens.BOOL_LIT _v ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, MenhirState0000, _startpos) in
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0804
            | Tokens.BNOT ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, MenhirState0000, _startpos) in
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0804
            | Tokens.BITVECTOR_LIT _v ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, MenhirState0000, _startpos) in
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0804
            | Tokens.ARBITRARY ->
                let _menhir_stack = MenhirCell1_RETURN (_menhir_stack, MenhirState0000, _startpos) in
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0804
            | _ ->
                _eRR ())
        | Tokens.REPEAT ->
            let _menhir_s = MenhirState0000 in
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell1_REPEAT (_menhir_stack, _menhir_s, _startpos) in
            let _menhir_s = MenhirState0810 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.WHILE ->
                _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.VAR ->
                _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.UNREACHABLE ->
                _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.TRY ->
                _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.THROW ->
                _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.RETURN ->
                _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.REPEAT ->
                _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRINT ->
                _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PRAGMA ->
                _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.PASS ->
                _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LET ->
                _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IF ->
                _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.FOR ->
                _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.DEBUG ->
                _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CONSTANT ->
                _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.CASE ->
                _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ASSERT ->
                _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.ARROBASE_LOOPLIMIT ->
                _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.PRINT ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LPAR ->
                let _startpos_4 = _menhir_lexbuf.Lexing.lex_start_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING_LIT _v ->
                    let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, MenhirState0000, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_4) in
                    _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0821
                | Tokens.RPAR ->
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.SEMI_COLON ->
                        let _endpos_5 = _menhir_lexbuf.Lexing.lex_curr_p in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.EOF ->
                            let (_startpos__1_, _1, _2, _endpos__2_inlined1_) = (_startpos, (), (), _endpos_5) in
                            let _v = _menhir_action_387 _1 _2 _endpos__2_inlined1_ _startpos__1_ in
                            _menhir_goto_opn _menhir_stack _v
                        | _ ->
                            _eRR ())
                    | _ ->
                        _eRR ())
                | Tokens.REAL_LIT _v ->
                    let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, MenhirState0000, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_4) in
                    _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0821
                | Tokens.NOT ->
                    let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, MenhirState0000, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_4) in
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0821
                | Tokens.MINUS ->
                    let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, MenhirState0000, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_4) in
                    _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0821
                | Tokens.LPAR ->
                    let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, MenhirState0000, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_4) in
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0821
                | Tokens.INT_LIT _v ->
                    let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, MenhirState0000, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_4) in
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0821
                | Tokens.IF ->
                    let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, MenhirState0000, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_4) in
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0821
                | Tokens.IDENTIFIER _v ->
                    let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, MenhirState0000, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_4) in
                    _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0821
                | Tokens.BOOL_LIT _v ->
                    let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, MenhirState0000, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_4) in
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0821
                | Tokens.BNOT ->
                    let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, MenhirState0000, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_4) in
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0821
                | Tokens.BITVECTOR_LIT _v ->
                    let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, MenhirState0000, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_4) in
                    _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0821
                | Tokens.ARBITRARY ->
                    let _menhir_stack = MenhirCell1_PRINT (_menhir_stack, MenhirState0000, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_4) in
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0821
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.PRAGMA ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.IDENTIFIER _v ->
                let _startpos_6 = _menhir_lexbuf.Lexing.lex_start_p in
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING_LIT _v_7 ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, MenhirState0000, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_6, _endpos) in
                    _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v_7 MenhirState0830
                | Tokens.SEMI_COLON ->
                    let _endpos_8 = _menhir_lexbuf.Lexing.lex_curr_p in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.EOF ->
                        let (_startpos__1_, _1, _2, _endpos__2_inlined1_, x) = (_startpos, (), (), _endpos_8, _v) in
                        let _v = _menhir_action_397 _1 _2 _endpos__2_inlined1_ _startpos__1_ x in
                        _menhir_goto_opn _menhir_stack _v
                    | _ ->
                        _eRR ())
                | Tokens.REAL_LIT _v_9 ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, MenhirState0000, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_6, _endpos) in
                    _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v_9 MenhirState0830
                | Tokens.NOT ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, MenhirState0000, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_6, _endpos) in
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0830
                | Tokens.MINUS ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, MenhirState0000, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_6, _endpos) in
                    _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0830
                | Tokens.LPAR ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, MenhirState0000, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_6, _endpos) in
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0830
                | Tokens.INT_LIT _v_10 ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, MenhirState0000, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_6, _endpos) in
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v_10 MenhirState0830
                | Tokens.IF ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, MenhirState0000, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_6, _endpos) in
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0830
                | Tokens.IDENTIFIER _v_11 ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, MenhirState0000, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_6, _endpos) in
                    _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v_11 MenhirState0830
                | Tokens.BOOL_LIT _v_12 ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, MenhirState0000, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_6, _endpos) in
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_12 MenhirState0830
                | Tokens.BNOT ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, MenhirState0000, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_6, _endpos) in
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0830
                | Tokens.BITVECTOR_LIT _v_13 ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, MenhirState0000, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_6, _endpos) in
                    _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v_13 MenhirState0830
                | Tokens.ARBITRARY ->
                    let _menhir_stack = MenhirCell1_PRAGMA (_menhir_stack, MenhirState0000, _startpos) in
                    let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos_6, _endpos) in
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0830
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.PASS ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.SEMI_COLON ->
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.EOF ->
                    let (_startpos__1_, _1, _2, _endpos__2_inlined1_) = (_startpos, (), (), _endpos) in
                    let _v = _menhir_action_355 _1 _2 _endpos__2_inlined1_ _startpos__1_ in
                    _menhir_goto_opn _menhir_stack _v
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.MINUS ->
            _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0000
        | Tokens.LPAR ->
            _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0000
        | Tokens.LET ->
            let _menhir_s = MenhirState0000 in
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell1_LET (_menhir_stack, _menhir_s, _startpos) in
            let _menhir_s = MenhirState0839 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.MINUS ->
                _menhir_run_0365 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0370 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0371 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.IF ->
            let _menhir_s = MenhirState0000 in
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell1_IF (_menhir_stack, _menhir_s, _startpos) in
            let _menhir_s = MenhirState0862 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.IDENTIFIER _v ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LPAR ->
                let _startpos_14 = _menhir_lexbuf.Lexing.lex_start_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING_LIT _v_15 ->
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0000, _v, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_14) in
                    _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v_15 MenhirState0870
                | Tokens.RPAR ->
                    let _endpos_16 = _menhir_lexbuf.Lexing.lex_curr_p in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.SEMI_COLON ->
                        let _endpos_17 = _menhir_lexbuf.Lexing.lex_curr_p in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.EOF ->
                            let (_2, _endpos__2_inlined1_, _startpos_name_, name) = ((), _endpos_17, _startpos, _v) in
                            let _v = _menhir_action_358 _2 _endpos__2_inlined1_ _startpos_name_ name in
                            _menhir_goto_opn _menhir_stack _v
                        | _ ->
                            _eRR ())
                    | Tokens.EQ ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0000, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_14) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0870, _endpos_16) in
                        let _menhir_s = MenhirState0874 in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.STRING_LIT _v ->
                            _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.REAL_LIT _v ->
                            _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.NOT ->
                            _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.MINUS ->
                            _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.LPAR ->
                            _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.INT_LIT _v ->
                            _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.IF ->
                            _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.IDENTIFIER _v ->
                            _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.BOOL_LIT _v ->
                            _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.BNOT ->
                            _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | Tokens.BITVECTOR_LIT _v ->
                            _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                        | Tokens.ARBITRARY ->
                            _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                        | _ ->
                            _eRR ())
                    | Tokens.DOT ->
                        let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0000, _v, _startpos, _endpos) in
                        let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_14) in
                        let _menhir_stack = MenhirCell1_RPAR (_menhir_stack, MenhirState0870, _endpos_16) in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.LBRACKET ->
                            let _tok = _menhir_lexer _menhir_lexbuf in
                            (match (_tok : MenhirBasics.token) with
                            | Tokens.IDENTIFIER _v ->
                                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
                                let _tok = _menhir_lexer _menhir_lexbuf in
                                (match (_tok : MenhirBasics.token) with
                                | Tokens.COMMA ->
                                    let _menhir_s = MenhirState0881 in
                                    let _tok = _menhir_lexer _menhir_lexbuf in
                                    (match (_tok : MenhirBasics.token) with
                                    | Tokens.IDENTIFIER _v ->
                                        _menhir_run_0045 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                                    | _ ->
                                        _eRR ())
                                | _ ->
                                    _eRR ())
                            | _ ->
                                _eRR ())
                        | Tokens.IDENTIFIER _v ->
                            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                            let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                            let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
                            let _tok = _menhir_lexer _menhir_lexbuf in
                            (match (_tok : MenhirBasics.token) with
                            | Tokens.EQ ->
                                let _menhir_s = MenhirState0889 in
                                let _tok = _menhir_lexer _menhir_lexbuf in
                                (match (_tok : MenhirBasics.token) with
                                | Tokens.STRING_LIT _v ->
                                    _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                                | Tokens.REAL_LIT _v ->
                                    _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                                | Tokens.NOT ->
                                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                                | Tokens.MINUS ->
                                    _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                                | Tokens.LPAR ->
                                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                                | Tokens.INT_LIT _v ->
                                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                                | Tokens.IF ->
                                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                                | Tokens.IDENTIFIER _v ->
                                    _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                                | Tokens.BOOL_LIT _v ->
                                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                                | Tokens.BNOT ->
                                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                                | Tokens.BITVECTOR_LIT _v ->
                                    _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                                | Tokens.ARBITRARY ->
                                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                                | _ ->
                                    _eRR ())
                            | _ ->
                                _eRR ())
                        | _ ->
                            _eRR ())
                    | _ ->
                        _eRR ())
                | Tokens.REAL_LIT _v_37 ->
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0000, _v, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_14) in
                    _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v_37 MenhirState0870
                | Tokens.NOT ->
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0000, _v, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_14) in
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0870
                | Tokens.MINUS ->
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0000, _v, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_14) in
                    _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0870
                | Tokens.LPAR ->
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0000, _v, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_14) in
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0870
                | Tokens.INT_LIT _v_38 ->
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0000, _v, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_14) in
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v_38 MenhirState0870
                | Tokens.IF ->
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0000, _v, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_14) in
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0870
                | Tokens.IDENTIFIER _v_39 ->
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0000, _v, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_14) in
                    _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v_39 MenhirState0870
                | Tokens.BOOL_LIT _v_40 ->
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0000, _v, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_14) in
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v_40 MenhirState0870
                | Tokens.BNOT ->
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0000, _v, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_14) in
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0870
                | Tokens.BITVECTOR_LIT _v_41 ->
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0000, _v, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_14) in
                    _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v_41 MenhirState0870
                | Tokens.ARBITRARY ->
                    let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0000, _v, _startpos, _endpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_14) in
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0870
                | _ ->
                    _eRR ())
            | Tokens.LBRACE ->
                let _menhir_stack = MenhirCell1_IDENTIFIER (_menhir_stack, MenhirState0000, _v, _startpos, _endpos) in
                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                let _menhir_stack = MenhirCell0_LBRACE (_menhir_stack, _startpos) in
                let _menhir_s = MenhirState0916 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.ARBITRARY ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | Tokens.DOT | Tokens.EQ | Tokens.LBRACKET | Tokens.LLBRACKET ->
                let (_menhir_s, _endpos__1_, _startpos__1_, _1) = (MenhirState0000, _endpos, _startpos, _v) in
                let _v = _menhir_action_127 _1 in
                _menhir_goto_lexpr_atom_desc _menhir_stack _menhir_lexbuf _menhir_lexer _endpos__1_ _startpos__1_ _v _menhir_s _tok
            | _ ->
                _eRR ())
        | Tokens.FOR ->
            let _menhir_s = MenhirState0000 in
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell1_FOR (_menhir_stack, _menhir_s, _startpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.IDENTIFIER _v ->
                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                let _endpos = _menhir_lexbuf.Lexing.lex_curr_p in
                let _menhir_stack = MenhirCell0_IDENTIFIER (_menhir_stack, _v, _startpos, _endpos) in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.EQ ->
                    let _menhir_s = MenhirState0943 in
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.STRING_LIT _v ->
                        _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.REAL_LIT _v ->
                        _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.NOT ->
                        _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.MINUS ->
                        _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.LPAR ->
                        _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.INT_LIT _v ->
                        _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.IF ->
                        _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.IDENTIFIER _v ->
                        _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.BOOL_LIT _v ->
                        _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.BNOT ->
                        _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | Tokens.BITVECTOR_LIT _v ->
                        _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                    | Tokens.ARBITRARY ->
                        _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                    | _ ->
                        _eRR ())
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.DEBUG ->
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LPAR ->
                let _startpos_56 = _menhir_lexbuf.Lexing.lex_start_p in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING_LIT _v ->
                    let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, MenhirState0000, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_56) in
                    _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0970
                | Tokens.RPAR ->
                    let _tok = _menhir_lexer _menhir_lexbuf in
                    (match (_tok : MenhirBasics.token) with
                    | Tokens.SEMI_COLON ->
                        let _endpos_57 = _menhir_lexbuf.Lexing.lex_curr_p in
                        let _tok = _menhir_lexer _menhir_lexbuf in
                        (match (_tok : MenhirBasics.token) with
                        | Tokens.EOF ->
                            let (_startpos__1_, _1, _2, _endpos__2_inlined1_) = (_startpos, (), (), _endpos_57) in
                            let _v = _menhir_action_389 _1 _2 _endpos__2_inlined1_ _startpos__1_ in
                            _menhir_goto_opn _menhir_stack _v
                        | _ ->
                            _eRR ())
                    | _ ->
                        _eRR ())
                | Tokens.REAL_LIT _v ->
                    let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, MenhirState0000, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_56) in
                    _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0970
                | Tokens.NOT ->
                    let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, MenhirState0000, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_56) in
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0970
                | Tokens.MINUS ->
                    let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, MenhirState0000, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_56) in
                    _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0970
                | Tokens.LPAR ->
                    let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, MenhirState0000, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_56) in
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0970
                | Tokens.INT_LIT _v ->
                    let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, MenhirState0000, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_56) in
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0970
                | Tokens.IF ->
                    let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, MenhirState0000, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_56) in
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0970
                | Tokens.IDENTIFIER _v ->
                    let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, MenhirState0000, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_56) in
                    _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0970
                | Tokens.BOOL_LIT _v ->
                    let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, MenhirState0000, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_56) in
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0970
                | Tokens.BNOT ->
                    let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, MenhirState0000, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_56) in
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0970
                | Tokens.BITVECTOR_LIT _v ->
                    let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, MenhirState0000, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_56) in
                    _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState0970
                | Tokens.ARBITRARY ->
                    let _menhir_stack = MenhirCell1_DEBUG (_menhir_stack, MenhirState0000, _startpos) in
                    let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos_56) in
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState0970
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | Tokens.CONSTANT ->
            let _menhir_s = MenhirState0000 in
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell1_CONSTANT (_menhir_stack, _menhir_s, _startpos) in
            let _menhir_s = MenhirState0978 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.MINUS ->
                _menhir_run_0365 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0370 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0371 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | _ ->
                _eRR ())
        | Tokens.CASE ->
            let _menhir_s = MenhirState0000 in
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell1_CASE (_menhir_stack, _menhir_s, _startpos) in
            let _menhir_s = MenhirState1001 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.ASSERT ->
            let _menhir_s = MenhirState0000 in
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell1_ASSERT (_menhir_stack, _menhir_s, _startpos) in
            let _menhir_s = MenhirState1007 in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.STRING_LIT _v ->
                _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.REAL_LIT _v ->
                _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.NOT ->
                _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.MINUS ->
                _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.LPAR ->
                _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.INT_LIT _v ->
                _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.IF ->
                _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.IDENTIFIER _v ->
                _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BOOL_LIT _v ->
                _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.BNOT ->
                _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | Tokens.BITVECTOR_LIT _v ->
                _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
            | Tokens.ARBITRARY ->
                _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
            | _ ->
                _eRR ())
        | Tokens.ARROBASE_LOOPLIMIT ->
            let _menhir_s = MenhirState0000 in
            let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
            let _menhir_stack = MenhirCell1_ARROBASE_LOOPLIMIT (_menhir_stack, _menhir_s, _startpos) in
            let _tok = _menhir_lexer _menhir_lexbuf in
            (match (_tok : MenhirBasics.token) with
            | Tokens.LPAR ->
                let _startpos = _menhir_lexbuf.Lexing.lex_start_p in
                let _menhir_stack = MenhirCell0_LPAR (_menhir_stack, _startpos) in
                let _menhir_s = MenhirState1012 in
                let _tok = _menhir_lexer _menhir_lexbuf in
                (match (_tok : MenhirBasics.token) with
                | Tokens.STRING_LIT _v ->
                    _menhir_run_0002 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.REAL_LIT _v ->
                    _menhir_run_0003 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.NOT ->
                    _menhir_run_0004 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.MINUS ->
                    _menhir_run_0005 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.LPAR ->
                    _menhir_run_0006 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.INT_LIT _v ->
                    _menhir_run_0007 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.IF ->
                    _menhir_run_0008 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.IDENTIFIER _v ->
                    _menhir_run_0009 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BOOL_LIT _v ->
                    _menhir_run_0012 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.BNOT ->
                    _menhir_run_0013 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | Tokens.BITVECTOR_LIT _v ->
                    _menhir_run_0014 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                | Tokens.ARBITRARY ->
                    _menhir_run_0015 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                | _ ->
                    _eRR ())
            | _ ->
                _eRR ())
        | _ ->
            _eRR ()
    
    let _menhir_run_1033 : type  ttv_stack. ttv_stack -> _ -> _ -> _menhir_box_spec =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.VAR ->
            _menhir_run_1034 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1033
        | Tokens.TYPE ->
            _menhir_run_1039 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1033
        | Tokens.SETTER ->
            _menhir_run_1059 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1033
        | Tokens.PRAGMA ->
            _menhir_run_1085 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1033
        | Tokens.LET ->
            _menhir_run_1088 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1033
        | Tokens.GETTER ->
            _menhir_run_1093 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1033
        | Tokens.FUNC ->
            _menhir_run_1103 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1033
        | Tokens.CONSTANT ->
            _menhir_run_1111 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1033
        | Tokens.CONFIG ->
            _menhir_run_1116 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState1033
        | Tokens.EOF ->
            let _v = _menhir_action_133 () in
            _menhir_run_1375 _menhir_stack _v
        | _ ->
            _eRR ()
    
    let _menhir_run_1377 : type  ttv_stack. ttv_stack -> _ -> _ -> _menhir_box_stmts =
      fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
        let _menhir_s = MenhirState1377 in
        let _tok = _menhir_lexer _menhir_lexbuf in
        match (_tok : MenhirBasics.token) with
        | Tokens.WHILE ->
            _menhir_run_0359 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.VAR ->
            _menhir_run_0364 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.UNREACHABLE ->
            _menhir_run_0393 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.TRY ->
            _menhir_run_0397 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.THROW ->
            _menhir_run_0398 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.RETURN ->
            _menhir_run_0400 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.REPEAT ->
            _menhir_run_0402 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.PRINT ->
            _menhir_run_0403 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.PRAGMA ->
            _menhir_run_0407 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.PASS ->
            _menhir_run_0410 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.MINUS ->
            _menhir_run_0412 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LPAR ->
            _menhir_run_0413 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.LET ->
            _menhir_run_0435 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IF ->
            _menhir_run_0444 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.IDENTIFIER _v ->
            _menhir_run_0447 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
        | Tokens.FOR ->
            _menhir_run_0451 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.DEBUG ->
            _menhir_run_0460 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.CONSTANT ->
            _menhir_run_0464 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.CASE ->
            _menhir_run_0473 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.ASSERT ->
            _menhir_run_0481 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | Tokens.ARROBASE_LOOPLIMIT ->
            _menhir_run_0484 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
        | _ ->
            _eRR ()
    
  end
  
  let stmts =
    fun _menhir_lexer _menhir_lexbuf ->
      let _menhir_stack = () in
      let MenhirBox_stmts v = _menhir_run_1377 _menhir_stack _menhir_lexbuf _menhir_lexer in
      v
  
  let spec =
    fun _menhir_lexer _menhir_lexbuf ->
      let _menhir_stack = () in
      let MenhirBox_spec v = _menhir_run_1033 _menhir_stack _menhir_lexbuf _menhir_lexer in
      v
  
  let opn =
    fun _menhir_lexer _menhir_lexbuf ->
      let _menhir_stack = () in
      let MenhirBox_opn v = _menhir_run_0000 _menhir_stack _menhir_lexbuf _menhir_lexer in
      v
  
end
