
module MenhirBasics = struct
  
  exception Error = Parsing.Parse_error
  
  let _eRR =
    fun _s ->
      raise Error
  
  type token = 
    | WITH
    | VALUE of (
# 73 "lib/stateParser.mly"
       (string)
# 16 "lib/stateParser.ml"
  )
    | TRUE
    | TOK_TAG
    | TOK_PTE
    | TOK_PA
    | TOK_OA
    | TOK_NOP
    | TOKAND
    | SYMB_REG of (
# 69 "lib/stateParser.mly"
       (string)
# 28 "lib/stateParser.ml"
  )
    | STAR
    | SEMI
    | RPAR
    | RCURLY
    | RBRK
    | PTX_REG_TYPE of (
# 89 "lib/stateParser.mly"
       (string)
# 38 "lib/stateParser.ml"
  )
    | PTX_REG_DEC
    | PROC of (
# 68 "lib/stateParser.mly"
       (int)
# 44 "lib/stateParser.ml"
  )
    | PLUS
    | OR
    | OBSERVED
    | NUM of (
# 72 "lib/stateParser.mly"
       (string)
# 52 "lib/stateParser.ml"
  )
    | NOTEQUAL
    | NOT
    | NAME of (
# 70 "lib/stateParser.mly"
       (string)
# 59 "lib/stateParser.ml"
  )
    | LPAR
    | LOCATIONS
    | LCURLY
    | LBRK
    | LABEL of (
# 87 "lib/stateParser.mly"
       (int * string)
# 68 "lib/stateParser.ml"
  )
    | INSTR of (
# 86 "lib/stateParser.mly"
       (string)
# 73 "lib/stateParser.ml"
  )
    | IMPLIES
    | FORALL
    | FINAL
    | FILTER
    | FAULT
    | FALSE
    | EXISTS
    | EQUALEQUAL
    | EQUAL
    | EOF
    | DOLLARNAME of (
# 71 "lib/stateParser.mly"
       (string)
# 88 "lib/stateParser.ml"
  )
    | COMMA
    | COLON
    | ATTRS
    | ATOMICINIT
    | ATOMIC
    | AND
    | AMPER
  
end

include MenhirBasics

# 1 "lib/stateParser.mly"
  
(****************************************************************************)
(*                           the diy toolsuite                              *)
(*                                                                          *)
(* Jade Alglave, University College London, UK.                             *)
(* Luc Maranget, INRIA Paris-Rocquencourt, France.                          *)
(*                                                                          *)
(* Copyright 2010-present Institut National de Recherche en Informatique et *)
(* en Automatique, ARM Ltd and the authors. All rights reserved.            *)
(*                                                                          *)
(* This software is governed by the CeCILL-B license under French law and   *)
(* abiding by the rules of distribution of free software. You can use,      *)
(* modify and/ or redistribute the software under the terms of the CeCILL-B *)
(* license as circulated by CEA, CNRS and INRIA at the following URL        *)
(* "http://www.cecill.info". We also give a copy in LICENSE.txt.            *)
(****************************************************************************)

open Constant
open TestType
open LocationsItem
open MiscParser
open ConstrGen

let mk_sym_tag s t =
  Symbolic (Virtual {default_symbolic_data with name=s;tag=Some t;})

let do_mk_sym_tagloc s o =
  if
    o < 0 ||
    o mod MachSize.granule_nbytes <> 0
  then raise Parsing.Parse_error ;
  Symbolic (TagAddr (VIR,s,o))

let mk_sym_tagloc s o =
  let o =
    try int_of_string o
    with Invalid_argument _ -> raise Parsing.Parse_error in
  do_mk_sym_tagloc s o

let mk_sym_tagloc_zero s = do_mk_sym_tagloc s 0

let mk_sym_morello p s t =
  let p_int = Misc.string_as_int64 p in
  if
    not (Int64.equal (Int64.logand p_int 0x7L) 0L)
    || Int64.compare p_int (Int64.shift_left 1L 36) >= 0
    || Int64.compare p_int 0L < 0
    then Printf.eprintf "Warning: incorrect address encoding: %#Lx\n" p_int ;
  let truncated_perms = Int64.shift_right_logical p_int 3 in
  let tag = if Misc.string_as_int t <> 0 then 1L else 0L in
  Symbolic
    (Virtual
       {default_symbolic_data
       with
       name=s;
       cap=Int64.logor truncated_perms (Int64.shift_left tag 33); })

let mk_sym_with_index s i =
  Symbolic
    (Virtual
       {default_symbolic_data
       with name=s; offset=Misc.string_as_int i})

let mk_lab (p, l) = Label (p, l)

# 168 "lib/stateParser.ml"

type ('s, 'r) _menhir_state = 
  | MenhirState000 : ('s, _menhir_box_constraints) _menhir_state
    (** State 000.
        Stack shape : .
        Start symbol: constraints. *)

  | MenhirState002 : (('s, 'r) _menhir_cell1_LOCATIONS, 'r) _menhir_state
    (** State 002.
        Stack shape : LOCATIONS.
        Start symbol: <undetermined>. *)

  | MenhirState026 : (('s, 'r) _menhir_cell1_PROC, 'r) _menhir_state
    (** State 026.
        Stack shape : PROC.
        Start symbol: <undetermined>. *)

  | MenhirState029 : ((('s, 'r) _menhir_cell1_PROC, 'r) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE, 'r) _menhir_state
    (** State 029.
        Stack shape : PROC PTX_REG_DEC PTX_REG_TYPE.
        Start symbol: <undetermined>. *)

  | MenhirState035 : (('s, 'r) _menhir_cell1_NUM, 'r) _menhir_state
    (** State 035.
        Stack shape : NUM.
        Start symbol: <undetermined>. *)

  | MenhirState038 : ((('s, 'r) _menhir_cell1_NUM, 'r) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE, 'r) _menhir_state
    (** State 038.
        Stack shape : NUM PTX_REG_DEC PTX_REG_TYPE.
        Start symbol: <undetermined>. *)

  | MenhirState044 : (('s, 'r) _menhir_cell1_NAME, 'r) _menhir_state
    (** State 044.
        Stack shape : NAME.
        Start symbol: <undetermined>. *)

  | MenhirState045 : ((('s, 'r) _menhir_cell1_NAME, 'r) _menhir_cell1_LBRK, 'r) _menhir_state
    (** State 045.
        Stack shape : NAME LBRK.
        Start symbol: <undetermined>. *)

  | MenhirState065 : (('s, 'r) _menhir_cell1_FAULT _menhir_cell0_lbl _menhir_cell0_NAME, 'r) _menhir_state
    (** State 065.
        Stack shape : FAULT lbl NAME.
        Start symbol: <undetermined>. *)

  | MenhirState067 : (('s, 'r) _menhir_cell1_NAME, 'r) _menhir_state
    (** State 067.
        Stack shape : NAME.
        Start symbol: <undetermined>. *)

  | MenhirState073 : (('s, 'r) _menhir_cell1_FAULT _menhir_cell0_lbl _menhir_cell0_fault_loc, 'r) _menhir_state
    (** State 073.
        Stack shape : FAULT lbl fault_loc.
        Start symbol: <undetermined>. *)

  | MenhirState089 : (('s, 'r) _menhir_cell1_loc_item, 'r) _menhir_state
    (** State 089.
        Stack shape : loc_item.
        Start symbol: <undetermined>. *)

  | MenhirState093 : (('s, _menhir_box_constraints) _menhir_cell1_locations, _menhir_box_constraints) _menhir_state
    (** State 093.
        Stack shape : locations.
        Start symbol: constraints. *)

  | MenhirState094 : (('s, 'r) _menhir_cell1_FILTER, 'r) _menhir_state
    (** State 094.
        Stack shape : FILTER.
        Start symbol: <undetermined>. *)

  | MenhirState096 : (('s, 'r) _menhir_cell1_NOT, 'r) _menhir_state
    (** State 096.
        Stack shape : NOT.
        Start symbol: <undetermined>. *)

  | MenhirState097 : (('s, 'r) _menhir_cell1_LPAR, 'r) _menhir_state
    (** State 097.
        Stack shape : LPAR.
        Start symbol: <undetermined>. *)

  | MenhirState098 : (('s, 'r) _menhir_cell1_LBRK, 'r) _menhir_state
    (** State 098.
        Stack shape : LBRK.
        Start symbol: <undetermined>. *)

  | MenhirState100 : ((('s, 'r) _menhir_cell1_LBRK, 'r) _menhir_cell1_NUM, 'r) _menhir_state
    (** State 100.
        Stack shape : LBRK NUM.
        Start symbol: <undetermined>. *)

  | MenhirState102 : (('s, 'r) _menhir_cell1_NAME, 'r) _menhir_state
    (** State 102.
        Stack shape : NAME.
        Start symbol: <undetermined>. *)

  | MenhirState105 : ((('s, 'r) _menhir_cell1_LBRK, 'r) _menhir_cell1_location_global, 'r) _menhir_state
    (** State 105.
        Stack shape : LBRK location_global.
        Start symbol: <undetermined>. *)

  | MenhirState108 : (('s, 'r) _menhir_cell1_NUM, 'r) _menhir_state
    (** State 108.
        Stack shape : NUM.
        Start symbol: <undetermined>. *)

  | MenhirState110 : (('s, 'r) _menhir_cell1_NAME, 'r) _menhir_state
    (** State 110.
        Stack shape : NAME.
        Start symbol: <undetermined>. *)

  | MenhirState119 : ((('s, 'r) _menhir_cell1_LBRK, 'r) _menhir_cell1_location_global, 'r) _menhir_state
    (** State 119.
        Stack shape : LBRK location_global.
        Start symbol: <undetermined>. *)

  | MenhirState122 : (((('s, 'r) _menhir_cell1_LBRK, 'r) _menhir_cell1_location_global, 'r) _menhir_cell1_PROC, 'r) _menhir_state
    (** State 122.
        Stack shape : LBRK location_global PROC.
        Start symbol: <undetermined>. *)

  | MenhirState125 : ((((('s, 'r) _menhir_cell1_LBRK, 'r) _menhir_cell1_location_global, 'r) _menhir_cell1_PROC, 'r) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE, 'r) _menhir_state
    (** State 125.
        Stack shape : LBRK location_global PROC PTX_REG_DEC PTX_REG_TYPE.
        Start symbol: <undetermined>. *)

  | MenhirState129 : (((('s, 'r) _menhir_cell1_LBRK, 'r) _menhir_cell1_location_global, 'r) _menhir_cell1_NUM, 'r) _menhir_state
    (** State 129.
        Stack shape : LBRK location_global NUM.
        Start symbol: <undetermined>. *)

  | MenhirState132 : ((((('s, 'r) _menhir_cell1_LBRK, 'r) _menhir_cell1_location_global, 'r) _menhir_cell1_NUM, 'r) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE, 'r) _menhir_state
    (** State 132.
        Stack shape : LBRK location_global NUM PTX_REG_DEC PTX_REG_TYPE.
        Start symbol: <undetermined>. *)

  | MenhirState135 : (('s, 'r) _menhir_cell1_LPAR, 'r) _menhir_state
    (** State 135.
        Stack shape : LPAR.
        Start symbol: <undetermined>. *)

  | MenhirState147 : (('s, 'r) _menhir_cell1_TOK_OA _menhir_cell0_output_address, 'r) _menhir_state
    (** State 147.
        Stack shape : TOK_OA output_address.
        Start symbol: <undetermined>. *)

  | MenhirState148 : (('s, 'r) _menhir_cell1_COMMA, 'r) _menhir_state
    (** State 148.
        Stack shape : COMMA.
        Start symbol: <undetermined>. *)

  | MenhirState149 : (('s, 'r) _menhir_cell1_NAME, 'r) _menhir_state
    (** State 149.
        Stack shape : NAME.
        Start symbol: <undetermined>. *)

  | MenhirState153 : ((('s, 'r) _menhir_cell1_NAME, 'r) _menhir_cell1_COLON _menhir_cell0_name_or_num, 'r) _menhir_state
    (** State 153.
        Stack shape : NAME COLON name_or_num.
        Start symbol: <undetermined>. *)

  | MenhirState158 : (('s, 'r) _menhir_cell1_ATTRS, 'r) _menhir_state
    (** State 158.
        Stack shape : ATTRS.
        Start symbol: <undetermined>. *)

  | MenhirState160 : (('s, 'r) _menhir_cell1_NAME, 'r) _menhir_state
    (** State 160.
        Stack shape : NAME.
        Start symbol: <undetermined>. *)

  | MenhirState163 : ((('s, 'r) _menhir_cell1_ATTRS, 'r) _menhir_cell1_separated_nonempty_list_COMMA_NAME_, 'r) _menhir_state
    (** State 163.
        Stack shape : ATTRS separated_nonempty_list(COMMA,NAME).
        Start symbol: <undetermined>. *)

  | MenhirState169 : (((('s, 'r) _menhir_cell1_LBRK, 'r) _menhir_cell1_location_global, 'r) _menhir_cell1_LBRK, 'r) _menhir_state
    (** State 169.
        Stack shape : LBRK location_global LBRK.
        Start symbol: <undetermined>. *)

  | MenhirState174 : ((('s, 'r) _menhir_cell1_LBRK, 'r) _menhir_cell1_location_global, 'r) _menhir_state
    (** State 174.
        Stack shape : LBRK location_global.
        Start symbol: <undetermined>. *)

  | MenhirState177 : (((('s, 'r) _menhir_cell1_LBRK, 'r) _menhir_cell1_location_global, 'r) _menhir_cell1_PROC, 'r) _menhir_state
    (** State 177.
        Stack shape : LBRK location_global PROC.
        Start symbol: <undetermined>. *)

  | MenhirState180 : ((((('s, 'r) _menhir_cell1_LBRK, 'r) _menhir_cell1_location_global, 'r) _menhir_cell1_PROC, 'r) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE, 'r) _menhir_state
    (** State 180.
        Stack shape : LBRK location_global PROC PTX_REG_DEC PTX_REG_TYPE.
        Start symbol: <undetermined>. *)

  | MenhirState184 : (((('s, 'r) _menhir_cell1_LBRK, 'r) _menhir_cell1_location_global, 'r) _menhir_cell1_NUM, 'r) _menhir_state
    (** State 184.
        Stack shape : LBRK location_global NUM.
        Start symbol: <undetermined>. *)

  | MenhirState187 : ((((('s, 'r) _menhir_cell1_LBRK, 'r) _menhir_cell1_location_global, 'r) _menhir_cell1_NUM, 'r) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE, 'r) _menhir_state
    (** State 187.
        Stack shape : LBRK location_global NUM PTX_REG_DEC PTX_REG_TYPE.
        Start symbol: <undetermined>. *)

  | MenhirState190 : (((('s, 'r) _menhir_cell1_LBRK, 'r) _menhir_cell1_location_global, 'r) _menhir_cell1_LBRK, 'r) _menhir_state
    (** State 190.
        Stack shape : LBRK location_global LBRK.
        Start symbol: <undetermined>. *)

  | MenhirState198 : (('s, 'r) _menhir_cell1_prop, 'r) _menhir_state
    (** State 198.
        Stack shape : prop.
        Start symbol: <undetermined>. *)

  | MenhirState200 : (('s, 'r) _menhir_cell1_prop, 'r) _menhir_state
    (** State 200.
        Stack shape : prop.
        Start symbol: <undetermined>. *)

  | MenhirState203 : (('s, 'r) _menhir_cell1_locindex, 'r) _menhir_state
    (** State 203.
        Stack shape : locindex.
        Start symbol: <undetermined>. *)

  | MenhirState205 : (('s, 'r) _menhir_cell1_locindex, 'r) _menhir_state
    (** State 205.
        Stack shape : locindex.
        Start symbol: <undetermined>. *)

  | MenhirState207 : (('s, 'r) _menhir_cell1_locindex, 'r) _menhir_state
    (** State 207.
        Stack shape : locindex.
        Start symbol: <undetermined>. *)

  | MenhirState210 : (('s, 'r) _menhir_cell1_location, 'r) _menhir_state
    (** State 210.
        Stack shape : location.
        Start symbol: <undetermined>. *)

  | MenhirState211 : ((('s, 'r) _menhir_cell1_location, 'r) _menhir_cell1_LCURLY, 'r) _menhir_state
    (** State 211.
        Stack shape : location LCURLY.
        Start symbol: <undetermined>. *)

  | MenhirState213 : (('s, 'r) _menhir_cell1_maybev_notag, 'r) _menhir_state
    (** State 213.
        Stack shape : maybev_notag.
        Start symbol: <undetermined>. *)

  | MenhirState218 : (('s, 'r) _menhir_cell1_location, 'r) _menhir_state
    (** State 218.
        Stack shape : location.
        Start symbol: <undetermined>. *)

  | MenhirState223 : (('s, 'r) _menhir_cell1_STAR _menhir_cell0_PROC, 'r) _menhir_state
    (** State 223.
        Stack shape : STAR PROC.
        Start symbol: <undetermined>. *)

  | MenhirState226 : ((('s, 'r) _menhir_cell1_STAR _menhir_cell0_PROC, 'r) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE, 'r) _menhir_state
    (** State 226.
        Stack shape : STAR PROC PTX_REG_DEC PTX_REG_TYPE.
        Start symbol: <undetermined>. *)

  | MenhirState230 : (('s, 'r) _menhir_cell1_STAR _menhir_cell0_NUM, 'r) _menhir_state
    (** State 230.
        Stack shape : STAR NUM.
        Start symbol: <undetermined>. *)

  | MenhirState233 : ((('s, 'r) _menhir_cell1_STAR _menhir_cell0_NUM, 'r) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE, 'r) _menhir_state
    (** State 233.
        Stack shape : STAR NUM PTX_REG_DEC PTX_REG_TYPE.
        Start symbol: <undetermined>. *)

  | MenhirState238 : (('s, 'r) _menhir_cell1_PROC, 'r) _menhir_state
    (** State 238.
        Stack shape : PROC.
        Start symbol: <undetermined>. *)

  | MenhirState241 : ((('s, 'r) _menhir_cell1_PROC, 'r) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE, 'r) _menhir_state
    (** State 241.
        Stack shape : PROC PTX_REG_DEC PTX_REG_TYPE.
        Start symbol: <undetermined>. *)

  | MenhirState245 : (('s, 'r) _menhir_cell1_NUM, 'r) _menhir_state
    (** State 245.
        Stack shape : NUM.
        Start symbol: <undetermined>. *)

  | MenhirState248 : ((('s, 'r) _menhir_cell1_NUM, 'r) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE, 'r) _menhir_state
    (** State 248.
        Stack shape : NUM PTX_REG_DEC PTX_REG_TYPE.
        Start symbol: <undetermined>. *)

  | MenhirState251 : ((('s, 'r) _menhir_cell1_location, 'r) _menhir_cell1_LCURLY, 'r) _menhir_state
    (** State 251.
        Stack shape : location LCURLY.
        Start symbol: <undetermined>. *)

  | MenhirState254 : (('s, 'r) _menhir_cell1_LBRK, 'r) _menhir_state
    (** State 254.
        Stack shape : LBRK.
        Start symbol: <undetermined>. *)

  | MenhirState260 : (('s, 'r) _menhir_cell1_location, 'r) _menhir_state
    (** State 260.
        Stack shape : location.
        Start symbol: <undetermined>. *)

  | MenhirState262 : ((('s, 'r) _menhir_cell1_location, 'r) _menhir_cell1_LCURLY, 'r) _menhir_state
    (** State 262.
        Stack shape : location LCURLY.
        Start symbol: <undetermined>. *)

  | MenhirState273 : (('s, 'r) _menhir_cell1_prop, 'r) _menhir_state
    (** State 273.
        Stack shape : prop.
        Start symbol: <undetermined>. *)

  | MenhirState277 : ((('s, _menhir_box_constraints) _menhir_cell1_locations, _menhir_box_constraints) _menhir_cell1_filter, _menhir_box_constraints) _menhir_state
    (** State 277.
        Stack shape : locations filter.
        Start symbol: constraints. *)

  | MenhirState278 : (('s, 'r) _menhir_cell1_OBSERVED, 'r) _menhir_state
    (** State 278.
        Stack shape : OBSERVED.
        Start symbol: <undetermined>. *)

  | MenhirState280 : (('s, 'r) _menhir_cell1_obsone, 'r) _menhir_state
    (** State 280.
        Stack shape : obsone.
        Start symbol: <undetermined>. *)

  | MenhirState283 : (('s, 'r) _menhir_cell1_atom_prop, 'r) _menhir_state
    (** State 283.
        Stack shape : atom_prop.
        Start symbol: <undetermined>. *)

  | MenhirState287 : (('s, 'r) _menhir_cell1_NOT, 'r) _menhir_state
    (** State 287.
        Stack shape : NOT.
        Start symbol: <undetermined>. *)

  | MenhirState289 : (('s, 'r) _menhir_cell1_LPAR, 'r) _menhir_state
    (** State 289.
        Stack shape : LPAR.
        Start symbol: <undetermined>. *)

  | MenhirState292 : (('s, 'r) _menhir_cell1_FORALL, 'r) _menhir_state
    (** State 292.
        Stack shape : FORALL.
        Start symbol: <undetermined>. *)

  | MenhirState294 : (('s, 'r) _menhir_cell1_FINAL, 'r) _menhir_state
    (** State 294.
        Stack shape : FINAL.
        Start symbol: <undetermined>. *)

  | MenhirState296 : (('s, 'r) _menhir_cell1_EXISTS, 'r) _menhir_state
    (** State 296.
        Stack shape : EXISTS.
        Start symbol: <undetermined>. *)

  | MenhirState301 : (((('s, _menhir_box_constraints) _menhir_cell1_locations, _menhir_box_constraints) _menhir_cell1_filter, _menhir_box_constraints) _menhir_cell1_final, _menhir_box_constraints) _menhir_state
    (** State 301.
        Stack shape : locations filter final.
        Start symbol: constraints. *)

  | MenhirState310 : (('s, _menhir_box_constraints) _menhir_cell1_kind, _menhir_box_constraints) _menhir_state
    (** State 310.
        Stack shape : kind.
        Start symbol: constraints. *)

  | MenhirState315 : ('s, _menhir_box_init) _menhir_state
    (** State 315.
        Stack shape : .
        Start symbol: init. *)

  | MenhirState316 : (('s, _menhir_box_init) _menhir_cell1_STAR, _menhir_box_init) _menhir_state
    (** State 316.
        Stack shape : STAR.
        Start symbol: init. *)

  | MenhirState317 : (('s, _menhir_box_init) _menhir_cell1_LBRK, _menhir_box_init) _menhir_state
    (** State 317.
        Stack shape : LBRK.
        Start symbol: init. *)

  | MenhirState322 : ((('s, _menhir_box_init) _menhir_cell1_STAR, _menhir_box_init) _menhir_cell1_left_loc, _menhir_box_init) _menhir_state
    (** State 322.
        Stack shape : STAR left_loc.
        Start symbol: init. *)

  | MenhirState324 : (((('s, _menhir_box_init) _menhir_cell1_STAR, _menhir_box_init) _menhir_cell1_left_loc, _menhir_box_init) _menhir_cell1_amperopt, _menhir_box_init) _menhir_state
    (** State 324.
        Stack shape : STAR left_loc amperopt.
        Start symbol: init. *)

  | MenhirState327 : (('s, _menhir_box_init) _menhir_cell1_NAME, _menhir_box_init) _menhir_state
    (** State 327.
        Stack shape : NAME.
        Start symbol: init. *)

  | MenhirState328 : ((('s, _menhir_box_init) _menhir_cell1_NAME, _menhir_box_init) _menhir_cell1_STAR, _menhir_box_init) _menhir_state
    (** State 328.
        Stack shape : NAME STAR.
        Start symbol: init. *)

  | MenhirState330 : (((('s, _menhir_box_init) _menhir_cell1_NAME, _menhir_box_init) _menhir_cell1_STAR, _menhir_box_init) _menhir_cell1_left_loc, _menhir_box_init) _menhir_state
    (** State 330.
        Stack shape : NAME STAR left_loc.
        Start symbol: init. *)

  | MenhirState331 : ((((('s, _menhir_box_init) _menhir_cell1_NAME, _menhir_box_init) _menhir_cell1_STAR, _menhir_box_init) _menhir_cell1_left_loc, _menhir_box_init) _menhir_cell1_amperopt, _menhir_box_init) _menhir_state
    (** State 331.
        Stack shape : NAME STAR left_loc amperopt.
        Start symbol: init. *)

  | MenhirState333 : ((('s, _menhir_box_init) _menhir_cell1_NAME, _menhir_box_init) _menhir_cell1_LBRK, _menhir_box_init) _menhir_state
    (** State 333.
        Stack shape : NAME LBRK.
        Start symbol: init. *)

  | MenhirState336 : ((('s, _menhir_box_init) _menhir_cell1_NAME, _menhir_box_init) _menhir_cell1_left_loc, _menhir_box_init) _menhir_state
    (** State 336.
        Stack shape : NAME left_loc.
        Start symbol: init. *)

  | MenhirState338 : (((('s, _menhir_box_init) _menhir_cell1_NAME, _menhir_box_init) _menhir_cell1_left_loc, _menhir_box_init) _menhir_cell1_ATOMICINIT, _menhir_box_init) _menhir_state
    (** State 338.
        Stack shape : NAME left_loc ATOMICINIT.
        Start symbol: init. *)

  | MenhirState346 : ((('s, _menhir_box_init) _menhir_cell1_NAME, _menhir_box_init) _menhir_cell1_arrayspec, _menhir_box_init) _menhir_state
    (** State 346.
        Stack shape : NAME arrayspec.
        Start symbol: init. *)

  | MenhirState350 : (('s, _menhir_box_init) _menhir_cell1_ATOMIC _menhir_cell0_NAME, _menhir_box_init) _menhir_state
    (** State 350.
        Stack shape : ATOMIC NAME.
        Start symbol: init. *)

  | MenhirState352 : ((('s, _menhir_box_init) _menhir_cell1_ATOMIC _menhir_cell0_NAME, _menhir_box_init) _menhir_cell1_left_loc, _menhir_box_init) _menhir_state
    (** State 352.
        Stack shape : ATOMIC NAME left_loc.
        Start symbol: init. *)

  | MenhirState355 : (('s, _menhir_box_init) _menhir_cell1_locindex, _menhir_box_init) _menhir_state
    (** State 355.
        Stack shape : locindex.
        Start symbol: init. *)

  | MenhirState359 : (('s, _menhir_box_init) _menhir_cell1_left_loc, _menhir_box_init) _menhir_state
    (** State 359.
        Stack shape : left_loc.
        Start symbol: init. *)

  | MenhirState364 : ((('s, _menhir_box_init) _menhir_cell1_left_loc, _menhir_box_init) _menhir_cell1_NUM, _menhir_box_init) _menhir_state
    (** State 364.
        Stack shape : left_loc NUM.
        Start symbol: init. *)

  | MenhirState375 : (('s, _menhir_box_init) _menhir_cell1_atom_init, _menhir_box_init) _menhir_state
    (** State 375.
        Stack shape : atom_init.
        Start symbol: init. *)

  | MenhirState378 : ('s, _menhir_box_main_constr) _menhir_state
    (** State 378.
        Stack shape : .
        Start symbol: main_constr. *)

  | MenhirState382 : ('s, _menhir_box_main_filter) _menhir_state
    (** State 382.
        Stack shape : .
        Start symbol: main_filter. *)

  | MenhirState386 : ('s, _menhir_box_main_loc_constr) _menhir_state
    (** State 386.
        Stack shape : .
        Start symbol: main_loc_constr. *)

  | MenhirState388 : (('s, _menhir_box_main_loc_constr) _menhir_cell1_locations, _menhir_box_main_loc_constr) _menhir_state
    (** State 388.
        Stack shape : locations.
        Start symbol: main_loc_constr. *)

  | MenhirState392 : ('s, _menhir_box_main_location) _menhir_state
    (** State 392.
        Stack shape : .
        Start symbol: main_location. *)

  | MenhirState396 : ('s, _menhir_box_pteval) _menhir_state
    (** State 396.
        Stack shape : .
        Start symbol: pteval. *)

  | MenhirState397 : (('s, _menhir_box_pteval) _menhir_cell1_LPAR, _menhir_box_pteval) _menhir_state
    (** State 397.
        Stack shape : LPAR.
        Start symbol: pteval. *)

  | MenhirState401 : ('s, _menhir_box_skip_loc_constr) _menhir_state
    (** State 401.
        Stack shape : .
        Start symbol: skip_loc_constr. *)

  | MenhirState403 : (('s, _menhir_box_skip_loc_constr) _menhir_cell1_locations, _menhir_box_skip_loc_constr) _menhir_state
    (** State 403.
        Stack shape : locations.
        Start symbol: skip_loc_constr. *)


and ('s, 'r) _menhir_cell1_amperopt = 
  | MenhirCell1_amperopt of 's * ('s, 'r) _menhir_state * (unit)

and ('s, 'r) _menhir_cell1_arrayspec = 
  | MenhirCell1_arrayspec of 's * ('s, 'r) _menhir_state * (MiscParser.location * int)

and ('s, 'r) _menhir_cell1_atom_init = 
  | MenhirCell1_atom_init of 's * ('s, 'r) _menhir_state * (MiscParser.state_atom)

and ('s, 'r) _menhir_cell1_atom_prop = 
  | MenhirCell1_atom_prop of 's * ('s, 'r) _menhir_state * (MiscParser.prop)

and 's _menhir_cell0_fault_loc = 
  | MenhirCell0_fault_loc of 's * (ParsedConstant.v)

and ('s, 'r) _menhir_cell1_filter = 
  | MenhirCell1_filter of 's * ('s, 'r) _menhir_state * (MiscParser.prop option)

and ('s, 'r) _menhir_cell1_final = 
  | MenhirCell1_final of 's * ('s, 'r) _menhir_state * (MiscParser.constr)

and ('s, 'r) _menhir_cell1_kind = 
  | MenhirCell1_kind of 's * ('s, 'r) _menhir_state * (string * MiscParser.quantifier)

and 's _menhir_cell0_lbl = 
  | MenhirCell0_lbl of 's * (int * string option)

and ('s, 'r) _menhir_cell1_left_loc = 
  | MenhirCell1_left_loc of 's * ('s, 'r) _menhir_state * (MiscParser.location)

and ('s, 'r) _menhir_cell1_loc_item = 
  | MenhirCell1_loc_item of 's * ('s, 'r) _menhir_state * ((MiscParser.location, ParsedConstant.v, string) LocationsItem.t)

and ('s, 'r) _menhir_cell1_location = 
  | MenhirCell1_location of 's * ('s, 'r) _menhir_state * (MiscParser.location)

and ('s, 'r) _menhir_cell1_location_global = 
  | MenhirCell1_location_global of 's * ('s, 'r) _menhir_state * (ParsedConstant.v)

and ('s, 'r) _menhir_cell1_locations = 
  | MenhirCell1_locations of 's * ('s, 'r) _menhir_state * ((MiscParser.location, ParsedConstant.v, string) LocationsItem.t list)

and ('s, 'r) _menhir_cell1_locindex = 
  | MenhirCell1_locindex of 's * ('s, 'r) _menhir_state * (MiscParser.location ConstrGen.rloc)

and ('s, 'r) _menhir_cell1_maybev_notag = 
  | MenhirCell1_maybev_notag of 's * ('s, 'r) _menhir_state * (ParsedConstant.v)

and 's _menhir_cell0_name_or_num = 
  | MenhirCell0_name_or_num of 's * (string)

and ('s, 'r) _menhir_cell1_obsone = 
  | MenhirCell1_obsone of 's * ('s, 'r) _menhir_state * (MiscParser.prop list)

and 's _menhir_cell0_output_address = 
  | MenhirCell0_output_address of 's * (OutputAddress.t)

and ('s, 'r) _menhir_cell1_prop = 
  | MenhirCell1_prop of 's * ('s, 'r) _menhir_state * (MiscParser.prop)

and ('s, 'r) _menhir_cell1_separated_nonempty_list_COMMA_NAME_ = 
  | MenhirCell1_separated_nonempty_list_COMMA_NAME_ of 's * ('s, 'r) _menhir_state * (string list)

and ('s, 'r) _menhir_cell1_ATOMIC = 
  | MenhirCell1_ATOMIC of 's * ('s, 'r) _menhir_state

and ('s, 'r) _menhir_cell1_ATOMICINIT = 
  | MenhirCell1_ATOMICINIT of 's * ('s, 'r) _menhir_state

and ('s, 'r) _menhir_cell1_ATTRS = 
  | MenhirCell1_ATTRS of 's * ('s, 'r) _menhir_state

and ('s, 'r) _menhir_cell1_COLON = 
  | MenhirCell1_COLON of 's * ('s, 'r) _menhir_state

and ('s, 'r) _menhir_cell1_COMMA = 
  | MenhirCell1_COMMA of 's * ('s, 'r) _menhir_state

and ('s, 'r) _menhir_cell1_EXISTS = 
  | MenhirCell1_EXISTS of 's * ('s, 'r) _menhir_state

and ('s, 'r) _menhir_cell1_FAULT = 
  | MenhirCell1_FAULT of 's * ('s, 'r) _menhir_state

and ('s, 'r) _menhir_cell1_FILTER = 
  | MenhirCell1_FILTER of 's * ('s, 'r) _menhir_state

and ('s, 'r) _menhir_cell1_FINAL = 
  | MenhirCell1_FINAL of 's * ('s, 'r) _menhir_state

and ('s, 'r) _menhir_cell1_FORALL = 
  | MenhirCell1_FORALL of 's * ('s, 'r) _menhir_state

and ('s, 'r) _menhir_cell1_LBRK = 
  | MenhirCell1_LBRK of 's * ('s, 'r) _menhir_state

and ('s, 'r) _menhir_cell1_LCURLY = 
  | MenhirCell1_LCURLY of 's * ('s, 'r) _menhir_state

and ('s, 'r) _menhir_cell1_LOCATIONS = 
  | MenhirCell1_LOCATIONS of 's * ('s, 'r) _menhir_state

and ('s, 'r) _menhir_cell1_LPAR = 
  | MenhirCell1_LPAR of 's * ('s, 'r) _menhir_state

and ('s, 'r) _menhir_cell1_NAME = 
  | MenhirCell1_NAME of 's * ('s, 'r) _menhir_state * (
# 70 "lib/stateParser.mly"
       (string)
# 796 "lib/stateParser.ml"
)

and 's _menhir_cell0_NAME = 
  | MenhirCell0_NAME of 's * (
# 70 "lib/stateParser.mly"
       (string)
# 803 "lib/stateParser.ml"
)

and ('s, 'r) _menhir_cell1_NOT = 
  | MenhirCell1_NOT of 's * ('s, 'r) _menhir_state

and ('s, 'r) _menhir_cell1_NUM = 
  | MenhirCell1_NUM of 's * ('s, 'r) _menhir_state * (
# 72 "lib/stateParser.mly"
       (string)
# 813 "lib/stateParser.ml"
)

and 's _menhir_cell0_NUM = 
  | MenhirCell0_NUM of 's * (
# 72 "lib/stateParser.mly"
       (string)
# 820 "lib/stateParser.ml"
)

and ('s, 'r) _menhir_cell1_OBSERVED = 
  | MenhirCell1_OBSERVED of 's * ('s, 'r) _menhir_state

and ('s, 'r) _menhir_cell1_PROC = 
  | MenhirCell1_PROC of 's * ('s, 'r) _menhir_state * (
# 68 "lib/stateParser.mly"
       (int)
# 830 "lib/stateParser.ml"
)

and 's _menhir_cell0_PROC = 
  | MenhirCell0_PROC of 's * (
# 68 "lib/stateParser.mly"
       (int)
# 837 "lib/stateParser.ml"
)

and ('s, 'r) _menhir_cell1_PTX_REG_DEC = 
  | MenhirCell1_PTX_REG_DEC of 's * ('s, 'r) _menhir_state

and 's _menhir_cell0_PTX_REG_TYPE = 
  | MenhirCell0_PTX_REG_TYPE of 's * (
# 89 "lib/stateParser.mly"
       (string)
# 847 "lib/stateParser.ml"
)

and ('s, 'r) _menhir_cell1_STAR = 
  | MenhirCell1_STAR of 's * ('s, 'r) _menhir_state

and ('s, 'r) _menhir_cell1_TOK_OA = 
  | MenhirCell1_TOK_OA of 's * ('s, 'r) _menhir_state

and _menhir_box_skip_loc_constr = 
  | MenhirBox_skip_loc_constr of (MiscParser.constr) [@@unboxed]

and _menhir_box_pteval = 
  | MenhirBox_pteval of (ParsedPteVal.t) [@@unboxed]

and _menhir_box_main_location = 
  | MenhirBox_main_location of (MiscParser.location) [@@unboxed]

and _menhir_box_main_loc_constr = 
  | MenhirBox_main_loc_constr of ((MiscParser.location, ParsedConstant.v, string) LocationsItem.t list *
  MiscParser.constr) [@@unboxed]

and _menhir_box_main_filter = 
  | MenhirBox_main_filter of (MiscParser.prop option) [@@unboxed]

and _menhir_box_main_constr = 
  | MenhirBox_main_constr of (MiscParser.constr) [@@unboxed]

and _menhir_box_init = 
  | MenhirBox_init of (MiscParser.state) [@@unboxed]

and _menhir_box_constraints = 
  | MenhirBox_constraints of ((MiscParser.location, ParsedConstant.v, string) LocationsItem.t list *
  MiscParser.prop option * MiscParser.constr *
  (string * MiscParser.quantifier) list) [@@unboxed]

let _menhir_action_008 =
  fun () ->
    (
# 261 "lib/stateParser.mly"
        ( () )
# 888 "lib/stateParser.ml"
     : (unit))

let _menhir_action_009 =
  fun () ->
    (
# 262 "lib/stateParser.mly"
  ( () )
# 896 "lib/stateParser.ml"
     : (unit))

let _menhir_action_010 =
  fun _1 _3 ->
    (
# 399 "lib/stateParser.mly"
    ( (Location_global (Constant.mk_sym _1),Misc.string_as_int _3) )
# 904 "lib/stateParser.ml"
     : (MiscParser.location * int))

let _menhir_action_011 =
  fun _1 ->
    (
# 220 "lib/stateParser.mly"
           ((_1,ParsedConstant.zero))
# 912 "lib/stateParser.ml"
     : (MiscParser.location * ParsedConstant.v))

let _menhir_action_012 =
  fun _1 _3 ->
    (
# 221 "lib/stateParser.mly"
                              ((_1,_3))
# 920 "lib/stateParser.ml"
     : (MiscParser.location * ParsedConstant.v))

let _menhir_action_013 =
  fun _1 ->
    (
# 227 "lib/stateParser.mly"
       ( let x,v = _1 in x,(TyDef,v) )
# 928 "lib/stateParser.ml"
     : (MiscParser.state_atom))

let _menhir_action_014 =
  fun loc typ ->
    (
# 228 "lib/stateParser.mly"
                         ( (loc, (Ty typ,ParsedConstant.zero)) )
# 936 "lib/stateParser.ml"
     : (MiscParser.state_atom))

let _menhir_action_015 =
  fun loc typ ->
    (
# 229 "lib/stateParser.mly"
                               ( loc,(Atomic typ,ParsedConstant.zero))
# 944 "lib/stateParser.ml"
     : (MiscParser.state_atom))

let _menhir_action_016 =
  fun i loc ->
    (
# 230 "lib/stateParser.mly"
                              ( (loc,(Ty "ins_t", mk_instr_val i)) )
# 952 "lib/stateParser.ml"
     : (MiscParser.state_atom))

let _menhir_action_017 =
  fun i loc ->
    (
# 231 "lib/stateParser.mly"
                                   ( (loc,(Ty "ins_t", mk_instr_val i)) )
# 960 "lib/stateParser.ml"
     : (MiscParser.state_atom))

let _menhir_action_018 =
  fun loc typ v ->
    (
# 232 "lib/stateParser.mly"
                                              ( loc,(Atomic typ,v))
# 968 "lib/stateParser.ml"
     : (MiscParser.state_atom))

let _menhir_action_019 =
  fun loc typ v ->
    (
# 233 "lib/stateParser.mly"
                                       ( (loc,(Ty typ,v)))
# 976 "lib/stateParser.ml"
     : (MiscParser.state_atom))

let _menhir_action_020 =
  fun loc typ v ->
    (
# 235 "lib/stateParser.mly"
   ( (loc,(Ty typ,v)))
# 984 "lib/stateParser.ml"
     : (MiscParser.state_atom))

let _menhir_action_021 =
  fun _1 _2 ->
    (
# 237 "lib/stateParser.mly"
   ( let (t,sz) = _2 in
     let v0 = Constant.mk_replicate sz ParsedConstant.zero in
     (t,(TyArray (_1,sz),v0)) )
# 994 "lib/stateParser.ml"
     : (MiscParser.state_atom))

let _menhir_action_022 =
  fun _1 _2 _5 ->
    (
# 241 "lib/stateParser.mly"
   ( let (t,sz) = _2 and vs = _5 in
     if sz = List.length vs then
       let arr = (TyArray (_1,sz),Constant.mk_vec sz vs) in
       (t, arr)
     else
       Warn.user_error
         "Declared size of array %s does not match initial value size"
	 (dump_location t) )
# 1009 "lib/stateParser.ml"
     : (MiscParser.state_atom))

let _menhir_action_023 =
  fun () ->
    (
# 250 "lib/stateParser.mly"
                        ( raise Parsing.Parse_error )
# 1017 "lib/stateParser.ml"
     : (MiscParser.state_atom))

let _menhir_action_024 =
  fun loc typ ->
    (
# 251 "lib/stateParser.mly"
                             ( (loc,(Pointer typ,ParsedConstant.zero)))
# 1025 "lib/stateParser.ml"
     : (MiscParser.state_atom))

let _menhir_action_025 =
  fun loc typ v ->
    (
# 252 "lib/stateParser.mly"
                                                     ( (loc,(Pointer typ,v)))
# 1033 "lib/stateParser.ml"
     : (MiscParser.state_atom))

let _menhir_action_026 =
  fun loc ->
    (
# 253 "lib/stateParser.mly"
                    ( (loc,(TyDefPointer,ParsedConstant.zero)))
# 1041 "lib/stateParser.ml"
     : (MiscParser.state_atom))

let _menhir_action_027 =
  fun loc v ->
    (
# 254 "lib/stateParser.mly"
                                            ( (loc,(TyDefPointer,v)))
# 1049 "lib/stateParser.ml"
     : (MiscParser.state_atom))

let _menhir_action_028 =
  fun loc typ v ->
    (
# 256 "lib/stateParser.mly"
  ( (loc,(Ty typ, MiscParser.add_oa_if_none loc v)) )
# 1057 "lib/stateParser.ml"
     : (MiscParser.state_atom))

let _menhir_action_029 =
  fun loc v ->
    (
# 258 "lib/stateParser.mly"
  ( (loc,(Ty "pteval_t", MiscParser.add_oa_if_none loc v)) )
# 1065 "lib/stateParser.ml"
     : (MiscParser.state_atom))

let _menhir_action_030 =
  fun i loc ->
    (
# 410 "lib/stateParser.mly"
  (Atom (LV (Loc loc,(mk_instr_val i))))
# 1073 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_031 =
  fun l loc ->
    (
# 412 "lib/stateParser.mly"
    (Atom (LV (Loc loc,(mk_lab l))))
# 1081 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_032 =
  fun _1 _3 ->
    let _2 = 
# 402 "lib/stateParser.mly"
        ( () )
# 1089 "lib/stateParser.ml"
     in
    (
# 413 "lib/stateParser.mly"
                        (Atom (LV (Loc _1,_3)))
# 1094 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_033 =
  fun _1 _3 ->
    let _2 = 
# 403 "lib/stateParser.mly"
             ( () )
# 1102 "lib/stateParser.ml"
     in
    (
# 413 "lib/stateParser.mly"
                        (Atom (LV (Loc _1,_3)))
# 1107 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_034 =
  fun loc v ->
    let _2 = 
# 402 "lib/stateParser.mly"
        ( () )
# 1115 "lib/stateParser.ml"
     in
    let loc = 
# 406 "lib/stateParser.mly"
                                ( Location_global loc )
# 1120 "lib/stateParser.ml"
     in
    (
# 415 "lib/stateParser.mly"
   (Atom (LV (Loc loc,v)))
# 1125 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_035 =
  fun loc v ->
    let _2 = 
# 403 "lib/stateParser.mly"
             ( () )
# 1133 "lib/stateParser.ml"
     in
    let loc = 
# 406 "lib/stateParser.mly"
                                ( Location_global loc )
# 1138 "lib/stateParser.ml"
     in
    (
# 415 "lib/stateParser.mly"
   (Atom (LV (Loc loc,v)))
# 1143 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_036 =
  fun _1 _3 ->
    (
# 416 "lib/stateParser.mly"
                           (Not (Atom (LV (Loc _1,_3))))
# 1151 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_037 =
  fun loc v ->
    let loc = 
# 406 "lib/stateParser.mly"
                                ( Location_global loc )
# 1159 "lib/stateParser.ml"
     in
    (
# 418 "lib/stateParser.mly"
   (Not (Atom (LV (Loc loc,v))))
# 1164 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_038 =
  fun loc v ->
    let _2 = 
# 402 "lib/stateParser.mly"
        ( () )
# 1172 "lib/stateParser.ml"
     in
    (
# 420 "lib/stateParser.mly"
  ( Atom (LV (Loc loc, MiscParser.add_oa_if_none loc v )) )
# 1177 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_039 =
  fun loc v ->
    let _2 = 
# 403 "lib/stateParser.mly"
             ( () )
# 1185 "lib/stateParser.ml"
     in
    (
# 420 "lib/stateParser.mly"
  ( Atom (LV (Loc loc, MiscParser.add_oa_if_none loc v )) )
# 1190 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_040 =
  fun loc v ->
    let _2 = 
# 402 "lib/stateParser.mly"
        ( () )
# 1198 "lib/stateParser.ml"
     in
    let loc = 
# 406 "lib/stateParser.mly"
                                ( Location_global loc )
# 1203 "lib/stateParser.ml"
     in
    (
# 422 "lib/stateParser.mly"
  ( Atom (LV (Loc loc, MiscParser.add_oa_if_none loc v)) )
# 1208 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_041 =
  fun loc v ->
    let _2 = 
# 403 "lib/stateParser.mly"
             ( () )
# 1216 "lib/stateParser.ml"
     in
    let loc = 
# 406 "lib/stateParser.mly"
                                ( Location_global loc )
# 1221 "lib/stateParser.ml"
     in
    (
# 422 "lib/stateParser.mly"
  ( Atom (LV (Loc loc, MiscParser.add_oa_if_none loc v)) )
# 1226 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_042 =
  fun _1 _4 ->
    let _2 = 
# 402 "lib/stateParser.mly"
        ( () )
# 1234 "lib/stateParser.ml"
     in
    (
# 425 "lib/stateParser.mly"
    ( let sz = List.length _4 in
      let vec = Constant.mk_vec sz _4 in
      Atom (LV (Loc _1,vec)) )
# 1241 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_043 =
  fun _1 _4 ->
    let _2 = 
# 403 "lib/stateParser.mly"
             ( () )
# 1249 "lib/stateParser.ml"
     in
    (
# 425 "lib/stateParser.mly"
    ( let sz = List.length _4 in
      let vec = Constant.mk_vec sz _4 in
      Atom (LV (Loc _1,vec)) )
# 1256 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_044 =
  fun _1 _4 ->
    (
# 429 "lib/stateParser.mly"
    ( let sz = List.length _4 in
      let vec = Constant.mk_vec sz _4 in
      Not (Atom (LV (Loc _1,vec))) )
# 1266 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_045 =
  fun _1 _3 ->
    let _2 = 
# 402 "lib/stateParser.mly"
        ( () )
# 1274 "lib/stateParser.ml"
     in
    (
# 432 "lib/stateParser.mly"
                        (Atom (LV (_1,_3)))
# 1279 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_046 =
  fun _1 _3 ->
    let _2 = 
# 403 "lib/stateParser.mly"
             ( () )
# 1287 "lib/stateParser.ml"
     in
    (
# 432 "lib/stateParser.mly"
                        (Atom (LV (_1,_3)))
# 1292 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_047 =
  fun _1 _3 ->
    (
# 433 "lib/stateParser.mly"
                           (Not (Atom (LV (_1,_3))))
# 1300 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_048 =
  fun _1 _3 ->
    let _2 = 
# 402 "lib/stateParser.mly"
        ( () )
# 1308 "lib/stateParser.ml"
     in
    (
# 434 "lib/stateParser.mly"
                                (Atom (LL (_1,_3)))
# 1313 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_049 =
  fun _1 _3 ->
    let _2 = 
# 403 "lib/stateParser.mly"
             ( () )
# 1321 "lib/stateParser.ml"
     in
    (
# 434 "lib/stateParser.mly"
                                (Atom (LL (_1,_3)))
# 1326 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_050 =
  fun loc loc_inlined1 ->
    let loc2 =
      let loc = loc_inlined1 in
      
# 406 "lib/stateParser.mly"
                                ( Location_global loc )
# 1336 "lib/stateParser.ml"
      
    in
    let _2 = 
# 402 "lib/stateParser.mly"
        ( () )
# 1342 "lib/stateParser.ml"
     in
    let loc1 = 
# 406 "lib/stateParser.mly"
                                ( Location_global loc )
# 1347 "lib/stateParser.ml"
     in
    (
# 436 "lib/stateParser.mly"
    ( Atom (LL (loc1,loc2)) )
# 1352 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_051 =
  fun loc loc_inlined1 ->
    let loc2 =
      let loc = loc_inlined1 in
      
# 406 "lib/stateParser.mly"
                                ( Location_global loc )
# 1362 "lib/stateParser.ml"
      
    in
    let _2 = 
# 403 "lib/stateParser.mly"
             ( () )
# 1368 "lib/stateParser.ml"
     in
    let loc1 = 
# 406 "lib/stateParser.mly"
                                ( Location_global loc )
# 1373 "lib/stateParser.ml"
     in
    (
# 436 "lib/stateParser.mly"
    ( Atom (LL (loc1,loc2)) )
# 1378 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_052 =
  fun _1_inlined2 _3_inlined1 loc ->
    let loc2 =
      let (_3, _1) = (_3_inlined1, _1_inlined2) in
      
# 188 "lib/stateParser.mly"
                  (Location_reg (_1,_3))
# 1388 "lib/stateParser.ml"
      
    in
    let _2 = 
# 402 "lib/stateParser.mly"
        ( () )
# 1394 "lib/stateParser.ml"
     in
    let loc1 = 
# 406 "lib/stateParser.mly"
                                ( Location_global loc )
# 1399 "lib/stateParser.ml"
     in
    (
# 438 "lib/stateParser.mly"
    ( Atom (LL (loc1,loc2)) )
# 1404 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_053 =
  fun _1_inlined2 _3_inlined1 loc ->
    let loc2 =
      let (_3, _1) = (_3_inlined1, _1_inlined2) in
      
# 189 "lib/stateParser.mly"
                  (Location_reg (Misc.string_as_int _1,_3))
# 1414 "lib/stateParser.ml"
      
    in
    let _2 = 
# 402 "lib/stateParser.mly"
        ( () )
# 1420 "lib/stateParser.ml"
     in
    let loc1 = 
# 406 "lib/stateParser.mly"
                                ( Location_global loc )
# 1425 "lib/stateParser.ml"
     in
    (
# 438 "lib/stateParser.mly"
    ( Atom (LL (loc1,loc2)) )
# 1430 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_054 =
  fun _1_inlined2 _3_inlined1 loc ->
    let loc2 =
      let (_3, _1) = (_3_inlined1, _1_inlined2) in
      
# 190 "lib/stateParser.mly"
                       (Location_reg (_1,_3))
# 1440 "lib/stateParser.ml"
      
    in
    let _2 = 
# 402 "lib/stateParser.mly"
        ( () )
# 1446 "lib/stateParser.ml"
     in
    let loc1 = 
# 406 "lib/stateParser.mly"
                                ( Location_global loc )
# 1451 "lib/stateParser.ml"
     in
    (
# 438 "lib/stateParser.mly"
    ( Atom (LL (loc1,loc2)) )
# 1456 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_055 =
  fun _1_inlined2 _3_inlined1 loc ->
    let loc2 =
      let (_3, _1) = (_3_inlined1, _1_inlined2) in
      
# 191 "lib/stateParser.mly"
                       (Location_reg (Misc.string_as_int _1,_3))
# 1466 "lib/stateParser.ml"
      
    in
    let _2 = 
# 402 "lib/stateParser.mly"
        ( () )
# 1472 "lib/stateParser.ml"
     in
    let loc1 = 
# 406 "lib/stateParser.mly"
                                ( Location_global loc )
# 1477 "lib/stateParser.ml"
     in
    (
# 438 "lib/stateParser.mly"
    ( Atom (LL (loc1,loc2)) )
# 1482 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_056 =
  fun _1_inlined2 loc ->
    let loc2 =
      let _1 = _1_inlined2 in
      
# 192 "lib/stateParser.mly"
                  (Location_sreg _1 )
# 1492 "lib/stateParser.ml"
      
    in
    let _2 = 
# 402 "lib/stateParser.mly"
        ( () )
# 1498 "lib/stateParser.ml"
     in
    let loc1 = 
# 406 "lib/stateParser.mly"
                                ( Location_global loc )
# 1503 "lib/stateParser.ml"
     in
    (
# 438 "lib/stateParser.mly"
    ( Atom (LL (loc1,loc2)) )
# 1508 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_057 =
  fun _1_inlined2 _5 loc ->
    let loc2 =
      let _1 = _1_inlined2 in
      
# 195 "lib/stateParser.mly"
                  (Location_reg(Misc.string_as_int _1,_5))
# 1518 "lib/stateParser.ml"
      
    in
    let _2 = 
# 402 "lib/stateParser.mly"
        ( () )
# 1524 "lib/stateParser.ml"
     in
    let loc1 = 
# 406 "lib/stateParser.mly"
                                ( Location_global loc )
# 1529 "lib/stateParser.ml"
     in
    (
# 438 "lib/stateParser.mly"
    ( Atom (LL (loc1,loc2)) )
# 1534 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_058 =
  fun _1_inlined2 _5 loc ->
    let loc2 =
      let _1 = _1_inlined2 in
      
# 197 "lib/stateParser.mly"
                  (Location_reg(_1,_5))
# 1544 "lib/stateParser.ml"
      
    in
    let _2 = 
# 402 "lib/stateParser.mly"
        ( () )
# 1550 "lib/stateParser.ml"
     in
    let loc1 = 
# 406 "lib/stateParser.mly"
                                ( Location_global loc )
# 1555 "lib/stateParser.ml"
     in
    (
# 438 "lib/stateParser.mly"
    ( Atom (LL (loc1,loc2)) )
# 1560 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_059 =
  fun _1_inlined2 _3_inlined1 loc ->
    let loc2 =
      let (_3, _1) = (_3_inlined1, _1_inlined2) in
      
# 188 "lib/stateParser.mly"
                  (Location_reg (_1,_3))
# 1570 "lib/stateParser.ml"
      
    in
    let _2 = 
# 403 "lib/stateParser.mly"
             ( () )
# 1576 "lib/stateParser.ml"
     in
    let loc1 = 
# 406 "lib/stateParser.mly"
                                ( Location_global loc )
# 1581 "lib/stateParser.ml"
     in
    (
# 438 "lib/stateParser.mly"
    ( Atom (LL (loc1,loc2)) )
# 1586 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_060 =
  fun _1_inlined2 _3_inlined1 loc ->
    let loc2 =
      let (_3, _1) = (_3_inlined1, _1_inlined2) in
      
# 189 "lib/stateParser.mly"
                  (Location_reg (Misc.string_as_int _1,_3))
# 1596 "lib/stateParser.ml"
      
    in
    let _2 = 
# 403 "lib/stateParser.mly"
             ( () )
# 1602 "lib/stateParser.ml"
     in
    let loc1 = 
# 406 "lib/stateParser.mly"
                                ( Location_global loc )
# 1607 "lib/stateParser.ml"
     in
    (
# 438 "lib/stateParser.mly"
    ( Atom (LL (loc1,loc2)) )
# 1612 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_061 =
  fun _1_inlined2 _3_inlined1 loc ->
    let loc2 =
      let (_3, _1) = (_3_inlined1, _1_inlined2) in
      
# 190 "lib/stateParser.mly"
                       (Location_reg (_1,_3))
# 1622 "lib/stateParser.ml"
      
    in
    let _2 = 
# 403 "lib/stateParser.mly"
             ( () )
# 1628 "lib/stateParser.ml"
     in
    let loc1 = 
# 406 "lib/stateParser.mly"
                                ( Location_global loc )
# 1633 "lib/stateParser.ml"
     in
    (
# 438 "lib/stateParser.mly"
    ( Atom (LL (loc1,loc2)) )
# 1638 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_062 =
  fun _1_inlined2 _3_inlined1 loc ->
    let loc2 =
      let (_3, _1) = (_3_inlined1, _1_inlined2) in
      
# 191 "lib/stateParser.mly"
                       (Location_reg (Misc.string_as_int _1,_3))
# 1648 "lib/stateParser.ml"
      
    in
    let _2 = 
# 403 "lib/stateParser.mly"
             ( () )
# 1654 "lib/stateParser.ml"
     in
    let loc1 = 
# 406 "lib/stateParser.mly"
                                ( Location_global loc )
# 1659 "lib/stateParser.ml"
     in
    (
# 438 "lib/stateParser.mly"
    ( Atom (LL (loc1,loc2)) )
# 1664 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_063 =
  fun _1_inlined2 loc ->
    let loc2 =
      let _1 = _1_inlined2 in
      
# 192 "lib/stateParser.mly"
                  (Location_sreg _1 )
# 1674 "lib/stateParser.ml"
      
    in
    let _2 = 
# 403 "lib/stateParser.mly"
             ( () )
# 1680 "lib/stateParser.ml"
     in
    let loc1 = 
# 406 "lib/stateParser.mly"
                                ( Location_global loc )
# 1685 "lib/stateParser.ml"
     in
    (
# 438 "lib/stateParser.mly"
    ( Atom (LL (loc1,loc2)) )
# 1690 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_064 =
  fun _1_inlined2 _5 loc ->
    let loc2 =
      let _1 = _1_inlined2 in
      
# 195 "lib/stateParser.mly"
                  (Location_reg(Misc.string_as_int _1,_5))
# 1700 "lib/stateParser.ml"
      
    in
    let _2 = 
# 403 "lib/stateParser.mly"
             ( () )
# 1706 "lib/stateParser.ml"
     in
    let loc1 = 
# 406 "lib/stateParser.mly"
                                ( Location_global loc )
# 1711 "lib/stateParser.ml"
     in
    (
# 438 "lib/stateParser.mly"
    ( Atom (LL (loc1,loc2)) )
# 1716 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_065 =
  fun _1_inlined2 _5 loc ->
    let loc2 =
      let _1 = _1_inlined2 in
      
# 197 "lib/stateParser.mly"
                  (Location_reg(_1,_5))
# 1726 "lib/stateParser.ml"
      
    in
    let _2 = 
# 403 "lib/stateParser.mly"
             ( () )
# 1732 "lib/stateParser.ml"
     in
    let loc1 = 
# 406 "lib/stateParser.mly"
                                ( Location_global loc )
# 1737 "lib/stateParser.ml"
     in
    (
# 438 "lib/stateParser.mly"
    ( Atom (LL (loc1,loc2)) )
# 1742 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_066 =
  fun _1 ->
    (
# 444 "lib/stateParser.mly"
        ( Atom (FF _1) )
# 1750 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_067 =
  fun fst rem ->
    (
# 289 "lib/stateParser.mly"
     ( String.concat ":" (fst::rem) )
# 1758 "lib/stateParser.ml"
     : (string))

let _menhir_action_068 =
  fun () ->
    (
# 354 "lib/stateParser.mly"
   ( ConstrGen.constr_true )
# 1766 "lib/stateParser.ml"
     : (MiscParser.constr))

let _menhir_action_069 =
  fun _2 ->
    (
# 356 "lib/stateParser.mly"
    (ForallStates _2)
# 1774 "lib/stateParser.ml"
     : (MiscParser.constr))

let _menhir_action_070 =
  fun _2 ->
    (
# 358 "lib/stateParser.mly"
    (ExistsState _2)
# 1782 "lib/stateParser.ml"
     : (MiscParser.constr))

let _menhir_action_071 =
  fun _3 ->
    (
# 360 "lib/stateParser.mly"
        ( NotExistsState _3 )
# 1790 "lib/stateParser.ml"
     : (MiscParser.constr))

let _menhir_action_072 =
  fun _2 ->
    (
# 362 "lib/stateParser.mly"
        ( ExistsState _2 )
# 1798 "lib/stateParser.ml"
     : (MiscParser.constr))

let _menhir_action_073 =
  fun _2 ->
    (
# 364 "lib/stateParser.mly"
    (ExistsState _2)
# 1806 "lib/stateParser.ml"
     : (MiscParser.constr))

let _menhir_action_074 =
  fun _2 ->
    (
# 366 "lib/stateParser.mly"
    ( ExistsState (Or _2) )
# 1814 "lib/stateParser.ml"
     : (MiscParser.constr))

let _menhir_action_075 =
  fun _1 _2 _3 ->
    (
# 325 "lib/stateParser.mly"
  ( let x = _1 in
    let f = _2 in
    let y,z = _3 in
    x,f,y,z )
# 1825 "lib/stateParser.ml"
     : ((MiscParser.location, ParsedConstant.v, string) LocationsItem.t list *
  MiscParser.prop option * MiscParser.constr *
  (string * MiscParser.quantifier) list))

let _menhir_action_076 =
  fun lab ->
    (
# 291 "lib/stateParser.mly"
                          ( (lab,None,None) )
# 1835 "lib/stateParser.ml"
     : ((ParsedConstant.v, string) Fault.atom))

let _menhir_action_077 =
  fun lab name ->
    (
# 293 "lib/stateParser.mly"
   ( if FaultType.is name then (lab,None,Some name)
     else (lab,Some (Constant.mk_sym name),None) )
# 1844 "lib/stateParser.ml"
     : ((ParsedConstant.v, string) Fault.atom))

let _menhir_action_078 =
  fun fl lab ->
    (
# 295 "lib/stateParser.mly"
                                                 ( (lab,Some fl,None) )
# 1852 "lib/stateParser.ml"
     : ((ParsedConstant.v, string) Fault.atom))

let _menhir_action_079 =
  fun ft lab ->
    (
# 296 "lib/stateParser.mly"
                                                       ( (lab,None,Some ft) )
# 1860 "lib/stateParser.ml"
     : ((ParsedConstant.v, string) Fault.atom))

let _menhir_action_080 =
  fun fl ft lab ->
    (
# 299 "lib/stateParser.mly"
    ( (lab,Some fl,Some (String.concat ":" ft)) )
# 1868 "lib/stateParser.ml"
     : ((ParsedConstant.v, string) Fault.atom))

let _menhir_action_081 =
  fun name ->
    (
# 284 "lib/stateParser.mly"
            ( Constant.mk_sym name )
# 1876 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_082 =
  fun fl ->
    (
# 285 "lib/stateParser.mly"
                   ( fl )
# 1884 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_083 =
  fun name ->
    (
# 281 "lib/stateParser.mly"
                              ( (Constant.mk_sym_pte name) )
# 1892 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_084 =
  fun () ->
    (
# 317 "lib/stateParser.mly"
  ( None )
# 1900 "lib/stateParser.ml"
     : (MiscParser.prop option))

let _menhir_action_085 =
  fun _2 ->
    (
# 318 "lib/stateParser.mly"
              ( Some _2 )
# 1908 "lib/stateParser.ml"
     : (MiscParser.prop option))

let _menhir_action_086 =
  fun _1 ->
    (
# 347 "lib/stateParser.mly"
         ( _1 )
# 1916 "lib/stateParser.ml"
     : (MiscParser.constr))

let _menhir_action_087 =
  fun _1 ->
    (
# 348 "lib/stateParser.mly"
              ( _1 )
# 1924 "lib/stateParser.ml"
     : (MiscParser.constr))

let _menhir_action_088 =
  fun st ->
    (
# 116 "lib/stateParser.mly"
                        ( st )
# 1932 "lib/stateParser.ml"
     : (MiscParser.state))

let _menhir_action_089 =
  fun () ->
    (
# 266 "lib/stateParser.mly"
  ([])
# 1940 "lib/stateParser.ml"
     : (MiscParser.state))

let _menhir_action_090 =
  fun () ->
    (
# 267 "lib/stateParser.mly"
       ([])
# 1948 "lib/stateParser.ml"
     : (MiscParser.state))

let _menhir_action_091 =
  fun _1 ->
    (
# 268 "lib/stateParser.mly"
            (_1::[])
# 1956 "lib/stateParser.ml"
     : (MiscParser.state))

let _menhir_action_092 =
  fun _1 _3 ->
    (
# 269 "lib/stateParser.mly"
                                 (_1::_3)
# 1964 "lib/stateParser.ml"
     : (MiscParser.state))

let _menhir_action_093 =
  fun () ->
    (
# 224 "lib/stateParser.mly"
          ( None )
# 1972 "lib/stateParser.ml"
     : (string option))

let _menhir_action_094 =
  fun i ->
    (
# 225 "lib/stateParser.mly"
          ( Some i )
# 1980 "lib/stateParser.ml"
     : (string option))

let _menhir_action_095 =
  fun _1 ->
    (
# 341 "lib/stateParser.mly"
                    ( (_1,Require) )
# 1988 "lib/stateParser.ml"
     : (string * MiscParser.quantifier))

let _menhir_action_096 =
  fun _1 ->
    (
# 342 "lib/stateParser.mly"
                    ( (_1,Allow) )
# 1996 "lib/stateParser.ml"
     : (string * MiscParser.quantifier))

let _menhir_action_097 =
  fun _1 ->
    (
# 343 "lib/stateParser.mly"
                        ( (_1,Forbid) )
# 2004 "lib/stateParser.ml"
     : (string * MiscParser.quantifier))

let _menhir_action_098 =
  fun _1 ->
    (
# 336 "lib/stateParser.mly"
               ( [_1] )
# 2012 "lib/stateParser.ml"
     : ((string * MiscParser.quantifier) list))

let _menhir_action_099 =
  fun _1 ->
    (
# 337 "lib/stateParser.mly"
               ( [_1] )
# 2020 "lib/stateParser.ml"
     : ((string * MiscParser.quantifier) list))

let _menhir_action_100 =
  fun _1 _3 ->
    (
# 338 "lib/stateParser.mly"
                    ( _1 :: _3 )
# 2028 "lib/stateParser.ml"
     : ((string * MiscParser.quantifier) list))

let _menhir_action_101 =
  fun _1 ->
    (
# 386 "lib/stateParser.mly"
       ( (_1,None) )
# 2036 "lib/stateParser.ml"
     : (int * string option))

let _menhir_action_102 =
  fun _1 _3 ->
    (
# 387 "lib/stateParser.mly"
                  ( (_1,Some _3) )
# 2044 "lib/stateParser.ml"
     : (int * string option))

let _menhir_action_103 =
  fun loc ->
    (
# 216 "lib/stateParser.mly"
               ( loc )
# 2052 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_104 =
  fun loc ->
    (
# 217 "lib/stateParser.mly"
                                ( Location_global loc )
# 2060 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_105 =
  fun _1 _2 ->
    (
# 380 "lib/stateParser.mly"
                   ( _1,_2 )
# 2068 "lib/stateParser.ml"
     : ((MiscParser.location, ParsedConstant.v, string) LocationsItem.t list *
  MiscParser.constr))

let _menhir_action_106 =
  fun _1 ->
    (
# 303 "lib/stateParser.mly"
           ( let a,t = _1 in LocationsItem.Loc (a,t) )
# 2077 "lib/stateParser.ml"
     : ((MiscParser.location, ParsedConstant.v, string) LocationsItem.t))

let _menhir_action_107 =
  fun _1 ->
    (
# 304 "lib/stateParser.mly"
        (  Fault _1 )
# 2085 "lib/stateParser.ml"
     : ((MiscParser.location, ParsedConstant.v, string) LocationsItem.t))

let _menhir_action_108 =
  fun _1 ->
    (
# 307 "lib/stateParser.mly"
           (_1::[])
# 2093 "lib/stateParser.ml"
     : ((MiscParser.location, ParsedConstant.v, string) LocationsItem.t list))

let _menhir_action_109 =
  fun _1 ->
    (
# 308 "lib/stateParser.mly"
                (_1::[])
# 2101 "lib/stateParser.ml"
     : ((MiscParser.location, ParsedConstant.v, string) LocationsItem.t list))

let _menhir_action_110 =
  fun _1 _3 ->
    (
# 309 "lib/stateParser.mly"
                           (_1::_3)
# 2109 "lib/stateParser.ml"
     : ((MiscParser.location, ParsedConstant.v, string) LocationsItem.t list))

let _menhir_action_111 =
  fun _1 _3 ->
    let _1 = 
# 188 "lib/stateParser.mly"
                  (Location_reg (_1,_3))
# 2117 "lib/stateParser.ml"
     in
    (
# 212 "lib/stateParser.mly"
               ( _1 )
# 2122 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_112 =
  fun _1 _3 ->
    let _1 = 
# 189 "lib/stateParser.mly"
                  (Location_reg (Misc.string_as_int _1,_3))
# 2130 "lib/stateParser.ml"
     in
    (
# 212 "lib/stateParser.mly"
               ( _1 )
# 2135 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_113 =
  fun _1 _3 ->
    let _1 = 
# 190 "lib/stateParser.mly"
                       (Location_reg (_1,_3))
# 2143 "lib/stateParser.ml"
     in
    (
# 212 "lib/stateParser.mly"
               ( _1 )
# 2148 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_114 =
  fun _1 _3 ->
    let _1 = 
# 191 "lib/stateParser.mly"
                       (Location_reg (Misc.string_as_int _1,_3))
# 2156 "lib/stateParser.ml"
     in
    (
# 212 "lib/stateParser.mly"
               ( _1 )
# 2161 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_115 =
  fun _1 ->
    let _1 = 
# 192 "lib/stateParser.mly"
                  (Location_sreg _1 )
# 2169 "lib/stateParser.ml"
     in
    (
# 212 "lib/stateParser.mly"
               ( _1 )
# 2174 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_116 =
  fun _1 _5 ->
    let _1 = 
# 195 "lib/stateParser.mly"
                  (Location_reg(Misc.string_as_int _1,_5))
# 2182 "lib/stateParser.ml"
     in
    (
# 212 "lib/stateParser.mly"
               ( _1 )
# 2187 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_117 =
  fun _1 _5 ->
    let _1 = 
# 197 "lib/stateParser.mly"
                  (Location_reg(_1,_5))
# 2195 "lib/stateParser.ml"
     in
    (
# 212 "lib/stateParser.mly"
               ( _1 )
# 2200 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_118 =
  fun _1 ->
    (
# 213 "lib/stateParser.mly"
                  ( Location_global _1 )
# 2208 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_119 =
  fun _1 _3 ->
    let _1 = 
# 188 "lib/stateParser.mly"
                  (Location_reg (_1,_3))
# 2216 "lib/stateParser.ml"
     in
    (
# 203 "lib/stateParser.mly"
               ( _1 )
# 2221 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_120 =
  fun _1 _3 ->
    let _1 = 
# 189 "lib/stateParser.mly"
                  (Location_reg (Misc.string_as_int _1,_3))
# 2229 "lib/stateParser.ml"
     in
    (
# 203 "lib/stateParser.mly"
               ( _1 )
# 2234 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_121 =
  fun _1 _3 ->
    let _1 = 
# 190 "lib/stateParser.mly"
                       (Location_reg (_1,_3))
# 2242 "lib/stateParser.ml"
     in
    (
# 203 "lib/stateParser.mly"
               ( _1 )
# 2247 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_122 =
  fun _1 _3 ->
    let _1 = 
# 191 "lib/stateParser.mly"
                       (Location_reg (Misc.string_as_int _1,_3))
# 2255 "lib/stateParser.ml"
     in
    (
# 203 "lib/stateParser.mly"
               ( _1 )
# 2260 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_123 =
  fun _1 ->
    let _1 = 
# 192 "lib/stateParser.mly"
                  (Location_sreg _1 )
# 2268 "lib/stateParser.ml"
     in
    (
# 203 "lib/stateParser.mly"
               ( _1 )
# 2273 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_124 =
  fun _1 _5 ->
    let _1 = 
# 195 "lib/stateParser.mly"
                  (Location_reg(Misc.string_as_int _1,_5))
# 2281 "lib/stateParser.ml"
     in
    (
# 203 "lib/stateParser.mly"
               ( _1 )
# 2286 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_125 =
  fun _1 _5 ->
    let _1 = 
# 197 "lib/stateParser.mly"
                  (Location_reg(_1,_5))
# 2294 "lib/stateParser.ml"
     in
    (
# 203 "lib/stateParser.mly"
               ( _1 )
# 2299 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_126 =
  fun _1_inlined1 _3 ->
    let _2 =
      let _1 = _1_inlined1 in
      
# 188 "lib/stateParser.mly"
                  (Location_reg (_1,_3))
# 2309 "lib/stateParser.ml"
      
    in
    (
# 204 "lib/stateParser.mly"
                    ( _2 )
# 2315 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_127 =
  fun _1_inlined1 _3 ->
    let _2 =
      let _1 = _1_inlined1 in
      
# 189 "lib/stateParser.mly"
                  (Location_reg (Misc.string_as_int _1,_3))
# 2325 "lib/stateParser.ml"
      
    in
    (
# 204 "lib/stateParser.mly"
                    ( _2 )
# 2331 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_128 =
  fun _1_inlined1 _3 ->
    let _2 =
      let _1 = _1_inlined1 in
      
# 190 "lib/stateParser.mly"
                       (Location_reg (_1,_3))
# 2341 "lib/stateParser.ml"
      
    in
    (
# 204 "lib/stateParser.mly"
                    ( _2 )
# 2347 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_129 =
  fun _1_inlined1 _3 ->
    let _2 =
      let _1 = _1_inlined1 in
      
# 191 "lib/stateParser.mly"
                       (Location_reg (Misc.string_as_int _1,_3))
# 2357 "lib/stateParser.ml"
      
    in
    (
# 204 "lib/stateParser.mly"
                    ( _2 )
# 2363 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_130 =
  fun _1_inlined1 ->
    let _2 =
      let _1 = _1_inlined1 in
      
# 192 "lib/stateParser.mly"
                  (Location_sreg _1 )
# 2373 "lib/stateParser.ml"
      
    in
    (
# 204 "lib/stateParser.mly"
                    ( _2 )
# 2379 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_131 =
  fun _1_inlined1 _5 ->
    let _2 =
      let _1 = _1_inlined1 in
      
# 195 "lib/stateParser.mly"
                  (Location_reg(Misc.string_as_int _1,_5))
# 2389 "lib/stateParser.ml"
      
    in
    (
# 204 "lib/stateParser.mly"
                    ( _2 )
# 2395 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_132 =
  fun _1_inlined1 _5 ->
    let _2 =
      let _1 = _1_inlined1 in
      
# 197 "lib/stateParser.mly"
                  (Location_reg(_1,_5))
# 2405 "lib/stateParser.ml"
      
    in
    (
# 204 "lib/stateParser.mly"
                    ( _2 )
# 2411 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_133 =
  fun _2 ->
    (
# 205 "lib/stateParser.mly"
            ( Location_global (Constant.mk_sym _2) )
# 2419 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_134 =
  fun loc ->
    let _1 = 
# 406 "lib/stateParser.mly"
                                ( Location_global loc )
# 2427 "lib/stateParser.ml"
     in
    (
# 206 "lib/stateParser.mly"
          ( _1 )
# 2432 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_135 =
  fun _1 ->
    (
# 124 "lib/stateParser.mly"
       ( Constant.mk_sym _1  )
# 2440 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_136 =
  fun _3 ->
    (
# 125 "lib/stateParser.mly"
                         ( Constant.mk_sym_pte  _3 )
# 2448 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_137 =
  fun _5 ->
    (
# 126 "lib/stateParser.mly"
                                           ( Constant.mk_sym_pte2 _5 )
# 2456 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_138 =
  fun _3 ->
    (
# 127 "lib/stateParser.mly"
                        ( Constant.mk_sym_pa _3 )
# 2464 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_139 =
  fun _1 _3 ->
    (
# 128 "lib/stateParser.mly"
                  ( mk_sym_tag _1 _3 )
# 2472 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_140 =
  fun id ->
    (
# 129 "lib/stateParser.mly"
                            ( mk_sym_tagloc_zero id )
# 2480 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_141 =
  fun id o ->
    (
# 130 "lib/stateParser.mly"
                                       ( mk_sym_tagloc id o )
# 2488 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_142 =
  fun _1 _3 _5 ->
    (
# 132 "lib/stateParser.mly"
                           (mk_sym_morello _1 _3 _5)
# 2496 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_143 =
  fun _1 _3 ->
    (
# 133 "lib/stateParser.mly"
                 ( mk_sym_morello "0" _1 _3 )
# 2504 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_144 =
  fun _3 ->
    (
# 312 "lib/stateParser.mly"
                                ( _3 )
# 2512 "lib/stateParser.ml"
     : ((MiscParser.location, ParsedConstant.v, string) LocationsItem.t list))

let _menhir_action_145 =
  fun () ->
    (
# 313 "lib/stateParser.mly"
                      ( [] )
# 2520 "lib/stateParser.ml"
     : ((MiscParser.location, ParsedConstant.v, string) LocationsItem.t list))

let _menhir_action_146 =
  fun () ->
    (
# 314 "lib/stateParser.mly"
  ( [] )
# 2528 "lib/stateParser.ml"
     : ((MiscParser.location, ParsedConstant.v, string) LocationsItem.t list))

let _menhir_action_147 =
  fun _1 ->
    (
# 395 "lib/stateParser.mly"
    ( let (loc,sz) = _1 in Deref (loc,sz) )
# 2536 "lib/stateParser.ml"
     : (MiscParser.location ConstrGen.rloc))

let _menhir_action_148 =
  fun c ->
    (
# 351 "lib/stateParser.mly"
                 ( c )
# 2544 "lib/stateParser.ml"
     : (MiscParser.constr))

let _menhir_action_149 =
  fun f ->
    (
# 321 "lib/stateParser.mly"
               ( f )
# 2552 "lib/stateParser.ml"
     : (MiscParser.prop option))

let _menhir_action_150 =
  fun lc ->
    (
# 377 "lib/stateParser.mly"
                      ( lc )
# 2560 "lib/stateParser.ml"
     : ((MiscParser.location, ParsedConstant.v, string) LocationsItem.t list *
  MiscParser.constr))

let _menhir_action_151 =
  fun loc ->
    (
# 209 "lib/stateParser.mly"
                   ( loc )
# 2569 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_152 =
  fun _1 ->
    (
# 174 "lib/stateParser.mly"
               ( _1 )
# 2577 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_153 =
  fun _2 ->
    (
# 175 "lib/stateParser.mly"
              ( Tag _2 )
# 2585 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_154 =
  fun _1 ->
    (
# 182 "lib/stateParser.mly"
         ( _1 )
# 2593 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_155 =
  fun _1 _3 ->
    (
# 183 "lib/stateParser.mly"
                  ( mk_lab (_1, _3) )
# 2601 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_156 =
  fun _1 _3 ->
    (
# 184 "lib/stateParser.mly"
                 ( mk_lab (Misc.string_as_int _1, _3) )
# 2609 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_157 =
  fun l ->
    (
# 185 "lib/stateParser.mly"
          ( mk_lab l )
# 2617 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_158 =
  fun _1 _3 ->
    (
# 178 "lib/stateParser.mly"
                                 ( _1::_3 )
# 2625 "lib/stateParser.ml"
     : (ParsedConstant.v list))

let _menhir_action_159 =
  fun _1 ->
    (
# 179 "lib/stateParser.mly"
               ( [_1] )
# 2633 "lib/stateParser.ml"
     : (ParsedConstant.v list))

let _menhir_action_160 =
  fun _1 ->
    (
# 163 "lib/stateParser.mly"
       ( Concrete _1 )
# 2641 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_161 =
  fun _1 ->
    (
# 164 "lib/stateParser.mly"
        ( Concrete _1 )
# 2649 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_162 =
  fun _1 ->
    (
# 165 "lib/stateParser.mly"
                  ( _1 )
# 2657 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_163 =
  fun _1 _3 ->
    (
# 170 "lib/stateParser.mly"
                ( Concrete (_1 ^ ":" ^ _3) )
# 2665 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_164 =
  fun _1 _3 ->
    (
# 171 "lib/stateParser.mly"
                     ( mk_sym_with_index _1 _3 )
# 2673 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_165 =
  fun _1 ->
    (
# 136 "lib/stateParser.mly"
       ( _1 )
# 2681 "lib/stateParser.ml"
     : (string))

let _menhir_action_166 =
  fun _1 ->
    (
# 137 "lib/stateParser.mly"
      ( _1 )
# 2689 "lib/stateParser.ml"
     : (string))

let _menhir_action_167 =
  fun _1 ->
    (
# 369 "lib/stateParser.mly"
          (  [ And _1 ] )
# 2697 "lib/stateParser.ml"
     : ((MiscParser.location, ParsedConstant.v, string) ConstrGen.prop list))

let _menhir_action_168 =
  fun _1 _3 ->
    (
# 370 "lib/stateParser.mly"
                     ( And _1 :: _3 )
# 2705 "lib/stateParser.ml"
     : ((MiscParser.location, ParsedConstant.v, string) ConstrGen.prop list))

let _menhir_action_169 =
  fun () ->
    (
# 373 "lib/stateParser.mly"
                        ( [] )
# 2713 "lib/stateParser.ml"
     : (MiscParser.prop list))

let _menhir_action_170 =
  fun _1 _3 ->
    (
# 374 "lib/stateParser.mly"
                        ( _1 :: _3 )
# 2721 "lib/stateParser.ml"
     : (MiscParser.prop list))

let _menhir_action_171 =
  fun _1 ->
    (
# 331 "lib/stateParser.mly"
        ( _1,[] )
# 2729 "lib/stateParser.ml"
     : (MiscParser.constr * (string * MiscParser.quantifier) list))

let _menhir_action_172 =
  fun _1 _3 ->
    (
# 332 "lib/stateParser.mly"
                   ( _1,_3 )
# 2737 "lib/stateParser.ml"
     : (MiscParser.constr * (string * MiscParser.quantifier) list))

let _menhir_action_173 =
  fun name ->
    (
# 140 "lib/stateParser.mly"
            ( OutputAddress.parse name )
# 2745 "lib/stateParser.ml"
     : (OutputAddress.t))

let _menhir_action_174 =
  fun name ->
    (
# 141 "lib/stateParser.mly"
                             ( OutputAddress.PHY name )
# 2753 "lib/stateParser.ml"
     : (OutputAddress.t))

let _menhir_action_175 =
  fun name ->
    (
# 142 "lib/stateParser.mly"
                              ( OutputAddress.PTE name )
# 2761 "lib/stateParser.ml"
     : (OutputAddress.t))

let _menhir_action_176 =
  fun () ->
    (
# 448 "lib/stateParser.mly"
    (And [])
# 2769 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_177 =
  fun () ->
    (
# 450 "lib/stateParser.mly"
    (Or [])
# 2777 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_178 =
  fun _1 ->
    (
# 452 "lib/stateParser.mly"
    ( _1 )
# 2785 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_179 =
  fun _2 ->
    (
# 454 "lib/stateParser.mly"
    (Not _2)
# 2793 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_180 =
  fun _1 _3 ->
    (
# 456 "lib/stateParser.mly"
    (And [_1;_3])
# 2801 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_181 =
  fun _1 _3 ->
    (
# 458 "lib/stateParser.mly"
    (Or [_1;_3])
# 2809 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_182 =
  fun _1 _3 ->
    (
# 460 "lib/stateParser.mly"
    ( Implies (_1,_3) )
# 2817 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_183 =
  fun _2 ->
    (
# 462 "lib/stateParser.mly"
    ( _2 )
# 2825 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_184 =
  fun oa tail ->
    (
# 150 "lib/stateParser.mly"
    ( ParsedPteVal.add_oa oa tail )
# 2833 "lib/stateParser.ml"
     : (ParsedPteVal.t))

let _menhir_action_185 =
  fun key tail v ->
    (
# 152 "lib/stateParser.mly"
    ( ParsedPteVal.add_kv key v tail )
# 2841 "lib/stateParser.ml"
     : (ParsedPteVal.t))

let _menhir_action_186 =
  fun a tail ->
    (
# 154 "lib/stateParser.mly"
    ( ParsedPteVal.add_attr a tail )
# 2849 "lib/stateParser.ml"
     : (ParsedPteVal.t))

let _menhir_action_187 =
  fun attrs tail ->
    (
# 157 "lib/stateParser.mly"
    ( ParsedPteVal.add_attrs attrs tail )
# 2857 "lib/stateParser.ml"
     : (ParsedPteVal.t))

let _menhir_action_188 =
  fun () ->
    (
# 145 "lib/stateParser.mly"
  ( ParsedPteVal.empty )
# 2865 "lib/stateParser.ml"
     : (ParsedPteVal.t))

let _menhir_action_189 =
  fun pteval ->
    (
# 146 "lib/stateParser.mly"
                         ( pteval )
# 2873 "lib/stateParser.ml"
     : (ParsedPteVal.t))

let _menhir_action_190 =
  fun pteval ->
    (
# 160 "lib/stateParser.mly"
                             ( pteval )
# 2881 "lib/stateParser.ml"
     : (ParsedPteVal.t))

let _menhir_action_191 =
  fun _1 ->
    (
# 120 "lib/stateParser.mly"
             (  _1 )
# 2889 "lib/stateParser.ml"
     : (string))

let _menhir_action_192 =
  fun _1 ->
    (
# 121 "lib/stateParser.mly"
             (  _1 )
# 2897 "lib/stateParser.ml"
     : (string))

let _menhir_action_193 =
  fun _1 ->
    (
# 390 "lib/stateParser.mly"
           ( ConstrGen.Loc _1 )
# 2905 "lib/stateParser.ml"
     : (MiscParser.location ConstrGen.rloc))

let _menhir_action_194 =
  fun _1 ->
    (
# 391 "lib/stateParser.mly"
           ( _1 )
# 2913 "lib/stateParser.ml"
     : (MiscParser.location ConstrGen.rloc))

let _menhir_action_195 =
  fun _1 ->
    (
# 275 "lib/stateParser.mly"
       ( (_1, TyDef) )
# 2921 "lib/stateParser.ml"
     : (MiscParser.location ConstrGen.rloc * TestType.t))

let _menhir_action_196 =
  fun _1 ->
    (
# 276 "lib/stateParser.mly"
            ( (_1, TyDefPointer) )
# 2929 "lib/stateParser.ml"
     : (MiscParser.location ConstrGen.rloc * TestType.t))

let _menhir_action_197 =
  fun _1 _2 ->
    (
# 277 "lib/stateParser.mly"
            ( (_1, Ty _2) )
# 2937 "lib/stateParser.ml"
     : (MiscParser.location ConstrGen.rloc * TestType.t))

let _menhir_action_198 =
  fun _1 _2 ->
    (
# 278 "lib/stateParser.mly"
                 ( (_1, Pointer _2) )
# 2945 "lib/stateParser.ml"
     : (MiscParser.location ConstrGen.rloc * TestType.t))

let _menhir_action_199 =
  fun x ->
    (
# 250 "<standard.mly>"
    ( [ x ] )
# 2953 "lib/stateParser.ml"
     : (string list))

let _menhir_action_200 =
  fun x xs ->
    (
# 253 "<standard.mly>"
    ( x :: xs )
# 2961 "lib/stateParser.ml"
     : (string list))

let _menhir_action_201 =
  fun x ->
    (
# 250 "<standard.mly>"
    ( [ x ] )
# 2969 "lib/stateParser.ml"
     : (string list))

let _menhir_action_202 =
  fun x xs ->
    (
# 253 "<standard.mly>"
    ( x :: xs )
# 2977 "lib/stateParser.ml"
     : (string list))

let _menhir_action_203 =
  fun _2 ->
    (
# 383 "lib/stateParser.mly"
                       ( _2 )
# 2985 "lib/stateParser.ml"
     : (MiscParser.constr))

let _menhir_print_token : token -> string =
  fun _tok ->
    match _tok with
    | AMPER ->
        "AMPER"
    | AND ->
        "AND"
    | ATOMIC ->
        "ATOMIC"
    | ATOMICINIT ->
        "ATOMICINIT"
    | ATTRS ->
        "ATTRS"
    | COLON ->
        "COLON"
    | COMMA ->
        "COMMA"
    | DOLLARNAME _ ->
        "DOLLARNAME"
    | EOF ->
        "EOF"
    | EQUAL ->
        "EQUAL"
    | EQUALEQUAL ->
        "EQUALEQUAL"
    | EXISTS ->
        "EXISTS"
    | FALSE ->
        "FALSE"
    | FAULT ->
        "FAULT"
    | FILTER ->
        "FILTER"
    | FINAL ->
        "FINAL"
    | FORALL ->
        "FORALL"
    | IMPLIES ->
        "IMPLIES"
    | INSTR _ ->
        "INSTR"
    | LABEL _ ->
        "LABEL"
    | LBRK ->
        "LBRK"
    | LCURLY ->
        "LCURLY"
    | LOCATIONS ->
        "LOCATIONS"
    | LPAR ->
        "LPAR"
    | NAME _ ->
        "NAME"
    | NOT ->
        "NOT"
    | NOTEQUAL ->
        "NOTEQUAL"
    | NUM _ ->
        "NUM"
    | OBSERVED ->
        "OBSERVED"
    | OR ->
        "OR"
    | PLUS ->
        "PLUS"
    | PROC _ ->
        "PROC"
    | PTX_REG_DEC ->
        "PTX_REG_DEC"
    | PTX_REG_TYPE _ ->
        "PTX_REG_TYPE"
    | RBRK ->
        "RBRK"
    | RCURLY ->
        "RCURLY"
    | RPAR ->
        "RPAR"
    | SEMI ->
        "SEMI"
    | STAR ->
        "STAR"
    | SYMB_REG _ ->
        "SYMB_REG"
    | TOKAND ->
        "TOKAND"
    | TOK_NOP ->
        "TOK_NOP"
    | TOK_OA ->
        "TOK_OA"
    | TOK_PA ->
        "TOK_PA"
    | TOK_PTE ->
        "TOK_PTE"
    | TOK_TAG ->
        "TOK_TAG"
    | TRUE ->
        "TRUE"
    | VALUE _ ->
        "VALUE"
    | WITH ->
        "WITH"

let _menhir_fail : unit -> 'a =
  fun () ->
    Printf.eprintf "Internal failure -- please contact the parser generator's developers.\n%!";
    assert false

include struct
  
  [@@@ocaml.warning "-4-37"]
  
  let _menhir_run_394 : type  ttv_stack. ttv_stack -> _ -> _ -> _menhir_box_main_location =
    fun _menhir_stack _v _tok ->
      match (_tok : MenhirBasics.token) with
      | EOF ->
          let loc = _v in
          let _v = _menhir_action_151 loc in
          MenhirBox_main_location _v
      | _ ->
          _eRR ()
  
  let _menhir_run_404 : type  ttv_stack. (ttv_stack, _menhir_box_skip_loc_constr) _menhir_cell1_locations -> _ -> _ -> _menhir_box_skip_loc_constr =
    fun _menhir_stack _v _tok ->
      match (_tok : MenhirBasics.token) with
      | EOF ->
          let MenhirCell1_locations (_menhir_stack, _, _) = _menhir_stack in
          let _2 = _v in
          let _v = _menhir_action_203 _2 in
          MenhirBox_skip_loc_constr _v
      | _ ->
          _eRR ()
  
  let _menhir_run_389 : type  ttv_stack. (ttv_stack, _menhir_box_main_loc_constr) _menhir_cell1_locations -> _ -> _ -> _menhir_box_main_loc_constr =
    fun _menhir_stack _v _tok ->
      let MenhirCell1_locations (_menhir_stack, _, _1) = _menhir_stack in
      let _2 = _v in
      let _v = _menhir_action_105 _1 _2 in
      match (_tok : MenhirBasics.token) with
      | EOF ->
          let lc = _v in
          let _v = _menhir_action_150 lc in
          MenhirBox_main_loc_constr _v
      | _ ->
          _eRR ()
  
  let _menhir_run_380 : type  ttv_stack. ttv_stack -> _ -> _ -> _menhir_box_main_constr =
    fun _menhir_stack _v _tok ->
      match (_tok : MenhirBasics.token) with
      | EOF ->
          let c = _v in
          let _v = _menhir_action_148 c in
          MenhirBox_main_constr _v
      | _ ->
          _eRR ()
  
  let _menhir_goto_old_constraints : type  ttv_stack. ((ttv_stack, _menhir_box_constraints) _menhir_cell1_locations, _menhir_box_constraints) _menhir_cell1_filter -> _ -> _menhir_box_constraints =
    fun _menhir_stack _v ->
      let MenhirCell1_filter (_menhir_stack, _, _2) = _menhir_stack in
      let MenhirCell1_locations (_menhir_stack, _, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_075 _1 _2 _3 in
      MenhirBox_constraints _v
  
  let _menhir_run_308 : type  ttv_stack. (((ttv_stack, _menhir_box_constraints) _menhir_cell1_locations, _menhir_box_constraints) _menhir_cell1_filter, _menhir_box_constraints) _menhir_cell1_final -> _ -> _menhir_box_constraints =
    fun _menhir_stack _v ->
      let MenhirCell1_final (_menhir_stack, _, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_172 _1 _3 in
      _menhir_goto_old_constraints _menhir_stack _v
  
  let rec _menhir_goto_kinds : type  ttv_stack. ttv_stack -> _ -> (ttv_stack, _menhir_box_constraints) _menhir_state -> _menhir_box_constraints =
    fun _menhir_stack _v _menhir_s ->
      match _menhir_s with
      | MenhirState310 ->
          _menhir_run_311 _menhir_stack _v
      | MenhirState301 ->
          _menhir_run_308 _menhir_stack _v
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_311 : type  ttv_stack. (ttv_stack, _menhir_box_constraints) _menhir_cell1_kind -> _ -> _menhir_box_constraints =
    fun _menhir_stack _v ->
      let MenhirCell1_kind (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_100 _1 _3 in
      _menhir_goto_kinds _menhir_stack _v _menhir_s
  
  let rec _menhir_run_302 : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_constraints) _menhir_state -> _menhir_box_constraints =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | COLON ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | NOT ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | EXISTS ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  let _1 = _v in
                  let _v = _menhir_action_097 _1 in
                  _menhir_goto_kind _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
              | _ ->
                  _eRR ())
          | FORALL ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              let _1 = _v in
              let _v = _menhir_action_095 _1 in
              _menhir_goto_kind _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
          | EXISTS ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              let _1 = _v in
              let _v = _menhir_action_096 _1 in
              _menhir_goto_kind _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  and _menhir_goto_kind : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_constraints) _menhir_state -> _ -> _menhir_box_constraints =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | SEMI ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | NAME _v_0 ->
              let _menhir_stack = MenhirCell1_kind (_menhir_stack, _menhir_s, _v) in
              _menhir_run_302 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState310
          | EOF ->
              let _1 = _v in
              let _v = _menhir_action_099 _1 in
              _menhir_goto_kinds _menhir_stack _v _menhir_s
          | _ ->
              _eRR ())
      | EOF ->
          let _1 = _v in
          let _v = _menhir_action_098 _1 in
          _menhir_goto_kinds _menhir_stack _v _menhir_s
      | _ ->
          _eRR ()
  
  let _menhir_goto_final : type  ttv_stack. (((ttv_stack, _menhir_box_constraints) _menhir_cell1_locations, _menhir_box_constraints) _menhir_cell1_filter as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_constraints) _menhir_state -> _ -> _menhir_box_constraints =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | WITH ->
          let _menhir_stack = MenhirCell1_final (_menhir_stack, _menhir_s, _v) in
          let _menhir_s = MenhirState301 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | NAME _v ->
              _menhir_run_302 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | _ ->
              _eRR ())
      | EOF ->
          let _1 = _v in
          let _v = _menhir_action_171 _1 in
          _menhir_goto_old_constraints _menhir_stack _v
      | _ ->
          _eRR ()
  
  let _menhir_run_312 : type  ttv_stack. (((ttv_stack, _menhir_box_constraints) _menhir_cell1_locations, _menhir_box_constraints) _menhir_cell1_filter as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_constraints) _menhir_state -> _ -> _menhir_box_constraints =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | SEMI ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          let _1 = _v in
          let _v = _menhir_action_087 _1 in
          _menhir_goto_final _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | EOF | WITH ->
          let _1 = _v in
          let _v = _menhir_action_086 _1 in
          _menhir_goto_final _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  let _menhir_goto_constr : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState403 ->
          _menhir_run_404 _menhir_stack _v _tok
      | MenhirState388 ->
          _menhir_run_389 _menhir_stack _v _tok
      | MenhirState378 ->
          _menhir_run_380 _menhir_stack _v _tok
      | MenhirState277 ->
          _menhir_run_312 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _menhir_fail ()
  
  let _menhir_run_285 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_OBSERVED -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_OBSERVED (_menhir_stack, _menhir_s) = _menhir_stack in
      let _2 = _v in
      let _v = _menhir_action_074 _2 in
      _menhir_goto_constr _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  let rec _menhir_goto_obs : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState278 ->
          _menhir_run_285 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState280 ->
          _menhir_run_281 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_281 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_obsone -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_obsone (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_168 _1 _3 in
      _menhir_goto_obs _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  let _menhir_run_384 : type  ttv_stack. ttv_stack -> _ -> _ -> _menhir_box_main_filter =
    fun _menhir_stack _v _tok ->
      match (_tok : MenhirBasics.token) with
      | EOF ->
          let f = _v in
          let _v = _menhir_action_149 f in
          MenhirBox_main_filter _v
      | _ ->
          _eRR ()
  
  let _menhir_run_400 : type  ttv_stack. ttv_stack -> _ -> _menhir_box_pteval =
    fun _menhir_stack _v ->
      MenhirBox_pteval _v
  
  let _menhir_run_371 : type  ttv_stack. ttv_stack -> _ -> _ -> _menhir_box_init =
    fun _menhir_stack _v _tok ->
      match (_tok : MenhirBasics.token) with
      | EOF ->
          let st = _v in
          let _v = _menhir_action_088 st in
          MenhirBox_init _v
      | _ ->
          _eRR ()
  
  let rec _menhir_goto_init_semi_list : type  ttv_stack. ttv_stack -> _ -> (ttv_stack, _menhir_box_init) _menhir_state -> _ -> _menhir_box_init =
    fun _menhir_stack _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState375 ->
          _menhir_run_376 _menhir_stack _v _tok
      | MenhirState315 ->
          _menhir_run_371 _menhir_stack _v _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_376 : type  ttv_stack. (ttv_stack, _menhir_box_init) _menhir_cell1_atom_init -> _ -> _ -> _menhir_box_init =
    fun _menhir_stack _v _tok ->
      let MenhirCell1_atom_init (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_092 _1 _3 in
      _menhir_goto_init_semi_list _menhir_stack _v _menhir_s _tok
  
  let _menhir_run_326 : type  ttv_stack. ttv_stack -> _ -> _ -> (ttv_stack, _menhir_box_init) _menhir_state -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let _v = _menhir_action_090 () in
      _menhir_goto_init_semi_list _menhir_stack _v _menhir_s _tok
  
  let rec _menhir_run_003 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | LPAR ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | NAME _v ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | RPAR ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  let id = _v in
                  let _v = _menhir_action_140 id in
                  _menhir_goto_location_global _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
              | PLUS ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | NUM _v_0 ->
                      let _tok = _menhir_lexer _menhir_lexbuf in
                      (match (_tok : MenhirBasics.token) with
                      | RPAR ->
                          let _tok = _menhir_lexer _menhir_lexbuf in
                          let (id, o) = (_v, _v_0) in
                          let _v = _menhir_action_141 id o in
                          _menhir_goto_location_global _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
                      | _ ->
                          _eRR ())
                  | _ ->
                      _eRR ())
              | _ ->
                  _eRR ())
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  and _menhir_goto_location_global : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState333 ->
          _menhir_run_318 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState317 ->
          _menhir_run_318 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState254 ->
          _menhir_run_255 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState190 ->
          _menhir_run_191 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState169 ->
          _menhir_run_170 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState359 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState355 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState352 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState346 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState336 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState338 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState331 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState324 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState260 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState262 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState218 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState251 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState210 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState211 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState213 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState207 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState205 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState203 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState174 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState119 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState105 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState098 ->
          _menhir_run_103 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState392 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState375 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState315 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState350 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState327 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState328 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState316 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState296 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState294 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState292 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState289 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState287 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState278 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState283 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState280 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState094 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState096 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState097 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState198 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState273 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState200 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState089 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState002 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_318 : type  ttv_stack. (ttv_stack, _menhir_box_init) _menhir_cell1_LBRK -> _ -> _ -> _ -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      match (_tok : MenhirBasics.token) with
      | RBRK ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
          let loc = _v in
          let _v = _menhir_action_104 loc in
          _menhir_goto_left_loc _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_goto_left_loc : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_init) _menhir_state -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState375 ->
          _menhir_run_358 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState315 ->
          _menhir_run_358 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState350 ->
          _menhir_run_351 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState327 ->
          _menhir_run_335 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState328 ->
          _menhir_run_329 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState316 ->
          _menhir_run_321 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_358 : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_init) _menhir_state -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _menhir_stack = MenhirCell1_left_loc (_menhir_stack, _menhir_s, _v) in
      match (_tok : MenhirBasics.token) with
      | EQUAL ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | VALUE _v_0 ->
              _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState359
          | TOK_TAG ->
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState359
          | TOK_PTE ->
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState359
          | TOK_PA ->
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState359
          | TOK_NOP ->
              _menhir_run_261 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState359
          | PROC _v_1 ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | COLON ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | NAME _v_2 ->
                      let _tok = _menhir_lexer _menhir_lexbuf in
                      let (_1, _3) = (_v_1, _v_2) in
                      let _v = _menhir_action_155 _1 _3 in
                      _menhir_goto_maybev_label _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
                  | _ ->
                      _eRR ())
              | _ ->
                  _eRR ())
          | NUM _v_4 ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | COLON ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | NUM _v_5 ->
                      let _menhir_stack = MenhirCell1_NUM (_menhir_stack, MenhirState359, _v_4) in
                      _menhir_run_109 _menhir_stack _menhir_lexbuf _menhir_lexer _v_5
                  | NAME _v_6 ->
                      let _tok = _menhir_lexer _menhir_lexbuf in
                      (match (_tok : MenhirBasics.token) with
                      | COLON ->
                          let _menhir_stack = MenhirCell1_NUM (_menhir_stack, MenhirState359, _v_4) in
                          let _menhir_stack = MenhirCell1_NAME (_menhir_stack, MenhirState364, _v_6) in
                          _menhir_run_041 _menhir_stack _menhir_lexbuf _menhir_lexer
                      | EOF | SEMI ->
                          let (_1, _3) = (_v_4, _v_6) in
                          let _v = _menhir_action_156 _1 _3 in
                          _menhir_goto_maybev_label _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
                      | _ ->
                          _eRR ())
                  | _ ->
                      _eRR ())
              | EOF | SEMI ->
                  let _v_8 =
                    let _1 = _v_4 in
                    _menhir_action_160 _1
                  in
                  _menhir_run_116 _menhir_stack _menhir_lexbuf _menhir_lexer _v_8 MenhirState359 _tok
              | _ ->
                  _eRR ())
          | NAME _v_9 ->
              _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v_9 MenhirState359
          | LPAR ->
              _menhir_run_135 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState359
          | LABEL _v_10 ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              let l = _v_10 in
              let _v = _menhir_action_157 l in
              _menhir_goto_maybev_label _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
          | INSTR _v_12 ->
              _menhir_run_266 _menhir_stack _menhir_lexbuf _menhir_lexer _v_12 MenhirState359
          | COLON ->
              _menhir_run_114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState359
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  and _menhir_run_106 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let _1 = _v in
      let _v = _menhir_action_161 _1 in
      _menhir_goto_maybev_notag _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_goto_maybev_notag : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState346 ->
          _menhir_run_212 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState262 ->
          _menhir_run_212 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState251 ->
          _menhir_run_212 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState213 ->
          _menhir_run_212 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState211 ->
          _menhir_run_212 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState359 ->
          _menhir_run_116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState355 ->
          _menhir_run_116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState352 ->
          _menhir_run_116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState336 ->
          _menhir_run_116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState338 ->
          _menhir_run_116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState331 ->
          _menhir_run_116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState324 ->
          _menhir_run_116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState260 ->
          _menhir_run_116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState218 ->
          _menhir_run_116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState210 ->
          _menhir_run_116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState207 ->
          _menhir_run_116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState205 ->
          _menhir_run_116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState203 ->
          _menhir_run_116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState174 ->
          _menhir_run_116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState119 ->
          _menhir_run_116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState105 ->
          _menhir_run_116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_212 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | COMMA ->
          let _menhir_stack = MenhirCell1_maybev_notag (_menhir_stack, _menhir_s, _v) in
          let _menhir_s = MenhirState213 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | VALUE _v ->
              _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | TOK_TAG ->
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PTE ->
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PA ->
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | NUM _v ->
              _menhir_run_107 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | NAME _v ->
              _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | _ ->
              _eRR ())
      | RCURLY ->
          let _1 = _v in
          let _v = _menhir_action_159 _1 in
          _menhir_goto_maybev_list _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_010 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | LPAR ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | TOK_PTE ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | LPAR ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | NAME _v ->
                      let _tok = _menhir_lexer _menhir_lexbuf in
                      (match (_tok : MenhirBasics.token) with
                      | RPAR ->
                          let _tok = _menhir_lexer _menhir_lexbuf in
                          (match (_tok : MenhirBasics.token) with
                          | RPAR ->
                              let _tok = _menhir_lexer _menhir_lexbuf in
                              let _5 = _v in
                              let _v = _menhir_action_137 _5 in
                              _menhir_goto_location_global _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
                          | _ ->
                              _eRR ())
                      | _ ->
                          _eRR ())
                  | _ ->
                      _eRR ())
              | _ ->
                  _eRR ())
          | NAME _v ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | RPAR ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  let _3 = _v in
                  let _v = _menhir_action_136 _3 in
                  _menhir_goto_location_global _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
              | _ ->
                  _eRR ())
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  and _menhir_run_019 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | LPAR ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | NAME _v ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | RPAR ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  let _3 = _v in
                  let _v = _menhir_action_138 _3 in
                  _menhir_goto_location_global _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
              | _ ->
                  _eRR ())
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  and _menhir_run_107 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | COLON ->
          let _menhir_stack = MenhirCell1_NUM (_menhir_stack, _menhir_s, _v) in
          let _menhir_s = MenhirState108 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | NUM _v ->
              _menhir_run_109 _menhir_stack _menhir_lexbuf _menhir_lexer _v
          | NAME _v ->
              _menhir_run_101 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | _ ->
              _eRR ())
      | AND | COMMA | EOF | EXISTS | FINAL | FORALL | IMPLIES | LPAR | NOT | OBSERVED | OR | RCURLY | RPAR | SEMI | WITH ->
          let _1 = _v in
          let _v = _menhir_action_160 _1 in
          _menhir_goto_maybev_notag _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_109 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_NUM -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let MenhirCell1_NUM (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_163 _1 _3 in
      _menhir_goto_maybev_notag _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_101 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_NUM as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _menhir_stack = MenhirCell1_NAME (_menhir_stack, _menhir_s, _v) in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | COLON ->
          _menhir_run_041 _menhir_stack _menhir_lexbuf _menhir_lexer
      | _ ->
          _eRR ()
  
  and _menhir_run_041 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_NUM, ttv_result) _menhir_cell1_NAME -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | NUM _v ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          let MenhirCell1_NAME (_menhir_stack, _, _3) = _menhir_stack in
          let MenhirCell1_NUM (_menhir_stack, _menhir_s, _1) = _menhir_stack in
          let _5 = _v in
          let _v = _menhir_action_142 _1 _3 _5 in
          _menhir_goto_location_global _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_110 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | LBRK ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | NUM _v_0 ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | RBRK ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  let (_1, _3) = (_v, _v_0) in
                  let _v = _menhir_action_164 _1 _3 in
                  _menhir_goto_maybev_notag _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
              | _ ->
                  _eRR ())
          | _ ->
              _eRR ())
      | COLON ->
          let _menhir_stack = MenhirCell1_NAME (_menhir_stack, _menhir_s, _v) in
          _menhir_run_048 _menhir_stack _menhir_lexbuf _menhir_lexer
      | AND | COMMA | EOF | EXISTS | FINAL | FORALL | IMPLIES | LPAR | NOT | OBSERVED | OR | RCURLY | RPAR | SEMI | WITH ->
          let _1 = _v in
          let _v = _menhir_action_135 _1 in
          _menhir_goto_location_global _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_048 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_NAME -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | NUM _v ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          let MenhirCell1_NAME (_menhir_stack, _menhir_s, _1) = _menhir_stack in
          let _3 = _v in
          let _v = _menhir_action_143 _1 _3 in
          _menhir_goto_location_global _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | NAME _v ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          let MenhirCell1_NAME (_menhir_stack, _menhir_s, _1) = _menhir_stack in
          let _3 = _v in
          let _v = _menhir_action_139 _1 _3 in
          _menhir_goto_location_global _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_goto_maybev_list : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      match _menhir_s with
      | MenhirState346 ->
          _menhir_run_347 _menhir_stack _menhir_lexbuf _menhir_lexer _v
      | MenhirState262 ->
          _menhir_run_263 _menhir_stack _menhir_lexbuf _menhir_lexer _v
      | MenhirState251 ->
          _menhir_run_252 _menhir_stack _menhir_lexbuf _menhir_lexer _v
      | MenhirState211 ->
          _menhir_run_215 _menhir_stack _menhir_lexbuf _menhir_lexer _v
      | MenhirState213 ->
          _menhir_run_214 _menhir_stack _menhir_lexbuf _menhir_lexer _v
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_347 : type  ttv_stack. ((ttv_stack, _menhir_box_init) _menhir_cell1_NAME, _menhir_box_init) _menhir_cell1_arrayspec -> _ -> _ -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let MenhirCell1_arrayspec (_menhir_stack, _, _2) = _menhir_stack in
      let MenhirCell1_NAME (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _5 = _v in
      let _v = _menhir_action_022 _1 _2 _5 in
      _menhir_goto_atom_init _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_goto_atom_init : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_init) _menhir_state -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | SEMI ->
          let _menhir_stack = MenhirCell1_atom_init (_menhir_stack, _menhir_s, _v) in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | TOK_TAG ->
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState375
          | TOK_PTE ->
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState375
          | TOK_PA ->
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState375
          | SYMB_REG _v_0 ->
              _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState375
          | STAR ->
              _menhir_run_316 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState375
          | SEMI ->
              _menhir_run_326 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState375
          | PROC _v_1 ->
              _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState375
          | NUM _v_2 ->
              _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v_2 MenhirState375
          | NAME _v_3 ->
              _menhir_run_327 _menhir_stack _menhir_lexbuf _menhir_lexer _v_3 MenhirState375
          | LBRK ->
              _menhir_run_317 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState375
          | ATOMIC ->
              _menhir_run_349 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState375
          | EOF ->
              let _v_4 = _menhir_action_089 () in
              _menhir_run_376 _menhir_stack _v_4 _tok
          | _ ->
              _eRR ())
      | EOF ->
          let _1 = _v in
          let _v = _menhir_action_091 _1 in
          _menhir_goto_init_semi_list _menhir_stack _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_023 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let _1 = _v in
      let _v = _menhir_action_115 _1 in
      _menhir_goto_location _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_goto_location : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState392 ->
          _menhir_run_394 _menhir_stack _v _tok
      | MenhirState375 ->
          _menhir_run_357 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState315 ->
          _menhir_run_357 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState350 ->
          _menhir_run_320 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState327 ->
          _menhir_run_320 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState328 ->
          _menhir_run_320 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState316 ->
          _menhir_run_320 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState296 ->
          _menhir_run_209 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState294 ->
          _menhir_run_209 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState292 ->
          _menhir_run_209 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState289 ->
          _menhir_run_209 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState287 ->
          _menhir_run_209 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState278 ->
          _menhir_run_209 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState283 ->
          _menhir_run_209 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState280 ->
          _menhir_run_209 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState094 ->
          _menhir_run_209 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState096 ->
          _menhir_run_209 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState097 ->
          _menhir_run_209 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState198 ->
          _menhir_run_209 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState273 ->
          _menhir_run_209 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState200 ->
          _menhir_run_209 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState089 ->
          _menhir_run_085 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState002 ->
          _menhir_run_085 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_357 : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_init) _menhir_state -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | EOF | SEMI ->
          let _1 = _v in
          let _v = _menhir_action_011 _1 in
          _menhir_goto_atom _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | EQUAL ->
          let loc = _v in
          let _v = _menhir_action_103 loc in
          _menhir_goto_left_loc _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_goto_atom : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_init) _menhir_state -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _1 = _v in
      let _v = _menhir_action_013 _1 in
      _menhir_goto_atom_init _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_320 : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_init) _menhir_state -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let loc = _v in
      let _v = _menhir_action_103 loc in
      _menhir_goto_left_loc _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_209 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | NOTEQUAL ->
          let _menhir_stack = MenhirCell1_location (_menhir_stack, _menhir_s, _v) in
          let _menhir_s = MenhirState210 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | VALUE _v ->
              _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | TOK_TAG ->
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PTE ->
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PA ->
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | NUM _v ->
              _menhir_run_107 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | NAME _v ->
              _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | LCURLY ->
              let _menhir_stack = MenhirCell1_LCURLY (_menhir_stack, _menhir_s) in
              let _menhir_s = MenhirState211 in
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | VALUE _v ->
                  _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
              | TOK_TAG ->
                  _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
              | TOK_PTE ->
                  _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
              | TOK_PA ->
                  _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
              | NUM _v ->
                  _menhir_run_107 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
              | NAME _v ->
                  _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
              | _ ->
                  _eRR ())
          | COLON ->
              _menhir_run_114 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | _ ->
              _eRR ())
      | EQUALEQUAL ->
          let _menhir_stack = MenhirCell1_location (_menhir_stack, _menhir_s, _v) in
          let _menhir_s = MenhirState218 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | VALUE _v ->
              _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | TOK_TAG ->
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PTE ->
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PA ->
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | SYMB_REG _v ->
              _menhir_run_219 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | STAR ->
              _menhir_run_220 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | PROC _v ->
              _menhir_run_237 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | NUM _v ->
              _menhir_run_244 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | NAME _v ->
              _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | LPAR ->
              _menhir_run_135 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | LCURLY ->
              let _menhir_stack = MenhirCell1_LCURLY (_menhir_stack, _menhir_s) in
              let _menhir_s = MenhirState251 in
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | VALUE _v ->
                  _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
              | TOK_TAG ->
                  _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
              | TOK_PTE ->
                  _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
              | TOK_PA ->
                  _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
              | NUM _v ->
                  _menhir_run_107 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
              | NAME _v ->
                  _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
              | _ ->
                  _eRR ())
          | LBRK ->
              _menhir_run_254 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | COLON ->
              _menhir_run_114 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | _ ->
              _eRR ())
      | EQUAL ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | VALUE _v_14 ->
              let _menhir_stack = MenhirCell1_location (_menhir_stack, _menhir_s, _v) in
              _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v_14 MenhirState260
          | TOK_TAG ->
              let _menhir_stack = MenhirCell1_location (_menhir_stack, _menhir_s, _v) in
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState260
          | TOK_PTE ->
              let _menhir_stack = MenhirCell1_location (_menhir_stack, _menhir_s, _v) in
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState260
          | TOK_PA ->
              let _menhir_stack = MenhirCell1_location (_menhir_stack, _menhir_s, _v) in
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState260
          | TOK_NOP ->
              let _menhir_stack = MenhirCell1_location (_menhir_stack, _menhir_s, _v) in
              _menhir_run_261 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState260
          | SYMB_REG _v_15 ->
              let _menhir_stack = MenhirCell1_location (_menhir_stack, _menhir_s, _v) in
              _menhir_run_219 _menhir_stack _menhir_lexbuf _menhir_lexer _v_15 MenhirState260
          | STAR ->
              let _menhir_stack = MenhirCell1_location (_menhir_stack, _menhir_s, _v) in
              _menhir_run_220 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState260
          | PROC _v_16 ->
              let _menhir_stack = MenhirCell1_location (_menhir_stack, _menhir_s, _v) in
              _menhir_run_237 _menhir_stack _menhir_lexbuf _menhir_lexer _v_16 MenhirState260
          | NUM _v_17 ->
              let _menhir_stack = MenhirCell1_location (_menhir_stack, _menhir_s, _v) in
              _menhir_run_244 _menhir_stack _menhir_lexbuf _menhir_lexer _v_17 MenhirState260
          | NAME _v_18 ->
              let _menhir_stack = MenhirCell1_location (_menhir_stack, _menhir_s, _v) in
              _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v_18 MenhirState260
          | LPAR ->
              let _menhir_stack = MenhirCell1_location (_menhir_stack, _menhir_s, _v) in
              _menhir_run_135 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState260
          | LCURLY ->
              let _menhir_stack = MenhirCell1_location (_menhir_stack, _menhir_s, _v) in
              let _menhir_stack = MenhirCell1_LCURLY (_menhir_stack, MenhirState260) in
              let _menhir_s = MenhirState262 in
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | VALUE _v ->
                  _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
              | TOK_TAG ->
                  _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
              | TOK_PTE ->
                  _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
              | TOK_PA ->
                  _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
              | NUM _v ->
                  _menhir_run_107 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
              | NAME _v ->
                  _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
              | _ ->
                  _eRR ())
          | LBRK ->
              let _menhir_stack = MenhirCell1_location (_menhir_stack, _menhir_s, _v) in
              _menhir_run_254 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState260
          | LABEL _v_22 ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              let (loc, l) = (_v, _v_22) in
              let _v = _menhir_action_031 l loc in
              _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
          | INSTR _v_23 ->
              let _menhir_stack = MenhirCell1_location (_menhir_stack, _menhir_s, _v) in
              _menhir_run_266 _menhir_stack _menhir_lexbuf _menhir_lexer _v_23 MenhirState260
          | COLON ->
              let _menhir_stack = MenhirCell1_location (_menhir_stack, _menhir_s, _v) in
              _menhir_run_114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState260
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  and _menhir_run_114 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | NAME _v ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          let _2 = _v in
          let _v = _menhir_action_153 _2 in
          _menhir_goto_maybev _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_goto_maybev : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState359 ->
          _menhir_run_369 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState355 ->
          _menhir_run_356 _menhir_stack _menhir_lexbuf _menhir_lexer _tok
      | MenhirState352 ->
          _menhir_run_353 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState336 ->
          _menhir_run_342 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState338 ->
          _menhir_run_339 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState331 ->
          _menhir_run_332 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState324 ->
          _menhir_run_325 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState260 ->
          _menhir_run_268 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState218 ->
          _menhir_run_258 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState210 ->
          _menhir_run_217 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState207 ->
          _menhir_run_208 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState205 ->
          _menhir_run_206 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState203 ->
          _menhir_run_204 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState174 ->
          _menhir_run_194 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState119 ->
          _menhir_run_173 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState105 ->
          _menhir_run_117 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_369 : type  ttv_stack. (ttv_stack, _menhir_box_init) _menhir_cell1_left_loc -> _ -> _ -> _ -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let _1 = _v in
      let _v = _menhir_action_154 _1 in
      _menhir_goto_maybev_label _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
  
  and _menhir_goto_maybev_label : type  ttv_stack. (ttv_stack, _menhir_box_init) _menhir_cell1_left_loc -> _ -> _ -> _ -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_left_loc (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_012 _1 _3 in
      _menhir_goto_atom _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_356 : type  ttv_stack. (ttv_stack, _menhir_box_init) _menhir_cell1_locindex -> _ -> _ -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _tok ->
      let MenhirCell1_locindex (_menhir_stack, _menhir_s, _) = _menhir_stack in
      let _v = _menhir_action_023 () in
      _menhir_goto_atom_init _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_353 : type  ttv_stack. ((ttv_stack, _menhir_box_init) _menhir_cell1_ATOMIC _menhir_cell0_NAME, _menhir_box_init) _menhir_cell1_left_loc -> _ -> _ -> _ -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_left_loc (_menhir_stack, _, loc) = _menhir_stack in
      let MenhirCell0_NAME (_menhir_stack, typ) = _menhir_stack in
      let MenhirCell1_ATOMIC (_menhir_stack, _menhir_s) = _menhir_stack in
      let v = _v in
      let _v = _menhir_action_018 loc typ v in
      _menhir_goto_atom_init _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_342 : type  ttv_stack. ((ttv_stack, _menhir_box_init) _menhir_cell1_NAME, _menhir_box_init) _menhir_cell1_left_loc -> _ -> _ -> _ -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_left_loc (_menhir_stack, _, loc) = _menhir_stack in
      let MenhirCell1_NAME (_menhir_stack, _menhir_s, typ) = _menhir_stack in
      let v = _v in
      let _v = _menhir_action_019 loc typ v in
      _menhir_goto_atom_init _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_339 : type  ttv_stack. (((ttv_stack, _menhir_box_init) _menhir_cell1_NAME, _menhir_box_init) _menhir_cell1_left_loc, _menhir_box_init) _menhir_cell1_ATOMICINIT -> _ -> _ -> _ -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      match (_tok : MenhirBasics.token) with
      | RPAR ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          let MenhirCell1_ATOMICINIT (_menhir_stack, _) = _menhir_stack in
          let MenhirCell1_left_loc (_menhir_stack, _, loc) = _menhir_stack in
          let MenhirCell1_NAME (_menhir_stack, _menhir_s, typ) = _menhir_stack in
          let v = _v in
          let _v = _menhir_action_020 loc typ v in
          _menhir_goto_atom_init _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_332 : type  ttv_stack. ((((ttv_stack, _menhir_box_init) _menhir_cell1_NAME, _menhir_box_init) _menhir_cell1_STAR, _menhir_box_init) _menhir_cell1_left_loc, _menhir_box_init) _menhir_cell1_amperopt -> _ -> _ -> _ -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_amperopt (_menhir_stack, _, _) = _menhir_stack in
      let MenhirCell1_left_loc (_menhir_stack, _, loc) = _menhir_stack in
      let MenhirCell1_STAR (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_NAME (_menhir_stack, _menhir_s, typ) = _menhir_stack in
      let v = _v in
      let _v = _menhir_action_025 loc typ v in
      _menhir_goto_atom_init _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_325 : type  ttv_stack. (((ttv_stack, _menhir_box_init) _menhir_cell1_STAR, _menhir_box_init) _menhir_cell1_left_loc, _menhir_box_init) _menhir_cell1_amperopt -> _ -> _ -> _ -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_amperopt (_menhir_stack, _, _) = _menhir_stack in
      let MenhirCell1_left_loc (_menhir_stack, _, loc) = _menhir_stack in
      let MenhirCell1_STAR (_menhir_stack, _menhir_s) = _menhir_stack in
      let v = _v in
      let _v = _menhir_action_027 loc v in
      _menhir_goto_atom_init _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_268 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_location -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_location (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_032 _1 _3 in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_goto_atom_prop : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState278 ->
          _menhir_run_282 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState283 ->
          _menhir_run_282 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState280 ->
          _menhir_run_282 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState296 ->
          _menhir_run_272 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState294 ->
          _menhir_run_272 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState292 ->
          _menhir_run_272 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState289 ->
          _menhir_run_272 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState287 ->
          _menhir_run_272 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState094 ->
          _menhir_run_272 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState096 ->
          _menhir_run_272 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState097 ->
          _menhir_run_272 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState198 ->
          _menhir_run_272 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState273 ->
          _menhir_run_272 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState200 ->
          _menhir_run_272 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_282 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _menhir_stack = MenhirCell1_atom_prop (_menhir_stack, _menhir_s, _v) in
      match (_tok : MenhirBasics.token) with
      | SEMI ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | TOK_TAG ->
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState283
          | TOK_PTE ->
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState283
          | TOK_PA ->
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState283
          | SYMB_REG _v_0 ->
              _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState283
          | PROC _v_1 ->
              _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState283
          | NUM _v_2 ->
              _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v_2 MenhirState283
          | NAME _v_3 ->
              _menhir_run_044 _menhir_stack _menhir_lexbuf _menhir_lexer _v_3 MenhirState283
          | LBRK ->
              _menhir_run_098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState283
          | FAULT ->
              _menhir_run_051 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState283
          | EOF | SEMI | TOKAND | WITH ->
              let _v_4 = _menhir_action_169 () in
              _menhir_run_284 _menhir_stack _menhir_lexbuf _menhir_lexer _v_4 _tok
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  and _menhir_run_025 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | COLON ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | SYMB_REG _v_0 ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              let (_1, _3) = (_v, _v_0) in
              let _v = _menhir_action_113 _1 _3 in
              _menhir_goto_location _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
          | PTX_REG_DEC ->
              let _menhir_stack = MenhirCell1_PROC (_menhir_stack, _menhir_s, _v) in
              let _menhir_stack = MenhirCell1_PTX_REG_DEC (_menhir_stack, MenhirState026) in
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | PTX_REG_TYPE _v ->
                  let _menhir_stack = MenhirCell0_PTX_REG_TYPE (_menhir_stack, _v) in
                  let _menhir_s = MenhirState029 in
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | NAME _v ->
                      _menhir_run_030 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                  | DOLLARNAME _v ->
                      _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                  | _ ->
                      _eRR ())
              | _ ->
                  _eRR ())
          | NAME _v_4 ->
              let _menhir_stack = MenhirCell1_PROC (_menhir_stack, _menhir_s, _v) in
              _menhir_run_030 _menhir_stack _menhir_lexbuf _menhir_lexer _v_4 MenhirState026
          | DOLLARNAME _v_5 ->
              let _menhir_stack = MenhirCell1_PROC (_menhir_stack, _menhir_s, _v) in
              _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v_5 MenhirState026
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  and _menhir_run_030 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let _1 = _v in
      let _v = _menhir_action_191 _1 in
      _menhir_goto_reg _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_goto_reg : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState245 ->
          _menhir_run_250 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState248 ->
          _menhir_run_249 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState238 ->
          _menhir_run_243 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState241 ->
          _menhir_run_242 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState230 ->
          _menhir_run_235 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState233 ->
          _menhir_run_234 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState223 ->
          _menhir_run_228 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState226 ->
          _menhir_run_227 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState184 ->
          _menhir_run_189 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState187 ->
          _menhir_run_188 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState177 ->
          _menhir_run_182 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState180 ->
          _menhir_run_181 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState129 ->
          _menhir_run_134 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState132 ->
          _menhir_run_133 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState122 ->
          _menhir_run_127 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState125 ->
          _menhir_run_126 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState035 ->
          _menhir_run_043 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState038 ->
          _menhir_run_039 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState026 ->
          _menhir_run_033 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState029 ->
          _menhir_run_032 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_250 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_NUM -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_NUM (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_120 _1 _3 in
      _menhir_goto_location_deref _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_goto_location_deref : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState260 ->
          _menhir_run_269 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState218 ->
          _menhir_run_259 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_269 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_location -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_location (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_048 _1 _3 in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_259 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_location -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_location (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_049 _1 _3 in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_249 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_NUM, ttv_result) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell0_PTX_REG_TYPE (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_PTX_REG_DEC (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_NUM (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _5 = _v in
      let _v = _menhir_action_124 _1 _5 in
      _menhir_goto_location_deref _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_243 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_PROC -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_PROC (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_119 _1 _3 in
      _menhir_goto_location_deref _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_242 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_PROC, ttv_result) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell0_PTX_REG_TYPE (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_PTX_REG_DEC (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_PROC (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _5 = _v in
      let _v = _menhir_action_125 _1 _5 in
      _menhir_goto_location_deref _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_235 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_STAR _menhir_cell0_NUM -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell0_NUM (_menhir_stack, _1_inlined1) = _menhir_stack in
      let MenhirCell1_STAR (_menhir_stack, _menhir_s) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_127 _1_inlined1 _3 in
      _menhir_goto_location_deref _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_234 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_STAR _menhir_cell0_NUM, ttv_result) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell0_PTX_REG_TYPE (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_PTX_REG_DEC (_menhir_stack, _) = _menhir_stack in
      let MenhirCell0_NUM (_menhir_stack, _1_inlined1) = _menhir_stack in
      let MenhirCell1_STAR (_menhir_stack, _menhir_s) = _menhir_stack in
      let _5 = _v in
      let _v = _menhir_action_131 _1_inlined1 _5 in
      _menhir_goto_location_deref _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_228 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_STAR _menhir_cell0_PROC -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell0_PROC (_menhir_stack, _1_inlined1) = _menhir_stack in
      let MenhirCell1_STAR (_menhir_stack, _menhir_s) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_126 _1_inlined1 _3 in
      _menhir_goto_location_deref _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_227 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_STAR _menhir_cell0_PROC, ttv_result) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell0_PTX_REG_TYPE (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_PTX_REG_DEC (_menhir_stack, _) = _menhir_stack in
      let MenhirCell0_PROC (_menhir_stack, _1_inlined1) = _menhir_stack in
      let MenhirCell1_STAR (_menhir_stack, _menhir_s) = _menhir_stack in
      let _5 = _v in
      let _v = _menhir_action_132 _1_inlined1 _5 in
      _menhir_goto_location_deref _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_189 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_LBRK, ttv_result) _menhir_cell1_location_global, ttv_result) _menhir_cell1_NUM -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_NUM (_menhir_stack, _, _1_inlined2) = _menhir_stack in
      let MenhirCell1_location_global (_menhir_stack, _, loc) = _menhir_stack in
      let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
      let _3_inlined1 = _v in
      let _v = _menhir_action_053 _1_inlined2 _3_inlined1 loc in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_188 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_LBRK, ttv_result) _menhir_cell1_location_global, ttv_result) _menhir_cell1_NUM, ttv_result) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell0_PTX_REG_TYPE (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_PTX_REG_DEC (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_NUM (_menhir_stack, _, _1_inlined2) = _menhir_stack in
      let MenhirCell1_location_global (_menhir_stack, _, loc) = _menhir_stack in
      let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
      let _5 = _v in
      let _v = _menhir_action_057 _1_inlined2 _5 loc in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_182 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_LBRK, ttv_result) _menhir_cell1_location_global, ttv_result) _menhir_cell1_PROC -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_PROC (_menhir_stack, _, _1_inlined2) = _menhir_stack in
      let MenhirCell1_location_global (_menhir_stack, _, loc) = _menhir_stack in
      let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
      let _3_inlined1 = _v in
      let _v = _menhir_action_052 _1_inlined2 _3_inlined1 loc in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_181 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_LBRK, ttv_result) _menhir_cell1_location_global, ttv_result) _menhir_cell1_PROC, ttv_result) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell0_PTX_REG_TYPE (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_PTX_REG_DEC (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_PROC (_menhir_stack, _, _1_inlined2) = _menhir_stack in
      let MenhirCell1_location_global (_menhir_stack, _, loc) = _menhir_stack in
      let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
      let _5 = _v in
      let _v = _menhir_action_058 _1_inlined2 _5 loc in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_134 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_LBRK, ttv_result) _menhir_cell1_location_global, ttv_result) _menhir_cell1_NUM -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_NUM (_menhir_stack, _, _1_inlined2) = _menhir_stack in
      let MenhirCell1_location_global (_menhir_stack, _, loc) = _menhir_stack in
      let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
      let _3_inlined1 = _v in
      let _v = _menhir_action_060 _1_inlined2 _3_inlined1 loc in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_133 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_LBRK, ttv_result) _menhir_cell1_location_global, ttv_result) _menhir_cell1_NUM, ttv_result) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell0_PTX_REG_TYPE (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_PTX_REG_DEC (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_NUM (_menhir_stack, _, _1_inlined2) = _menhir_stack in
      let MenhirCell1_location_global (_menhir_stack, _, loc) = _menhir_stack in
      let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
      let _5 = _v in
      let _v = _menhir_action_064 _1_inlined2 _5 loc in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_127 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_LBRK, ttv_result) _menhir_cell1_location_global, ttv_result) _menhir_cell1_PROC -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_PROC (_menhir_stack, _, _1_inlined2) = _menhir_stack in
      let MenhirCell1_location_global (_menhir_stack, _, loc) = _menhir_stack in
      let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
      let _3_inlined1 = _v in
      let _v = _menhir_action_059 _1_inlined2 _3_inlined1 loc in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_126 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_LBRK, ttv_result) _menhir_cell1_location_global, ttv_result) _menhir_cell1_PROC, ttv_result) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell0_PTX_REG_TYPE (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_PTX_REG_DEC (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_PROC (_menhir_stack, _, _1_inlined2) = _menhir_stack in
      let MenhirCell1_location_global (_menhir_stack, _, loc) = _menhir_stack in
      let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
      let _5 = _v in
      let _v = _menhir_action_065 _1_inlined2 _5 loc in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_043 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_NUM -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_NUM (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_112 _1 _3 in
      _menhir_goto_location _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_039 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_NUM, ttv_result) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell0_PTX_REG_TYPE (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_PTX_REG_DEC (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_NUM (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _5 = _v in
      let _v = _menhir_action_116 _1 _5 in
      _menhir_goto_location _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_033 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_PROC -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_PROC (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_111 _1 _3 in
      _menhir_goto_location _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_032 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_PROC, ttv_result) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell0_PTX_REG_TYPE (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_PTX_REG_DEC (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_PROC (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _5 = _v in
      let _v = _menhir_action_117 _1 _5 in
      _menhir_goto_location _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_031 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let _1 = _v in
      let _v = _menhir_action_192 _1 in
      _menhir_goto_reg _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_034 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | COLON ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | SYMB_REG _v_0 ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              let (_1, _3) = (_v, _v_0) in
              let _v = _menhir_action_114 _1 _3 in
              _menhir_goto_location _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
          | PTX_REG_DEC ->
              let _menhir_stack = MenhirCell1_NUM (_menhir_stack, _menhir_s, _v) in
              let _menhir_stack = MenhirCell1_PTX_REG_DEC (_menhir_stack, MenhirState035) in
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | PTX_REG_TYPE _v ->
                  let _menhir_stack = MenhirCell0_PTX_REG_TYPE (_menhir_stack, _v) in
                  let _menhir_s = MenhirState038 in
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | NAME _v ->
                      _menhir_run_030 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                  | DOLLARNAME _v ->
                      _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                  | _ ->
                      _eRR ())
              | _ ->
                  _eRR ())
          | NAME _v_4 ->
              let _menhir_stack = MenhirCell1_NUM (_menhir_stack, _menhir_s, _v) in
              _menhir_run_040 _menhir_stack _menhir_lexbuf _menhir_lexer _v_4 MenhirState035
          | DOLLARNAME _v_5 ->
              let _menhir_stack = MenhirCell1_NUM (_menhir_stack, _menhir_s, _v) in
              _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v_5 MenhirState035
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  and _menhir_run_040 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_NUM as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | COLON ->
          let _menhir_stack = MenhirCell1_NAME (_menhir_stack, _menhir_s, _v) in
          _menhir_run_041 _menhir_stack _menhir_lexbuf _menhir_lexer
      | AND | EOF | EQUAL | EQUALEQUAL | EXISTS | FINAL | FORALL | IMPLIES | LPAR | NAME _ | NOT | NOTEQUAL | OBSERVED | OR | RBRK | RPAR | SEMI | STAR | WITH ->
          let _1 = _v in
          let _v = _menhir_action_191 _1 in
          _menhir_goto_reg _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_044 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | LBRK ->
          let _menhir_stack = MenhirCell1_NAME (_menhir_stack, _menhir_s, _v) in
          let _menhir_stack = MenhirCell1_LBRK (_menhir_stack, MenhirState044) in
          let _menhir_s = MenhirState045 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | NUM _v ->
              let _menhir_stack = MenhirCell1_NUM (_menhir_stack, _menhir_s, _v) in
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | RBRK ->
                  _menhir_run_047 _menhir_stack _menhir_lexbuf _menhir_lexer
              | _ ->
                  _eRR ())
          | _ ->
              _eRR ())
      | COLON ->
          let _menhir_stack = MenhirCell1_NAME (_menhir_stack, _menhir_s, _v) in
          _menhir_run_048 _menhir_stack _menhir_lexbuf _menhir_lexer
      | EOF | EQUAL | EQUALEQUAL | NAME _ | NOTEQUAL | RBRK | SEMI | STAR ->
          let _1 = _v in
          let _v = _menhir_action_135 _1 in
          _menhir_goto_location_global _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_047 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_NAME, ttv_result) _menhir_cell1_LBRK, ttv_result) _menhir_cell1_NUM -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let MenhirCell1_NUM (_menhir_stack, _, _3) = _menhir_stack in
      let MenhirCell1_LBRK (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_NAME (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _v = _menhir_action_010 _1 _3 in
      _menhir_goto_arrayspec _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_goto_arrayspec : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState327 ->
          _menhir_run_344 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState315 ->
          _menhir_run_092 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState375 ->
          _menhir_run_092 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState296 ->
          _menhir_run_092 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState294 ->
          _menhir_run_092 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState292 ->
          _menhir_run_092 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState289 ->
          _menhir_run_092 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState287 ->
          _menhir_run_092 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState278 ->
          _menhir_run_092 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState280 ->
          _menhir_run_092 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState283 ->
          _menhir_run_092 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState094 ->
          _menhir_run_092 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState096 ->
          _menhir_run_092 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState097 ->
          _menhir_run_092 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState198 ->
          _menhir_run_092 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState273 ->
          _menhir_run_092 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState200 ->
          _menhir_run_092 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState002 ->
          _menhir_run_092 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState089 ->
          _menhir_run_092 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_344 : type  ttv_stack. ((ttv_stack, _menhir_box_init) _menhir_cell1_NAME as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_init) _menhir_state -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | EQUAL ->
          let _menhir_stack = MenhirCell1_arrayspec (_menhir_stack, _menhir_s, _v) in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | LCURLY ->
              let _menhir_s = MenhirState346 in
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | VALUE _v ->
                  _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
              | TOK_TAG ->
                  _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
              | TOK_PTE ->
                  _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
              | TOK_PA ->
                  _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
              | NUM _v ->
                  _menhir_run_107 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
              | NAME _v ->
                  _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
              | _ ->
                  _eRR ())
          | _ ->
              _eRR ())
      | EOF | SEMI ->
          let MenhirCell1_NAME (_menhir_stack, _menhir_s, _1) = _menhir_stack in
          let _2 = _v in
          let _v = _menhir_action_021 _1 _2 in
          _menhir_goto_atom_init _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_092 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _1 = _v in
      let _v = _menhir_action_147 _1 in
      _menhir_goto_locindex _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_goto_locindex : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState375 ->
          _menhir_run_354 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState315 ->
          _menhir_run_354 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState296 ->
          _menhir_run_202 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState294 ->
          _menhir_run_202 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState292 ->
          _menhir_run_202 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState289 ->
          _menhir_run_202 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState287 ->
          _menhir_run_202 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState278 ->
          _menhir_run_202 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState283 ->
          _menhir_run_202 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState280 ->
          _menhir_run_202 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState094 ->
          _menhir_run_202 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState096 ->
          _menhir_run_202 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState097 ->
          _menhir_run_202 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState198 ->
          _menhir_run_202 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState273 ->
          _menhir_run_202 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState200 ->
          _menhir_run_202 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState089 ->
          _menhir_run_083 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState002 ->
          _menhir_run_083 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_354 : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_init) _menhir_state -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _menhir_stack = MenhirCell1_locindex (_menhir_stack, _menhir_s, _v) in
      match (_tok : MenhirBasics.token) with
      | EQUAL ->
          let _menhir_s = MenhirState355 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | VALUE _v ->
              _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | TOK_TAG ->
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PTE ->
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PA ->
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | NUM _v ->
              _menhir_run_107 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | NAME _v ->
              _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | COLON ->
              _menhir_run_114 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  and _menhir_run_202 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _menhir_stack = MenhirCell1_locindex (_menhir_stack, _menhir_s, _v) in
      match (_tok : MenhirBasics.token) with
      | NOTEQUAL ->
          let _menhir_s = MenhirState203 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | VALUE _v ->
              _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | TOK_TAG ->
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PTE ->
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PA ->
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | NUM _v ->
              _menhir_run_107 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | NAME _v ->
              _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | COLON ->
              _menhir_run_114 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | _ ->
              _eRR ())
      | EQUALEQUAL ->
          let _menhir_s = MenhirState205 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | VALUE _v ->
              _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | TOK_TAG ->
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PTE ->
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PA ->
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | NUM _v ->
              _menhir_run_107 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | NAME _v ->
              _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | COLON ->
              _menhir_run_114 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | _ ->
              _eRR ())
      | EQUAL ->
          let _menhir_s = MenhirState207 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | VALUE _v ->
              _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | TOK_TAG ->
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PTE ->
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PA ->
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | NUM _v ->
              _menhir_run_107 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | NAME _v ->
              _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | COLON ->
              _menhir_run_114 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  and _menhir_run_083 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _1 = _v in
      let _v = _menhir_action_194 _1 in
      _menhir_goto_rloc _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_goto_rloc : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | STAR ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          let _1 = _v in
          let _v = _menhir_action_196 _1 in
          _menhir_goto_rloc_typ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | NAME _v_0 ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | STAR ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              let (_1, _2) = (_v, _v_0) in
              let _v = _menhir_action_198 _1 _2 in
              _menhir_goto_rloc_typ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
          | RBRK | SEMI ->
              let (_1, _2) = (_v, _v_0) in
              let _v = _menhir_action_197 _1 _2 in
              _menhir_goto_rloc_typ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
          | _ ->
              _eRR ())
      | RBRK | SEMI ->
          let _1 = _v in
          let _v = _menhir_action_195 _1 in
          _menhir_goto_rloc_typ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_goto_rloc_typ : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _1 = _v in
      let _v = _menhir_action_106 _1 in
      _menhir_goto_loc_item _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_goto_loc_item : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | SEMI ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | TOK_TAG ->
              let _menhir_stack = MenhirCell1_loc_item (_menhir_stack, _menhir_s, _v) in
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState089
          | TOK_PTE ->
              let _menhir_stack = MenhirCell1_loc_item (_menhir_stack, _menhir_s, _v) in
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState089
          | TOK_PA ->
              let _menhir_stack = MenhirCell1_loc_item (_menhir_stack, _menhir_s, _v) in
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState089
          | SYMB_REG _v_0 ->
              let _menhir_stack = MenhirCell1_loc_item (_menhir_stack, _menhir_s, _v) in
              _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState089
          | PROC _v_1 ->
              let _menhir_stack = MenhirCell1_loc_item (_menhir_stack, _menhir_s, _v) in
              _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState089
          | NUM _v_2 ->
              let _menhir_stack = MenhirCell1_loc_item (_menhir_stack, _menhir_s, _v) in
              _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v_2 MenhirState089
          | NAME _v_3 ->
              let _menhir_stack = MenhirCell1_loc_item (_menhir_stack, _menhir_s, _v) in
              _menhir_run_044 _menhir_stack _menhir_lexbuf _menhir_lexer _v_3 MenhirState089
          | FAULT ->
              let _menhir_stack = MenhirCell1_loc_item (_menhir_stack, _menhir_s, _v) in
              _menhir_run_051 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState089
          | RBRK ->
              let _1 = _v in
              let _v = _menhir_action_109 _1 in
              _menhir_goto_loc_items _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | _ ->
              _eRR ())
      | RBRK ->
          let _1 = _v in
          let _v = _menhir_action_108 _1 in
          _menhir_goto_loc_items _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_051 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _menhir_stack = MenhirCell1_FAULT (_menhir_stack, _menhir_s) in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | LPAR ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | PROC _v ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | COLON ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | NAME _v_0 ->
                      let _tok = _menhir_lexer _menhir_lexbuf in
                      let (_1, _3) = (_v, _v_0) in
                      let _v = _menhir_action_102 _1 _3 in
                      _menhir_goto_lbl _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
                  | _ ->
                      _eRR ())
              | COMMA | RPAR ->
                  let _1 = _v in
                  let _v = _menhir_action_101 _1 in
                  _menhir_goto_lbl _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
              | _ ->
                  _eRR ())
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  and _menhir_goto_lbl : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_FAULT -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      match (_tok : MenhirBasics.token) with
      | RPAR ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          let MenhirCell1_FAULT (_menhir_stack, _menhir_s) = _menhir_stack in
          let lab = _v in
          let _v = _menhir_action_076 lab in
          _menhir_goto_fault _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | COMMA ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | TOK_PTE ->
              let _menhir_stack = MenhirCell0_lbl (_menhir_stack, _v) in
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | LPAR ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | NAME _v_0 ->
                      let _tok = _menhir_lexer _menhir_lexbuf in
                      (match (_tok : MenhirBasics.token) with
                      | RPAR ->
                          let _tok = _menhir_lexer _menhir_lexbuf in
                          let name = _v_0 in
                          let _v = _menhir_action_083 name in
                          (match (_tok : MenhirBasics.token) with
                          | RPAR ->
                              let _tok = _menhir_lexer _menhir_lexbuf in
                              let MenhirCell0_lbl (_menhir_stack, lab) = _menhir_stack in
                              let MenhirCell1_FAULT (_menhir_stack, _menhir_s) = _menhir_stack in
                              let fl = _v in
                              let _v = _menhir_action_078 fl lab in
                              _menhir_goto_fault _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
                          | COMMA ->
                              let fl = _v in
                              let _v = _menhir_action_082 fl in
                              _menhir_goto_fault_loc _menhir_stack _menhir_lexbuf _menhir_lexer _v
                          | _ ->
                              _eRR ())
                      | _ ->
                          _eRR ())
                  | _ ->
                      _eRR ())
              | _ ->
                  _eRR ())
          | NAME _v_2 ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | RPAR ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  let MenhirCell1_FAULT (_menhir_stack, _menhir_s) = _menhir_stack in
                  let (name, lab) = (_v_2, _v) in
                  let _v = _menhir_action_077 lab name in
                  _menhir_goto_fault _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
              | COLON ->
                  let _menhir_stack = MenhirCell0_lbl (_menhir_stack, _v) in
                  let _menhir_stack = MenhirCell0_NAME (_menhir_stack, _v_2) in
                  let _menhir_s = MenhirState065 in
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | NAME _v ->
                      _menhir_run_066 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                  | _ ->
                      _eRR ())
              | COMMA ->
                  let _menhir_stack = MenhirCell0_lbl (_menhir_stack, _v) in
                  let name = _v_2 in
                  let _v = _menhir_action_081 name in
                  _menhir_goto_fault_loc _menhir_stack _menhir_lexbuf _menhir_lexer _v
              | _ ->
                  _eRR ())
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  and _menhir_goto_fault : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState296 ->
          _menhir_run_271 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState294 ->
          _menhir_run_271 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState292 ->
          _menhir_run_271 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState289 ->
          _menhir_run_271 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState287 ->
          _menhir_run_271 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState278 ->
          _menhir_run_271 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState283 ->
          _menhir_run_271 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState280 ->
          _menhir_run_271 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState094 ->
          _menhir_run_271 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState096 ->
          _menhir_run_271 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState097 ->
          _menhir_run_271 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState198 ->
          _menhir_run_271 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState273 ->
          _menhir_run_271 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState200 ->
          _menhir_run_271 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState002 ->
          _menhir_run_091 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState089 ->
          _menhir_run_091 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_271 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _1 = _v in
      let _v = _menhir_action_066 _1 in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_091 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _1 = _v in
      let _v = _menhir_action_107 _1 in
      _menhir_goto_loc_item _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_goto_fault_loc : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_FAULT _menhir_cell0_lbl -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
      let _menhir_stack = MenhirCell0_fault_loc (_menhir_stack, _v) in
      let _menhir_s = MenhirState073 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | NAME _v ->
          _menhir_run_066 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_066 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | COLON ->
          let _menhir_stack = MenhirCell1_NAME (_menhir_stack, _menhir_s, _v) in
          let _menhir_s = MenhirState067 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | NAME _v ->
              _menhir_run_066 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | _ ->
              _eRR ())
      | RPAR ->
          let x = _v in
          let _v = _menhir_action_199 x in
          _menhir_goto_separated_nonempty_list_COLON_NAME_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_goto_separated_nonempty_list_COLON_NAME_ : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      match _menhir_s with
      | MenhirState073 ->
          _menhir_run_074 _menhir_stack _menhir_lexbuf _menhir_lexer _v
      | MenhirState065 ->
          _menhir_run_069 _menhir_stack _menhir_lexbuf _menhir_lexer _v
      | MenhirState067 ->
          _menhir_run_068 _menhir_stack _menhir_lexbuf _menhir_lexer _v
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_074 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_FAULT _menhir_cell0_lbl _menhir_cell0_fault_loc -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let MenhirCell0_fault_loc (_menhir_stack, fl) = _menhir_stack in
      let MenhirCell0_lbl (_menhir_stack, lab) = _menhir_stack in
      let MenhirCell1_FAULT (_menhir_stack, _menhir_s) = _menhir_stack in
      let ft = _v in
      let _v = _menhir_action_080 fl ft lab in
      _menhir_goto_fault _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_069 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_FAULT _menhir_cell0_lbl _menhir_cell0_NAME -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
      let MenhirCell0_NAME (_menhir_stack, fst) = _menhir_stack in
      let rem = _v in
      let _v = _menhir_action_067 fst rem in
      let _tok = _menhir_lexer _menhir_lexbuf in
      let MenhirCell0_lbl (_menhir_stack, lab) = _menhir_stack in
      let MenhirCell1_FAULT (_menhir_stack, _menhir_s) = _menhir_stack in
      let ft = _v in
      let _v = _menhir_action_079 ft lab in
      _menhir_goto_fault _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_068 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_NAME -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
      let MenhirCell1_NAME (_menhir_stack, _menhir_s, x) = _menhir_stack in
      let xs = _v in
      let _v = _menhir_action_200 x xs in
      _menhir_goto_separated_nonempty_list_COLON_NAME_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
  
  and _menhir_goto_loc_items : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      match _menhir_s with
      | MenhirState089 ->
          _menhir_run_090 _menhir_stack _menhir_lexbuf _menhir_lexer _v
      | MenhirState002 ->
          _menhir_run_086 _menhir_stack _menhir_lexbuf _menhir_lexer _v
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_090 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_loc_item -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
      let MenhirCell1_loc_item (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_110 _1 _3 in
      _menhir_goto_loc_items _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
  
  and _menhir_run_086 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_LOCATIONS -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let MenhirCell1_LOCATIONS (_menhir_stack, _menhir_s) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_144 _3 in
      _menhir_goto_locations _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_goto_locations : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState401 ->
          _menhir_run_403 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState386 ->
          _menhir_run_388 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState000 ->
          _menhir_run_093 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_403 : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_skip_loc_constr) _menhir_state -> _ -> _menhir_box_skip_loc_constr =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _menhir_stack = MenhirCell1_locations (_menhir_stack, _menhir_s, _v) in
      match (_tok : MenhirBasics.token) with
      | OBSERVED ->
          _menhir_run_278 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState403
      | NOT ->
          _menhir_run_286 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState403
      | LPAR ->
          _menhir_run_289 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState403
      | FORALL ->
          _menhir_run_292 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState403
      | FINAL ->
          _menhir_run_294 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState403
      | EXISTS ->
          _menhir_run_296 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState403
      | EOF ->
          let _v_0 = _menhir_action_068 () in
          _menhir_run_404 _menhir_stack _v_0 _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_278 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _menhir_stack = MenhirCell1_OBSERVED (_menhir_stack, _menhir_s) in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TOK_TAG ->
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState278
      | TOK_PTE ->
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState278
      | TOK_PA ->
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState278
      | SYMB_REG _v ->
          _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState278
      | PROC _v ->
          _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState278
      | NUM _v ->
          _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState278
      | NAME _v ->
          _menhir_run_044 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState278
      | LBRK ->
          _menhir_run_098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState278
      | FAULT ->
          _menhir_run_051 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState278
      | EOF | SEMI | TOKAND | WITH ->
          let _v = _menhir_action_169 () in
          _menhir_run_279 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState278 _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_098 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _menhir_stack = MenhirCell1_LBRK (_menhir_stack, _menhir_s) in
      let _menhir_s = MenhirState098 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TOK_TAG ->
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PTE ->
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PA ->
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | NUM _v ->
          _menhir_run_099 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NAME _v ->
          _menhir_run_102 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_099 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LBRK as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _menhir_stack = MenhirCell1_NUM (_menhir_stack, _menhir_s, _v) in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | COLON ->
          _menhir_run_100 _menhir_stack _menhir_lexbuf _menhir_lexer
      | _ ->
          _eRR ()
  
  and _menhir_run_100 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LBRK, ttv_result) _menhir_cell1_NUM -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
      let _menhir_s = MenhirState100 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | NAME _v ->
          _menhir_run_101 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_102 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | COLON ->
          let _menhir_stack = MenhirCell1_NAME (_menhir_stack, _menhir_s, _v) in
          _menhir_run_048 _menhir_stack _menhir_lexbuf _menhir_lexer
      | EOF | EQUAL | RBRK | SEMI ->
          let _1 = _v in
          let _v = _menhir_action_135 _1 in
          _menhir_goto_location_global _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_279 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | TOKAND ->
          let _menhir_stack = MenhirCell1_obsone (_menhir_stack, _menhir_s, _v) in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | TOK_TAG ->
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState280
          | TOK_PTE ->
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState280
          | TOK_PA ->
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState280
          | SYMB_REG _v_0 ->
              _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState280
          | PROC _v_1 ->
              _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState280
          | NUM _v_2 ->
              _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v_2 MenhirState280
          | NAME _v_3 ->
              _menhir_run_044 _menhir_stack _menhir_lexbuf _menhir_lexer _v_3 MenhirState280
          | LBRK ->
              _menhir_run_098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState280
          | FAULT ->
              _menhir_run_051 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState280
          | EOF | SEMI | TOKAND | WITH ->
              let _v_4 = _menhir_action_169 () in
              _menhir_run_279 _menhir_stack _menhir_lexbuf _menhir_lexer _v_4 MenhirState280 _tok
          | _ ->
              _eRR ())
      | EOF | SEMI | WITH ->
          let _1 = _v in
          let _v = _menhir_action_167 _1 in
          _menhir_goto_obs _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_286 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _menhir_stack = MenhirCell1_NOT (_menhir_stack, _menhir_s) in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | EXISTS ->
          let _menhir_s = MenhirState287 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | TRUE ->
              _menhir_run_095 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_TAG ->
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PTE ->
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PA ->
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | SYMB_REG _v ->
              _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | PROC _v ->
              _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | NUM _v ->
              _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | NOT ->
              _menhir_run_096 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | NAME _v ->
              _menhir_run_044 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | LPAR ->
              _menhir_run_097 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | LBRK ->
              _menhir_run_098 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | FAULT ->
              _menhir_run_051 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | FALSE ->
              _menhir_run_195 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  and _menhir_run_095 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let _v = _menhir_action_176 () in
      _menhir_goto_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_goto_prop : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState296 ->
          _menhir_run_297 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState294 ->
          _menhir_run_295 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState292 ->
          _menhir_run_293 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState289 ->
          _menhir_run_290 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState287 ->
          _menhir_run_288 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState094 ->
          _menhir_run_276 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState096 ->
          _menhir_run_275 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState273 ->
          _menhir_run_274 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState200 ->
          _menhir_run_201 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState198 ->
          _menhir_run_199 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState097 ->
          _menhir_run_196 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_297 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_EXISTS as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | OR ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_198 _menhir_stack _menhir_lexbuf _menhir_lexer
      | IMPLIES ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_200 _menhir_stack _menhir_lexbuf _menhir_lexer
      | AND ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_273 _menhir_stack _menhir_lexbuf _menhir_lexer
      | EOF | SEMI | WITH ->
          let MenhirCell1_EXISTS (_menhir_stack, _menhir_s) = _menhir_stack in
          let _2 = _v in
          let _v = _menhir_action_070 _2 in
          _menhir_goto_constr _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_198 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_prop -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
      let _menhir_s = MenhirState198 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TRUE ->
          _menhir_run_095 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_TAG ->
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PTE ->
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PA ->
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | SYMB_REG _v ->
          _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | PROC _v ->
          _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NUM _v ->
          _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NOT ->
          _menhir_run_096 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | NAME _v ->
          _menhir_run_044 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | LPAR ->
          _menhir_run_097 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | LBRK ->
          _menhir_run_098 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FAULT ->
          _menhir_run_051 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FALSE ->
          _menhir_run_195 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_096 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _menhir_stack = MenhirCell1_NOT (_menhir_stack, _menhir_s) in
      let _menhir_s = MenhirState096 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TRUE ->
          _menhir_run_095 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_TAG ->
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PTE ->
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PA ->
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | SYMB_REG _v ->
          _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | PROC _v ->
          _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NUM _v ->
          _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NOT ->
          _menhir_run_096 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | NAME _v ->
          _menhir_run_044 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | LPAR ->
          _menhir_run_097 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | LBRK ->
          _menhir_run_098 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FAULT ->
          _menhir_run_051 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FALSE ->
          _menhir_run_195 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_097 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s) in
      let _menhir_s = MenhirState097 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TRUE ->
          _menhir_run_095 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_TAG ->
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PTE ->
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PA ->
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | SYMB_REG _v ->
          _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | PROC _v ->
          _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NUM _v ->
          _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NOT ->
          _menhir_run_096 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | NAME _v ->
          _menhir_run_044 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | LPAR ->
          _menhir_run_097 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | LBRK ->
          _menhir_run_098 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FAULT ->
          _menhir_run_051 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FALSE ->
          _menhir_run_195 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_195 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let _v = _menhir_action_177 () in
      _menhir_goto_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_200 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_prop -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
      let _menhir_s = MenhirState200 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TRUE ->
          _menhir_run_095 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_TAG ->
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PTE ->
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PA ->
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | SYMB_REG _v ->
          _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | PROC _v ->
          _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NUM _v ->
          _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NOT ->
          _menhir_run_096 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | NAME _v ->
          _menhir_run_044 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | LPAR ->
          _menhir_run_097 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | LBRK ->
          _menhir_run_098 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FAULT ->
          _menhir_run_051 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FALSE ->
          _menhir_run_195 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_273 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_prop -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
      let _menhir_s = MenhirState273 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TRUE ->
          _menhir_run_095 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_TAG ->
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PTE ->
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PA ->
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | SYMB_REG _v ->
          _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | PROC _v ->
          _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NUM _v ->
          _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NOT ->
          _menhir_run_096 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | NAME _v ->
          _menhir_run_044 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | LPAR ->
          _menhir_run_097 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | LBRK ->
          _menhir_run_098 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FAULT ->
          _menhir_run_051 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FALSE ->
          _menhir_run_195 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_295 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_FINAL as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | OR ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_198 _menhir_stack _menhir_lexbuf _menhir_lexer
      | IMPLIES ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_200 _menhir_stack _menhir_lexbuf _menhir_lexer
      | AND ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_273 _menhir_stack _menhir_lexbuf _menhir_lexer
      | EOF | SEMI | WITH ->
          let MenhirCell1_FINAL (_menhir_stack, _menhir_s) = _menhir_stack in
          let _2 = _v in
          let _v = _menhir_action_072 _2 in
          _menhir_goto_constr _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_293 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_FORALL as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | OR ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_198 _menhir_stack _menhir_lexbuf _menhir_lexer
      | IMPLIES ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_200 _menhir_stack _menhir_lexbuf _menhir_lexer
      | AND ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_273 _menhir_stack _menhir_lexbuf _menhir_lexer
      | EOF | SEMI | WITH ->
          let MenhirCell1_FORALL (_menhir_stack, _menhir_s) = _menhir_stack in
          let _2 = _v in
          let _v = _menhir_action_069 _2 in
          _menhir_goto_constr _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_290 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LPAR as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | RPAR ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          let MenhirCell1_LPAR (_menhir_stack, _menhir_s) = _menhir_stack in
          let _2 = _v in
          let _v = _menhir_action_073 _2 in
          _menhir_goto_constr _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | OR ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_198 _menhir_stack _menhir_lexbuf _menhir_lexer
      | IMPLIES ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_200 _menhir_stack _menhir_lexbuf _menhir_lexer
      | AND ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_273 _menhir_stack _menhir_lexbuf _menhir_lexer
      | _ ->
          _eRR ()
  
  and _menhir_run_288 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_NOT as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | OR ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_198 _menhir_stack _menhir_lexbuf _menhir_lexer
      | IMPLIES ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_200 _menhir_stack _menhir_lexbuf _menhir_lexer
      | AND ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_273 _menhir_stack _menhir_lexbuf _menhir_lexer
      | EOF | SEMI | WITH ->
          let MenhirCell1_NOT (_menhir_stack, _menhir_s) = _menhir_stack in
          let _3 = _v in
          let _v = _menhir_action_071 _3 in
          _menhir_goto_constr _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_276 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_FILTER as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | OR ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_198 _menhir_stack _menhir_lexbuf _menhir_lexer
      | IMPLIES ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_200 _menhir_stack _menhir_lexbuf _menhir_lexer
      | AND ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_273 _menhir_stack _menhir_lexbuf _menhir_lexer
      | EOF | EXISTS | FINAL | FORALL | LPAR | NOT | OBSERVED | SEMI | WITH ->
          let MenhirCell1_FILTER (_menhir_stack, _menhir_s) = _menhir_stack in
          let _2 = _v in
          let _v = _menhir_action_085 _2 in
          _menhir_goto_filter _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_goto_filter : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState382 ->
          _menhir_run_384 _menhir_stack _v _tok
      | MenhirState093 ->
          _menhir_run_277 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_277 : type  ttv_stack. ((ttv_stack, _menhir_box_constraints) _menhir_cell1_locations as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_constraints) _menhir_state -> _ -> _menhir_box_constraints =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _menhir_stack = MenhirCell1_filter (_menhir_stack, _menhir_s, _v) in
      match (_tok : MenhirBasics.token) with
      | OBSERVED ->
          _menhir_run_278 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState277
      | NOT ->
          _menhir_run_286 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState277
      | LPAR ->
          _menhir_run_289 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState277
      | FORALL ->
          _menhir_run_292 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState277
      | FINAL ->
          _menhir_run_294 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState277
      | EXISTS ->
          _menhir_run_296 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState277
      | EOF | SEMI | WITH ->
          let _v_0 = _menhir_action_068 () in
          _menhir_run_312 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState277 _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_289 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s) in
      let _menhir_s = MenhirState289 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TRUE ->
          _menhir_run_095 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_TAG ->
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PTE ->
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PA ->
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | SYMB_REG _v ->
          _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | PROC _v ->
          _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NUM _v ->
          _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NOT ->
          _menhir_run_096 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | NAME _v ->
          _menhir_run_044 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | LPAR ->
          _menhir_run_097 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | LBRK ->
          _menhir_run_098 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FAULT ->
          _menhir_run_051 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FALSE ->
          _menhir_run_195 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_292 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _menhir_stack = MenhirCell1_FORALL (_menhir_stack, _menhir_s) in
      let _menhir_s = MenhirState292 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TRUE ->
          _menhir_run_095 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_TAG ->
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PTE ->
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PA ->
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | SYMB_REG _v ->
          _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | PROC _v ->
          _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NUM _v ->
          _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NOT ->
          _menhir_run_096 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | NAME _v ->
          _menhir_run_044 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | LPAR ->
          _menhir_run_097 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | LBRK ->
          _menhir_run_098 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FAULT ->
          _menhir_run_051 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FALSE ->
          _menhir_run_195 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_294 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _menhir_stack = MenhirCell1_FINAL (_menhir_stack, _menhir_s) in
      let _menhir_s = MenhirState294 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TRUE ->
          _menhir_run_095 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_TAG ->
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PTE ->
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PA ->
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | SYMB_REG _v ->
          _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | PROC _v ->
          _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NUM _v ->
          _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NOT ->
          _menhir_run_096 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | NAME _v ->
          _menhir_run_044 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | LPAR ->
          _menhir_run_097 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | LBRK ->
          _menhir_run_098 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FAULT ->
          _menhir_run_051 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FALSE ->
          _menhir_run_195 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_296 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _menhir_stack = MenhirCell1_EXISTS (_menhir_stack, _menhir_s) in
      let _menhir_s = MenhirState296 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TRUE ->
          _menhir_run_095 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_TAG ->
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PTE ->
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PA ->
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | SYMB_REG _v ->
          _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | PROC _v ->
          _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NUM _v ->
          _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NOT ->
          _menhir_run_096 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | NAME _v ->
          _menhir_run_044 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | LPAR ->
          _menhir_run_097 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | LBRK ->
          _menhir_run_098 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FAULT ->
          _menhir_run_051 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FALSE ->
          _menhir_run_195 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_275 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_NOT -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_NOT (_menhir_stack, _menhir_s) = _menhir_stack in
      let _2 = _v in
      let _v = _menhir_action_179 _2 in
      _menhir_goto_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_274 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_prop as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | IMPLIES ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_200 _menhir_stack _menhir_lexbuf _menhir_lexer
      | AND | EOF | EXISTS | FINAL | FORALL | LPAR | NOT | OBSERVED | OR | RPAR | SEMI | WITH ->
          let MenhirCell1_prop (_menhir_stack, _menhir_s, _1) = _menhir_stack in
          let _3 = _v in
          let _v = _menhir_action_180 _1 _3 in
          _menhir_goto_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_201 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_prop as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | IMPLIES ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_200 _menhir_stack _menhir_lexbuf _menhir_lexer
      | AND | EOF | EXISTS | FINAL | FORALL | LPAR | NOT | OBSERVED | OR | RPAR | SEMI | WITH ->
          let MenhirCell1_prop (_menhir_stack, _menhir_s, _1) = _menhir_stack in
          let _3 = _v in
          let _v = _menhir_action_182 _1 _3 in
          _menhir_goto_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_199 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_prop as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | IMPLIES ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_200 _menhir_stack _menhir_lexbuf _menhir_lexer
      | AND ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_273 _menhir_stack _menhir_lexbuf _menhir_lexer
      | EOF | EXISTS | FINAL | FORALL | LPAR | NOT | OBSERVED | OR | RPAR | SEMI | WITH ->
          let MenhirCell1_prop (_menhir_stack, _menhir_s, _1) = _menhir_stack in
          let _3 = _v in
          let _v = _menhir_action_181 _1 _3 in
          _menhir_goto_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_196 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LPAR as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | RPAR ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          let MenhirCell1_LPAR (_menhir_stack, _menhir_s) = _menhir_stack in
          let _2 = _v in
          let _v = _menhir_action_183 _2 in
          _menhir_goto_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | OR ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_198 _menhir_stack _menhir_lexbuf _menhir_lexer
      | IMPLIES ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_200 _menhir_stack _menhir_lexbuf _menhir_lexer
      | AND ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_273 _menhir_stack _menhir_lexbuf _menhir_lexer
      | _ ->
          _eRR ()
  
  and _menhir_run_388 : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_main_loc_constr) _menhir_state -> _ -> _menhir_box_main_loc_constr =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _menhir_stack = MenhirCell1_locations (_menhir_stack, _menhir_s, _v) in
      match (_tok : MenhirBasics.token) with
      | OBSERVED ->
          _menhir_run_278 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState388
      | NOT ->
          _menhir_run_286 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState388
      | LPAR ->
          _menhir_run_289 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState388
      | FORALL ->
          _menhir_run_292 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState388
      | FINAL ->
          _menhir_run_294 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState388
      | EXISTS ->
          _menhir_run_296 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState388
      | EOF ->
          let _v_0 = _menhir_action_068 () in
          _menhir_run_389 _menhir_stack _v_0 _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_093 : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_constraints) _menhir_state -> _ -> _menhir_box_constraints =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _menhir_stack = MenhirCell1_locations (_menhir_stack, _menhir_s, _v) in
      match (_tok : MenhirBasics.token) with
      | FILTER ->
          _menhir_run_094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState093
      | EOF | EXISTS | FINAL | FORALL | LPAR | NOT | OBSERVED | SEMI | WITH ->
          let _v_0 = _menhir_action_084 () in
          _menhir_run_277 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState093 _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_094 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _menhir_stack = MenhirCell1_FILTER (_menhir_stack, _menhir_s) in
      let _menhir_s = MenhirState094 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TRUE ->
          _menhir_run_095 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_TAG ->
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PTE ->
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PA ->
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | SYMB_REG _v ->
          _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | PROC _v ->
          _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NUM _v ->
          _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NOT ->
          _menhir_run_096 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | NAME _v ->
          _menhir_run_044 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | LPAR ->
          _menhir_run_097 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | LBRK ->
          _menhir_run_098 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FAULT ->
          _menhir_run_051 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FALSE ->
          _menhir_run_195 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_284 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_atom_prop -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_atom_prop (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_170 _1 _3 in
      _menhir_goto_obsone _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_goto_obsone : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState283 ->
          _menhir_run_284 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState280 ->
          _menhir_run_279 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState278 ->
          _menhir_run_279 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_272 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _1 = _v in
      let _v = _menhir_action_178 _1 in
      _menhir_goto_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_258 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_location -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_location (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_033 _1 _3 in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_217 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_location -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_location (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_036 _1 _3 in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_208 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_locindex -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_locindex (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_045 _1 _3 in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_206 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_locindex -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_locindex (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_046 _1 _3 in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_204 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_locindex -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_locindex (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_047 _1 _3 in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_194 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LBRK, ttv_result) _menhir_cell1_location_global -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_location_global (_menhir_stack, _, loc) = _menhir_stack in
      let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
      let v = _v in
      let _v = _menhir_action_034 loc v in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_173 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LBRK, ttv_result) _menhir_cell1_location_global -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_location_global (_menhir_stack, _, loc) = _menhir_stack in
      let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
      let v = _v in
      let _v = _menhir_action_035 loc v in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_117 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LBRK, ttv_result) _menhir_cell1_location_global -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_location_global (_menhir_stack, _, loc) = _menhir_stack in
      let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
      let v = _v in
      let _v = _menhir_action_037 loc v in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_219 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let _1 = _v in
      let _v = _menhir_action_123 _1 in
      _menhir_goto_location_deref _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_220 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | SYMB_REG _v ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          let _1_inlined1 = _v in
          let _v = _menhir_action_130 _1_inlined1 in
          _menhir_goto_location_deref _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | PROC _v ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | COLON ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | SYMB_REG _v_0 ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  let (_3, _1_inlined1) = (_v_0, _v) in
                  let _v = _menhir_action_128 _1_inlined1 _3 in
                  _menhir_goto_location_deref _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
              | PTX_REG_DEC ->
                  let _menhir_stack = MenhirCell1_STAR (_menhir_stack, _menhir_s) in
                  let _menhir_stack = MenhirCell0_PROC (_menhir_stack, _v) in
                  let _menhir_stack = MenhirCell1_PTX_REG_DEC (_menhir_stack, MenhirState223) in
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | PTX_REG_TYPE _v ->
                      let _menhir_stack = MenhirCell0_PTX_REG_TYPE (_menhir_stack, _v) in
                      let _menhir_s = MenhirState226 in
                      let _tok = _menhir_lexer _menhir_lexbuf in
                      (match (_tok : MenhirBasics.token) with
                      | NAME _v ->
                          _menhir_run_030 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                      | DOLLARNAME _v ->
                          _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                      | _ ->
                          _eRR ())
                  | _ ->
                      _eRR ())
              | NAME _v_4 ->
                  let _menhir_stack = MenhirCell1_STAR (_menhir_stack, _menhir_s) in
                  let _menhir_stack = MenhirCell0_PROC (_menhir_stack, _v) in
                  _menhir_run_030 _menhir_stack _menhir_lexbuf _menhir_lexer _v_4 MenhirState223
              | DOLLARNAME _v_5 ->
                  let _menhir_stack = MenhirCell1_STAR (_menhir_stack, _menhir_s) in
                  let _menhir_stack = MenhirCell0_PROC (_menhir_stack, _v) in
                  _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v_5 MenhirState223
              | _ ->
                  _eRR ())
          | _ ->
              _eRR ())
      | NUM _v ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | COLON ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | SYMB_REG _v_6 ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  let (_3, _1_inlined1) = (_v_6, _v) in
                  let _v = _menhir_action_129 _1_inlined1 _3 in
                  _menhir_goto_location_deref _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
              | PTX_REG_DEC ->
                  let _menhir_stack = MenhirCell1_STAR (_menhir_stack, _menhir_s) in
                  let _menhir_stack = MenhirCell0_NUM (_menhir_stack, _v) in
                  let _menhir_stack = MenhirCell1_PTX_REG_DEC (_menhir_stack, MenhirState230) in
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | PTX_REG_TYPE _v ->
                      let _menhir_stack = MenhirCell0_PTX_REG_TYPE (_menhir_stack, _v) in
                      let _menhir_s = MenhirState233 in
                      let _tok = _menhir_lexer _menhir_lexbuf in
                      (match (_tok : MenhirBasics.token) with
                      | NAME _v ->
                          _menhir_run_030 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                      | DOLLARNAME _v ->
                          _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                      | _ ->
                          _eRR ())
                  | _ ->
                      _eRR ())
              | NAME _v_10 ->
                  let _menhir_stack = MenhirCell1_STAR (_menhir_stack, _menhir_s) in
                  let _menhir_stack = MenhirCell0_NUM (_menhir_stack, _v) in
                  _menhir_run_030 _menhir_stack _menhir_lexbuf _menhir_lexer _v_10 MenhirState230
              | DOLLARNAME _v_11 ->
                  let _menhir_stack = MenhirCell1_STAR (_menhir_stack, _menhir_s) in
                  let _menhir_stack = MenhirCell0_NUM (_menhir_stack, _v) in
                  _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v_11 MenhirState230
              | _ ->
                  _eRR ())
          | _ ->
              _eRR ())
      | NAME _v ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          let _2 = _v in
          let _v = _menhir_action_133 _2 in
          _menhir_goto_location_deref _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_237 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | COLON ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | SYMB_REG _v_0 ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              let (_1, _3) = (_v, _v_0) in
              let _v = _menhir_action_121 _1 _3 in
              _menhir_goto_location_deref _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
          | PTX_REG_DEC ->
              let _menhir_stack = MenhirCell1_PROC (_menhir_stack, _menhir_s, _v) in
              let _menhir_stack = MenhirCell1_PTX_REG_DEC (_menhir_stack, MenhirState238) in
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | PTX_REG_TYPE _v ->
                  let _menhir_stack = MenhirCell0_PTX_REG_TYPE (_menhir_stack, _v) in
                  let _menhir_s = MenhirState241 in
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | NAME _v ->
                      _menhir_run_030 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                  | DOLLARNAME _v ->
                      _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                  | _ ->
                      _eRR ())
              | _ ->
                  _eRR ())
          | NAME _v_4 ->
              let _menhir_stack = MenhirCell1_PROC (_menhir_stack, _menhir_s, _v) in
              _menhir_run_030 _menhir_stack _menhir_lexbuf _menhir_lexer _v_4 MenhirState238
          | DOLLARNAME _v_5 ->
              let _menhir_stack = MenhirCell1_PROC (_menhir_stack, _menhir_s, _v) in
              _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v_5 MenhirState238
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  and _menhir_run_244 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | COLON ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | SYMB_REG _v_0 ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              let (_1, _3) = (_v, _v_0) in
              let _v = _menhir_action_122 _1 _3 in
              _menhir_goto_location_deref _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
          | PTX_REG_DEC ->
              let _menhir_stack = MenhirCell1_NUM (_menhir_stack, _menhir_s, _v) in
              let _menhir_stack = MenhirCell1_PTX_REG_DEC (_menhir_stack, MenhirState245) in
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | PTX_REG_TYPE _v ->
                  let _menhir_stack = MenhirCell0_PTX_REG_TYPE (_menhir_stack, _v) in
                  let _menhir_s = MenhirState248 in
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | NAME _v ->
                      _menhir_run_030 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                  | DOLLARNAME _v ->
                      _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                  | _ ->
                      _eRR ())
              | _ ->
                  _eRR ())
          | NUM _v_4 ->
              let _menhir_stack = MenhirCell1_NUM (_menhir_stack, _menhir_s, _v) in
              _menhir_run_109 _menhir_stack _menhir_lexbuf _menhir_lexer _v_4
          | NAME _v_5 ->
              let _menhir_stack = MenhirCell1_NUM (_menhir_stack, _menhir_s, _v) in
              _menhir_run_040 _menhir_stack _menhir_lexbuf _menhir_lexer _v_5 MenhirState245
          | DOLLARNAME _v_6 ->
              let _menhir_stack = MenhirCell1_NUM (_menhir_stack, _menhir_s, _v) in
              _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v_6 MenhirState245
          | _ ->
              _eRR ())
      | AND | EOF | EXISTS | FINAL | FORALL | IMPLIES | LPAR | NOT | OBSERVED | OR | RPAR | SEMI | WITH ->
          let _1 = _v in
          let _v = _menhir_action_160 _1 in
          _menhir_goto_maybev_notag _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_135 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s) in
      let _menhir_s = MenhirState135 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TOK_OA ->
          _menhir_run_136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | NAME _v ->
          _menhir_run_149 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | ATTRS ->
          _menhir_run_156 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_136 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _menhir_stack = MenhirCell1_TOK_OA (_menhir_stack, _menhir_s) in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | COLON ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | TOK_PTE ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | LPAR ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | NAME _v ->
                      let _tok = _menhir_lexer _menhir_lexbuf in
                      (match (_tok : MenhirBasics.token) with
                      | RPAR ->
                          let _tok = _menhir_lexer _menhir_lexbuf in
                          let name = _v in
                          let _v = _menhir_action_175 name in
                          _menhir_goto_output_address _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
                      | _ ->
                          _eRR ())
                  | _ ->
                      _eRR ())
              | _ ->
                  _eRR ())
          | TOK_PA ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | LPAR ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | NAME _v ->
                      let _tok = _menhir_lexer _menhir_lexbuf in
                      (match (_tok : MenhirBasics.token) with
                      | RPAR ->
                          let _tok = _menhir_lexer _menhir_lexbuf in
                          let name = _v in
                          let _v = _menhir_action_174 name in
                          _menhir_goto_output_address _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
                      | _ ->
                          _eRR ())
                  | _ ->
                      _eRR ())
              | _ ->
                  _eRR ())
          | NAME _v ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              let name = _v in
              let _v = _menhir_action_173 name in
              _menhir_goto_output_address _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  and _menhir_goto_output_address : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_TOK_OA -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let _menhir_stack = MenhirCell0_output_address (_menhir_stack, _v) in
      match (_tok : MenhirBasics.token) with
      | COMMA ->
          _menhir_run_148 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState147
      | RPAR ->
          let _v_0 = _menhir_action_188 () in
          _menhir_run_166 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_148 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _menhir_stack = MenhirCell1_COMMA (_menhir_stack, _menhir_s) in
      let _menhir_s = MenhirState148 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TOK_OA ->
          _menhir_run_136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | NAME _v ->
          _menhir_run_149 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | ATTRS ->
          _menhir_run_156 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_149 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _menhir_stack = MenhirCell1_NAME (_menhir_stack, _menhir_s, _v) in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | COMMA ->
          _menhir_run_148 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState149
      | COLON ->
          let _menhir_stack = MenhirCell1_COLON (_menhir_stack, MenhirState149) in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | NUM _v_0 ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              let _1 = _v_0 in
              let _v = _menhir_action_166 _1 in
              _menhir_goto_name_or_num _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
          | NAME _v_2 ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              let _1 = _v_2 in
              let _v = _menhir_action_165 _1 in
              _menhir_goto_name_or_num _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
          | _ ->
              _eRR ())
      | RPAR ->
          let _v_4 = _menhir_action_188 () in
          _menhir_run_155 _menhir_stack _menhir_lexbuf _menhir_lexer _v_4 _tok
      | _ ->
          _eRR ()
  
  and _menhir_goto_name_or_num : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_NAME, ttv_result) _menhir_cell1_COLON -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let _menhir_stack = MenhirCell0_name_or_num (_menhir_stack, _v) in
      match (_tok : MenhirBasics.token) with
      | COMMA ->
          _menhir_run_148 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState153
      | RPAR ->
          let _v_0 = _menhir_action_188 () in
          _menhir_run_154 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_154 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_NAME, ttv_result) _menhir_cell1_COLON _menhir_cell0_name_or_num -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell0_name_or_num (_menhir_stack, v) = _menhir_stack in
      let MenhirCell1_COLON (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_NAME (_menhir_stack, _menhir_s, key) = _menhir_stack in
      let tail = _v in
      let _v = _menhir_action_185 key tail v in
      _menhir_goto_prop_head _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_goto_prop_head : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState397 ->
          _menhir_run_398 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState135 ->
          _menhir_run_167 _menhir_stack _menhir_lexbuf _menhir_lexer _v
      | MenhirState148 ->
          _menhir_run_165 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_398 : type  ttv_stack. (ttv_stack, _menhir_box_pteval) _menhir_cell1_LPAR -> _ -> _ -> _ -> _ -> _menhir_box_pteval =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_LPAR (_menhir_stack, _menhir_s) = _menhir_stack in
      let pteval = _v in
      let _v = _menhir_action_190 pteval in
      _menhir_goto_pteval _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_goto_pteval : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState396 ->
          _menhir_run_400 _menhir_stack _v
      | MenhirState359 ->
          _menhir_run_367 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState336 ->
          _menhir_run_341 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState260 ->
          _menhir_run_267 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState218 ->
          _menhir_run_257 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState174 ->
          _menhir_run_193 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState119 ->
          _menhir_run_172 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_367 : type  ttv_stack. (ttv_stack, _menhir_box_init) _menhir_cell1_left_loc -> _ -> _ -> _ -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_left_loc (_menhir_stack, _menhir_s, loc) = _menhir_stack in
      let v = _v in
      let _v = _menhir_action_029 loc v in
      _menhir_goto_atom_init _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_341 : type  ttv_stack. ((ttv_stack, _menhir_box_init) _menhir_cell1_NAME, _menhir_box_init) _menhir_cell1_left_loc -> _ -> _ -> _ -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_left_loc (_menhir_stack, _, loc) = _menhir_stack in
      let MenhirCell1_NAME (_menhir_stack, _menhir_s, typ) = _menhir_stack in
      let v = _v in
      let _v = _menhir_action_028 loc typ v in
      _menhir_goto_atom_init _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_267 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_location -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_location (_menhir_stack, _menhir_s, loc) = _menhir_stack in
      let v = _v in
      let _v = _menhir_action_038 loc v in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_257 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_location -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_location (_menhir_stack, _menhir_s, loc) = _menhir_stack in
      let v = _v in
      let _v = _menhir_action_039 loc v in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_193 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LBRK, ttv_result) _menhir_cell1_location_global -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_location_global (_menhir_stack, _, loc) = _menhir_stack in
      let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
      let v = _v in
      let _v = _menhir_action_040 loc v in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_172 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LBRK, ttv_result) _menhir_cell1_location_global -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_location_global (_menhir_stack, _, loc) = _menhir_stack in
      let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
      let v = _v in
      let _v = _menhir_action_041 loc v in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_167 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_LPAR -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let MenhirCell1_LPAR (_menhir_stack, _menhir_s) = _menhir_stack in
      let pteval = _v in
      let _v = _menhir_action_190 pteval in
      _menhir_goto_pteval _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_165 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_COMMA -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_COMMA (_menhir_stack, _menhir_s) = _menhir_stack in
      let pteval = _v in
      let _v = _menhir_action_189 pteval in
      _menhir_goto_prop_tail _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_goto_prop_tail : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState147 ->
          _menhir_run_166 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState163 ->
          _menhir_run_164 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState149 ->
          _menhir_run_155 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState153 ->
          _menhir_run_154 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_166 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_TOK_OA _menhir_cell0_output_address -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell0_output_address (_menhir_stack, oa) = _menhir_stack in
      let MenhirCell1_TOK_OA (_menhir_stack, _menhir_s) = _menhir_stack in
      let tail = _v in
      let _v = _menhir_action_184 oa tail in
      _menhir_goto_prop_head _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_164 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_ATTRS, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_NAME_ -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_separated_nonempty_list_COMMA_NAME_ (_menhir_stack, _, attrs) = _menhir_stack in
      let MenhirCell1_ATTRS (_menhir_stack, _menhir_s) = _menhir_stack in
      let tail = _v in
      let _v = _menhir_action_187 attrs tail in
      _menhir_goto_prop_head _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_155 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_NAME -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_NAME (_menhir_stack, _menhir_s, a) = _menhir_stack in
      let tail = _v in
      let _v = _menhir_action_186 a tail in
      _menhir_goto_prop_head _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_156 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _menhir_stack = MenhirCell1_ATTRS (_menhir_stack, _menhir_s) in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | COLON ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | LPAR ->
              let _menhir_s = MenhirState158 in
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | NAME _v ->
                  _menhir_run_159 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
              | _ ->
                  _eRR ())
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  and _menhir_run_159 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | COMMA ->
          let _menhir_stack = MenhirCell1_NAME (_menhir_stack, _menhir_s, _v) in
          let _menhir_s = MenhirState160 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | NAME _v ->
              _menhir_run_159 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | _ ->
              _eRR ())
      | RPAR ->
          let x = _v in
          let _v = _menhir_action_201 x in
          _menhir_goto_separated_nonempty_list_COMMA_NAME_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_goto_separated_nonempty_list_COMMA_NAME_ : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      match _menhir_s with
      | MenhirState158 ->
          _menhir_run_162 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | MenhirState160 ->
          _menhir_run_161 _menhir_stack _menhir_lexbuf _menhir_lexer _v
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_162 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_ATTRS as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_NAME_ (_menhir_stack, _menhir_s, _v) in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | COMMA ->
          _menhir_run_148 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState163
      | RPAR ->
          let _v_0 = _menhir_action_188 () in
          _menhir_run_164 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_161 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_NAME -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
      let MenhirCell1_NAME (_menhir_stack, _menhir_s, x) = _menhir_stack in
      let xs = _v in
      let _v = _menhir_action_202 x xs in
      _menhir_goto_separated_nonempty_list_COMMA_NAME_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
  
  and _menhir_run_254 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _menhir_stack = MenhirCell1_LBRK (_menhir_stack, _menhir_s) in
      let _menhir_s = MenhirState254 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TOK_TAG ->
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PTE ->
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PA ->
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | NUM _v ->
          _menhir_run_099 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NAME _v ->
          _menhir_run_102 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_261 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let _v = _menhir_action_093 () in
      _menhir_goto_instr _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_goto_instr : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState359 ->
          _menhir_run_370 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState336 ->
          _menhir_run_343 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState260 ->
          _menhir_run_270 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_370 : type  ttv_stack. (ttv_stack, _menhir_box_init) _menhir_cell1_left_loc -> _ -> _ -> _ -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_left_loc (_menhir_stack, _menhir_s, loc) = _menhir_stack in
      let i = _v in
      let _v = _menhir_action_016 i loc in
      _menhir_goto_atom_init _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_343 : type  ttv_stack. ((ttv_stack, _menhir_box_init) _menhir_cell1_NAME, _menhir_box_init) _menhir_cell1_left_loc -> _ -> _ -> _ -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_left_loc (_menhir_stack, _, loc) = _menhir_stack in
      let MenhirCell1_NAME (_menhir_stack, _menhir_s, _) = _menhir_stack in
      let i = _v in
      let _v = _menhir_action_017 i loc in
      _menhir_goto_atom_init _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_270 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_location -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_location (_menhir_stack, _menhir_s, loc) = _menhir_stack in
      let i = _v in
      let _v = _menhir_action_030 i loc in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_266 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let i = _v in
      let _v = _menhir_action_094 i in
      _menhir_goto_instr _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_085 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _1 = _v in
      let _v = _menhir_action_193 _1 in
      _menhir_goto_rloc _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_316 : type  ttv_stack. ttv_stack -> _ -> _ -> (ttv_stack, _menhir_box_init) _menhir_state -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _menhir_stack = MenhirCell1_STAR (_menhir_stack, _menhir_s) in
      let _menhir_s = MenhirState316 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TOK_TAG ->
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PTE ->
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PA ->
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | SYMB_REG _v ->
          _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | PROC _v ->
          _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NUM _v ->
          _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NAME _v ->
          _menhir_run_102 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | LBRK ->
          _menhir_run_317 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_317 : type  ttv_stack. ttv_stack -> _ -> _ -> (ttv_stack, _menhir_box_init) _menhir_state -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _menhir_stack = MenhirCell1_LBRK (_menhir_stack, _menhir_s) in
      let _menhir_s = MenhirState317 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TOK_TAG ->
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PTE ->
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PA ->
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | NUM _v ->
          _menhir_run_099 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NAME _v ->
          _menhir_run_102 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_327 : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_init) _menhir_state -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TOK_TAG ->
          let _menhir_stack = MenhirCell1_NAME (_menhir_stack, _menhir_s, _v) in
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState327
      | TOK_PTE ->
          let _menhir_stack = MenhirCell1_NAME (_menhir_stack, _menhir_s, _v) in
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState327
      | TOK_PA ->
          let _menhir_stack = MenhirCell1_NAME (_menhir_stack, _menhir_s, _v) in
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState327
      | SYMB_REG _v_0 ->
          let _menhir_stack = MenhirCell1_NAME (_menhir_stack, _menhir_s, _v) in
          _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState327
      | STAR ->
          let _menhir_stack = MenhirCell1_NAME (_menhir_stack, _menhir_s, _v) in
          let _menhir_stack = MenhirCell1_STAR (_menhir_stack, MenhirState327) in
          let _menhir_s = MenhirState328 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | TOK_TAG ->
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PTE ->
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PA ->
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | SYMB_REG _v ->
              _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | PROC _v ->
              _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | NUM _v ->
              _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | NAME _v ->
              _menhir_run_102 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | LBRK ->
              _menhir_run_317 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | _ ->
              _eRR ())
      | PROC _v_5 ->
          let _menhir_stack = MenhirCell1_NAME (_menhir_stack, _menhir_s, _v) in
          _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v_5 MenhirState327
      | NUM _v_6 ->
          let _menhir_stack = MenhirCell1_NAME (_menhir_stack, _menhir_s, _v) in
          _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v_6 MenhirState327
      | NAME _v_7 ->
          let _menhir_stack = MenhirCell1_NAME (_menhir_stack, _menhir_s, _v) in
          _menhir_run_044 _menhir_stack _menhir_lexbuf _menhir_lexer _v_7 MenhirState327
      | LBRK ->
          let _menhir_stack = MenhirCell1_NAME (_menhir_stack, _menhir_s, _v) in
          let _menhir_stack = MenhirCell1_LBRK (_menhir_stack, MenhirState327) in
          let _menhir_s = MenhirState333 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | TOK_TAG ->
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PTE ->
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PA ->
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | NUM _v ->
              let _menhir_stack = MenhirCell1_NUM (_menhir_stack, _menhir_s, _v) in
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | RBRK ->
                  _menhir_run_047 _menhir_stack _menhir_lexbuf _menhir_lexer
              | COLON ->
                  _menhir_run_100 _menhir_stack _menhir_lexbuf _menhir_lexer
              | _ ->
                  _eRR ())
          | NAME _v ->
              _menhir_run_102 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | _ ->
              _eRR ())
      | COLON ->
          let _menhir_stack = MenhirCell1_NAME (_menhir_stack, _menhir_s, _v) in
          _menhir_run_048 _menhir_stack _menhir_lexbuf _menhir_lexer
      | EOF | EQUAL | SEMI ->
          let _1 = _v in
          let _v = _menhir_action_135 _1 in
          _menhir_goto_location_global _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_349 : type  ttv_stack. ttv_stack -> _ -> _ -> (ttv_stack, _menhir_box_init) _menhir_state -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _menhir_stack = MenhirCell1_ATOMIC (_menhir_stack, _menhir_s) in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | NAME _v ->
          let _menhir_stack = MenhirCell0_NAME (_menhir_stack, _v) in
          let _menhir_s = MenhirState350 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | TOK_TAG ->
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PTE ->
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PA ->
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | SYMB_REG _v ->
              _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | PROC _v ->
              _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | NUM _v ->
              _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | NAME _v ->
              _menhir_run_102 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | LBRK ->
              _menhir_run_317 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  and _menhir_run_263 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_location, ttv_result) _menhir_cell1_LCURLY -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let MenhirCell1_LCURLY (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_location (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _4 = _v in
      let _v = _menhir_action_042 _1 _4 in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_252 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_location, ttv_result) _menhir_cell1_LCURLY -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let MenhirCell1_LCURLY (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_location (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _4 = _v in
      let _v = _menhir_action_043 _1 _4 in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_215 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_location, ttv_result) _menhir_cell1_LCURLY -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let MenhirCell1_LCURLY (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_location (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _4 = _v in
      let _v = _menhir_action_044 _1 _4 in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_214 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_maybev_notag -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
      let MenhirCell1_maybev_notag (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_158 _1 _3 in
      _menhir_goto_maybev_list _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
  
  and _menhir_run_116 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _1 = _v in
      let _v = _menhir_action_152 _1 in
      _menhir_goto_maybev _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_351 : type  ttv_stack. ((ttv_stack, _menhir_box_init) _menhir_cell1_ATOMIC _menhir_cell0_NAME as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_init) _menhir_state -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | EQUAL ->
          let _menhir_stack = MenhirCell1_left_loc (_menhir_stack, _menhir_s, _v) in
          let _menhir_s = MenhirState352 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | VALUE _v ->
              _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | TOK_TAG ->
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PTE ->
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PA ->
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | NUM _v ->
              _menhir_run_107 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | NAME _v ->
              _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | COLON ->
              _menhir_run_114 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | _ ->
              _eRR ())
      | EOF | SEMI ->
          let MenhirCell0_NAME (_menhir_stack, typ) = _menhir_stack in
          let MenhirCell1_ATOMIC (_menhir_stack, _menhir_s) = _menhir_stack in
          let loc = _v in
          let _v = _menhir_action_015 loc typ in
          _menhir_goto_atom_init _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_335 : type  ttv_stack. ((ttv_stack, _menhir_box_init) _menhir_cell1_NAME as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_init) _menhir_state -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | EQUAL ->
          let _menhir_stack = MenhirCell1_left_loc (_menhir_stack, _menhir_s, _v) in
          let _menhir_s = MenhirState336 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | VALUE _v ->
              _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | TOK_TAG ->
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PTE ->
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PA ->
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_NOP ->
              _menhir_run_261 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | NUM _v ->
              _menhir_run_107 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | NAME _v ->
              _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | LPAR ->
              _menhir_run_135 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | INSTR _v ->
              _menhir_run_266 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | COLON ->
              _menhir_run_114 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | ATOMICINIT ->
              let _menhir_stack = MenhirCell1_ATOMICINIT (_menhir_stack, _menhir_s) in
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | LPAR ->
                  let _menhir_s = MenhirState338 in
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | VALUE _v ->
                      _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                  | TOK_TAG ->
                      _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                  | TOK_PTE ->
                      _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                  | TOK_PA ->
                      _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                  | NUM _v ->
                      _menhir_run_107 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                  | NAME _v ->
                      _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                  | COLON ->
                      _menhir_run_114 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                  | _ ->
                      _eRR ())
              | _ ->
                  _eRR ())
          | _ ->
              _eRR ())
      | EOF | SEMI ->
          let MenhirCell1_NAME (_menhir_stack, _menhir_s, typ) = _menhir_stack in
          let loc = _v in
          let _v = _menhir_action_014 loc typ in
          _menhir_goto_atom_init _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_329 : type  ttv_stack. (((ttv_stack, _menhir_box_init) _menhir_cell1_NAME, _menhir_box_init) _menhir_cell1_STAR as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_init) _menhir_state -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | EQUAL ->
          let _menhir_stack = MenhirCell1_left_loc (_menhir_stack, _menhir_s, _v) in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | AMPER ->
              _menhir_run_323 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState330
          | COLON | NAME _ | NUM _ | TOK_PA | TOK_PTE | TOK_TAG | VALUE _ ->
              let _v_0 = _menhir_action_009 () in
              _menhir_run_331 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState330 _tok
          | _ ->
              _eRR ())
      | EOF | SEMI ->
          let MenhirCell1_STAR (_menhir_stack, _) = _menhir_stack in
          let MenhirCell1_NAME (_menhir_stack, _menhir_s, typ) = _menhir_stack in
          let loc = _v in
          let _v = _menhir_action_024 loc typ in
          _menhir_goto_atom_init _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_323 : type  ttv_stack. (((ttv_stack, _menhir_box_init) _menhir_cell1_STAR, _menhir_box_init) _menhir_cell1_left_loc as 'stack) -> _ -> _ -> ('stack, _menhir_box_init) _menhir_state -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let _v = _menhir_action_008 () in
      _menhir_goto_amperopt _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_goto_amperopt : type  ttv_stack. (((ttv_stack, _menhir_box_init) _menhir_cell1_STAR, _menhir_box_init) _menhir_cell1_left_loc as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_init) _menhir_state -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState330 ->
          _menhir_run_331 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState322 ->
          _menhir_run_324 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_331 : type  ttv_stack. ((((ttv_stack, _menhir_box_init) _menhir_cell1_NAME, _menhir_box_init) _menhir_cell1_STAR, _menhir_box_init) _menhir_cell1_left_loc as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_init) _menhir_state -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _menhir_stack = MenhirCell1_amperopt (_menhir_stack, _menhir_s, _v) in
      match (_tok : MenhirBasics.token) with
      | VALUE _v_0 ->
          _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState331
      | TOK_TAG ->
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState331
      | TOK_PTE ->
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState331
      | TOK_PA ->
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState331
      | NUM _v_1 ->
          _menhir_run_107 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState331
      | NAME _v_2 ->
          _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v_2 MenhirState331
      | COLON ->
          _menhir_run_114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState331
      | _ ->
          _eRR ()
  
  and _menhir_run_324 : type  ttv_stack. (((ttv_stack, _menhir_box_init) _menhir_cell1_STAR, _menhir_box_init) _menhir_cell1_left_loc as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_init) _menhir_state -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _menhir_stack = MenhirCell1_amperopt (_menhir_stack, _menhir_s, _v) in
      match (_tok : MenhirBasics.token) with
      | VALUE _v_0 ->
          _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState324
      | TOK_TAG ->
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState324
      | TOK_PTE ->
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState324
      | TOK_PA ->
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState324
      | NUM _v_1 ->
          _menhir_run_107 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState324
      | NAME _v_2 ->
          _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v_2 MenhirState324
      | COLON ->
          _menhir_run_114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState324
      | _ ->
          _eRR ()
  
  and _menhir_run_321 : type  ttv_stack. ((ttv_stack, _menhir_box_init) _menhir_cell1_STAR as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_init) _menhir_state -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | EQUAL ->
          let _menhir_stack = MenhirCell1_left_loc (_menhir_stack, _menhir_s, _v) in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | AMPER ->
              _menhir_run_323 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState322
          | COLON | NAME _ | NUM _ | TOK_PA | TOK_PTE | TOK_TAG | VALUE _ ->
              let _v_0 = _menhir_action_009 () in
              _menhir_run_324 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState322 _tok
          | _ ->
              _eRR ())
      | EOF | SEMI ->
          let MenhirCell1_STAR (_menhir_stack, _menhir_s) = _menhir_stack in
          let loc = _v in
          let _v = _menhir_action_026 loc in
          _menhir_goto_atom_init _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_255 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_LBRK -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      match (_tok : MenhirBasics.token) with
      | RBRK ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
          let loc = _v in
          let _v = _menhir_action_134 loc in
          _menhir_goto_location_deref _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_191 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_LBRK, ttv_result) _menhir_cell1_location_global, ttv_result) _menhir_cell1_LBRK -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      match (_tok : MenhirBasics.token) with
      | RBRK ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          let MenhirCell1_LBRK (_menhir_stack, _) = _menhir_stack in
          let MenhirCell1_location_global (_menhir_stack, _, loc) = _menhir_stack in
          let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
          let loc_inlined1 = _v in
          let _v = _menhir_action_050 loc loc_inlined1 in
          _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_170 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_LBRK, ttv_result) _menhir_cell1_location_global, ttv_result) _menhir_cell1_LBRK -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      match (_tok : MenhirBasics.token) with
      | RBRK ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          let MenhirCell1_LBRK (_menhir_stack, _) = _menhir_stack in
          let MenhirCell1_location_global (_menhir_stack, _, loc) = _menhir_stack in
          let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
          let loc_inlined1 = _v in
          let _v = _menhir_action_051 loc loc_inlined1 in
          _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_118 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _1 = _v in
      let _v = _menhir_action_162 _1 in
      _menhir_goto_maybev_notag _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_103 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LBRK as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | RBRK ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | NOTEQUAL ->
              let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
              let _menhir_s = MenhirState105 in
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | VALUE _v ->
                  _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
              | TOK_TAG ->
                  _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
              | TOK_PTE ->
                  _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
              | TOK_PA ->
                  _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
              | NUM _v ->
                  _menhir_run_107 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
              | NAME _v ->
                  _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
              | COLON ->
                  _menhir_run_114 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
              | _ ->
                  _eRR ())
          | EQUALEQUAL ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | VALUE _v_3 ->
                  let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                  _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v_3 MenhirState119
              | TOK_TAG ->
                  let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                  _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState119
              | TOK_PTE ->
                  let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                  _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState119
              | TOK_PA ->
                  let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                  _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState119
              | SYMB_REG _v_4 ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
                  let (loc, _1_inlined2) = (_v, _v_4) in
                  let _v = _menhir_action_063 _1_inlined2 loc in
                  _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
              | PROC _v_5 ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | COLON ->
                      let _tok = _menhir_lexer _menhir_lexbuf in
                      (match (_tok : MenhirBasics.token) with
                      | SYMB_REG _v_6 ->
                          let _tok = _menhir_lexer _menhir_lexbuf in
                          let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
                          let (loc, _3_inlined1, _1_inlined2) = (_v, _v_6, _v_5) in
                          let _v = _menhir_action_061 _1_inlined2 _3_inlined1 loc in
                          _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
                      | PTX_REG_DEC ->
                          let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                          let _menhir_stack = MenhirCell1_PROC (_menhir_stack, MenhirState119, _v_5) in
                          let _menhir_stack = MenhirCell1_PTX_REG_DEC (_menhir_stack, MenhirState122) in
                          let _tok = _menhir_lexer _menhir_lexbuf in
                          (match (_tok : MenhirBasics.token) with
                          | PTX_REG_TYPE _v ->
                              let _menhir_stack = MenhirCell0_PTX_REG_TYPE (_menhir_stack, _v) in
                              let _menhir_s = MenhirState125 in
                              let _tok = _menhir_lexer _menhir_lexbuf in
                              (match (_tok : MenhirBasics.token) with
                              | NAME _v ->
                                  _menhir_run_030 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                              | DOLLARNAME _v ->
                                  _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                              | _ ->
                                  _eRR ())
                          | _ ->
                              _eRR ())
                      | NAME _v_10 ->
                          let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                          let _menhir_stack = MenhirCell1_PROC (_menhir_stack, MenhirState119, _v_5) in
                          _menhir_run_030 _menhir_stack _menhir_lexbuf _menhir_lexer _v_10 MenhirState122
                      | DOLLARNAME _v_11 ->
                          let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                          let _menhir_stack = MenhirCell1_PROC (_menhir_stack, MenhirState119, _v_5) in
                          _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v_11 MenhirState122
                      | _ ->
                          _eRR ())
                  | _ ->
                      _eRR ())
              | NUM _v_12 ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | COLON ->
                      let _tok = _menhir_lexer _menhir_lexbuf in
                      (match (_tok : MenhirBasics.token) with
                      | SYMB_REG _v_13 ->
                          let _tok = _menhir_lexer _menhir_lexbuf in
                          let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
                          let (loc, _3_inlined1, _1_inlined2) = (_v, _v_13, _v_12) in
                          let _v = _menhir_action_062 _1_inlined2 _3_inlined1 loc in
                          _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
                      | PTX_REG_DEC ->
                          let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                          let _menhir_stack = MenhirCell1_NUM (_menhir_stack, MenhirState119, _v_12) in
                          let _menhir_stack = MenhirCell1_PTX_REG_DEC (_menhir_stack, MenhirState129) in
                          let _tok = _menhir_lexer _menhir_lexbuf in
                          (match (_tok : MenhirBasics.token) with
                          | PTX_REG_TYPE _v ->
                              let _menhir_stack = MenhirCell0_PTX_REG_TYPE (_menhir_stack, _v) in
                              let _menhir_s = MenhirState132 in
                              let _tok = _menhir_lexer _menhir_lexbuf in
                              (match (_tok : MenhirBasics.token) with
                              | NAME _v ->
                                  _menhir_run_030 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                              | DOLLARNAME _v ->
                                  _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                              | _ ->
                                  _eRR ())
                          | _ ->
                              _eRR ())
                      | NUM _v_17 ->
                          let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                          let _menhir_stack = MenhirCell1_NUM (_menhir_stack, MenhirState119, _v_12) in
                          _menhir_run_109 _menhir_stack _menhir_lexbuf _menhir_lexer _v_17
                      | NAME _v_18 ->
                          let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                          let _menhir_stack = MenhirCell1_NUM (_menhir_stack, MenhirState119, _v_12) in
                          _menhir_run_040 _menhir_stack _menhir_lexbuf _menhir_lexer _v_18 MenhirState129
                      | DOLLARNAME _v_19 ->
                          let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                          let _menhir_stack = MenhirCell1_NUM (_menhir_stack, MenhirState119, _v_12) in
                          _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v_19 MenhirState129
                      | _ ->
                          _eRR ())
                  | AND | EOF | EXISTS | FINAL | FORALL | IMPLIES | LPAR | NOT | OBSERVED | OR | RPAR | SEMI | WITH ->
                      let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                      let _v_20 =
                        let _1 = _v_12 in
                        _menhir_action_160 _1
                      in
                      _menhir_run_116 _menhir_stack _menhir_lexbuf _menhir_lexer _v_20 MenhirState119 _tok
                  | _ ->
                      _eRR ())
              | NAME _v_21 ->
                  let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                  _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v_21 MenhirState119
              | LPAR ->
                  let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                  _menhir_run_135 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState119
              | LBRK ->
                  let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                  let _menhir_stack = MenhirCell1_LBRK (_menhir_stack, MenhirState119) in
                  let _menhir_s = MenhirState169 in
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | TOK_TAG ->
                      _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                  | TOK_PTE ->
                      _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                  | TOK_PA ->
                      _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                  | NUM _v ->
                      _menhir_run_099 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                  | NAME _v ->
                      _menhir_run_102 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                  | _ ->
                      _eRR ())
              | COLON ->
                  let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                  _menhir_run_114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState119
              | _ ->
                  _eRR ())
          | EQUAL ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | VALUE _v_24 ->
                  let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                  _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v_24 MenhirState174
              | TOK_TAG ->
                  let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                  _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState174
              | TOK_PTE ->
                  let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                  _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState174
              | TOK_PA ->
                  let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                  _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState174
              | SYMB_REG _v_25 ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
                  let (loc, _1_inlined2) = (_v, _v_25) in
                  let _v = _menhir_action_056 _1_inlined2 loc in
                  _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
              | PROC _v_26 ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | COLON ->
                      let _tok = _menhir_lexer _menhir_lexbuf in
                      (match (_tok : MenhirBasics.token) with
                      | SYMB_REG _v_27 ->
                          let _tok = _menhir_lexer _menhir_lexbuf in
                          let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
                          let (loc, _3_inlined1, _1_inlined2) = (_v, _v_27, _v_26) in
                          let _v = _menhir_action_054 _1_inlined2 _3_inlined1 loc in
                          _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
                      | PTX_REG_DEC ->
                          let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                          let _menhir_stack = MenhirCell1_PROC (_menhir_stack, MenhirState174, _v_26) in
                          let _menhir_stack = MenhirCell1_PTX_REG_DEC (_menhir_stack, MenhirState177) in
                          let _tok = _menhir_lexer _menhir_lexbuf in
                          (match (_tok : MenhirBasics.token) with
                          | PTX_REG_TYPE _v ->
                              let _menhir_stack = MenhirCell0_PTX_REG_TYPE (_menhir_stack, _v) in
                              let _menhir_s = MenhirState180 in
                              let _tok = _menhir_lexer _menhir_lexbuf in
                              (match (_tok : MenhirBasics.token) with
                              | NAME _v ->
                                  _menhir_run_030 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                              | DOLLARNAME _v ->
                                  _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                              | _ ->
                                  _eRR ())
                          | _ ->
                              _eRR ())
                      | NAME _v_31 ->
                          let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                          let _menhir_stack = MenhirCell1_PROC (_menhir_stack, MenhirState174, _v_26) in
                          _menhir_run_030 _menhir_stack _menhir_lexbuf _menhir_lexer _v_31 MenhirState177
                      | DOLLARNAME _v_32 ->
                          let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                          let _menhir_stack = MenhirCell1_PROC (_menhir_stack, MenhirState174, _v_26) in
                          _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v_32 MenhirState177
                      | _ ->
                          _eRR ())
                  | _ ->
                      _eRR ())
              | NUM _v_33 ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | COLON ->
                      let _tok = _menhir_lexer _menhir_lexbuf in
                      (match (_tok : MenhirBasics.token) with
                      | SYMB_REG _v_34 ->
                          let _tok = _menhir_lexer _menhir_lexbuf in
                          let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
                          let (loc, _3_inlined1, _1_inlined2) = (_v, _v_34, _v_33) in
                          let _v = _menhir_action_055 _1_inlined2 _3_inlined1 loc in
                          _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
                      | PTX_REG_DEC ->
                          let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                          let _menhir_stack = MenhirCell1_NUM (_menhir_stack, MenhirState174, _v_33) in
                          let _menhir_stack = MenhirCell1_PTX_REG_DEC (_menhir_stack, MenhirState184) in
                          let _tok = _menhir_lexer _menhir_lexbuf in
                          (match (_tok : MenhirBasics.token) with
                          | PTX_REG_TYPE _v ->
                              let _menhir_stack = MenhirCell0_PTX_REG_TYPE (_menhir_stack, _v) in
                              let _menhir_s = MenhirState187 in
                              let _tok = _menhir_lexer _menhir_lexbuf in
                              (match (_tok : MenhirBasics.token) with
                              | NAME _v ->
                                  _menhir_run_030 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                              | DOLLARNAME _v ->
                                  _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                              | _ ->
                                  _eRR ())
                          | _ ->
                              _eRR ())
                      | NUM _v_38 ->
                          let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                          let _menhir_stack = MenhirCell1_NUM (_menhir_stack, MenhirState174, _v_33) in
                          _menhir_run_109 _menhir_stack _menhir_lexbuf _menhir_lexer _v_38
                      | NAME _v_39 ->
                          let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                          let _menhir_stack = MenhirCell1_NUM (_menhir_stack, MenhirState174, _v_33) in
                          _menhir_run_040 _menhir_stack _menhir_lexbuf _menhir_lexer _v_39 MenhirState184
                      | DOLLARNAME _v_40 ->
                          let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                          let _menhir_stack = MenhirCell1_NUM (_menhir_stack, MenhirState174, _v_33) in
                          _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v_40 MenhirState184
                      | _ ->
                          _eRR ())
                  | AND | EOF | EXISTS | FINAL | FORALL | IMPLIES | LPAR | NOT | OBSERVED | OR | RPAR | SEMI | WITH ->
                      let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                      let _v_41 =
                        let _1 = _v_33 in
                        _menhir_action_160 _1
                      in
                      _menhir_run_116 _menhir_stack _menhir_lexbuf _menhir_lexer _v_41 MenhirState174 _tok
                  | _ ->
                      _eRR ())
              | NAME _v_42 ->
                  let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                  _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v_42 MenhirState174
              | LPAR ->
                  let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                  _menhir_run_135 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState174
              | LBRK ->
                  let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                  let _menhir_stack = MenhirCell1_LBRK (_menhir_stack, MenhirState174) in
                  let _menhir_s = MenhirState190 in
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | TOK_TAG ->
                      _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                  | TOK_PTE ->
                      _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                  | TOK_PA ->
                      _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                  | NUM _v ->
                      _menhir_run_099 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                  | NAME _v ->
                      _menhir_run_102 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                  | _ ->
                      _eRR ())
              | COLON ->
                  let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                  _menhir_run_114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState174
              | _ ->
                  _eRR ())
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  and _menhir_run_084 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _1 = _v in
      let _v = _menhir_action_118 _1 in
      _menhir_goto_location _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  let _menhir_run_001 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | LBRK ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | TOK_TAG ->
              let _menhir_stack = MenhirCell1_LOCATIONS (_menhir_stack, _menhir_s) in
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState002
          | TOK_PTE ->
              let _menhir_stack = MenhirCell1_LOCATIONS (_menhir_stack, _menhir_s) in
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState002
          | TOK_PA ->
              let _menhir_stack = MenhirCell1_LOCATIONS (_menhir_stack, _menhir_s) in
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState002
          | SYMB_REG _v ->
              let _menhir_stack = MenhirCell1_LOCATIONS (_menhir_stack, _menhir_s) in
              _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState002
          | RBRK ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              let _v = _menhir_action_145 () in
              _menhir_goto_locations _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
          | PROC _v ->
              let _menhir_stack = MenhirCell1_LOCATIONS (_menhir_stack, _menhir_s) in
              _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState002
          | NUM _v ->
              let _menhir_stack = MenhirCell1_LOCATIONS (_menhir_stack, _menhir_s) in
              _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState002
          | NAME _v ->
              let _menhir_stack = MenhirCell1_LOCATIONS (_menhir_stack, _menhir_s) in
              _menhir_run_044 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState002
          | FAULT ->
              let _menhir_stack = MenhirCell1_LOCATIONS (_menhir_stack, _menhir_s) in
              _menhir_run_051 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState002
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  let _menhir_run_000 : type  ttv_stack. ttv_stack -> _ -> _ -> _menhir_box_constraints =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | LOCATIONS ->
          _menhir_run_001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState000
      | EOF | EXISTS | FILTER | FINAL | FORALL | LPAR | NOT | OBSERVED | SEMI | WITH ->
          let _v = _menhir_action_146 () in
          _menhir_run_093 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState000 _tok
      | _ ->
          _eRR ()
  
  let _menhir_run_315 : type  ttv_stack. ttv_stack -> _ -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TOK_TAG ->
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState315
      | TOK_PTE ->
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState315
      | TOK_PA ->
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState315
      | SYMB_REG _v ->
          _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState315
      | STAR ->
          _menhir_run_316 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState315
      | SEMI ->
          _menhir_run_326 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState315
      | PROC _v ->
          _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState315
      | NUM _v ->
          _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState315
      | NAME _v ->
          _menhir_run_327 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState315
      | LBRK ->
          _menhir_run_317 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState315
      | ATOMIC ->
          _menhir_run_349 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState315
      | EOF ->
          let _v = _menhir_action_089 () in
          _menhir_run_371 _menhir_stack _v _tok
      | _ ->
          _eRR ()
  
  let _menhir_run_378 : type  ttv_stack. ttv_stack -> _ -> _ -> _menhir_box_main_constr =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | OBSERVED ->
          _menhir_run_278 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState378
      | NOT ->
          _menhir_run_286 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState378
      | LPAR ->
          _menhir_run_289 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState378
      | FORALL ->
          _menhir_run_292 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState378
      | FINAL ->
          _menhir_run_294 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState378
      | EXISTS ->
          _menhir_run_296 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState378
      | EOF ->
          let _v = _menhir_action_068 () in
          _menhir_run_380 _menhir_stack _v _tok
      | _ ->
          _eRR ()
  
  let _menhir_run_382 : type  ttv_stack. ttv_stack -> _ -> _ -> _menhir_box_main_filter =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | FILTER ->
          _menhir_run_094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState382
      | EOF ->
          let _v = _menhir_action_084 () in
          _menhir_run_384 _menhir_stack _v _tok
      | _ ->
          _eRR ()
  
  let _menhir_run_386 : type  ttv_stack. ttv_stack -> _ -> _ -> _menhir_box_main_loc_constr =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | LOCATIONS ->
          _menhir_run_001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState386
      | EOF | EXISTS | FINAL | FORALL | LPAR | NOT | OBSERVED ->
          let _v = _menhir_action_146 () in
          _menhir_run_388 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState386 _tok
      | _ ->
          _eRR ()
  
  let _menhir_run_392 : type  ttv_stack. ttv_stack -> _ -> _ -> _menhir_box_main_location =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
      let _menhir_s = MenhirState392 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TOK_TAG ->
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PTE ->
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PA ->
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | SYMB_REG _v ->
          _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | PROC _v ->
          _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NUM _v ->
          _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NAME _v ->
          _menhir_run_102 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | _ ->
          _eRR ()
  
  let _menhir_run_396 : type  ttv_stack. ttv_stack -> _ -> _ -> _menhir_box_pteval =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
      let _menhir_s = MenhirState396 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | LPAR ->
          let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s) in
          let _menhir_s = MenhirState397 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | TOK_OA ->
              _menhir_run_136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | NAME _v ->
              _menhir_run_149 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | ATTRS ->
              _menhir_run_156 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  let _menhir_run_401 : type  ttv_stack. ttv_stack -> _ -> _ -> _menhir_box_skip_loc_constr =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | LOCATIONS ->
          _menhir_run_001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState401
      | EOF | EXISTS | FINAL | FORALL | LPAR | NOT | OBSERVED ->
          let _v = _menhir_action_146 () in
          _menhir_run_403 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState401 _tok
      | _ ->
          _eRR ()
  
end

let skip_loc_constr =
  fun _menhir_lexer _menhir_lexbuf ->
    let _menhir_stack = () in
    let MenhirBox_skip_loc_constr v = _menhir_run_401 _menhir_stack _menhir_lexbuf _menhir_lexer in
    v

let pteval =
  fun _menhir_lexer _menhir_lexbuf ->
    let _menhir_stack = () in
    let MenhirBox_pteval v = _menhir_run_396 _menhir_stack _menhir_lexbuf _menhir_lexer in
    v

let main_location =
  fun _menhir_lexer _menhir_lexbuf ->
    let _menhir_stack = () in
    let MenhirBox_main_location v = _menhir_run_392 _menhir_stack _menhir_lexbuf _menhir_lexer in
    v

let main_loc_constr =
  fun _menhir_lexer _menhir_lexbuf ->
    let _menhir_stack = () in
    let MenhirBox_main_loc_constr v = _menhir_run_386 _menhir_stack _menhir_lexbuf _menhir_lexer in
    v

let main_filter =
  fun _menhir_lexer _menhir_lexbuf ->
    let _menhir_stack = () in
    let MenhirBox_main_filter v = _menhir_run_382 _menhir_stack _menhir_lexbuf _menhir_lexer in
    v

let main_constr =
  fun _menhir_lexer _menhir_lexbuf ->
    let _menhir_stack = () in
    let MenhirBox_main_constr v = _menhir_run_378 _menhir_stack _menhir_lexbuf _menhir_lexer in
    v

let init =
  fun _menhir_lexer _menhir_lexbuf ->
    let _menhir_stack = () in
    let MenhirBox_init v = _menhir_run_315 _menhir_stack _menhir_lexbuf _menhir_lexer in
    v

let constraints =
  fun _menhir_lexer _menhir_lexbuf ->
    let _menhir_stack = () in
    let MenhirBox_constraints v = _menhir_run_000 _menhir_stack _menhir_lexbuf _menhir_lexer in
    v
